//==========================================================================
// Rglage du style de chaque courbe
// pj : handle de la courbe
// j  : n reprsentant le style de ligne  (entre 1 et 6)
// couleur : entier positif dfinissant la couleur des marks si actives
// voir l'aide sur les polyline properties
//==========================================================================
function []= regle_style(pj,j,couleur)
  pj.thickness=2; // paiseur du trait
  
   //line_style entre 1 et 6 voir figure a=getlinestyle();
   i=modulo(j-1,6)+1;
   if i==1 then
     i=2;
    end; 
  pj.line_style=i;
  
  //mark_style entre 0 et 14 voir figure a=getmark();
  pj.mark_style = modulo(j-1,15) // entre 0 et 14 pour le type de mark
  //mark_size entre 0 et 5 soit 8pt,10,12,14,18 et 24
  // voir figure a=getmark();
  pj.mark_size = 0;
  pj.mark_mode = "off"
  pj.mark_foreground = couleur;
endfunction

//==========================================================================
// Affichage de la lgende et rglage
// Trace la lgende avec la mme couleur et le mme style que les courbes
// coul :tableau d'entier positif reprsentant des couleurs, il est retourn par trace_auto
// ch : tableau de nc chaines de caractre reprsentant les nc courbes
// nc : le nombre de courbes
// pos : la position de la lgende (voir l'aide sur legends)
//==========================================================================
function []=trace_legende(coul,ch,nc,pos)
  drawlater();
  legends(ch,coul(1:nc), opt=pos);
  e=gce();
  drawlater();
  for j=nc:-1:1,
    pj=e.children(j*2);
    regle_style(pj,nc-j+1,coul(nc-j+1));
  end
  
endfunction

//==========================================================================
// Rglage concernant les axes et la grille
//==========================================================================

function []=regle_axes();
  xgrid(12);
  a = get("current_axes");
  a.tight_limits =  "on";
  a.x_location = "bottom";
  a.y_location = "left";
  a.title.font_size = 2;
  a.title.foreground=2;
  a.x_label.foreground=2;
  a.y_label.foreground=2;
endfunction

//==========================================================================
// Rglage des limites des axes
// limide_boite = ([XMIN YMIN XMAX YMAX]);
//==========================================================================
function []=limite_axes(limite_boite);
  a = get("current_axes");
  a.tight_limits =  "on";
  a.data_bounds = limite_boite;
endfunction


//==========================================================================
// Trac des courbes, rglage auto des couleurs et du style de chaque
// courbe
// x : tableau 1D de rels reprsentant l'abscisse
// y : tableau  n colonnes reprsentant les diffrentes courbes  tracer
// axesflag : type d'axes, voir l'aide sur plot2d
//==========================================================================
function [coul,nc]=trace_auto(x,y,type_axes)
//------------- Les couleurs -------------------
coul = [color("blue"),color("purple"),color("green"), color("red"),color("brown4"), ...
        color("cyan"),color("magenta"),color("green3"),color("red3"),color("orange"),...
        color("blue4"),color("magenta4"),color("green4"),color("pink4"),color("gold"),...
        color("cyan4"),color("violet"),color("grey80")];
// 18 courbes maximum !!!
l = size(y);nc = l(2);
if nc>18 then
  printf('TROP de courbes !!!!')
  return
end
plot2d(x,y(:,1:nc),style=coul(1:nc),axesflag=type_axes);
e=gce(); 
drawlater();
for j=nc:-1:1,
  pj=e.children(j);
  regle_style(pj,nc-j+1,coul(nc-j+1));
end

endfunction

//==========================================================================
// Trac des courbes, rglage auto des couleurs et du style de chaque
// courbe
// x : tableau 1D de rels reprsentant l'abscisse
// y : tableau  n colonnes reprsentant les diffrentes courbes  tracer
// axesflag : type d'axes, voir l'aide sur plot2d
//==========================================================================
function [coul,nc]=trace_auto2(x,y,nu,type_axes)
//------------- Les couleurs -------------------
coul = [color("blue"),color("purple"),color("green"), color("red"),color("brown4"), ...
        color("cyan"),color("magenta"),color("green3"),color("red3"),color("orange"),...
        color("blue4"),color("magenta4"),color("green4"),color("pink4"),color("gold"),...
        color("cyan4"),color("violet"),color("grey80")];
// 18 courbes maximum !!!
l = size(y);nc = l(2);
if nc>18 then
  printf('TROP de courbes !!!!')
  return
end
plot2d(x,y(:,1:nc),style=coul(nu:nc+nu-1),axesflag=type_axes);
e=gce(); 
drawlater();
for j=nc:-1:1,
 pj=e.children(j);
 regle_style(pj,nc+nu-j,coul(nc+nu-j));
end

endfunction
