//==========================================================================
// Réglage du style de chaque courbe
// pj : handle de la courbe
// j  : n° représentant le style de ligne  (entre 1 et 6)
// couleur : entier positif définissant la couleur des marks si activées
// voir l'aide sur les polyline properties
//==========================================================================
function []= regle_style(pj,j,couleur)
  pj.thickness=2;
  pj.line_style=modulo(j,7);
  pj.mark_style = modulo(j+1,15) // entre 1 et 14 pour le type de mark
  pj.mark_size = 0  //entre 0 et 5, 8pt,10,12,14,18 et 24
  pj.mark_mode = "off"
  pj.mark_foreground = couleur;
endfunction

//==========================================================================
// Affichage de la légende et réglage
// Trace la légende avec la même couleur et le même style que les courbes
// coul :tableau d'entier positif représentant des couleurs, il est retourné par trace_auto
// ch : tableau de nc chaines de caractère représentant les nc courbes
// nc : le nombre de courbes
// pos : la position de la légende (voir l'aide sur legends)
//==========================================================================
function []=trace_legende(coul,ch,nc,pos)
  legends(ch,coul(1:nc), opt=pos);
  e=gce();
  for j=nc:-1:1,
    pj=e.children(j*2);
    regle_style(pj,j,coul(nc-j+1));
  end
  
endfunction

//==========================================================================
// Réglage concernant les axes et la grille
//==========================================================================

function []=regle_axes();
  xgrid(12);
  a = get("current_axes");
  a.tight_limits =  "on";
  a.x_location = "bottom";
  a.y_location = "left";
  a.title.font_size = 2;
  a.title.foreground=2;
  a.x_label.foreground=2;
  a.y_label.foreground=2;
endfunction

//==========================================================================
// Réglage des limites des axes
// limide_boite = ([XMIN YMIN XMAX YMAX]);
//==========================================================================
function []=limite_axes(limite_boite);
  a = get("current_axes");
  a.tight_limits =  "on";
  a.data_bounds = limite_boite;
endfunction


//==========================================================================
// Tracé des courbes, réglage auto des couleurs et du style de chaque
// courbe
// x : tableau 1D de réels représentant l'abscisse
// y : tableau à n colonnes représentant les différentes courbes à tracer
// axesflag : type d'axes, voir l'aide sur plot2d
//==========================================================================
function [coul,nc]=trace_auto(x,y,type_axes)
//------------- Les couleurs -------------------
coul = [color("blue1"),color("red"),color("green"), color("purple"), ...
        color("turquoise1"),color("orange1"),color("violet"),...
	color("cyan1"),color("chartreuse"),color("grey80")];

l = size(y);nc = l(2);
plot2d(x,y(:,1:nc),style=coul(1:nc),axesflag=type_axes);
e=gce(); 
for j=nc:-1:1,
  pj=e.children(j);
  regle_style(pj,j,coul(nc-j+1));
end

endfunction

//==========================================================================
// DÉBUT DU PROGRAMME 
//==========================================================================

//M11= fscanfMat('pt1_t.dtvide');
M11= fscanfMat('pt1_t.dt90_29_i_drmod');
x=M11(:,1)*1e9;

//---------- pt1, Ez ----------------------
scf(0);clf(gcf(0)); // handle de la fenêtre
[coul,nc] = trace_auto(x,M11(:,4),1);
ch = ['EZ'];  // label de la légende
trace_legende(coul,ch,nc,1);
regle_axes();
xtitle('Amplitude de E incident au point 1 ','t (ns)','V/m');

//---------- pt1, Hx, Hy ----------------------
scf(1);clf(gcf(1)); // handle de la fenêtre
[coul,nc] = trace_auto(x,M11(:,5:6),1);
ch = ['HX';'HY'];
trace_legende(coul,ch,nc,1);
regle_axes();
limite_axes([0,-2e-3;50,2e-3]);
xtitle('Amplitude de H incident au point 1 ','t (ns)','A/m');


//---------- pt 2 et 3, Ez ----------------------
M2= fscanfMat('pt2_t.dt90_29_i_drmod');
M3= fscanfMat('pt3_t.dt90_29_i_drmod');
x=M2(:,1)*1e9;
y = [M2(:,4),M3(:,4)];
scf(3);clf(gcf(3)); // handle de la fenêtre
[coul,nc] = trace_auto(x,y,1);
ch = ['EZ pt2','EZ pt3'];
trace_legende(coul,ch,nc,2);
regle_axes();
xtitle('Amplitude de E diffracté aux points 2 et 3 ','t (ns)','V/m');

