!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_calculat.f90
!// Purpose:  Interprétation d'une chaine de caractère correspondant
!               à des instructions de calcul flottant
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Juin 2007
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
module m_calculat
use m_constantes, only : LEN_CHAINE, LEN_MOT_CLE,rang,RANG_MAITRE_AFF
!use m_constantes_scientifiques
use m_chaine_operation, only :LowerCaseChaine,entier_chaine

implicit none


integer,private, parameter :: NB_OP_MAX = 15

integer,private, parameter :: OP_PLUS = 0
integer,private, parameter :: OP_MOINS = 1
integer,private, parameter :: OP_DEB_P = 2
integer,private, parameter :: OP_FIN_P = 3
integer,private, parameter :: OP_PROD = 4
integer,private, parameter :: OP_DIV  = 5
integer,private, parameter :: OP_EXPO  = 6
logical,private, parameter :: flag_calculat_affiche = .false.
character(len=5),dimension(13) :: FONC_MATH=(/'sqrt ','exp  ','log  ','log10','sin  ','cos  ','tan  ', &
                                              'asin ','acos ','atan ','cosh ','sinh ','tanh '/)

private :: affichage_etat_eq_formelle
!===============================================================
type st_chaine_calcul
  type(st_chaine_calcul), pointer :: pprec
  type(st_chaine_calcul), pointer :: psuiv
  integer :: posi  ! position dans la liste chainée ?
  integer :: nopav  !nbre d'opération avant 
  integer,dimension(NB_OP_MAX) ::opavant  ! type d'opération : +,//,(,) 
  
  logical     :: flag_fonction
  character(LEN=5) :: fonction
  integer     :: nufonc
  real              :: valeur
end type st_chaine_calcul
!===============================================================
type st_calculat
  type(st_chaine_calcul), pointer :: cc
  character(LEN=LEN_CHAINE)  :: ch
  real   :: resultat
  integer  :: n_op
end type st_calculat
!===============================================================


contains


! ==========================================================
! == subroutine initdefaut_chaine_calcul                  ==
! == Initialisation par défaut
! ==========================================================
subroutine initdefaut_chaine_calcul(cc)
type(st_chaine_calcul), intent(out) :: cc

nullify(cc%pprec)
nullify(cc%psuiv)
cc%posi = -1 !le début de la chaine
cc%nopav =  0
cc%opavant = -1
cc%flag_fonction = .false.
cc%fonction=''
cc%valeur  = 0.0
end subroutine initdefaut_chaine_calcul

! ==========================================================
! == function type_operation                              ==
! == Retourne un n° correspondant au type d'opération
! ==========================================================
integer function type_operation(c)
character,intent(in) :: c

select case (c)
  case('+');type_operation = OP_PLUS
  case('-');type_operation = OP_MOINS
  case('*');type_operation = OP_PROD
  case('/');type_operation = OP_DIV
  case('^');type_operation = OP_EXPO
  case('(');type_operation = OP_DEB_P
  case(')');type_operation = OP_FIN_P
  case default
    type_operation =-1
end select

end function type_operation

! ==========================================================
! == function calcul_coefficient_pade                     ==
! == Interprétation de l'équation formelle
! == Puis calcul des coefficients de PADE
! ==========================================================
real function calculat_reel(tcalc,ch_operation,numligne)
!=== DECLARATION DES ARGUMENTS ===
type(st_calculat), intent(inout) :: tcalc
character(LEN=LEN_CHAINE),intent(in) :: ch_operation
integer, intent(in)   :: numligne

!=== DECLARATION DES VARIABLES LOCALES ===
type(st_chaine_calcul), pointer ::c1,cap,cav

integer :: ideb, ifin
integer :: l,l1,i,n,j,j1,j2,k,nopprec,statut,numvar
character(LEN=LEN_MOT_CLE) :: variable
CHARACTER :: chop_ap(6),chop(5),chop_interdit(1),chop_p(6)
character(LEN=LEN_CHAINE) :: cha,chz
character :: p(2),c(2),a(2)
INTEGER :: operation(5),operation1(6)


!=== DEBUT DES TRAITEMENTS ===
p = (/'(',')'/);
c = (/'[',']'/);
a = (/'{','}'/);
chop = (/'^','/','*','+','-'/);
chop_interdit = (/'('/)
chop_ap = (/'^','-','*','+','/',')'/);
chop_p = (/'(','{','[',']','}',')'/);
operation = (/OP_PLUS,OP_MOINS,OP_PROD,OP_DIV,OP_EXPO/)
operation1 = (/OP_DEB_P,OP_PLUS,OP_MOINS,OP_PROD,OP_DIV,OP_EXPO/)


!==============================================================================
!=========== Interprétation de l'équation formelle ================
chz = adjustl(ch_operation)
tcalc%ch = chz
l = len_trim(chz)

! On supprime tous les blancs dans la chaine
l1=l
i=1
do 
 i = i+1
 if (i>=l1) exit
 if (chz(i:i)==' ') then
   chz(i:l1-1) = chz(i+1:l1)
   chz(l1:l1) = ' '
   l1 = l1-1
   if (flag_calculat_affiche.and.rang==RANG_MAITRE_AFF) print '("Chaine =",A)',trim(chz) 
   i=i-1
 end if
end do

if (flag_calculat_affiche.and.rang==RANG_MAITRE_AFF) print '("Chaine sans blanc :",A)',trim(chz)

!--- On crée la liste doublement chainée d'opération ---
tcalc%n_op = 0 ! correspond au nbre d'opérations chainées
l = len_trim(chz)
l1=l 

!--- vérification de la cohérence des accolades ---
i=0
cha = ''
do j=1,l1
  if (any(chz(j:j)==chop_p)) then
    i=i+1
    cha(i:i) = chz(j:j)
  end if
end do
if (flag_calculat_affiche.and.rang==RANG_MAITRE_AFF) &
  print '("Chaine uniquement avec les accolade :",A)',trim(cha)
if (mod(i,2)==1) call stop_calculat(0,0,chz,numligne)
j=0

do
  if (i==0) exit ! on sort
  j=j+1
  if (any(cha(j:j)==chop_p(1:3)).and.any(cha(j+1:j+1)==chop_p(4:6))) then
    if (cha(j:j)=='(' .and.cha(j+1:j+1)/=')') call stop_calculat(-1,j,chz,numligne)
    if (cha(j:j)=='[' .and.cha(j+1:j+1)/=']') call stop_calculat(-1,j,chz,numligne)
    if (cha(j:j)=='{' .and.cha(j+1:j+1)/='}') call stop_calculat(-1,j,chz,numligne)
    do k=j,i-2
      cha(k:k)=cha(k+2:k+2)
    end do
    cha(i-1:i) = '  '
    j=0
    if (flag_calculat_affiche.and.rang==RANG_MAITRE_AFF)&
      print '("Chaine uniquement avec les accolade en réduction:",A)',trim(cha)
    
    i=i-2
  end if
end do
if (flag_calculat_affiche.and.rang==RANG_MAITRE_AFF) print '("Cohérence des accolades ")'

! On harmonise les parenthèse avec ()
do i=1,l1
  select case(chz(i:i))
  case('{','[')
    chz(i:i) = '('
  case('}',']')
    chz(i:i) = ')'
  end select  
end do


!---------------------------------------------------
! préparation de la structure chainée
allocate(tcalc%cc)
c1=>tcalc%cc
call initdefaut_chaine_calcul(c1)
i=0 ! incrément de caractère



do
  i=i+1 ! n° du caractère
  if (i>l1) exit ! on sort de la boucle
  
  select case(chz(i:i))
  
  !----------------------------------------------------------------------------
  case('(')
    c1%nopav=c1%nopav+1
    c1%opavant(c1%nopav) = OP_DEB_P
    !--- Vérification ---
    if (c1%nopav>1) then ! nbre d'opération avant >1?
      nopprec = c1%opavant(c1%nopav-1)
      if (nopprec==OP_FIN_P) &
        call stop_calculat(1,i,chz,numligne)
    else if (c1%posi >= 0) then ! manque un + ou //
      if (.not.c1%pprec%flag_fonction) call stop_calculat(2,i,chz,numligne)
    end if
    
  !----------------------------------------------------------------------------
  case(')')
    c1%nopav=c1%nopav+1
    c1%opavant(c1%nopav) = OP_FIN_P
    
    !--- Vérification ---
    if (c1%nopav>1) then
      nopprec = c1%opavant(c1%nopav-1)
      if (any(operation==nopprec).or.nopprec==OP_DEB_P) &
        call stop_calculat(1,i,chz,numligne)
    else if (c1%posi == -1) then ! incorrect si première opération de la chaine
      call stop_calculat(1,i,chz,numligne)
    end if
  
  !----------------------------------------------------------------------------
  case('+','-')
    c1%nopav=c1%nopav+1
    c1%opavant(c1%nopav) = type_operation(chz(i:i))
  
    !--- Vérification ---
    if (c1%nopav>1) then
      nopprec = c1%opavant(c1%nopav-1)
      if (any(operation==nopprec)) &
        call stop_calculat(1,i,chz,numligne)
    !else if (c1%pos == -1) then ! incorrect si première opération de la chaine
    !  call stop_calculat(1,i,chz)
    end if
  !----------------------------------------------------------------------------
  case('*','^','/')
    c1%nopav=c1%nopav+1
    c1%opavant(c1%nopav) = type_operation(chz(i:i))
  
    !--- Vérification ---
    if (c1%nopav>1) then
      nopprec = c1%opavant(c1%nopav-1)
      if (any(operation==nopprec).or.nopprec==OP_DEB_P) &
        call stop_calculat(1,i,chz,numligne)
    else if (c1%posi == -1) then ! incorrect si première opération de la chaine
      call stop_calculat(1,i,chz,numligne)
    end if
    
  !----------------------------------------------------------------------------
  !----------------------------------------------------------------------------
  case('$') !----------- Détection  variables ----------------
    if (rang==RANG_MAITRE_AFF) print '(" ERREUR, Variable devrait etre déjà évaluée !!!")' 
    call stop_calculat(3,ideb,chz,numligne)
  !----------------------------------------------------------------------------
  !----------------------------------------------------------------------------
  case('0','1','2','3','4','5','6','7','8','9') ! Détection d'une valeur
    if (c1%nopav>=1) then
      nopprec = c1%opavant(c1%nopav)
      if (nopprec==OP_FIN_P) call stop_calculat(1,i-1,chz,numligne)
    end if
    ideb = i
    j=0;j1=0;
    ! ----- Détermination de la longueur de la valeur flottante --------
    do 
      n = iachar(chz(i:i))      
      if ((n>=48 .and. n<=57)) then ! valeur de 1 à 9
        continue
	
      else if (n==46) then !le point '.'
        j=j+1; if (j>1) call stop_calculat(8,i,chz,numligne)
	
      else if (n==69 .or.n==101) then !exposant 'e', 'E'
        j1=j1+1; if (j1>1) call stop_calculat(8,i,chz,numligne)
	if (chz(i+1:i+1)=='-'.or.chz(i+1:i+1)=='+') i=i+1
      else if (any(chz(i:i)==chop_ap)) then
        exit
      else if (any(chz(i:i)==chop_interdit)) then
        call stop_calculat(1,i,chz,numligne)        
      end if
      
      ifin = i 
      i=i+1 ! passage au caractère suivant      
      if (i>l1) exit ! fin de chaine
    end do
    i=i-1
    !---- Récupération de la valeur ---
    variable = chz(ideb:ifin)
    read(variable,*,ERR=999) c1%valeur 
    
    if (flag_calculat_affiche.and.rang==RANG_MAITRE_AFF) then
        print '(T5,"Nombre d''opération avant = ",I4)',c1%nopav
        if (c1%nopav>0) print '(T5,"les opérations = ",20I4)',c1%opavant(1:c1%nopav)
        print '(T5,"valeur trouvée = ",ES15.7)',c1%valeur   
        if (associated(c1%pprec)) print '(T5,"valeur amont=",ES15.7)',c1%pprec%valeur
    end if
    
    ! passage à la structure suivante
    allocate(c1%psuiv)
    call initdefaut_chaine_calcul(c1%psuiv)
    c1%psuiv%posi = c1%posi+1
    c1%psuiv%pprec=>c1
    c1=>c1%psuiv ! passage à la structure suivante
    tcalc%n_op = tcalc%n_op +1
  !----------------------------------------------------------------------------
  !----------------------------------------------------------------------------
  case default ! Ca doit etre une fonction !
    if (c1%nopav>=1) then
      nopprec = c1%opavant(c1%nopav)
      if (nopprec==OP_FIN_P) call stop_calculat(1,i-1,chz,numligne)
    end if
    
    n = index(chz(i:l1),'(') +i-1   
    variable = chz(i:n-1)
    variable = LowerCaseChaine(variable)
    do i=1,size(FONC_MATH)
      if (trim(variable)==FONC_MATH(i)) then
        c1%flag_fonction = .true.
	c1%nufonc = i
	c1%fonction = FONC_MATH(i) 
	exit
      end if
    end do
    !print '(T10,"l1=",I3, "  n=",I3)',l1,n
    !print '(T10,"Nom de la fonction = ",A)',trim(variable)
    if (.not.c1%flag_fonction) call stop_calculat(9,i,chz,numligne)
    i=n-1
    
    if (flag_calculat_affiche.and.rang==RANG_MAITRE_AFF) then
      print '(T5,"Nombre d''opération = ",I4)',c1%nopav
      print '(T5,"les opérations = ",20I4)',c1%opavant(1:c1%nopav)
      print '(T5,"fonction détectée =",A)',trim(c1%fonction)   
    end if
    ! passage à la structure suivante
    allocate(c1%psuiv)
    call initdefaut_chaine_calcul(c1%psuiv)
    c1%psuiv%posi = c1%posi+1
    c1%psuiv%pprec=>c1
    c1=>c1%psuiv ! passage à la structure suivante
    tcalc%n_op = tcalc%n_op +1
    
  end select
  
end do

! Initialisation de la dernière structure
do j=1,c1%nopav
  if (c1%opavant(j)/=OP_FIN_P) call stop_calculat(10,i,chz,numligne)
end do

! introduction  de parenthèse en début et en fin
c1%nopav= c1%nopav+1
c1%opavant(c1%nopav)=OP_FIN_P

c1%nopav= c1%nopav+1
c1%opavant(c1%nopav)=OP_PLUS
c1=>tcalc%cc
do j=c1%nopav,1,-1 ! décalage des opération
  c1%opavant(j+1) = c1%opavant(j) 
end do
c1%nopav= c1%nopav+1
c1%opavant(1)=OP_DEB_P

!==============================================================================
!==============================================================================
!=== Evluation de l'équation formelle ===

call affichage_etat_eq_formelle(tcalc%cc)
do
  if (.not.associated(c1%psuiv).and.associated(c1%pprec)) then
    c1=>c1%pprec
  else if (.not.associated(c1%psuiv)) then
    exit
  end if
  !----------------------------------------------------------------------------
  if (c1%flag_fonction) then
    c1=>c1%psuiv
    cycle
  else if (c1%opavant(c1%nopav)==OP_DEB_P .and. c1%psuiv%opavant(1)==OP_FIN_P) then
    !---- Supression des parenthèses ----
    do j=2,c1%psuiv%nopav ! décalage des opérations
      c1%psuiv%opavant(j-1) = c1%psuiv%opavant(j)
    end do
    c1%nopav=c1%nopav-1
    c1%psuiv%nopav=c1%psuiv%nopav-1
    
    !------- Traitement d'une fonction mathématique ----------
    if (associated(c1%pprec)) then
      if (c1%pprec%flag_fonction) then
        c1 => c1%pprec
        c1%valeur = calculat_fonc_math(c1%fonction,c1%psuiv%valeur)
        c1%flag_fonction = .false.
        call suppr_item_liste(c1%psuiv)      
      end if
    end if
    call affichage_etat_eq_formelle(tcalc%cc)
    
  !----------------------------------------------------------------------------
  else if (c1%psuiv%opavant(1)==OP_FIN_P) then
    if (c1%nopav==2 .and. c1%opavant(c1%nopav-1) == OP_DEB_P) then !cas (+-valeur)
      if (c1%opavant(c1%nopav) == OP_MOINS) then
        c1%valeur = -c1%valeur
	c1%nopav=c1%nopav-1
      else if (c1%opavant(c1%nopav) == OP_PLUS) then
	c1%nopav=c1%nopav-1
      end if
    else
      c1=>c1%pprec ! on traite l'intérieur de la parenthèse jusqu'à disparition de celle-ci
    end if
  !----------------------------------------------------------------------------
  else if (c1%psuiv%nopav>1) then ! au moins deux opérations donc une parenthèse
    c1=>c1%psuiv
    
  !----------------------------------------------------------------------------
  else if (c1%psuiv%opavant(1)==OP_EXPO) then ! opération exposant
    if (associated(c1%psuiv%psuiv)) then
      if (c1%psuiv%psuiv%opavant(1)==OP_EXPO.or.c1%psuiv%psuiv%opavant(1)== OP_DEB_P) then
        c1=>c1%psuiv
        cycle
      end if
    end if
    c1%valeur = c1%valeur**c1%psuiv%valeur
    call suppr_item_liste(c1%psuiv)
    call affichage_etat_eq_formelle(tcalc%cc)
  !----------------------------------------------------------------------------
  else if (c1%psuiv%opavant(1)==OP_DIV) then ! opération division
    ! opérateurs ^ et / prioritaire à gauche
    if ((c1%opavant(c1%nopav) == OP_DIV).or.(c1%opavant(c1%nopav)==OP_EXPO)) then
      c1=>c1%pprec
      cycle
    ! opérateur exposant prioritaire à droite
    else if (associated(c1%psuiv%psuiv)) then
      if (c1%psuiv%psuiv%opavant(1)==OP_EXPO.or.c1%psuiv%psuiv%opavant(1)== OP_DEB_P) then
        c1=>c1%psuiv
        cycle
      end if
    end if
    c1%valeur = c1%valeur/c1%psuiv%valeur
    call suppr_item_liste(c1%psuiv)
    call affichage_etat_eq_formelle(tcalc%cc)
  !----------------------------------------------------------------------------
  else if (c1%psuiv%opavant(1)==OP_PROD) then ! opération multiplication
    ! opérateurs ^ et / prioritaire à gauche
    if ((c1%opavant(c1%nopav) == OP_DIV).or.(c1%opavant(c1%nopav)==OP_EXPO)) then
      c1=>c1%pprec
      cycle
    ! opérateur exposant prioritaire si à droite
    else if (associated(c1%psuiv%psuiv)) then
      if (c1%psuiv%psuiv%opavant(1)==OP_EXPO.or.c1%psuiv%psuiv%opavant(1)== OP_DEB_P) then
        c1=>c1%psuiv
        cycle
      end if
    end if
    c1%valeur = c1%valeur*c1%psuiv%valeur
    call suppr_item_liste(c1%psuiv)
    call affichage_etat_eq_formelle(tcalc%cc)
  !----------------------------------------------------------------------------
  else if (c1%psuiv%opavant(1)==OP_MOINS) then ! opération -
    ! opérateurs ^, /, * et - prioritaire à gauche
    if ((c1%opavant(c1%nopav) == OP_DIV).or.(c1%opavant(c1%nopav)==OP_EXPO) .or.&
        (c1%opavant(c1%nopav) == OP_PROD).or.(c1%opavant(c1%nopav) == OP_MOINS) ) then
      c1=>c1%pprec
      cycle
    ! opérateur ^,/,* prioritaire si à droite
    else if (associated(c1%psuiv%psuiv)) then
      if ((c1%psuiv%psuiv%opavant(1) == OP_DIV) .or.&
          (c1%psuiv%psuiv%opavant(1) == OP_EXPO) .or.&
	  (c1%psuiv%psuiv%opavant(1) == OP_PROD) .or.&
	  (c1%psuiv%psuiv%opavant(1) == OP_DEB_P) ) then
        c1=>c1%psuiv
        cycle
      end if
    end if
    ! Réalisation de l'opération
    c1%valeur = c1%valeur-c1%psuiv%valeur
    call suppr_item_liste(c1%psuiv)
    call affichage_etat_eq_formelle(tcalc%cc)
  !----------------------------------------------------------------------------
  else if (c1%psuiv%opavant(1)==OP_PLUS) then ! opération + 
    ! opérateurs ^, /, * et - prioritaire à gauche
    if ((c1%opavant(c1%nopav) == OP_DIV).or.(c1%opavant(c1%nopav)==OP_EXPO) .or.&
        (c1%opavant(c1%nopav) == OP_PROD).or.(c1%opavant(c1%nopav) == OP_MOINS) ) then
      c1=>c1%pprec
      cycle
    ! opérateur ^,/,* prioritaire si à droite
    else if (associated(c1%psuiv%psuiv)) then
      if ((c1%psuiv%psuiv%opavant(1) == OP_DIV) .or.&
          (c1%psuiv%psuiv%opavant(1) == OP_EXPO) .or.&
	  (c1%psuiv%psuiv%opavant(1) == OP_PROD) .or. &
	  (c1%psuiv%psuiv%opavant(1) == OP_DEB_P) ) then
        c1=>c1%psuiv
        cycle
      end if
    end if
    ! Réalisation de l'opération
    c1%valeur = c1%valeur+c1%psuiv%valeur
    call suppr_item_liste(c1%psuiv)
    call affichage_etat_eq_formelle(tcalc%cc)
  !----------------------------------------------------------------------------
  else
    call stop_calculat(7,i,chz,numligne)
  end if
end do

calculat_reel = c1%valeur
tcalc%resultat= c1%valeur
if (flag_calculat_affiche.and.rang==RANG_MAITRE_AFF) &
  print '(T5," Résultat de l''équation = ",ES15.7)',c1%valeur
deallocate (c1)
return
999  call stop_calculat(8,i,chz,numligne)

end function calculat_reel


!==========================================================

! ==========================================================
! == subroutine calculat_fonc_math                        ==
!  Evaluation d'une fonction mathématique intrinsèque sur 
!  la valeur fournie
! ==========================================================
real function calculat_fonc_math(nom_fonc,valeur) result(nouvval)
character(LEN=5), intent(in) :: nom_fonc
real, intent(in) :: valeur

nouvval = valeur
select case (nom_fonc)
  case('sqrt')
    nouvval = sqrt(valeur)
  case('exp')
    nouvval = exp(valeur)
  case('log')
    nouvval = log(valeur)
  case('log10')
    nouvval = log10(valeur)
  case('sin')
    nouvval = sin(valeur)
  case('cos')
    nouvval = cos(valeur)
  case('tan')
    nouvval = tan(valeur)
  case('asin')
    nouvval = asin(valeur)
  case('acos')
    nouvval = acos(valeur)
  case('atan')
    nouvval = atan(valeur)
!  case('atan2')
!    nouvval = atan2(valeur)
  case('cosh')
    nouvval = cosh(valeur)
  case('sinh')
    nouvval = sinh(valeur)
  case('tanh')
    nouvval = tanh(valeur)
end select
end function calculat_fonc_math



! ==========================================================
! == Subroutine suppr_item_liste                    ==
! == on supprime l'item de la liste doublement
! == chainée
! ==========================================================
subroutine suppr_item_liste(c1)
type(st_chaine_calcul), pointer ::c1

type(st_chaine_calcul), pointer ::pprec,psuiv

! on relie l'item d'avant avec l'item d'après
if (flag_calculat_affiche.and.rang==RANG_MAITRE_AFF) print '(T10,"Supression item n°",I3)',c1%posi

!print '(T10,"Circuit précédent = ",A," et circuit suivant =",A)',&
!      trim(c1%pprec%nom_circuit),trim(c1%psuiv%nom_circuit)

nullify(pprec);nullify(psuiv)
pprec=>c1%pprec
psuiv=>c1%psuiv

nullify(c1%pprec)
nullify(c1%psuiv)
deallocate(c1)
if (associated(psuiv)) then 
  psuiv%pprec => pprec
end if

if (associated(pprec)) then
  pprec%psuiv => psuiv
end if


end subroutine suppr_item_liste

! ==========================================================
! == Subroutine stop_calculat                      ==
!  Affichage message d'erreur si problème
! ==========================================================
subroutine stop_calculat(numerr,nuc,ch,numligne)
integer, intent(in) :: numerr,nuc,numligne
character(LEN=*), intent(in) :: ch

character (LEN=15) :: ch_numligne

select case (numerr)
case(-1)
  print '("Erreur, conflit d''accolade de type (],[),(},{)... :",A)',trim(ch)
case(0)
  print '("Erreur, nombre d''accolade incorrect dans :",A)',trim(ch)
case(1)
  print '("Erreur, caractère incorrect vers la position ",I3," dans :",A)',nuc,trim(ch)
case(2)
  print '("Erreur, manque opérateur arithmétique à la position ",I3," dans :",A)',nuc,trim(ch)
case(3)
  print '("Erreur, non de la variable incorrect à la position ",I3,", variable=",A)',nuc,trim(ch)
case(4)
  print '("Erreur, opérateur gauche manquant dans l''évaluation de ",A)',trim(ch)
case(5)
  print '("Erreur, accolade droite imprévu dans l''évaluation de ",A)',trim(ch)
case(6)
  print '("Erreur, impossible de faire l''évaluation avec ",A)',trim(ch)
case(7)
  print '("Erreur, problème d''interprétation de l''éuqation formelle :",A)',trim(ch)
case(8)
  print '("Erreur, valeur flottante incorrecte, problème près de la position ",I3," dans ",A)',nuc,trim(ch)
case(9)
  print '("Erreur, fonction mathématique incorrecte à la position ",I3," dans ",A)',nuc,trim(ch)
case(10)
  print '("Erreur, équation formelle incorrecte après la dernière valeur dans ",A)',trim(ch)
end select

ch_numligne = entier_chaine(numligne)
print '(/,t10,"Arret de la lecture a la ligne ", A, " du fichier de modélisation.")', TRIM(ch_numligne)

print '("Erreur détectée dans la subroutine calculat_reel, ARRET !!!")'
stop
end subroutine stop_calculat


! ==========================================================
! == Subroutine affichage_etat_eq_formelle                ==
! ==========================================================
subroutine affichage_etat_eq_formelle(cc)
type(st_chaine_calcul), pointer :: cc

type(st_chaine_calcul), pointer :: c1
character(LEN=LEN_CHAINE) :: ch1
character(LEN=LEN_MOT_CLE) :: ch2

integer :: i

c1=>cc
ch1 = ''
do
  do i=1,c1%nopav
    select case (c1%opavant(i))
    case (OP_DEB_P)
      ch1 = trim(ch1)//'(';
    case (OP_FIN_P)
      ch1 = trim(ch1)//')';
    case (OP_MOINS)
      ch1 = trim(ch1)//'-';
    case (OP_PLUS)
      ch1 = trim(ch1)//'+';
    case (OP_PROD)
      ch1 = trim(ch1)//'*';
    case (OP_DIV)
      ch1 = trim(ch1)//'/';
    case (OP_EXPO)
      ch1 = trim(ch1)//'^';
    case default
      ch1 = trim(ch1)//'?';      
    end select  
  end do
  if (c1%flag_fonction) then
    ch1=trim(ch1)//trim(c1%fonction)
  else
    write (ch2,'(ES9.2)') c1%valeur
    ch1=trim(ch1)//trim(ch2)    
  end if
  if (.not.associated(c1%psuiv)) exit
  c1=>c1%psuiv
end do

if (rang==RANG_MAITRE_AFF) print '(T5,"Chaine restant à traiter :",A)',trim(ch1)

end subroutine affichage_etat_eq_formelle


end module m_calculat
