!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_chaine_operation.f90
!// Purpose:  Opération sur les chaines de caractère
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Mai 2007
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
module m_chaine_operation
use m_constantes, only : LEN_CHAINE, LEN_MOT_CLE,rang,RANG_MAITRE_AFF

implicit none


contains

! ==========================================================
! == Fonction LowerCaseChaine                             ==
! == Convertit tous les caractères en majuscule de l'alphabet    ==
! == en minuscule dans la chaine ch
! ==========================================================
function LowerCaseChaine(ch) result(chnew)
!=== Déclaration des arguments ===
character(LEN=*), intent(in) :: ch

!===  Déclaration des variables  ===
character(LEN=len(ch))  :: chnew

integer :: i,l,n,id
!===   Début des traitements   ===
chnew = ch
l=len_trim(ch);
n=32 ! Saut décimal de "A" à "a" 
do i=1,l
  id = IACHAR(ch(i:i)) 
  if (id >=65 .and.id<=90 ) chnew(i:i) = achar(id+n)
end do
end function LowerCaseChaine

! ==========================================================
! == Fonction UpperCaseChaine                             ==
! == Convertit tous les caractères en minuscule de l'alphabet    ==
! == en majuscule dans la chaine ch
! ==========================================================
function UpperCaseChaine(ch) result(chnew)
!=== Déclaration des arguments ===
character(LEN=*), intent(in) :: ch

!===  Déclaration des variables  ===
character(LEN=len(ch))  :: chnew

integer :: i,l,n,id
!===   Début des traitements   ===
chnew = ch
l=len_trim(ch);
n=32 ! Saut décimal de "A" à "a" 
do i=1,l
  id = IACHAR(ch(i:i)) 
  if (id >=97 .and.id<=122) chnew(i:i) = achar(id-n)
end do
end function UpperCaseChaine


! ==========================================================
! == Fonction LowerCaseChar                               ==
! == Convertit le caractère en majuscule de l'alphabet    ==
! == en minuscule 
! ==========================================================
function LowerCaseChar(c) result(cnew)
!=== Déclaration des arguments ===
character, intent(in) :: c

!===  Déclaration des variables  ===
character  :: cnew
integer :: id
!===   Début des traitements   ===
cnew = c
id = IACHAR(c) 
if (id >=65 .and.id<=90 ) cnew = achar(id+32) !32 --> Saut décimal de "A" à "a" 
end function LowerCaseChar

! ==========================================================
! == Fonction UpperCaseChar                               ==
! == Convertit le caractère en majuscule de l'alphabet    ==
! == en minuscule 
! ==========================================================
function UpperCaseChar(c) result(cnew)
!=== Déclaration des arguments ===
character, intent(in) :: c

!===  Déclaration des variables  ===
character  :: cnew
integer :: id
!===   Début des traitements   ===
cnew = c
id = IACHAR(c) 
if (id >=97 .and.id<=122) cnew = achar(id-32) !32 --> Saut décimal de "A" à "a" 
end function UpperCaseChar

! ==========================================================
! == Fonction entier_chaine                               ==
! == Convertit un entier en chaine, les blancs à gauche   ==
! == sont retirés                                         ==
! ==========================================================
function entier_chaine(i) result(chaine)

integer, intent(in) :: i

character(LEN=30) :: chaine
write(chaine,*) i
chaine = ADJUSTL(chaine)

end function entier_chaine

!===========================================================	
! Function nb_mot_chaine
! Compte le nombre de mot de la chaine en se basant sur
! le séparateur de mot (espace par défaut)
!===========================================================	
integer function nb_mot_chaine(chaine,separateur)
!=== Déclaration des arguments ===
character(LEN=LEN_CHAINE), intent(in) :: chaine
character, optional, intent(in) :: separateur
!===  Déclaration des variables  ===
integer  :: i,lchaine,compteur
character  :: sep
character(LEN=LEN_CHAINE) :: chaine1
!===   Début des traitements   ===
sep = ' ';
if (PRESENT(separateur)) sep = separateur
chaine1 = ADJUSTL(chaine)
lchaine = LEN_TRIM(chaine1)

compteur = 0
!print '("lchaine1",I)',lchaine
!print '("chaine1 = ",A)',TRIM(chaine1)
if (lchaine==0) then
  nb_mot_chaine = 0
  return
end if
do i=2,lchaine
  !if (chaine1(i:i) == COMMENTAIRE) then
  !  compteur = compteur - 1
  !  exit 
  !end if 
  if ((chaine1(i:i) == sep).and.(chaine1(i-1:i-1) /= sep)) &
    compteur = compteur + 1
  
end do				
nb_mot_chaine = compteur + 1;
end function nb_mot_chaine

!===========================================================	
! Function nb_mot_chaine
! Compte le nombre de mot de la chaine en se basant sur
! le séparateur de mot (espace par défaut)
!===========================================================	
integer function nb_mot_chaine_long(chaine,longueur,separateur)
!=== Déclaration des arguments ===
integer, intent(in)  :: longueur
character(LEN=longueur), intent(in) :: chaine
character, optional, intent(in) :: separateur
!===  Déclaration des variables  ===
integer  :: i,lchaine,compteur
character  :: sep
character(LEN=longueur) :: chaine1
!===   Début des traitements   ===
sep = ' ';
if (PRESENT(separateur)) sep = separateur
chaine1 = ADJUSTL(chaine)
lchaine = LEN_TRIM(chaine1)

compteur = 0
do i=2,lchaine
  !if (chaine1(i:i) == COMMENTAIRE) then
  !  compteur = compteur - 1
  !  exit 
  !end if 
  if ((chaine1(i:i) == sep).and.(chaine1(i-1:i-1) /= sep)) &
    compteur = compteur + 1
  
end do				
nb_mot_chaine_long = compteur + 1;
end function nb_mot_chaine_long

!===========================================================	
! Function extrait_sous_chaine
! Retourne une sous-chaine composé des m1 mots à m2 mots
!===========================================================	
function extrait_sous_chaine(chaine,m1,m2,separateur)

!=== Déclaration des arguments ===
character(LEN=LEN_CHAINE), intent(in) :: chaine
integer, intent(in) :: m1
integer, optional, intent(in) :: m2
character, optional, intent(in) :: separateur

character(LEN=LEN_CHAINE) :: extrait_sous_chaine
!===  Déclaration des variables  ===
integer  :: i,lchaine,compteur, mfin,ideb,ifin
character  :: sep
character(LEN=LEN_CHAINE) :: chaine1
!===   Début des traitements   ===
sep = ' ';
mfin = m1;
if (present(m2)) mfin = m2
if (PRESENT(separateur)) sep = separateur
!print '("mfin=",I)',mfin
chaine1 = ADJUSTL(chaine)
lchaine = LEN_TRIM(chaine1)+1

!print '("chaine1=",A)',TRIM(chaine1)
!print '("lchaine=",I)',lchaine

compteur = 1
ideb = 0
ifin = 0
! Indice début
do i=2,lchaine
   ! modification 26/04/2006
   ideb = i
  if (m1 == compteur) then
    ideb = i-1
    exit
  end if
  if ((chaine1(i:i) /= sep).and.(chaine1(i-1:i-1) == sep)) &
    compteur = compteur + 1  
end do				

! Indice fin
compteur = 0
 ! modification 26/04/2006
 ifin = ideb
do i=2,lchaine
  if ((chaine1(i:i) == sep).and.(chaine1(i-1:i-1) /= sep)) &
    compteur = compteur + 1  
  if (mfin == compteur) then
    ifin = i-1
    exit
  end if
end do				

extrait_sous_chaine = chaine1(ideb:ifin)
end function extrait_sous_chaine


end module m_chaine_operation
