!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_constantes.f90
!// Purpose:     Liste de constantes commune aux autres modules
!      Simulateur FDTD
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Constantes non scientifiques                         ==  
! == Module module_constantes                             ==
!===========================================================
module m_constantes

implicit none
integer, parameter  :: SIMPLE_PRECISION  = 6
integer, parameter  :: DOUBLE_PRECISION  = 12
#ifdef FLAG_DP_TEMSI
  integer, parameter  :: SDP  = selected_real_kind(DOUBLE_PRECISION)
#else
  integer, parameter  :: SDP  = selected_real_kind(SIMPLE_PRECISION)
#endif

integer, parameter :: LEN_CHAINE = 255
integer, parameter :: LEN_MOT_CLE= 40

integer, parameter :: CPML           = 1
integer, parameter :: MUR_MAGNETIQUE = 2
integer, parameter :: MUR_ELECTRIQUE = 3
integer, parameter :: SANS_FRONTIERE = 4
integer, parameter :: FLOQUET        = 5
integer, parameter :: FRONTIERE_MPI  = 6

integer, parameter :: PLAN_XOY       = 1
integer, parameter :: PLAN_YOZ       = 2
integer, parameter :: PLAN_XOZ       = 3
integer, parameter :: PLAN_ZOX       = 3
integer, parameter :: PLAN_QCQ       = 4  !plan quelconque

integer, parameter :: VOL_1D  = 1
integer, parameter :: VOL_2D  = 2
integer, parameter :: VOL_3D  = 3

integer, parameter :: DIRECTION_X       = 1
integer, parameter :: DIRECTION_Y       = 2
integer, parameter :: DIRECTION_Z       = 3
integer, parameter :: DIR_X       = 1
integer, parameter :: DIR_Y       = 2
integer, parameter :: DIR_Z       = 3

integer, parameter :: XMIN       = 1
integer, parameter :: XMAX       = 2
integer, parameter :: YMIN       = 3
integer, parameter :: YMAX       = 4
integer, parameter :: ZMIN       = 5
integer, parameter :: ZMAX       = 6
integer, parameter :: XOY_INF       = 1
integer, parameter :: XOY_SUP       = 2
integer, parameter :: YOZ_INF       = 3
integer, parameter :: YOZ_SUP       = 4
integer, parameter :: XOZ_INF       = 5
integer, parameter :: XOZ_SUP       = 6
integer, parameter :: ZINF       = 1
integer, parameter :: ZSUP       = 2
integer, parameter :: XINF       = 3
integer, parameter :: XSUP       = 4
integer, parameter :: YINF       = 5
integer, parameter :: YSUP       = 6
integer, dimension(3), parameter :: PC1=(/2,3,1/) ! permutation circulaire de 1
integer, dimension(3), parameter :: PC2=(/3,1,2/) ! permutation circulaire de 2
integer, dimension(3), parameter :: PC21=(/1,3,5/) ! permutation circulaire de 2 à partir de 1
integer, dimension(3), parameter :: PC23=(/3,5,1/) ! permutation circulaire de 2 à partir de 3
integer, dimension(3), parameter :: PC25=(/5,1,3/) ! permutation circulaire de 2 à partir de 5
LOGICAL, dimension(0:1), parameter :: PCL0=(/.false.,.true./) ! Conversion en booléen

character(LEN=2), dimension(6), parameter :: COMPOSANTE_EH = &
                    (/'ex','ey','ez','hx','hy','hz'/) 
character(LEN=2), dimension(8), parameter :: LISTE_EH = &
                    (/'ex','ey','ez','hx','hy','hz','e ','h '/) 

integer, parameter :: VOL_DIELEC_HOMOGENE          =1
integer, parameter :: VOL_DIELEC_PERTES_HOMOGENE   =2
integer, parameter :: VOL_DIELEC_INHOMOGENE        =3
integer, parameter :: VOL_DIELEC_PERTES_INHOMOGENE =4
integer, parameter :: VOL_MAGNET_HOMOGENE          =1
integer, parameter :: VOL_MAGNET_PERTES_HOMOGENE   =2
integer, parameter :: VOL_MAGNET_INHOMOGENE        =3
integer, parameter :: VOL_MAGNET_PERTES_INHOMOGENE =4

integer, parameter :: VOL_DUREE_MANUEL  =0
integer, parameter :: VOL_DUREE_AUTO    =1
integer, parameter :: VOL_DUREE_SEMIAUTO=2
integer, parameter :: NBRE_ITERATION_MAX_AUTO=500000

integer, parameter :: RANG_MAITRE_AFF=0

! déclaration d'une variable globale par rapport à MPI !
integer  :: rang=0 ! rang du processeur chargé de l'affichage

end module m_constantes	



