!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_entree.f90
!// Purpose:  Traite la lecture du fichier d'entrée, 
!             il contient un ensemble de fonctions destiné à faciliter 
!             la lecture des données utiles  
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Novembre 2001
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Module contenant la structure globale du programme   ==
! == qui intègrent toutes les autres structures           ==
! ==========================================================
module m_entree
#ifdef FLAG_MPI_TEMSIFD
use mpi
#endif

use m_constantes
use m_constantes_scientifiques
use m_chaine_operation
use m_es_variable
implicit none
! === CONSTANTES DU FICHIER D'ENTREE ==
character, parameter :: COMMENTAIRE = '#'
character, parameter :: SYMBOLE_DEBUTBLOC = '<'
character, parameter :: SYMBOLE_FINBLOC   = '>'

character(LEN=*), parameter :: FIN_FICHIER = 'fin_fichier'

integer, parameter :: MIN_UNIT   = 10
integer, parameter :: MAX_UNIT   = 10000


! ==========================================================
! == Type st_fichier                                      ==
! == Structure gérant la lecture du fichier d'entrée      ==
! ==========================================================

type st_fichier
  integer :: unite
  integer :: numligne
  integer :: numbackspace
  character(len=LEN_CHAINE) :: nom_fichier
  character(len=LEN_CHAINE) :: nom_repertoire
  character(len=LEN_CHAINE) :: chaine
  character(len=LEN_CHAINE) :: premiermot
  character(len=LEN_MOT_CLE) :: nombloc, nomfinbloc
  type (st_es_variable)     :: tvar
 end type st_fichier

contains

! ==========================================================
! == Subroutine generateur_initdefaut                     ==
! == Initialisation par défaut des variables              ==
! ==========================================================	
subroutine entree_sortie_initdefaut(f)

type(st_fichier), intent(inout) :: f

f%numbackspace = 0
f%numligne = 0
call es_variable_initdefaut(f%tvar);
end subroutine entree_sortie_initdefaut

! ==========================================================
! == Subroutine generateur_initdefaut                     ==
! == Libération mémoire                                   ==
! ==========================================================	
subroutine entree_deallocate(t_fichier)
type(st_fichier), intent(inout) :: t_fichier
!RIEN
end subroutine entree_deallocate

! ==========================================================
! == Subroutine entree_nettoyage_fichier                  ==
! ==========================================================	
subroutine entree_nettoyage_fichier(f)
type(st_fichier), intent(inout) :: f

character(len=LEN_CHAINE)::fichnouv
character(len=LEN_MOT_CLE)::premiermot
integer  :: unite,statut
if (rang==RANG_MAITRE_AFF) print '(T5,"--- Nettoyage des données du fichier d''entrée ---")'

!--> Les données sont transférées dans un fichier de meme nom
! et préfixé par mod_
fichnouv = trim(f%nom_fichier)//'_mod'
unite = unitefichier()
if (unite<=0) stop
if (rang==RANG_MAITRE_AFF) then
  open(unite,file=fichnouv,status = 'replace',action='write',iostat=statut)
  if (statut>0) then
    print '(/,t10,"ATTENTION, ouverture de ",A," impossible => STOP !!!")', &
      TRIM(fichnouv)
    stop
  endif
  do
    premiermot = lineread_mini(f)
    write (unite,'(A)') trim(f%chaine)
    if (premiermot==FIN_FICHIER) exit
  end do 
  close(unite)
end if
close (f%unite)
! Initialisation
f%unite = unite
f%numbackspace = 0
f%numligne = 0

#ifdef FLAG_MPI_TEMSIFD
!DEC$ IF DEFINED (FLAG_MPI_TEMSIFD)
call MPI_BARRIER(MPI_COMM_WORLD,statut)
!DEC$ ENDIF
#endif
! -- Ouvre le fichier d'entree en lecture --
open(unite,file=fichnouv,status = 'old',action='read',iostat=statut)
if (statut>0) then
  print '(/,t10,"ATTENTION, ouverture de ",A," impossible => STOP !!!")', &
    TRIM(fichnouv)
  stop
endif

end subroutine entree_nettoyage_fichier


! ==========================================================
! == Subroutine fichier_lecture                           ==
! == Lecture dans le fichier d'entrée...                  ==
!===========================================================	
function lineread(t_fichier,back) result(premiermot)

type(st_fichier), target, intent(inout) :: t_fichier
integer, optional, intent(in) :: back

integer  :: lchaine,i
character :: tabulation
character(len=LEN_CHAINE) :: chaine
character(len=LEN_MOT_CLE) :: premiermot, longueur

longueur = entier_chaine(LEN_CHAINE)
tabulation = ACHAR(9)
premiermot=''
do 
  chaine = '';
  read(t_fichier%unite,'(a'//TRIM(longueur)//')',end=999) chaine
  
  !------------------------------------------------- 
!  chaine = ADJUSTL(chaine) !on retire les blancs de début de la chaine
  
  ! compteur ligne 
  t_fichier%numligne = t_fichier%numligne + 1
  if (PRESENT(back)) t_fichier%numbackspace = t_fichier%numbackspace + back
  
  if (chaine=='') then
    ! chaine vide
    cycle
  elseif (chaine(1:1)==SYMBOLE_DEBUTBLOC) then
    ! Début d'un bloc
    premiermot = chaine(2:)
    t_fichier%nombloc = premiermot 
    !construction du nom de fin de bloc
    lchaine = LEN_TRIM(premiermot)
    t_fichier%nomfinbloc = premiermot(1:lchaine) // SYMBOLE_FINBLOC
    
  !--------------- Une chaine de données utiles -----------------------
  else
    ! une chaine de données
    read(chaine,*) premiermot
  endif

  t_fichier%premiermot = premiermot
  t_fichier%chaine = chaine
  return
enddo

999 premiermot=FIN_FICHIER
    t_fichier%chaine = SYMBOLE_DEBUTBLOC//premiermot//SYMBOLE_FINBLOC
    return

end function lineread

! ==========================================================
! == Subroutine lineread_mini                           ==
! == Lecture dans le fichier d'entrée...                  ==
! --> pour le netoyage du fichier
!===========================================================	
function lineread_mini(t_fichier) result(premiermot)

type(st_fichier), target, intent(inout) :: t_fichier

integer  :: lchaine,i
character :: tabulation
character(len=LEN_CHAINE) :: chaine, chaine_utile
character(len=LEN_MOT_CLE) :: premiermot, longueur

longueur = entier_chaine(LEN_CHAINE)
tabulation = ACHAR(9)
premiermot=''

do 
  chaine = '';
  read(t_fichier%unite,'(a'//TRIM(longueur)//')',end=999) chaine
  !---- Remplacement tabulation par des blancs ----
  lchaine = LEN_TRIM(chaine)
  do i=1,lchaine
    if (chaine(i:i)==tabulation) then
      chaine(i:i) = ' ';
    end if
  end do
  
  !------------------------------------------------- 
  chaine = ADJUSTL(chaine) !on retire les blancs de début de la chaine
  
  !--- on retire les commentaires en bout de chaine ---
  i = INDEX(chaine,COMMENTAIRE)
  chaine_utile = ''
  if (i <= 0) then
    chaine_utile = chaine
  else if (i>1) then
    chaine_utile = chaine (1:i-1)
  end if
  
  !---- Remplacement virgule et point virgule par des blancs ----
  lchaine = LEN_TRIM(chaine_utile)
  do i=1,lchaine
    if (chaine_utile(i:i)==','.or.chaine_utile(i:i)==';') then
      chaine_utile(i:i) = ' ';
    end if
  end do
  
  ! compteur ligne 
  t_fichier%numligne = t_fichier%numligne + 1
  
  if ((chaine=='').or.(chaine(1:1)==COMMENTAIRE)) then
    chaine_utile = ''
    ! chaine vide ou chaine commentée : rien
    !cycle
  elseif (chaine_utile(1:1)==SYMBOLE_DEBUTBLOC) then
    ! Début d'un bloc
    premiermot = chaine_utile(2:)
    t_fichier%nombloc = premiermot 
    !construction du nom de fin de bloc
    lchaine = LEN_TRIM(premiermot)
    t_fichier%nomfinbloc = premiermot(1:lchaine) // SYMBOLE_FINBLOC
    
  else if (chaine_utile(1:1)==SYMBOLE_VARIABLE) then
    call es_variable_lecture(t_fichier%tvar,chaine_utile,t_fichier%numligne)
    chaine_utile=''
  !--------------- Une chaine de données utiles -----------------------
  else
    ! une chaine de données
    read(chaine_utile,*) premiermot
    call es_variable_conv(t_fichier%tvar,lchaine,chaine_utile,t_fichier%numligne)
  endif

  t_fichier%premiermot = premiermot
  t_fichier%chaine = chaine_utile
  return
enddo

999 premiermot=FIN_FICHIER
    t_fichier%chaine = SYMBOLE_DEBUTBLOC//FIN_FICHIER//SYMBOLE_FINBLOC
    return

end function lineread_mini

! ==========================================================
! == Subroutine lineread_all                              ==
! == Lecture dans le fichier d'entrée...                  ==
!===========================================================	
function lineread_all(t_fichier,back) result(premiermot)

type(st_fichier), target, intent(inout) :: t_fichier
integer, optional, intent(in) :: back

integer  :: lchaine,i
character :: tabulation
character(len=LEN_CHAINE) :: chaine, chaine_utile
character(len=LEN_MOT_CLE) :: premiermot, longueur

longueur = entier_chaine(LEN_CHAINE)
tabulation = ACHAR(9)
do 
  chaine = '';
  read(t_fichier%unite,'(a'//TRIM(longueur)//')',end=999) chaine
  !---- Remplacement tabulation par des blancs ----
  lchaine = LEN_TRIM(chaine)
  do i=1,lchaine
    if (chaine(i:i)==tabulation) then
      chaine(i:i) = ' ';
    end if
  end do
  
  !------------------------------------------------- 
  chaine = ADJUSTL(chaine) !on retire les blancs de début de la chaine
  
  !--- on retire les commentaires en bout de chaine ---
  i = INDEX(chaine,COMMENTAIRE)
  chaine_utile = ''
  if (i <= 0) then
    chaine_utile = chaine
  else if (i>1) then
    chaine_utile = chaine (1:i-1)
  end if
  
  !---- Remplacement virgule et point virgule par des blancs ----
  lchaine = LEN_TRIM(chaine_utile)
  do i=1,lchaine
    if (chaine_utile(i:i)==','.or.chaine_utile(i:i)==';') then
      chaine_utile(i:i) = ' ';
    end if
  end do
  
  ! compteur ligne 
  t_fichier%numligne = t_fichier%numligne + 1
  if (PRESENT(back)) t_fichier%numbackspace = t_fichier%numbackspace + back
  
  if ((chaine=='').or.(chaine(1:1)==COMMENTAIRE)) then
    ! chaine vide ou chaine commentée
    cycle
  elseif (chaine_utile(1:1)==SYMBOLE_DEBUTBLOC) then
    ! Début d'un bloc
    premiermot = chaine_utile(2:)
    t_fichier%nombloc = premiermot 
    !construction du nom de fin de bloc
    lchaine = LEN_TRIM(premiermot)
    t_fichier%nomfinbloc = premiermot(1:lchaine) // SYMBOLE_FINBLOC
    
  else if (chaine_utile(1:1)==SYMBOLE_VARIABLE) then
    t_fichier%chaine = chaine_utile
    call es_variable_lecture(t_fichier%tvar,chaine_utile,t_fichier%numligne)
    cycle
  !--------------- Une chaine de données utiles -----------------------
  else
    ! une chaine de données
    read(chaine_utile,*) premiermot
    call es_variable_conv(t_fichier%tvar,lchaine,chaine_utile,t_fichier%numligne)
  endif

  t_fichier%premiermot = premiermot
  t_fichier%chaine = chaine_utile
  return
enddo

999 premiermot=FIN_FICHIER
    t_fichier%chaine = SYMBOLE_DEBUTBLOC//FIN_FICHIER//SYMBOLE_FINBLOC
    return

end function lineread_all


! ==========================================================
! == Function unitefichier                                ==
! == Determine un numero de fichier non utilise           ==
! ==========================================================
integer function unitefichier()

integer :: num
logical :: existence

unitefichier = -1 ! pas  d'unité disponible
do num=MIN_UNIT,MAX_UNIT
  inquire(unit=num,opened=existence)
  if (.not.existence) then
    unitefichier = num
    exit
  end if
enddo

if (num == MAX_UNIT+1) print '(t10," Dépassement du n° d''unité max = ",I6)',MAX_UNIT
if (unitefichier == -1) print '(t10," Pas d''unité de fichiers trouvé !!!")'

end function unitefichier


! ==========================================================
! == Subroutine numbackspace_set                          ==
! == Positionnne la valeur de numbackspace a val          ==
! == Si val asbsent positionne numlbackspace a 0          ==
! ==========================================================
subroutine numbackspace_set(t_fichier,val)

type(st_fichier), intent(inout) :: t_fichier
integer, optional, intent(in) :: val

integer :: val1

if (.not.PRESENT(val)) then
  val1 = 0
else
  val1 = val
endif

t_fichier%numbackspace = val1

end subroutine numbackspace_set


! ==========================================================
! == function multibackspace                              ==
! == Remonte dans le fichier de n enregistrements         ==
! ==========================================================
integer function multibackspace(t_fichier)

type(st_fichier), intent(inout) :: t_fichier

integer :: i

multibackspace = 0

do i=1,t_fichier%numbackspace
  backspace(t_fichier%unite)
  multibackspace = multibackspace + 1
  t_fichier%numligne = t_fichier%numligne - 1
enddo

t_fichier%numbackspace = 0

end function multibackspace

! ==========================================================
! == function temsi_rewind                                ==
! == Remonte au début du fichier                          ==
! ==========================================================
subroutine temsi_rewind(t_fichier)

type(st_fichier), intent(inout) :: t_fichier

rewind(t_fichier%unite)
t_fichier%numligne     = 0
t_fichier%numbackspace = 0
t_fichier%nombloc    = ""
t_fichier%nomfinbloc = ""

end subroutine temsi_rewind


! ==========================================================
! == Subroutine ligne_precedente                          ==
! == Remonte d'une ligne en mettant a jour numbackspace   ==
! ==========================================================
subroutine ligne_precedente(t_fichier)

type(st_fichier), intent(inout) :: t_fichier

t_fichier%numbackspace = t_fichier%numbackspace - 1
t_fichier%numligne = t_fichier%numligne - 1
backspace(t_fichier%unite)

end subroutine ligne_precedente


! ==========================================================
! == Subroutine lecture_stop                              ==
! ==========================================================
subroutine lecture_stop(f)

type(st_fichier), intent(in) :: f
character (LEN=15) :: ch_numligne

ch_numligne = entier_chaine(f%numligne)
print '(/,t10,"Arret de la lecture a la ligne ", A, &
  &" du fichier ", A)', TRIM(ch_numligne), TRIM(f%nom_fichier)
  
print '(t10,"Ligne ", A, " : ", A, ".")', TRIM(ch_numligne), TRIM(f%chaine) 
stop

end subroutine lecture_stop


! ==========================================================
! == Fonction len_int                                     ==
! == Calcul le nombre de caracteres que prend un entier   ==
! ==========================================================
integer function len_int(i)

integer, intent(in) :: i

character(LEN=LEN_CHAINE) :: i_chaine
write(i_chaine,*) i
i_chaine = ADJUSTL(i_chaine)

len_int = LEN_TRIM(i_chaine)

end function len_int


!===========================================================	
! Function ctrl_nb_mot_chaine
! Controle si le nombre de mot de la chaine est correct
! Dans le cas contraire le programme est arreté
!===========================================================	
integer function ctrl_nb_mot_chaine(f,chaine,nbmin,nbmax)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout):: f
character(LEN=LEN_CHAINE), intent(in) :: chaine
integer, intent(in)  :: nbmin
integer, optional, intent(in)  :: nbmax

!===  Déclaration des variables  ===
integer  :: i, nb_mot,nb_mot_max
!===   Début des traitements   ===
nb_mot_max = nbmin
if (PRESENT(nbmax)) nb_mot_max = nbmax
nb_mot = nb_mot_chaine(chaine)
ctrl_nb_mot_chaine = nb_mot
if ((nb_mot <nbmin).or.(nb_mot>nb_mot_max)) then
  print '("ERREUR, Le nombre de mot doit etre compris entre",I3," et",I3," dans la chaine :")',nbmin,nb_mot_max
  print '(A)',TRIM(chaine)
  ! Arret du programme
  call lecture_stop(f)
  ctrl_nb_mot_chaine = -1
end if 


end function ctrl_nb_mot_chaine

!===========================================================	
! Function compteur_ligne_bloc
! Compte le nombre de ligne utile jusqu'à la fin du bloc
! Remonte ensuite à la ligne de départ
!===========================================================	
integer function compteur_ligne_bloc(f)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout):: f

!===  Déclaration des variables  ===
integer  :: compteur,i
character(LEN=LEN_CHAINE)  :: premier_mot

compteur = 0
call numbackspace_set(f,0) ! compteur retour ligne dans fichier à 0
do
  premier_mot = lineread(f,1)
  if (premier_mot==f%nomfinbloc) then
    i = multibackspace(f)
    exit
  else
    compteur = compteur+1
  end if
enddo

compteur_ligne_bloc = compteur

end function compteur_ligne_bloc


!===========================================================	
! Function compteur_mot_cle
! Compte le nombre de mot clé de la ligne en cours
! de lecture dans le fichier d'entrée jusqu'au nom de fin bloc
! ou le mot clé de fin (chainefin).
! Remonte ensuite à la ligne de départ
!===========================================================	
integer function compteur_mot_cle(f,mot_cle,chaine_fin)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout):: f
! longueur chaine de mot_cle non défini à cause des constantes chaines
character(LEN=*), intent(in) :: mot_cle 
character(LEN=*), optional, intent(in) :: chaine_fin

!===  Déclaration des variables  ===
integer  :: compteur,i,l,l1
character(LEN=LEN_CHAINE)  :: premier_mot
character(LEN=LEN_CHAINE)  :: ch_fin

compteur = 0

call numbackspace_set(f,0) ! compteur retour ligne dans fichier à 0
if (PRESENT(chaine_fin)) then
  ch_fin = chaine_fin
else
  ch_fin = f%nomfinbloc
end if
l1 = len_trim(ch_fin)

l = len_trim(mot_cle);
do
  premier_mot = lineread(f,1)
  if ((premier_mot==f%nomfinbloc).or.(premier_mot(1:l1)==ch_fin(1:l1))) then
    i = multibackspace(f)
    exit
  end if
  if (premier_mot(1:l)==mot_cle) then
    compteur = compteur + 1
    cycle
  endif
enddo

if ((compteur == 0) .and. (rang==RANG_MAITRE_AFF)) then
  print '("Remarque : Aucun mot cle : ",A)',TRIM(mot_cle)    
end if  

compteur_mot_cle = compteur

end function compteur_mot_cle

!===========================================================	
! Function compteur_N_mot_cle
! Compte le nombre de N mots clés de la ligne en cours
! de lecture dans le fichier d'entrée jusqu'au nom de fin bloc
! ou le mot clé de fin (chainefin).
! Remonte ensuite à la ligne de départ
!===========================================================	
function compteur_N_mot_cle(f,N, n_mots_cles,chaine_fin) result(N_compteur)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout):: f
integer, intent(in)            :: N
! longueur chaine de n_mots_cles non défini à cause des constantes chaines
character(LEN=*),dimension(1:N), intent(in) :: n_mots_cles
character(LEN=*), optional, intent(in) :: chaine_fin

integer, dimension(1:N)  :: N_compteur

!===  Déclaration des variables  ===
integer  :: i
integer,dimension(N)  :: l
character(LEN=LEN_CHAINE)  :: premier_mot
character(LEN=LEN_CHAINE)  :: ch_fin

N_compteur(:) = 0

call numbackspace_set(f,0) ! compteur retour ligne dans fichier à 0
if (PRESENT(chaine_fin)) then
  ch_fin = chaine_fin
else
  ch_fin = f%nomfinbloc
end if

do i=1,N
  l(i)= LEN_TRIM(n_mots_cles(i))
end do

do
  premier_mot = lineread(f,1)
  if ((premier_mot==f%nomfinbloc).or.(premier_mot==ch_fin)) then
    i = multibackspace(f)
    exit
  end if
  do i=1,N
    if (premier_mot(1:l(i))== n_mots_cles(i)) then
      N_compteur(i) = N_compteur(i) + 1
      cycle
    endif
  end do
enddo

!if (ALL(N_compteur == 0)) then
!  print '("Avertissement : Pas de mots clés")'  
!end if  

end function compteur_N_mot_cle


!===========================================================	
! Function apres_fin_bloc
! Se déplace à la ligne juste après la fin du bloc
!===========================================================	
subroutine apres_fin_bloc(f)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout):: f

!===  Déclaration des variables  ===
character(LEN=LEN_CHAINE)  :: premier_mot

do
  premier_mot = lineread(f,1)
  if (premier_mot==f%nomfinbloc) then
    exit
  end if
end do
end subroutine apres_fin_bloc


! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module m_entree
