!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_es_variable.f90
!// Purpose:  Traitement des variables du fichier d'entrée
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Mai 2007
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
module m_es_variable
use m_constantes, only : LEN_CHAINE, LEN_MOT_CLE,rang,RANG_MAITRE_AFF
use m_constantes_scientifiques
use m_chaine_operation
use m_calculat
implicit none

!======== LES CONSTANTES =========
character, parameter :: SYMBOLE_VARIABLE = '$'
character(LEN=2), parameter :: SYMBOLE_VARREEL = '$R'
character(LEN=2), parameter :: SYMBOLE_VARENTIER = '$I'
character(LEN=2), parameter :: SYMBOLE_VARCHAINE = '$C'

integer, parameter :: NBRE_VARIABLE_MAX   = 200


type st_es_variable
  !compteur
  integer :: nr,ni,nc
  ! Gestion des variables par type
  character(LEN=LEN_MOT_CLE), dimension(NBRE_VARIABLE_MAX) :: nomvarR,nomvarI,nomvarC
  character(LEN=20), dimension(NBRE_VARIABLE_MAX) :: chvalvarR
  character(LEN=10), dimension(NBRE_VARIABLE_MAX) :: chvalvarI
  
  character(LEN=LEN_MOT_CLE), dimension(NBRE_VARIABLE_MAX) :: valvarC
  real,dimension(NBRE_VARIABLE_MAX)    :: valvarR 
  integer,dimension(NBRE_VARIABLE_MAX) :: valvarI 
end type st_es_variable

contains

! ==========================================================
! == Subroutine es_variable_initdefaut                    ==
! == Initialisation par défaut des variables              ==
! ==========================================================	
subroutine es_variable_initdefaut(tvar)

type(st_es_variable), intent(inout) :: tvar

integer :: i
tvar%nr=0;tvar%ni=0;tvar%nc=0;
tvar%nomvarI(:) = ''
tvar%nomvarR(:) = ''
tvar%nomvarC(:) = ''
tvar%valvarC(:) = ''
tvar%valvarI(:) = 0
tvar%valvarR(:) = 0.0

tvar%chvalvarR(:) = ''
tvar%chvalvarI(:) = ''

i=1;
tvar%valvarR(i) = PI;i=i+1
tvar%valvarR(i) = EPSILON_0;i=i+1
tvar%valvarR(i) = MU_0;i=i+1
tvar%valvarR(i) = CVIDE;i=i+1
tvar%valvarR(i) = ZVIDE;i=i+1
i=1;
tvar%nomvarR(i) = SYMBOLE_VARREEL//'pi';i=i+1
tvar%nomvarR(i) = SYMBOLE_VARREEL//'eps0';i=i+1
tvar%nomvarR(i) = SYMBOLE_VARREEL//'mu0';i=i+1
tvar%nomvarR(i) = SYMBOLE_VARREEL//'cvide';i=i+1
tvar%nomvarR(i) = SYMBOLE_VARREEL//'zvide';i=i+1
i=1;
write(tvar%chvalvarR(i),'(ES15.8E2)') PI;i=i+1
write(tvar%chvalvarR(i),'(ES15.8E2)') EPSILON_0;i=i+1
write(tvar%chvalvarR(i),'(ES15.8E2)') MU_0;i=i+1
write(tvar%chvalvarR(i),'(ES15.8E2)') CVIDE;i=i+1
write(tvar%chvalvarR(i),'(ES15.8E2)') ZVIDE;i=i+1
tvar%nr = i-1;

end subroutine es_variable_initdefaut

! ==========================================================
! == Subroutine lecture_variable_stop                     ==
! == arret car erreur dans le fichier d'entrée
! ==========================================================
subroutine lecture_variable_stop(chaine,numligne)
character(len=LEN_CHAINE),intent(in) :: chaine
integer, intent(in) :: numligne

character (LEN=15) :: ch_numligne

write(ch_numligne,*) numligne
ch_numligne = ADJUSTL(ch_numligne)

print '(/,T10,"Arret de la lecture a la ligne ", A," du fichier d''entrée")', TRIM(ch_numligne)
print '(T10,"Ligne ", A, " : ", A, ".")', TRIM(ch_numligne), TRIM(chaine) 
stop

end subroutine lecture_variable_stop


! ==========================================================
! == Subroutine es_variable_conv                          ==
! == Remplace les variables dans la chaine de caractère
! == par leurs valeurs            
! ==========================================================	
subroutine es_variable_conv(tvar,lchaine,chaine_utile,numligne)

!=== Déclaration des arguments ===
type(st_es_variable), intent(inout) :: tvar
integer, intent(in)             :: lchaine ! longueur de la chaine
character(len=LEN_CHAINE),intent(inout) :: chaine_utile
integer, intent(in)             :: numligne !n° de ligne dans le fichier

!===  Déclaration des variables  ===
character(len=LEN_CHAINE)  :: ch,chnew
character(len=LEN_MOT_CLE)  :: nomvar

integer :: i,j,k,num
logical :: flag
!===   Début des traitements   ===
flag = .false.
chnew = chaine_utile
k=0;i=0
do while (i<=lchaine)
  k=k+1;i=i+1
  if (chaine_utile(i:i) == SYMBOLE_VARIABLE) then
    nomvar = ''
    flag = .true.
    do j=i+1,lchaine+1
      if (chaine_utile(j:j) == ' ') then ! fin du nom de la variable
        nomvar=chaine_utile(i:j-1)
        select case (nomvar(1:2))
	  case(SYMBOLE_VARREEL)
	    num=es_variable_typeR_cherche(tvar,nomvar)
	    if (num<0) call lecture_variable_stop(chaine_utile,numligne)
	    chnew = chnew(1:k-1)//' '//trim(tvar%chvalvarR(num))//chaine_utile(j:lchaine)
	    i=j-1
	    k=k+len_trim(tvar%chvalvarR(num))
	    exit
	  case(SYMBOLE_VARENTIER)
	    num=es_variable_typeI_cherche(tvar,nomvar)
	    if (num<0) call lecture_variable_stop(chaine_utile,numligne)
	    chnew = chnew(1:k-1)//' '//trim(tvar%chvalvarI(num))//chaine_utile(j:lchaine)
	    i=j-1
	    k=k+len_trim(tvar%chvalvarI(num))
	    exit
	  case(SYMBOLE_VARCHAINE)
	    num=es_variable_typeC_cherche(tvar,nomvar)
	    if (num<0) call lecture_variable_stop(chaine_utile,numligne)
	    !chaine_utile = chaine_utile(1:i-1)//' '//trim(tvar%valvarC(num))//chaine_utile(j:lchaine)
	    chnew = chnew(1:k-1)//' '//trim(tvar%valvarC(num))//chaine_utile(j:lchaine)
	    i=j-1
	    k=k+len_trim(tvar%valvarC(num))
	    exit	  
	  case default
	    print '("Erreur, Type de variable non reconnu !!! : ",A)',trim(nomvar)
	    call lecture_variable_stop(chaine_utile,numligne)
	end select
      end if
    end do
  end if
end do  

if (flag.and.rang==RANG_MAITRE_AFF) then
  chaine_utile = chnew
  print '(T5,"chaine corrigée :",A)',trim(chaine_utile)
end if
end subroutine es_variable_conv

! ==========================================================
! == Subroutine es_variable_reel_convop                     ==
! == Remplace les variables dans la chaine de caractère
! == par leurs valeurs            
! ==========================================================	
subroutine es_variable_reel_convop(tvar,chaine_utile,numligne)

!=== Déclaration des arguments ===
type(st_es_variable), intent(inout) :: tvar
character(len=LEN_CHAINE),intent(inout) :: chaine_utile
integer, intent(in)             :: numligne !n° de ligne dans le fichier

!===  Déclaration des variables  ===
character(len=LEN_CHAINE)  :: ch,chnew
character(len=LEN_MOT_CLE)  :: nomvar
character :: chop_ap(7)
integer :: i,j,k,num
integer :: lchaine ! longueur de la chaine
logical :: flag
!===   Début des traitements   ===
chop_ap = (/' ','^','-','*','+','/',')'/);
flag = .false.
lchaine = len_trim(chaine_utile)
chnew = chaine_utile
k=0;i=0
do while (i<=lchaine)
  k=k+1;i=i+1
  if (chaine_utile(i:i) == SYMBOLE_VARIABLE) then
    nomvar = ''
    flag = .true.
    do j=i+1,lchaine+1
      if (any(chaine_utile(j:j)==chop_ap)) then ! fin du nom de la variable
        nomvar=chaine_utile(i:j-1)
        select case (nomvar(1:2))
	  case(SYMBOLE_VARREEL)
	    num=es_variable_typeR_cherche(tvar,nomvar)
	    if (num<0) call lecture_variable_stop(chaine_utile,numligne)
	    chnew = chnew(1:k-1)//' '//trim(tvar%chvalvarR(num))//' '//chaine_utile(j:lchaine)
	    i=j-1
	    k=k+len_trim(tvar%chvalvarR(num))+1
	    exit
	  case default
	    print '("Erreur, Type de variable non reconnu !!! : ",A)',trim(nomvar)
	    call lecture_variable_stop(chaine_utile,numligne)
	end select
      end if
    end do
  end if
end do  

if (flag.and.rang==RANG_MAITRE_AFF) then
  chaine_utile = chnew
  print '(T5,"chaine corrigée :",A)',trim(chaine_utile)
end if
end subroutine es_variable_reel_convop


! ==========================================================
! == function es_variable_valeur_typeI                    ==
! == Retourne la valeur de la variable de type entier
! ==========================================================	
function es_variable_valeur_typeI(tvar,nomvar,numvar) result(valeur)

!=== Déclaration des arguments ===
type(st_es_variable), intent(inout) :: tvar
character(len=LEN_MOT_CLE),intent(in) :: nomvar
integer, intent(out)                :: numvar

integer :: valeur

valeur = 0
numvar = es_variable_typeI_cherche(tvar,nomvar)
if (numvar>0) then
  valeur = tvar%valvarI(numvar)
end if
end function es_variable_valeur_typeI

! ==========================================================
! == function es_variable_valeur_typeR                    ==
! == Retourne la valeur de la variable de type réel
! ==========================================================	
function es_variable_valeur_typeR(tvar,nomvar,numvar) result(valeur)

!=== Déclaration des arguments ===
type(st_es_variable), intent(inout) :: tvar
character(len=LEN_MOT_CLE),intent(in) :: nomvar
integer, intent(out)                :: numvar

real :: valeur

valeur = 0.0
numvar = es_variable_typeR_cherche(tvar,nomvar)
if (numvar>0) then
  valeur = tvar%valvarR(numvar)
end if
end function es_variable_valeur_typeR

! ==========================================================
! == function es_variable_typeR_cherche                   ==
! == Recherche une variable identifiée par son nom dans la 
! == liste de variables. Variable de type Réel
! ==========================================================	
function es_variable_typeR_cherche(tvar,nomvar) result(numvar)

!=== Déclaration des arguments ===
type(st_es_variable), intent(inout) :: tvar
character(len=LEN_MOT_CLE),intent(in) :: nomvar

!===  Déclaration des variables  ===
integer :: numvar
integer :: i

!===   Début des traitements   ===
do i=1,tvar%nr
  if (tvar%nomvarR(i) == nomvar) then
    numvar = i
    return
  end if
end do  

print '("Nom d''une variable de type réel non reconnu :",A)',trim(nomvar)
numvar = -1

end function es_variable_typeR_cherche

! ==========================================================
! == Subroutine es_variable_typeI_cherche                 ==
! == Recherche une variable identifiée par son nom dans la 
! == liste de variables. Variable de type Entier
! ==========================================================	
function es_variable_typeI_cherche(tvar,nomvar) result(numvar)

!=== Déclaration des arguments ===
type(st_es_variable), intent(inout) :: tvar
character(len=LEN_MOT_CLE),intent(in) :: nomvar

!===  Déclaration des variables  ===
integer :: numvar
integer :: i

!===   Début des traitements   ===
do i=1,tvar%ni
  if (tvar%nomvarI(i) == nomvar) then
    numvar = i
    return
  end if
end do  

print '("Nom d''une variable de type entière non reconnu :",A)',trim(nomvar)
numvar = -1

end function es_variable_typeI_cherche

! ==========================================================
! == Subroutine es_variable_typeC_cherche                 ==
! == Recherche une variable identifiée par son nom dans la 
! == liste de variables. Variable de type Chaine de caractère
! ==========================================================	
function es_variable_typeC_cherche(tvar,nomvar) result(numvar)

!=== Déclaration des arguments ===
type(st_es_variable), intent(inout) :: tvar
character(len=LEN_MOT_CLE),intent(in) :: nomvar

!===  Déclaration des variables  ===
integer :: numvar
integer :: i

!===   Début des traitements   ===
do i=1,tvar%nc
  if (tvar%nomvarC(i) == nomvar) then
    numvar = i
    return
  end if
end do  

print '("Nom d''une variable de type chaine non reconnu :",A)',trim(nomvar)
numvar = -1

end function es_variable_typeC_cherche


! ==========================================================
! == Subroutine es_variable_lecture                       ==
! == lecture des données d'une variable                   ==
! ==========================================================	
subroutine es_variable_lecture(tvar,chaine,numligne)

!=== Déclaration des arguments ===
type(st_es_variable), intent(inout) :: tvar
character(len=LEN_CHAINE),intent(in) :: chaine
integer, intent(in)                :: numligne !n° de ligne dans le fichier

!===  Déclaration des variables  ===
character(len=LEN_MOT_CLE) :: premiermot,secondmot
character(len=LEN_CHAINE)  :: ch,chnew
integer :: i,nbmot,lchaine
type(st_calculat) :: tcalc

!===   Début des traitements   ===
chnew = chaine
lchaine = len_trim(chaine)
do i=1,lchaine
  if (chaine(i:i)=='=') then
    chnew = chaine(1:i-1)//' = '//chaine(i+1:lchaine)
  end if
end do
nbmot = nb_mot_chaine(chnew)
read(chnew,*,ERR=998) premiermot,secondmot
if (secondmot /= '=') then
  print '("Erreur, manque l''opérateur d''affectation =  !!! ")'
  call lecture_variable_stop(chaine,numligne)
end if
select case (premiermot(1:2))
  case (SYMBOLE_VARREEL)
    i=tvar%nr+1

    tvar%nomvarR(i) = premiermot
    ch = extrait_sous_chaine(chnew,3,nbmot)
    lchaine = len_trim(ch)
    call es_variable_reel_convop(tvar,ch,numligne)
    if (rang==RANG_MAITRE_AFF) print '("ch=",A)',trim(ch)
    nbmot = nb_mot_chaine(ch)
    if (nbmot>=1) then
      tvar%valvarR(i) = calculat_reel(tcalc,ch,numligne)
      write(ch,'(ES15.8E2)') tvar%valvarR(i)
    else
      call lecture_variable_stop(chaine,numligne)
      !read(ch,*,ERR=998) tvar%valvarR(i) 
      !ch = extrait_sous_chaine(ch,1,1)
    end if
    
    if (rang==RANG_MAITRE_AFF) print '(T10,"Variable, ",A," =",ES15.7E2)',trim(premiermot),tvar%valvarR(i)
    tvar%chvalvarR(i) = trim(ch)
    tvar%nr= i
  case (SYMBOLE_VARENTIER)
    i=tvar%ni+1
    tvar%nomvarI(i) = premiermot
    ch = extrait_sous_chaine(chnew,3,3)
    read(ch,*,ERR=998) tvar%valvarI(i)
    tvar%chvalvarI(i) = trim(ch)
    tvar%ni= i
  case (SYMBOLE_VARCHAINE)
    if (nbmot /=3) then
      print '("Erreur, le nombre de mots doit etre égale à 3 !!! ")'
      Call lecture_variable_stop(chaine,numligne)
    end if
    i=tvar%nc+1
    tvar%nomvarC(i) = premiermot
    ch = extrait_sous_chaine(chnew,3,3)
    tvar%valvarC(i) = trim(ch)
    tvar%nc= i  
  case default
    print '("Erreur, Type de variable non reconnu !!! ",A)',trim(premiermot)
    call lecture_variable_stop(chaine,numligne)
end select
return

998 print '("Erreur dans la lecture d''une variable !!! ")'
    Call lecture_variable_stop(chaine,numligne)
end subroutine es_variable_lecture


end module m_es_variable