!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_generateur.f90
!// Purpose:     Gestion de toutes les fonctions sources
!// Author:      Christophe Guiffaut
!// Modified by:
!// Created:     08 octobre 2004
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
!================================================================
! Traitement des generateurs cellulaires
!================================================================
module m_generateur
use m_constantes
use m_gene_identifiant
use m_entree, only : st_fichier, lineread, &
                                  multibackspace,unitefichier,&
				  numbackspace_set, lecture_stop, &
				  nb_mot_chaine, &
				  ctrl_nb_mot_chaine, &
				  ligne_precedente
use m_temps_frequence, only : st_liste_temps_freq
use m_math_operation, only : fft_sp
use m_gene_sinus
use m_gene_pulse_sinus
use m_gene_gaussien
use m_gene_sino_gauss
use m_gene_derive_gaussien
use m_gene_echelon_exp
use m_gene_double_expo
use m_gene_iemn_expo_q
use m_gene_rayleigh
use m_gene_heidler
use m_gene_dans_fichier
implicit none

!=== Déclaration des types ===


real, parameter   :: GEN_CRITERE_DYNAMIQUE = 1.0e3;
!=========== STRUCTURE st_gene_borne ==========================
!      Limites en temps et en fréquence
type st_gene_borne
  real  :: Atmax
  real  :: tmax
  real  :: fmin, fmax ! spectre en fréquence
end type st_gene_borne



!=========== STRUCTURE st_liste_gene ==========================
! Structure générateur, englobe les différents les structures
! de tous les types de generateur
type st_liste_gene
  character(len=LEN_MOT_CLE)    :: nom
  character(len=LEN_MOT_CLE)    :: typegene
  type(st_gene_sinus), pointer  ::  t_sinus
  type(st_gene_pulse_sinus), pointer  ::  t_ps
  type(st_gene_gaussien), pointer  ::  t_gg
  type(st_gene_sino_gauss), pointer  ::  t_gsg
  type(st_gene_derive_gauss), pointer  ::  t_gdg
  type(st_gene_echelon_exp), pointer  ::  t_ee
  type(st_gene_double_expo), pointer  ::  t_gde
  type(st_gene_rayleigh), pointer  ::  t_gor
  type(st_gene_iemn_expo_q), pointer :: t_ie
  type(st_gene_heidler), pointer :: t_hei
  type(st_gene_dans_fichier), pointer :: t_f

  ! information complémentaire
  type(st_gene_borne)     :: t_borne
end type st_liste_gene

type st_generateur
  integer  :: nb_generateur
  character(len=LEN_MOT_CLE),dimension(:), pointer :: liste_nomgene
  type(st_liste_gene), dimension(:), pointer :: t_liste_gene
  logical,dimension(:), pointer :: l_flaggene
  real, dimension(:), pointer  :: tab_zero
  
  
  ! Information complémentaire
  integer    :: niter ! nbre d'itérations temporelles
  real       :: dt    ! le pas temporel
  type(st_gene_borne)     :: t_borne
end type st_generateur 

type st_temp
 real, dimension(:), pointer  :: tab1d
 complex, dimension(:), pointer  :: tabf1d
end type st_temp
! Pas de variables globales...

contains

!===========================================================	
! Subroutine generateur_initdefaut
!===========================================================	
subroutine generateur_initdefaut(t)
!=== Déclaration des arguments ===
type(st_generateur), intent(inout)  :: t

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t%nb_generateur = 0
nullify(t%tab_zero)
nullify(t%t_liste_gene)				
nullify(t%liste_nomgene)
nullify(t%l_flaggene)
end subroutine generateur_initdefaut

!===========================================================	
! Subroutine un_generateur_copy
!===========================================================	
subroutine un_generateur_copy(gin,gout)
!=== Déclaration des arguments ===
type(st_liste_gene), intent(in)  :: gin
type(st_liste_gene), intent(out)  :: gout

!===   Début des traitements   ===
gout%nom = gin%nom
gout%typegene = gin%typegene

select case (gin%typegene)
  case (BLOC_FS_SINUS) 
    allocate(gout%t_sinus)
    gout%t_sinus = gin%t_sinus
    
  case (BLOC_FS_PULSE_SINUS);        
    allocate(gout%t_ps)
    gout%t_ps = gin%t_ps
    
  case (BLOC_FS_GAUSSIENNE);           
    allocate(gout%t_gg)
    gout%t_gg = gin%t_gg
    
  case (BLOC_FS_SINO_GAUSSIENNE);      
    allocate(gout%t_gsg)
    gout%t_gsg = gin%t_gsg
    
  case (BLOC_FS_DERIVEE_GAUSSIENNE);   
    allocate(gout%t_gdg)
    gout%t_gdg = gin%t_gdg
    
  case (BLOC_FS_ECHELON_EXPO);
    allocate(gout%t_ee)
    gout%t_ee = gin%t_ee
    
  case (BLOC_FS_IEMN_EXPO_Q); 
    allocate(gout%t_ie)
    gout%t_ie = gin%t_ie
    
  case (BLOC_FS_FCT_FICHIER);   
    allocate(gout%t_f)
    call gene_dans_fichier_copy(gin%t_f,gout%t_f)
    
  case (BLOC_FS_DOUBLE_EXPO); 
    allocate(gout%t_gde)
    gout%t_gde = gin%t_gde
    
  case (BLOC_FS_RAYLEIGH);   
    allocate(gout%t_gor)
    gout%t_gor = gin%t_gor
    
  case (BLOC_FS_HEIDLER);   
    allocate(gout%t_hei)
    call gene_heidler_copy(gin%t_hei,gout%t_hei)
  case default
end select	 

end subroutine un_generateur_copy

!===========================================================	
! Subroutine liste_gene_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine liste_gene_initdefaut(t_lgene)
!=== Déclaration des arguments ===
type(st_liste_gene), intent(inout)  :: t_lgene

!===   Début des traitements   ===
t_lgene%nom      = ' ';
t_lgene%typegene = ' ';
nullify(t_lgene%t_sinus)
nullify(t_lgene%t_ps)
nullify(t_lgene%t_gg)
nullify(t_lgene%t_ee)
nullify(t_lgene%t_gsg)
nullify(t_lgene%t_gdg)
nullify(t_lgene%t_gde)
nullify(t_lgene%t_gor)				
nullify(t_lgene%t_ie)				
nullify(t_lgene%t_f)				
NULLIFY(t_lgene%t_hei)

end subroutine liste_gene_initdefaut

!===========================================================	
! Subroutine generateur_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
subroutine generateur_lecture(t_fich,t_gene)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout)    :: t_fich
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: mot1, corbeille, typegene
integer  :: lchaine, nbligne,igene, nbmot
integer  :: nbgene
!===   Début des traitements   ===
call generateur_initdefaut(t_gene)
!on compte le nombre de générateur
nbgene = 0
call numbackspace_set(t_fich,0)
do
  mot1 = lineread(t_fich,1)
  if (mot1 == BLOC_FS_SINUS .or. mot1 == BLOC_FS_GAUSSIENNE .or. &
      mot1 == BLOC_FS_SINO_GAUSSIENNE .or.mot1 == BLOC_FS_DERIVEE_GAUSSIENNE .or. &
      mot1 == BLOC_FS_DOUBLE_EXPO .or. mot1 == BLOC_FS_ECHELON_EXPO .or. &
      mot1 == BLOC_FS_RAYLEIGH .or. mot1 == BLOC_FS_PULSE_SINUS .or. &
      mot1 == BLOC_FS_IEMN_EXPO_Q.or. mot1==BLOC_FS_FCT_FICHIER.or. &
      mot1 == BLOC_FS_HEIDLER) then
      
    nbgene = nbgene +1  
  else if (mot1 == t_fich%nomfinbloc) then
    ! le pointeur de fichier est remonté au début du bloc
    nbligne = multibackspace(t_fich)
    exit 
  end if
end do
if (nbgene == 0) then
  print *,'Pas de fonction source de définie dans le bloc :',t_fich%nombloc 
  call lecture_stop(t_fich)
end if

! Allocation et initialisation de la structure generateur
t_gene%nb_generateur = nbgene
allocate(t_gene%liste_nomgene(1:nbgene))
allocate(t_gene%t_liste_gene(1:nbgene))
allocate(t_gene%l_flaggene(1:nbgene))

t_gene%liste_nomgene(:) = ' ';
t_gene%l_flaggene(:) = .false.
! Initialisation par défaut
do igene=1,nbgene
  call liste_gene_initdefaut(t_gene%t_liste_gene(igene))
end do

! Lecture des informations de chaque générateur
igene = 1
do
  mot1 = lineread(t_fich)
  if (t_fich%nomfinbloc == mot1) exit ! fin du bloc
  
  nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,3)   
  read(t_fich%chaine,*) typegene, t_gene%liste_nomgene(igene)
  t_gene%t_liste_gene(igene)%typegene = typegene 
  t_gene%t_liste_gene(igene)%nom = t_gene%liste_nomgene(igene) 

  select case (typegene)        
    case (BLOC_FS_SINUS) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_sinus)
      call gene_sinus_lecture(t_fich,t_gene%t_liste_gene(igene)%t_sinus)
    case (BLOC_FS_PULSE_SINUS) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_ps)
      call gene_pulse_sinus_lecture(t_fich,t_gene%t_liste_gene(igene)%t_ps)
    case (BLOC_FS_GAUSSIENNE) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_gg)
      call gene_gaussien_lecture(t_fich,t_gene%t_liste_gene(igene)%t_gg)
    case (BLOC_FS_SINO_GAUSSIENNE) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_gsg)
      call gene_sino_gauss_lecture(t_fich,t_gene%t_liste_gene(igene)%t_gsg)
    case (BLOC_FS_DERIVEE_GAUSSIENNE) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_gdg)
      call gene_derive_gauss_lecture(t_fich,t_gene%t_liste_gene(igene)%t_gdg)
    case (BLOC_FS_ECHELON_EXPO) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_ee)
      call gene_echelon_exp_lecture(t_fich,t_gene%t_liste_gene(igene)%t_ee)
    case (BLOC_FS_DOUBLE_EXPO) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_gde)
      call gene_double_expo_lecture(t_fich,t_gene%t_liste_gene(igene)%t_gde)
    case (BLOC_FS_IEMN_EXPO_Q) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_ie)
      call gene_iemn_expo_q_lecture(t_fich,t_gene%t_liste_gene(igene)%t_ie)
    case (BLOC_FS_FCT_FICHIER) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_f)
      call gene_dans_fichier_lecture(t_fich,t_gene%t_liste_gene(igene)%t_f)
    case (BLOC_FS_RAYLEIGH) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)    
      allocate(t_gene%t_liste_gene(igene)%t_gor)
      call gene_rayleigh_lecture(t_fich,t_gene%t_liste_gene(igene)%t_gor)
    case (BLOC_FS_HEIDLER) 
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,3,3)   
      allocate(t_gene%t_liste_gene(igene)%t_hei)
      call gene_heidler_initdefaut(t_gene%t_liste_gene(igene)%t_hei)
      read(t_fich%chaine,*) corbeille, mot1,t_gene%t_liste_gene(igene)%t_hei%nbh 
      if (t_gene%t_liste_gene(igene)%t_hei%nbh<1) call lecture_stop(t_fich)
      call gene_heidler_lecture(t_fich,t_gene%t_liste_gene(igene)%t_hei)
      
    case default
      print *,'Mot clé non reconnu dans le bloc générateur : ',typegene 
      call lecture_stop(t_fich)
  end select  
  ! On remonte d'une ligne
  call ligne_precedente(t_fich)
  igene = igene+1
end do				
				
end subroutine generateur_lecture

!===========================================================	
! Subroutine generateur_ecriture
! Lecture dans le fichier d'entrée...
!===========================================================
subroutine generateur_ecriture(f,t_gene)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)    :: f
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer :: i

!===  Déclaration des variables  ===
type(st_liste_gene), pointer :: t_lgene

!===   Début des traitements   ===
do i=1,t_gene%nb_generateur
  
  t_lgene=>t_gene%t_liste_gene(i)
  if (t_lgene%typegene/=BLOC_FS_HEIDLER) &
    write(f%unite,'(T3,A,3x,A)')  trim(t_lgene%typegene),trim(t_lgene%nom)
    
  select case (t_lgene%typegene)
    case (BLOC_FS_SINUS) 
      call gene_sinus_ecriture(f,t_lgene%t_sinus)
    case (BLOC_FS_PULSE_SINUS) 
      call gene_pulse_sinus_ecriture(f,t_lgene%t_ps)
    case (BLOC_FS_GAUSSIENNE) 
      call gene_gaussien_ecriture(f,t_lgene%t_gg)
    case (BLOC_FS_SINO_GAUSSIENNE) 
      call gene_sino_gauss_ecriture(f,t_lgene%t_gsg)
    case (BLOC_FS_DERIVEE_GAUSSIENNE) 
      call gene_derive_gauss_ecriture(f,t_lgene%t_gdg)
    case (BLOC_FS_ECHELON_EXPO) 
      call gene_echelon_exp_ecriture(f,t_lgene%t_ee)
    case (BLOC_FS_IEMN_EXPO_Q) 
      call gene_iemn_expo_q_ecriture(f,t_lgene%t_ie)
    case (BLOC_FS_FCT_FICHIER) 
      call gene_dans_fichier_ecriture(f,t_lgene%t_f)
    case (BLOC_FS_HEIDLER) 
      write(f%unite,'(T3,A,3x,A,3x,I2)') trim(t_lgene%typegene),trim(t_lgene%nom), &
                                          t_gene%t_liste_gene(i)%t_hei%nbh 
      call gene_heidler_ecriture(f,t_lgene%t_hei)
    case (BLOC_FS_DOUBLE_EXPO) 
      call gene_double_expo_ecriture(f,t_lgene%t_gde)
    case (BLOC_FS_RAYLEIGH) 
      call gene_rayleigh_ecriture(f,t_lgene%t_gor)
    case default
      print *,'Type de fonction source non reconnu  : ',t_lgene%typegene 
  end select	 
  write(f%unite,*)
end do
			
end subroutine generateur_ecriture




!===========================================================	
! Subroutine generateur_init
!  
!===========================================================	
subroutine generateur_init(tgen,tf,nt,dt,type_duree,ntout)

!=== Déclaration des arguments ===
type(st_generateur), intent(inout) :: tgen
type(st_liste_temps_freq), intent(inout) :: tf
integer, intent(in)          :: nt
real, intent(in)                :: dt
integer, intent(in)             :: type_duree
integer, intent(out)            :: ntout
!=== Déclaration des variables locales ===
integer :: i,n,ntfct
real    :: res1,Tmax,Fmin,FMax
real, dimension(:),allocatable   :: tabgen
type(st_liste_gene), pointer :: ungen
logical :: flag_duree

!===   Début des traitements   ===

!--> Traitement en fonction du type de durée de la simulation
if (type_duree==VOL_DUREE_AUTO .or.type_duree==VOL_DUREE_SEMIAUTO) then
  ntout=0
  do i =1,tgen%nb_generateur
    if (.not.tgen%l_flaggene(i)) cycle
    flag_duree = generateur_duree(tgen,tgen%liste_nomgene(i),dt,Tmax)
    if (flag_duree) then
      n = nint(Tmax/dt)+1
      if (rang==RANG_MAITRE_AFF) then
        print '(T10,"Calcul Durée pour la Fonction source ",A)',trim(tgen%liste_nomgene(i))
        print '(T12,"DUREE = ",ES11.3,"s,  NBRE Itérations = ",I7)',Tmax,n
      end if
    else
      if (type_duree==VOL_DUREE_AUTO) then
        print '("Erreur dans l''évaluation de la fonction source de nom ",A)',trim(tgen%liste_nomgene(i))
	print '("  Elle ne revient pas à zéro au bout de 500000 itérations !!!")'
	stop
      end if
      ntout = NBRE_ITERATION_MAX_AUTO
    end if
    ntout=max(ntout,n)
 end do
else
  ntout = nt
end if

ntfct = ntout
if (type_duree==VOL_DUREE_SEMIAUTO) then
  ntfct=nt
end if
!----------------------------------------------------------------
allocate( tgen%tab_zero(ntfct) )
tgen%tab_zero = 0.0
tgen%niter = ntfct
tgen%dt    = dt

allocate(tabgen(1:ntfct))
!print '(T5," ntfct = ",I6)',ntfct
if (rang==RANG_MAITRE_AFF) &
  print '(T5," TEMPS MAX ET BANDE SPECTRALE DES FONCTIONS SOURCES DéFINIES, DYNAMIQUE = ",F7.1,"dB")',&
                          20*log10(GEN_CRITERE_DYNAMIQUE)
Tmax = 0.0
Fmin = 1/dt
Fmax = 0.0
do i =1,tgen%nb_generateur
  ungen =>tgen%t_liste_gene(i)
  res1 = generateur_calcul_t(tgen, tgen%liste_nomgene(i),1.0, 0.0, 0.0,dt, ntfct,tabgen)
  call generateur_borne(tgen,tgen%liste_nomgene(i),tabgen,ungen%t_borne%Tmax, &
                         ungen%t_borne%Atmax,ungen%t_borne%Fmin,ungen%t_borne%Fmax)
  if (rang==RANG_MAITRE_AFF) &
    print '(T10,"FONCTION DE NOM :",A," ; DUREE = ",ES11.3,"s ; FMIN ET FMAX = ",2ES11.3," Hz")',&
          trim(tgen%liste_nomgene(i)),ungen%t_borne%Tmax,ungen%t_borne%Fmin,ungen%t_borne%Fmax
  if (tgen%l_flaggene(i)) then
    if (rang==RANG_MAITRE_AFF) &
      print '(T12," --> Le générateur est utilisé")'
    Tmax = max(Tmax,ungen%t_borne%Tmax)
    Fmin = mIN(Fmin,ungen%t_borne%Fmin)
    Fmax = mAX(Fmax,ungen%t_borne%Fmax)
  end if
end do

if (Fmax ==0) Fmax = 1/dt
tgen%t_borne%Tmax = Tmax
tgen%t_borne%Fmin = Fmin
tgen%t_borne%Fmax = Fmax
tf%fmax_fctsrc = Fmax
if (Fmin >= 0.99999/dt)  tgen%t_borne%Fmin = 0.0 ! cas où pas de générateurs utilisés
if (rang==RANG_MAITRE_AFF) then
  print '(/,T10,"BORNES communes à tous les générateurs utilisés:")'
  print '(T12,"--> TMAX = ",ES11.3," s")',Tmax
  print '(T12,"--> FMIN ET FMAX = ",2ES11.3," Hz",/)',Fmin,Fmax
end if
deallocate(tabgen)
end subroutine generateur_init


!===========================================================	
! Subroutine generateur_borne
!  Recherche des limites en temps et en fréquence d'une
!   fonction
!===========================================================	
subroutine generateur_borne(tgen,nomgen,tabgen,Tmax,Atmax,Fmin,Fmax)

!=== Déclaration des arguments ===
type(st_generateur), intent(inout) :: tgen
character(len=LEN_MOT_CLE),intent(in) :: nomgen
real, dimension(1:tgen%niter),intent(in) :: tabgen
real, intent(out) :: Tmax,Atmax,Fmin,Fmax
!=== Déclaration des variables locales ===
real    :: Amin,Amax,Afmax,Fpas
real    :: Energie,dEn,df2
integer :: i,n,Nt,Nmax,isaut
complex, dimension(:),allocatable :: tabfreq
character(LEN=20) :: nomfichier
!===   Début des traitements   ===

!============= Recherche Du Tmax en temporel ============
Amax = maxval(abs(tabgen))
Amin = Amax/GEN_CRITERE_DYNAMIQUE
Tmax = (tgen%niter-1)*tgen%dt
Atmax= abs(tabgen(tgen%niter))
Fmin=0.0
Fmax=2.0/tgen%dt
Nmax = tgen%niter
!------- Recherche Tmax en temporel -------------------
IF (Atmax>Amin) then
  Atmax = Atmax/Amax
  if (rang==RANG_MAITRE_AFF) then
    PRINT '(T5,"AVERTISSEMENT !!!  Dans le bloc ",A," Fonction de nom ",A," Amplitude pas assez atténué à Tmax :")',& 
	     BLOC_FONCTION_SOURCE,trim(nomgen )
    PRINT '(T5, " Atténuation de ",F7.1,"dB à Tmax=",ES15.6,"s !!!!!")',-20*log10(Atmax),Tmax
  end if
else
  do i=tgen%niter-1,1,-1
    if (Amin<abs(tabgen(i))) then
      Atmax = GEN_CRITERE_DYNAMIQUE
      Tmax = (i+1)*tgen%dt
      exit
    end if 
  end do
  Nmax = i+1
  !print '(T10,"--> TMAX = ",ES11.3," s")',Tmax
  !print '(T10,"--> Nmax = ",I6," ")',Nmax

end if

!--> Calcul du nombre d'échantillons en 2^n
n = min(size(tabgen),Nmax) ! on limite le nombre d'échantillons pour les temps longs
!n=size(tabgen)
i=0
do 
  i=i+1
  if (2**i>n) then
    Nt = 2**(i)
    exit
  end if
end do


!--> écriture de la forme d'onde dans fichier ---
if (rang==RANG_MAITRE_AFF) then
  nomfichier=trim(nomgen)//'_t.src'
  open(500, FILE=nomfichier, ACTION='write',FORM = 'formatted')    
  Energie = 0.0
  df2 = 0.0
  !print '(T15,"tgen%dt=",ES15.6)',tgen%dt
  isaut = int(n/10000)+1
  do i=1,n,isaut
    Energie = Energie + tabgen(i)**2.0
    dEn = (tabgen(i)**2.0)
    if (i>1) df2 = (tabgen(i)-tabgen(i-1))
    write(500,'(2ES15.6)') tgen%dt*(i-1),tabgen(i) !Energie,dEn,df2
  end do
  close(500)
end if

!-------- Recherche Fmin et Fmax en fréquence ----------------

! fft de la fonction source, on travaille par la suite 
! que sur la moitié des échantillons en fréquence
if (Nt<=2**16) then
  allocate(tabfreq(1:Nt))
  tabfreq = cmplx(0.0,0.0)
  tabfreq(1:n)=CMPLX(tabgen(1:n),0.0)
  call fft_sp(tabfreq,Nt,1)
  Amax = maxval(abs(tabfreq))
  Amin = Amax/GEN_CRITERE_DYNAMIQUE
  Fpas = 1/(Nt*tgen%dt)
  
  if (rang==RANG_MAITRE_AFF) then
    nomfichier=trim(nomgen)//'_f.src'
    open(500, FILE=nomfichier, ACTION='write',FORM = 'formatted')    
    do i=1,Nt
      write(500,'(2ES15.6)') Fpas*(i-1),abs(tabfreq(i)) 
    end do
    close(500)
  end if
  !---> Recherche de Fmin
  do i=1,Nt/2
    if (Amin<abs(tabfreq(i))) then
      Fmin = Fpas*(i-1) ! 
      exit
    end if
  end do
  
  !---> Recherche de Fmax
  Afmax = abs(tabfreq(Nt/2))
  IF (Afmax>Amin) then
    Afmax = Afmax/Amax
    if (rang==RANG_MAITRE_AFF) then
      PRINT '(T5,"AVERTISSEMENT !!!  Dans le bloc ",A," Fonction de nom ",A," Bande spectrale trop large :")',& 
	     BLOC_FONCTION_SOURCE,trim(nomgen )
      PRINT '(T5, " Atténuation de ",F7.1," à Fmax=",ES15.6,"Hz !!!!!")',-20*log10(Afmax),Fmax
      PRINT '(T5,"  Risque de recouvrement spectral avec la TFD !!! ")'
    end if
  else
    do i=Nt/2-1,1,-1
      if (Amin<abs(tabfreq(i))) then
	Fmax = Fpas*(i-1)
	!print '(T10,"i,Fpas=",I6,ES15.6)',i,Fpas
	exit
      end if
    end do
  end if
  deallocate(tabfreq)
end if
end subroutine generateur_borne

!===========================================================	
! Subroutine generateur_borne
!  Recherche la durée d'une forme d'onde
!   calcul de la forme d'onde par palier de 500,5000,50000,500000
!
!===========================================================	
logical function generateur_duree(tgen,nomgen,dt,Tmax) result(flag_duree)

!=== Déclaration des arguments ===
type(st_generateur), intent(inout) :: tgen
character(len=LEN_MOT_CLE),intent(in) :: nomgen
real, intent(in)  :: dt
real, intent(out) :: Tmax

!=== Déclaration des variables locales ===
real    :: Amin,Amax,Atmax,res1
integer :: i,Nt,Nmax
real, dimension(:),allocatable :: tabgen,dtabgen
!===   Début des traitements   ===

!============= Recherche Du Tmax en temporel ============
Amin = Amax/GEN_CRITERE_DYNAMIQUE

Nt=NBRE_ITERATION_MAX_AUTO/10000
flag_duree = .false.
do while(Nt<NBRE_ITERATION_MAX_AUTO)
  Nt=Nt*10
  allocate(tabgen(1:Nt))
  tabgen = 0.0
  res1 = generateur_calcul_t(tgen,nomgen,1.0, 0.0, 0.0,dt, Nt,tabgen)
  Amax = maxval(abs(tabgen))
  Amin = Amax/GEN_CRITERE_DYNAMIQUE
  !--- Test sur les 10% derniers échantillons ---
  Atmax= maxval(abs(tabgen(Nt-Nt/100:Nt)))
  if (Atmax<=Amin) then
    !--- Critère sur la dérivée ---
    allocate(dtabgen(1:Nt-1))
    dtabgen(1:Nt-1)=tabgen(2:Nt)-tabgen(1:Nt-1)
    Amax = maxval(abs(dtabgen))
    Amin = Amax/GEN_CRITERE_DYNAMIQUE
    Atmax= maxval(abs(dtabgen(Nt-Nt/100:Nt-1)))
    deallocate(dtabgen)
    if (Atmax<=Amin) then
      flag_duree = .true.
      exit
    end if
  end if  
  deallocate(tabgen)
end do  

Tmax = dt*Nt
if (flag_duree) then
  Amax = maxval(abs(tabgen))
  Amin = Amax/GEN_CRITERE_DYNAMIQUE
  do i=Nt-1,1,-1
    if (Amin<abs(tabgen(i))) then
      Tmax = (i+1)*dt
      exit
    end if 
  end do
  deallocate(tabgen)
end if

!print '(T10,"NT=",I7,"  TMAX=",ES15.7)',Nt,Tmax

end function generateur_duree


!===========================================================	
! subroutine generateur_affecte
! Affecte la bonne structure générateur en fonction du nom 
! de celui ci
!===========================================================	
subroutine generateur_affecte(t_gene,nomgenerateur,t_lgene)

!=== Déclaration des arguments ===
type(st_generateur), intent(in)    :: t_gene
type(st_liste_gene), pointer  :: t_lgene
character (len=LEN_MOT_CLE), intent(in)  :: nomgenerateur 
!===  Déclaration des variables  ===
integer :: i
!===   Début des traitements   ===
NULLIFY(t_lgene)
do i =1,t_gene%nb_generateur	
  if (nomgenerateur == t_gene%liste_nomgene(i)) then
    t_lgene=>t_gene%t_liste_gene(i)
  end if
end do
if (.not.ASSOCIATED(t_lgene)) then
  print *,'le nom suivant de fonction source n''existe pas  : ',nomgenerateur
  stop 'Arret dans la routine generateur_affecte'
end if  			
end subroutine generateur_affecte

!===========================================================
! function generateur_existe_actif
! Vérifie si le nom du générateur existe dans la liste des générateurs
! Considère le générateur trouvé comme actif pour la simulation
!===========================================================
function generateur_existe_actif(t_gene,nomgenerateur)
!=== Déclaration des arguments ===
type(st_generateur), intent(inout)    :: t_gene
character (len=LEN_MOT_CLE), intent(in)  :: nomgenerateur 
logical  :: generateur_existe_actif
!===  Déclaration des variables  ===
integer :: i
!===   Début des traitements   ===
generateur_existe_actif = .false.
do i =1,t_gene%nb_generateur	
  if (nomgenerateur == t_gene%liste_nomgene(i)) then
    generateur_existe_actif = .true. 
    t_gene%l_flaggene(i) = .true.
    exit   
  end if
end do
if (.not.generateur_existe_actif) then
  print '("ERREUR, Le nom du générateur n''existe pas !!!! nom =",A)',trim(nomgenerateur)
end if

end function generateur_existe_actif

!===========================================================	
! Subroutine generateur_calcul_t
! Appel de la fonction qui convient en fonction du type de 
! générateur.
! Le calcul de la forme du générateur est réalisée sur ntinc 
! itérations
! Calculs avant les itérations temporelles
!===========================================================	
function generateur_calcul_t(t_gene, nomgenerateur,A0, phase, retard, &
                                    dt, ntinc, tab1d) result(func_dervaleur)
implicit none
!=== Déclaration des arguments ===
type(st_generateur), intent(inout) :: t_gene
character (len=LEN_MOT_CLE), intent(in)  :: nomgenerateur 
real, intent(in)          :: A0
real, intent(in)          :: retard,phase,dt  ! la phase est en degré
integer, intent(in)       :: ntinc
real, dimension(:), intent(inout)   :: tab1d
real                      :: func_dervaleur

!===  Déclaration des variables  ===
type(st_liste_gene), pointer :: t_lgene

!===   Début des traitements   ===
!On récupère le bon pointeur de générateur en fonction du nom de celui-ci
call generateur_affecte(t_gene,nomgenerateur,t_lgene)

select case (t_lgene%typegene)
  
  case (BLOC_FS_SINUS) 
    func_dervaleur = gene_sinus_calcul_t(t_lgene%t_sinus,A0, phase, & 
                              retard, dt, ntinc,tab1d)
  case (BLOC_FS_PULSE_SINUS) 
    func_dervaleur = gene_pulse_sinus_calcul_t(t_lgene%t_ps,A0, phase, & 
                              retard, dt, ntinc,tab1d)
  case (BLOC_FS_GAUSSIENNE) 
    func_dervaleur = gene_gaussien_calcul_t(t_lgene%t_gg,A0, retard, &
                                dt, ntinc,tab1d)
  case (BLOC_FS_SINO_GAUSSIENNE) 
    func_dervaleur = gene_sino_gauss_calcul_t(t_lgene%t_gsg,A0, &
                              phase, retard, dt, ntinc,tab1d)
  case (BLOC_FS_DERIVEE_GAUSSIENNE) 
    func_dervaleur = gene_derive_gauss_calcul_t(t_lgene%t_gdg,A0, &
                               retard, dt, ntinc,tab1d)
  case (BLOC_FS_ECHELON_EXPO) 
    func_dervaleur = gene_echelon_exp_calcul_t(t_lgene%t_ee,A0, &
                               retard, dt, ntinc,tab1d)

  case (BLOC_FS_IEMN_EXPO_Q) 
    func_dervaleur = gene_iemn_expo_q_calcul_t(t_lgene%t_ie,A0,  & 
                              retard, dt, ntinc,tab1d)
  case (BLOC_FS_FCT_FICHIER) 
    func_dervaleur = gene_dans_fichier_calcul_t(t_lgene%t_f,A0,  & 
                              retard, dt, ntinc,tab1d)
  case (BLOC_FS_DOUBLE_EXPO) 
    func_dervaleur = gene_double_expo_calcul_t(t_lgene%t_gde,A0,  & 
                              retard, dt, ntinc,tab1d)
  case (BLOC_FS_RAYLEIGH) 
    func_dervaleur = gene_rayleigh_calcul_t(t_lgene%t_gor,A0, & 
                               phase, retard, dt, ntinc,tab1d)
  case (BLOC_FS_HEIDLER) 
    func_dervaleur = gene_heidler_calcul_t(t_lgene%t_hei,A0,  & 
                              retard, dt, ntinc,tab1d)
  case default
    print *,'Type de fonction source non reconnu  : ',t_lgene%typegene 
    stop 'Arret dans la routine generateur_calcul_t'
end select	 

				
end function generateur_calcul_t

!===========================================================	
! Subroutine generateur_calcul_1t
! Appel de la fonction qui convient en fonction du type de 
! générateur.
! Le calcul de la forme du générateur est réalisée sur ntinc 
! itérations
! Calculs avant les itérations temporelles
!===========================================================	
function generateur_calcul_1t(t_lgene, A0, phase, retard, &
                                    dt, t) result(func_dervaleur)
implicit none
!=== Déclaration des arguments ===
type(st_liste_gene), intent(inout) :: t_lgene
real, intent(in)          :: A0
real, intent(in)          :: retard,phase,dt,t  ! la phase est en degré
real                      :: func_dervaleur

!===  Déclaration des variables  ===

!===   Début des traitements   ===
select case (t_lgene%typegene)
  
  case (BLOC_FS_SINUS) 
    func_dervaleur = gene_sinus_calcul_1t(t_lgene%t_sinus,A0, phase, & 
                              retard, t)
  case (BLOC_FS_GAUSSIENNE) 
    func_dervaleur = gene_gaussien_calcul_1t(t_lgene%t_gg,A0, retard,t)
  case (BLOC_FS_SINO_GAUSSIENNE) 
    func_dervaleur = gene_sino_gauss_calcul_1t(t_lgene%t_gsg,A0, &
                              phase, retard, t)
  case (BLOC_FS_DERIVEE_GAUSSIENNE) 
    func_dervaleur = gene_derive_gauss_calcul_1t(t_lgene%t_gdg,A0, &
                               retard, t, dt)
  case (BLOC_FS_ECHELON_EXPO) 
    func_dervaleur = gene_echelon_exp_calcul_1t(t_lgene%t_ee,A0, &
                               retard, t)
  case (BLOC_FS_DOUBLE_EXPO) 

  case (BLOC_FS_RAYLEIGH) 
    func_dervaleur = gene_rayleigh_calcul_1t(t_lgene%t_gor,A0, & 
                               phase, retard,t)
  case default
    print *,'Type de générateur non reconnu  : ',t_lgene%typegene 
    stop 'Arret dans la routine generateur_calcul_t'
end select	 

				
end function generateur_calcul_1t


!===========================================================	
! Subroutine generateur_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine generateur_affichage(t)
implicit none
!=== Déclaration des arguments ===
type(st_generateur), intent(inout) :: t

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine generateur_affichage

!===========================================================	
! Subroutine generateur_impression
!  Ecriture des résultats dans les fichiers de sortie
!===========================================================	
subroutine generateur_impression(t_fich,t_gene)
implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: t_fich
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer  :: i,nbiter
integer  :: nfmin,nfmax
real     :: resultat, dt,df,tnorm, fnorm
real, dimension(:,:), pointer  :: tab1d
complex, dimension(:,:), pointer  :: tabf1d
type(st_temp)  :: t_temp

!===   Début des traitements   ===
nfmin = 0
nfmax = 400

nbiter = 500
dt = 1.867e-9
df = 2.5e4

allocate(tab1d(1:nbiter,1:t_gene%nb_generateur))
allocate(tabf1d(1:nfmax-nfmin+1,1:t_gene%nb_generateur))
tab1d(:,:) = 0.0
tabf1d(:,:) = CMPLX(0.0,0.0)

do i = 1, t_gene%nb_generateur
  print *,'générateur',i,t_gene%liste_nomgene(i)
  resultat = generateur_calcul_t(t_gene, t_gene%liste_nomgene(i), &
       1.0, 90.0, 0.0, dt, nbiter, tab1d(:,i))
       
  ! Transformée de Fourrier discrète
!  call tfd_t1d(tab1d(:,i),tabf1d(:,i),0,nbiter-1,nfmin,nfmax,dt,df)
end do		

! Ecriture des données en temps dans un fichier
t_fich%unite =  unitefichier()
if (t_fich%unite<=0) &
  stop 'pas d''unité disponible pour l''ouverture du fichier d''entree'

open(t_fich%unite, file='excit_t.txt', status = 'replace',action='write')

do i = 1,nbiter
  tnorm = dt * (i-1)
  write(t_fich%unite,200) tnorm, tab1d(i,1:t_gene%nb_generateur)
end do
200 FORMAT(100ES19.10E3)
close(t_fich%unite)

! Ecriture des données en fréquence
t_fich%unite =  unitefichier()
if (t_fich%unite<=0) &
  stop 'pas d''unité disponible pour l''ouverture du fichier d''entree'

open(t_fich%unite, file='excit_f.txt', status = 'replace',action='write')

do i = 1,nfmax-nfmin+1
  fnorm = df * (i-1+nfmin)
!  write(t_fich%unit,200) fnorm, REAL(tabf1d(i,1:t_gene%nb_generateur)), &
!                               AIMAG(tabf1d(i,1:t_gene%nb_generateur))
  write(t_fich%unite,200) fnorm, tabf1d(i,1:t_gene%nb_generateur)
                          
end do
close(t_fich%unite)

		
end subroutine generateur_impression

!===========================================================	
! Subroutine generateur_deallocate
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine generateur_deallocate(t_gene)
implicit none
!=== Déclaration des arguments ===
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer :: i
type(st_liste_gene), pointer :: t_lgene

!===   Début des traitements   ===
if (t_gene%nb_generateur == 0) return

do i=1, t_gene%nb_generateur
  call un_generateur_deallocate(t_gene%t_liste_gene(i))
end do

if (associated(t_gene%tab_zero))      deallocate(t_gene%tab_zero)
if (associated(t_gene%liste_nomgene)) deallocate(t_gene%liste_nomgene)
if (associated(t_gene%t_liste_gene))  deallocate(t_gene%t_liste_gene)
if (associated(t_gene%l_flaggene))    deallocate(t_gene%l_flaggene)
nullify(t_gene%tab_zero);
nullify(t_gene%liste_nomgene);
nullify(t_gene%t_liste_gene);

End Subroutine generateur_deallocate

!===========================================================	
! Subroutine un_generateur_deallocate
!===========================================================	
Subroutine un_generateur_deallocate(t_lgene)
implicit none
!=== Déclaration des arguments ===
type(st_liste_gene), intent(inout) :: t_lgene

!===   Début des traitements   ===
select case (t_lgene%typegene)
  case (BLOC_FS_SINUS)                
    call gene_sinus_deallocate(t_lgene%t_sinus)
    deallocate(t_lgene%t_sinus)
  case (BLOC_FS_PULSE_SINUS);        
    CALL gene_pulse_sinus_deallocate(t_lgene%t_ps)
    deallocate(t_lgene%t_ps)
  case (BLOC_FS_GAUSSIENNE);           
    CALL gene_gaussien_deallocate(t_lgene%t_gg)
    deallocate(t_lgene%t_gg)
  case (BLOC_FS_SINO_GAUSSIENNE);      
    CALL gene_sino_gauss_deallocate(t_lgene%t_gsg)
    deallocate(t_lgene%t_gsg)
  case (BLOC_FS_DERIVEE_GAUSSIENNE);   
    CALL gene_derive_gauss_deallocate(t_lgene%t_gdg)
    deallocate(t_lgene%t_gdg)
  case (BLOC_FS_ECHELON_EXPO);
    CALL gene_echelon_exp_deallocate(t_lgene%t_ee)
    deallocate(t_lgene%t_ee)
  case (BLOC_FS_IEMN_EXPO_Q); 
    CALL gene_iemn_expo_q_deallocate(t_lgene%t_ie)
    deallocate(t_lgene%t_ie)
  case (BLOC_FS_FCT_FICHIER);   
    CALL gene_dans_fichier_deallocate(t_lgene%t_f)
    deallocate(t_lgene%t_f)
  case (BLOC_FS_DOUBLE_EXPO); 
    call gene_double_expo_deallocate(t_lgene%t_gde)
    deallocate(t_lgene%t_gde)
  case (BLOC_FS_RAYLEIGH);   
    CALL gene_rayleigh_deallocate(t_lgene%t_gor)
    deallocate(t_lgene%t_gor)
  case (BLOC_FS_HEIDLER);   
    CALL gene_heidler_deallocate(t_lgene%t_hei)
    deallocate(t_lgene%t_hei)
  end select	 

End Subroutine un_generateur_deallocate


end module m_generateur	



