!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_liste_milieu.f90
!// Purpose:     Gestion de tous les milieux, dispersif et non dispersif, fractal
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

module m_liste_milieu


use m_constantes, only : LEN_MOT_CLE,rang,RANG_MAITRE_AFF

use m_entree , only : st_fichier, &
                                  lineread, &
                                  lecture_stop, &
				  ligne_precedente, &
				  compteur_mot_cle, &
				  ctrl_nb_mot_chaine, &
				  apres_fin_bloc,&
				  multibackspace,numbackspace_set

use m_milieu_simple, only : st_milieu_simple, &
                                 milieu_simple_initdefaut, &
                                 milieu_simple_lecture,milieu_simple_ecriture, &
				 milieu_simple_init, &
				 milieu_simple_anisotrope, &
				 milieu_simple_non_defaut

use m_milieu_fractal, only : st_milieu_fractal, &
                                 milieu_fractal_initdefaut, &
                                 milieu_fractal_lecture, &
				 milieu_fractal_ecriture
use m_milieu_dispersif, only : st_milieu_dispersif
use m_maillage

implicit none

! ==========================================================
! == Type listes_milieu_simple                            ==
! milieu simple incluant la partie non dispersive
! des milieux dispersifs pour le calcul des coefficients 
! multiplicatif des champs
! ==========================================================
type st_liste_milieu_simple
  logical :: flag_data
  logical :: flag_eps_anisotrope, flag_epsr_nondefaut
  logical :: flag_mu_anisotrope, flag_mur_nondefaut
  logical :: flag_sige_anisotrope, flag_sige_nondefaut
  logical :: flag_sigh_anisotrope, flag_sigh_nondefaut
  
  integer :: nb_ms, nb_mf ! dimension de t_ms et t_mf
  type(st_milieu_simple), dimension(:), pointer :: t_ms
  type(st_milieu_fractal), dimension(:), pointer :: t_mf
  
  ! tableau de milieu simple incluant la partie non dispersive
  ! des milieux dispersifs pour le calcul des coefficients 
  ! multiplicatif des champs :
  integer :: nb_ms_tous ! dimension de t_ms_tous
  type(st_milieu_simple), dimension(:), pointer :: t_ms_tous

end type st_liste_milieu_simple

contains

! ==========================================================
! == Subroutine milieu_simple_initdefaut           ==
! == Initialisation par defaut des variables contenant    ==
! == les parametres definis par l'utilisateur             ==
! == dans le fichier d'entree                             ==
! ==========================================================	
subroutine liste_milieu_simple_initdefaut(l)
type(st_liste_milieu_simple), intent(inout) :: l

l%nb_ms = 0; l%nb_mf = 0;
l%nb_ms_tous = 0;
l%flag_data = .false.
l%flag_eps_anisotrope = .false.;  l%flag_epsr_nondefaut = .false.
l%flag_mu_anisotrope = .false.;   l%flag_mur_nondefaut  = .false.
l%flag_sige_anisotrope = .false.; l%flag_sige_nondefaut = .false.
l%flag_sigh_anisotrope = .false.; l%flag_sigh_nondefaut = .false.
nullify(l%t_ms);		
nullify(l%t_mf);		
nullify(l%t_ms_tous);		
end subroutine liste_milieu_simple_initdefaut

!===========================================================	
! Subroutine liste_milieu_simple_deallocate
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine liste_milieu_simple_deallocate(l)
!=== Déclaration des arguments ===
type(st_liste_milieu_simple), intent(inout) :: l

!===  Déclaration des variables  ===

!===   Début des traitements   ===
if (l%nb_ms>0) deallocate(l%t_ms)
if (l%nb_mf>0) deallocate(l%t_mf)
if (l%nb_ms_tous>0) deallocate(l%t_ms_tous)

End Subroutine liste_milieu_simple_deallocate


! ==========================================================
! == Subroutine milieu_simple_lecture              ==
! == Lecture dans le fichier d'entrée...                  ==
! ==========================================================	
subroutine liste_milieu_lecture(f,l,tm)

type(st_fichier), intent(inout):: f
type(st_liste_milieu_simple), intent(inout) :: l
type(st_maillage)              :: tm
type(st_milieu_simple) :: m
character(LEN=LEN_MOT_CLE) :: premier_mot
integer :: i,j, status

! On compte le nombre de milieu
l%nb_ms = compteur_mot_cle(f,'milieu')
l%nb_mf = compteur_mot_cle(f,'milieu_fractal')
l%nb_ms = l%nb_ms - l%nb_mf
if (rang==RANG_MAITRE_AFF) then
  print '(T3,"NB de milieux simples =",I3)',l%nb_ms
  print '(T3,"NB de milieux fractals =",I3)',l%nb_mf
end if  
if (l%nb_ms == 0 .and. l%nb_mf == 0) then
  call apres_fin_bloc(f)
  return ! on sort
end if
l%flag_data = .true.
if (l%nb_ms > 0) allocate(l%t_ms(l%nb_ms))
if (l%nb_mf > 0) allocate(l%t_mf(l%nb_mf))
i = 0
j = 0
do 
  premier_mot = lineread(f)

  if (premier_mot=='milieu_fractal') then
    j = j + 1
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*) premier_mot, l%t_mf(j)%nom
    call milieu_fractal_initdefaut(l%t_mf(j));
    call milieu_fractal_lecture(f,l%t_mf(j),tm)

  else if (premier_mot=='milieu') then
    i = i + 1
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
!    print '("imilieu =",I3)',i
!    print '("f%chaine =",A)',TRIM(f%chaine)
    
    read(f%chaine,*) premier_mot, l%t_ms(i)%nom
    call milieu_simple_initdefaut(l%t_ms(i));
    call milieu_simple_lecture(f,l%t_ms(i))
  else if (premier_mot == f%nomfinbloc) then
    exit
  else
    print '("Mot clé non reconnu : ",A)', premier_mot
    call lecture_stop(f)  
  endif
enddo

end subroutine liste_milieu_lecture

! ==========================================================
! == Subroutine milieu_simple_lecture              ==
! == Lecture dans le fichier d'entrée...                  ==
! ==========================================================	
subroutine liste_milieu_simple_lecture(f,l)

type(st_fichier), intent(inout):: f
type(st_liste_milieu_simple), intent(inout) :: l
type(st_milieu_simple) :: m
character(LEN=LEN_MOT_CLE) :: premier_mot
integer :: i,j, status

! On compte le nombre de milieu
l%nb_ms = compteur_mot_cle(f,'milieu')
if (l%nb_ms == 0) then
  return ! on sort
end if
l%flag_data = .true.
if (l%nb_ms > 0) allocate(l%t_ms(l%nb_ms))
i = 0
j = 0
do 
  premier_mot = lineread(f)


  if (premier_mot=='milieu') then
    i = i + 1
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    
    read(f%chaine,*) premier_mot, l%t_ms(i)%nom
    call milieu_simple_initdefaut(l%t_ms(i));
    call milieu_simple_lecture(f,l%t_ms(i))
  else if (premier_mot == f%nomfinbloc) then
    exit
  else
    print '("Mot clé non reconnu : ",A)', premier_mot
    call lecture_stop(f)  
  endif
enddo

end subroutine liste_milieu_simple_lecture

! ==========================================================
! == Subroutine liste_milieu_simple_ecriture              ==
! == Ecriture dans le fichier d'entrée...                  ==
! ==========================================================	
subroutine liste_milieu_simple_ecriture(f,l)

type(st_fichier), intent(inout):: f
type(st_liste_milieu_simple), intent(inout) :: l

integer  :: i

do i=1,l%nb_ms
  write(f%unite,'(T3,A,3x,A)') 'milieu', trim(l%t_ms(i)%nom)
  call milieu_simple_ecriture(f,l%t_ms(i))
end do

do i=1,l%nb_mf
  write(f%unite,'(T3,A,3x,A)') 'milieu_fractal', trim(l%t_mf(i)%nom)
  call milieu_fractal_ecriture(f,l%t_mf(i))
end do

end subroutine liste_milieu_simple_ecriture

! ==========================================================
! == Subroutine liste_milieu_simple              ==
! ==========================================================
subroutine liste_milieu_simple_init(l,t_md,eps_r,mu_r,sig_e,sig_h)

type(st_liste_milieu_simple), intent(inout) :: l
type(st_milieu_dispersif), pointer  :: t_md
real, intent(in) ::eps_r,mu_r,sig_e,sig_h ! Les valeurs par défaut dans 
                                          ! tout le volume.
integer :: i,ntot1
type(st_milieu_simple), pointer:: un_ms
type(st_milieu_fractal), pointer:: un_mf

! construction du tableau de structure contenant les
! milieu simple de tous les milieux
l%nb_ms_tous = l%nb_ms
if (ASSOCIATED(t_md)) then
  l%nb_ms_tous = l%nb_ms + t_md%nb_milieu
end if
ntot1 = l%nb_ms_tous
if (l%nb_mf>0) l%nb_ms_tous = l%nb_ms_tous+ l%nb_mf

allocate(l%t_ms_tous(l%nb_ms_tous))
do i=1,l%nb_ms
  l%t_ms_tous(i) = l%t_ms(i)
end do

!--> ajout milieu dispersif
if (ASSOCIATED(t_md)) then
  ! ajout des milieux simples des milieux dispersifs
  do i=l%nb_ms+1,l%nb_ms + t_md%nb_milieu
    l%t_ms_tous(i) = t_md%t_un_md(i-l%nb_ms)%t_ms
  end do
end if

!--> ajout milieu fractal
if (l%nb_mf>0) then
  do i=ntot1+1,l%nb_ms_tous
    l%t_ms_tous(i) = l%t_mf(i)%t_ms
  end do
end if

if (rang==RANG_MAITRE_AFF) &
  print '(T10,"Nbmilieu = ",I3)',l%nb_ms_tous
do i=1,l%nb_ms_tous
  un_ms =>l%t_ms_tous(i)

  call milieu_simple_init(un_ms)
  ! recherche des paramètres anisotropes
  if (milieu_simple_anisotrope(un_ms%eps_r)) l%flag_eps_anisotrope  = .true.
  if (milieu_simple_anisotrope(un_ms%mu_r )) l%flag_mu_anisotrope   = .true. 
  if (milieu_simple_anisotrope(un_ms%sig_e)) l%flag_sige_anisotrope = .true.
  if (milieu_simple_anisotrope(un_ms%sig_h)) l%flag_sigh_anisotrope = .true.


  ! recherche des paramètres différents de ceux par défaut
  if (milieu_simple_non_defaut(un_ms%eps_r,eps_r)) l%flag_epsr_nondefaut = .true.
  if (milieu_simple_non_defaut(un_ms%mu_r, mu_r))  l%flag_mur_nondefaut  = .true.
  if (milieu_simple_non_defaut(un_ms%sig_e, sig_e))  l%flag_sige_nondefaut = .true.
  if (milieu_simple_non_defaut(un_ms%sig_h, sig_h))  l%flag_sigh_nondefaut = .true.
!  print '("nom milieu = ",A,"  sigma_e = ",3ES15.6)',trim(un_ms%nom),un_ms%sig_e(1:3)
enddo

!---- Cas particulier pour les milieux dispersifs------
if (ASSOCIATED(t_md)) then
  do i=1,t_md%nb_milieu
    if (t_md%t_un_md(i)%flag_eps) l%flag_epsr_nondefaut = .true.
    if (t_md%t_un_md(i)%flag_mu) l%flag_mur_nondefaut = .true.
  end do
end if

!--- traitement à part des milieux fractals ---
do i=1,l%nb_mf
  un_mf =>l%t_mf(i)
  if (un_mf%sige_moyen > 0) then 
    l%flag_sige_anisotrope = .true.
    l%flag_sige_nondefaut = .true.
  end if
  if (un_mf%eps_moyen > 0) then 
    l%flag_eps_anisotrope = .true.
    l%flag_epsr_nondefaut = .true.
  end if
end do

end subroutine liste_milieu_simple_init

! ==========================================================
! == Subroutine liste_milieu_simple              ==
! ==========================================================
subroutine liste_milieu_simple_init2(l,eps_r,mu_r,sig_e,sig_h)

type(st_liste_milieu_simple), intent(inout) :: l
real, intent(in) ::eps_r,mu_r,sig_e,sig_h ! Les valeurs par défaut dans 
                                          ! tout le volume.
integer :: i
type(st_milieu_simple), pointer:: un_ms
! construction du tableau de structure contenant les
! milieu simple de tous les milieux
l%nb_ms_tous = l%nb_ms

allocate(l%t_ms_tous(l%nb_ms_tous))
do i=1,l%nb_ms
  l%t_ms_tous(i) = l%t_ms(i)
end do

do i=1,l%nb_ms_tous
  un_ms =>l%t_ms_tous(i)

  call milieu_simple_init(un_ms)
  ! recherche des paramètres anisotropes
  if (milieu_simple_anisotrope(un_ms%eps_r)) l%flag_eps_anisotrope  = .true.
  if (milieu_simple_anisotrope(un_ms%mu_r )) l%flag_mu_anisotrope   = .true. 
  if (milieu_simple_anisotrope(un_ms%sig_e)) l%flag_sige_anisotrope = .true.
  if (milieu_simple_anisotrope(un_ms%sig_h)) l%flag_sigh_anisotrope = .true.


  ! recherche des paramètres différents de ceux par défaut
  if (milieu_simple_non_defaut(un_ms%eps_r,eps_r)) l%flag_epsr_nondefaut = .true.
  if (milieu_simple_non_defaut(un_ms%mu_r, mu_r))  l%flag_mur_nondefaut  = .true.
  if (milieu_simple_non_defaut(un_ms%sig_e, sig_e))  l%flag_sige_nondefaut = .true.
  if (milieu_simple_non_defaut(un_ms%sig_h, sig_h))  l%flag_sigh_nondefaut = .true.

enddo


end subroutine liste_milieu_simple_init2


!===========================================================	
! Function identifie_milieu_fractal
! Indique si le nom d'un milieu est un milieu fractal
!===========================================================	
logical function identifie_milieu_fractal(l,nom_milieu)
implicit none
!=== Déclaration des arguments ===
type(st_liste_milieu_simple), intent(in) :: l
character (len=LEN_MOT_CLE), intent(in)  :: nom_milieu

!===  Déclaration des variables  ===
integer  :: i
type(st_milieu_fractal), pointer :: un_mf

!===   Début des traitements   ===
identifie_milieu_fractal = .false.
do i=1,l%nb_mf
  un_mf => l%t_mf(i)
  if (nom_milieu == un_mf%nom) then
      identifie_milieu_fractal = .true.
      exit
  end if
end do

End function identifie_milieu_fractal

!===========================================================	
! Function milieu_fractal_pointe
! Retourne un pointeur sur la structure st_milieu_fractal
! identifié par le nom passé en argument
!===========================================================	
function pointe_milieu_fractal(l,nom_milieu) result(un_mf)
implicit none
!=== Déclaration des arguments ===
type(st_liste_milieu_simple), intent(in) :: l
character (len=LEN_MOT_CLE), intent(in)  :: nom_milieu

type(st_milieu_fractal), pointer :: un_mf
!===  Déclaration des variables  ===
integer  :: i

!===   Début des traitements   ===
NULLIFY(un_mf)

do i=1,l%nb_mf
  if (nom_milieu == l%t_mf(i)%nom) then
    un_mf => l%t_mf(i)
    exit
  end if
end do

End function pointe_milieu_fractal


!===========================================================	
! Subroutine milieu_dispersif_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine liste_milieu_simple_affichage(l)
implicit none
!=== Déclaration des arguments ===
type(st_liste_milieu_simple), intent(in) :: l

!===  Déclaration des variables  ===
integer  :: i,j,k
!===   Début des traitements   ===
if (rang/=RANG_MAITRE_AFF) return

print '("**************************************************")'
print '("Nombre de milieux simples : ",I3)', l%nb_ms
print '("Nombre de milieux fractals : ",I3)', l%nb_mf
print '("Nombre total de milieux simples et dispersifs  : ",I3)', l%nb_ms_tous
print '(" permittivité (anisotrope , non milieu defaut) =(",L2,",",L2,")")', &
                     l%flag_eps_anisotrope, l%flag_epsr_nondefaut

print '(" perméabilité (anisotrope , non milieu defaut) =(",L2,",",L2,")")', &
                     l%flag_mu_anisotrope, l%flag_mur_nondefaut
print '(" sigma E (anisotrope , non milieu defaut) =(",L2,",",L2,")")', &
                     l%flag_sige_anisotrope, l%flag_sige_nondefaut
print '(" sigma H (anisotrope , non milieu defaut) =(",L2,",",L2,")")', &
                     l%flag_sigh_anisotrope, l%flag_sigh_nondefaut

print '("**************************************************")'
				
End Subroutine liste_milieu_simple_affichage


!===========================================================	
! Subroutine liste_milieu_epsr
!===========================================================	
function liste_milieu_epsr(l,nu)
!=== Déclaration des arguments ===
type(st_liste_milieu_simple), intent(in) :: l
integer, intent(in)               :: nu

real, dimension(3)  :: liste_milieu_epsr
!===  Déclaration des variables  ===
integer  :: i
!===   Début des traitements   ===
liste_milieu_epsr(1:3) = 1.0
if (nu == 0) return
do i=1,l%nb_ms_tous
  if (i==nu) then
    liste_milieu_epsr = l%t_ms_tous(i)%eps_r 
    return
  end if
end do

end function liste_milieu_epsr
!===========================================================	
! Subroutine liste_milieu_sige
!===========================================================	
function liste_milieu_sige(l,nu)
!=== Déclaration des arguments ===
type(st_liste_milieu_simple), intent(in) :: l
integer, intent(in)               :: nu

real, dimension(3)  :: liste_milieu_sige
!===  Déclaration des variables  ===
integer  :: i
!===   Début des traitements   ===
liste_milieu_sige(1:3) = 0.0
if (nu == 0) return
do i=1,l%nb_ms_tous
  if (i==nu) then
    liste_milieu_sige = l%t_ms_tous(i)%sig_e 
    return
  end if
end do

end function liste_milieu_sige

!===========================================================	
! Subroutine liste_milieu_nom
!  retourne le nom du milieu associé au n° de milieu
!===========================================================	
function liste_milieu_nom(l,nu)
implicit none
!=== Déclaration des arguments ===
type(st_liste_milieu_simple), intent(in) :: l
integer, intent(in)               :: nu
character(LEN=LEN_MOT_CLE)        :: liste_milieu_nom

!===  Déclaration des variables  ===

!===   Début des traitements   ===
if (nu == 0 .or. nu>l%nb_ms_tous) then
  liste_milieu_nom = "VIDE";
  return
end if
liste_milieu_nom = l%t_ms_tous(nu)%nom

end function liste_milieu_nom

!===========================================================	
! Subroutine liste_milieu_nu
!  retourne le n° de milieu associé au nom du milieu
!===========================================================	
function liste_milieu_nu(l,nom)
implicit none
!=== Déclaration des arguments ===
type(st_liste_milieu_simple), intent(in) :: l
character(LEN=LEN_MOT_CLE), intent(in) :: nom
integer :: liste_milieu_nu

!===  Déclaration des variables  ===
integer  :: i
!===   Début des traitements   ===
liste_milieu_nu = 0;
do i=1,l%nb_ms_tous
  if (nom==l%t_ms_tous(i)%nom) then
    liste_milieu_nu = i
    return
  end if
end do

end function liste_milieu_nu


! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module m_liste_milieu
