!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_maillage.f90
!// Purpose:      Gère le maillage des structures
!// Author:      Christophe Guiffaut
!// Modified by:  
!// Created  :     Juillet 2006
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!// Liste des subroutines


module m_maillage

use m_constantes
use m_entree, only : st_fichier, lecture_stop



implicit none


character(*), parameter, private :: MOINS_INF='-inf'
character(*), parameter, private :: PLUS_INF ='+inf'

type st_maillage
  integer :: nxyz(3)
  integer :: borne_volume(6) !borne volume utile
  real :: dxyz(3)
  real :: arrondi
  real :: dt
  integer :: type_volume
  integer, dimension(3) :: origine_cell
  real, dimension(3)    :: origine
  integer, dimension(6) :: borne_cell
  real, dimension(6)    :: borne_reel
end type st_maillage


!==================== CHANGEMENT D'ORIGINE =========================
! origine_reelle_abs3 --> pour les coordonnées réelles
! origine_reelle_abs6 --> pour les coordonnées réelles

! ================ INTERFACE origine_cellule3 =======================
! == Change les 3 coordonnées en nbre de cellules dans le repère absolu.
!      le nombre de cell peut etre non-entier
! == flag_inverse : permet de faire l'opération inverse, coordonnées
!     dans le repère utilisateur
interface origine_cellule3
  module procedure  origine_cell_abs3,origine_Rcell_abs3
end interface

! ================ INTERFACE origine_cellule6 =======================
! == Change les 6 coordonnées en nbre de cellules dans le repère absolu.
!      le nombre de cell peut etre non-entier
! == flag_inverse : permet de faire l'opération inverse, coordonnées
!     dans le repère utilisateur
interface origine_cellule6
  module procedure  origine_cell_abs6,origine_Rcell_abs6
end interface


! ==========================================================
! == function maillage_Unif_ConvToCellPt                  ==
! == conversion en nombre de cellule, valeur entière de prise
! == gestion du passage du repère utilisateur au repère de calcul
!function maillage_Unif_ConvToCellPt(m,valeur,typeaxe,change_repere) result(n)
! --> non utilisé !


! ================ INTERFACE Maillage_U_RealToCell =======================
! ==> Conversion des coordonnées réelles en nombre de cellule.
!  - pas d'arrondi si le nombre de cellules est réelle
! L'interface gère la conversion pour une coordonnée dans une direction
!  ou un point défini par 3 coordonnées. La conversion est entière ou réelle
!  suivant la nature des arguments
! ==> Par défaut le repère d'origine est supposé etre le repère utilisateur.
! ==  Et toujours par défaut, la conversion est faite pour le repère de calcul
! == Un flag optionnel mis à .false. permet de faire la conversion sans changement
! ==   d'origine. Dans ce cas, l'origine est associé obligatoirement au repère de
! ==  calcul
interface Maillage_U_RealToCell
  module procedure  maillage_Unif_ConvToCellR, &
		    maillage_Unif_ConvToCellR3
end interface


! Meme chose mais pour un volume défini par 6 coordonnées ou deux points
! maillage_Unif_ConvToCell6 --> arrondi suivant la règle fixée
! maillage_Unif_ConvToCellR6 --> pas d'arrondi, conversion exacte

! Meme chose avec un nbre de cellule retourné entier
!  - arrondi suivant la règle fixée si le nombre de cellules est entier
!maillage_Unif_ConvToCell
!maillage_Unif_ConvToCell3


! ================ INTERFACE Maillage_U_CellToReal =======================
! ==> Conversion des coordonnées nombre de cellule en coordonnées réelles
! L'interface gère la conversion pour une coordonnée dans une direction
!  ou un point défini par 3 coordonnées. La fonction appelée dépend de la
!  nature des variables spécifiant les coordonnées en nbre de cellules (entier ou réelles)
! ==> Par défaut le repère d'origine est supposé etre le repère utilisateur.
! ==  Et toujours par défaut, la conversion est faite pour le repère de calcul
! == Un flag optionnel mis à .false. permet de faire la conversion sans changement
! ==   d'origine. Dans ce cas, l'origine est associé obligatoirement au repère de
! ==  calcul
interface Maillage_U_CellToReal
  module procedure  maillage_Unif_ConvToReal, maillage_Unif_RcellConvToReal, &
		    maillage_Unif_ConvToReal3,maillage_Unif_RcellConvToReal3 
end interface

! ================ INTERFACE Maillage_U_CellToReal6 =======================
! ==> Conversion des coordonnées nombre de cellule en coordonnées réelles
! L'interface gère la conversion pour un volume défini par deux points soit
!  6 coordonées. La fonction appelée dépend de la nature des variables spécifiant 
!  les coordonnées en nbre de cellules (entier ou réelles)
! ==> Par défaut le repère d'origine est supposé etre le repère utilisateur.
! ==  Et toujours par défaut, la conversion est faite pour le repère de calcul
! == Un flag optionnel mis à .false. permet de faire la conversion sans changement
! ==   d'origine. Dans ce cas, l'origine est associé obligatoirement au repère de
! ==  calcul
interface Maillage_U_CellToReal6
  module procedure  maillage_Unif_ConvToReal6,maillage_Unif_RcellConvToReal6
end interface


contains
! ==========================================================
! == Subroutine maillage_initdefaut                       ==
! == Initialisation des variables
! ==========================================================	
subroutine maillage_initdefaut(m)
type(st_maillage),intent(inout) :: m
m%dxyz       = 1.0
m%arrondi    = 0.5
m%type_volume = VOL_3D
m%origine_cell=-1
m%origine = 0.0
end subroutine maillage_initdefaut

! ==========================================================
! == Subroutine maillage_init                             ==
! == Initialisation des variables
! ==========================================================	
subroutine maillage_init(m,dt,dxyz,nxyz,type_volume,origcell,orig)
type(st_maillage),intent(inout)  :: m
real, intent(in)                 :: dt
real,dimension(3), intent(in)    :: dxyz
integer,dimension(3), intent(in) :: nxyz
integer, intent(in)              :: type_volume
integer,dimension(3), intent(in) ::origcell
real,dimension(3), intent(in) ::orig

m%dxyz       = dxyz
m%arrondi    = 0.5
m%type_volume = type_volume
m%dt         = dt
m%origine_cell = origcell
m%origine      = orig
m%nxyz         = nxyz
m%borne_cell(1:5:2) = -origcell  ! valeur minimal
m%borne_cell(2:6:2) = nxyz-origcell ! valeur maximal
m%borne_reel(1:5:2) = -orig
m%borne_reel(2:6:2) = nxyz*dxyz-orig

m%borne_volume(1:5:2) = 1
m%borne_volume(2:6:2) = nxyz+1 !!borne volume utile
end subroutine maillage_init


! ==========================================================
! == Subroutine maillage_deallocate                       ==
! == Libération de la mémoire
! ==========================================================	
subroutine maillage_deallocate(m)
type(st_maillage),intent(inout) :: m

end subroutine maillage_deallocate


! ===================================================================
! ===================================================================
! ===================================================================
! SUBROUTINES DE CONVERSION COORDONNÉES REELLES EN NBRE DE CELLULES
! ===================================================================

! ==========================================================
! == Subroutine maillage_Unif_ConvToCellR                 ==
! == conversion en nombre de cellule, pas d'arrondi toutefois
! == gestion du passage du repère utilisateur au repère de calcul
! ==========================================================	
function maillage_Unif_ConvToCellR(m,valeur,typeaxe,infsup,change_repere) result(n)
type(st_maillage),intent(in) :: m
real, intent(in)   :: valeur
integer,intent(in)  :: typeaxe,infsup
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real :: n,coord

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

if (flag_chrep) then
  coord = valeur+m%origine(typeaxe)
else
  coord = valeur
end if

! Conversion en nombre de cellules,  gère les différents types d'espace
n     = (coord/m%dxyz(typeaxe))+1.0
if (m%type_volume == VOL_3D) then
else if (m%type_volume == VOL_2D) then
  if (typeaxe==DIR_Z) then
    n=1.0; if (infsup==1) n=2.0
  end if
else
  if (typeaxe==DIR_Y .or. typeaxe==DIR_Z) then
    n=1.0; if (infsup==1) n=2.0
  end if
end if

end function maillage_Unif_ConvToCellR

! ==========================================================
! == Subroutine maillage_Unif_ConvToCellPt                  ==
! == conversion en nombre de cellule, valeur entière de prise
! == gestion du passage du repère utilisateur au repère de calcul
! ==========================================================	
function maillage_Unif_ConvToCellPt(m,valeur,typeaxe,change_repere) result(n)
type(st_maillage),intent(in) :: m
real, intent(in)   :: valeur
integer,intent(in)  :: typeaxe
logical,intent(in),optional :: change_repere

logical :: flag_chrep
integer :: n
real    :: coord

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

if (flag_chrep) then
  ! Conversion en nombre de cellules,  gère les différents types d'espace
  coord = valeur+m%origine(typeaxe)
else
  coord = valeur
end if

n   = int(coord/m%dxyz(typeaxe))+1

if (m%type_volume == VOL_3D) then
else if (m%type_volume == VOL_2D) then
  if (typeaxe==DIR_Z) then
    n=1;
  end if
else
  if (typeaxe==DIR_Y .or. typeaxe==DIR_Z) then
    n=1;
  end if
end if

end function maillage_Unif_ConvToCellPt

! ==========================================================
! == Subroutine maillage_Unif_ConvToCell                  ==
! == conversion en nombre de cellule dans le repère de calcul
! == gestion du passage du repère utilisateur au repère de calcul
! ==========================================================	
function maillage_Unif_ConvToCell(m,valeur,typeaxe,infsup,change_repere) result(n)
type(st_maillage),intent(in) :: m
real, intent(in)   :: valeur
integer,intent(in)  :: typeaxe,infsup
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real  :: coord,reste
integer :: n

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

if (flag_chrep) then
  coord = valeur+m%origine(typeaxe)
else
  coord = valeur
end if
! Conversion en nombre de cellules,  gère les différents types d'espace
!reste = MOD(valeur,m%dxyz(typeaxe))/m%dxyz(typeaxe)
reste = coord/m%dxyz(typeaxe)-1.0*int(coord/m%dxyz(typeaxe))
n     = int(coord/m%dxyz(typeaxe))+1

!print '("reste = ",F8.5)',reste
!print '("n = ",I4)',n
if (reste>m%arrondi.and.infsup<=0) then
  n = n+1
else if ((1-reste)<=m%arrondi.and.infsup==1) then
  n = n+1
end if

if (m%type_volume == VOL_3D) then
else if (m%type_volume == VOL_2D) then
  if (typeaxe==DIR_Z) then
    n=1; if (infsup==1) n=2
  end if
else
  if (typeaxe==DIR_Y .or. typeaxe==DIR_Z) then
    n=1; if (infsup==1) n=2
  end if
end if

end function maillage_Unif_ConvToCell

! ==========================================================
! == Subroutine maillage_Unif_ConvToCell3                  ==
! == conversion en nombre de cellule pour 3 coordonnées(x,y,z)
! ==========================================================	
function maillage_Unif_ConvToCell3(m,valeur,change_repere) result(n)
type(st_maillage),intent(in) :: m
real,dimension(3), intent(in)   :: valeur
logical,intent(in),optional :: change_repere

logical :: flag_chrep

integer, dimension(3) :: n
integer :: i

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

do i =1,3
  n(i) = maillage_Unif_ConvToCell(m,valeur(i),i,0,flag_chrep)
end do

end function maillage_Unif_ConvToCell3

! ==========================================================
! == Subroutine maillage_Unif_ConvToCellR3                  ==
! == conversion en nombre de cellule pour 3 coordonnées(x,y,z)
! == Les coord en nombre de cellule peuvent etre non entières
! ==========================================================	
function maillage_Unif_ConvToCellR3(m,valeur,change_repere) result(n)
type(st_maillage),intent(in) :: m
real,dimension(3), intent(in)   :: valeur
logical,intent(in),optional :: change_repere

logical :: flag_chrep

real, dimension(3) :: n
integer :: i

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

do i =1,3
  n(i) = maillage_Unif_ConvToCellR(m,valeur(i),i,0,flag_chrep)
end do

end function maillage_Unif_ConvToCellR3

! ==========================================================
! == Subroutine maillage_Unif_ConvToCell6                  ==
! == conversion en nombre de cellule pour 6 coordonnées ! ==
! == xmin, xmax, ymin, ymax zmin, zmax
! ==========================================================	
function maillage_Unif_ConvToCell6(m,valeur,change_repere) result(n)
type(st_maillage),intent(in) :: m
real,dimension(6), intent(in)   :: valeur
logical,intent(in),optional :: change_repere

logical :: flag_chrep

integer, dimension(6) :: n
integer :: i,j

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere
j=0
do i =1,5,2
  j=j+1
  n(i)   = maillage_Unif_ConvToCell(m,valeur(i),j,-1,flag_chrep)
  n(i+1) = maillage_Unif_ConvToCell(m,valeur(i+1),j,1,flag_chrep)
end do

end function maillage_Unif_ConvToCell6

! ==========================================================
! == Subroutine maillage_Unif_ConvToCellR6                ==
! == conversion en nombre de cellule pour 6 coordonnées ! ==
! == xmin, xmax, ymin, ymax zmin, zmax
! == Les coord en nombre de cellule peuvent etre non entières
! ==========================================================	
function maillage_Unif_ConvToCellR6(m,valeur,change_repere) result(n)
type(st_maillage),intent(in) :: m
real,dimension(6), intent(in)   :: valeur
logical,intent(in),optional :: change_repere

logical :: flag_chrep

real, dimension(6) :: n
integer :: i,j

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere
j=0
do i =1,5,2
  j=j+1
  n(i)   = maillage_Unif_ConvToCellR(m,valeur(i),j,-1,flag_chrep)
  n(i+1) = maillage_Unif_ConvToCellR(m,valeur(i+1),j,1,flag_chrep)
end do

end function maillage_Unif_ConvToCellR6

! ===================================================================
! ===================================================================
! ===================================================================
! SUBROUTINES DE CONVERSION COORDONNÉES NBRE DE CELLULES EN COORD RÉELLES
! ===================================================================

! ==========================================================
! == Subroutine maillage_Unif_ConvToReal                  ==
! == conversion du nombre de cellule en coordonnée réelle
! == gestion du passage du repère utilisateur au repère de calcul
! ==========================================================	
function maillage_Unif_ConvToReal(m,n,typeaxe,change_repere) result(r)
type(st_maillage),intent(in) :: m
integer, intent(in)   :: n
integer,intent(in)  :: typeaxe
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real :: r

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

! Conversion en nombre de cellules,  gère les différents types d'espace
if (flag_chrep) then
  r = (n+m%origine_cell(typeaxe))*m%dxyz(typeaxe)
ELSE
  R = (n-1)*m%dxyz(typeaxe)
END IF

if (m%type_volume == VOL_3D) then
else if (m%type_volume == VOL_2D) then
  if (typeaxe==DIR_Z) then
    r = 0.0
  end if
else
  if (typeaxe==DIR_Y .or. typeaxe==DIR_Z) then
    r = 0.0
  end if
end if

end function maillage_Unif_ConvToReal

! ==========================================================
! == Subroutine maillage_Unif_RcellConvToReal             ==
! == conversion du nombre de cellule en coordonnée réelle
! == Le nombre de cellule peut etre non entier
! ==========================================================	
function maillage_Unif_RcellConvToReal(m,n,typeaxe,change_repere) result(r)
type(st_maillage),intent(in) :: m
real, intent(in)   :: n
integer,intent(in)  :: typeaxe
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real :: r

! Conversion en nombre de cellules,  gère les différents types d'espace
flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

if (flag_chrep) then
  r  = (n+1.0*m%origine_cell(typeaxe))*m%dxyz(typeaxe)
else
  r  = (n-1)*m%dxyz(typeaxe)
end if

if (m%type_volume == VOL_3D) then
else if (m%type_volume == VOL_2D) then
  if (typeaxe==DIR_Z) then
    r = 0.0
  end if
else
  if (typeaxe==DIR_Y .or. typeaxe==DIR_Z) then
    r = 0.0
  end if
end if

end function maillage_Unif_RcellConvToReal

! ==========================================================
! == Subroutine maillage_Unif_ConvToReal3                  ==
! == conversion du nombre de cellule en coordonnée réelle
! ==========================================================	
function maillage_Unif_ConvToReal3(m,n,change_repere) result(r)
type(st_maillage),intent(in) :: m
integer,dimension(3), intent(in)   :: n
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real,dimension(3) :: r

integer :: i

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere
! Conversion
do i =1,3
  r(i) = maillage_Unif_ConvToReal(m,n(i),i,flag_chrep)
end do
end function maillage_Unif_ConvToReal3

! ==========================================================
! == Subroutine maillage_Unif_ConvToReal3                  ==
! == conversion du nombre de cellule en coordonnée réelle
! == Le nombre de cellule peut etre non entier
! ==========================================================	
function maillage_Unif_RcellConvToReal3(m,n,change_repere) result(r)
type(st_maillage),intent(in) :: m
real,dimension(3), intent(in)   :: n
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real,dimension(3) :: r

integer :: i

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere
! Conversion
do i =1,3
  r(i) = maillage_Unif_RcellConvToReal(m,n(i),i,flag_chrep)
end do
end function maillage_Unif_RcellConvToReal3

! ==========================================================
! == Subroutine maillage_Unif_ConvToReal6                  ==
! == conversion du nombre de cellule en coordonnée réelle
! ==========================================================	
function maillage_Unif_ConvToReal6(m,n,change_repere) result(r)
type(st_maillage),intent(in) :: m
integer,dimension(6), intent(in)   :: n
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real,dimension(6) :: r

integer :: i,j

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere
! Conversion
j=0
do i =1,5,2
  j=j+1
  r(i) = maillage_Unif_ConvToReal(m,n(i),j,flag_chrep)
  r(i+1) = maillage_Unif_ConvToReal(m,n(i+1),j,flag_chrep)
end do
end function maillage_Unif_ConvToReal6

! ==========================================================
! == Subroutine maillage_Unif_RcellConvToReal6            ==
! == conversion du nombre de cellule en coordonnée réelle
! == Le nombre de cellule peut etre non entier
! ==========================================================	
function maillage_Unif_RcellConvToReal6(m,n,change_repere) result(r)
type(st_maillage),intent(in) :: m
real,dimension(6), intent(in)   :: n
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real,dimension(6) :: r

integer :: i,j

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere
! Conversion
j=0
do i =1,5,2
  j=j+1
  r(i)   = maillage_Unif_RcellConvToReal(m,n(i),j,flag_chrep)
  r(i+1) = maillage_Unif_RcellConvToReal(m,n(i+1),j,flag_chrep)
end do
end function maillage_Unif_RcellConvToReal6


! ==========================================================
! ==========================================================
!  SUBROUTINES DE CHANGEMENT D'ORIGINE
! ==========================================================

! ==========================================================
! == Subroutine origine_reelle_abs6                        ==
! == convertit les 6 coordonnées dans le repère absolu
! == flag_inverse : permet de faire l'opération inverse, coordonnées
! ==            dans le repère utilisateur
! ==========================================================	
function origine_reelle_abs3(m,creelin,flag_inverse) result(creelout)
type(st_maillage),intent(in) :: m
real,dimension(3), intent(in)   :: creelin
logical,optional,intent(in)  :: flag_inverse 

real :: inverse
real, dimension(3) :: creelout
integer :: i,j

inverse = 1.0
if (present(flag_inverse)) then
  if (flag_inverse) inverse = -1.0
end if
creelout = creelin+m%origine*inverse

end function origine_reelle_abs3
! ==========================================================
! == Subroutine origine_reelle_abs6                        ==
! == convertit les 6 coordonnées dans le repère absolu
! == flag_inverse : permet de faire l'opération inverse, coordonnées
! ==            dans le repère utilisateur
! ==========================================================	
function origine_reelle_abs6(m,creelin,flag_inverse) result(creelout)
type(st_maillage),intent(in) :: m
real,dimension(1:6), intent(in)   :: creelin
logical,optional,intent(in)  :: flag_inverse 

real :: inverse
real, dimension(6) :: creelout
integer :: i,j

inverse = 1.0
if (present(flag_inverse)) then
  if (flag_inverse) inverse = -1.0
end if
creelout(1:5:2) = creelin(1:5:2)+(m%origine)*inverse
creelout(2:6:2) = creelin(2:6:2)+(m%origine)*inverse

end function origine_reelle_abs6

! ==========================================================
! ==========================================================


! ==========================================================
! == Subroutine origine_Rcell_abs3                        ==
! == Change les 3 coordonnées en nbre de cellules
! == dans le repère absolu, le nombre de cell peut etre 
! == non-entier
! == flag_inverse : permet de faire l'opération inverse, coordonnées
! ==            dans le repère utilisateur
! ==========================================================	
function origine_Rcell_abs3(m,creelin,flag_inverse) result(creelout)
type(st_maillage),intent(in) :: m
real,dimension(1:3), intent(in)   :: creelin
logical,optional,intent(in)  :: flag_inverse 

real :: inverse
real, dimension(3) :: creelout
integer :: i,j

inverse = 1.0
if (present(flag_inverse)) then
  if (flag_inverse) inverse = -1.0
end if
creelout = creelin+(m%origine_cell*1.0+1.0)*inverse

end function origine_Rcell_abs3

! ==========================================================
! == Subroutine origine_cell_abs3                        ==
! == Change les 6 coordonnées en nbre de cellules
! == dans le repère de calcul, le nombre de cell est entier 
! == flag_inverse : permet de faire l'opération inverse, coordonnées
! ==            dans le repère utilisateur
! ==========================================================	
function origine_cell_abs3(m,valin,flag_inverse) result(valout)
type(st_maillage),intent(in) :: m
integer,dimension(1:3), intent(in)   :: valin
logical,optional,intent(in)  :: flag_inverse 

integer :: inverse
integer, dimension(3) :: valout
integer :: i,j

inverse = 1
if (present(flag_inverse)) then
  if (flag_inverse) inverse = -1
end if
valout = valin+(m%origine_cell+1)*inverse

end function origine_cell_abs3


! ==========================================================
! == Subroutine origine_Rcell_abs6                        ==
! == Change les 6 coordonnées en nbre de cellules
! == dans le repère absolu, le nombre de cell peut etre 
! == non-entier
! == flag_inverse : permet de faire l'opération inverse, coordonnées
! ==            dans le repère utilisateur
! ==========================================================	
function origine_Rcell_abs6(m,creelin,flag_inverse) result(creelout)
type(st_maillage),intent(in) :: m
real,dimension(6), intent(in)   :: creelin
logical,optional,intent(in)  :: flag_inverse 

real :: inverse
real, dimension(1:6) :: creelout
integer :: i,j

inverse = 1.0
if (present(flag_inverse)) then
  if (flag_inverse) inverse = -1.0
end if
creelout(1:5:2) = creelin(1:5:2)+(m%origine_cell*1.0+1.0)*inverse
creelout(2:6:2) = creelin(2:6:2)+(m%origine_cell*1.0+1.0)*inverse

end function origine_Rcell_abs6



! ==========================================================
! == Subroutine origine_cell_abs6                        ==
! == Change les 6 coordonnées en nbre de cellules
! == dans le repère de calcul, le nombre de cell est entier
! == flag_inverse : permet de faire l'opération inverse, coordonnées
! ==            dans le repère utilisateur
! ==========================================================	
function origine_cell_abs6(m,valin,flag_inverse) result(valout)
type(st_maillage),intent(in) :: m
integer,dimension(6), intent(in)   :: valin
logical,optional,intent(in)  :: flag_inverse 

integer :: inverse
integer, dimension(6) :: valout

inverse = 1
if (present(flag_inverse)) then
  if (flag_inverse) inverse = -1
end if
valout(1:5:2) = valin(1:5:2)+(m%origine_cell+1)*inverse
valout(2:6:2) = valin(2:6:2)+(m%origine_cell+1)*inverse

end function origine_cell_abs6


! ==========================================================
! == function conv_chcoord6_cell_int                        ==
! == Convertit les coordonnées en nbre de cellules
! == de la chaine chin en coordonnées
! == de type entier, fonction indispensable pour gérer
! == les mots clés -inf,+inf
! == Change les 6 coordonnées en nbre de cellules
! == Change de repère : passage repère utilisateur au repère 
! == de calcul
! == le nombre de cellules est entier
! ==========================================================	
function conv_chcoord6_cell_int(f,m,chin,change_repere) result(valout)
type(st_fichier), intent(inout) :: f
type(st_maillage),intent(in) :: m
character(LEN=LEN_MOT_CLE),dimension(6), intent(in)   :: chin
logical,intent(in),optional :: change_repere

logical :: flag_chrep
integer, dimension(6) :: valin
integer, dimension(6) :: valout ! coordonnées retournées dans le repère de calcul
integer :: i,j

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

do i=1,3
  ! traitement des coords min
  j=i*2-1
  if (chin(j)==MOINS_INF) then
    valin(j) = m%borne_cell(j)  ! valeur infinie
  else
    read(chin(j),*,err=997) valin(j)
  end if
  j=j+1
  if (chin(j)==PLUS_INF) then
    valin(j) = m%borne_cell(j)  ! valeur infinie
  else
    read(chin(j),*,err=997) valin(j)
  end if
  
end do

if (flag_chrep) valout = origine_cellule6(m,valin)

return
997 print '(/,t10,"ERREUR d''extraction des données de la ligne sur le mot :",A)',trim(chin(j))
    call lecture_stop(f)


end function conv_chcoord6_cell_int

! ==========================================================
! == function conv_chcoord6_cell_reel                     ==
! == Convertit les coordonnées en nbre de cellules de la chaine
! == chin en coordonnées de type réel, 
! == function indispensable pour gérer les mots clés -inf,+inf
! == Change les 6 coordonnées en nbre de cellules
! == dans le repère de calcul, le nombre de cell est réel
! ==========================================================	
function conv_chcoord6_cell_reel(f,m,chin,change_repere) result(valout)
type(st_fichier), intent(inout) :: f
type(st_maillage),intent(in) :: m
character(LEN=LEN_MOT_CLE),dimension(6), intent(in)   :: chin
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real, dimension(6) :: valin
real, dimension(6) :: valout ! coordonnées retournées dans le repère de calcul
integer :: i,j

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

do i=1,3
  ! traitement des coords min
  j=i*2-1
  if (chin(j)==MOINS_INF) then
    valin(j) = m%borne_cell(j)  ! valeur infinie
  else
    read(chin(j),*,err=998) valin(j)
  end if
  j=j+1
  if (chin(j)==PLUS_INF) then
    valin(j) = m%borne_cell(j)  ! valeur infinie
  else
    read(chin(j),*,err=998) valin(j)
  end if
  
end do

if (flag_chrep) valout = origine_cellule6(m,valin)

return
998 print '(/,t10,"ERREUR d''extraction des données de la ligne sur le mot :",A)',trim(chin(j))
    call lecture_stop(f)


end function conv_chcoord6_cell_reel

! ==========================================================
! == function conv_chcoord6_reel_reel                     ==
! == Convertit les coordonnées réelles de la chaine
! == chin en coordonnées de type réel avec changement de repère, 
! == function indispensable pour gérer les mots clés -inf,+inf
! == Change les 6 coordonnées réelles du repère utilisateur vers
! == le repère de calcul
! ==========================================================	
function conv_chcoord6_reel_reel(f,m,chin,change_repere) result(valout)
type(st_fichier), intent(inout) :: f
type(st_maillage),intent(in) :: m
character(LEN=LEN_MOT_CLE),dimension(6), intent(in)   :: chin
logical,intent(in),optional :: change_repere

logical :: flag_chrep
real, dimension(6) :: valin
real, dimension(6) :: valout ! coordonnées retournées dans le repère de calcul
integer :: i,j

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere
do i=1,3
  ! traitement des coords min
  j=i*2-1
  if (chin(j)==MOINS_INF) then
    valin(j) = m%borne_reel(j)  ! valeur infinie
  else
    read(chin(j),*,err=999) valin(j)
  end if
  j=j+1
  if (chin(j)==PLUS_INF) then
    valin(j) = m%borne_reel(j)  ! valeur infinie
  else
    read(chin(j),*,err=999) valin(j)
  end if
  
end do

if (flag_chrep) valout = origine_reelle_abs6(m,valin)

return
999 print '(/,t10,"ERREUR d''extraction des données de la ligne sur le mot :",A)',trim(chin(j))
    call lecture_stop(f)


end function conv_chcoord6_reel_reel


! ==========================================================
! == function conv_coord6_cell_inttostr                    ==
! == Convertit les coordonnées de type entier en coordonnées
! == de type cfaine, function indispensable pour gérer
! == les mots clés -inf,+inf
! == Change les 6 coordonnées en nbre de cellules
! == dans le repère UTILISATEUR, le nombre de cell est entier
! ==========================================================	
function conv_coord6_cell_inttostr(m,valin,change_repere) result(chout)
type(st_maillage),intent(in) :: m
integer, dimension(6),intent(in) :: valin
logical,intent(in),optional :: change_repere

logical :: flag_chrep
character(LEN=LEN_MOT_CLE),dimension(6)  :: chout
integer, dimension(6) :: valout ! coordonnées retournées dans le repère de calcul
integer :: i,j


flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

! changement de repère, repère calcul --> repère utilisateur
if (flag_chrep) valout= origine_cellule6(m,valin,.true.)

do i=1,3
  ! traitement des coords min
  j=i*2-1
  if (valout(j)<=m%borne_cell(j)) then
    chout(j) = MOINS_INF ! valeur infinie
  else
    write(chout(j),'(I6)') valout(j)
  end if
  j=j+1
  if (valout(j)>=m%borne_cell(j)) then
    chout(j) = PLUS_INF  ! valeur infinie
  else
    write(chout(j),'(I6)') valout(j)
  end if
  
end do

end function conv_coord6_cell_inttostr

! ==========================================================
! == function conv_coord6_cell_reeltostr                  ==
! == Convertit les coordonnées en nbre de cellules de type réel en coordonnées
! == de type cfaine, function indispensable pour gérer
! == les mots clés -inf,+inf
! == Change les 6 coordonnées en nbre de cellules
! == dans le repère UTILISATEUR, le nombre de cell est réel
! ==========================================================	
function conv_coord6_cell_reeltostr(m,valin,change_repere) result(chout)
type(st_maillage),intent(in) :: m
real, dimension(6),intent(in) :: valin
logical,intent(in),optional :: change_repere

logical :: flag_chrep
character(LEN=LEN_MOT_CLE),dimension(6)  :: chout
real, dimension(6) :: valout ! coordonnées retournées dans le repère de calcul
integer :: i,j

flag_chrep = .true.
if (present(change_repere)) flag_chrep = change_repere

! changement de repère, repère calcul --> repère utilisateur
if (flag_chrep) valout= origine_cellule6(m,valin,.true.)

do i=1,3
  ! traitement des coords min
  j=i*2-1
  if (valout(j)<=m%borne_cell(j)) then
    chout(j) = MOINS_INF ! valeur infinie
  else
    write(chout(j),'(F10.4)') valout(j)
  end if
  j=j+1
  if (valout(j)>=m%borne_cell(j)) then
    chout(j) = PLUS_INF  ! valeur infinie
  else
    write(chout(j),'(F10.4)') valout(j)
  end if
  
end do

end function conv_coord6_cell_reeltostr


end module m_maillage

