!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_math_operation.f90
!// Purpose:   quelques fonctions mathématiques de base ...
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Novembre 2001
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
module m_math_operation
use m_constantes_scientifiques, only : PI

implicit none
integer, parameter  :: k14 = selected_real_kind(14)
integer, parameter  :: PRECISION_dp  = 14
integer, parameter  :: dp  = selected_real_kind(PRECISION_dp)

contains


! ==========================================================
! == subroutine sqrtm_d_dp
! == Racine carré d'une matrice diagonale                ==
! == double précision
! ==========================================================
subroutine sqrtm_d_dp(N,A)

integer,intent(in)::N
complex(kind=8),dimension(N,N),intent(inout)::A
integer::i,j

do i=1,N
   do j=1,N
     if ((i/=j).and.(A(i,j)/=0)) then
       print*,"erreur la matrice n''est pas diagonale"
       stop
     end if   
   end do
end do

do i=1,N
    A(i,i)=sqrt(A(i,i))
end do

end subroutine sqrtm_d_dp

! ==========================================================
! == subroutine expm_d_dp
! == exponentielle d'une matrice diagonale                ==
! == double précision
! ==========================================================
subroutine expm_d_dp(N,A)

implicit none

integer,intent(in)::N
complex(kind=8),dimension(N,N),intent(inout)::A
integer::i,j,k
complex(kind=8)::temp,temp2

do i=1,N
   do j=1,N
     if ((i/=j)) then
       if (A(i,j)/=0)then
            print*,"ERREUR SUBROUTINE expm_d_dp, erreur la matrice n''est pas diagonale"
            stop
       end if
     end if   
   end do
end do
!print*,exp((5000,5))
do k=1,N
    temp2=A(k,k)
    print *,temp2
    temp=exp(temp2)
    print *,temp
    A(k,k)=temp
end do

end subroutine expm_d_dp


! ==========================================================
! == subroutine expm_d
! == Exponentielle d'une matrice diagonale                ==
! == Simple précision
! ==========================================================
subroutine expm_d(N,A)

integer,intent(in)::N
complex,dimension(N,N),intent(inout)::A
integer::i,j,k
complex::temp,temp2

do i=1,N
   do j=1,N
     if ((i/=j)) then
       if (A(i,j)/=0)then
            print*,"ERREUR SUBROUTINE expm_d, erreur la matrice n''est pas diagonale"
             !stop
       end if
     end if   
   end do
end do
do k=1,N
    
    A(k,k)=cexp(A(k,k))
end do

end subroutine expm_d


! ==========================================================
! == function asincmplx
! == Calcul le sinus inverse d'un nombre complexe         ==
! == Valable entre -pi/2 et pi/2                          ==
! ==========================================================
complex(kind=k14) function asincmplx(a)
complex(kind=k14), intent(in) :: a
asincmplx = -CMPLX(0.0,1.0)*log(CMPLX(0.0,1.0)*a+sqrt(1-a**2))
end function asincmplx
! ==========================================================
! == function tanhcmplx
! == Calcul le tangente hyperbolique d'un nombre complexe ==
! ==========================================================
complex function tanhcmplx(a)
complex, intent(in) :: a
tanhcmplx = 1-2*exp(-a)/(exp(a)+exp(-a))
end function tanhcmplx

!==========================================================
! == function coeff_binomiale_dp
! == Calcul de coefficients biniomaux à partir d'un binome
! == d'entier (calcul en double précision)
!==========================================================
real(dp) function coeff_binomiale_dp(l,i)
integer, intent(in) :: l,i

integer :: m
integer :: i1,i2,i3

coeff_binomiale_dp = 0.0_dp
if ((i<0).or.(i>l)) return

coeff_binomiale_dp = 1.0_dp
if (i==0) return

i1=1;i2=1;i3=1
do m=1,l
  i1 = i1*m
end do
do m=1,i
  i2 = i2*m
end do
if (i==l) then
  coeff_binomiale_dp = real(i1,dp)/real(i2,dp)
else
  do m=1,l-i
    i3 = i3*m
  end do
  coeff_binomiale_dp = real(i1,dp)/real(i2*i3,dp)
end if
end function coeff_binomiale_dp
!==========================================================


! ==========================================================
! == function ginvmat_dp
! == Calcul de la matrice inverse par le pivot de gauss   ==
! == Double précision
! ==========================================================
function ginvmat_dp(amat,m)

integer, intent(in)  :: m
real(dp), dimension(1:m,1:m), intent(in) :: amat
real(dp), dimension(1:m,1:m)   :: ginvmat_dp ! matrice inverse retournée	    

real(dp), dimension(1:m,1:m)   :: un(m,m)	    
real(dp), dimension(1:m,1:m+1) :: a(m,m+1)	    
integer  :: i,j,k, icol

! Construction de la matrice identité
un(:,:) = 0.0_dp
do i=1,m
  un(i,i)=1.0_dp
enddo


do icol=1,m

! Initialisation de la matrice et du vecteur de travail
! placement dans une matrice (A) m*(m+1)
  do i=1,m
    do j=1,m
      a(i,j)=amat(i,j)
    enddo
    a(i,m+1)=un(i,icol)
  enddo

! Résolution du système linéaire 
  do k=1,m
     do j=k+1,m+1
      a(k,j)=a(k,j)/a(k,k)
    enddo
    do i=1,m
      if (i/=k) then
        do j=k+1,m+1
          a(i,j)=a(i,j)-a(i,k)*a(k,j)
        enddo
      endif
    enddo
  enddo

  do k=1,m
    ginvmat_dp(k,icol)=a(k,m+1)
  enddo
enddo

end function ginvmat_dp



! ==========================================================
! == function ginvmat
! == Calcul de la matrice inverse par le pivot de gauss   ==
! ==========================================================
function ginvmat(amat,m)

integer, intent(in)  :: m
real, dimension(1:m,1:m), intent(in) :: amat
real, dimension(1:m,1:m)   :: ginvmat ! matrice inverse retournée	    

real, dimension(1:m,1:m)   :: un(m,m)	    
real, dimension(1:m,1:m+1) :: a(m,m+1)	    
integer  :: i,j,k, icol

! Construction de la matrice identité
un(:,:) = 0.0
do i=1,m
  un(i,i)=1.0
enddo


do icol=1,m

! Initialisation de la matrice et du vecteur de travail
! placement dans une matrice (A) m*(m+1)
  do i=1,m
    do j=1,m
      a(i,j)=amat(i,j)
    enddo
    a(i,m+1)=un(i,icol)
  enddo

! Résolution du système linéaire 
  do k=1,m
     do j=k+1,m+1
      a(k,j)=a(k,j)/a(k,k)
    enddo
    do i=1,m
      if (i/=k) then
        do j=k+1,m+1
          a(i,j)=a(i,j)-a(i,k)*a(k,j)
        enddo
      endif
    enddo
  enddo

  do k=1,m
    ginvmat(k,icol)=a(k,m+1)
  enddo
enddo

end function ginvmat

!===========================================================	
! subroutine tfd_tab1d
! Calcul de la transformée de fourrier discrete 
!===========================================================	
subroutine tfd_tab1d(tab_t,tab_f,ntmin,ntmax,nfmin,nfmax,dt,df)

!=== Déclaration des arguments ===
real, dimension(0:), intent(in)  :: tab_t
complex, dimension(0:), intent(out)  :: tab_f
integer, intent(in)  :: ntmin,ntmax,nfmin,nfmax
real, intent(in)  :: dt, df

!===  Déclaration des variables  ===
integer  :: it , ifreq
complex  :: exp_i2pidtdf,cmp_1

!===   Début des traitements   ===
cmp_1 = CMPLX(0.0,-2*PI*dt*df)
exp_i2pidtdf = CEXP(cmp_1)

do ifreq = nfmin, nfmax
do it = ntmin, ntmax
  tab_f(ifreq-nfmin) = tab_f(ifreq-nfmin) + tab_t(it-ntmin) * exp_i2pidtdf**(it*ifreq) 
end do
end do

end subroutine tfd_tab1d

!===========================================================	
! subroutine atan2d_cor
! Encapsule la fonction f90 atan2d en corrigeant le cas où
! la partie réelle du nombre complexe est nulle 
! 
! Retourne l'angle en degré
!===========================================================	
real function atan2d_cor(valeur_complexe)


!=== Déclaration des arguments ===
complex, intent(in)  :: valeur_complexe

!===  Déclaration des variables  ===
real  :: partie_reelle, partie_immaginaire
real  :: rad_deg
!===   Début des traitements   ===
partie_reelle = REAL(valeur_complexe)
partie_immaginaire = AIMAG(valeur_complexe)
rad_deg = 180.0 / PI
if (partie_reelle == 0.0) then
  if (partie_immaginaire >= 0.0) then
    atan2d_cor = 90.0
  else 
    atan2d_cor = -90.0
  end if
else
  atan2d_cor = ATAN2(partie_immaginaire, partie_reelle) * rad_deg
end if

end function atan2d_cor

! ==========================================================
! == function deriv_prem_fonct_cont
! == Dérivée centrée d'une fonction réelle définie par ses 
! == échantillons contenus dans fonc1.
! == La fonction func1 est nulle au départ et à la fin
! == La dérivée est retournée par le tableau fonc2
! ==========================================================
function deriv_prem_fonct_cont(fonc1,dt) result(fonc2)

real, intent(in)  :: dt ! le pas d'échantillonage dans fonc1
real, dimension(1:), intent(in) :: fonc1
real, dimension(1:size(fonc1))   :: fonc2 ! tableau retourné

integer  :: i,n

n = size(fonc1)
fonc2(1:n) = 0.0
! Dérivée première de func1
do i=2,n-1
 fonc2(i) = fonc1(i+1)-fonc1(i-1);
end do
fonc2(1:n) = fonc2(1:n) / (2*dt)

end function deriv_prem_fonct_cont

! ==========================================================
! == function integrale_prem_fonct_cont
! == intégrale première d'une d'une fonction réelle définie par ses 
! == échantillons contenus dans fonc1.
! == La fonction func1 est nulle au départ et à la fin
! == L'intégrale est retournée par le tableau fonc2
! ==========================================================
function integrale_prem_fonct_cont(fonc1,dt) result(fonc2)

real, intent(in)  :: dt ! le pas d'échantillonage dans fonc1
real, dimension(1:), intent(in) :: fonc1
real, dimension(1:size(fonc1))   :: fonc2 ! tableau retourné

integer  :: i,n

n = size(fonc1)
fonc2(1:n) = 0.0
! Dérivée première de func1
do i=2,n
 fonc2(i) = fonc2(i-1)+fonc1(i-1);
end do
fonc2(1:n) = fonc2(1:n) *dt

end function integrale_prem_fonct_cont

! ==========================================================
! == function deriv_sec_fonct_cont
! == Dérivée seconde et centrée d'une fonction réelle définie par ses 
! == échantillons contenus dans fonc1.
! == La fonction func1 est nulle au départ et à la fin
! == La dérivée seconde est retournée par le tableau fonc2
! ==========================================================
function deriv_sec_fonct_cont(fonc1,dt) result(fonc2)

real, intent(in)  :: dt ! le pas d'échantillonage dans fonc1
real, dimension(1:), intent(in) :: fonc1
real, dimension(1:size(fonc1))   :: fonc2 ! tableau retourné

integer  :: i,n

n = size(fonc1)
fonc2(1:n) = 0.0
! Dérivée seconde de func1
do i=2,n-1
 fonc2(i) = fonc1(i+1)+fonc1(i-1)-2*fonc1(i);
end do
fonc2(1:n) = fonc2(1:n) / (dt**2)

end function deriv_sec_fonct_cont


!========================================================================
!  It runs and computes correctly although it is not in any sense
!  sophisticated.  However, it is free, and useful if you are in a hurry
!  and don't want to think things out for yourself.

!  Arthur Wouk (wouk@brl-vgr)

!  *******************
!  Fast Fourier Transform
!  *******************

! Code converted using TO_F90 by Alan Miller
! Date: 2003-07-09  Time: 17:02:48

SUBROUTINE fft_sp(x, n, k)
!    FFT COMPUTES THE (FAST) FOURIER TRANSFORM OF THE VECTOR X
!    (A COMPLEX ARRAY OF DIMENSION N).
!    SOURCE: Joel H. Ferziger; Numerical methods for engineering applications.
!    2nd edition, Wiley (1998).

!    X = DATA TO BE TRANSFORMED; ON RETURN IT CONTAINS THE TRANSFORM.
!    N = SIZE OF VECTOR.  MUST BE A POWER OF 2 (<32769).
!    K = 1 FOR FORWARD TRANSFORM.
!    K = -1 FOR INVERSE TRANSFORM.

IMPLICIT NONE
INTEGER, INTENT(IN)       :: n
COMPLEX , INTENT(IN OUT)  :: x(n)
INTEGER, INTENT(IN)       :: k

INTEGER   :: sby2, s
REAL      :: ang, re, im
COMPLEX   :: xtemp, t, u(16), v, w
LOGICAL   :: NEW
INTEGER   :: i, INDEXE, itemp, j, jndex, l, l2n, p, q, stage
REAL      :: pi2 = 6.2831853071796, gain = 1.0
INTEGER   :: no = 0, ko = 0

!        TEST FIRST CALL?
NEW = (no /= n)
NEW = .true. ! modification C. GUIFFAUT
IF (NEW) THEN
  
!        IF FIRST CALL COMPUTE LOG2 (N).
  
  l2n = 0
  no = 1
  10 l2n = l2n + 1
  no = no + no
  IF (no < n) GO TO 10
  gain = 1. / n
  ang = pi2 * gain
  re = COS(ang)
  im = SIN(ang)
!  print '(T10,"ang=",ES15.6)',ang
  
END IF

!        COMPUTE COMPLEX EXPONENTIALS IF NOT FIRST CALL

IF (.NOT.(.NOT.NEW .AND. k*ko >= 1)) THEN
  u(1) = CMPLX(re, -SIGN(im, REAL(k)))
  DO  i = 2, l2n
    u(i) = u(i-1) * u(i-1)
  END DO
!  The code from netlib cntained the line:
!  K0 = K
  ko = k
END IF
!if (k==-1) then
!  print '(T10,"u=",32ES15.3)',u(:)
!end if

!        MAIN LOOP

sby2 = n
DO  stage = 1, l2n
  v = u(stage)
  w = (1.0, 0.0)
  s = sby2
  sby2 = s / 2
  DO  l = 1, sby2
    DO  i = 1, n, s
      p = i + l - 1
      q = p + sby2
      t = x(p) + x(q)
      x(q) = (x(p)-x(q)) * w
      x(p) = t
    END DO
    w = w * v
  END DO
END DO

!        REORDER THE ELEMENTS BY BIT REVERSAL

DO  i = 1, n
  INDEXE = i - 1
  jndex = 0
  DO  j = 1, l2n
    jndex = jndex + jndex
    itemp = INDEXE / 2
    IF (itemp+itemp /= INDEXE) jndex = jndex + 1
    INDEXE = itemp
  END DO
  j = jndex + 1
  IF (j >= i) THEN
    xtemp = x(j)
    x(j) = x(i)
    x(i) = xtemp
  END IF
END DO

!        FORWARD TRANSFORM DONE

IF (k > 0) RETURN
!if (k==-1) then
!  print '(T10,"l2n=",I6)',l2n
!  print '(T10,"NEW=",L)',NEW
!  print '(T10,"no=",I6)',no
!  print '(T10,"gain=",ES15.6)',gain
!  print '(T10,"ang=",ES15.6)',ang
!  print '(T10,"pi2=",ES15.6)',pi2
!end if


!        INVERSE TRANSFORM

x(1:n) = x(1:n) * gain
RETURN
END SUBROUTINE fft_sp

!  It runs and computes correctly although it is not in any sense
!  sophisticated.  However, it is free, and useful if you are in a hurry
!  and don't want to think things out for yourself.

!  Arthur Wouk (wouk@brl-vgr)

!  *******************
!  Fast Fourier Transform
!  *******************

! Code converted using TO_F90 by Alan Miller
! Date: 2003-07-09  Time: 17:02:48

SUBROUTINE fftd(x, n, k)
!    FFT COMPUTES THE (FAST) FOURIER TRANSFORM OF THE VECTOR X
!    (A COMPLEX ARRAY OF DIMENSION N).
!    SOURCE: Joel H. Ferziger; Numerical methods for engineering applications.
!    2nd edition, Wiley (1998).

!    X = DATA TO BE TRANSFORMED; ON RETURN IT CONTAINS THE TRANSFORM.
!    N = SIZE OF VECTOR.  MUST BE A POWER OF 2 (<32769).
!    K = 1 FOR FORWARD TRANSFORM.
!    K = -1 FOR INVERSE TRANSFORM.

IMPLICIT NONE
INTEGER, PARAMETER  :: dp = SELECTED_REAL_KIND(14, 60)

INTEGER, INTENT(IN)           :: n
COMPLEX (dp), INTENT(IN OUT)  :: x(n)
INTEGER, INTENT(IN)           :: k

INTEGER       :: sby2, s
REAL (dp)     :: ang, re, im
COMPLEX (dp)  :: xtemp, t, u(16), v, w
LOGICAL       :: NEW
INTEGER       :: i, INDEXE, itemp, j, jndex, l, l2n, p, q, stage
REAL (dp)     :: pi2 = 6.2831853071796_dp, gain = 1.0_dp
INTEGER       :: no = 0, ko = 0

!        TEST FIRST CALL?

NEW = (no /= n)
IF (NEW) THEN
  
!        IF FIRST CALL COMPUTE LOG2 (N).
  
  l2n = 0
  no = 1
  10 l2n = l2n + 1
  no = no + no
  IF (no < n) GO TO 10
  gain = 1. / n
  ang = pi2 * gain
  re = COS(ang)
  im = SIN(ang)
END IF

!        COMPUTE COMPLEX EXPONENTIALS IF NOT FIRST CALL

IF (.NOT.(.NOT.NEW .AND. k*ko >= 1)) THEN
  u(1) = CMPLX(re, -SIGN(im, DBLE(k)), KIND=dp)
  DO  i = 2, l2n
    u(i) = u(i-1) * u(i-1)
  END DO
!  The code from netlib cntained the line:
!  K0 = K
  ko = k
END IF

!        MAIN LOOP

sby2 = n
DO  stage = 1, l2n
  v = u(stage)
  w = (1.0_dp, 0.0_dp)
  s = sby2
  sby2 = s / 2
  DO  l = 1, sby2
    DO  i = 1, n, s
      p = i + l - 1
      q = p + sby2
      t = x(p) + x(q)
      x(q) = (x(p)-x(q)) * w
      x(p) = t
    END DO
    w = w * v
  END DO
END DO

!        REORDER THE ELEMENTS BY BIT REVERSAL

DO  i = 1, n
  INDEXE = i - 1
  jndex = 0
  DO  j = 1, l2n
    jndex = jndex + jndex
    itemp = INDEXE / 2
    IF (itemp+itemp /= INDEXE) jndex = jndex + 1
    INDEXE = itemp
  END DO
  j = jndex + 1
  IF (j >= i) THEN
    xtemp = x(j)
    x(j) = x(i)
    x(i) = xtemp
  END IF
END DO

!        FORWARD TRANSFORM DONE

IF (k > 0) RETURN

!        INVERSE TRANSFORM

x(1:n) = x(1:n) * gain
RETURN
END SUBROUTINE fftd

end module m_math_operation
