!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_milieu_dispersif.f90
!// Purpose:     Gestion de milieux dispersif de type debye, Lorentz ou généralisé
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Octobre 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////


!================================================================
! Traitement des milieux dispersifs
!================================================================
module m_milieu_dispersif
use m_constantes
use m_constantes_scientifiques
use m_entree , only : st_fichier, &
                                  lineread, &
				  compteur_mot_cle, &
                                  compteur_N_mot_cle, &
				  lecture_stop, &
				  nb_mot_chaine, &
				  ctrl_nb_mot_chaine, &
				  ligne_precedente
				  
use m_milieu_simple, only : st_milieu_simple, &
				 milieu_simple_initdefaut,&
				 milieu_simple_copy,&
				 milieu_simple_deallocate,&
				 milieu_simple_ecriture

implicit none
!=== Déclaration des types ===
!--- Description d'un pole dispersif complexe ---
type st_un_pole_dispersif
  ! paramètre d'un milieu général
  logical  :: flag_reel
  character(LEN=LEN_MOT_CLE) :: nom_modele
  real  :: G      ! facteur pondératif des poles compris entre 0 et 1
  real  :: A, B, alpha, beta
end type st_un_pole_dispersif

type st_un_pole_dispersif_ES
  character(LEN=LEN_MOT_CLE) :: type_milieu, nom_modele
  real   :: g
  real   :: val_inf,val_stat,tau
  real   :: w,alpha,beta
  real   :: A,B
end type st_un_pole_dispersif_ES


!--- la structure d'un milieu_dispersif ---
type st_un_milieu_dispersif
  character(LEN=LEN_MOT_CLE) :: nom
  integer, dimension(3)   :: nb_pole_eps, nb_pole_mu
  logical                 :: flag_eps, flag_mu
  real, dimension(3)      :: eps_inf, mu_inf,eps_stat,mu_stat
  real, dimension(3)      :: sige, sigh
  ! partie non dispersive du milieu
  type(st_milieu_simple)  :: t_ms
  
  ! partie diagonale du tenseur de permittivité et perméabilité
  type(st_un_pole_dispersif), dimension(:), pointer :: t_epsx, t_epsy, t_epsz
  type(st_un_pole_dispersif), dimension(:), pointer :: t_mux, t_muy, t_muz
  
  ! Garde les données pour l'écriture
  integer :: nb_pole
  type(st_un_pole_dispersif_ES), dimension(:), pointer :: t_pole
  
end type st_un_milieu_dispersif

!--- Structure globale pour les milieux dispersifs ---
type st_milieu_dispersif
  integer  :: nb_milieu
  type(st_un_milieu_dispersif), dimension(:), pointer :: t_un_md    

end type st_milieu_dispersif
 
! Pas de variables globales...

contains

!===========================================================	
! Subroutine milieu_dispersif_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine milieu_dispersif_initdefaut(tmd)
 implicit none
!=== Déclaration des arguments ===
type(st_milieu_dispersif), intent(inout) :: tmd

!===  Déclaration des variables  ===

!===   Début des traitements   ===
tmd%nb_milieu = 0
nullify(tmd%t_un_md)

End Subroutine milieu_dispersif_initdefaut

!===========================================================	
! Subroutine milieu_dispersif_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine un_milieu_dispersif_initdefaut(un_md)
 implicit none
!=== Déclaration des arguments ===
type(st_un_milieu_dispersif), intent(inout) :: un_md

!===  Déclaration des variables  ===

!===   Début des traitements   ===
un_md%nom = ''
un_md%nb_pole_eps(:)  = 0
un_md%nb_pole_mu(:) = 0
un_md%flag_eps = .false.
un_md%flag_mu  = .false.
un_md%eps_inf = 1.0
un_md%mu_inf  = 1.0
un_md%eps_stat = 1.0
un_md%mu_stat  = 1.0
un_md%sige = 0.0
un_md%sigh = 0.0
un_md%nb_pole = 0
NULLIFY(un_md%t_epsx);NULLIFY(un_md%t_epsy);NULLIFY(un_md%t_epsz);
NULLIFY(un_md%t_mux); NULLIFY(un_md%t_muy); NULLIFY(un_md%t_muz);
nullify(un_md%t_pole)
call milieu_simple_initdefaut(un_md%t_ms)
				
End Subroutine un_milieu_dispersif_initdefaut

!===========================================================	
! Subroutine un_pole_dispersif_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine un_pole_dispersif_initdefaut(un_pd)
 implicit none
!=== Déclaration des arguments ===
type(st_un_pole_dispersif), intent(inout) :: un_pd

!===  Déclaration des variables  ===

!===   Début des traitements   ===
un_pd%nom_modele = ''
un_pd%flag_reel  = .false.
un_pd%A = 0; un_pd%B = 0;
un_pd%beta = 0; un_pd%alpha = 0;
un_pd%G = 1.0
End Subroutine un_pole_dispersif_initdefaut

!===========================================================	
! Subroutine un_pole_dispersifES_initdefaut
!===========================================================	
Subroutine un_pole_dispersifES_initdefaut(un_pd)
 implicit none
!=== Déclaration des arguments ===
type(st_un_pole_dispersif_ES), intent(out) :: un_pd

!===  Déclaration des variables  ===

!===   Début des traitements   ===
un_pd%nom_modele = ''
un_pd%type_milieu = ''
un_pd%g  = 1.0
un_pd%val_inf = 0; un_pd%val_stat = 0;
un_pd%tau = 0.0;
un_pd%w =0.0;
un_pd%A = 0; un_pd%B = 0;
un_pd%beta = 0; un_pd%alpha = 0;
End Subroutine un_pole_dispersifES_initdefaut

!===========================================================	
! Subroutine un_milieu_dispersif_copy
!===========================================================	
Subroutine un_milieu_dispersif_copy(mdin,mdout)
 implicit none
!=== Déclaration des arguments ===
type(st_un_milieu_dispersif), intent(in)  :: mdin
type(st_un_milieu_dispersif), intent(out) :: mdout

!===  Déclaration des variables  ===
integer :: i 

call un_milieu_dispersif_initdefaut(mdout)
mdout%nom = mdin%nom ;
mdout%nb_pole_eps = mdin%nb_pole_eps ;
mdout%nb_pole_mu = mdin%nb_pole_mu ;
mdout%flag_eps = mdin%flag_eps ;
mdout%flag_mu = mdin%flag_mu ;
mdout%eps_inf = mdin%eps_inf ;
mdout%mu_inf = mdin%mu_inf ;
mdout%eps_stat=mdin%eps_stat;
mdout%mu_stat = mdin%mu_stat;
mdout%sige = mdin%sige ;
mdout%sigh = mdin%sigh ;

call milieu_simple_copy(mdin%t_ms,mdout%t_ms)

mdout%nb_pole = mdin%nb_pole
if (mdin%nb_pole>0) then
  allocate(mdout%t_pole(1:mdin%nb_pole))
  mdout%t_pole = mdin%t_pole
end if
 
! inutile :
!t_epsx, t_epsy, t_epsz
!t_mux, t_muy, t_muz

end Subroutine un_milieu_dispersif_copy

!===========================================================	
! Subroutine un_milieu_dispersif_deallocate
!===========================================================	
Subroutine un_milieu_dispersif_deallocate(un_md)
 implicit none
!=== Déclaration des arguments ===
type(st_un_milieu_dispersif), intent(inout) :: un_md

!===  Déclaration des variables  ===
integer :: i

if (associated(un_md%t_epsx)) deallocate(un_md%t_epsx) 
if (associated(un_md%t_epsy)) deallocate(un_md%t_epsy) 
if (associated(un_md%t_epsz)) deallocate(un_md%t_epsz)
if (associated(un_md%t_mux)) deallocate(un_md%t_mux) 
if (associated(un_md%t_muy)) deallocate(un_md%t_muy) 
if (associated(un_md%t_muz)) deallocate(un_md%t_muz) 

if (un_md%nb_pole>0) deallocate(un_md%t_pole)
call milieu_simple_deallocate(un_md%t_ms)
end Subroutine un_milieu_dispersif_deallocate

!===========================================================	
! Subroutine milieu_dispersif_deallocate
!===========================================================	
Subroutine milieu_dispersif_deallocate(tmd)
 implicit none
!=== Déclaration des arguments ===
type(st_milieu_dispersif), intent(inout) :: tmd

!===  Déclaration des variables  ===
integer :: i
!===   Début des traitements   ===
if (tmd%nb_milieu>0) then
  do i=1,tmd%nb_milieu
    call un_milieu_dispersif_deallocate(tmd%t_un_md(i))
  end do
  deallocate(tmd%t_un_md);nullify(tmd%t_un_md)
end if
End Subroutine milieu_dispersif_deallocate

!===========================================================	
! Subroutine milieu_dispersif_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine milieu_dispersif_lecture(f,tmd)
implicit none
!=== Déclaration des arguments ===
type(st_fichier), intent(inout):: f
type(st_milieu_dispersif), intent(inout) :: tmd
!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: premier_mot, corbeille
integer :: ind, status

!===   Début des traitements   ===
call milieu_dispersif_initdefaut(tmd)

! on compte le nbre de milieux
tmd%nb_milieu = compteur_mot_cle(f,'milieu')
if (tmd%nb_milieu==0) return ! on sort

allocate(tmd%t_un_md(tmd%nb_milieu))

! lecture des données de chaque milieu
ind = 0
do 
  premier_mot = lineread(f)
  if (premier_mot=='milieu') then
    ind = ind +1
    call un_milieu_dispersif_initdefaut(tmd%t_un_md(ind))
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*) corbeille, tmd%t_un_md(ind)%nom
    call un_milieu_dispersif_lecture(f,tmd%t_un_md(ind))
    
  else if (premier_mot == f%nomfinbloc) then
    exit
    
  else
    print '("Mot clé non reconnu : ",A)', premier_mot
    call lecture_stop(f)  
    
  endif
enddo
				
End Subroutine milieu_dispersif_lecture


!===========================================================	
! Subroutine un_milieu_dispersif_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine un_milieu_dispersif_lecture(f,un_md)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout):: f
type(st_un_milieu_dispersif), intent(inout) :: un_md
!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: corbeille, premier_mot, nom_modele
character(LEN=LEN_MOT_CLE), dimension(4) :: mots_eps, mots_mu
integer  :: erreur
integer, dimension(4)  :: pole_eps, pole_mu 
logical  :: flag_reel
real   :: val_inf, val_statique
real   :: tau, gd  ! paramètres modèle Debye
real   :: W, alpha, gl ! paramètres modèle Lorentz
 
integer  :: i,j, status,ipole
integer  :: iepsx, iepsy, iepsz
integer  :: imux, imuy, imuz
type(st_un_pole_dispersif)  :: un_pd
type(st_un_pole_dispersif_ES), pointer :: unpole

!===   Début des traitements   ===
mots_eps = (/"epsrd  ","epsrdxx","epsrdyy","epsrdzz"/)
mots_mu = (/"murd  ","murdxx","murdyy","murdzz"/)

! on compte le nbre de poles par mot clés
pole_eps(:) = compteur_N_mot_cle(f,4,mots_eps,'milieu')
pole_mu(:) = compteur_N_mot_cle(f,4,mots_mu,'milieu')

! ajout des poles des "epsrd" à ses composantes
do i=2,4
  pole_eps(i) = pole_eps(i)+pole_eps(1)
  pole_mu(i) = pole_mu(i)+pole_mu(1)
end do

un_md%nb_pole_eps(1:3) = pole_eps(2:4) ! nombre de poles par composante de la permittivité
un_md%nb_pole_mu(1:3)  = pole_mu(2:4)

! allocation des structures pour chaque composante 
! de permittivité et perméabilité
if (pole_eps(2) > 0) allocate(un_md%t_epsx(pole_eps(2)))  
if (pole_eps(3) > 0) allocate(un_md%t_epsy(pole_eps(3)))  
if (pole_eps(4) > 0) allocate(un_md%t_epsz(pole_eps(4)))  

if (pole_mu(2) > 0) allocate(un_md%t_mux(pole_mu(2)))  
if (pole_mu(3) > 0) allocate(un_md%t_muy(pole_mu(3)))  
if (pole_mu(4) > 0) allocate(un_md%t_muz(pole_mu(4)))  

! pour les E/S 
un_md%nb_pole = sum(pole_eps(1:4))+sum(pole_mu(1:4))
if (un_md%nb_pole>0) then
  allocate(un_md%t_pole(1:un_md%nb_pole))
else
  return
end if


! lecture des données de chaque pole du milieu
iepsx = 0 ; iepsy = 0 ; iepsz = 0;
imux = 0  ; imuy = 0  ; imuz = 0;
ipole =0
un_md%t_ms%nom = un_md%nom
do 
  premier_mot = lineread(f)

  if (premier_mot == f%nomfinbloc .or.premier_mot == 'milieu' ) then
    call ligne_precedente(f)
    exit
  end if
  !--- Cas des paramètres non dispersifs ---
  status = ctrl_nb_mot_chaine(f,f%chaine,2,7)
  select case(premier_mot)
    case('epsr')  ; 
      read(f%chaine,*) corbeille,un_md%t_ms%eps_r(1)
      un_md%t_ms%eps_r(1:3) = un_md%t_ms%eps_r(1)
    case('epsrx') ; read(f%chaine,*) corbeille,un_md%t_ms%eps_r(1) 
    case('epsry') ; read(f%chaine,*) corbeille,un_md%t_ms%eps_r(2)
    case('epsrz') ; read(f%chaine,*) corbeille,un_md%t_ms%eps_r(3)

    case('mur')  ; 
      read(f%chaine,*) corbeille,un_md%t_ms%mu_r(1)
      un_md%t_ms%mu_r(1:3) = un_md%t_ms%mu_r(1)
    case('murx') ; read(f%chaine,*) corbeille,un_md%t_ms%mu_r(1)
    case('mury') ; read(f%chaine,*) corbeille,un_md%t_ms%mu_r(2)
    case('murz') ; read(f%chaine,*) corbeille,un_md%t_ms%mu_r(3)

    case('sige')  ; 
      read(f%chaine,*) corbeille,un_md%t_ms%sig_e(1)
      un_md%t_ms%sig_e(1:3) = un_md%t_ms%sig_e(1)
    case('sigex') ; read(f%chaine,*) corbeille,un_md%t_ms%sig_e(1)
    case('sigey') ; read(f%chaine,*) corbeille,un_md%t_ms%sig_e(2)
    case('sigez') ; read(f%chaine,*) corbeille,un_md%t_ms%sig_e(3)

    case('sigh')  ; 
      read(f%chaine,*) corbeille,un_md%t_ms%sig_h(1)
      un_md%t_ms%sig_h(1:3) = un_md%t_ms%sig_h(1)
    case('sighx') ; read(f%chaine,*) corbeille,un_md%t_ms%sig_h(1)
    case('sighy') ; read(f%chaine,*) corbeille,un_md%t_ms%sig_h(2)
    case('sighz') ; read(f%chaine,*) corbeille,un_md%t_ms%sig_h(3)
  end select
  !--- Cas des paramètres dispersifs ---
  nom_modele = '';
  if (nb_mot_chaine(f%chaine) == 2) cycle ! lecture ligne suivante dans le fichier
  
  read(f%chaine,*) premier_mot, nom_modele
  
  ! init variables du pole en cours de lecture
  call un_pole_dispersif_initdefaut(un_pd)
  ! Traitement des données du pole en cours
  select case (nom_modele)
    case ('debye')
      status = ctrl_nb_mot_chaine(f,f%chaine,6)
      read(f%chaine,*) premier_mot, un_pd%nom_modele, val_inf, val_statique, tau, gd
      erreur = converti_debye_modele_general(val_inf, val_statique, tau, gd, un_pd%A, un_pd%alpha)
      if (erreur < 0) call lecture_stop(f)
      un_pd%flag_reel = .true.
      !--- pour les E/S ---
      ipole = ipole+1
      unpole =>un_md%t_pole(ipole)
      unpole%type_milieu = premier_mot
      unpole%nom_modele = un_pd%nom_modele
      unpole%val_inf = val_inf ; unpole%val_stat = val_statique
      unpole%tau = tau;   unpole%g = gd
      
    case ('lorentz')
      status = ctrl_nb_mot_chaine(f,f%chaine,7)
      read(f%chaine,*) premier_mot, un_pd%nom_modele, val_inf, val_statique,W, alpha,gl
      erreur = converti_lorentz_modele_general(val_inf, val_statique,W,alpha,gl,un_pd%B,un_pd%beta,un_pd%alpha)
      if (erreur < 0) call lecture_stop(f)
      !--- pour les E/S ---
      ipole = ipole+1
      unpole =>un_md%t_pole(ipole)
      unpole%type_milieu = premier_mot
      unpole%nom_modele = un_pd%nom_modele
      unpole%val_inf = val_inf ; unpole%val_stat = val_statique
      unpole%w = W;   unpole%g = gl
      unpole%alpha = alpha
    case ('modele_global')
      status = ctrl_nb_mot_chaine(f,f%chaine,7)
      read(f%chaine,*) premier_mot,un_pd%nom_modele,val_inf,un_pd%A,un_pd%B,un_pd%alpha,un_pd%beta
      if (un_pd%B==0 .or. un_pd%beta == 0) un_pd%flag_reel = .true.            
      !--- pour les E/S ---
      ipole = ipole+1
      unpole =>un_md%t_pole(ipole)
      unpole%type_milieu = premier_mot
      unpole%nom_modele = un_pd%nom_modele
      unpole%val_inf = val_inf ; 
      unpole%A = un_pd%A;   unpole%B = un_pd%B
      unpole%alpha = un_pd%alpha ; unpole%beta = un_pd%beta;
    case default
      print '(T5,"Nom du modèle de milieu dispersif erroné !!!")'
      call lecture_stop(f)
  end select
  ! Affectation des données à la bonne composante epsilon ou mu
  select case (premier_mot)
  case ('epsrdxx')
    un_md%eps_inf(1) = val_inf
    un_md%eps_stat(1) = val_statique
    iepsx = iepsx + 1
    un_md%t_epsx(iepsx) = un_pd
  case ('epsrdyy')
    un_md%eps_inf(2) = val_inf
    un_md%eps_stat(2) = val_statique
    iepsy = iepsy + 1
    un_md%t_epsy(iepsy) = un_pd
  case ('epsrdzz')
    un_md%eps_inf(3) = val_inf
    un_md%eps_stat(3) = val_statique
    iepsz = iepsz + 1
    un_md%t_epsz(iepsz) = un_pd
  case ('epsrd') 
    un_md%eps_inf(:) = val_inf
    un_md%eps_stat(:) = val_statique
    iepsx = iepsx + 1
    iepsy = iepsy + 1
    iepsz = iepsz + 1
    un_md%t_epsx(iepsx) = un_pd
    un_md%t_epsy(iepsy) = un_pd
    un_md%t_epsz(iepsz) = un_pd
  case ('murdxx')
    un_md%mu_inf(1) = val_inf
    un_md%mu_stat(1) = val_statique
    imux = imux + 1
    un_md%t_mux(imux) = un_pd
  case ('murdyy')
    un_md%mu_inf(2) = val_inf
    un_md%mu_stat(2) = val_statique
    imuy = imuy + 1
    un_md%t_muy(imuy) = un_pd
  case ('murdzz')
    un_md%mu_inf(3) = val_inf
    un_md%mu_stat(3) = val_statique
    imuz = imuz + 1
    un_md%t_muz(imuz) = un_pd
  case ('murd')
    un_md%mu_inf(:) = val_inf
    un_md%mu_stat(:) = val_statique
    imux = imux + 1
    imuy = imuy + 1
    imuz = imuz + 1
    un_md%t_mux(imux) = un_pd
    un_md%t_muy(imuy) = un_pd
    un_md%t_muz(imuz) = un_pd
  case default
    print '("Mot clé non reconnu : ",A)', premier_mot
    call lecture_stop(f)      
  end select 

enddo

End Subroutine un_milieu_dispersif_lecture


! ==========================================================
! == Subroutine milieu_dispersif_ecriture                   ==
! ==========================================================
subroutine milieu_dispersif_ecriture(f,tmd)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_milieu_dispersif), intent(inout) :: tmd

integer  :: i,j
type(st_un_milieu_dispersif), pointer :: un_md
type(st_un_pole_dispersif_ES),pointer :: unpole
!----- Ecriture  ---------
if (tmd%nb_milieu==0) return ! on sort

do i=1,tmd%nb_milieu
  un_md => tmd%t_un_md(i)
  write(f%unite,'(T3,A,3x,A)') 'milieu',un_md%nom
  !do j = 1,3
  !  if (un_md%eps_inf(j) /= 1.0) un_md%t_ms%eps_r(j) = 1.0
  !  if (un_md%mu_inf(j)  /= 1.0) un_md%t_ms%mu_r(j)  = 1.0
  !  un_md%t_ms%sig_e(j) = un_md%t_ms%sig_e(j)-un_md%sige(j)
  !  un_md%t_ms%sig_h(j) = un_md%t_ms%sig_h(j)-un_md%sigh(j)
  !end do
  
  call milieu_simple_ecriture(f,un_md%t_ms)
  
  do j=1,un_md%nb_pole
    unpole => un_md%t_pole(j)
    
    select case (unpole%nom_modele)
      case ('debye')
        write(f%unite,'(T5,A,1x,A,1x,2F10.3,1x,G14.6,1x,F10.4)') trim(unpole%type_milieu), &
	  trim(unpole%nom_modele),unpole%val_inf,unpole%val_stat,unpole%tau,unpole%g
      case ('lorentz')
        write(f%unite,'(T5,A,1x,A,1x,2F10.3,1x,2G14.6,1x,F10.4)') trim(unpole%type_milieu), &
	  trim(unpole%nom_modele),unpole%val_inf,unpole%val_stat,unpole%w,unpole%alpha,unpole%g
      
      case ('modele_global')
        write(f%unite,'(T5,A,1x,A,1x,F10.3,1x,4G14.6,1x)') trim(unpole%type_milieu), &
	  trim(unpole%nom_modele),unpole%val_inf,unpole%A,unpole%B,unpole%alpha,unpole%beta
    end select   
  end do
end do

end subroutine milieu_dispersif_ecriture


!===========================================================	
! Function converti_debye_modele_general
! Converti modèle Debye en modèle général
!===========================================================	
integer function converti_debye_modele_general(val_inf, val_statique, tau, g, A, alpha)
implicit none
!=== Déclaration des arguments ===
real, intent(in)  :: val_inf, val_statique, tau, g
real, intent(out) :: A, alpha
!===  Déclaration des variables  ===

!===   Début des traitements   ===
converti_debye_modele_general = 0				
if (tau == 0) then 
  converti_debye_modele_general = -1
  print '(T5,"Erreur, il faut que tau = ",ES15.6E2," soit different de zéro")',tau
  return
end if
alpha = 1/tau
A = g * (val_statique - val_inf)/tau

End function converti_debye_modele_general

!===========================================================	
! Function converti_lorentz_modele_general
! Converti modèle Lorentz en modèle général
!===========================================================	
integer function converti_lorentz_modele_general(val_inf,val_statique,W,alpha, g, B, betag,alphag)
implicit none
!=== Déclaration des arguments ===
real, intent(in)  :: val_inf, val_statique,W,alpha,g
real, intent(out) :: B, betag,alphag
!===  Déclaration des variables  ===

!===   Début des traitements   ===
alphag = alpha

converti_Lorentz_modele_general = 0				
if (W <=alpha) then 
  converti_Lorentz_modele_general = -1
  print '(T5,"Erreur, il faut que ",ES15.6E2," > ",ES15.6E2)',W, alpha
  return
end if

betag  = SQRT(W**2-alpha**2)
B = g * (val_statique - val_inf) * W**2 / betag
				
End function converti_lorentz_modele_general


!===========================================================	
! Subroutine milieu_dispersif_init
! Initialisation des variables, lorsque qu'il n'y a pas de lecture du fichier
! d'entrée
!===========================================================	
Subroutine milieu_dispersif_init(tmd)
implicit none
!=== Déclaration des arguments ===
type(st_milieu_dispersif), intent(inout) :: tmd
!===  Déclaration des variables  ===
integer  :: i

!===   Début des traitements   ===
do i=1,tmd%nb_milieu
  call un_milieu_dispersif_init(tmd%t_un_md(i))
end do
				
End Subroutine milieu_dispersif_init

!===========================================================	
! Subroutine milieu_dispersif_init
! Calculs avant les itérations temporelles
!===========================================================	
Subroutine un_milieu_dispersif_init(un_md)
implicit none
!=== Déclaration des arguments ===
type(st_un_milieu_dispersif), intent(inout) :: un_md

!===  Déclaration des variables  ===
integer :: i

!===   Début des traitements   ===
if (ANY(un_md%nb_pole_eps>0)) then
  un_md%flag_eps = .true.
end if
if (ANY(un_md%nb_pole_mu>0)) then
  un_md%flag_mu = .true.
end if


! Mise à jour des conductivités
do i =1, un_md%nb_pole_eps(1)
  un_md%sige(1) = un_md%sige(1) + un_md%t_epsx(i)%A*EPSILON_0
end do
do i =1, un_md%nb_pole_eps(2)
  un_md%sige(2) = un_md%sige(2) + un_md%t_epsy(i)%A*EPSILON_0
end do
do i =1, un_md%nb_pole_eps(3)
  un_md%sige(3) = un_md%sige(3) + un_md%t_epsz(i)%A*EPSILON_0
  if (rang==RANG_MAITRE_AFF) then
    print '("nom milieu = ",A,"  A*EPS0 = ",ES15.6)',trim(un_md%nom),un_md%t_epsz(i)%A*EPSILON_0
    print '("nom milieu = ",A,"  un_md%sige(3) = ",ES15.6)',trim(un_md%nom),un_md%sige(3)
  end if
end do
do i =1, un_md%nb_pole_mu(1)
  un_md%sigh(1) = un_md%sigh(1) + un_md%t_mux(i)%A*MU_0
end do
do i =1, un_md%nb_pole_mu(2)
  un_md%sigh(2) = un_md%sigh(2) + un_md%t_muy(i)%A*MU_0
end do
do i =1, un_md%nb_pole_mu(3)
  un_md%sigh(3) = un_md%sigh(3) + un_md%t_muz(i)%A*MU_0
end do

! Mise à jour de la structure 				
do i = 1,3
  if (un_md%eps_inf(i) /= 1.0) un_md%t_ms%eps_r(i) = un_md%eps_inf(i)
  if (un_md%mu_inf(i)  /= 1.0) un_md%t_ms%mu_r(i) = un_md%mu_inf(i)
  un_md%t_ms%sig_e(i) = un_md%t_ms%sig_e(i)+un_md%sige(i)
  un_md%t_ms%sig_h(i) = un_md%t_ms%sig_h(i)+un_md%sigh(i)
end do
!print '("nom milieu = ",A,"  un_md%t_ms%sig_e(3) = ",ES15.6)',trim(un_md%nom),un_md%t_ms%sig_e(3)

End Subroutine un_milieu_dispersif_init

!===========================================================	
! Function identifie_milieu_dispersif
! Indique si le nom d'un milieu est un milieu dispersif
!===========================================================	
logical function identifie_milieu_dispersif(tmd,nom_milieu)
implicit none
!=== Déclaration des arguments ===
type(st_milieu_dispersif), intent(in) :: tmd
character (len=LEN_MOT_CLE), intent(in)  :: nom_milieu

!===  Déclaration des variables  ===
integer  :: i
type(st_un_milieu_dispersif), pointer :: un_md

!===   Début des traitements   ===
identifie_milieu_dispersif = .false.
do i=1,tmd%nb_milieu
  un_md => tmd%t_un_md(i)
  if (nom_milieu == un_md%nom) then
    if (ANY(un_md%nb_pole_eps > 0) .or. ANY(un_md%nb_pole_mu > 0)) then
      identifie_milieu_dispersif = .true.
      exit
    end if
  end if
end do

End function identifie_milieu_dispersif

!===========================================================	
! Function milieu_dispersif_pointe
! Retourne un pointeur sur la structure st_un_milieu_dispersif
! identifié par le nom passé en argument
!===========================================================	
function milieu_dispersif_pointe(tmd,nom_milieu) result(un_md)
implicit none
!=== Déclaration des arguments ===
type(st_milieu_dispersif), intent(in) :: tmd
character (len=LEN_MOT_CLE), intent(in)  :: nom_milieu

type(st_un_milieu_dispersif), pointer :: un_md

!===  Déclaration des variables  ===
integer  :: i

!===   Début des traitements   ===
NULLIFY(un_md)

do i=1,tmd%nb_milieu
  if (nom_milieu == tmd%t_un_md(i)%nom) then
    un_md => tmd%t_un_md(i)
    exit
  end if
end do

End function milieu_dispersif_pointe


!===========================================================	
! Subroutine milieu_dispersif_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine milieu_dispersif_affichage(tmd)
implicit none
!=== Déclaration des arguments ===
type(st_milieu_dispersif), intent(in) :: tmd

!===  Déclaration des variables  ===
integer  :: i,j,k
!===   Début des traitements   ===
if (rang/=RANG_MAITRE_AFF) return

print '("**************************************************")'
print '("Nombre de milieux dispersifs : ",I3)', tmd%nb_milieu

do i=1, tmd%nb_milieu
  print '(T5,"Milieu dispersif de nom : ",A)', TRIM(tmd%t_un_md(i)%nom)
  print '(T10,"Nb poles électriques : ", 3I3)', tmd%t_un_md(i)%nb_pole_eps(1:3)
  print '(T10,"Nb poles magnétiques : ", 3I3)', tmd%t_un_md(i)%nb_pole_mu(1:3)
  print '(T10,"eps_inf : ", 3F6.3)', tmd%t_un_md(i)%eps_inf(1:3)
  print '(T10,"mu_inf : ", 3F6.3)', tmd%t_un_md(i)%mu_inf(1:3)
  print '(T10,"sige : ", 3ES12.4E2)', tmd%t_un_md(i)%sige(1:3)
  print '(T10,"sigh : ", 3ES12.4E2)', tmd%t_un_md(i)%sigh(1:3)
end do
print '("**************************************************")'
				
End Subroutine milieu_dispersif_affichage

!===========================================================	
! Subroutine milieu_dispersif_postcalcul
!  Calcul après les itérations temporelles
!===========================================================	
Subroutine milieu_dispersif_postcalcul()
implicit none
!=== Déclaration des arguments ===

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine milieu_dispersif_postcalcul

!===========================================================	
! Subroutine milieu_dispersif_impression
!  Ecriture des résultats dans les fichiers de sortie
!===========================================================	
Subroutine milieu_dispersif_impression()
implicit none
!=== Déclaration des arguments ===

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine milieu_dispersif_impression

End Module m_milieu_dispersif	



