!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_milieu_simple.f90
!// Purpose:     Gestion des donnéesd'un milieu non dispersif
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Module module_milieu_simple                   ==
! == Traitement des milieux non dispersifs                ==
! == Type st_milieu_simple                         ==
! ==========================================================
module m_milieu_simple

use m_constantes, only : LEN_MOT_CLE, LEN_CHAINE
use m_entree , only : st_fichier, &
                                  lineread, &
				  ligne_precedente, &
				  ctrl_nb_mot_chaine, &
                                  lecture_stop
implicit none

! ==========================================================
! == Type milieu_simple                            ==
! ==========================================================
type st_milieu_simple
  character(LEN=LEN_MOT_CLE) :: nom
  real, dimension(3) :: eps_r  ! epsilon_relatif(x,y,z) 
  real, dimension(3) :: mu_r   ! epsilon_relatif(x,y,z)  
  real, dimension(3) :: sig_e  ! sigma électrique (x,y,z)
  real, dimension(3) :: sig_h  ! sigma magnétique (x,y,z)
  real, dimension(3) :: f0_tang ! f0 du tangente de perte
end type st_milieu_simple
 
contains

! ==========================================================
! == Subroutine milieu_simple_initdefaut           ==
! == Initialisation par defaut des variables contenant    ==
! == les parametres definis par l'utilisateur             ==
! == dans le fichier d'entree                             ==
! ==========================================================	
subroutine milieu_simple_initdefaut(m)

type(st_milieu_simple), intent(inout) :: m

m%eps_r(:) = 1.0			
m%mu_r(:) = 1.0		
m%sig_e(:) = 0.0			
m%sig_h(:) = 0.0
m%f0_tang(:) = 0.0
end subroutine milieu_simple_initdefaut

! ==========================================================
! == Subroutine milieu_simple_copy                        ==
! ==========================================================
subroutine milieu_simple_copy(m_in,m_out)
type(st_milieu_simple), intent(in)  :: m_in
type(st_milieu_simple), intent(out) :: m_out

m_out=m_in
end subroutine milieu_simple_copy
! ==========================================================
! == Subroutine milieu_simple_deallocate                  ==
! ==========================================================
subroutine milieu_simple_deallocate(m)
type(st_milieu_simple), intent(inout) :: m
! rien à faire
end subroutine milieu_simple_deallocate

! ==========================================================
! == Subroutine milieu_simple_lecture              ==
! ==========================================================
subroutine milieu_simple_lecture(f,m)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_milieu_simple), intent(inout) :: m

!===  Déclaration des variables  ===
real  :: valeur
character(LEN=LEN_MOT_CLE) :: premier_mot, ch
integer :: status
!===   Début des traitements   ===
do
  premier_mot = lineread(f)

  if (premier_mot == f%nomfinbloc .or.premier_mot(1:6) == 'milieu'.or. & 
      premier_mot(1:14) == 'milieu_fractal') then
  
     call ligne_precedente(f)
     exit
  end if
  status = ctrl_nb_mot_chaine(f,f%chaine,2)
  
  select case(premier_mot)
    case('epsr')  ; read(f%chaine,*) ch, valeur; m%eps_r(1:3) = valeur
    case('epsrx') ; read(f%chaine,*) ch, valeur; m%eps_r(1) = valeur 
    case('epsry') ; read(f%chaine,*) ch, valeur; m%eps_r(2) = valeur
    case('epsrz') ; read(f%chaine,*) ch, valeur; m%eps_r(3) = valeur

    case('mur')  ; read(f%chaine,*) ch, valeur; m%mu_r(1:3) = valeur
    case('murx') ; read(f%chaine,*) ch, valeur; m%mu_r(1) = valeur
    case('mury') ; read(f%chaine,*) ch, valeur; m%mu_r(2) = valeur
    case('murz') ; read(f%chaine,*) ch, valeur; m%mu_r(3) = valeur

    case('sige')  ; read(f%chaine,*) ch, valeur; m%sig_e(1:3) = valeur
    case('sigex') ; read(f%chaine,*) ch, valeur; m%sig_e(1) = valeur
    case('sigey') ; read(f%chaine,*) ch, valeur; m%sig_e(2) = valeur
    case('sigez') ; read(f%chaine,*) ch, valeur; m%sig_e(3) = valeur

    case('sigh')  ; read(f%chaine,*) ch, valeur; m%sig_h(1:3) = valeur
    case('sighx') ; read(f%chaine,*) ch, valeur; m%sig_h(1) = valeur
    case('sighy') ; read(f%chaine,*) ch, valeur; m%sig_h(2) = valeur
    case('sighz') ; read(f%chaine,*) ch, valeur; m%sig_h(3) = valeur
    case default
      print '("Mot clé non reconnu : ",A)', premier_mot
      call lecture_stop(f)      
  end select
enddo

end subroutine milieu_simple_lecture

! ==========================================================
! == Subroutine milieu_simple_ecriture                    ==
! == Ecriture dans le fichier d'entrée...                 ==
! ==========================================================	
subroutine milieu_simple_ecriture(f,m)

type(st_fichier), intent(inout):: f
type(st_milieu_simple), intent(inout) :: m

if (any(m%eps_r /=1.0)) then
  if (milieu_simple_anisotrope(m%eps_r)) then
    write(f%unite,'(T5,A,3x,F9.3)') 'epsrx', m%eps_r(1)
    write(f%unite,'(T5,A,3x,F9.3)') 'epsry', m%eps_r(2)
    write(f%unite,'(T5,A,3x,F9.3)') 'epsrz', m%eps_r(3)
  else
    write(f%unite,'(T5,A,3x,F9.3)') 'epsr', m%eps_r(1)
  end if
end if

if (any(m%mu_r /=1.0)) then
  if (milieu_simple_anisotrope(m%mu_r)) then
    write(f%unite,'(T5,A,3x,F9.3)') 'murx', m%mu_r(1)
    write(f%unite,'(T5,A,3x,F9.3)') 'mury', m%mu_r(2)
    write(f%unite,'(T5,A,3x,F9.3)') 'murz', m%mu_r(3)
  else
    write(f%unite,'(T5,A,3x,F9.3)') 'mur', m%mu_r(1)
  end if
end if

if (any(m%sig_e /=0.0)) then
  if (milieu_simple_anisotrope(m%sig_e)) then
    write(f%unite,'(T5,A,3x,ES15.6)') 'sigex', m%sig_e(1)
    write(f%unite,'(T5,A,3x,ES15.6)') 'sigey', m%sig_e(2)
    write(f%unite,'(T5,A,3x,ES15.6)') 'sigez', m%sig_e(3)
  else
    write(f%unite,'(T5,A,3x,ES15.6)') 'sige', m%sig_e(1)
  end if
end if

if (any(m%sig_h /=0.0)) then
  if (milieu_simple_anisotrope(m%sig_h)) then
    write(f%unite,'(T5,A,3x,ES15.6)') 'sighx', m%sig_h(1)
    write(f%unite,'(T5,A,3x,ES15.6)') 'sighy', m%sig_h(2)
    write(f%unite,'(T5,A,3x,ES15.6)') 'sighz', m%sig_h(3)
  else
    write(f%unite,'(T5,A,3x,ES15.6)') 'sigh', m%sig_h(1)
  end if
end if
end subroutine milieu_simple_ecriture



! ==========================================================
! == Subroutine milieu_simple_init                 ==
! ==========================================================
subroutine milieu_simple_init(m)

type(st_milieu_simple), intent(inout) :: m

				
end subroutine milieu_simple_init

! ==========================================================
! == Subroutine milieu_simple_anisotrope           ==
! == Indique si le milieu est anisotrope ou pas           ==
! ==========================================================
function milieu_simple_anisotrope(param)

real, dimension(3), intent(in) :: param
logical          :: milieu_simple_anisotrope

milieu_simple_anisotrope = .true.
if (ALL(param == param(1))) milieu_simple_anisotrope = .false.
   				
end function milieu_simple_anisotrope

! ==========================================================
! == Subroutine milieu_simple_non_defaut           ==
! == Indique si un paramètre du milieu est différent      ==
! == de celui du vide (reference ou pas) vide             ==
! ==========================================================
function milieu_simple_non_defaut(param,reference)

real, dimension(3), intent(in) :: param
real, intent(in) :: reference
logical          :: milieu_simple_non_defaut

milieu_simple_non_defaut = .false.
if (ANY(param /=reference )) then
  milieu_simple_non_defaut = .true.
end if
   				
end function milieu_simple_non_defaut

	
! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module m_milieu_simple	



