!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_sortie.f90
!// Purpose:  Traite l'écriture des données de sorties et les noms de fichiers 
!             il contient un ensemble de fonctions destiné à faciliter 
!             l'écriture des données   
!      Simulateur FDTD
!// Author:      Christophe Guiffaut, Cyril Giraudon
!// Modified by:  Christophe Guiffaut
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Novembre 2001
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

module m_sortie

use m_constantes, only : LEN_MOT_CLE,rang,RANG_MAITRE_AFF
			      			      
use m_entree, only : st_fichier, &
		    lineread, &
		    unitefichier, &
		    entier_chaine, &
		    ligne_precedente, &
		    ctrl_nb_mot_chaine, &
		    lecture_stop

use m_math_operation, only : atan2d_cor
use m_es_identifiant

implicit none

type st_fichier_sortie
  character(LEN=LEN_MOT_CLE) :: nom
  logical :: fusion
  character(LEN=LEN_MOT_CLE) :: nom_temp
  character(LEN=LEN_MOT_CLE) :: nom_freq
  character(LEN=LEN_MOT_CLE) :: mantisse_reel
  character(LEN=LEN_MOT_CLE) :: mantisse_entier
  integer :: id_temp
  integer :: id_freq
  ! -- format_fichier = FORMAT_ASCII=1 ou FORMAT_NETCDF=2 ou FORMAT_HDF5=3 --
  integer :: format_fichier
  character(LEN=LEN_MOT_CLE) :: extension
end type st_fichier_sortie

contains

! ==========================================================
! == Subroutine fichier_sortie_initdefaut()            ==
! ==========================================================
subroutine fichier_sortie_initdefaut(f)

type(st_fichier_sortie), intent(inout) :: f

f%format_fichier = FORMAT_ASCII
f%fusion = .false.
f%nom = ''
f%nom_temp = ''
f%nom_freq = ''
f%extension = EXTENSION
f%id_temp = 0
f%id_freq = 0
f%mantisse_reel = MANTISSE_REEL
f%mantisse_entier = MANTISSE_ENTIER

end subroutine fichier_sortie_initdefaut


! ==========================================================
! == Subroutine fichier_sortie_lecture()               ==
! ==                                                      ==
! == Lecture des donnees correspondant aux sorties        ==
! ==========================================================
subroutine fichier_sortie_lecture(fe,fs)

type(st_fichier_sortie), intent(inout) :: fs
type(st_fichier), intent(inout) :: fe

character(LEN=LEN_MOT_CLE) :: poubelle, mot_format
integer  :: status

do
  select case(lineread(fe))
    case(BFS_FORMAT)
      status = ctrl_nb_mot_chaine(fe,fe%chaine,2)
      read(fe%chaine,*) poubelle, mot_format
      if (mot_format=='ascii')  fs%format_fichier = FORMAT_ASCII

      if (mot_format=='hdf5') then
	  print '(/,t10,"ATTENTION, format hdf5 non disponible ...")'
	  call lecture_stop(fe)
      endif
      
!    case('nom')
!      status = ctrl_nb_mot_chaine(fe,fe%chaine,2)
!      read(fe%chaine,*) poubelle, fs%nom

    case(BFS_EXTENSION)
      status = ctrl_nb_mot_chaine(fe,fe%chaine,2)
      read(fe%chaine,*) poubelle, fs%extension
      fs%extension = '.' // TRIM(fs%extension)
      
    case(BFS_MANTISSE_REEL)
      status = ctrl_nb_mot_chaine(fe,fe%chaine,2)
      read(fe%chaine,*) poubelle, fs%mantisse_reel

    case(BFS_MANTISSE_ENTIER)
      status = ctrl_nb_mot_chaine(fe,fe%chaine,2)
      read(fe%chaine,*) poubelle, fs%mantisse_entier
      
!    case('fusion')
!      fs%fusion = .true.
    
    case default
      if (fe%chaine==fe%nomfinbloc) then
	exit
      else
        print '(/,t10,"ATTENTION, Mot clef inconnu !!!")'
	call lecture_stop(fe)
      endif
  end select
enddo
  
end subroutine fichier_sortie_lecture

! ==========================================================
! == Subroutine fichier_sortie_ecriture                   ==
!===========================================================	
subroutine fichier_sortie_ecriture(f, fs)

! === Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_fichier_sortie), intent(inout)       :: fs

integer :: l
! ===   Début des traitements   ===

l =len_trim(fs%extension)
write(f%unite,'(T3,A,3x,A)')  BFS_EXTENSION,fs%extension(2:l)
if (fs%mantisse_reel/=FS_MANTISSE_REEL)  &
  write(f%unite,'(T3,A,3x,A)')  BFS_MANTISSE_REEL,trim(fs%mantisse_reel)
if (fs%mantisse_entier/=FS_MANTISSE_ENTIER)  &
  write(f%unite,'(T3,A,3x,A)')  BFS_MANTISSE_ENTIER,trim(fs%mantisse_entier)
end subroutine fichier_sortie_ecriture

! ==========================================================
! == Subroutine fichier_sortie_lect_format_freq           ==
! Détecte si dans le mot clé donné, un commutateur de 
! format en fréquence s'y trouve
! ==========================================================
logical function fichier_sortie_lect_format_freq(mot_cle,com_format_tf)
!=== Déclaration des arguments ===
character(LEN=LEN_MOT_CLE), intent(in) :: mot_cle

integer, intent(inout) :: com_format_tf
!===  Déclaration des variables  ===

!===   Début des traitements   ===
fichier_sortie_lect_format_freq = .false.

if (mot_cle == 'reelle_imaginaire' .or. mot_cle == 'reel_imaginaire') then
  com_format_tf = REELLE_IMAGINAIRE
  fichier_sortie_lect_format_freq = .true.
else if (mot_cle == 'module_phase') then
  com_format_tf = MODULE_PHASE    
  fichier_sortie_lect_format_freq = .true.
else if (mot_cle == 'module_db_phase') then
  com_format_tf = MODULE_DB_PHASE      
  fichier_sortie_lect_format_freq = .true.
else if (mot_cle == 'module_seul') then
  com_format_tf = MODULE_SEUL      
  fichier_sortie_lect_format_freq = .true.
else if (mot_cle == 'module_db') then
  com_format_tf = MODULE_DB      
  fichier_sortie_lect_format_freq = .true.
end if

end function fichier_sortie_lect_format_freq

! ==========================================================
! == Function ouvre_en_ecriture                           ==
! == Ouvre un fichier en écriture                         ==
! ==========================================================
integer function ouvre_en_ecriture(nomfichier,status_chaine,pos_chaine)

character(LEN=*), intent(in) :: nomfichier
character(LEN=*),optional, intent(in)  :: status_chaine
character(LEN=*),optional, intent(in)  :: pos_chaine

integer  :: i,l,unite
character(LEN=LEN_MOT_CLE)  :: ch_err, status_ch,pos
integer  :: i_err

! -- fournit une unité libre pour le fichier --
unite = unitefichier();

ouvre_en_ecriture = unite

if (unite == -1) then
    print '(/,t10,"Erreur dans l''attribution d''un n° d''unite pour &
      &l''ouverture du fichier de nom : ", &
      &A)', TRIM(nomfichier)
    print '(t10,"Arret dans la fonction : ouvre_en_ecriture  ...")'
    stop
end if 
if (PRESENT(status_chaine)) then; status_ch = status_chaine
else;  status_ch = 'new';end if

if (PRESENT(pos_chaine)) then; pos = pos_chaine
else;  pos = 'asis';end if

open(unite, FILE=nomfichier, STATUS = status_ch,ACTION='write', &
       position=pos, FORM = 'formatted', iostat = i_err, err=998)    
					    
return

! -- gestion erreur d'ouverture --

998 ch_err = entier_chaine(i_err)
    print '(/,t10,"Erreur n°",a ," à l ouverture du fichier de nom : ", &
      &A )',TRIM(ch_err),TRIM(nomfichier)
    i=INDEX(nomfichier,'.',.true.)
    l=len_trim(nomfichier)
    print '(/,t10," Le fichier existe déjà !!!, pour effacer, taper (sous linux) rm -f *",A)',&
                         nomfichier(i:l)
    print '(t10,"Arret dans la function : ouvre_en_ecriture  ...")'
    stop
  
end function ouvre_en_ecriture

! ==========================================================
! == Subroutine fichier_sortie_ecrit_format           ==
! ==========================================================
subroutine fichier_sortie_ecrit_format(f,com_format_tf)
!=== Déclaration des arguments ===
type(st_fichier), intent(in)       :: f
integer, intent(in) :: com_format_tf

!===  Déclaration des variables  ===

!===   Début des traitements   ===
select case(com_format_tf)
  case(REELLE_IMAGINAIRE)
    write (f%unite,'(T3,A)') 'reel_imaginaire'
  case(MODULE_PHASE)
    write (f%unite,'(T3,A)') 'module_phase'
  case(MODULE_DB_PHASE)
    write (f%unite,'(T3,A)') 'module_db_phase'
  case(MODULE_SEUL)
    write (f%unite,'(T3,A)') 'module_seul'
  case(MODULE_DB)
    write (f%unite,'(T3,A)') 'module_db'
end select
end subroutine fichier_sortie_ecrit_format

! ==========================================================
! == Subroutine fichier_sortie_nom_temps()                ==
! ==========================================================
function fichier_sortie_nom_temps(fs,fich) result(nomfichier)

type(st_fichier_sortie), intent(in) :: fs
character(LEN=*),intent(in) :: fich

character(LEN=LEN_MOT_CLE) :: nomfichier

nomfichier = TRIM(fich)//TRIM(SUFFIXE_TEMP)//TRIM(fs%extension)

end function fichier_sortie_nom_temps
! ==========================================================
! == Subroutine fichier_sortie_nom_freq()                ==
! ==========================================================
function fichier_sortie_nom_freq(fs,fich) result(nomfichier)

type(st_fichier_sortie), intent(in) :: fs
character(LEN=*),intent(in) :: fich

character(LEN=LEN_MOT_CLE) :: nomfichier

nomfichier = TRIM(fich)//TRIM(SUFFIXE_FREQ)//TRIM(fs%extension)

end function fichier_sortie_nom_freq


! ==========================================================
! == Ouverture du/des fichier(s) de sortie                ==
! ==========================================================
subroutine fichier_sortie_init(fs)

implicit none

type(st_fichier_sortie), intent(inout) :: fs

! -- Determination des noms des fichiers de sortie --


end subroutine fichier_sortie_init 


! ==========================================================
! == Subroutine fichier_sortie_precalcul(fs)           ==
! == fs : st_fichier_sortie                            ==
! ==                                                      ==
! == Fermeture du mode definition du/des fichier(s)       ==
! == de sortie.                                           ==
! ==========================================================
subroutine fichier_sortie_precalcul(fs)

type(st_fichier_sortie), intent(inout) :: fs


end subroutine fichier_sortie_precalcul


! ==========================================================
! == Fermeture du/des fichier(s) de sortie                ==
! ==========================================================
subroutine fichier_sortie_fermeture(fs)

type(st_fichier_sortie), intent(inout) :: fs


end subroutine fichier_sortie_fermeture


!===========================================================	
! Subroutine impression_ascii_reel
!  Stockage ascci de données reelles et fonction
! d'une variable écrite dans la première colonne
! La phase n'existe pas.
!===========================================================	
subroutine impression_ascii_reel(unite,&
               mantisse, com_format_tf, tab_x, tab_y,nbx,nby)


!=== Déclaration des arguments ===
integer, intent(in)  :: unite ! n°id du fichier
integer, intent(in)  :: com_format_tf !commutateur de format
character(LEN=LEN_MOT_CLE), intent(in) ::  mantisse ! format d'écriture de chaque valeur
integer, intent(in)  :: nbx, nby !nombre de valeurs complexes en abscisse et en ordonnée
real, dimension(1:nbx), intent(in)     :: tab_x ! valeur abscisse
real, dimension(1:nbx,1:nby), intent(in)  :: tab_y ! valeur ordonnée, plusieurs colonnes

!===  Déclaration des variables  ===
integer  :: i,j
real, dimension(1:nby) :: phase

!=== Debut des traitements ===

if ((com_format_tf == REELLE_IMAGINAIRE).or.(com_format_tf == MODULE_PHASE) .or. &
         (com_format_tf == MODULE_SEUL)) then
  do i=1,nbx
    write(unite,mantisse) tab_x(i), tab_y(i,1:nby)
  end do

else if ((com_format_tf == MODULE_DB_PHASE).or.(com_format_tf == MODULE_DB)) then
  do i=1,nbx
    write(unite,mantisse) tab_x(i), 20*LOG10(tab_y(i,1:nby))
  end do
else if (com_format_tf == MODULE_DBW) then
  do i=1,nbx
    write(unite,mantisse) tab_x(i), 10*LOG10(tab_y(i,1:nby))
  end do

end if

end subroutine impression_ascii_reel

!===========================================================	
! Subroutine impression_ascii_reel_2x
!  Stockage ascci de données reelles et fonction
! de deux variables écrite dans les deux premières colonnes
!===========================================================	
subroutine impression_ascii_reel_2x(unite, mantisse, &
                com_format_tf, tab_x1, tab_x2, tab_y,nbx,nby)

!=== Déclaration des arguments ===
integer, intent(in)  :: unite ! n°id du fichier
integer, intent(in)  :: com_format_tf !commutateur de format
character(LEN=LEN_MOT_CLE), intent(in) ::  mantisse ! format d'écriture de chaque valeur
integer, intent(in)  :: nbx, nby !nombre de valeurs complexes en abscisse et en ordonnée
real, dimension(1:nbx), intent(in)     :: tab_x1, tab_x2 ! valeur abscisse
real, dimension(1:nbx,1:nby), intent(in)  :: tab_y ! valeur ordonnée, plusieurs colonnes

!===  Déclaration des variables  ===
integer  :: i,j
real, dimension(1:nby) :: phase

!=== Debut des traitements ===

if ((com_format_tf == REELLE_IMAGINAIRE).or.(com_format_tf == MODULE_PHASE).or. &
         (com_format_tf == MODULE_SEUL)) then
  do i=1,nbx
    write(unite,mantisse) tab_x1(i), tab_x2(i), tab_y(i,1:nby)
  end do

else if ((com_format_tf == MODULE_DB_PHASE).or.(com_format_tf == MODULE_DB)) then
  do i=1,nbx
    write(unite,mantisse) tab_x1(i), tab_x2(i), 20*LOG10(tab_y(i,1:nby))
  end do

else if (com_format_tf == MODULE_DBW) then
  do i=1,nbx
    write(unite,mantisse) tab_x1(i), tab_x2(i), 10*LOG10(tab_y(i,1:nby))
  end do

end if

end subroutine impression_ascii_reel_2x

!===========================================================	
! Subroutine impression_ascii_complexe
!  Stockage ascci de données complexes et fonction
! d'une variable écrite dans la première colonne
!===========================================================	
subroutine impression_ascii_complexe(unite,&
               mantisse, com_format_tf, tab_x, tab_y,nbx,nby)

!=== Déclaration des arguments ===
integer, intent(in)  :: unite ! n°id du fichier
integer, intent(in)  :: com_format_tf !commutateur de format
character(LEN=LEN_MOT_CLE), intent(in) ::  mantisse ! format d'écriture de chaque valeur
integer, intent(in)  :: nbx, nby !nombre de valeurs complexes en abscisse et en ordonnée
real, dimension(1:nbx), intent(in)  :: tab_x ! valeur abscisse
complex, dimension(1:nbx,1:nby), intent(in)  :: tab_y ! valeur ordonnée, plusieurs colonnes

!===  Déclaration des variables  ===
integer  :: i,j
real, dimension(1:nby) :: phase

!=== Debut des traitements ===

if (com_format_tf == REELLE_IMAGINAIRE) then
  do i=1,nbx
    write(unite,mantisse) tab_x(i), (REAL(tab_y(i,j)), AIMAG(tab_y(i,j)),j=1,nby)
  end do

else if (com_format_tf == MODULE_SEUL) then
  do i=1,nbx
    write(unite,mantisse) tab_x(i), CABS(tab_y(i,1:nby))
  end do

else if (com_format_tf == MODULE_PHASE) then
  do i=1,nbx
    do j=1,nby
      phase(j) = atan2d_cor(tab_y(i,j))
    end do
    write(unite,mantisse) tab_x(i), (CABS(tab_y(i,j)), phase(j),j=1,nby)
  end do
else if (com_format_tf == MODULE_DB) then
  do i=1,nbx
    write(unite,mantisse) tab_x(i), 20*LOG10(CABS(tab_y(i,1:nby)))
  end do

else if (com_format_tf == MODULE_DB_PHASE) then
  do i=1,nbx
    do j=1,nby
      phase(j) = atan2d_cor(tab_y(i,j))
    end do
    write(unite,mantisse) tab_x(i), (20*LOG10(CABS(tab_y(i,j))), phase(j), j=1,nby)
  end do

end if

end subroutine impression_ascii_complexe

!===========================================================	
! Subroutine impression_ascii_complexe_2x
!  Stockage ascci de données complexes et fonction
! d'une variable écrite dans la première colonne
!===========================================================	
subroutine impression_ascii_complexe_2x(unite,&
               mantisse, com_format_tf, tab_x1, tab_x2, tab_y,nbx,nby)

!=== Déclaration des arguments ===
integer, intent(in)  :: unite ! n°id du fichier
integer, intent(in)  :: com_format_tf !commutateur de format
character(LEN=LEN_MOT_CLE), intent(in) ::  mantisse ! format d'écriture de chaque valeur
integer, intent(in)  :: nbx, nby !nombre de valeurs complexes en abscisse et en ordonnée
real, dimension(1:nbx), intent(in)  :: tab_x1, tab_x2 ! valeur abscisse
complex, dimension(1:nbx,1:nby), intent(in)  :: tab_y ! valeur ordonnée, plusieurs colonnes

!===  Déclaration des variables  ===
integer  :: i,j
real, dimension(1:nby) :: phase

!=== Debut des traitements ===

if (com_format_tf == REELLE_IMAGINAIRE) then
  do i=1,nbx
    write(unite,mantisse) tab_x1(i), tab_x2(i), (REAL(tab_y(i,j)), AIMAG(tab_y(i,j)),j=1,nby)
  end do

else if (com_format_tf == MODULE_SEUL) then
  do i=1,nbx
    write(unite,mantisse) tab_x1(i), tab_x2(i), CABS(tab_y(i,1:nby))
  end do

else if (com_format_tf == MODULE_PHASE) then
  do i=1,nbx
    do j=1,nby
      phase(j) = atan2d_cor(tab_y(i,j))
    end do
    write(unite,mantisse) tab_x1(i), tab_x2(i), (CABS(tab_y(i,j)), phase(j),j=1,nby)
  end do

else if (com_format_tf == MODULE_DB) then
  do i=1,nbx
    write(unite,mantisse) tab_x1(i), tab_x2(i), 20*LOG10(CABS(tab_y(i,1:nby)))
  end do

else if (com_format_tf == MODULE_DB_PHASE) then
  do i=1,nbx
    do j=1,nby
      phase(j) = atan2d_cor(tab_y(i,j))
    end do
    write(unite,mantisse) tab_x1(i), tab_x2(i), (20*LOG10(CABS(tab_y(i,j))), phase(j), j=1,nby)
  end do

end if

end subroutine impression_ascii_complexe_2x

!===========================================================	
! Subroutine resultat_champ_surf_impr_ascii
!  Stockage ascci des champs en fréquence
!  nb1 : nb colonne
!  nb2 : nb lignes
!  nbc : nb composantes
!  nbf : nb fréquences
!===========================================================	
subroutine impression_ascii_surface(unite,&
               mantisse, com_format_tf, tab_f, tab_y,nb1,nb2,nbf,nbc)

!=== Déclaration des arguments ===
integer, intent(in)  :: unite, com_format_tf
character(LEN=LEN_MOT_CLE), intent(in) ::  mantisse
real, dimension(1:), intent(in)  :: tab_f 
complex, dimension(1:,1:,1:,1:), intent(in)  :: tab_y 
integer, intent(in)  :: nb1,nb2,nbf, nbc

!===  Déclaration des variables  ===
integer  :: i,j,k,l
real, dimension(1:nb1) :: phase

!=== Debut des traitements ===
if (com_format_tf == REELLE_IMAGINAIRE) then
  do k=1,nbf 
  do j=1,nbc 
  do i=1,nb2 
    write(unite,mantisse)  REAL(tab_y(:,i,j,k)), AIMAG(tab_y(:,i,j,k))
  end do; end do; end do

else if (com_format_tf == MODULE_SEUL) then
  do k=1,nbf 
  do j=1,nbc 
  do i=1,nb2 
    do l=1,nb1
      phase(l) = atan2d_cor(tab_y(l,i,j,k))
    end do
    write(unite,mantisse) CABS(tab_y(:,i,j,k))
  end do; end do; end do


else if (com_format_tf == MODULE_PHASE) then
  do k=1,nbf 
  do j=1,nbc 
  do i=1,nb2 
    do l=1,nb1
      phase(l) = atan2d_cor(tab_y(l,i,j,k))
    end do
    write(unite,mantisse) CABS(tab_y(:,i,j,k)), phase(1:nb1)
  end do; end do; end do
else if (com_format_tf == MODULE_DB) then
  do k=1,nbf 
  do j=1,nbc 
  do i=1,nb2 
    do l=1,nb1
      phase(l) = atan2d_cor(tab_y(l,i,j,k))
    end do
    write(unite,mantisse)  20*LOG10(CABS(tab_y(:,i,j,k)))
end do; end do; end do

else if (com_format_tf == MODULE_DB_PHASE) then
  do k=1,nbf 
  do j=1,nbc 
  do i=1,nb2 
    do l=1,nb1
      phase(l) = atan2d_cor(tab_y(l,i,j,k))
    end do
    write(unite,mantisse)  20*LOG10(CABS(tab_y(:,i,j,k))), phase(1:nb1)
end do; end do; end do

end if

end subroutine impression_ascii_surface

! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module m_sortie
