!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_sortie_normalise.f90
!// Purpose:      Traite la normalisation des réponse en fréquence
!// Author:      Christophe Guiffaut
!// Created  :     Octobre 2005
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
MODULE m_sortie_normalise

use m_constantes, only : LEN_MOT_CLE
use m_constantes_scientifiques, only : PI, CVIDE

use m_entree, only : st_fichier, &
		nb_mot_chaine, &
		ctrl_nb_mot_chaine
				 
use m_generateur, only : st_generateur, &
			generateur_calcul_t

use m_temps_frequence, only : st_temps_frequence
implicit none

! ==========================================================
! == Definition du type principal                         ==
! ==========================================================
type st_sortie_normalise
  logical  :: flag_normalisation
  character(LEN=LEN_MOT_CLE) :: nomgene
  real  :: A0,phase, retard ! phase en degré, retard en seconde
end type st_sortie_normalise


! ==========================================================
contains

! ============================================================
! == Subroutine sortie_normalise_initdefaut                 ==
! == Initialisation des données par défaut                  == 
! ============================================================
subroutine sortie_normalise_initdefaut(sn)
type(st_sortie_normalise), intent(inout) :: sn

sn%flag_normalisation = .false.
sn%nomgene = '';
sn%A0 = 1.0; sn%phase = 0.0; sn%retard = 0.0
  
end subroutine sortie_normalise_initdefaut


! ============================================================
! == Subroutine sortie_normalise_lecture                    ==
! == Lecture des données relatif à la normalisation si existe 
! ============================================================
logical function sortie_normalise_lecture(f,premiermot,sn)
type(st_fichier), intent(inout) :: f
character (len=LEN_MOT_CLE), intent(in)  :: premiermot
type(st_sortie_normalise), intent(inout) :: sn

character (len=LEN_MOT_CLE)  :: corbeille
integer  :: nbmot
sortie_normalise_lecture = .false.
if (premiermot(1:18) == 'normalisation_freq') then
  ! -- Calcul du nombre de composantes de champ dans la chaine --
  nbmot = ctrl_nb_mot_chaine(f,f%chaine,2,5)      
  sn%nomgene = '';
  sn%A0 = 1.0; sn%phase = 0.0; sn%retard = 0.0
  select case (nbmot)
    case(2)
      read(f%chaine,*) corbeille,sn%nomgene
    case(3)
      read(f%chaine,*) corbeille,sn%nomgene,sn%A0
    case(4)
      read(f%chaine,*) corbeille,sn%nomgene,sn%A0,sn%phase
    case(5)
      read(f%chaine,*) corbeille,sn%nomgene,sn%A0,sn%phase,sn%retard
  end select	
  sn%flag_normalisation = .true.
  sortie_normalise_lecture = .true.
!----------------------------------------------------------------------------
else if (premiermot(1:22) == 'non_normalisation_freq') then
  ! -- Calcul du nombre de composantes de champ dans la chaine --
  nbmot = ctrl_nb_mot_chaine(f,f%chaine,1,1)
  sn%nomgene = '';
  sn%A0 = 1.0; sn%phase = 0.0; sn%retard = 0.0
  sn%flag_normalisation = .false.
  sortie_normalise_lecture = .true.
end if

end function sortie_normalise_lecture


! ============================================================
! == Subroutine sortie_normalise_compare                   ==
! == Comparaison de deux structures                        == 
! ============================================================
logical function sortie_normalise_compare(sn1,sn2)
type(st_sortie_normalise), intent(in) :: sn1,sn2

if (.not.(sn1%flag_normalisation .eqv. sn2%flag_normalisation).or. &
     (sn1%nomgene/=sn2%nomgene).or.(sn1%A0/=sn2%A0).or. &
        (sn1%phase/=sn2%phase).or.(sn1%retard/=sn2%retard)) then
  sortie_normalise_compare = .false.
else
  sortie_normalise_compare = .true.
end if
end function sortie_normalise_compare

! ============================================================
! == Subroutine sortie_normalise_ecriture                   ==
! == Lecture des données relatif à la normalisation si existe 
! ============================================================
subroutine sortie_normalise_ecriture(f,sn)
type(st_fichier), intent(inout) :: f
type(st_sortie_normalise), intent(in) :: sn

if (sn%flag_normalisation) then
!  print '("sn%A0,sn%phase,sn%retard",3G14.4)',sn%A0,sn%phase,sn%retard
  if (sn%retard /= 0.0) then 
    write(f%unite,'(T3,A,3x,A,3x,3G14.6)') 'normalisation_freq',trim(sn%nomgene),&
                                         sn%A0,sn%phase,sn%retard
  else if (sn%phase /= 0.0) then
    write(f%unite,'(T3,A,3x,A,3x,2G14.6)') 'normalisation_freq',trim(sn%nomgene),&
                                         sn%A0,sn%phase
  else
    write(f%unite,'(T3,A,3x,A,3x,G14.6)') 'normalisation_freq',trim(sn%nomgene),sn%A0
  end if  
else
  write(f%unite,'(T3,A)') 'non_normalisation_freq'
end if

end subroutine sortie_normalise_ecriture


! ============================================================
! == Subroutine Normalise_Reponse_Freq                      ==
! ============================================================
subroutine Normalise_Reponse_Freq(t_gene,tf,sn,tabf)
type(st_generateur), intent(inout)    :: t_gene
type(st_temps_frequence), intent(in)  :: tf
type(st_sortie_normalise), intent(in) :: sn
complex, dimension(1:tf%n_liste), intent(out)   :: tabf

!=== DÉCLARATION DES VARIABLES LOCALES ===
integer                      :: i,j
real, dimension(1:tf%nech_t) :: tabt
real :: dervaleur

!=== DÉBUT DES TRAITEMENTS ===

!--- Fonction référence en temps ---
tabt = 0.0
tabf = cmplx(0.0,0.0)
dervaleur = generateur_calcul_t(t_gene, sn%nomgene,sn%A0,sn%phase,sn%retard,&
                                        tf%dt_tfd,tf%nech_t,tabt)

!--- Fonction référence en fréquence ---
do i = 1,tf%nech_t ! boucle sur les échantillons en temps
    tabf(:) = tabf(:) +  tabt(i)* (tf%expi2pidtf(:))**(i-1)
end do
tabf(:) = tabf(:) * tf%dt_tfd

end subroutine Normalise_Reponse_Freq

end MODULE m_sortie_normalise
