!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_surface_huygens.f90
!// Purpose:      Gère les surfaces de Huygens, utile notamment pour le champ lointain
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Septembre 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Définition et propriétés des surfaces de Huygens    ==
! ==========================================================

module m_surface_huygens

use m_constantes

use m_constantes_scientifiques, only : PI

use m_entree, only : st_fichier, &
                                 lineread, &
				 ligne_precedente, &
				 lecture_stop, &
				 len_int, &
				 nb_mot_chaine, &
				 unitefichier, &
				 multibackspace, &
				 numbackspace_set, &
				 ctrl_nb_mot_chaine				 
use m_sortie, only : ouvre_en_ecriture
use m_maillage
use m_commun_identifiant, only : ID_COORD_REEL,ID_COORD_CELL,BSH_CENTRE_PHASE,BSH_ORIGINE,&
				BSH_SURF

implicit none
integer, parameter   :: PHI = 2
 
! ==========================================================
! == Definition du type st_surface_huygens                ==
! ==========================================================
type st_une_surface_huygens
  ! -- nom : nom de la surface --
  character(LEN=LEN_MOT_CLE) :: nom
  logical                :: flag_image,flag_coordreel
  integer                :: type_plan_image, position_plan_image
  integer                :: nbsurface
  logical, dimension(6)  :: flag_surface
  integer, dimension(6)  :: coord,coord_ssd
  !real, dimension(3)     :: origine
  real, dimension(6)     :: coord_reel
  real, dimension(3)     :: origine_reel
end type st_une_surface_huygens

! -- Liste des surfaces de Huygens --
type st_surface_huygens
  ! -- nza : nombre de surfaces de Huygens --
  integer :: nsh
  ! -- tf : tableau de surfaces de Huygens --
  type(st_une_surface_huygens), dimension(:), pointer :: l_sh
end type  

contains

! ==========================================================
! == Subroutine surface_huygens_initdefaut                 ==
! ==========================================================
subroutine surface_huygens_initdefaut(tsh)

implicit none

type(st_surface_huygens), intent(inout) :: tsh

nullify(tsh%l_sh)
tsh%nsh = 0
end subroutine surface_huygens_initdefaut

! ==========================================================
! == Subroutine une_surface_huygens_initdefaut            ==
! ==========================================================
subroutine une_surface_huygens_initdefaut(sh)

implicit none

type(st_une_surface_huygens), intent(inout) :: sh

sh%nom = ""
sh%flag_image = .false.
sh%type_plan_image = 0
sh%position_plan_image = 0
sh%nbsurface = 0
sh%flag_surface(:) = .false.
sh%origine_reel(:) = 0
sh%flag_coordreel = .false.
sh%coord=0
sh%coord_ssd=0
sh%coord_reel=0.0

end subroutine une_surface_huygens_initdefaut

! ==========================================================
! == Subroutine une_surface_huygens_initdefaut            ==
! ==========================================================
subroutine une_surface_huygens_copy(shin,shout)
type(st_une_surface_huygens), intent(in) :: shin
type(st_une_surface_huygens), intent(out) :: shout

! pas de pointeur ni de tableaux dynamique
! copy de structures à structures
shout = shin

end subroutine une_surface_huygens_copy

! ==========================================================
! == Subroutine une_surf_huygens_deallocate               ==
! ==========================================================
subroutine une_surf_huygens_deallocate(sh)
type(st_une_surface_huygens), intent(inout) :: sh
! rien à faire

end subroutine une_surf_huygens_deallocate


! ==========================================================
! == Subroutine surface_huygens_deallocate                 ==
! ==========================================================
subroutine surface_huygens_deallocate(tsh)

implicit none

!=== Déclaration des arguments ===
type(st_surface_huygens), intent(inout) :: tsh
!=== Déclaration des variables ===
integer  :: i
type(st_une_surface_huygens), pointer  :: sh
!=== Déclaration des traitements ===
if (associated(tsh%l_sh)) then
  do i = 1, tsh%nsh
    sh => tsh%l_sh(i)
  end do
  deallocate(tsh%l_sh)
end if
end subroutine surface_huygens_deallocate


! ==========================================================
! == Subroutine surface_huygens_lecture                    ==
! == f : fichier d'entree                                 ==
! Lecture des données dans le fichier d'entrée
! ==========================================================
subroutine surface_huygens_lecture(f,tsh,tm)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_surface_huygens), intent(inout) :: tsh
type(st_maillage), intent(in) :: tm

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: corbeille,nom, premiermot
character(LEN=LEN_MOT_CLE), dimension(6)  :: listesurface
integer            :: i, i_sh, status
real, dimension(3) :: origine_reel
real, dimension(6)    :: coord_reel 
logical :: flag_reel
type(st_une_surface_huygens), pointer  :: sh

!===   Début des traitements   ===

! -- Positionnement de numbackspace a 0 --
! -- numbackspace : nombre de sauts de ligne --
! -- vers le haut a effectuer lors d'un multibackspace() --
call numbackspace_set(f,0)

! -- ON COMPTE LE NOMBRE DE SURFACES DE HUYGENS ... --
i_sh = 0
do
  premiermot = lineread(f,1)
  if (premiermot(1:4) == BSH_SURF)  then
      
      i_sh = i_sh +1
      
  else if (premiermot==f%nomfinbloc) then
    exit
  endif
enddo

! -- Allocation memoire pour toutes les srfaces --
tsh%nsh = i_sh
if (i_sh > 0) then
  allocate(tsh%l_sh(i_sh))
else
  print *,'Avertissement: pas de surfaces de Huygens définies dans le bloc correspondant !'
  return
end if
! -- Retour vers la premiere ligne du bloc --
i = multibackspace(f)

i_sh = 0
flag_reel = .false.

do
  ! -- On recommence la lecture pour les données --
  premiermot = lineread(f)
  select case(premiermot(1:4))
    case(BSH_SURF)
      i_sh = i_sh + 1
      sh =>tsh%l_sh(i_sh)
      call une_surface_huygens_initdefaut(sh)
      
      sh%nbsurface = nb_mot_chaine(f%chaine) -7
      if (sh%nbsurface == 0) then
        read(f%chaine,*,err=999) sh%nom,coord_reel(1:6)
	sh%flag_surface(:) = .true.
      else
        read(f%chaine,*,err=999) sh%nom, listesurface(1:sh%nbsurface),coord_reel(1:6)
      end if
      
      do i=1, sh%nbsurface
        select case (listesurface(i))
	  case('xoyinf','zinf')
	    sh%flag_surface(1) = .true.
	  case('xoysup','zsup')
	    sh%flag_surface(2) = .true.
	  case('yozinf','xinf')
	    sh%flag_surface(3) = .true.
	  case('yozsup','xsup')
	    sh%flag_surface(4) = .true.
	  case('xozinf','zoxinf','yinf')
	    sh%flag_surface(5) = .true.
	  case('xozsup','zoxsup','ysup')
	    sh%flag_surface(6) = .true.
	  case default
            print *,'Mot clé non reconnu dans le bloc ',f%nombloc
            call lecture_stop(f)
	end select
      end do
      sh%nbsurface = 0
      do i=1,6
        if (sh%flag_surface(i)) sh%nbsurface = sh%nbsurface+1
      end do
      sh%flag_coordreel = flag_reel
      if (flag_reel) then
	sh%coord = maillage_Unif_ConvToCell6(tm,coord_reel)
	sh%coord_reel =  origine_reelle_abs6(tm,coord_reel) 
	if (rang==RANG_MAITRE_AFF) &
	  print '(T3,A,", Coordonnées en Nbre de cellule = ",6I5)',trim(sh%nom),sh%coord
      else
        sh%coord = nint(coord_reel)
	sh%coord_reel = maillage_Unif_ConvToReal6(tm,sh%coord)
	sh%coord = origine_cell_abs6(tm,sh%coord)
	if (rang==RANG_MAITRE_AFF) &
	  print '(T3,A,", Coordonnées réelles = ",6ES15.6)',trim(sh%nom),sh%coord_reel
      end if
      sh%coord_ssd = sh%coord
      ! Calcul de l'origine par défaut à partir des coordonnées des surfaces
      ! de Huygens. L'origine est pris au centre des surfaces de Huygens
      do i=1, 3
        sh%origine_reel(i) = (sh%coord_reel(i*2) - sh%coord_reel(i*2-1)) / 2 +  sh%coord_reel(i*2-1)
      end do
      cycle
  end select   
  
  select case(premiermot)
    case (ID_COORD_REEL)
      flag_reel = .true.
    case (ID_COORD_CELL)
      flag_reel = .false.
    case (BSH_CENTRE_PHASE,BSH_ORIGINE)
      status = ctrl_nb_mot_chaine(f,f%chaine,5)      
      read(f%chaine,*,err=999) corbeille,nom,origine_reel(1:3)
      if (.not.flag_reel) then
        origine_reel = maillage_Unif_ConvToReal3(tm,int(origine_reel))
      else
	origine_reel = origine_reelle_abs3(tm,origine_reel) ! repère de calcul
      end if
      do i = 1, i_sh
        if (tsh%l_sh(i)%nom == nom) then
         tsh%l_sh(i)%origine_reel = origine_reel
	 exit
        end if
      end do
      if (i>i_sh) then
	print *,'Erreur d''association de l''origine avec une surface de Huygens !!!'
	call lecture_stop(f)
      end if
    case default
      if (premiermot==f%nomfinbloc) then
	exit
      endif
      ! -- Si ca se passe pas bien : arret de la lecture --
      print *,'Erreur, Mot clé non reconnu :'
      call lecture_stop(f)

  end select
enddo

return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(f)

end subroutine surface_huygens_lecture

! ==========================================================
! == Subroutine surface_huygens_ecriture                  ==
! ==========================================================
subroutine surface_huygens_ecriture(f,tsh,tm)

type(st_fichier), intent(inout) :: f
type(st_surface_huygens), intent(inout) :: tsh
type(st_maillage), intent(in) :: tm

integer :: i,j
real    :: coord(6), origine(3)
character(LEN=10),dimension(1:6) :: ch1
character(LEN=LEN_MOT_CLE) :: ch
type(st_une_surface_huygens), pointer  :: sh

ch1(XOY_INF) = 'zinf' ; ch1(XOY_SUP) = 'zsup'
ch1(YOZ_INF) = 'xinf' ; ch1(YOZ_SUP) = 'xsup'
ch1(XOZ_INF) = 'yinf' ; ch1(XOZ_SUP) = 'ysup'
write(f%unite,'(T3,A)') trim(ID_COORD_REEL)
do i=1,tsh%nsh
  sh =>tsh%l_sh(i)
  ch = ''
  do j=1,6
    if (sh%flag_surface(j)) ch = trim(ch)//' '//trim(ch1(j))
  end do
!  write(f%unite,'(T3,A,1x,A,1x,6I5)')  trim(sh%nom),trim(ch),sh%coord(1:6)
  coord = origine_reelle_abs6(tm,sh%coord_reel,.true.) ! relatif à l'origine utilisateur
  write(f%unite,'(T3,A,1x,A,1x,6ES15.6E2)')  trim(sh%nom),trim(ch),coord(1:6)
  
  if (any(sh%origine_reel /= 0.0)) then
    origine = origine_reelle_abs3(tm,sh%origine_reel,.true.) ! relatif à l'origine utilisateur  
    write(f%unite,'(T3,A,1x,A,1x,3ES15.6E2)') BSH_CENTRE_PHASE,trim(sh%nom),origine
  end if
end do


end subroutine surface_huygens_ecriture

! ==========================================================
! == Function surface_huygens_pointe(tsh,chaine)          ==
! == tsh : liste de surfaces                              ==
! == chaine : character(LEN=LEN_MOT_CLE)                  ==
! ==                                                      ==
! == Retourne un pointeur pointant sur la                 ==
! == st_une_surface_huygens de nom "chaine"               ==
! ==========================================================
function surface_huygens_pointe(tsh,chaine) result(sh)

type(st_surface_huygens), intent(inout) :: tsh
character(LEN=LEN_MOT_CLE), intent(in) :: chaine

type(st_une_surface_huygens), pointer :: sh
integer :: i

nullify(sh)
do i=1,tsh%nsh
  if (chaine==tsh%l_sh(i)%nom) then
    sh => tsh%l_sh(i)
    exit
  endif
enddo

end function surface_huygens_pointe


end module m_surface_huygens


