!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_tableau_operation.f90
!// Purpose:     Quelques fonctions de bases sur le tri dans un tableau
!// Author:      Cyril Giraudon
!// Modified by: 
!// Lieu     :  XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Novembre 2001
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

module m_tableau_operation

implicit none

interface tri_croissant
  module procedure tri_croissant_entier_dim, &
		   tri_croissant_reel_dim
!		   tri_croissant_entier_sans_dim, &
!		   tri_croissant_reel_sans_dim
end interface tri_croissant

interface tri_decroissant
  module procedure tri_decroissant_entier_dim, &
		   tri_decroissant_reel_dim
!		   tri_decroissant_entier_sans_dim, &
!		   tri_decroissant_reel_sans_dim
end interface tri_decroissant

contains

! ==========================================================
! == tri_croissant_entier                                 ==
! ==                                                      ==
! == n : entier, dimension de tab                         ==
! == tab : tableau d'entier de dimension 1                ==
! == Tri les elements d'un tableau d'entier de            ==
! == maniere croissante                                   ==
! ==========================================================
subroutine tri_croissant_entier_dim(n,tab)

implicit none

integer, intent(in) :: n
integer, dimension(n), intent(inout) :: tab

integer :: i, k
! -- La fonction MINLOC retourne un tableau de dim : rang de tab --
integer, dimension(1) :: j

do i=1,n-1
  j = MINLOC(tab(i:n))
  k = tab(i)
  tab(i) = tab(j(1)+i-1)
  tab(j(1)+i-1) = k
enddo

end subroutine tri_croissant_entier_dim


! ==========================================================
! == tri_croissant_entier                                 ==
! ==                                                      ==
! == n : entier, dimension de tab                         ==
! == tab : tableau d'entier de dimension 1                ==
! == Tri les elements d'un tableau d'entiers de           ==
! == maniere croissante                                   ==
! ==========================================================
subroutine tri_croissant_entier_sans_dim(tab)

implicit none

integer, dimension(:), intent(inout) :: tab

integer :: i, k, n
! -- La fonction MINLOC retourne un tableau de dim : rang de tab --
integer, dimension(1) :: j

n = SIZE(tab)

do i=1,n-1
  j = MINLOC(tab(i:n))
  k = tab(i)
  tab(i) = tab(j(1)+i-1)
  tab(j(1)+i-1) = k
enddo

end subroutine tri_croissant_entier_sans_dim


! ==========================================================
! == tri_croissant_reel                                   ==
! ==                                                      ==
! == n : entier, dimension de tab                         ==
! == tab : tableau de reels de dimension 1                ==
! == Tri les elements d'un tableau de reels de            ==
! == maniere croissante                                   ==
! ==========================================================
subroutine tri_croissant_reel_dim(n,tab)

implicit none

integer, intent(in) :: n
real, dimension(n), intent(inout) :: tab

integer :: i
! -- La fonction MINLOC retourne un tableau de dim : rang de tab --
integer, dimension(1) :: j
real :: k

do i=1,n-1
  j = MINLOC(tab(i:n))
  k = tab(i)
  tab(i) = tab(j(1)+i-1)
  tab(j(1)+i-1) = k
enddo

end subroutine tri_croissant_reel_dim


! ==========================================================
! == tri_croissant_reel                                   ==
! ==                                                      ==
! == n : entier, dimension de tab                         ==
! == tab : tableau de reels de dimension 1                ==
! == Tri les elements d'un tableau de reels de            ==
! == maniere croissante                                   ==
! ==========================================================
subroutine tri_croissant_reel_sans_dim(tab)

implicit none

integer, dimension(:), intent(inout) :: tab

integer :: i, n
! -- La fonction MINLOC retourne un tableau de dim : rang de tab --
integer, dimension(1) :: j
real :: k

n = SIZE(tab)

do i=1,n-1
  j = MINLOC(tab(i:n))
  k = tab(i)
  tab(i) = tab(j(1)+i-1)
  tab(j(1)+i-1) = k
enddo

end subroutine tri_croissant_reel_sans_dim


! ==========================================================
! == tri_decroissant_entier                               ==
! ==                                                      ==
! == n : entier, dimension de tab                         ==
! == tab : tableau d'entier de dimension 1                ==
! == Tri les elements d'un tableau d'entier de            ==
! == maniere decroissante                                 ==
! ==========================================================
subroutine tri_decroissant_entier_dim(n,tab)

implicit none

integer, intent(in) :: n
integer, dimension(n), intent(inout) :: tab

integer :: i, k
! -- La fonction MAXLOC retourne un tableau de dim : rang de tab --
integer, dimension(1) :: j

do i=1,n-1
  j = MAXLOC(tab(i:n))
  k = tab(i)
  tab(i) = tab(j(1)+i-1)
  tab(j(1)+i-1) = k
enddo

end subroutine tri_decroissant_entier_dim


! ==========================================================
! == tri_croissant_entier                                 ==
! ==                                                      ==
! == n : entier, dimension de tab                         ==
! == tab : tableau d'entier de dimension 1                ==
! == Tri les elements d'un tableau d'entiers de           ==
! == maniere croissante                                   ==
! ==========================================================
subroutine tri_decroissant_entier_sans_dim(tab)

implicit none

integer, dimension(:), intent(inout) :: tab

integer :: i, k, n
! -- La fonction MAXLOC retourne un tableau de dim : rang de tab --
integer, dimension(1) :: j

n = SIZE(tab)

do i=1,n-1
  j = MAXLOC(tab(i:n))
  k = tab(i)
  tab(i) = tab(j(1)+i-1)
  tab(j(1)+i-1) = k
enddo

end subroutine tri_decroissant_entier_sans_dim


! ==========================================================
! == tri_decroissant_reel                                 ==
! ==                                                      ==
! == n : entier, dimension de tab                         ==
! == tab : tableau de reels de dimension 1                ==
! == Tri les elements d'un tableau de reels de            ==
! == maniere decroissante                                 ==
! ==========================================================
subroutine tri_decroissant_reel_dim(n,tab)

implicit none

integer, intent(in) :: n
real, dimension(n), intent(inout) :: tab

integer :: i
! -- La fonction MAXLOC retourne un tableau de dim : rang de tab --
integer, dimension(1) :: j
real :: k

do i=1,n-1
  j = MAXLOC(tab(i:n))
  k = tab(i)
  tab(i) = tab(j(1)+i-1)
  tab(j(1)+i-1) = k
enddo

end subroutine tri_decroissant_reel_dim


! ==========================================================
! == tri_decroissant_reel                                 ==
! ==                                                      ==
! == n : entier, dimension de tab                         ==
! == tab : tableau de reels de dimension 1                ==
! == Tri les elements d'un tableau de reels de            ==
! == maniere decroissante                                 ==
! ==========================================================
subroutine tri_decroissant_reel_sans_dim(tab)

implicit none

integer, dimension(:), intent(inout) :: tab

integer :: i, n
! -- La fonction MAXLOC retourne un tableau de dim : rang de tab --
integer, dimension(1) :: j
real :: k

n = SIZE(tab)

do i=1,n-1
  j = MAXLOC(tab(i:n))
  k = tab(i)
  tab(i) = tab(j(1)+i-1)
  tab(j(1)+i-1) = k
enddo

end subroutine tri_decroissant_reel_sans_dim


! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module m_tableau_operation
