!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_temps_frequence.f90
!// Purpose:      Gère les intervalles en temps et en fréquence
!// Author:      Christophe Guiffaut, Cyril Giraudon
!// Modified by:  Christophe Guiffaut
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Novembre 2001
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
! ==========================================================
! == Position frequenciel ou temporel des stockages       ==
! ==========================================================

module m_temps_frequence

use m_sortie, only : st_fichier_sortie, &
				  fichier_sortie_nom_temps, &
				  ouvre_en_ecriture, &
				  fichier_sortie_nom_freq

use m_constantes, only : LEN_MOT_CLE,LEN_CHAINE,rang,RANG_MAITRE_AFF

use m_constantes_scientifiques, only : PI, CVIDE

use m_chaine_operation, only : extrait_sous_chaine,nb_mot_chaine

use m_entree, only : st_fichier, &
                                 lineread, &
				 ligne_precedente, &
				 lecture_stop, &
				 len_int, &
				 ctrl_nb_mot_chaine, &
				 unitefichier, &
				 entier_chaine, &
				 multibackspace, &
				 numbackspace_set

use m_tableau_operation, only : tri_croissant
use m_math_operation
implicit none

integer, parameter :: INTERVALLE_FREQUENTIEL = 1
integer, parameter :: INTERVALLE_TEMPOREL = 2
integer, parameter :: LISTE_FREQUENTIELLE = 3
integer, parameter :: LISTE_TEMPORELLE = 4
integer, parameter :: INTERVALLE_LOGFREQ = 5
integer, parameter :: NB_ECHANTILLON_AUTO = 0
integer, parameter :: NB_ECHANTILLON_FMAX = 5
integer, parameter :: TF_ECHANTILLON_AUTO = 0
integer, parameter :: TF_ECHANTILLON_TOUS = 1
integer, parameter :: TF_ECHANTILLON_NBDT = 2
integer, parameter :: TF_ECHANTILLON_FECH = 3

integer, parameter :: TF_FILTRE_NONE = 0
integer, parameter :: TF_FILTRE_EXP  = 1



! ==========================================================
! == Definition du type st_temps_frequence                ==
! ==========================================================
type st_tf_filtre
  integer  :: type_filtre
  real     :: tau
  real     :: c_t
end type st_tf_filtre

! ==========================================================
! == Definition du type st_temps_frequence                ==
! ==========================================================
type st_temps_frequence
  ! -- nom : nom du st_temps_frequence --
  logical  :: flag_ecrit_base,flag_freq
  character(LEN=LEN_MOT_CLE) :: nom
  ! -- itype : type du st_temps_frequence : INTERVALLE_FREQENTIEL, INTERVALLE_TEMPOREL,
  ! --                                      LISTE_FREQUENTIELLE, LISTE_TEMPORELLE
  ! -- curseur : position dans les liste lors des ecritures --
  integer :: itype, curseur
  ! -- deb : debut d'un intervalle --
  ! -- fin : fin d'un intervalle --
  ! -- pas : pas d'un intervalle --
  real :: deb, fin, pas, saut
  ! -- ideb : deb en iterations --
  ! -- ifin : fin en iterations --
  ! -- ipas : pas en iterations --
  ! -- nbech : Nbre d'echantillons --
  
  integer :: ideb, ifin, ipas, isaut,nbech
  logical :: flag_calcul
  ! -- n_liste : nombre d'elements dans le tableau liste -- 
  ! -- id_n_liste : identifiant dans un fichier netcdf ou hdf5 --
  integer :: n_liste ! id_n_liste
  ! -- liste : liste d'intants ou de frequences --
  ! -- id_liste : id de dataset pour netcdf, hdf5 mais aussi de fichier pour ascii --
  integer :: id_liste
  real, dimension(:), pointer :: liste
  ! -- iliste : liste en iteration --
  integer, dimension(:), pointer :: iliste
  ! -- expi2ipitf : constante pour le calcul des TF --
  complex, dimension(:), pointer :: expi2pidtf, expi2pitf,expi2pitf_h
  ! -- expi2ipitf : constante pour le calcul des TF --
  complex(dp), dimension(:), pointer :: expi2pidtf_dp, expi2pitf_dp

  real    :: tencours

  ! -- Paramètre pour le sous échantillonage du signal pour la TFD
  integer :: type_ech
  integer :: nb_echantillon
  real    :: fe
  real    :: dt_tfd ! pas en temps pour la tfd
  integer :: nech_t ! nombre d'échantillons temporelles

  ! Filtrage
  logical :: flag_filtre
  type(st_tf_filtre) :: filtre

end type st_temps_frequence

!==============================================================================
!           -- structure principal :  st_liste_temps_freq --
!==============================================================================
type st_liste_temps_freq
  ! -- dt : pointeur sur le dt du RUN --
  ! -- tlimit : pointeur sur le tlimit du RUN --
  real :: dt, tlimit
  ! -- n_tf : nombre de st_temps_frequence dans la liste --
  integer :: n_tf
  ! -- tf : tableau de temps frequence --
  type(st_temps_frequence), dimension(:), pointer :: tf
  real :: fmax_fctsrc
end type  st_liste_temps_freq

contains

! ==========================================================
! == Subroutine tf_filtre_copy                            ==
! ==========================================================	
subroutine tf_filtre_initdefaut(fout)
type(st_tf_filtre), intent(out) :: fout

fout%type_filtre = TF_FILTRE_NONE
fout%tau    = 0.0
fout%c_t  = 1.0
end subroutine tf_filtre_initdefaut


! ==========================================================
! == Subroutine temps_frequence_initdefaut                ==
! ==========================================================	
subroutine temps_frequence_initdefaut(tf)

type(st_temps_frequence), intent(inout) :: tf

tf%tencours = 0.0
tf%nom =''
tf%flag_ecrit_base = .false.;tf%flag_freq = .false.
tf%itype= 0; tf%curseur = 0
tf%deb = 0; tf%fin = 0; tf%pas = 0; tf%saut = 0
tf%ideb = 0; tf%ifin = 0; tf%ipas = 0; tf%isaut = 1
tf%nbech = 0
tf%n_liste = 0

tf%flag_calcul = .true.
tf%type_ech =TF_ECHANTILLON_AUTO
tf%nb_echantillon = NB_ECHANTILLON_AUTO
tf%fe = 0.0
tf%dt_tfd = 0.0; tf%nech_t = 0

call tf_filtre_initdefaut(tf%filtre)

nullify(tf%liste)
nullify(tf%iliste)
nullify(tf%expi2pidtf)
nullify(tf%expi2pitf)
nullify(tf%expi2pitf_h)
nullify(tf%expi2pidtf_dp)
nullify(tf%expi2pitf_dp)

end subroutine temps_frequence_initdefaut


! ==========================================================
! == Subroutine liste_temps_freq_initdefaut          ==
! == ltf : st_liste_tamps_frequence                       ==
! ==========================================================
subroutine liste_temps_freq_initdefaut(ltf)

implicit none

type(st_liste_temps_freq), intent(inout) :: ltf
ltf%dt = 0.0
ltf%tlimit = 0.0
ltf%n_tf = 0
ltf%fmax_fctsrc = 0.0
nullify(ltf%tf)

end subroutine liste_temps_freq_initdefaut


! ============================================================
! == Subroutine tf_filtre_compare                          ==
! == Comparaison de deux structures                        == 
! ============================================================
logical function tf_filtre_compare(f1,f2)
type(st_tf_filtre), intent(in) :: f1,f2

if ((f1%type_filtre/=f2%type_filtre) .or.&
    (f1%tau/=f2%tau).or.(f1%c_t/=f2%c_t)) then
  tf_filtre_compare = .false.
else
  tf_filtre_compare = .true.
end if
end function tf_filtre_compare


! ==========================================================
! == Subroutine tf_filtre_copy                            ==
! ==========================================================	
subroutine tf_filtre_copy(fin,fout)
type(st_tf_filtre), intent(in)  :: fin
type(st_tf_filtre), intent(out) :: fout

fout%type_filtre = fin%type_filtre
fout%tau    = fin%tau
fout%c_t  = fin%c_t
end subroutine tf_filtre_copy

! ==========================================================
! == Subroutine temps_frequence_copy                      ==
! ==========================================================	
subroutine temps_frequence_copy(tfin,tfout)

type(st_temps_frequence), intent(in) :: tfin
type(st_temps_frequence), intent(out) :: tfout

tfout%nom = tfin%nom
tfout%flag_ecrit_base = tfin%flag_ecrit_base
tfout%flag_freq = tfin%flag_freq 
tfout%itype= tfin%itype; tfout%curseur = tfin%curseur
tfout%deb = tfin%deb; tfout%fin = tfin%fin; tfout%pas = tfin%pas; tfout%saut = tfin%saut
tfout%ideb = tfin%ideb; tfout%ifin = tfin%ifin; tfout%ipas = tfin%ipas; tfout%isaut = tfin%isaut
tfout%nbech = tfin%nbech
tfout%n_liste = tfin%n_liste

tfout%tencours=tfin%tencours

tfout%flag_calcul = tfin%flag_calcul
tfout%type_ech = tfin%type_ech
tfout%nb_echantillon = tfin%nb_echantillon
tfout%fe = tfin%fe
tfout%dt_tfd = tfin%dt_tfd; tfout%nech_t = tfin%nech_t

call tf_filtre_copy(tfin%filtre,tfout%filtre)

if (ASSOCIATED(tfin%liste)) then
  allocate(tfout%liste(tfout%n_liste))
  tfout%liste = tfin%liste
else
  nullify(tfout%liste)
end if

! non utile
nullify(tfout%iliste)
nullify(tfout%expi2pidtf)
nullify(tfout%expi2pitf_h)
nullify(tfout%expi2pidtf_dp)
nullify(tfout%expi2pitf_dp)

end subroutine temps_frequence_copy

! ==========================================================
! == Subroutine temps_frequence_deallocate                ==
! == tf : st_temps_frequence                              ==
! == Liberation de toute la memoire allouee pour          ==
! == un st_temps_frequence                                ==
! ==========================================================
subroutine temps_frequence_deallocate(tf)

type(st_temps_frequence), intent(inout) :: tf

if (ASSOCIATED(tf%liste)) deallocate(tf%liste)
if (ASSOCIATED(tf%iliste)) deallocate(tf%iliste)
if (ASSOCIATED(tf%expi2pidtf)) deallocate(tf%expi2pidtf)
if (ASSOCIATED(tf%expi2pitf)) deallocate(tf%expi2pitf)
if (ASSOCIATED(tf%expi2pitf_h)) deallocate(tf%expi2pitf_h)
nullify(tf%liste);nullify(tf%iliste);
nullify(tf%expi2pidtf);nullify(tf%expi2pitf);nullify(tf%expi2pitf_h)

end subroutine temps_frequence_deallocate

! ==========================================================
! == Subroutine liste_temps_freq_deallocate(ltf)          ==
! == ltf :liste_temps_freq                                ==
! == Desalloue la memoire occupee par les champs de ltf   ==
! ==========================================================
subroutine liste_temps_freq_deallocate(ltf)

type(st_liste_temps_freq), intent(inout) :: ltf

integer :: i
if (ASSOCIATED(ltf%tf)) then
  do i=1,ltf%n_tf
    call temps_frequence_deallocate(ltf%tf(i))
  enddo
  deallocate(ltf%tf);nullify(ltf%tf)
end if
end subroutine liste_temps_freq_deallocate

! ==========================================================
! == Subroutine temps_frequence_lecture                   ==
! == f : fichier d'entree                                 ==
! == ltf : st_liste_temps_freq                       ==
! ==                                                      ==
! == Lecture des caracteristiques des st_temps_frequence  ==
! == a partir du fichier de donnees f.                    ==
! ==========================================================
subroutine liste_temps_freq_lecture(f,ltf)

implicit none

type(st_fichier), intent(inout) :: f
type(st_liste_temps_freq), intent(inout) :: ltf

character(LEN=LEN_MOT_CLE) :: nom, nom1, premiermot
character(LEN=LEN_CHAINE) :: chaine
logical :: flag_fe, flag_ecrit_base
real :: fe
real :: deb, fin, pas
integer :: i, i_tf, nbmot,type_ech
integer :: nb_echantillon, status
real, dimension(3) :: intervalle
logical  :: flag_filtre

type(st_temps_frequence), pointer :: ptf
type(st_tf_filtre)  :: filtre


! -- Positionnement de numbackspace a 0 --
! -- numbackspace : nombre de sauts de ligne --
! -- vers le haut a effectuer lors d'un multibackspace() --
call numbackspace_set(f,0)
! -- Premiere lecture pour les dimensions de tableaux ... --
do
  ! -- Lecture de la premiere chaine valide et retour du premier mot --
  premiermot = lineread(f,1)
  if (premiermot==f%nomfinbloc) then
    call ligne_precedente(f)
    exit
  endif
  select case(premiermot(1:2))
  case ('IF','LF','IT','LT','IL')
    ltf%n_tf = ltf%n_tf + 1
  end select
enddo

! -- Allocation memoire pour les st_temps_frequences --
allocate(ltf%tf(ltf%n_tf))
do i=1,ltf%n_tf
  call temps_frequence_initdefaut(ltf%tf(i))
enddo

! -- Retour vers la premiere ligne du bloc temps_frequence --
i = multibackspace(f)
i_tf = 0
nb_echantillon = NB_ECHANTILLON_AUTO
fe = 0.0
flag_fe = .false.
flag_ecrit_base = .false.
type_ech = TF_ECHANTILLON_AUTO
flag_filtre = .false.
call tf_filtre_initdefaut(filtre)

do
  ! -- On recommence la lecture pour les caracteristiques de st_temps_frequence --
  premiermot = lineread(f)
  
  !----------------------------------------------------------------------------
  select case(premiermot(1:2))
    case('IL','IF','IT','LF','LT')
      i_tf = i_tf + 1
      ptf =>ltf%tf(i_tf)
      ptf%flag_filtre = flag_filtre
      call tf_filtre_copy(filtre,ptf%filtre)
  end select
  
  select case(premiermot(1:2))
    case('IL')
      status = ctrl_nb_mot_chaine(f,f%chaine,4)      
      read(f%chaine,*,err=999) ptf%nom, ptf%deb, ptf%fin, ptf%nbech
      ptf%pas = ptf%deb
      ptf%itype = INTERVALLE_LOGFREQ
      ptf%nb_echantillon = nb_echantillon
      if (flag_fe) ptf%fe = fe
      ptf%flag_ecrit_base = flag_ecrit_base
      ptf%flag_freq = .true.
      ptf%type_ech = type_ech
      cycle
    case('IF')
      status = ctrl_nb_mot_chaine(f,f%chaine,4)      
      read(f%chaine,*,err=999) ptf%nom, ptf%deb, &
                               ptf%fin, ptf%pas
      ptf%itype = INTERVALLE_FREQUENTIEL
      ptf%nb_echantillon = nb_echantillon
      if (flag_fe) ptf%fe = fe
      ptf%flag_ecrit_base = flag_ecrit_base
      ptf%flag_freq = .true.
      ptf%type_ech = type_ech
      cycle
    case('IT')
      status = ctrl_nb_mot_chaine(f,f%chaine,1,4)
      intervalle(:) = 0.0      
      intervalle(2) = 1.0e6       
      nbmot = nb_mot_chaine(f%chaine)
      if (nbmot == 1) then
        read(f%chaine,*,err=999) ptf%nom
      else
        read(f%chaine,*,err=999) ptf%nom, intervalle(1:nbmot-1)
      end if			       
      ptf%deb = intervalle(1)
      ptf%fin = intervalle(2)
      ptf%pas = intervalle(3)

      ptf%itype = INTERVALLE_TEMPOREL
      ptf%flag_ecrit_base = flag_ecrit_base
      ptf%type_ech = type_ech
      cycle

    case('LF')
      ptf%n_liste = nb_mot_chaine(f%chaine) - 1
      allocate(ptf%liste(ptf%n_liste)); ptf%liste = 0
      read(f%chaine,*,err=999) ptf%nom, ptf%liste
      ptf%itype = LISTE_FREQUENTIELLE
      ptf%nb_echantillon = nb_echantillon
      if (flag_fe) ptf%fe = fe
      ptf%flag_ecrit_base = flag_ecrit_base
      ptf%flag_freq = .true.
      ptf%type_ech = type_ech
      cycle
      
    case('LT')
      ptf%n_liste = nb_mot_chaine(f%chaine) - 1
      allocate(ptf%liste(ptf%n_liste)); ptf%liste = 0
      read(f%chaine,*,err=999) ptf%nom, ptf%liste
      ptf%itype = LISTE_TEMPORELLE
      ptf%flag_ecrit_base = flag_ecrit_base
      ptf%type_ech = type_ech
      cycle
  end select
    
  if (premiermot == 'echantillon_fmax') then
      status = ctrl_nb_mot_chaine(f,f%chaine,2)      
      read(f%chaine,*,err=999) nom, nom1
      flag_fe = .false.
      if (nom1 == 'auto') then
        nb_echantillon = NB_ECHANTILLON_AUTO
	type_ech = TF_ECHANTILLON_AUTO
      else if (nom1 == 'tous') then
        nb_echantillon = 1e9
	type_ech = TF_ECHANTILLON_TOUS
      else
        status = ctrl_nb_mot_chaine(f,f%chaine,2)      
        read(f%chaine,*,err=999) nom, nb_echantillon
	type_ech = TF_ECHANTILLON_NBDT
        if (nb_echantillon<2) then
	  print '("Nb échantillon doit être >= à 2 !!!!, Vous connaissez Shannon ???")'
	  call lecture_stop(f)
	end if
      end if  
  else if (premiermot == 'frequence_ech') then
      status = ctrl_nb_mot_chaine(f,f%chaine,2)      
      read(f%chaine,*,err=999) nom, fe
      flag_fe = .true.
      type_ech= TF_ECHANTILLON_FECH
  else if (premiermot == 'ecrit_base') then 
      ! s'applique à tous les intervalles définis en dessous
      status = ctrl_nb_mot_chaine(f,f%chaine,2)      
      read(f%chaine,*,err=999) nom, i
      flag_ecrit_base = .true.
      if (i==0)  flag_ecrit_base = .false.
  !------------------------------------------------------------------
  else if (premiermot=='filtre') then
    nbmot = nb_mot_chaine(f%chaine);
    read(f%chaine,*) nom,nom1
    flag_filtre = .true.
    select case (nom1)
      case ('exp')
        nbmot = ctrl_nb_mot_chaine(f,f%chaine,3);
        chaine = extrait_sous_chaine(f%chaine,3,nbmot)
        filtre%type_filtre=TF_FILTRE_EXP
        read(chaine,*,ERR=999) filtre%tau
	if (filtre%tau<=0.0) then
	  print '(/,t10,"Erreur, Valeur de tau erronée !!!")'
	  call lecture_stop(f)  
	end if
      case('non','none')
        filtre%type_filtre=TF_FILTRE_NONE
	flag_filtre = .false.
      case default
	print '(/,t10,"Erreur, nom=",A," nom du filtre non reconnu !!!")',trim(nom1)
	call lecture_stop(f)  
    end select
  !------------------------------------------------------------------
  else
      if (premiermot==f%nomfinbloc) then
	exit
      endif
      ! -- Si ca se passe pas bien : arret de la lecture --
      call lecture_stop(f)

  end if
enddo

return
999 print '(/,t10,"Erreur de lecture !!!")'
    call lecture_stop(f)

end subroutine liste_temps_freq_lecture

! ==========================================================
! == Subroutine liste_temps_freq_ecriture                 ==
! ==========================================================
subroutine liste_temps_freq_ecriture(f,ltf)

type(st_fichier), intent(inout) :: f
type(st_liste_temps_freq), intent(inout) :: ltf

logical :: flag_ecrit_base
integer :: type_ech
integer :: i,l
type(st_temps_frequence),  pointer :: tf

flag_ecrit_base = .false.
type_ech = TF_ECHANTILLON_AUTO
l=0;
do i=1,ltf%n_tf
  tf =>ltf%tf(i)
  !----------------------------------------------------------------------------
  if (.not.(tf%flag_ecrit_base .eqv. flag_ecrit_base)) then
    flag_ecrit_base = tf%flag_ecrit_base;
    l=mod(1+l,2)
    write(f%unite,'(T3,A,3x,I1)') 'ecrit_base',l
  end if
  
  !----------------------------------------------------------------------------
  if (tf%type_ech/=type_ech) then
    type_ech = tf%type_ech
    if (tf%type_ech==TF_ECHANTILLON_AUTO) then 
      write(f%unite,'(T3,A,3x,A)') 'echantillon_fmax','auto'
    else if (tf%type_ech==TF_ECHANTILLON_TOUS) then
      write(f%unite,'(T3,A,3x,A)') 'echantillon_fmax','tous'
    else if (tf%type_ech==TF_ECHANTILLON_NBDT) then
      write(f%unite,'(T3,A,3x,I4)') 'echantillon_fmax',tf%nb_echantillon  
    else if (tf%type_ech==TF_ECHANTILLON_FECH) then
      write(f%unite,'(T3,A,3x,G14.6)') 'frequence_ech',tf%fe  
    end if
  end if

  !----------------------------------------------------------------------------
  if (tf%itype==INTERVALLE_TEMPOREL) then
    write(f%unite,'(T3,A,3x,3G14.6)')  trim(tf%nom),tf%deb,tf%fin,tf%pas
  !----------------------------------------------------------------------------
  else if (tf%itype==LISTE_TEMPORELLE) then
!    write(f%unite,'(T3,A,3x,<tf%n_liste>G14.6)') trim(tf%nom),tf%liste(1:tf%n_liste)
    write(f%unite,'(T3,A,3x,50G14.6)') trim(tf%nom),tf%liste(1:tf%n_liste)
  !----------------------------------------------------------------------------
  else if (tf%itype==LISTE_FREQUENTIELLE) then
!    write(f%unite,'(T3,A,3x,<tf%n_liste>G14.6)') trim(tf%nom),tf%liste(1:tf%n_liste)
    write(f%unite,'(T3,A,3x,50G14.6)') trim(tf%nom),tf%liste(1:tf%n_liste)
  !----------------------------------------------------------------------------
  else if (tf%itype==INTERVALLE_LOGFREQ) then
    write(f%unite,'(T3,A,3x,2G14.6,3x,I5)') trim(tf%nom),tf%deb,tf%fin,tf%nbech
  !----------------------------------------------------------------------------
  else if (tf%itype==INTERVALLE_FREQUENTIEL) then
    write(f%unite,'(T3,A,3x,3G14.6)') trim(tf%nom),tf%deb,tf%fin,tf%pas
  
  end if
end do


end subroutine liste_temps_freq_ecriture

! ==========================================================
! == Subroutine liste_temps_freq_init      ==
! == ltf : st_liste_temps_freq                       ==
! == Verifie l'ensemble des st_temps_frequence et         ==
! == convertit le temporel en iterations                  ==
! ==                                                      ==
! == Ouverture de fichiers - Creation des datasets        ==
! ==========================================================
subroutine liste_temps_freq_init(ltf,fs,dt,duree_totale,deltamax)

type(st_liste_temps_freq), intent(inout) :: ltf
type(st_fichier_sortie), intent(inout) :: fs
real, intent(in) :: dt            !pas temporel fdtd
real, intent(in) :: duree_totale  !durée simu fdtd
real, intent(in) :: deltamax      ! pas spatial maxi

integer :: i, status

! -- Association des pointeur dt et tlimit --
ltf%dt = dt
ltf%tlimit = duree_totale

! -- Quelques verifications sur les donnees --
! -- Transformations temps => itérations --
do i=1,ltf%n_tf
  call temps_frequence_init(ltf%tf(i),fs,dt,duree_totale,deltamax,ltf%fmax_fctsrc)
enddo

call liste_temps_freq_affichage(ltf)
end subroutine liste_temps_freq_init
	
! ==========================================================
! == Subroutine resultat_intervalle_init        ==
! == tf : st_temps_frequence                              ==
! == Quelques verification par rapport                    ==
! == a l'ensemble du RUN                                  ==
! == Conversion dfu temporel en iterations                ==
! ==========================================================
subroutine temps_frequence_init(tf,fs,dt,duree_totale,deltamax,fmax_fctsrc)

type(st_temps_frequence), intent(inout) :: tf
type(st_fichier_sortie), intent(inout) :: fs
real, intent(in) :: dt           ! pas de temps de la simu 
real, intent(in) :: duree_totale ! duree de la simu
real, intent(in) :: deltamax     ! pas spatial maxi
real, intent(in) :: fmax_fctsrc ! Fréquence maxi des réponses en fréquences de toutes les 
				 !  fonctions sources actives
					
integer :: n,m,ndeb,nbech_initial,niter
integer :: i1,i2, nbdecade, nbech_decade1, nbech_decade2
real    :: fdeb,fpas
real    :: pasdecade1,pasdecade2
real    :: freq, dt_ech,fmax, fmax_valide
real    :: a
complex :: i

i = cmplx(0,1)
tf%isaut  = conv_tmps_iter(tf%saut,dt)
if (tf%isaut==0) tf%isaut = 1
!----------------------------------------------------------
if (tf%itype==INTERVALLE_TEMPOREL) then
  if (.not.convert_inter_temp(tf,dt,duree_totale)) then
    stop  'stop'
  endif

  tf%n_liste = NINT((tf%ifin-tf%ideb)*1.0/tf%ipas) + 1  
  allocate(tf%iliste(1:tf%n_liste))
  allocate(tf%liste(1:tf%n_liste))
  
  do n=1,tf%n_liste
   tf%iliste(n) = tf%ideb + (n-1)*tf%ipas
   tf%liste(n)  = tf%iliste(n) * dt
  end do
  
!----------------------------------------------------------
elseif (tf%itype==INTERVALLE_FREQUENTIEL) then
  if (.not.verif_rectif_inter_freq(tf)) then
    stop 'stop'
  endif

  tf%n_liste = NINT((tf%fin-tf%deb)/tf%pas) + 1  
  allocate(tf%expi2pidtf(1:tf%n_liste))
  allocate(tf%expi2pitf(1:tf%n_liste))
  allocate(tf%expi2pitf_h(1:tf%n_liste))
  allocate(tf%liste(1:tf%n_liste))
   
  do n=1,tf%n_liste
   tf%liste(n) = tf%deb + (n-1) * tf%pas
  enddo
!----------------------------------------------------------
elseif (tf%itype==INTERVALLE_LOGFREQ) then
  if (.not.verif_rectif_inter_logfreq(tf)) then
    stop 'TRIDIMO n''ira pas plus loin !!!'
  endif
  
  
  i1 = int(log10(tf%deb))
  i2 = int(log10(tf%fin))
  nbdecade = i2-i1
  pasdecade1 = 10.0**(i1+1)/tf%nbech
  pasdecade2 = 10.0**(i2+1)/tf%nbech
  nbech_decade1 = NINT((10.0**(i1+1) -tf%deb)/pasdecade1)
  nbech_decade2 = NINT((tf%fin-10.0**i2)/pasdecade2)+1
  tf%n_liste = nbech_decade1+nbech_decade2
  
  ! Correction du nombre d'échantillons par décade
  nbech_initial = tf%nbech
  tf%nbech = NINT(0.9*tf%nbech)
  
  if (nbdecade>=1) tf%n_liste = tf%n_liste + (nbdecade-1)*tf%nbech
  allocate(tf%expi2pidtf(1:tf%n_liste))
  allocate(tf%expi2pitf(1:tf%n_liste))
  allocate(tf%expi2pitf_h(1:tf%n_liste))
  allocate(tf%liste(1:tf%n_liste))
   
  do n=1,nbech_decade1
   tf%liste(n) = tf%deb + (n-1) * pasdecade1
  enddo
  if (rang==RANG_MAITRE_AFF) then
    print '(T10,"INTERVALLE LOGARITHME FREQUENCE",2I4)',i1,i2
    print '(T10,"i1,i2 = ",2I4)',i1,i2
    print '(T10,"nbdecade = ",I4,"; nbech de la decade principale et celle de fin = ",2I4)',nbdecade,&
              nbech_decade1,nbech_decade2
    print '(T10,"n_liste = ",I4)',tf%n_liste
  end if
  ndeb = nbech_decade1
  do m=1,nbdecade-1
    fdeb = 10.0**(i1+m)
    fpas = 10.0**(i1+m+1)/nbech_initial
    do n=1,tf%nbech
     tf%liste(n+ndeb) = fdeb + (n-1) * fpas
    enddo
    ndeb = ndeb+tf%nbech
  end do
  
  do n=1,nbech_decade2
   tf%liste(tf%n_liste-nbech_decade2+n) = 10.0**i2 + (n-1) * pasdecade2
  enddo
  
!----------------------------------------------------------
elseif (tf%itype==LISTE_TEMPORELLE) then
  call tri_croissant(tf%n_liste,tf%liste)
  allocate(tf%iliste(tf%n_liste))
  tf%iliste = NINT(tf%liste/dt)

!----------------------------------------------------------
elseif (tf%itype==LISTE_FREQUENTIELLE) then
  call tri_croissant(tf%n_liste,tf%liste)
  allocate(tf%expi2pidtf(1:tf%n_liste))
  allocate(tf%expi2pitf(1:tf%n_liste))
  allocate(tf%expi2pitf_h(1:tf%n_liste))
   
endif

! -- Sous échantillonnage pour la TFD --
! -- Calcul du pas temporel d'échantillonnage --
if (tf%itype==LISTE_FREQUENTIELLE .or. tf%itype==INTERVALLE_FREQUENTIEL.or. &
    tf%itype==INTERVALLE_LOGFREQ) then
  fmax_valide = CVIDE / (10.0 * deltamax) ! critère d'échantillonnage spatiale
  fmax = tf%liste(tf%n_liste)   ! fréquence max d'étude
  fmax = max(fmax,fmax_fctsrc)

!integer, parameter :: TF_ECHANTILLON_AUTO = 0
!integer, parameter :: TF_ECHANTILLON_TOUS = 1
!integer, parameter :: TF_ECHANTILLON_NBDT = 2
  dt_ech = dt;
  if (tf%type_ech == TF_ECHANTILLON_FECH .and. tf%fe > 0.0) then
    dt_ech = 1 / tf%fe
  else if (tf%type_ech == TF_ECHANTILLON_AUTO) then
    !dt_ech = 1/(NB_ECHANTILLON_FMAX * MAX(fmax,fmax_valide)) 
    dt_ech = 1/(NB_ECHANTILLON_FMAX * fmax) 
  else if (tf%type_ech == TF_ECHANTILLON_TOUS) then
    dt_ech = dt
  else if (tf%type_ech == TF_ECHANTILLON_NBDT) then
    !dt_ech = 1/(tf%nb_echantillon * fmax) 
    dt_ech = tf%nb_echantillon * dt;
  end if
  
  if (dt_ech < dt) then
    tf%isaut = 1
  else 
    tf%isaut = NINT(dt_ech/dt)
  end if
  
  tf%dt_tfd =  dt * tf%isaut; ! pas d'échantillonage rectifié
  niter = NINT(duree_totale/dt)
  tf%nech_t = INT((niter-1)/tf%isaut)
  
  tf%fe = 1 / tf%dt_tfd
  
  !--- Introduction d'un filtre --
  a=0.0
  if (tf%flag_filtre) then
    if (tf%filtre%type_filtre == TF_FILTRE_EXP) then
      a = -tf%dt_tfd/tf%filtre%tau
    end if
  end if 
  !--- Calcul de exp(-i2pifdt) pour la TFD ---
  ! --> on tient compte du filtre 
  do n=1,tf%n_liste
    tf%expi2pidtf(n) = EXP(cmplx(a,-2*PI*tf%dt_tfd*tf%liste(n)))
    tf%expi2pitf_h(n) = tf%dt_tfd / EXP(cmplx(a,-2*PI*(tf%dt_tfd+dt*0.5)*tf%liste(n)))
  enddo
 
  tf%expi2pitf(:) = tf%dt_tfd / tf%expi2pidtf(:) 
end if

! -- Ouverture de fichiers - Creation des datasets --
if (tf%flag_ecrit_base.and.(rang==RANG_MAITRE_AFF)) then
  call temps_frequence_ascii_init(tf,fs)
endif

end subroutine temps_frequence_init

! ==========================================================
! == Subroutine temps_frequence_allonge_tmax              ==
! == Le tmax est modifié en cours de simulation
! == Seul, les listes pour les intervalles en temps sont modifiées
! == aucune conséquence pour les listes d'instants ou les intervalles
! == en fréquence
! ==========================================================
subroutine temps_frequence_allonge_tmax(ltf,dt,tmax_new)

type(st_liste_temps_freq), intent(inout) :: ltf
real, intent(in) :: dt            !pas temporel fdtd
real, intent(in) :: tmax_new      !durée simu fdtd

integer :: i,n,status
type(st_temps_frequence), pointer :: tf

! -- Association des pointeur dt et tlimit --

!if (tmax_new <= ltf%tlimit .and. rang==RANG_MAITRE_AFF) then
!  print '("Avertissement, le nouveau TMAX <= tlimit dans la subroutine temps_frequence_allonge_tmax!!!")'
!  return
!end if

do i=1,ltf%n_tf
  tf=>ltf%tf(i)
  if (tf%itype==INTERVALLE_TEMPOREL) then
    if (tf%fin==ltf%tlimit) then ! Intervalle jusqu'à la limite
      tf%fin  = tmax_new
      tf%ifin = conv_tmps_iter(tf%fin,dt)
      tf%n_liste = NINT((tf%ifin-tf%ideb)*1.0/tf%ipas) + 1  
      deallocate(tf%iliste,tf%liste)
      allocate(tf%iliste(1:tf%n_liste))
      allocate(tf%liste(1:tf%n_liste))
      
      do n=1,tf%n_liste
       tf%iliste(n) = tf%ideb + (n-1)*tf%ipas
      end do
      tf%liste  = tf%iliste * dt
    end if
  end if
enddo

! La nouveau temps limite de simulation
ltf%tlimit = tmax_new

end subroutine temps_frequence_allonge_tmax


! ==========================================================
! == Function verif_rectif_inter_temp                     ==
! == Verifie dans le cas d'un INTERVALLE_TEMPOREL que :   ==
! ==                                                      ==
! == - pas > 0, sinon arret                               ==
! == - deb  < fin, sinon arret                            ==
! == - pas < dt, sinon pas = dt                           ==
! == - deb > 0, sinon deb = 0                             ==
! == - fin < tlimit, sinon fin = tlimit                   ==
! ==========================================================
logical function verif_rectif_inter_temp(tf,dt,tlimit)

type(st_temps_frequence), intent(inout) :: tf
real, intent(in)  :: dt, tlimit

real  :: dpas

verif_rectif_inter_temp = .true.
if (tf%pas < 0) then 
  print '(/,t10,"Erreur : le pas de temps doit etre superieur a 0")'
  verif_rectif_inter_temp = .false.
else  
  ! -- Rectification --
  if (tf%pas < dt) tf%pas = dt
  if (tf%deb < 0.0) tf%deb = 0.0
  if (tf%fin > tlimit) tf%fin = tlimit
  if (tf%deb > tf%fin) then 
    print '(/,t10,"Erreur : temps min > temps max")'
    verif_rectif_inter_temp = .false.
  end if  
endif
				
end function verif_rectif_inter_temp


! ==========================================================
! == Function verif_rectif_intervalle_frequentiel         ==
! == Quelques verifications sur les bandes de frequence   ==
! ==                                                      ==
! == - Si deb > fin : arret                               ==
! == - si pas <= 0 : arret                                ==
! == - Si deb < 0 : deb = 0                               ==
! ==========================================================
logical function verif_rectif_inter_freq(tf)

type(st_temps_frequence), intent(inout) :: tf

real  :: dpas

verif_rectif_inter_freq = .true.
if (tf%deb > tf%fin) then 
  print '(/,t10,"Erreur : frequence min > frequence max")'
  verif_rectif_inter_freq = .false.
elseif (tf%pas <= 0) then 
  print '(/,t10,"Erreur : pas de frequence doit etre > 0")'
  verif_rectif_inter_freq = .false.
else
  ! -- Rectification --
  if (tf%deb < 0.0) tf%deb = 0.0
endif

end function verif_rectif_inter_freq

! ==========================================================
! == Function verif_rectif_inter_logfreq                  ==
! == Quelques verifications l'intervelle logarithme frequence   ==
! ==                                                      ==
! == - Si deb > fin : arret                               ==
! == - si pas <= 0 : arret                                ==
! == - Si deb < 0 : deb = 0                               ==
! ==========================================================
logical function verif_rectif_inter_logfreq(tf)

type(st_temps_frequence), intent(inout) :: tf

real  :: dpas

verif_rectif_inter_logfreq = .true.
  ! -- Rectification --
!  if (tf%deb <= 0.0) tf%deb = 1.0  ! 1 Hz
if (tf%deb > tf%fin) then 
  print '(/,t10,"Erreur : frequence min > frequence max")'
  verif_rectif_inter_logfreq = .false.
elseif (tf%deb < 1.0e-6) then 
  print '(/,t10,"Erreur :  fmin >= 1.0e-6 Hz")'
  verif_rectif_inter_logfreq = .false.
endif

if (tf%nbech<1) then
  print '(/,t10,"Erreur : le nombre d''échantillon doit etre >1 sur l''intervalle logarithme fréquence")'
  verif_rectif_inter_logfreq = .false.
end if

end function verif_rectif_inter_logfreq


! ==========================================================
! == Function convert_inter_temp                          ==
! == Convertit un intervalle temporel en seconde en       ==
! == intervalle d'iterations                              ==
! ==========================================================
logical function convert_inter_temp(tf,dt,tlimit)

type(st_temps_frequence), intent(inout) :: tf
real, intent(in)  :: dt, tlimit

real  :: dpas

convert_inter_temp = .true.
if (.not.verif_rectif_inter_temp(tf,dt,tlimit)) then
  print '(/,t10,"Erreur : intervalle temporel mal défini")'
  convert_inter_temp = .false.
else
  tf%ideb = conv_tmps_iter(tf%deb,dt)
  tf%ifin = conv_tmps_iter(tf%fin,dt)
  tf%ipas = conv_tmps_iter(tf%pas,dt)

  if (tf%ipas < 1) tf%ipas = 1
endif
				
end function convert_inter_temp


! ==========================================================
! == Function conv_tmps_en_iter                           ==
! == t : date a convertir                                 ==
! == dt : pas temporel                                    ==
! ==                                                      ==
! == Convertit une valeur temporelle en iterations        ==
! ==========================================================
integer function conv_tmps_iter(t,dt)

real, intent(in) :: t, dt

conv_tmps_iter = NINT(t/dt)

end function conv_tmps_iter

! ==========================================================
! == Subroutine temps_freq_compteur_incr                    ==
! == Incrémente des compteurs en temps et en fréquence si 
! == le descripteur d'intervalle correspondant est trouvé
! == nom1, nom2 : noms d'un descripteur temps-frequence   
! == i_t, i_f : compteur temps et fréquence  ==
! ==========================================================
subroutine temps_freq_compteur_incr(nom1,nom2,i_t,i_f)

!=== Déclaration des arguments ===
character(LEN=*), intent(in) :: nom1, nom2
integer, intent(inout)  :: i_t,i_f

if (nom1(2:2) == 'T' .or. nom2(2:2) == 'T') i_t = i_t + 1
if (nom1(2:2) == 'F' .or. nom2(2:2) == 'F') i_f = i_f + 1

if (nom1(1:2)=='IL' .or.nom2(1:2) == 'IL') i_f = i_f + 1

end subroutine temps_freq_compteur_incr

! ==========================================================
! == Subroutine temps_freq_compte                    ==
! == Incrémente des compteurs en temps et en fréquence si 
! == le descripteur d'intervalle correspondant est trouvé
! == i_t, i_f : compteur temps et fréquence  ==
! ==========================================================
subroutine temps_freq_compte(f,mot_cle,i_t,i_f,chaine_fin)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout):: f
integer, intent(out)  :: i_t,i_f
character(LEN=*), intent(in) :: mot_cle 
character(LEN=*), optional, intent(in) :: chaine_fin

!===  Déclaration des variables  ===
integer  :: compteur,i,l
character(LEN=LEN_MOT_CLE) :: nom, nom1,nom2
character(LEN=LEN_CHAINE)  :: premier_mot
character(LEN=LEN_CHAINE)  :: ch_fin

compteur = 0
i_t=0 
i_f=0
l = LEN_TRIM(mot_cle)
call numbackspace_set(f,0) ! compteur retour ligne dans fichier à 0
if (PRESENT(chaine_fin)) then
  ch_fin = chaine_fin
else
  ch_fin = f%nomfinbloc
end if

do
  premier_mot = lineread(f,1)
  if ((premier_mot==f%nomfinbloc).or.(premier_mot==ch_fin)) then
    i = multibackspace(f)
    exit
  end if
  if (premier_mot(1:l) == mot_cle) then
    read(f%chaine,*) nom, nom1, nom2
    call temps_freq_compteur_incr(nom1,nom2,i_t,i_f)
    cycle
  endif
enddo
 
end subroutine temps_freq_compte



! ==========================================================
! == Function temps_freq_identifie_freq                   ==
! == nom : noms d'un descripteur temps-frequence   ==
! ==  retourne un flag qui indique si on a un stockage  en frequence  ==
! ==========================================================
logical function temps_freq_identifie_freq(nom)

character(LEN=*), intent(in) :: nom

temps_freq_identifie_freq = .false.
if (nom(1:2) == 'IF' .or. nom(1:2) == 'LF'.or. nom(1:2) == 'IL') then
  temps_freq_identifie_freq = .true.
end if
 
end function temps_freq_identifie_freq

! ==========================================================
! == Function temps_freq_identifie_temps                  ==
! == nom : noms d'un descripteur temps-frequence   ==
! ==  retourne un flag qui indique si on a un stockage  en temps      ==
! ==========================================================
logical function temps_freq_identifie_temps(nom)

character(LEN=*), intent(in) :: nom

temps_freq_identifie_temps = .false.
if (nom(1:2) == 'IT' .or. nom(1:2) == 'LT'  ) then
  temps_freq_identifie_temps = .true.
end if
 
end function temps_freq_identifie_temps

! ==========================================================
! == Function identifie_deux_nom                          ==
! == nom1, nom2 : noms d'un descripteur temps-frequence   ==
! == flag_t, flag_f : indique si stockage en temps et ou  ==
! ==          en frequence                                ==
! == nom_t, nom_f : nom du descripteur temps et frequence ==
! ==        respectivement                                ==
! ==========================================================
logical function identifie_deux_nom(nom1,nom2,flag_t,flag_f,nom_t,nom_f)

character(LEN=LEN_MOT_CLE), intent(in) :: nom1, nom2
logical, intent(out)  :: flag_t, flag_f
character(LEN=LEN_MOT_CLE), intent(out) :: nom_t, nom_f

identifie_deux_nom = .true.
flag_t = .false.
flag_f = .false.
if (nom1(2:2) == 'T') then
  flag_t = .true.
  nom_t = nom1 
  if (nom2(2:2) == 'F'.or.nom2(1:2) == 'IL') then
    flag_f = .true. 
    nom_f = nom2 
  else if (nom2(2:2) == 'T') then
    print '(/,t10, "Deux stockages en temps interdits en meme temps !!!")'
    identifie_deux_nom = .false.        
  end if
else if (nom1(2:2) == 'F'.or.nom1(1:2) == 'IL')  then
  flag_f = .true.
  nom_f = nom1
  if (nom2(2:2) == 'T') then
    flag_t = .true. 
    nom_t = nom2
  else if (nom2(2:2) == 'F'.or.nom2(1:2) == 'IL') then
    print '(/,t10, "Deux stockages en frequence interdits en meme temps !!!")'
    identifie_deux_nom = .false.        
  end if    
else
  print '(/,t10, "Nom du type de stockage erroné !!!")'
  identifie_deux_nom = .false.
end if
 
end function identifie_deux_nom

! ==========================================================
! == Function liste_temps_freq_pointe(ltf,chaine)  ==
! == ltf : liste de temps_frequence                       ==
! == chaine : character(LEN=LEN_MOT_CLE)                  ==
! ==                                                      ==
! == Retourne un pointeur pointant sur le                 ==
! == st_temps_frequence de nom "chaine"                   ==
! ==========================================================
function liste_temps_freq_pointe(ltf,chaine) result(tf)

type(st_liste_temps_freq), intent(inout) :: ltf
character(LEN=LEN_MOT_CLE), intent(in) :: chaine

type(st_temps_frequence), pointer :: tf
integer :: i

nullify(tf)
!print '("   chaine_liste_temps_freq_pointe = ",A)',chaine
do i=1,ltf%n_tf
  if (chaine==ltf%tf(i)%nom) then
    tf => ltf%tf(i)
!    print '("   trouve_liste_temps_freq_pointe, nu = ",I2)',i
    return
  endif
enddo
print '(" Avertissement, la base de temps ou de fréquence de stockage est erronée !!!  : --> ",A)',TRIM(chaine)

end function liste_temps_freq_pointe


! ==========================================================
! == Subroutine liste_temps_freq_affichage           ==
! == ltf : une st_liste_temps_freq                   ==
! ==                                                      ==
! == Affiche les carac. d'une st_liste_temps_freq    ==
! ==========================================================
subroutine liste_temps_freq_affichage(ltf)

implicit none

type(st_liste_temps_freq), target, intent(in) :: ltf

type(st_temps_frequence), pointer :: tf
integer :: i

if (rang/=RANG_MAITRE_AFF) return
print '(/,"Listes et echantillons pour calcul frequentiel:")'
do i=1,ltf%n_tf
  tf => ltf%tf(i)
  if (ltf%tf(i)%itype==INTERVALLE_FREQUENTIEL.or.ltf%tf(i)%itype==LISTE_FREQUENTIELLE.or. &
      ltf%tf(i)%itype==INTERVALLE_LOGFREQ) cycle
  call temps_frequence_affichage(ltf%tf(i))

enddo

print '(/,"Listes et echantillons temporels pour stockage:")'
do i=1,ltf%n_tf
  if (ltf%tf(i)%itype==INTERVALLE_TEMPOREL.or.ltf%tf(i)%itype==LISTE_TEMPORELLE) cycle
  call temps_frequence_affichage(ltf%tf(i))
enddo

end subroutine liste_temps_freq_affichage


! ==========================================================
! == Subroutine temps_frequence_calcul(ltf,nuiter)        ==
! == tf : st_temps_frequence                              ==
! == nuliter : numero d'iteration                         ==
! ==                                                      ==
! == Calcul de exp(i2pift)                                ==
! == Creation des variables de coordonnees dans le/les    ==
! == fichier(s) de sortie                                 ==
! ==========================================================
subroutine liste_temps_freq_precalcul(ltf,fs)

implicit none

type(st_liste_temps_freq), intent(inout) :: ltf
type(st_fichier_sortie), intent(in) :: fs

integer  :: i, status
integer, dimension(2) :: liste

do i=1, ltf%n_tf
  call temps_frequence_precalcul(ltf%tf(i),fs)
end do

end subroutine liste_temps_freq_precalcul

! ==========================================================
! == Subroutine temps_frequence_calcul(ltf,nuiter)        ==
! == tf : st_temps_frequence                              ==
! == nuliter : numero d'iteration                         ==
! ==                                                      ==
! == Calcul de exp(i2pift)                                ==
! ==========================================================
subroutine liste_temps_freq_calcul(ltf,nuiter)

type(st_liste_temps_freq), intent(inout) :: ltf
integer, intent(in) :: nuiter

type(st_temps_frequence), pointer :: tf
integer  :: i,j

do i=1, ltf%n_tf
  tf => ltf%tf(i)

  tf%flag_calcul = .false.
  !-- Cas des calculs en temps --
  if ((tf%itype == INTERVALLE_TEMPOREL) .or.(tf%itype == LISTE_TEMPORELLE)) then
    ! -- Active flag_calcul si l'instant dans iliste correspond --
    ! -- au n° d'iteration temporelle --
    if (tf%iliste(tf%curseur+1) == nuiter-1) then
      tf%curseur = MIN(tf%curseur +1,tf%n_liste)
      tf%tencours=tf%liste(tf%curseur)
      if (tf%filtre%type_filtre==TF_FILTRE_EXP) then
        tf%filtre%c_t = exp(-tf%tencours/tf%filtre%tau)
      !  print '("tf%filtre%c_t=",ES15.5)',tf%filtre%c_t
      end if
      tf%flag_calcul = .true.
    end if
    
  !-- Cas des calculs en frequence --
  else
    ! -- Active flag_calcul si l'échantillon de l'iteration temporel --
    ! -- est à retenir --
    if (MOD(nuiter-1,tf%isaut) == 0) then
      tf%flag_calcul = .true.
      tf%curseur = tf%curseur +1
      tf%expi2pitf(:) =  tf%expi2pitf(:) * tf%expi2pidtf(:)
      tf%expi2pitf_h(:) =  tf%expi2pitf_h(:) * tf%expi2pidtf(:)
      !write (1000,'(I8,200ES15.7E2)') tf%curseur, real(tf%expi2pitf(:)),aimag(tf%expi2pitf(:))
    end if
  end if 
end do

end subroutine liste_temps_freq_calcul


! ==========================================================
! ==========================================================
! == Traitement de st_temps_frequence                     ==
! ==========================================================
! ==========================================================


! ==========================================================
! == Subroutine temps_frequence_precalcul(tf,fs)          ==
! == tf : st_temps_frequence                              ==
! == fs : st_fichier_sortie                               ==
! ==                                                      ==
! == Creation des variables de coordonnees dans le/les    ==
! == fichier(s) de sortie                                 ==
! ==========================================================
subroutine temps_frequence_precalcul(tf,fs)

type(st_temps_frequence), intent(inout) :: tf
type(st_fichier_sortie), intent(in) :: fs

! -- Ecriture des variables de coordonnees dans le/les fichier(s) --
if (tf%flag_ecrit_base.and. (rang==RANG_MAITRE_AFF)) then
  call temps_frequence_ascii_precalcul(tf,fs)
endif

end subroutine temps_frequence_precalcul


! ==========================================================
! == Subroutine temps_frequence_affichage                 ==
! == tf : st_temps_frequence                              ==
! ==                                                      ==
! == Affichage des carac. d'un st_temps_frequence         ==
! ==========================================================
subroutine temps_frequence_affichage(tf)

type(st_temps_frequence), intent(in) :: tf
character (LEN=LEN_MOT_CLE), dimension(3) :: ch_coord
character (LEN=LEN_MOT_CLE) :: ch1, ch2, ch3
character (LEN=LEN_CHAINE), dimension(3) :: chaine

integer :: i, j,jmax, k, l, m

l = 3
m = 4

if (tf%itype==INTERVALLE_TEMPOREL) then
  print '(t5,"Intervalle temporel ", A, " : Tmin=",EN12.3," ; Tmax=",EN12.3," ; Tpas=",EN12.3)',&
                               TRIM(tf%nom),tf%deb, tf%fin, tf%pas
  ch_coord(1) = entier_chaine(tf%ideb)
  ch_coord(2) = entier_chaine(tf%ifin)
  ch_coord(3) = entier_chaine(tf%ipas)
  print '(t10,"  Itération : ITMIN= ", A, " ; ITMAX= ", A, " ; ITPAS= ", A)', &
    TRIM(ch_coord(1)), TRIM(ch_coord(2)), TRIM(ch_coord(3))
elseif (tf%itype==LISTE_TEMPORELLE) then
  print '(t5,"Liste temporelle ", A, " :")', TRIM(tf%nom)
  print '(t10,"Dates (iterations correspondantes) :")'
  k = maxval(tf%iliste)
  k = len_int(k)
  j = 1
  do
    if (j>tf%n_liste) exit
    chaine = ' '
    jmax = min(j+l-1,tf%n_liste)
    do i=j,jmax
      chaine(i-j+1) = TRIM(entier_chaine(tf%iliste(i)))
    end do
    print '(t15,5(e12.3," s (", A, ")"))', & 
      (tf%liste(i), TRIM(chaine(i-j+1)), i=j,jmax)
    j = j + l
  enddo
elseif (tf%itype==INTERVALLE_FREQUENTIEL) then
  print '(T5,"Bande frequentielle ", A," : Fmin=",EN12.3," ; Fmax=",EN12.3," ; Fpas=",EN12.3, " ; Nb éch=",I6)',&
               TRIM(tf%nom),tf%liste(1),tf%liste(tf%n_liste),tf%liste(2)-tf%liste(2),tf%n_liste
  print '(t10,"échantillonage tous les dt * (isaut =",i3,") ; fe = ", EN11.3, " Hz")', &
                     tf%isaut, tf%fe
elseif (tf%itype==INTERVALLE_LOGFREQ) then
  print '(t5,"Intervalle log frequentielle ", A," : Fmin=",EN12.3," ; Fmax=",EN12.3," ; Fpas=",EN12.3, " ; Nb éch=",I6)',&
               TRIM(tf%nom),tf%liste(1),tf%liste(tf%n_liste),tf%liste(2)-tf%liste(2),tf%n_liste
	       
  print '(t10,"échantillonage tous les dt * (isaut =",i3,") ; fe = ", EN11.3, " Hz")', &
                     tf%isaut, tf%fe
elseif (tf%itype==LISTE_FREQUENTIELLE) then
  print '(t5,"Liste de frequences ", A, " :")', TRIM(tf%nom)
  print '(t10,i4," frequences ; fmin = ", EN11.3, " Hz et fmax = ", EN11.3, " Hz")', &
         tf%n_liste, tf%liste(1),tf%liste(tf%n_liste)
  print '(t10,"échantillonage tous les dt * (isaut =",i3,") ; fe = ", EN11.3, " Hz")', &
                     tf%isaut, tf%fe
endif  

end subroutine temps_frequence_affichage


! ============================================================
! == Subroutine temps_frequence_ascii_init(tf,fs) ==
! == tf : st_temps_frequence                                ==
! == fs : st_fichier_sortie                              ==
! ==                                                        ==
! == Creation du nom de fichier et son ouverture            ==
! ============================================================
subroutine temps_frequence_ascii_init(tf,fs)

type(st_temps_frequence), intent(inout) :: tf
type(st_fichier_sortie), intent(in) :: fs

character(LEN=LEN_MOT_CLE) :: nom_liste

nom_liste = TRIM(tf%nom) // TRIM(fs%extension)

tf%id_liste = ouvre_en_ecriture(nom_liste)

end subroutine temps_frequence_ascii_init


! ============================================================
! == Subroutine temps_frequence_ascii_precalcul(tf,fs)      ==
! == tf : st_temps_frequence                                ==
! == fs : st_fichier_sortie                              ==
! ==                                                        ==
! == Ecriture de tf%liste dans tf%id_liste                  ==
! ============================================================
subroutine temps_frequence_ascii_precalcul(tf,fs)

type(st_temps_frequence), intent(inout) :: tf
type(st_fichier_sortie), intent(in) :: fs

integer :: i

write(tf%id_liste,'('//TRIM(fs%mantisse_reel)//')') (tf%liste(i), i=1,tf%n_liste)
close(tf%id_liste)

end subroutine temps_frequence_ascii_precalcul


! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module m_temps_frequence

