!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_verif_data.f90
!// Purpose:     vérification de l'intégrité des coordonnées
!// Author:      Christophe Guiffaut
!// Modified by:  
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Novembre 2001
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!================================================================
!== Controle des données ...
!================================================================
module m_verif_data

use m_constantes, only : LEN_MOT_CLE, PC1
implicit none

interface verif_pt_ds_volume
  module procedure  verif_pt_ds_volume_entier,verif_pt_ds_volume_reel
end interface
interface verif_fil_ds_volume
  module procedure  verif_fil_ds_volume_entier,verif_fil_ds_volume_reel
end interface

contains

! ==========================================================
! == Subroutine verif_surface                             ==
! == Verifie que les six entiers forment une surface      ==
! == Déduit le type de plan      			  ==
! ==========================================================
logical function verif_surface(nom,coord6,type_plan)

character(LEN=LEN_MOT_CLE), intent(in) :: nom

integer, dimension(1:6), intent(in)    :: coord6
integer, intent(out)  :: type_plan

integer  :: i,j

verif_surface = .true.
type_plan = -1

! vérifie que la surface en est une : coordonnées égales dans 1 direction
j=0
do i=1,3
 if (coord6(i*2-1) == coord6(i*2)) then
   j = j+1
   type_plan = PC1(i)   
 end if
end do
if (j==0) then
  verif_surface = .false.
  print '(/,t10,"ATTENTION, ", A, " ne represente pas une surface !!!")', TRIM(nom)
  return
endif

! vérifie que les coordonnées sont correctes (imax>imin)
do i=1,3
 if (coord6(i*2) < coord6(i*2-1)) then
   verif_surface = .false.
   print '(/,t10,"Veillez à ce que indices de debut < indices de fin  !!!")'
   return 
 end if
end do
end function verif_surface



! ==========================================================
! == Subroutine verif_pt_ds_volume_reel                        ==
! == Verifie le point se trouvent dans le volume          ==
! ==========================================================
logical function verif_pt_ds_volume_reel(nom,coord6,coord3)

character(LEN=*), intent(in) :: nom

real, dimension(1:6), intent(in)    :: coord6
real, dimension(1:3), intent(in)    :: coord3

integer  :: i

verif_pt_ds_volume_reel = .true.

do i=1,3
 if ((coord6(i*2-1) > coord3(i)) .or. &
     (coord6(i*2)   < coord3(i))) then
    print '(/,t10,"Erreur, le point (", A, ") est en dehors du volume spécifié !!!")', TRIM(nom)
    print '(/,t10,"  coordonnées =",3ES15.6)', coord3
    verif_pt_ds_volume_reel = .false.
    return
 end if
end do

end function verif_pt_ds_volume_reel

! ==========================================================
! == Subroutine verif_pt_ds_volume_entier                 ==
! == Verifie le point se trouvent dans le volume          ==
! ==========================================================
logical function verif_pt_ds_volume_entier(nom,coord6,coord3)

character(LEN=*), intent(in) :: nom

integer, dimension(1:6), intent(in)    :: coord6
integer, dimension(1:3), intent(in)    :: coord3

integer  :: i

verif_pt_ds_volume_entier = .true.

do i=1,3
 if ((coord6(i*2-1) > coord3(i)) .or. &
     (coord6(i*2)   < coord3(i))) then
    print '(/,t10,"Erreur, le point (", A, ") est en dehors du volume spécifié !!!")', TRIM(nom)
    print '(/,t10,"  Coordonnées en nbre de cellules =",3I6)', coord3
    verif_pt_ds_volume_entier = .false.
    return
 end if
end do

end function verif_pt_ds_volume_entier

! ==========================================================
! == Subroutine verif_fil_ds_volume_reel                  ==
! == Verifie si le fil se trouve dans le volume           ==
! ==========================================================
logical function verif_fil_ds_volume_reel(nom,coord6,coordfil)

character(LEN=*), intent(in) :: nom

real, dimension(1:6), intent(in)    :: coord6
real, dimension(1:6), intent(in)    :: coordfil

integer  :: i

verif_fil_ds_volume_reel = .true.

do i=1,3
 if ((coord6(i*2-1) > coordfil(i*2-1)) .or. &
     (coord6(i*2)   < coordfil(i*2-1)) .or. &
     (coord6(i*2-1) > coordfil(i*2  )) .or. &
     (coord6(i*2)   < coordfil(i*2  )) ) then
    print '(/,t10,"Erreur, le fil (", A, ") est en dehors du volume spécifié !!!")', TRIM(nom)
    print '(/,t10,"  coordonnées =",6ES15.6)', coordfil
    verif_fil_ds_volume_reel = .false.
    return
 end if
end do

end function verif_fil_ds_volume_reel

! ==========================================================
! == Subroutine verif_fil_ds_volume_entier                ==
! == Verifie si le fil se trouve dans le volume           ==
! ==========================================================
logical function verif_fil_ds_volume_entier(nom,coord6,coordfil)

character(LEN=*), intent(in) :: nom

integer, dimension(1:6), intent(in)    :: coord6
integer, dimension(1:6), intent(in)    :: coordfil

integer  :: i

verif_fil_ds_volume_entier = .true.

do i=1,3
 if ((coord6(i*2-1) > coordfil(i*2-1)) .or. &
     (coord6(i*2)   < coordfil(i*2-1)) .or. &
     (coord6(i*2-1) > coordfil(i*2  )) .or. &
     (coord6(i*2)   < coordfil(i*2  )) ) then
    print '(/,t10,"Erreur, le fil (", A, ") est en dehors du volume spécifié !!!")', TRIM(nom)
    print '(/,t10,"  coordonnées =",6I6)', coordfil
    verif_fil_ds_volume_entier = .false.
    return
 end if
end do

end function verif_fil_ds_volume_entier

end module m_verif_data
