!////////////////////////////////////////////////////////////////////////
!// Name:        m_zone_angulaire.f90
!// Purpose:     Gestion de zones angulaires  ==
!// Author:      Christophe Guiffaut
!// Modified by:
!// Created  :     Aout 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Position frequenciel ou temporel des stockages       ==
! ==========================================================

module m_zone_angulaire

use m_constantes

use m_constantes_scientifiques, only : PI

use m_entree, only : st_fichier, &
                                 lineread, &
				 ligne_precedente, &
				 lecture_stop, &
				 len_int, &
				 nb_mot_chaine, &
				 unitefichier, &
				 multibackspace, &
				 numbackspace_set, &
				 ctrl_nb_mot_chaine
use m_sortie, only : ouvre_en_ecriture

implicit none
integer, parameter   :: INTERVALLE_PLAN_THETA = 1
integer, parameter   :: INTERVALLE_PLAN_PHI = 2
integer, parameter   :: LISTE_PLAN_THETA = 3
integer, parameter   :: LISTE_PLAN_PHI = 4
integer, parameter   :: ZONE_ANGULAIRE_3D = 5
integer, parameter   :: DEB = 1
integer, parameter   :: FIN = 2
integer, parameter   :: PAS = 3
integer, parameter   :: THETA = 1
integer, parameter   :: PHI = 2
 
! ==========================================================
! == Definition du type st_temps_frequence                ==
! ==========================================================
type st_une_zone
  ! -- nom : nom de la zone angulaire --
  character(LEN=LEN_MOT_CLE) :: nom
  ! -- type de la zone angulaire --
  integer  :: type_zone
  integer  :: nbangle_theta, nbangle_phi
  integer  :: nbcoupleangle
  
  real     :: anglemin,anglemax,anglepas
  real     :: anglemin2,anglemax2,anglepas2
  ! -- liste des angles --
  real, dimension(:), pointer :: liste_angle_theta
  real, dimension(:), pointer :: liste_angle_phi
  real, dimension(:,:), pointer :: liste_couple_angle
  real, dimension(:,:), pointer :: liste_couple_angle_degre
  
end type st_une_zone

! -- Liste des st_temps_frequence --
type st_zone_angulaire
  ! -- nza : nombre de zones angulaires --
  integer :: nza
  ! -- tf : tableau de temps frequence --
  type(st_une_zone), dimension(:), pointer :: l_za
end type  st_zone_angulaire

contains

! ==========================================================
! == Subroutine zone_angulaire_initdefaut                 ==
! ==========================================================
subroutine zone_angulaire_initdefaut(tza)

implicit none

type(st_zone_angulaire), intent(inout) :: tza

nullify(tza%l_za)
tza%nza = 0
end subroutine zone_angulaire_initdefaut


! ==========================================================
! == Subroutine zone_angulaire_lecture                    ==
! == f : fichier d'entree                                 ==
! Lecture des données dans le fichier d'entrée
! ==========================================================
subroutine zone_angulaire_lecture(f,tza)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_zone_angulaire), intent(inout) :: tza


!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nom, premiermot
integer            :: nbangle
real, dimension(3) :: angle, angle2
real, allocatable, dimension(:) :: val
integer            :: i,j, i_za, ind, status
type(st_une_zone), pointer  :: za

!===   Début des traitements   ===

! -- Positionnement de numbackspace a 0 --
! -- numbackspace : nombre de sauts de ligne --
! -- vers le haut a effectuer lors d'un multibackspace() --
call numbackspace_set(f,0)
! -- ON COMPTE LE NOMBRE DE ZONES ANGULAIRES ... --
i_za = 0
do
  premiermot = lineread(f,1)
  if (premiermot(1:3) == 'ITH' .or. premiermot(1:3) == 'IPH' .or. &
      premiermot(1:3) == 'LTH' .or. premiermot(1:3) == 'LPH' .or. &
      premiermot(1:3) == 'ITP')  then
      
      i_za = i_za +1
      
  else if (premiermot==f%nomfinbloc) then
    exit
  endif
enddo

! -- Allocation memoire pour toutes les zones --
if (i_za > 0) then
  tza%nza = i_za
  allocate(tza%l_za(i_za))
else
  print *,'Avertissement: pas de zones angulaires définies dans le bloc correspondant'
  return
end if
! -- Retour vers la premiere ligne du bloc --
i = multibackspace(f)

i_za = 0
do
  ! -- On recommence la lecture pour les données --
  premiermot = lineread(f)
  select case(premiermot(1:3))
    case('ITH')
      i_za = i_za + 1
      za =>tza%l_za(i_za)
      za%nbangle_phi = nb_mot_chaine(f%chaine) - 4
      allocate(za%liste_angle_phi(1:za%nbangle_phi))
      
      read(f%chaine,*,err=999) za%nom,angle(1:3),za%liste_angle_phi(:)
      za%anglemin = angle(1) ; za%anglemax = angle(2)
      za%anglepas = angle(3)
      ! Liste des angles theta
      za%nbangle_theta =  NINT((angle(FIN) - angle(DEB)) / angle(PAS))+1
      allocate(za%liste_angle_theta(1:za%nbangle_theta))
      do i = 1,  za%nbangle_theta
        za%liste_angle_theta(i) = angle(DEB) + (i-1) * angle(PAS)
      end do       
      
      ! Création de la liste de couples d'angle (theta,phi)
      nbangle = za%nbangle_theta * za%nbangle_phi
      za%nbcoupleangle = nbangle
      allocate(za%liste_couple_angle(2,nbangle))
      do i = 1,  za%nbangle_phi
        za%liste_couple_angle(THETA,(i-1)*za%nbangle_theta+1:i*za%nbangle_theta) = &
	       za%liste_angle_theta(:)        
        za%liste_couple_angle(PHI,(i-1)*za%nbangle_theta+1:i*za%nbangle_theta) = &
	       za%liste_angle_phi(i)        
      end do
      za%type_zone = INTERVALLE_PLAN_THETA
      cycle
    case('IPH')
      i_za = i_za + 1
      za =>tza%l_za(i_za)
      za%nbangle_theta = nb_mot_chaine(f%chaine) - 4
      allocate(za%liste_angle_theta(1:za%nbangle_theta))
      
      read(f%chaine,*,err=999) za%nom,angle(1:3),za%liste_angle_theta(:)
      za%anglemin = angle(1) ; za%anglemax = angle(2)
      za%anglepas = angle(3)
      
      ! Liste des angles phi
      za%nbangle_phi =  NINT((angle(FIN) - angle(DEB)) / angle(PAS))+1
      allocate(za%liste_angle_phi(1:za%nbangle_phi))
      do i = 1,  za%nbangle_phi
        za%liste_angle_phi(i) = angle(DEB) + (i-1) * angle(PAS)
      end do       
      
      ! Création de la liste de couples d'angle (theta,phi)
      nbangle = za%nbangle_theta * za%nbangle_phi
      za%nbcoupleangle = nbangle
      allocate(za%liste_couple_angle(2,nbangle))
      do i = 1,  za%nbangle_theta
        za%liste_couple_angle(THETA,(i-1)*za%nbangle_phi+1:i*za%nbangle_phi) = &
	       za%liste_angle_theta(i)        
        za%liste_couple_angle(PHI,(i-1)*za%nbangle_phi+1:i*za%nbangle_phi) = &
	       za%liste_angle_phi(:)        
      end do
      za%type_zone = INTERVALLE_PLAN_PHI


      cycle
    case('LTH')
      i_za = i_za + 1
      za =>tza%l_za(i_za)

      za%nbangle_theta = nb_mot_chaine(f%chaine) - 2
      za%nbangle_phi = 1
      allocate(za%liste_angle_theta(1:za%nbangle_theta))
      ! un seul angle phi
      allocate(za%liste_angle_phi(za%nbangle_phi))
      read(f%chaine,*,err=999) za%nom,za%liste_angle_phi(1),za%liste_angle_theta(:)
      
      ! Création de la liste de couples d'angle (theta,phi)
      za%nbcoupleangle = za%nbangle_theta
      allocate(za%liste_couple_angle(2,za%nbangle_theta))
      za%liste_couple_angle(THETA,:) = za%liste_angle_theta(:)        
      za%liste_couple_angle(PHI,:) = za%liste_angle_phi(1)        
      za%type_zone = LISTE_PLAN_THETA

      cycle
    case('LPH')
      i_za = i_za + 1
      za =>tza%l_za(i_za)

      za%nbangle_phi = nb_mot_chaine(f%chaine) - 2
      za%nbangle_theta = 1
      allocate(za%liste_angle_phi(1:za%nbangle_phi))
      ! un seul angle theta
      allocate(za%liste_angle_theta(za%nbangle_theta))
      read(f%chaine,*,err=999) za%nom,za%liste_angle_theta(1),za%liste_angle_phi(:)
      
      ! Création de la liste de couples d'angle (theta,phi)
      za%nbcoupleangle = za%nbangle_phi
      allocate(za%liste_couple_angle(2,za%nbangle_phi))
      za%liste_couple_angle(THETA,:) = za%liste_angle_theta(1)        
      za%liste_couple_angle(PHI,:) = za%liste_angle_phi(:)        
      za%type_zone = LISTE_PLAN_PHI
      
      cycle
    case('ITP')
      i_za = i_za + 1
      za =>tza%l_za(i_za)

      status = ctrl_nb_mot_chaine(f,f%chaine,7)      
      read(f%chaine,*,err=999) za%nom,angle(1:3),angle2(1:3)
      
      za%anglemin = angle(1) ; za%anglemax = angle(2)
      za%anglepas = angle(3)
      za%anglemin2 = angle2(1) ; za%anglemax2 = angle2(2)
      za%anglepas2 = angle2(3)
      ! Liste des angles theta
      za%nbangle_theta =  NINT((angle(FIN) - angle(DEB))   / angle(PAS))+1
      za%nbangle_phi   =  NINT((angle2(FIN) - angle2(DEB)) / angle2(PAS))+1
      allocate(za%liste_angle_theta(1:za%nbangle_theta))
      allocate(za%liste_angle_phi(1:za%nbangle_phi))
      do i = 1,  za%nbangle_theta
        za%liste_angle_theta(i) = angle(DEB) + (i-1) * angle(PAS)
      end do       
      do i = 1,  za%nbangle_phi
        za%liste_angle_phi(i) = angle2(DEB) + (i-1) * angle2(PAS)
      end do       
      
      za%type_zone = ZONE_ANGULAIRE_3D

      ! Création de la liste de couples d'angle (theta,phi)
      nbangle = za%nbangle_theta * za%nbangle_phi
      za%nbcoupleangle = nbangle
      allocate(za%liste_couple_angle(2,nbangle))
      ind = 0
      do j = 1,  za%nbangle_phi
      do i = 1,  za%nbangle_theta
        ind = ind +1
        za%liste_couple_angle(THETA,ind) = za%liste_angle_theta(i)        
        za%liste_couple_angle(PHI  ,ind) = za%liste_angle_phi(j)        
      end do; end do

      cycle    
  end select   
  
  select case(premiermot)
    case default
      if (premiermot==f%nomfinbloc) then
	exit
      endif
      ! -- Si ca se passe pas bien : arret de la lecture --
      print *,'Erreur, Mot clé non reconnu :'
      call lecture_stop(f)

  end select
enddo

return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(f)

end subroutine zone_angulaire_lecture


! ==========================================================
! == Subroutine zone_angulaire_ecriture                   ==
! ==========================================================
subroutine zone_angulaire_ecriture(f,tza)

type(st_fichier), intent(inout) :: f
type(st_zone_angulaire), intent(inout) :: tza

integer :: i,j
type(st_une_zone), pointer  :: za


do i=1,tza%nza
  za =>tza%l_za(i)

  !----------------------------------------------------------------------------
  if (za%type_zone==INTERVALLE_PLAN_THETA) then
!    write(f%unite,'(T3,A,1x,3F9.3,1x,<za%nbangle_phi>F9.3)')  trim(za%nom),za%anglemin,&
    write(f%unite,'(T3,A,1x,3F9.3,1x,50F9.3)')  trim(za%nom),za%anglemin,&
                 za%anglemax,za%anglepas,za%liste_angle_phi(1:za%nbangle_phi)
                                       
  !----------------------------------------------------------------------------
  else if (za%type_zone==INTERVALLE_PLAN_PHI) then
!    write(f%unite,'(T3,A,1x,3F9.3,1x,<za%nbangle_theta>F9.3)')  trim(za%nom),za%anglemin,&
    write(f%unite,'(T3,A,1x,3F9.3,1x,50F9.3)')  trim(za%nom),za%anglemin,&
                 za%anglemax,za%anglepas,za%liste_angle_theta(1:za%nbangle_theta)
		 
  !----------------------------------------------------------------------------
  else if (za%type_zone==LISTE_PLAN_THETA) then
!    write(f%unite,'(T3,A,1x,F9.3,1x,<za%nbangle_theta>F9.3)') trim(za%nom), &
    write(f%unite,'(T3,A,1x,F9.3,1x,50F9.3)') trim(za%nom), &
              za%liste_angle_phi(1),za%liste_angle_theta(1:za%nbangle_theta)
	      
  !----------------------------------------------------------------------------
  else if (za%type_zone==LISTE_PLAN_PHI) then
!    write(f%unite,'(T3,A,1x,F9.3,1x,<za%nbangle_phi>F9.3)') trim(za%nom), &
    write(f%unite,'(T3,A,1x,F9.3,1x,50F9.3)') trim(za%nom), &
              za%liste_angle_theta(1),za%liste_angle_phi(1:za%nbangle_phi)
	      
  !----------------------------------------------------------------------------
  else if (za%type_zone==ZONE_ANGULAIRE_3D) then
    write(f%unite,'(T3,A,1x,3F9.3,1x,3F9.3)') trim(za%nom),za%anglemin,za%anglemax,za%anglepas,&
                    za%anglemin2,za%anglemax2,za%anglepas2
    
  end if
end do


end subroutine zone_angulaire_ecriture

! ==========================================================
! == Subroutine zone_angulaire_init             ==
! ==========================================================
subroutine zone_angulaire_init(tza)

implicit none

!=== Déclaration des arguments ===
type(st_zone_angulaire), intent(inout) :: tza
!=== Déclaration des variables ===
integer  :: i
type(st_une_zone), pointer  :: za
!=== Déclaration des traitements ===

do i = 1, tza%nza
  za => tza%l_za(i)
  
  ! Conversion des angles en radian
  za%liste_angle_theta  = za%liste_angle_theta  * PI/180.0
  za%liste_angle_phi    = za%liste_angle_phi    * PI/180.0
  
  allocate(za%liste_couple_angle_degre(2,za%nbcoupleangle))
  za%liste_couple_angle_degre(:,:) = za%liste_couple_angle(:,:)
  za%liste_couple_angle = za%liste_couple_angle * PI/180.0
end do

end subroutine zone_angulaire_init


! ==========================================================
! == Function zone_angulaire_pointeur(tza,chaine)         ==
! == Retourne un pointeur pointant sur le                 ==
! == st_zone_angulaire de nom "chaine"                    ==
! ==========================================================
function zone_angulaire_pointe(tza,chaine) result(za)

type(st_zone_angulaire), intent(inout) :: tza
character(LEN=LEN_MOT_CLE), intent(in) :: chaine

type(st_une_zone), pointer :: za
integer :: i

nullify(za)
do i=1,tza%nza
  if (chaine==tza%l_za(i)%nom) then
    za => tza%l_za(i)
    exit
  endif
enddo

end function zone_angulaire_pointe

! ==========================================================
! == Subroutine zone_angulaire_deallocate                 ==
! ==========================================================
subroutine zone_angulaire_deallocate(tza)

!=== Déclaration des arguments ===
type(st_zone_angulaire), intent(inout) :: tza
!=== Déclaration des variables ===
integer  :: i
type(st_une_zone), pointer  :: za
!=== Déclaration des traitements ===

do i = 1, tza%nza
  za => tza%l_za(i)
  deallocate(za%liste_angle_theta, za%liste_angle_phi)
  deallocate(za%liste_couple_angle)
  deallocate(za%liste_couple_angle_degre)
end do
if (tza%nza>0) deallocate(tza%l_za)

end subroutine zone_angulaire_deallocate


end module m_zone_angulaire


