!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_champ.f90
!// Purpose:     Calcul des composantes du champ
!// Author:      Cyril Giraudon, Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

module mfdtd_champ

use m_constantes
use m_constantes_scientifiques, only : EPSILON_0, MU_0
use mfdtd_volume, only : st_volume 
use mfdtd_structure, only : st_structure,structure_matqcq_face
implicit none

! ==========================================================
! == Declaration de type st_champ                         ==
! ==========================================================
type st_champ
  ! type de milieu dans le volume, l'information existe dans 
  ! le module structure
  integer :: type_milieu_dielec
  integer :: type_milieu_magnet
  
  integer :: idex, idey, idez	 ! borne début suivant x pour le champ E
  integer :: idhx, idhy, idhz	 ! borne début suivant x pour le champ H
  integer :: ifex, ifey, ifez	 ! borne fin suivant x pour le champ E
  integer :: ifhx, ifhy, ifhz	 ! borne fin suivant x pour le champ H

  integer :: jdex, jdey, jdez	 ! borne début suivant y pour le champ E
  integer :: jdhx, jdhy, jdhz	 ! borne début suivant y pour le champ H
  integer :: jfex, jfey, jfez	 ! borne fin suivant y pour le champ E
  integer :: jfhx, jfhy, jfhz	 ! borne fin suivant y pour le champ H

  integer :: kdex, kdey, kdez	 ! borne début suivant y pour le champ E
  integer :: kdhx, kdhy, kdhz	 ! borne début suivant y pour le champ H
  integer :: kfex, kfey, kfez	 ! borne fin suivant y pour le champ E
  integer :: kfhx, kfhy, kfhz	 ! borne fin suivant y pour le champ H
  
  ! borne sous une forme plus réutilisable et modifiable
  integer, dimension(1:6) :: borne_ex, borne_ey, borne_ez
  integer, dimension(1:6) :: borne_hx, borne_hy, borne_hz

  real, dimension(:,:,:), pointer :: ex, ey, ez, hx, hy, hz  ! composantes du champ

  !--------  Tableau 2D supplémentaire pour MPI ---------------
  real, dimension(:,:), pointer :: ex_ysup,ex_zsup, ey_xsup, ey_zsup, ez_xsup,ez_ysup
  
  !-------- Tableau de coefficients multiplicatifs des champs --
  real, dimension(:,:,:), pointer :: cex1, cey1, cez1      ! coefficients pour le champ E
  real, dimension(:,:,:), pointer :: cex2, cey2, cez2      ! coefficients pour le champ E
  
  real, dimension(:,:,:), pointer :: chx1, chy1, chz1      ! coefficients pour le champ H
  real, dimension(:,:,:), pointer :: chx2, chy2, chz2      ! coefficients pour le champ H

  logical :: flag_ce1 , flag_ce2 ! Existence des cex, cey, cez
  logical :: flag_ch1 , flag_ch2 ! Existence des chx, chy, chz

  real,dimension(3) ::  dxyz ! les pas spatiaux, copie de ceux du module "mfdtd_volume"
  
  real ::  cer1 ! Coefficients pour E, pour un mulieu homogene a pertes homogenes
  real ::  chr1 ! Coefficients pour H, pour un mulieu homogene a pertes homogenes

  logical :: flag_cer1, flag_cer2 ! Existence des cerx, cery, cerz
  logical :: flag_chr1, flag_chr2 ! Existence des chrx, chry, chrz

  logical :: flag_dielectrique_homogene
  logical :: flag_diel_pertes_homogene
  logical :: flag_dielectrique_inhomogene    
  logical :: flag_dielectrique_quelconque

  logical :: flag_magnetique_homogene
  logical :: flag_magnet_pertes_homogene
  logical :: flag_magnetique_inhomogene
  logical :: flag_magnetique_quelconque

  real, dimension(:), pointer ::     dt_epsodx,     dt_epsody,     dt_epsodz
  real, dimension(:), pointer :: cer2dt_epsodx, cer2dt_epsody, cer2dt_epsodz
  real, dimension(:), pointer ::     dt_muodxo,     dt_muodyo,     dt_muodzo
  real, dimension(:), pointer :: chr2dt_muodxo, chr2dt_muodyo, chr2dt_muodzo
  

  !-------- Taille mémoire et Nbre d'opération par seconde ----
  real      :: taille_mem
  real      :: nb_oper
  integer   :: nb_cell_tot
end type st_champ
 
contains

! ==========================================================
! == Subroutine champ_initdefaut                          ==
! == Initialisation par défaut des variables contenant    ==
! == les parametres definis par l'utilisateur dans le     ==
! == fichier d'entree                                     ==
! ==========================================================	
subroutine champ_initdefaut(t_champ)

type(st_champ), intent(inout) :: t_champ

t_champ%flag_ce1 = .false.  ! Pas de pertes electriques inhomogenes
t_champ%flag_ce2 = .false.  ! Pas de dielectriques inhomogenes
t_champ%flag_ch1 = .false.  ! Pas de pertes magnetiques inhomogenes
t_champ%flag_ch2 = .false.  ! Pas de magnetiques inhomogenes
t_champ%flag_cer1 = .false. ! Pas de pertes electriques homogenes
t_champ%flag_cer2 = .false. ! Pas de dielectriques homogenes
t_champ%flag_chr1 = .false. ! Pas de pertes magnetiques homogenes
t_champ%flag_chr2 = .false. ! pas de magnetiques homogenes

t_champ%flag_diel_pertes_homogene = .false.
t_champ%flag_dielectrique_inhomogene = .false.
t_champ%flag_dielectrique_quelconque = .false.
t_champ%flag_magnet_pertes_homogene = .false.
t_champ%flag_magnetique_inhomogene = .false.
t_champ%flag_magnetique_quelconque = .false.

t_champ%flag_magnetique_homogene = .true. ! Analyse dans le vide magnetique 
t_champ%flag_dielectrique_homogene = .true. ! Analyse dans le vide dielectrique

! redondance de l'information...
t_champ%type_milieu_dielec=VOL_DIELEC_HOMOGENE
t_champ%type_milieu_magnet=VOL_MAGNET_HOMOGENE

t_champ%taille_mem = 0.0
t_champ%nb_oper    = 0.0
t_champ%nb_cell_tot= 0
end subroutine champ_initdefaut


! ==========================================================
! == Subroutine champ_lecture                             ==
! == Lecture dans le fichier d'entrée...                  ==
! ==========================================================
subroutine champ_lecture()

				
end Subroutine champ_lecture


! ==========================================================
! == subroutine champ_init                      ==
! == Initialisation des variables, lorsque qu'il n'y a    ==
! == pas de lecture du fichier d'entrée                   ==
!===========================================================	
subroutine champ_init(c,v,s,face_fontiere)

type(st_champ), intent(inout) :: c
type(st_volume), intent(in) :: v
type(st_structure), intent(in) :: s
integer, dimension(6), intent(in)  :: face_fontiere 

integer  :: xinf, xsup, yinf, ysup, zinf, zsup ! type de frontière

!--> les faces frontières 
xinf = face_fontiere(YOZ_INF)
xsup = face_fontiere(YOZ_SUP)
yinf = face_fontiere(XOZ_INF)
ysup = face_fontiere(XOZ_SUP)
zinf = face_fontiere(XOY_INF)
zsup = face_fontiere(XOY_SUP)

!--> bornes min et max des composantes du champ --
!-->XINF 
c%idex = v%ivold    ; c%idey = v%ivold + 1; c%idez = v%ivold + 1
c%idhx = v%ivold + 1; c%idhy = v%ivold    ; c%idhz = v%ivold
if (xinf==MUR_MAGNETIQUE.or.xinf==SANS_FRONTIERE.or.xinf==FLOQUET.or.xinf==FRONTIERE_MPI) then
  c%idhx = v%ivold; c%idey = v%ivold; c%idez = v%ivold
endif

!-->XSUP 
c%ifex = v%ivolf    ; c%ifey = v%ivolf    ; c%ifez = v%ivolf
c%ifhx = v%ivolf    ; c%ifhy = v%ivolf    ; c%ifhz = v%ivolf
if (xsup==MUR_MAGNETIQUE.or.xsup==SANS_FRONTIERE.or.xsup==FLOQUET) then
  c%ifhx = v%ivolf + 1; c%ifey = v%ivolf + 1; c%ifez = v%ivolf + 1
endif
if (xsup==FRONTIERE_MPI) then
  c%ifhx = v%ivolf + 1
end if

!-->YINF 
c%jdex = v%jvold + 1; c%jdey = v%jvold    ; c%jdez = v%jvold + 1
c%jdhx = v%jvold    ; c%jdhy = v%jvold + 1; c%jdhz = v%jvold
if (yinf==MUR_MAGNETIQUE.or.yinf==SANS_FRONTIERE.or.yinf==FLOQUET.or.yinf==FRONTIERE_MPI) then
  c%jdex = v%jvold; c%jdhy = v%jvold; c%jdez = v%jvold
endif

!-->YSUP
c%jfex = v%jvolf    ; c%jfey = v%jvolf    ; c%jfez = v%jvolf
c%jfhx = v%jvolf    ; c%jfhy = v%jvolf    ; c%jfhz = v%jvolf
if (ysup==MUR_MAGNETIQUE.or.ysup==SANS_FRONTIERE.or.ysup==FLOQUET) then
  c%jfex = v%jvolf + 1; c%jfhy = v%jvolf + 1; c%jfez = v%jvolf + 1
endif
if (ysup==FRONTIERE_MPI) then
  c%jfhy = v%jvolf + 1
end if

!-->ZINF 
c%kdex = v%kvold + 1; c%kdey = v%kvold + 1; c%kdez = v%kvold
c%kdhx = v%kvold    ; c%kdhy = v%kvold    ; c%kdhz = v%kvold + 1
if (zinf==MUR_MAGNETIQUE.or.zinf==SANS_FRONTIERE.or.zinf==FLOQUET.or.zinf==FRONTIERE_MPI) then
  c%kdex = v%kvold; c%kdey = v%kvold; c%kdhz = v%kvold
endif

!-->ZSUP 
c%kfex = v%kvolf    ; c%kfey = v%kvolf    ; c%kfez = v%kvolf
c%kfhx = v%kvolf    ; c%kfhy = v%kvolf    ; c%kfhz = v%kvolf
if (zsup==MUR_MAGNETIQUE.or.zsup==SANS_FRONTIERE.or.zsup==FLOQUET) then
  c%kfex = v%kvolf + 1; c%kfey = v%kvolf + 1; c%kfhz = v%kvolf + 1
endif
if (zsup==FRONTIERE_MPI) then
  c%kfhz = v%kvolf + 1
end if

! mise à jour de tableaux de bornes limites de calcul
! plus facile à réutiliser
call champ_definition_borne(c)

!--> type de milieu remplissant le volume
if (v%sigma_e/=0) then
  ! -- Plus une analyse dans le vide dielectrique --
  c%flag_dielectrique_homogene = .false.
  ! -- Mais une analyse dans le vide dielectrique a pertes homogenes --
  c%flag_diel_pertes_homogene = .true.
endif

if (v%sigma_h/=0) then
  ! -- Plus une analyse dans le vide magnetique --
  c%flag_magnetique_homogene = .false.
  ! -- Mais une analyse dans le vide magnetiques a pertes homogenes --
  c%flag_magnet_pertes_homogene = .true.
  c%type_milieu_magnet=VOL_MAGNET_PERTES_HOMOGENE
endif

c%dxyz=v%dxyz
c%type_milieu_dielec=s%type_milieu_dielec
c%type_milieu_magnet=s%type_milieu_magnet

!--> mémoire et nombre d'opération
call champ_mem_nboper(c,v)

end subroutine champ_init

! ==========================================================
! == Subroutine milieu_volume_mem_nboper                  ==
! ==   Evaluation de la taille mémoire en octets dans la
! ==    structure et du nbre d'opération flottante si impactant
! ==     sur le temps de calcul
!===========================================================	
subroutine champ_mem_nboper(c,v)
type(st_champ), intent(inout)  :: c
type(st_volume), intent(in) :: v

real :: test,mem
real :: opere,operh
test=0.0
mem = (v%ivolf-v%ivold+3)*(v%jvolf-v%jvold+3)*(v%kvolf-v%kvold+3)*kind(test)
c%taille_mem = mem * 6 ! les 6 composantes de champ
c%nb_cell_tot=(v%ivolf-v%ivold+1)*(v%jvolf-v%jvold+1)*(v%kvolf-v%kvold+1)
!--> Ajout de la mémoire des tableaux de coefficients multiplicatifs des champs
select case (c%type_milieu_dielec)
  case(VOL_DIELEC_INHOMOGENE)
    c%taille_mem = c%taille_mem + mem*3
  case(VOL_DIELEC_PERTES_INHOMOGENE)
    c%taille_mem = c%taille_mem + mem*6
end select  

select case (c%type_milieu_magnet)
  case(VOL_MAGNET_INHOMOGENE)
    c%taille_mem = c%taille_mem + mem*3
  case(VOL_MAGNET_PERTES_INHOMOGENE)
    c%taille_mem = c%taille_mem + mem*6
end select  

!--> Calcul du nombre d'opération
opere= 0.0
opere=       product(c%borne_ex(2:6:2)-c%borne_ex(1:5:2)+1)
opere= opere+product(c%borne_ey(2:6:2)-c%borne_ey(1:5:2)+1)
opere= opere+product(c%borne_ez(2:6:2)-c%borne_ez(1:5:2)+1)
operh= 0.0
operh=       product(c%borne_hx(2:6:2)-c%borne_hx(1:5:2)+1)
operh= operh+product(c%borne_hy(2:6:2)-c%borne_hy(1:5:2)+1)
operh= operh+product(c%borne_hz(2:6:2)-c%borne_hz(1:5:2)+1)

select case (c%type_milieu_dielec)
  case(VOL_DIELEC_HOMOGENE)
    opere = opere*6
  case(VOL_DIELEC_PERTES_HOMOGENE)
    opere = opere*7
  case(VOL_DIELEC_INHOMOGENE)
    opere = opere*7
  case(VOL_DIELEC_PERTES_INHOMOGENE)
    opere = opere*8
end select  

select case (c%type_milieu_magnet)
  case(VOL_MAGNET_HOMOGENE)
    operh = operh*6
  case(VOL_MAGNET_PERTES_HOMOGENE)
    operh = operh*7
  case(VOL_MAGNET_INHOMOGENE)
    operh = operh*7
  case(VOL_MAGNET_PERTES_INHOMOGENE)
    operh = operh*8
end select  

c%nb_oper = opere +operh

end subroutine champ_mem_nboper

! ==========================================================
! == subroutine champ_definition_borne                    ==
! == Définition des bornes limites du volume              ==
! ==========================================================	
subroutine champ_definition_borne(c)

type(st_champ), intent(inout) :: c

! Uniquement des affectations, 
! utilisable comme information par d'autre module

c%borne_ex(XMIN) = c%idex ; c%borne_ex(XMAX) = c%ifex ;
c%borne_ex(YMIN) = c%jdex ; c%borne_ex(YMAX) = c%jfex ;
c%borne_ex(ZMIN) = c%kdex ; c%borne_ex(ZMAX) = c%kfex ;

c%borne_ey(XMIN) = c%idey ; c%borne_ey(XMAX) = c%ifey ;
c%borne_ey(YMIN) = c%jdey ; c%borne_ey(YMAX) = c%jfey ;
c%borne_ey(ZMIN) = c%kdey ; c%borne_ey(ZMAX) = c%kfey ;

c%borne_ez(XMIN) = c%idez ; c%borne_ez(XMAX) = c%ifez ;
c%borne_ez(YMIN) = c%jdez ; c%borne_ez(YMAX) = c%jfez ;
c%borne_ez(ZMIN) = c%kdez ; c%borne_ez(ZMAX) = c%kfez ;

c%borne_hx(XMIN) = c%idhx ; c%borne_hx(XMAX) = c%ifhx ;
c%borne_hx(YMIN) = c%jdhx ; c%borne_hx(YMAX) = c%jfhx ;
c%borne_hx(ZMIN) = c%kdhx ; c%borne_hx(ZMAX) = c%kfhx ;

c%borne_hy(XMIN) = c%idhy ; c%borne_hy(XMAX) = c%ifhy ;
c%borne_hy(YMIN) = c%jdhy ; c%borne_hy(YMAX) = c%jfhy ;
c%borne_hy(ZMIN) = c%kdhy ; c%borne_hy(ZMAX) = c%kfhy ;

c%borne_hz(XMIN) = c%idhz ; c%borne_hz(XMAX) = c%ifhz ;
c%borne_hz(YMIN) = c%jdhz ; c%borne_hz(YMAX) = c%jfhz ;
c%borne_hz(ZMIN) = c%kdhz ; c%borne_hz(ZMAX) = c%kfhz ;

print '(T5,"Rang=",I5," ; BORNES pour le calcul de EX :",6I6)',rang,c%borne_ex(1:6)
print '(T5,"Rang=",I5," ; BORNES pour le calcul de EY :",6I6)',rang,c%borne_ey(1:6)
print '(T5,"Rang=",I5," ; BORNES pour le calcul de EZ :",6I6)',rang,c%borne_ez(1:6)
print '(T5,"Rang=",I5," ; BORNES pour le calcul de HX :",6I6)',rang,c%borne_hx(1:6)
print '(T5,"Rang=",I5," ; BORNES pour le calcul de HY :",6I6)',rang,c%borne_hy(1:6)
print '(T5,"Rang=",I5," ; BORNES pour le calcul de HZ :",6I6)',rang,c%borne_hz(1:6)
end subroutine champ_definition_borne


! ==========================================================
! == subroutine champ_precalcul                           ==
! == calculs avant les itérations temporelles             ==
! ==========================================================	
subroutine champ_precalcul(c,v)

type(st_champ), intent(inout) :: c
type(st_volume), intent(inout) :: v

! -- info --
if (c%flag_dielectrique_homogene) &
  print '(T5,"Rang=",I5,"Espace diélectrique homogène sans pertes")',rang
if (c%flag_diel_pertes_homogene) &
  print '(T5,"Rang=",I5,"Espace diélectrique homogène, avec pertes électriques homogènes")',rang
if (c%flag_dielectrique_inhomogene) & 
  print '(T5,"Rang=",I5,"Espace diélectrique inhomogène sans pertes")',rang
if (c%flag_dielectrique_quelconque) & 
  print '(T5,"Rang=",I5,"Espace diélectrique inhomogène avec pertes")',rang
  
if (c%flag_magnetique_homogene) &
  print '(T5,"Rang=",I5,"Espace magnétique homogène sans pertes")',rang
if (c%flag_magnet_pertes_homogene) & 
  print '(T5,"Rang=",I5,"Espace magnétique homogène, avec pertes magnétiques homogènes")',rang
if (c%flag_magnetique_inhomogene) &
  print '(T5,"Rang=",I5,"Espace magnétique inhomogène sans pertes")',rang
if (c%flag_magnetique_quelconque) & 
  print '(T5,"Rang=",I5,"Espace magnétique inhomogène avec pertes")',rang

! -- Mise à jour des flags --
if (c%flag_dielectrique_inhomogene .or.c%flag_dielectrique_quelconque) c%flag_ce2 = .true.
  
if (c%flag_magnetique_inhomogene .or.c%flag_magnetique_quelconque) c%flag_ch2 = .true.

!print '("ivold,ivolf =",2I6)',v%ivold,v%ivolf
!print '("jvold,jvolf =",2I6)',v%jvold,v%jvolf
!print '("kvold,kvolf =",2I6)',v%kvold,v%kvolf

allocate(c%ex(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); c%ex = 0.0
allocate(c%ey(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); c%ey = 0.0
allocate(c%ez(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); c%ez = 0.0
allocate(c%hx(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
allocate(c%hy(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
allocate(c%hz(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1)); 
c%hx = 0.0;
c%hy = 0.0; 
c%hz = 0.0 

! -- Calcul des coefficients multiplicatifs des champs en milieu homogène --
! -- composantes x --
allocate(    c%dt_epsodx(v%ivold:v%ivolf+1), &
         c%cer2dt_epsodx(v%ivold:v%ivolf+1), &
	     c%dt_muodxo(v%ivold:v%ivolf+1), &
	 c%chr2dt_muodxo(v%ivold:v%ivolf+1))

c%dt_epsodx     = v%dt/(EPSILON_0*v%celx)
c%cer2dt_epsodx = c2(v%dt,v%epsilon_r,EPSILON_0,v%sigma_e,v%celx,v%ivold,v%ivolf+1)
c%dt_muodxo     = v%dt/(MU_0*v%celx)
c%chr2dt_muodxo = c2(v%dt,v%mu_r,MU_0,v%sigma_h,v%celx,v%ivold,v%ivolf+1)

if (c%flag_ce2) c%cer2dt_epsodx(:) = c%dt_epsodx(:)
if (c%flag_ch2) c%chr2dt_muodxo(:) = c%dt_muodxo(:)

! -- composantes y --
allocate(    c%dt_epsody(v%jvold:v%jvolf+1), &
         c%cer2dt_epsody(v%jvold:v%jvolf+1), &
	     c%dt_muodyo(v%jvold:v%jvolf+1), &
	 c%chr2dt_muodyo(v%jvold:v%jvolf+1))

c%dt_epsody     = v%dt/(EPSILON_0*v%cely)
c%cer2dt_epsody = c2(v%dt,v%epsilon_r,EPSILON_0,v%sigma_e,v%cely,v%jvold,v%jvolf+1)
c%dt_muodyo     = v%dt/(MU_0*v%cely)
c%chr2dt_muodyo = c2(v%dt,v%mu_r,MU_0,v%sigma_h,v%cely,v%jvold,v%jvolf+1)

if (c%flag_ce2) c%cer2dt_epsody(:) = c%dt_epsody(:)
if (c%flag_ch2) c%chr2dt_muodyo(:) = c%dt_muodyo(:)

! -- composantes z --
allocate(    c%dt_epsodz(v%kvold:v%kvolf+1), &
         c%cer2dt_epsodz(v%kvold:v%kvolf+1), &
	     c%dt_muodzo(v%kvold:v%kvolf+1), &
	 c%chr2dt_muodzo(v%kvold:v%kvolf+1))

c%dt_epsodz     = v%dt/(EPSILON_0*v%celz)
c%cer2dt_epsodz = c2(v%dt,v%epsilon_r,EPSILON_0,v%sigma_e,v%celz,v%kvold,v%kvolf+1)
c%dt_muodzo     = v%dt/(MU_0*v%celz)
c%chr2dt_muodzo = c2(v%dt,v%mu_r,MU_0,v%sigma_h,v%celz,v%kvold,v%kvolf+1)

!----------------------- Milieu à pertes Homogène ----------------------
! initialisation
c%cer1 = 1.0;c%chr1=1.0
if (c%flag_ce2) c%cer2dt_epsodz(:) = c%dt_epsodz(:)
if (c%flag_ch2) c%chr2dt_muodzo(:) = c%dt_muodzo(:)

! -- cer1 en milieu dielectrique homogene a pertes electriques homogenes --
if (c%flag_diel_pertes_homogene) then
  c%cer1 = (1 - (v%sigma_e*v%dt/(2*v%epsilon_r*EPSILON_0))) / &
           (1 + (v%sigma_e*v%dt/(2*v%epsilon_r*EPSILON_0))) 
endif

! -- chr1 en milieu magnetique homogene a pertes magnetique homogenes --
if (c%flag_magnet_pertes_homogene) then
  c%chr1 = (1 - (v%sigma_h*v%dt/(2*v%mu_r*MU_0))) / &
           (1 + (v%sigma_h*v%dt/(2*v%mu_r*MU_0))) 
endif

!------------------------------------------------------------------------------
!--> Si le maillage est non uniforme, rectification des coefficients spatiaux !
!print '(T10," cer2dt_epsodx=",10ES15.6)',c%cer2dt_epsodx(400:402)

if (v%flag_maillage_nu) then
  c%dt_epsodx   = c%dt_epsodx *v%celx / v%mnu(1)%pas_e(v%ivold:v%ivolf+1)
  c%dt_epsody   = c%dt_epsody *v%cely / v%mnu(2)%pas_e(v%jvold:v%jvolf+1)
  c%dt_epsodz   = c%dt_epsodz *v%celz / v%mnu(3)%pas_e(v%kvold:v%kvolf+1)
  c%dt_muodxo   = c%dt_muodxo *v%celx / v%mnu(1)%pas_h(v%ivold:v%ivolf+1)
  c%dt_muodyo   = c%dt_muodyo *v%cely / v%mnu(2)%pas_h(v%jvold:v%jvolf+1)
  c%dt_muodzo   = c%dt_muodzo *v%celz / v%mnu(3)%pas_h(v%kvold:v%kvolf+1)

  c%cer2dt_epsodx   = c%cer2dt_epsodx *v%celx / v%mnu(1)%pas_e(v%ivold:v%ivolf+1)
  c%cer2dt_epsody   = c%cer2dt_epsody *v%cely / v%mnu(2)%pas_e(v%jvold:v%jvolf+1)
  c%cer2dt_epsodz   = c%cer2dt_epsodz *v%celz / v%mnu(3)%pas_e(v%kvold:v%kvolf+1)
  c%chr2dt_muodxo   = c%chr2dt_muodxo *v%celx / v%mnu(1)%pas_h(v%ivold:v%ivolf+1)
  c%chr2dt_muodyo   = c%chr2dt_muodyo *v%cely / v%mnu(2)%pas_h(v%jvold:v%jvolf+1)
  c%chr2dt_muodzo   = c%chr2dt_muodzo *v%celz / v%mnu(3)%pas_h(v%kvold:v%kvolf+1)
end if
!print '(T10," cer2dt_epsodx=",10ES15.6)',c%cer2dt_epsodx(400:402)

!------------------------------------------------------------------------------

end subroutine champ_precalcul

! ==========================================================
! == Fonction c2                                          ==
! ==========================================================
function c2(dt,epsmu_r,epsmu_o,sigma_eh,dl,ideb,ifin) result(coefficient)

integer, intent(in) :: ideb, ifin
real, dimension(ideb:ifin) :: coefficient
real, intent(in) :: epsmu_r, epsmu_o, sigma_eh, dt
real, intent(in) :: dl

coefficient = 1/(1+(sigma_eh*dt)/(2*epsmu_o*epsmu_r))*(dt/(epsmu_o*epsmu_r*dl))

end function c2

! ==========================================================
! == Subroutine champ_annule_e_tangent_metal              ==
! == pour l'annulation des composantes tangentielles du   ==
! == champ électrique sur les métallisations              == 
! == Utilisé uniquement si le milieu est homogène         ==
! ==========================================================	
subroutine champ_annule_e_tangent_metal(t_c,t_s)
type(st_champ), intent(inout)           :: t_c
type(st_structure), intent(in)          :: t_s

integer   :: i,imin,imax,jmin,jmax,kmin,kmax

!$OMP PARALLEL PRIVATE(imin,imax,jmin,jmax,kmin,kmax) IF (t_s%nbmetal>1000)
  !$OMP DO SCHEDULE(DYNAMIC,500)
do i= 1,t_s%nbmetal
  imin = t_s%t_metal(1,i)
  imax = t_s%t_metal(2,i) 
  jmin = t_s%t_metal(3,i) 
  jmax = t_s%t_metal(4,i) 
  kmin = t_s%t_metal(5,i) 
  kmax = t_s%t_metal(6,i) 
  ! -- face xoy --
  t_c%ex(imin:imax-1,jmin:jmax,kmin) = 0.0
  t_c%ey(imin:imax,jmin:jmax-1,kmin) = 0.0	
  if (kmax>kmin) then
    t_c%ex(imin:imax-1,jmin:jmax,kmax) = 0.0	  
    t_c%ey(imin:imax,jmin:jmax-1,kmax) = 0.0  
  end if
  
  ! -- face yoz -- 
  t_c%ey(imin,jmin:jmax-1,kmin:kmax) = 0.0	
  t_c%ez(imin,jmin:jmax,kmin:kmax-1) = 0.0	
  if (imax>imin) then
    t_c%ey(imax,jmin:jmax-1,kmin:kmax) = 0.0	  
    t_c%ez(imax,jmin:jmax,kmin:kmax-1) = 0.0	    
  end if
  
  ! -- face xoz --
  t_c%ex(imin:imax-1,jmin,kmin:kmax) = 0.0	
  t_c%ez(imin:imax,jmin,kmin:kmax-1) = 0.0	
  if (jmax>jmin) then
    t_c%ex(imin:imax-1,jmax,kmin:kmax) = 0.0	  
    t_c%ez(imin:imax,jmax,kmin:kmax-1) = 0.0	  
  end if
end do
  !$OMP END DO
!$OMP END PARALLEL 
end subroutine champ_annule_e_tangent_metal

! ==========================================================
! == function champ_coeff_Ex                            ==
! == Retourne les trois coefficients pour la mise à jour  ==
! == de Ex dans la cellule de coordonnées (i,j,k)
! ==========================================================
real function champ_coeff_Ex(c,i,j,k,nu) result(CE)
type(st_champ), intent(in) :: c
integer, intent(in) :: i,j,k,nu

select case (nu)  ! Traite des coefficients de la relation : E1 = c1*E1+ c3*H3 - c2*H2 
case (1) ! coefficient c1 
  cE = c%cer1
  if (c%flag_dielectrique_quelconque) cE = c%cex1(i,j,k)

case (2) ! coefficient c2 
  cE = c%cer2dt_epsodz(k) ! avec hy
  if (c%flag_dielectrique_inhomogene.or.c%flag_dielectrique_quelconque) then
    cE = c%dt_epsodz(k)*c%cex2(i,j,k) ! avec hy
  end if

case (3) ! coefficient c3
  cE = c%cer2dt_epsody(j) ! avec hz
  if (c%flag_dielectrique_inhomogene.or.c%flag_dielectrique_quelconque) then
    cE = c%dt_epsody(j)*c%cex2(i,j,k) ! avec hz
  end if

case (4) ! coefficient c2*delta3=c3*delta2 (sans le pas spatial)
  cE = c%cer2dt_epsody(j)*c%dxyz(2)
  if (c%flag_dielectrique_inhomogene.or.c%flag_dielectrique_quelconque) then
    cE = c%dt_epsody(j)*c%cex2(i,j,k)*c%dxyz(2) ! avec hz
  end if

case default
  print '("Erreur de n° de coeff dans l''appel à la subroutine champ_coeff_Ex, n°=",I7)',nu
  stop
end select


!champ_coeff_Ex=cE(nu)
end function champ_coeff_Ex

! ==========================================================
! == function champ_coeff_Ey                            ==
! == Retourne les trois coefficients pour la mise à jour  ==
! == de Ey dans la cellule de coordonnées (i,j,k)
! ==========================================================
real function champ_coeff_Ey(c,i,j,k,nu) result(CE)
type(st_champ), intent(in) :: c
integer, intent(in) :: i,j,k,nu

select case (nu)  ! Traite des coefficients de la relation : E1 = c1*E1+ c3*H3 - c2*H2 
case (1) ! coefficient c1 
  cE = c%cer1
  if (c%flag_dielectrique_quelconque) cE = c%cey1(i,j,k)

case (2) ! coefficient c2 
  cE = c%cer2dt_epsodx(i) ! avec hz
  if (c%flag_dielectrique_inhomogene.or.c%flag_dielectrique_quelconque) then
    cE = c%dt_epsodx(i)*c%cey2(i,j,k) ! avec hz
  end if

case (3) ! coefficient c3
  cE = c%cer2dt_epsodz(k) ! avec hx
  if (c%flag_dielectrique_inhomogene.or.c%flag_dielectrique_quelconque) then
    cE = c%dt_epsodz(k)*c%cey2(i,j,k) ! avec hx
  end if

case (4) ! coefficient c2*delta3=c3*delta2 (sans le pas spatial)
  cE = c%cer2dt_epsodx(i)*c%dxyz(1) ! avec hz
  if (c%flag_dielectrique_inhomogene.or.c%flag_dielectrique_quelconque) then
    cE = c%dt_epsodx(i)*c%cey2(i,j,k)*c%dxyz(1) ! avec hz
  end if


case default
  print '("Erreur de n° de coeff dans l''appel à la subroutine champ_coeff_Ey, n°=",I7)',nu
  stop
end select

end function champ_coeff_Ey

! ==========================================================
! == function champ_coeff_Ez                            ==
! == Retourne les trois coefficients pour la mise à jour  ==
! == de Ez dans la cellule de coordonnées (i,j,k)
! ==========================================================
real function champ_coeff_Ez(c,i,j,k,nu) result(CE)
type(st_champ), intent(in) :: c
integer, intent(in) :: i,j,k,nu

select case (nu)  ! Traite des coefficients de la relation : E1 = c1*E1+ c3*H3 - c2*H2 
case (1) ! coefficient c1 
  cE = c%cer1
  if (c%flag_dielectrique_quelconque) cE = c%cez1(i,j,k)

case (2) ! coefficient c2 
  cE = c%cer2dt_epsody(j) ! avec hx
  if (c%flag_dielectrique_inhomogene.or.c%flag_dielectrique_quelconque) then
    cE = c%dt_epsody(j)*c%cez2(i,j,k) ! avec hx
  end if

case (3) ! coefficient c3
  cE = c%cer2dt_epsodx(i) ! avec hy
  if (c%flag_dielectrique_inhomogene.or.c%flag_dielectrique_quelconque) then
    cE = c%dt_epsodx(i)*c%cez2(i,j,k) ! avec hy
  end if

case (4) ! coefficient c2*delta3=c3*delta2 (sans le pas spatial)
  cE = c%cer2dt_epsody(j)*c%dxyz(2) ! avec hx
  if (c%flag_dielectrique_inhomogene.or.c%flag_dielectrique_quelconque) then
    cE = c%dt_epsody(j)*c%cez2(i,j,k)*c%dxyz(2) ! avec hx
  end if

case default
  print '("Erreur de n° de coeff dans l''appel à la subroutine champ_coeff_Ez, n°=",I7)',nu
  stop
end select

end function champ_coeff_Ez

! ==========================================================
! == function champ_coeff_Hx                            ==
! == Retourne les trois coefficients pour la mise à jour  ==
! == de Hx dans la cellule de coordonnées (i,j,k)
! ==========================================================
real function champ_coeff_Hx(c,i,j,k,nu) result(CH)
type(st_champ), intent(in) :: c
integer, intent(in) :: i,j,k,nu

select case (nu)  ! Traite des coefficients de la relation : H1 = c1*H1- c3*H3 + c2*H2 
case (1) ! coefficient c1 
  cH = c%chr1
  if (c%flag_magnetique_quelconque) cH = c%chx1(i,j,k)

case (2) ! coefficient c2 
  cH = c%chr2dt_muodzo(k) ! avec ey
  if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
    cH = c%dt_muodzo(k)*c%chx2(i,j,k) ! avec ey
  end if

case (3) ! coefficient c3
  cH = c%chr2dt_muodyo(j) ! avec ez
  if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
    cH = c%dt_muodyo(j)*c%chx2(i,j,k) ! avec ez
  end if

case (4) ! coefficient c2*delta3=c3*delta2 (sans le pas spatial)
  cH = c%chr2dt_muodzo(k)*c%dxyz(3) ! avec ey
  if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
    cH = c%dt_muodzo(k)*c%chx2(i,j,k)*c%dxyz(3) ! avec ey
  end if

case default
  print '("Erreur de n° de coeff dans l''appel à la subroutine champ_coeff_Hx, n°=",I7)',nu
  stop
end select

end function champ_coeff_Hx

! ==========================================================
! == function champ_coeff_Hy                            ==
! == Retourne les trois coefficients pour la mise à jour  ==
! == de Hy dans la cellule de coordonnées (i,j,k)
! ==========================================================
real function champ_coeff_Hy(c,i,j,k,nu) result(CH)
type(st_champ), intent(in) :: c
integer, intent(in) :: i,j,k,nu

select case (nu)  ! Traite des coefficients de la relation : H1 = c1*H1- c3*H3 + c2*H2 
case (1) ! coefficient c1 
  cH = c%chr1
  if (c%flag_magnetique_quelconque) cH = c%chy1(i,j,k)

case (2) ! coefficient c2 
  cH = c%chr2dt_muodxo(i) ! avec ez
  if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
    cH = c%dt_muodxo(i)*c%chy2(i,j,k) ! avec ez
  end if

case (3) ! coefficient c3
 cH = c%chr2dt_muodzo(k) ! avec ex
  if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
    cH = c%dt_muodzo(k)*c%chy2(i,j,k) ! avec ex
  end if

case (4) ! coefficient c2*delta3=c3*delta2 (sans le pas spatial)
  cH = c%chr2dt_muodxo(i)*c%dxyz(1) ! avec ez
  if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
    cH = c%dt_muodxo(i)*c%chy2(i,j,k)*c%dxyz(1) ! avec ez
  end if

case default
  print '("Erreur de n° de coeff dans l''appel à la subroutine champ_coeff_Hy, n°=",I7)',nu
  stop
end select

end function champ_coeff_Hy

! ==========================================================
! == function champ_coeff_Hz                            ==
! == Retourne les trois coefficients pour la mise à jour  ==
! == de Hz dans la cellule de coordonnées (i,j,k)
! == nu =n° du coeff soit 1,2 ou 3
! ==========================================================
real function champ_coeff_Hz(c,i,j,k,nu) result(CH)
type(st_champ), intent(in) :: c
integer, intent(in) :: i,j,k,nu

select case (nu)  ! Traite des coefficients de la relation : H1 = c1*H1- c3*H3 + c2*H2 
case (1) ! coefficient c1 
  cH = c%chr1
  if (c%flag_magnetique_quelconque) cH = c%chz1(i,j,k)

case (2) ! coefficient c2 
  cH = c%chr2dt_muodyo(j) ! avec ex
  if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
    cH = c%dt_muodyo(j)*c%chz2(i,j,k) ! avec ex
  end if

case (3) ! coefficient c3
  cH = c%chr2dt_muodxo(i) ! avec ey
  if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
    cH = c%dt_muodxo(i)*c%chz2(i,j,k) ! avec ey
  end if

case (4) ! coefficient c2*delta3=c3*delta2 (sans le pas spatial)
  cH = c%chr2dt_muodyo(j)*c%dxyz(2) ! avec ex
  if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
    cH = c%dt_muodyo(j)*c%chz2(i,j,k)*c%dxyz(2) ! avec ex
  end if

case default
  print '("Erreur de n° de coeff dans l''appel à la subroutine champ_coeff_Hz, n°=",I7)',nu
  stop
end select

end function champ_coeff_Hz

! ==========================================================
! == function champ_coeff_Hx_surf                         ==
! == Retourne les trois coefficients pour la mise à jour  ==
! == de Hx dans la cellule de coordonnées (i,j,k)
! == Les coefficients sont annulés si la surface est de 
! == type : metal, sibc ou bibc
! ==========================================================
real function champ_coeff_Hx_surf(c,t,i,j,k,nu)
type(st_champ), intent(in) :: c
type(st_structure), intent(in) :: t
integer, intent(in) :: i,j,k,nu
real, dimension(3) :: cH
integer :: m
! Détection d'un matériau en surface de type metal,sibc ou bibc
m=structure_matqcq_face(t,(/i,j,k/),PLAN_YOZ)
if (m>0.and.m<8) then
  champ_coeff_Hx_surf=0.0
  !print '("HX=0 pour i,j,k=",3I5)',i,j,k
  return
end if

cH(1) = c%chr1
cH(2) = c%chr2dt_muodzo(k) ! avec ey
cH(3) = c%chr2dt_muodyo(j) ! avec ez
if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
  if (c%flag_magnetique_quelconque) cH(1) = c%chx1(i,j,k)
  cH(2) = c%dt_muodzo(k)*c%chx2(i,j,k) ! avec ey
  cH(3) = c%dt_muodyo(j)*c%chx2(i,j,k) ! avec ez
end if
champ_coeff_Hx_surf=cH(nu)
end function champ_coeff_Hx_surf

! ==========================================================
! == function champ_coeff_Hy_surf                         ==
! == Retourne les trois coefficients pour la mise à jour  ==
! == de Hy dans la cellule de coordonnées (i,j,k)
! == Les coefficients sont annulés si la surface est de 
! == type : metal, sibc ou bibc
! ==========================================================
real function champ_coeff_Hy_surf(c,t,i,j,k,nu)
type(st_champ), intent(in) :: c
type(st_structure), intent(in) :: t
integer, intent(in) :: i,j,k,nu
real, dimension(3) :: cH
integer :: m
! Détection d'un matériau en surface de type metal,sibc ou bibc
m=structure_matqcq_face(t,(/i,j,k/),PLAN_XOZ)
if (m>0.and.m<8) then
  champ_coeff_Hy_surf=0.0
  !print '("HY=0 pour i,j,k=",3I5)',i,j,k
  return
end if
! initialisation
cH(1) = c%chr1
cH(2) = c%chr2dt_muodxo(i) ! avec ez
cH(3) = c%chr2dt_muodzo(k) ! avec ex
if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
  if (c%flag_magnetique_quelconque) cH(1) = c%chy1(i,j,k)
  cH(2) = c%dt_muodxo(i)*c%chy2(i,j,k) ! avec ez
  cH(3) = c%dt_muodzo(k)*c%chy2(i,j,k) ! avec ex
end if
champ_coeff_Hy_surf=cH(nu)
end function champ_coeff_Hy_surf

! ==========================================================
! == function champ_coeff_Hz_surf                         ==
! == Retourne les trois coefficients pour la mise à jour  ==
! == de Hz dans la cellule de coordonnées (i,j,k)
! == nu =n° du coeff soit 1,2 ou 3
! == Les coefficients sont annulés si la surface est de 
! == type : metal, sibc ou bibc
! ==========================================================
real function champ_coeff_Hz_surf(c,t,i,j,k,nu)
type(st_champ), intent(in) :: c
type(st_structure), intent(in) :: t
integer, intent(in) :: i,j,k,nu
real, dimension(3) :: cH
integer :: m
! Détection d'un matériau en surface de type metal,sibc ou bibc
m=structure_matqcq_face(t,(/i,j,k/),PLAN_XOY)
if (m>0.and.m<8) then
  champ_coeff_Hz_surf=0.0
  !print '("HZ=0 pour i,j,k=",3I5)',i,j,k
  return
end if
! initialisation
cH(1) = c%chr1
cH(2) = c%chr2dt_muodyo(j) ! avec ex
cH(3) = c%chr2dt_muodxo(i) ! avec ey
if (c%flag_magnetique_inhomogene.or.c%flag_magnetique_quelconque) then
  if (c%flag_magnetique_quelconque) cH(1) = c%chz1(i,j,k)
  cH(2) = c%dt_muodyo(j)*c%chz2(i,j,k) ! avec ex
  cH(3) = c%dt_muodxo(i)*c%chz2(i,j,k) ! avec ey
end if
champ_coeff_Hz_surf=cH(nu)
end function champ_coeff_Hz_surf

! ==========================================================
! == Subroutine champ_calcul_h                            ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine champ_calcul_h(c)

type(st_champ), intent(inout) :: c

integer :: i, j, k
real, dimension(:,:,:),pointer :: ex,ey,ez,hx,hy,hz
real, dimension(:),pointer :: chr2dt_muodzo,chr2dt_muodyo,chr2dt_muodxo
real, dimension(:),pointer :: dt_muodzo,dt_muodyo,dt_muodxo
real, dimension(:,:,:),pointer :: chx1,chx2,chy1,chy2,chz1,chz2

ex=>c%ex;ey=>c%ey;ez=>c%ez
hx=>c%hx;hy=>c%hy;hz=>c%hz



! -- Pas de pertes magnetiques & permeabilite homogene --
if (c%flag_magnetique_homogene) then
  chr2dt_muodxo=>c%chr2dt_muodxo;
  chr2dt_muodyo=>c%chr2dt_muodyo;
  chr2dt_muodzo=>c%chr2dt_muodzo;
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdhx,c%kfhx
    do j=c%jdhx,c%jfhx
      do i=c%idhx,c%ifhx
	hx(i,j,k) = hx(i,j,k) + &
          ((ey(i,j,k+1)-ey(i,j,k))*chr2dt_muodzo(k) - &
	   (ez(i,j+1,k)-ez(i,j,k))*chr2dt_muodyo(j))
      enddo
    enddo
  enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdhy,c%kfhy
    do j=c%jdhy,c%jfhy
      do i=c%idhy,c%ifhy
	hy(i,j,k) = hy(i,j,k) + &
          ((ez(i+1,j,k)-ez(i,j,k))*chr2dt_muodxo(i) - &
	   (ex(i,j,k+1)-ex(i,j,k))*chr2dt_muodzo(k))
      enddo 
    enddo
  enddo
  !$OMP END DO

  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdhz,c%kfhz
    do j=c%jdhz,c%jfhz
      do i=c%idhz,c%ifhz
	hz(i,j,k) = hz(i,j,k) + &
          ((ex(i,j+1,k)-ex(i,j,k))*chr2dt_muodyo(j) - &
	   (ey(i+1,j,k)-ey(i,j,k))*chr2dt_muodxo(i))
      enddo
    enddo
  enddo
  !$OMP END DO
!$OMP END PARALLEL

! -- Pas de pertes magnetiques & milieu magnetique inhomogene--
elseif (c%flag_magnetique_inhomogene) then
  chx2=>c%chx2
  chy2=>c%chy2
  chz2=>c%chz2
  dt_muodxo =>c%dt_muodxo
  dt_muodyo =>c%dt_muodyo
  dt_muodzo =>c%dt_muodzo

!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdhx,c%kfhx
    do j=c%jdhx,c%jfhx
      do i=c%idhx,c%ifhx
	hx(i,j,k) = hx(i,j,k) + &
          ((ey(i,j,k+1)-ey(i,j,k))*dt_muodzo(k) - &
	   (ez(i,j+1,k)-ez(i,j,k))*dt_muodyo(j)) * chx2(i,j,k)
      enddo
    enddo
  enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)

  do k=c%kdhy,c%kfhy
    do j=c%jdhy,c%jfhy
      do i=c%idhy,c%ifhy
	hy(i,j,k) = hy(i,j,k) + &
          ((ez(i+1,j,k)-ez(i,j,k))*dt_muodxo(i) - &
	   (ex(i,j,k+1)-ex(i,j,k))*dt_muodzo(k)) * chy2(i,j,k)
      enddo 
    enddo
  enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)

  do k=c%kdhz,c%kfhz
    do j=c%jdhz,c%jfhz
      do i=c%idhz,c%ifhz
	hz(i,j,k) = hz(i,j,k) + &
          ((ex(i,j+1,k)-ex(i,j,k))*dt_muodyo(j) - &
	   (ey(i+1,j,k)-ey(i,j,k))*dt_muodxo(i)) * chz2(i,j,k)
      enddo
    enddo
  enddo
  !$OMP END DO
!$OMP END PARALLEL

! -- Pertes magnetiques et milieu magnetique homogene --
elseif (c%flag_magnet_pertes_homogene) then
  chr2dt_muodxo=>c%chr2dt_muodxo;
  chr2dt_muodyo=>c%chr2dt_muodyo;
  chr2dt_muodzo=>c%chr2dt_muodzo;
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdhx,c%kfhx
    do j=c%jdhx,c%jfhx
      do i=c%idhx,c%ifhx
	hx(i,j,k) = hx(i,j,k)*c%chr1 + &
          ((ey(i,j,k+1)-ey(i,j,k))*chr2dt_muodzo(k) - &
	   (ez(i,j+1,k)-ez(i,j,k))*chr2dt_muodyo(j))
      enddo
    enddo
  enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)

  do k=c%kdhy,c%kfhy
    do j=c%jdhy,c%jfhy
      do i=c%idhy,c%ifhy
	hy(i,j,k) = hy(i,j,k)*c%chr1 + &
          ((ez(i+1,j,k)-ez(i,j,k))*chr2dt_muodxo(i) - &
	   (ex(i,j,k+1)-ex(i,j,k))*chr2dt_muodzo(k))
      enddo 
    enddo
  enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)

  do k=c%kdhz,c%kfhz
    do j=c%jdhz,c%jfhz
      do i=c%idhz,c%ifhz
	hz(i,j,k) = hz(i,j,k)*c%chr1 + &
          ((ex(i,j+1,k)-ex(i,j,k))*chr2dt_muodyo(j) - &
	   (ey(i+1,j,k)-ey(i,j,k))*chr2dt_muodxo(i))
      enddo
    enddo
  enddo
  !$OMP END DO
!$OMP END PARALLEL

! -- Gestion d'un milieu quelconque -- 
elseif (c%flag_magnetique_quelconque) then
  chx1=>c%chx1
  chy1=>c%chy1
  chz1=>c%chz1
  chx2=>c%chx2
  chy2=>c%chy2
  chz2=>c%chz2
  dt_muodxo =>c%dt_muodxo
  dt_muodyo =>c%dt_muodyo
  dt_muodzo =>c%dt_muodzo
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdhx,c%kfhx
    do j=c%jdhx,c%jfhx
      do i=c%idhx,c%ifhx
	hx(i,j,k) = hx(i,j,k)*chx1(i,j,k) + &
          ((ey(i,j,k+1)-ey(i,j,k))*dt_muodzo(k) - &
	   (ez(i,j+1,k)-ez(i,j,k))*dt_muodyo(j)) * chx2(i,j,k)
      enddo
    enddo
  enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)

  do k=c%kdhy,c%kfhy
    do j=c%jdhy,c%jfhy
      do i=c%idhy,c%ifhy
	hy(i,j,k) = hy(i,j,k)*chy1(i,j,k) + &
          ((ez(i+1,j,k)-ez(i,j,k))*dt_muodxo(i) - &
	   (ex(i,j,k+1)-ex(i,j,k))*dt_muodzo(k)) * chy2(i,j,k)
      enddo 
    enddo
  enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)

  do k=c%kdhz,c%kfhz
    do j=c%jdhz,c%jfhz
      do i=c%idhz,c%ifhz
	hz(i,j,k) = hz(i,j,k)*chz1(i,j,k) + &
          ((ex(i,j+1,k)-ex(i,j,k))*dt_muodyo(j) - &
	   (ey(i+1,j,k)-ey(i,j,k))*dt_muodxo(i)) * chz2(i,j,k)
      enddo
    enddo
  enddo
  !$OMP END DO
!$OMP END PARALLEL
endif

end Subroutine champ_calcul_h

! ==========================================================
! == Subroutine champ_calcul_e                            ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine champ_calcul_e(c)

implicit none

type(st_champ), intent(inout) :: c
integer :: i, j, k
real :: dhx, dhy, dhz
real, dimension(:,:,:),pointer :: ex,ey,ez,hx,hy,hz
real, dimension(:),pointer :: cer2dt_epsodx,cer2dt_epsody,cer2dt_epsodz
real, dimension(:),pointer :: dt_epsodx,dt_epsody,dt_epsodz
real, dimension(:,:,:),pointer :: cex1,cex2,cey1,cey2,cez1,cez2

ex=>c%ex;ey=>c%ey;ez=>c%ez
hx=>c%hx;hy=>c%hy;hz=>c%hz


if (ex(2,2,2) > 1e25 .or. ex(c%ifex-1,c%jfex-1,c%kfex-1) > 1e25) then
  print '("Arret car le signal semble diverger (ex>1e25) !!!!!!!!!!!")'
  stop
end if 
if (ey(2,2,2) > 1e25 .or. ey(c%ifey-1,c%jfey-1,c%kfey-1) > 1e25) then
  print '("Arret car le signal semble diverger (ey>1e25) !!!!!!!!!!!")'
  stop
end if 
if (ez(2,2,2) > 1e25 .or. ez(c%ifez-1,c%jfez-1,c%kfez-1) > 1e25) then
  print '("Arret car le signal semble diverger (ez>1e25) !!!!!!!!!!!")'
  stop
end if 

! -- Pas de pertes electriques & dielectriques inhomogene --
if (c%flag_dielectrique_inhomogene) then
  cex2=>c%cex2
  cey2=>c%cey2
  cez2=>c%cez2
  dt_epsodx =>c%dt_epsodx
  dt_epsody =>c%dt_epsody
  dt_epsodz =>c%dt_epsodz
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdex,c%kfex
    do j=c%jdex,c%jfex
      do i=c%idex,c%ifex
	dhz = hz(i,j,k)-hz(i,j-1,k)
	dhy = hy(i,j,k)-hy(i,j,k-1)
	ex(i,j,k) = ex(i,j,k) + (dhz*dt_epsody(j)-dhy*dt_epsodz(k))*cex2(i,j,k)
      enddo
    enddo
  enddo

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdey,c%kfey
    do j=c%jdey,c%jfey
      do i=c%idey,c%ifey
	dhx = hx(i,j,k)-hx(i,j,k-1)
	dhz = hz(i,j,k)-hz(i-1,j,k)
	ey(i,j,k) = ey(i,j,k) + (dhx*dt_epsodz(k)-dhz*dt_epsodx(i))*cey2(i,j,k)
      enddo
    enddo
  enddo

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdez,c%kfez
    do j=c%jdez,c%jfez
      do i=c%idez,c%ifez
	dhy = hy(i,j,k)-hy(i-1,j,k)
	dhx = hx(i,j,k)-hx(i,j-1,k)
	ez(i,j,k) = ez(i,j,k) + (dhy*dt_epsodx(i)-dhx*dt_epsody(j))*cez2(i,j,k)
      enddo
    enddo
  enddo
  !$OMP END DO
!$OMP END PARALLEL

! -- Pas de pertes electriques & dielectrique homogene -- 
elseif (c%flag_dielectrique_homogene) then
  cer2dt_epsodx =>c%cer2dt_epsodx
  cer2dt_epsody =>c%cer2dt_epsody
  cer2dt_epsodz =>c%cer2dt_epsodz
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdex,c%kfex
    do j=c%jdex,c%jfex
      do i=c%idex,c%ifex
	dhz = hz(i,j,k)-hz(i,j-1,k)
	dhy = hy(i,j,k)-hy(i,j,k-1)
	ex(i,j,k) = ex(i,j,k) + (dhz*cer2dt_epsody(j)-dhy*cer2dt_epsodz(k))
      enddo
    enddo
  enddo

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdey,c%kfey
    do j=c%jdey,c%jfey
      do i=c%idey,c%ifey
	dhx = hx(i,j,k)-hx(i,j,k-1)
	dhz = hz(i,j,k)-hz(i-1,j,k)
	ey(i,j,k) = ey(i,j,k) + (dhx*cer2dt_epsodz(k)-dhz*cer2dt_epsodx(i))
      enddo
    enddo
  enddo

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdez,c%kfez
    do j=c%jdez,c%jfez
      do i=c%idez,c%ifez
	dhy = hy(i,j,k)-hy(i-1,j,k)
	dhx = hx(i,j,k)-hx(i,j-1,k)
	ez(i,j,k) = ez(i,j,k) + (dhy*cer2dt_epsodx(i)-dhx*cer2dt_epsody(j))
      enddo
    enddo
  enddo
  !$OMP END DO
!$OMP END PARALLEL

! -- Dielectrique quelconque -- 
elseif (c%flag_dielectrique_quelconque) then
  cex1=>c%cex1
  cey1=>c%cey1
  cez1=>c%cez1
  cex2=>c%cex2
  cey2=>c%cey2
  cez2=>c%cez2
  dt_epsodx =>c%dt_epsodx
  dt_epsody =>c%dt_epsody
  dt_epsodz =>c%dt_epsodz
  
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdex,c%kfex
    do j=c%jdex,c%jfex
      do i=c%idex,c%ifex
	dhz = hz(i,j,k)-hz(i,j-1,k)
	dhy = hy(i,j,k)-hy(i,j,k-1)
	ex(i,j,k) = ex(i,j,k)*cex1(i,j,k) + (dhz*dt_epsody(j)-dhy*dt_epsodz(k)) * cex2(i,j,k)
      enddo
    enddo
  enddo

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdey,c%kfey
    do j=c%jdey,c%jfey
      do i=c%idey,c%ifey
	dhx = hx(i,j,k)-hx(i,j,k-1)
	dhz = hz(i,j,k)-hz(i-1,j,k)
	ey(i,j,k) = ey(i,j,k)*cey1(i,j,k) + (dhx*dt_epsodz(k)-dhz*dt_epsodx(i)) * cey2(i,j,k)
      enddo
    enddo
  enddo

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdez,c%kfez
    do j=c%jdez,c%jfez
      do i=c%idez,c%ifez
	dhy = hy(i,j,k)-hy(i-1,j,k)
	dhx = hx(i,j,k)-hx(i,j-1,k)
	ez(i,j,k) = ez(i,j,k)*cez1(i,j,k) + (dhy*dt_epsodx(i)-dhx*dt_epsody(j))*cez2(i,j,k)
      enddo
    enddo
  enddo
  !$OMP END DO
!$OMP END PARALLEL

! -- Pertes electriques homogenes & dielectrique homogene -- 
elseif (c%flag_diel_pertes_homogene) then
  cer2dt_epsodx =>c%cer2dt_epsodx
  cer2dt_epsody =>c%cer2dt_epsody
  cer2dt_epsodz =>c%cer2dt_epsodz
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdex,c%kfex
    do j=c%jdex,c%jfex
      do i=c%idex,c%ifex
	dhz = hz(i,j,k)-hz(i,j-1,k)
	dhy = hy(i,j,k)-hy(i,j,k-1)
	ex(i,j,k) = ex(i,j,k)*c%cer1 + (dhz*cer2dt_epsody(j)-dhy*cer2dt_epsodz(k))
      enddo
    enddo
  enddo

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdey,c%kfey
    do j=c%jdey,c%jfey
      do i=c%idey,c%ifey
	dhx = hx(i,j,k)-hx(i,j,k-1)
	dhz = hz(i,j,k)-hz(i-1,j,k)
	ey(i,j,k) = ey(i,j,k)*c%cer1 + (dhx*cer2dt_epsodz(k)-dhz*cer2dt_epsodx(i))
      enddo
    enddo
  enddo

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=c%kdez,c%kfez
    do j=c%jdez,c%jfez
      do i=c%idez,c%ifez
	dhy = hy(i,j,k)-hy(i-1,j,k)
	dhx = hx(i,j,k)-hx(i,j-1,k)
	ez(i,j,k) = ez(i,j,k)*c%cer1 + (dhy*cer2dt_epsodx(i)-dhx*cer2dt_epsody(j))
      enddo
    enddo
  enddo
  !$OMP END DO
!$OMP END PARALLEL
endif

end subroutine champ_calcul_e

! ==========================================================
! == Subroutine champ_calcul_h                            ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine champ_calcul_h_old(c)

type(st_champ), intent(inout) :: c

integer :: i, j, k

! -- Pas de pertes magnetiques & permeabilite homogene --
if (c%flag_magnetique_homogene) then
  do k=c%kdhx,c%kfhx
    do j=c%jdhx,c%jfhx
      do i=c%idhx,c%ifhx
	c%hx(i,j,k) = c%hx(i,j,k) + &
          ((c%ey(i,j,k+1)-c%ey(i,j,k))*c%chr2dt_muodzo(k) - &
	   (c%ez(i,j+1,k)-c%ez(i,j,k))*c%chr2dt_muodyo(j))
      enddo
    enddo
  enddo

  do k=c%kdhy,c%kfhy
    do j=c%jdhy,c%jfhy
      do i=c%idhy,c%ifhy
	c%hy(i,j,k) = c%hy(i,j,k) + &
          ((c%ez(i+1,j,k)-c%ez(i,j,k))*c%chr2dt_muodxo(i) - &
	   (c%ex(i,j,k+1)-c%ex(i,j,k))*c%chr2dt_muodzo(k))
      enddo 
    enddo
  enddo

  do k=c%kdhz,c%kfhz
    do j=c%jdhz,c%jfhz
      do i=c%idhz,c%ifhz
	c%hz(i,j,k) = c%hz(i,j,k) + &
          ((c%ex(i,j+1,k)-c%ex(i,j,k))*c%chr2dt_muodyo(j) - &
	   (c%ey(i+1,j,k)-c%ey(i,j,k))*c%chr2dt_muodxo(i))
      enddo
    enddo
  enddo

! -- Pas de pertes magnetiques & milieu magnetique inhomogene--
elseif (c%flag_magnetique_inhomogene) then
  do k=c%kdhx,c%kfhx
    do j=c%jdhx,c%jfhx
      do i=c%idhx,c%ifhx
	c%hx(i,j,k) = c%hx(i,j,k) + &
          ((c%ey(i,j,k+1)-c%ey(i,j,k))*c%dt_muodzo(k) - &
	   (c%ez(i,j+1,k)-c%ez(i,j,k))*c%dt_muodyo(j)) * c%chx2(i,j,k)
      enddo
    enddo
  enddo

  do k=c%kdhy,c%kfhy
    do j=c%jdhy,c%jfhy
      do i=c%idhy,c%ifhy
	c%hy(i,j,k) = c%hy(i,j,k) + &
          ((c%ez(i+1,j,k)-c%ez(i,j,k))*c%dt_muodxo(i) - &
	   (c%ex(i,j,k+1)-c%ex(i,j,k))*c%dt_muodzo(k)) * c%chy2(i,j,k)
      enddo 
    enddo
  enddo

  do k=c%kdhz,c%kfhz
    do j=c%jdhz,c%jfhz
      do i=c%idhz,c%ifhz
	c%hz(i,j,k) = c%hz(i,j,k) + &
          ((c%ex(i,j+1,k)-c%ex(i,j,k))*c%dt_muodyo(j) - &
	   (c%ey(i+1,j,k)-c%ey(i,j,k))*c%dt_muodxo(i)) * c%chz2(i,j,k)
      enddo
    enddo
  enddo

! -- Pertes magnetiques et milieu magnetique homogene --
elseif (c%flag_magnet_pertes_homogene) then
  do k=c%kdhx,c%kfhx
    do j=c%jdhx,c%jfhx
      do i=c%idhx,c%ifhx
	c%hx(i,j,k) = c%hx(i,j,k)*c%chr1 + &
          ((c%ey(i,j,k+1)-c%ey(i,j,k))*c%chr2dt_muodzo(k) - &
	   (c%ez(i,j+1,k)-c%ez(i,j,k))*c%chr2dt_muodyo(j))
      enddo
    enddo
  enddo

  do k=c%kdhy,c%kfhy
    do j=c%jdhy,c%jfhy
      do i=c%idhy,c%ifhy
	c%hy(i,j,k) = c%hy(i,j,k)*c%chr1 + &
          ((c%ez(i+1,j,k)-c%ez(i,j,k))*c%chr2dt_muodxo(i) - &
	   (c%ex(i,j,k+1)-c%ex(i,j,k))*c%chr2dt_muodzo(k))
      enddo 
    enddo
  enddo

  do k=c%kdhz,c%kfhz
    do j=c%jdhz,c%jfhz
      do i=c%idhz,c%ifhz
	c%hz(i,j,k) = c%hz(i,j,k)*c%chr1 + &
          ((c%ex(i,j+1,k)-c%ex(i,j,k))*c%chr2dt_muodyo(j) - &
	   (c%ey(i+1,j,k)-c%ey(i,j,k))*c%chr2dt_muodxo(i))
      enddo
    enddo
  enddo

! -- Gestion d'un milieu quelconque -- 
elseif (c%flag_magnetique_quelconque) then
  do k=c%kdhx,c%kfhx
    do j=c%jdhx,c%jfhx
      do i=c%idhx,c%ifhx
	c%hx(i,j,k) = c%hx(i,j,k)*c%chx1(i,j,k) + &
          ((c%ey(i,j,k+1)-c%ey(i,j,k))*c%dt_muodzo(k) - &
	   (c%ez(i,j+1,k)-c%ez(i,j,k))*c%dt_muodyo(j)) * c%chx2(i,j,k)
      enddo
    enddo
  enddo

  do k=c%kdhy,c%kfhy
    do j=c%jdhy,c%jfhy
      do i=c%idhy,c%ifhy
	c%hy(i,j,k) = c%hy(i,j,k)*c%chy1(i,j,k) + &
          ((c%ez(i+1,j,k)-c%ez(i,j,k))*c%dt_muodxo(i) - &
	   (c%ex(i,j,k+1)-c%ex(i,j,k))*c%dt_muodzo(k)) * c%chy2(i,j,k)
      enddo 
    enddo
  enddo

  do k=c%kdhz,c%kfhz
    do j=c%jdhz,c%jfhz
      do i=c%idhz,c%ifhz
	c%hz(i,j,k) = c%hz(i,j,k)*c%chz1(i,j,k) + &
          ((c%ex(i,j+1,k)-c%ex(i,j,k))*c%dt_muodyo(j) - &
	   (c%ey(i+1,j,k)-c%ey(i,j,k))*c%dt_muodxo(i)) * c%chz2(i,j,k)
      enddo
    enddo
  enddo
endif

end Subroutine champ_calcul_h_old

! ==========================================================
! == Subroutine champ_calcul_e                            ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine champ_calcul_e_old(c)

implicit none

type(st_champ), intent(inout) :: c

integer :: i, j, k
real :: dhx, dhy, dhz

if (c%ex(2,2,2) > 1e25 .or. c%ex(c%ifex-1,c%jfex-1,c%kfex-1) > 1e25) then
  print '("Arret car le signal semble diverger (ex>1e25) !!!!!!!!!!!")'
  stop
end if 
if (c%ey(2,2,2) > 1e25 .or. c%ey(c%ifey-1,c%jfey-1,c%kfey-1) > 1e25) then
  print '("Arret car le signal semble diverger (ey>1e25) !!!!!!!!!!!")'
  stop
end if 
if (c%ez(2,2,2) > 1e25 .or. c%ez(c%ifez-1,c%jfez-1,c%kfez-1) > 1e25) then
  print '("Arret car le signal semble diverger (ez>1e25) !!!!!!!!!!!")'
  stop
end if 

! -- Pas de pertes electriques & dielectriques inhomogene --
if (c%flag_dielectrique_inhomogene) then
  do k=c%kdex,c%kfex
    do j=c%jdex,c%jfex
      do i=c%idex,c%ifex
	dhz = c%hz(i,j,k)-c%hz(i,j-1,k)
	dhy = c%hy(i,j,k)-c%hy(i,j,k-1)
	c%ex(i,j,k) = c%ex(i,j,k) + &
	  (dhz*c%dt_epsody(j)-dhy*c%dt_epsodz(k))*c%cex2(i,j,k)
      enddo
    enddo
  enddo

  do k=c%kdey,c%kfey
    do j=c%jdey,c%jfey
      do i=c%idey,c%ifey
	dhx = c%hx(i,j,k)-c%hx(i,j,k-1)
	dhz = c%hz(i,j,k)-c%hz(i-1,j,k)
	c%ey(i,j,k) = c%ey(i,j,k) + &
	  (dhx*c%dt_epsodz(k)-dhz*c%dt_epsodx(i))*c%cey2(i,j,k)
      enddo
    enddo
  enddo

  do k=c%kdez,c%kfez
    do j=c%jdez,c%jfez
      do i=c%idez,c%ifez
	dhy = c%hy(i,j,k)-c%hy(i-1,j,k)
	dhx = c%hx(i,j,k)-c%hx(i,j-1,k)
	c%ez(i,j,k) = c%ez(i,j,k) + &
	  (dhy*c%dt_epsodx(i)-dhx*c%dt_epsody(j))*c%cez2(i,j,k)
      enddo
    enddo
  enddo

! -- Pas de pertes electriques & dielectrique homogene -- 
elseif (c%flag_dielectrique_homogene) then
  do k=c%kdex,c%kfex
    do j=c%jdex,c%jfex
      do i=c%idex,c%ifex
	dhz = c%hz(i,j,k)-c%hz(i,j-1,k)
	dhy = c%hy(i,j,k)-c%hy(i,j,k-1)
	c%ex(i,j,k) = c%ex(i,j,k) + &
	  (dhz*c%cer2dt_epsody(j)-dhy*c%cer2dt_epsodz(k))
      enddo
    enddo
  enddo

  do k=c%kdey,c%kfey
    do j=c%jdey,c%jfey
      do i=c%idey,c%ifey
	dhx = c%hx(i,j,k)-c%hx(i,j,k-1)
	dhz = c%hz(i,j,k)-c%hz(i-1,j,k)
	c%ey(i,j,k) = c%ey(i,j,k) + &
	  (dhx*c%cer2dt_epsodz(k)-dhz*c%cer2dt_epsodx(i))
      enddo
    enddo
  enddo

  do k=c%kdez,c%kfez
    do j=c%jdez,c%jfez
      do i=c%idez,c%ifez
	dhy = c%hy(i,j,k)-c%hy(i-1,j,k)
	dhx = c%hx(i,j,k)-c%hx(i,j-1,k)
	c%ez(i,j,k) = c%ez(i,j,k) + &
	  (dhy*c%cer2dt_epsodx(i)-dhx*c%cer2dt_epsody(j))
      enddo
    enddo
  enddo

! -- Dielectrique quelconque -- 
elseif (c%flag_dielectrique_quelconque) then
  do k=c%kdex,c%kfex
    do j=c%jdex,c%jfex
      do i=c%idex,c%ifex
	dhz = c%hz(i,j,k)-c%hz(i,j-1,k)
	dhy = c%hy(i,j,k)-c%hy(i,j,k-1)
	c%ex(i,j,k) = c%ex(i,j,k)*c%cex1(i,j,k) + & 
	  (dhz*c%dt_epsody(j)-dhy*c%dt_epsodz(k)) * c%cex2(i,j,k)
      enddo
    enddo
  enddo

  do k=c%kdey,c%kfey
    do j=c%jdey,c%jfey
      do i=c%idey,c%ifey
	dhx = c%hx(i,j,k)-c%hx(i,j,k-1)
	dhz = c%hz(i,j,k)-c%hz(i-1,j,k)
	c%ey(i,j,k) = c%ey(i,j,k)*c%cey1(i,j,k) + &
	  (dhx*c%dt_epsodz(k)-dhz*c%dt_epsodx(i)) * c%cey2(i,j,k)
      enddo
    enddo
  enddo

  do k=c%kdez,c%kfez
    do j=c%jdez,c%jfez
      do i=c%idez,c%ifez
	dhy = c%hy(i,j,k)-c%hy(i-1,j,k)
	dhx = c%hx(i,j,k)-c%hx(i,j-1,k)
	c%ez(i,j,k) = c%ez(i,j,k)*c%cez1(i,j,k) + &
	  (dhy*c%dt_epsodx(i)-dhx*c%dt_epsody(j))*c%cez2(i,j,k)
      enddo
    enddo
  enddo

! -- Pertes electriques homogenes & dielectrique homogene -- 
elseif (c%flag_diel_pertes_homogene) then
  do k=c%kdex,c%kfex
    do j=c%jdex,c%jfex
      do i=c%idex,c%ifex
	dhz = c%hz(i,j,k)-c%hz(i,j-1,k)
	dhy = c%hy(i,j,k)-c%hy(i,j,k-1)
	c%ex(i,j,k) = c%ex(i,j,k)*c%cer1 + &
	  (dhz*c%cer2dt_epsody(j)-dhy*c%cer2dt_epsodz(k))
      enddo
    enddo
  enddo

  do k=c%kdey,c%kfey
    do j=c%jdey,c%jfey
      do i=c%idey,c%ifey
	dhx = c%hx(i,j,k)-c%hx(i,j,k-1)
	dhz = c%hz(i,j,k)-c%hz(i-1,j,k)
	c%ey(i,j,k) = c%ey(i,j,k)*c%cer1 + &
	  (dhx*c%cer2dt_epsodz(k)-dhz*c%cer2dt_epsodx(i))
      enddo
    enddo
  enddo

  do k=c%kdez,c%kfez
    do j=c%jdez,c%jfez
      do i=c%idez,c%ifez
	dhy = c%hy(i,j,k)-c%hy(i-1,j,k)
	dhx = c%hx(i,j,k)-c%hx(i,j-1,k)
	c%ez(i,j,k) = c%ez(i,j,k)*c%cer1 + &
	  (dhy*c%cer2dt_epsodx(i)-dhx*c%cer2dt_epsody(j))
      enddo
    enddo
  enddo
endif

end subroutine champ_calcul_e_old


! ==========================================================
! == Subroutine champ_affichage                           ==
! == Affichage de résultats dans le terminal              ==
! ==========================================================
subroutine champ_affichage()

				
end subroutine champ_affichage


! ==========================================================
! == Subroutine champ_deallocate                          ==
! == Calcul après les itérations temporelles              ==
! ==========================================================
subroutine champ_deallocate(c)

type(st_champ), intent(inout) :: c
deallocate(c%ex, c%ey, c%ez, c%hx, c%hy, c%hz)

if (c%flag_magnetique_inhomogene) deallocate(c%chx2, c%chy2, c%chz2)
if (c%flag_magnetique_quelconque) deallocate(c%chx1, c%chy1, c%chz1,c%chx2, c%chy2, c%chz2)
if (c%flag_dielectrique_inhomogene) deallocate(c%cex2, c%cey2, c%cez2)
if (c%flag_dielectrique_quelconque) deallocate(c%cex1, c%cey1, c%cez1,c%cex2, c%cey2, c%cez2)

deallocate(c%dt_epsodx,     c%dt_epsody,     c%dt_epsodz)
deallocate(c%cer2dt_epsodx, c%cer2dt_epsody, c%cer2dt_epsodz)
deallocate(c%dt_muodxo,     c%dt_muodyo,     c%dt_muodzo)
deallocate(c%chr2dt_muodxo, c%chr2dt_muodyo, c%chr2dt_muodzo)
				
end subroutine champ_deallocate


! ==========================================================
! == Subroutine champ_impression                          ==
! == Ecriture des résultats dans les fichiers de sortie   ==
! ==========================================================	
subroutine champ_impression()


end subroutine champ_impression


! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module mfdtd_champ	



