!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_champ_lointain_frequence.f90
!// Purpose:     Calcul du champ lointain par deux méthodes :
!                  1- méthode mixte temp-fréquence à partir du 
!                     concept des surfaces de Huygens
!                  2- méthode mixte temp-fréquence à partir des
!                     courants circulant sur les conducteurs parfaits
!                       (concernent que les courants horizontaux mais 
!                       prend en compte la présence d'un demi-espace (sol)
!                       avec une constante diélectrique et des pertes.
!      Simulateur FDTD
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :   -  Septembre 2002 pour la première méthode
!                -  Janvier 2004 pour la seconde
!// Copyright:   (c) Christophe Guiffaut, équipe CNRS IRCOM/DEM
!//              Utilisation des sources réservées à l'IRCOM
!// Licence  :
!/////////////////////////////////////////////////////////////////////////////

!================================================================
! Calcul du champ lointain par la méthode frequentiel des surfaces de Huygens
!================================================================
module mfdtd_champ_lointain_freq
use m_constantes
use m_constantes_scientifiques
use m_math_operation, only : dp

use m_es_identifiant
use m_entree , only : st_fichier, &
                lineread, &
		nb_mot_chaine, &
		ctrl_nb_mot_chaine, &
		multibackspace, &
		lecture_stop, &
		numbackspace_set, &
		entier_chaine, &
		ligne_precedente
use m_sortie, only : st_fichier_sortie, &
                impression_ascii_reel_2x, &
		impression_ascii_complexe_2x, &
		fichier_sortie_lect_format_freq, &
		impression_ascii_complexe, &
		fichier_sortie_ecrit_format,&
		ouvre_en_ecriture

use m_sortie_normalise, only : st_sortie_normalise, &
                              sortie_normalise_initdefaut,&
			      sortie_normalise_lecture   ,&
			      sortie_normalise_ecriture ,&
			      sortie_normalise_compare, &
			      Normalise_Reponse_Freq
use m_generateur, only : st_generateur

use m_zone_angulaire, only : st_zone_angulaire, &
                                  st_une_zone, &
				  C_PHI=>PHI, C_THETA=>THETA, &
				  zone_angulaire_pointe

use m_maillage
use m_surface_huygens
				  
use m_temps_frequence, only : st_temps_frequence, &
                                   st_liste_temps_freq, &
				   temps_freq_identifie_freq, &
				   liste_temps_freq_pointe

use mfdtd_volume, only : st_volume
use mfdtd_structure, only : st_structure, &
                             st_structure_objet, &
			     structure_objet_pointe

use mfdtd_milieu_volume, only : st_milieu_volume, &
                          milieu_volume_extrait_epsilon, &
			  milieu_volume_extrait_mu,&
			  cherche_epsilon_cellule,&
			  cherche_mu_cellule
                 

use mfdtd_champ, only : st_champ 

use mfdtd_structure_fil,   only : st_structure_fil, &
                                   st_fil_objet, &
				   structure_fil_objet_pointe
use champ_lointain_surface
use champ_lointain_demi_espace, only : st_chl_demi_espace, &
				      chl_demi_espace_initdefaut, &
				      chl_demi_espace_init, &
				      chl_demi_espace_boite_huygens, &
				      chl_demi_espace_precalcul, &
				      chl_demi_espace_alias_champ, &
				      chl_demi_espace_affichage, &
				      chl_demi_espace_calcul, &
				      chl_demi_espace_J_M, &
				      chl_demi_espace_calcul_eth_eph, &
				      chl_demi_espace_calcul_dth_dph, &
				      chl_demi_espace_deallocate

implicit none 

integer, parameter  :: NB_MOT_CLE_STOCKAGE = 7

!=== Déclaration des types ===

! Structure pour les alias avec les composantes du champ  

type st_resultat_champ_directivite
  logical    :: flag_eth, flag_eph
  logical    :: flag_dth, flag_dph
  logical    ::  flag_pr
  integer    :: com_format_tf
  integer    :: unite_eth, unite_eph, unite_dth, unite_dph, unite_pr
  character(LEN=LEN_MOT_CLE) :: nom_zone_angulaire
  character(LEN=LEN_MOT_CLE) :: nom_eth, nom_eph, nom_pr, nom_dth, nom_dph
  character(LEN=LEN_MOT_CLE) :: mantisse_champ, mantisse_directivite, mantisse_pr
  type(st_une_zone), pointer    :: tza
  complex, pointer, dimension(:,:)  :: etheta, ephi
  real   , pointer, dimension(:,:)  :: dtheta, dphi
  complex, pointer, dimension(:)  :: pr
  complex(dp), pointer, dimension(:)  :: pr_dp
  !--> Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
end type st_resultat_champ_directivite

! Structure pour le calcul du champ lointain d'une surface de Huygens
type st_un_champ_lointain
  logical  :: flag_pr ! flag d'intégration du vecteur de poynting sur la surface de Huygens
  logical  :: flag_image, flag_image_ideale, flag_espace_libre
  logical  :: flag_centre_phase_auto
  logical  :: flag_dispersion
  integer  :: plan_image, position_plan_image,type_image
  integer  :: nb_resultat
  REAL, dimension(3)  :: origine ! centre de phase
  real  :: zmilieu, vphase
  real  :: dxdy, dydz, dxdz
  character(LEN=LEN_MOT_CLE) :: nom,nom_surfhuyg,nom_objetmetal,nom_objetfil,interv_f
  type(st_temps_frequence), pointer     :: tf
  type(st_une_surface_huygens),pointer  :: tsurfhuyg
  type(st_resultat_champ_directivite), pointer, dimension(:) :: trcl
  type(st_pointeur_champ_surface), dimension(:), pointer :: tpchamp
  
  !--> champ lointain d'un demi-espace, sol réel, epsilon et sigma
  logical  :: flag_sol
  type(st_chl_demi_espace), pointer :: tcde

  !--> champ lointain en milieu stratifié
  logical  :: flag_strate ! présence d'un milieu stratifié
  logical  :: flag_interface ! présence d'un milieu air sol
  
end type st_un_champ_lointain

! Les champs lointains en fréquence
type st_champ_lointain_freq
  integer     :: nchl ! nombre de champ lointain
  type (st_un_champ_lointain), pointer, dimension(:) :: l_chl
end type st_champ_lointain_freq
 
! Pas de variables globales...

contains
!===========================================================	
! Subroutine champ_lointain_freq_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine champ_lointain_freq_initdefaut(tcl)
!=== Déclaration des arguments ===
type(st_champ_lointain_freq), intent(inout)  :: tcl
!===  Déclaration des variables  ===

!===   Début des traitements   ===
tcl%nchl = 0

				
End Subroutine champ_lointain_freq_initdefaut

!===========================================================	
! Subroutine un_champ_lointain_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine un_champ_lointain_initdefaut(cl)
!=== Déclaration des arguments ===
type(st_un_champ_lointain), intent(inout)  :: cl
!===  Déclaration des variables  ===

!===   Début des traitements   ===
cl%flag_pr    = .false.
cl%flag_image = .false.
cl%flag_image_ideale = .false.
cl%flag_strate = .false.
cl%flag_interface = .false.
cl%flag_centre_phase_auto = .true.
cl%plan_image = PLAN_XOY
cl%type_image = MUR_ELECTRIQUE
cl%position_plan_image = 0
cl%flag_sol   = .false.
cl%flag_espace_libre = .true.
cl%flag_dispersion = .false.

nullify(cl%tcde)
cl%interv_f = '';
cl%nom_surfhuyg = '';
cl%nom_objetmetal =''
cl%nom_objetfil = ''
cl%nb_resultat = 0
cl%vphase  = CVIDE				
cl%zmilieu = ZVIDE				
cl%origine(:) = 0.0

End Subroutine un_champ_lointain_initdefaut

!===========================================================	
! Subroutine champ_directivite_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine champ_directivite_initdefaut(cd)
!=== Déclaration des arguments ===
type(st_resultat_champ_directivite), intent(inout)  :: cd
!===  Déclaration des variables  ===

!===   Début des traitements   ===
cd%flag_eth = .false. ; cd%flag_eph = .false. ; 
cd%flag_dth = .false. ; cd%flag_dph = .false. ; 
cd%flag_pr = .false.
cd%nom_zone_angulaire = ''
cd%nom_eth = '' ; cd%nom_eph = ''
cd%nom_dth = '' ; cd%nom_dph = ''
cd%nom_pr = ''
call sortie_normalise_initdefaut(cd%sn)

				
End Subroutine champ_directivite_initdefaut

!===========================================================	
! Subroutine champ_lointain_freq_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine champ_lointain_freq_lecture(f, tcl, ltf,tza,tsh,ts,sf,tm)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout)      :: f
type(st_champ_lointain_freq), intent(inout)  :: tcl
type(st_liste_temps_freq), intent(inout) :: ltf
type(st_zone_angulaire), intent(inout)  :: tza
type(st_surface_huygens), intent(inout)  :: tsh
type(st_structure), intent(inout)  :: ts
type(st_structure_fil), pointer  :: sf
type(st_maillage), intent(in) :: tm

!===  Déclaration des variables  ===
integer  :: i,j, i_ch, i1
integer  :: com_format_tf
character(LEN=LEN_MOT_CLE) :: nom, premiermot
character(LEN=LEN_MOT_CLE), dimension(3)  :: noms
type(st_un_champ_lointain), pointer  :: cl
! Normalisation en fréquence
type(st_sortie_normalise) :: sn

!===   Début des traitements   ===
call numbackspace_set(f,0)

! -- ON COMPTE LE NOMBRE DE CALCUL DE CHAMP LOINTAIN ... --
i_ch = 0
do
  premiermot = lineread(f,1)
  if (premiermot(1:3) == 'CHL')  then
      i_ch = i_ch +1
      
  else if (premiermot==f%nomfinbloc) then
    exit
  endif
enddo

! -- Allocation memoire pour les champs lointains --
if (i_ch > 0) then
  tcl%nchl = i_ch
  allocate(tcl%l_chl(i_ch))
else
  print *,'Avertissement: pas de surfaces de Huygens définies dans le bloc correspondant !'
  return
end if
! -- Retour vers la premiere ligne du bloc --
i = multibackspace(f)
com_format_tf = MODULE_PHASE
i_ch = 0
! Init normalisation
call sortie_normalise_initdefaut(sn)

!=================== Début de la lecture du bloc ==============================
do
  ! -- On recommence la lecture pour les données --
  premiermot = lineread(f)
  if (premiermot(1:3)=='CHL') then
      i_ch = i_ch + 1
      
      cl =>tcl%l_chl(i_ch)
      
      call un_champ_lointain_initdefaut(cl)
      j = ctrl_nb_mot_chaine(f,f%chaine,3,4)

      read(f%chaine,*,err=999) cl%nom, noms(1:j-1)
!      if (noms(1)(1:2) ==  noms(2)(1:2)) then
!	 print '("Erreur, redondance de mots clés !!!")'
!	 call lecture_stop(f)
!      end if
      
      ! récupération du pointeur sur intervalle de fréquence et surface de Huygens
      do i=1,j-1
        !----------------------------------------------------------------------
        if (noms(i)(1:4) == 'SURF') then
	  cl%nom_surfhuyg = trim(noms(i));
	  cl%tsurfhuyg => surface_huygens_pointe(tsh,noms(i))
	  if (.not.associated(cl%tsurfhuyg)) then
            print '(/,t10,"Nom du type de surface de Huygens erroné !!!")'
            call lecture_stop(f)      
	  end if
	   
        !----------------------------------------------------------------------
        else if (noms(i)(1:5) == 'OBJET') then
	  if (.not.associated(cl%tcde)) then
	    allocate(cl%tcde)
            call chl_demi_espace_initdefaut(cl%tcde)
	  end if
	  if (noms(i)(1:9) == 'OBJET_FIL') then
	    cl%tcde%flag_fil = .true.
            cl%nom_objetfil = trim(noms(i))
	    cl%tcde%objfil => structure_fil_objet_pointe(sf,noms(i))
	    if (.not.associated(cl%tcde%objfil)) then
              print '(/,t10,"Nom du type d''objet fil erroné !!!")'
              call lecture_stop(f)      
	    end if
	  else ! Objet metal
	    cl%tcde%tmetal => structure_objet_pointe(ts,noms(i))
            cl%nom_objetmetal = trim(noms(i))
	    if (.not.associated(cl%tcde%tmetal).or.(cl%tcde%tmetal%typemilieu(1:5) /= 'metal') ) then
              print '(/,t10,"Nom du type d''objet métal erroné !!!")'
              call lecture_stop(f)      
	    end if
	    cl%tcde%flag_metal = .true.
	  end if
          cl%flag_sol = .true.
        !----------------------------------------------------------------------
	else if (temps_freq_identifie_freq(noms(i))) then
	  cl%interv_f = trim(noms(i))
	  cl%tf => liste_temps_freq_pointe(ltf,noms(i))
	  if (.not.associated(cl%tf)) then
            print '(/,t10,"Nom du type de stockage en fréquence erroné !!!")'
            call lecture_stop(f)      
	  end if
	
        !----------------------------------------------------------------------
	else
	  print '("Erreur, mot clé non renonnu !!!")'
	  call lecture_stop(f)
	end if
      end do

      ! Lecture des données propres à un champ lointain
      call un_champ_lointain_lecture(f,cl,tza,tm,com_format_tf,sn)

  else if (premiermot==f%nomfinbloc) then
	exit
  !----------------------------------------------------------------------------
  else if (sortie_normalise_lecture(f,premiermot,sn)) then
    cycle
  !----------------------------------------------------------------------------
  else if (.not. fichier_sortie_lect_format_freq(premiermot,com_format_tf)) then
	! -- Si ca ne se passe pas bien : arret de la lecture --
	print *,'Erreur, Mot clé non reconnu :'
	call lecture_stop(f)
  end if
enddo

return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(f)
				
End Subroutine champ_lointain_freq_lecture

!===========================================================	
! Subroutine un_champ_lointain_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine un_champ_lointain_lecture(f, cl, tza,tm, com_format_tf,sn)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout)         :: f
type(st_un_champ_lointain), pointer     :: cl
type(st_zone_angulaire), intent(inout)  :: tza
type(st_maillage), intent(in) :: tm
integer, intent(inout)                  :: com_format_tf
type(st_sortie_normalise),intent(inout) :: sn

!===  Déclaration des variables  ===

character(LEN=LEN_MOT_CLE) :: nom, premiermot, typlan
character(LEN=LEN_MOT_CLE), dimension(2)  :: noms
character(LEN=LEN_MOT_CLE), dimension(NB_MOT_CLE_STOCKAGE)  :: liste_param
character(LEN=3), dimension(6)  :: motcle = (/'ETH','EPH', 'ETP', 'DTH', 'DPH', 'DTP'/)
integer            :: i, i1, nbmot,status
integer            :: coord3(3)
type(st_resultat_champ_directivite), pointer :: cd

!===   Début des traitements   ===
				
! Nombre de post-traitement pour ce champ lointain :
      
call numbackspace_set(f,0)
i1 = 0
do
  premiermot = lineread(f,1)
  select case (premiermot(1:3))
    case('ETH', 'EPH', 'ETP', 'DTH', 'DPH', 'DTP')
      i1 = i1 +1
      cycle
    case('CHL')
      exit
  end select

  if (premiermot(1:2)== 'PR') then
    i1 = i1+1
  else if (premiermot==f%nomfinbloc) then
    exit
  endif

enddo

! -- Allocation memoire pour les champs lointains --
if (i1 > 0) then
  cl%nb_resultat = i1
  allocate(cl%trcl(i1))
else
  print *,'Erreur : Pas de paramètres de stockage !!!'
  call lecture_stop(f)
end if
! -- Retour vers la premiere ligne du sous-bloc --
i = multibackspace(f)
i1 = 0      

!=================== Début de la lecture du bloc ==============================
do
  premiermot = lineread(f)
  
  if (ANY(motcle == premiermot(1:3)).or.(premiermot(1:2)== 'PR')) then
    i1 = i1+1
    cd => cl%trcl(i1)
    call champ_directivite_initdefaut(cd)
    cd%com_format_tf = com_format_tf
    cd%sn = sn
    ! Etude de la chaine
    nbmot = nb_mot_chaine(f%chaine)
    
    if (nbmot > 6) then
      print *,'Erreur, trop de mot dans la chaine :',f%chaine
      call lecture_stop(f)
    end if

    !--- Cas de la puissance rayonnée seule ---
    if (nbmot == 1) then 
      if (premiermot(1:2)== 'PR') then
        read(f%chaine,*,err=999) cd%nom_pr
        cd%flag_pr = .true. 
      else
        print '("Erreur, information manquante avec ce mot clé :",A)',TRIM(premiermot)
        call lecture_stop(f)
      end if  
    else
      read(f%chaine,*,err=999) liste_param(1:nbmot-1), nom
      cd%nom_zone_angulaire = nom
      ! Récupération du pointeur de zone angulaire
      cd%tza => zone_angulaire_pointe(tza,nom)
      if (.not.associated(cd%tza)) then
	print '(/,t10,"Erreur : Nom de la zone angulaire erroné ou celle-ci est manquante!!!")'
	call lecture_stop(f)      
      end if

      ! Activation des flags des paramètres à stocker
      do i = 1, nbmot-1
	select case (liste_param(i)(1:3))
	case('ETH')
          cd%flag_eth = .true. 
	  cd%nom_eth = liste_param(i)
	case('EPH')
          cd%flag_eph = .true. 
	  cd%nom_eph = liste_param(i)
	case('ETP')
          cd%flag_eth = .true. 
          cd%flag_eph = .true. 
	  cd%nom_eth = 'ETH'//liste_param(i)(4:)
	  cd%nom_eph = 'EPH'//liste_param(i)(4:)
	case('DTH')
          cd%flag_dth = .true.
	  cl%flag_pr  = .true.
	  cd%nom_dth = liste_param(i)
	case('DPH')
          cd%flag_dph = .true. 
	  cl%flag_pr  = .true.
	  cd%nom_dph = liste_param(i)
	case('DTP')
          cd%flag_dth = .true. 
          cd%flag_dph = .true. 
	  cl%flag_pr  = .true.
	  cd%nom_dth = 'DTH'//liste_param(i)(4:)
	  cd%nom_dph = 'DPH'//liste_param(i)(4:)
	case default
	  if (liste_param(i)(1:2) == 'PR') then
            cd%flag_pr = .true. 
  	    cl%flag_pr  = .true.
	    cd%nom_pr = liste_param(i)
	  else
	    ! -- Si ca se passe pas bien : arret de la lecture --
	    print '("Erreur, Mot clé n°",i2," non reconnu !!!")',i
	    call lecture_stop(f)	
	  end if
	end select
      end do
    end if
  else if (premiermot(1:3) == 'CHL' .or. premiermot==f%nomfinbloc) then
    call ligne_precedente(f)
    exit
         
  else if (premiermot == 'centre_phase') then
    status = ctrl_nb_mot_chaine(f,f%chaine,4)
    read(f%chaine,*,err=999)  nom, cl%origine(1:3)
    cl%origine = origine_cellule3(tm,cl%origine)
    cl%flag_centre_phase_auto = .false.
    if (rang == RANG_MAITRE_AFF) Print '(t10,A," : Coords ds le repère de calcul = ",3F8.2)',trim(nom),cl%origine
  else if (premiermot == 'knumerique') then
    cl%flag_dispersion = .true.
  else if (premiermot == 'interface') then
    if (.not.cl%flag_sol) then
      print '("Erreur, interface concerne uniquement le calcul champ lointain avec des objets métals ou fil")'
      call lecture_stop(f)
    end if
    status = ctrl_nb_mot_chaine(f,f%chaine,3)
    read(f%chaine,*,err=999)  nom, typlan,cl%tcde%position_sol_interface
    coord3=0
    coord3(3) = cl%tcde%position_sol_interface
    coord3 = origine_cellule3(tm,coord3)
    cl%tcde%position_sol_interface = coord3(3)
    if (rang == RANG_MAITRE_AFF) &
      Print '(t10,A," : Coords ds le repère de calcul = ",I6)',trim(premiermot),cl%tcde%position_sol_interface
    
    if (typlan /= 'xoy') call lecture_stop(f)
    
  else if (premiermot == 'image'.or.premiermot=='image_magnetique') then
    nbmot = nb_mot_chaine(f%chaine)
    if (nbmot == 2) then
      read(f%chaine,*,err=999)  nom, typlan
      cl%flag_image_ideale = .true.
    else if (nbmot == 3) then
      read(f%chaine,*,err=999)  nom, typlan, cl%position_plan_image
      coord3=0;coord3(3) = cl%position_plan_image
      coord3 = origine_cellule3(tm,coord3)
      cl%position_plan_image = coord3(3)
      if (rang == RANG_MAITRE_AFF) &
        Print '(t10,A," : Coords ds le repère de calcul = ",I6)',trim(premiermot),cl%position_plan_image
    else
      print '("Erreur, Nombre de mot erroné ")'
      call lecture_stop(f)
    end if
    if (typlan == 'xoy') then
      cl%plan_image = PLAN_XOY
    else if (typlan == 'yoz') then
      cl%plan_image = PLAN_YOZ
    else if (typlan == 'xoz' .or. typlan == 'zox') then
      cl%plan_image = PLAN_XOZ
    else
      print '("Erreur, nom du type de plan erroné :",A)',TRIM(typlan)
      call lecture_stop(f)
    end if
    cl%flag_image = .true.
    if (premiermot=='image_magnetique') cl%type_image = MUR_MAGNETIQUE
  !----------------------------------------------------------------------------
  else if (sortie_normalise_lecture(f,premiermot,sn)) then
    cycle
  !----------------------------------------------------------------------------
  else if (.not.fichier_sortie_lect_format_freq(premiermot,com_format_tf)) then
      ! -- Si ca ne se passe pas bien : arret de la lecture --
      print *,'Erreur, Mot clé non reconnu :'
      call lecture_stop(f)
  endif
enddo

return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(f)
				
End Subroutine un_champ_lointain_lecture


! ==========================================================
! == Subroutine champ_lointain_freq_ecriture              ==
! ==========================================================
subroutine champ_lointain_freq_ecriture(f,tcl,tm)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_champ_lointain_freq), intent(inout)  :: tcl
type(st_maillage), intent(in) :: tm

integer  :: i,j,commutateur
integer  :: coord3(3)
real     :: coordreel(3)
character(LEN=LEN_MOT_CLE) :: ch1
character(LEN=3),dimension(3) :: chplan= (/'xoy','yoz','xoz'/)
type(st_un_champ_lointain), pointer  :: cl
type(st_resultat_champ_directivite), pointer :: cd
type(st_sortie_normalise) :: sn

commutateur = MODULE_PHASE
call sortie_normalise_initdefaut(sn)

do i=1,tcl%nchl
  cl =>tcl%l_chl(i)
  
  !---------- Type de champ lointain -----------------
  if (cl%flag_sol) then
    ch1 = trim(cl%nom_objetmetal)//' '//trim(cl%nom_objetfil)
    write(f%unite,'(T3,A,3x,A,3x,A)') trim(cl%nom),trim(ch1),trim(cl%interv_f)
    coord3=0;coord3(3) = cl%tcde%position_sol_interface
    coord3 = origine_cellule3(tm,coord3,.true.)
    write(f%unite,'(T5,A,3x,A,1x,I5)') 'interface','xoy',coord3(3)
  else 
    write(f%unite,'(T3,A,3x,A,3x,A)') trim(cl%nom),trim(cl%nom_surfhuyg),trim(cl%interv_f)
  end if
  
  !----------- Les options ---------------------
  if (cl%flag_image) then
    ch1='image'
    if (cl%type_image==MUR_MAGNETIQUE) ch1 = 'image_magnetique'
    if (cl%flag_image_ideale) then
      write(f%unite,'(T5,A,3x,A)') trim(ch1),chplan(cl%plan_image)
    else
    coord3=0;coord3(3) = cl%position_plan_image
    coord3 = origine_cellule3(tm,coord3,.true.)
      write(f%unite,'(T5,A,3x,A,1x,I5)') trim(ch1),chplan(cl%plan_image),coord3(3)
    end if
  end if
  
  if (.not.cl%flag_centre_phase_auto) then
      coordreel = origine_cellule3(tm,cl%origine,.true.)
      write(f%unite,'(T5,A,3x,3F9.3)') 'centre_phase',coordreel
  end if
  

  !------------- Les résultats demandés ---------------------------
  do j=1,cl%nb_resultat
    cd => cl%trcl(j)
    if (.not.sortie_normalise_compare(sn,cd%sn)) then
      call sortie_normalise_ecriture(f,cd%sn)
      sn = cd%sn
    end if
    if (cd%flag_pr)  write(f%unite,'(T5,A)')  trim(cd%nom_pr)
    
    if (commutateur/=cd%com_format_tf) then
      call fichier_sortie_ecrit_format(f,cd%com_format_tf)
      commutateur = cd%com_format_tf
    end if
    ch1 ='';
    if (cd%flag_eth) ch1 = trim(ch1) //' '//trim(cd%nom_eth)
    if (cd%flag_eph) ch1 = trim(ch1) //' '//trim(cd%nom_eph)
    if (cd%flag_dth) ch1 = trim(ch1) //' '//trim(cd%nom_dth)
    if (cd%flag_dph) ch1 = trim(ch1) //' '//trim(cd%nom_dph)
    if (trim(ch1)/='') write(f%unite,'(T5,A,3x,A)')  trim(ch1),trim(cd%nom_zone_angulaire)
  end do


end do

end subroutine champ_lointain_freq_ecriture


!===========================================================	
! Subroutine champ_lointain_freq_init
! Initialisation des variables
!===========================================================	
Subroutine champ_lointain_freq_init(tcl)
!=== Déclaration des arguments ===
type(st_champ_lointain_freq), intent(inout)  :: tcl
type(st_un_champ_lointain), pointer :: cl
!===  Déclaration des variables  ===
integer :: i
!===   Début des traitements   ===
do i=1, tcl%nchl
  cl =>tcl%l_chl(i)
  if (cl%flag_sol) then
    cl%flag_espace_libre = .false.
  end if

  if (cl%flag_sol) then
    call chl_demi_espace_init(cl%tcde)
    if  (cl%flag_pr) then
      allocate(cl%tsurfhuyg)
      call une_surface_huygens_initdefaut(cl%tsurfhuyg)
      cl%tsurfhuyg%nbsurface = 6 ! boite fermée
      cl%tsurfhuyg%flag_surface(:) = .true.
      cl%tsurfhuyg%coord = chl_demi_espace_boite_huygens(cl%tcde)
      cl%tsurfhuyg%origine_reel(:) = 0

    end if
    cycle
  end if
end do  
End Subroutine champ_lointain_freq_init
	
!===========================================================	
! Subroutine champ_lointain_freq_precalcul
! Calculs avant les itérations temporelles
! 
!===========================================================	
Subroutine champ_lointain_freq_precalcul(tcl,tm,tfs,tma)
!=== Déclaration des arguments ===
type(st_champ_lointain_freq), intent(inout)  :: tcl
type(st_milieu_volume), intent(in) :: tm
type(st_fichier_sortie), intent(in)   :: tfs
type(st_maillage), intent(in) :: tma

!===  Déclaration des variables  ===
logical :: flag_idemmilieu
integer  :: i, j,ic,jc,kc
real  :: eps, mu
real,dimension(3,6) :: epsr,mur
character(LEN=LEN_MOT_CLE) :: nom_fichier, ch_num
type(st_un_champ_lointain), pointer :: cl
type(st_resultat_champ_directivite), pointer :: rcl
!===   Début des traitements   ===
! boucle sur tous les champs lointains
do i=1, tcl%nchl
  cl =>tcl%l_chl(i)
  
  if (cl%flag_sol) call chl_demi_espace_precalcul(cl%tcde,tm,cl%tf%n_liste,cl%origine,cl%flag_centre_phase_auto)

  ! Origine :
  
  if (rang == RANG_MAITRE_AFF)  print '("Origine ",3ES12.2)',cl%origine
  if (cl%flag_centre_phase_auto) then
    !cl%origine=Maillage_U_RealToCell(tma,cl%tsurfhuyg%origine_reel,.false.)
    !cl%origine=Maillage_U_RealToCell(tma,cl%tsurfhuyg%origine_reel,.false.)
    cl%origine =(cl%tsurfhuyg%coord(2:6:2)+cl%tsurfhuyg%coord(1:5:2))/2
  end if
  if (rang == RANG_MAITRE_AFF)  print '("Origine ",3ES15.7)',cl%origine
  
  
  if ((.not.cl%flag_sol).or.(cl%flag_pr)) then

    allocate(cl%tpchamp(NB_SURFACE_HUYGENS_MAX))
    ! boucle sur toutes les surfaces de Huygens
    ! allocation des tableaux poour les courants en fréquence 
    epsr = 1.0
    mur  = 1.0
    eps  = 1.0
    mu   = 1.0
    do j=1, 6
      if (cl%tsurfhuyg%flag_surface(j)) then
	if (j==1 .or. j==2) then
	  cl%tpchamp(j)%ideb = cl%tsurfhuyg%coord_ssd(XMIN)
	  cl%tpchamp(j)%ifin = cl%tsurfhuyg%coord_ssd(XMAX)
	  cl%tpchamp(j)%jdeb = cl%tsurfhuyg%coord_ssd(YMIN)
	  cl%tpchamp(j)%jfin = cl%tsurfhuyg%coord_ssd(YMAX)
	  cl%tpchamp(j)%position = cl%tsurfhuyg%coord_ssd(ZMIN+j-1)
	  
	  ic = (cl%tpchamp(j)%ifin+cl%tpchamp(j)%ideb)/2
	  jc = (cl%tpchamp(j)%jfin+cl%tpchamp(j)%jdeb)/2
	  kc = cl%tpchamp(j)%position -mod(j,2)
	else if (j==3 .or. j==4) then
	  cl%tpchamp(j)%ideb = cl%tsurfhuyg%coord_ssd(YMIN)
	  cl%tpchamp(j)%ifin = cl%tsurfhuyg%coord_ssd(YMAX)
	  cl%tpchamp(j)%jdeb = cl%tsurfhuyg%coord_ssd(ZMIN)
	  cl%tpchamp(j)%jfin = cl%tsurfhuyg%coord_ssd(ZMAX)
	  cl%tpchamp(j)%position = cl%tsurfhuyg%coord_ssd(XMIN+j-3)
	  jc = (cl%tpchamp(j)%ifin+cl%tpchamp(j)%ideb)/2
	  kc = (cl%tpchamp(j)%jfin+cl%tpchamp(j)%jdeb)/2
	  ic = cl%tpchamp(j)%position -mod(j,2)
	else 
	  cl%tpchamp(j)%ideb = cl%tsurfhuyg%coord_ssd(XMIN)
	  cl%tpchamp(j)%ifin = cl%tsurfhuyg%coord_ssd(XMAX)
	  cl%tpchamp(j)%jdeb = cl%tsurfhuyg%coord_ssd(ZMIN)
	  cl%tpchamp(j)%jfin = cl%tsurfhuyg%coord_ssd(ZMAX)
	  cl%tpchamp(j)%position = cl%tsurfhuyg%coord_ssd(YMIN+j-5)
	  ic = (cl%tpchamp(j)%ifin+cl%tpchamp(j)%ideb)/2
	  kc = (cl%tpchamp(j)%jfin+cl%tpchamp(j)%jdeb)/2
	  jc = cl%tpchamp(j)%position -mod(j,2)
	end if
	!--- Récupération du milieu ---
	epsr(:,j) = cherche_epsilon_cellule(tm,ic,jc,kc)
        mur(:,j)  = cherche_mu_cellule(tm,ic,jc,kc)
	eps = epsr(1,j)
	mu  = mur(1,j)
	!------------------------------
	cl%tpchamp(j)%n1 = cl%tpchamp(j)%ifin - cl%tpchamp(j)%ideb + 1
	cl%tpchamp(j)%n2 = cl%tpchamp(j)%jfin - cl%tpchamp(j)%jdeb + 1
	allocate(cl%tpchamp(j)%J1(1:cl%tpchamp(j)%n1,1:cl%tpchamp(j)%n2,1:cl%tf%n_liste))
	allocate(cl%tpchamp(j)%J2(1:cl%tpchamp(j)%n1,1:cl%tpchamp(j)%n2,1:cl%tf%n_liste))
	allocate(cl%tpchamp(j)%M1(1:cl%tpchamp(j)%n1,1:cl%tpchamp(j)%n2,1:cl%tf%n_liste))
	allocate(cl%tpchamp(j)%M2(1:cl%tpchamp(j)%n1,1:cl%tpchamp(j)%n2,1:cl%tf%n_liste))
	cl%tpchamp(j)%J1(:,:,:) = 0.0
	cl%tpchamp(j)%J2(:,:,:) = 0.0
	cl%tpchamp(j)%M1(:,:,:) = 0.0
	cl%tpchamp(j)%M2(:,:,:) = 0.0
	
	allocate(cl%tpchamp(j)%rscosfiJ1(1:cl%tpchamp(j)%n1))
	allocate(cl%tpchamp(j)%rscosfiJ2(1:cl%tpchamp(j)%n1))
	allocate(cl%tpchamp(j)%rscosfiM1(1:cl%tpchamp(j)%n1))
	allocate(cl%tpchamp(j)%rscosfiM2(1:cl%tpchamp(j)%n1))
	cl%tpchamp(j)%rscosfiJ1(:) = 0.0
	cl%tpchamp(j)%rscosfiJ2(:) = 0.0
	cl%tpchamp(j)%rscosfiM1(:) = 0.0
	cl%tpchamp(j)%rscosfiM2(:) = 0.0
      end if
    end do
    
    flag_idemmilieu = .true.
    ! Vérification que le milieu est le meme sur toute la surface de Huygens
    do j=1,6
      if (cl%tsurfhuyg%flag_surface(j)) then
        if (epsr(1,j)/=eps .or.mur(1,j)/=mu) then
	  flag_idemmilieu = .false.
	  exit
	end if
      end if
    end do
    eps = eps * EPSILON_0
    mu  = mu  * MU_0
    
    ! Type de milieu du champ lointain
    !eps = milieu_volume_extrait_epsilon(tm,cl%tsurfhuyg%coord(XMIN), &
    !                                         cl%tsurfhuyg%coord(YMIN), &
!					     cl%tsurfhuyg%coord(ZMIN), DIRECTION_X)

 !   mu      = milieu_volume_extrait_mu(tm,cl%tsurfhuyg%coord(XMIN), &
 !                                       cl%tsurfhuyg%coord(YMIN), &
!					cl%tsurfhuyg%coord(ZMIN), DIRECTION_X)

    cl%zmilieu = SQRT(mu/eps)
    cl%vphase  = 1 / SQRT(mu*eps)
  end if

  ! boucle sur toutes les structures résultats, ouverture des fichiers
  ! en écriture pour les différents paramètres à stocker,
  ! création des mantisses.
  if (rang == RANG_MAITRE_AFF) then
    do j = 1, cl%nb_resultat
      rcl => cl%trcl(j)
      !--> controle de l'homogénéité du milieu pour la surface de Huygens
      !    end espace libre
      if (rcl%flag_eth.or.rcl%flag_eph.or.rcl%flag_dth.or.rcl%flag_dph) then
	if (cl%flag_espace_libre .and. (.not.flag_idemmilieu)) then
	  print '("ERREUR dans le calcul de champ lointain de nom  ",A)',trim(cl%nom)
	  print '("   Pour un calcul en espace libre, la surface de Huygens doit")'
	  print '("   se trouver dans un milieu unique !!!!")'
	  stop
	end if
      end if
      
      if (rcl%flag_eth) then
	nom_fichier = TRIM(cl%nom)//'_'//TRIM(rcl%nom_eth) //'_'// &
		      TRIM(rcl%nom_zone_angulaire)//TRIM(tfs%extension)
	rcl%unite_eth = ouvre_en_ecriture(nom_fichier)
      end if
      if (rcl%flag_eph) then
	nom_fichier = TRIM(cl%nom)//'_'//TRIM(rcl%nom_eph)//'_'// &
		      TRIM(rcl%nom_zone_angulaire)//TRIM(tfs%extension)
	rcl%unite_eph = ouvre_en_ecriture(nom_fichier)
      end if
      if (rcl%flag_dth) then
	nom_fichier = TRIM(cl%nom)//'_'//TRIM(rcl%nom_dth)//'_'// &
		      TRIM(rcl%nom_zone_angulaire)//TRIM(tfs%extension)
	rcl%unite_dth = ouvre_en_ecriture(nom_fichier)
      end if
      if (rcl%flag_dph) then
	nom_fichier = TRIM(cl%nom)//'_'//TRIM(rcl%nom_dph)//'_'// &
		      TRIM(rcl%nom_zone_angulaire)//TRIM(tfs%extension)
	rcl%unite_dph = ouvre_en_ecriture(nom_fichier)
      end if
      if (rcl%flag_pr) then
	nom_fichier = TRIM(cl%nom)//'_'//TRIM(rcl%nom_pr)//TRIM(tfs%extension)
	rcl%unite_pr = ouvre_en_ecriture(nom_fichier)
      end if
      ! création des mantisses
      ch_num = entier_chaine(cl%tf%n_liste*2+2) ! n_liste = nb fréquence
      rcl%mantisse_champ = '('//TRIM(ch_num)//TRIM(tfs%mantisse_reel)//')'
      
      ch_num = entier_chaine(cl%tf%n_liste+2) ! n_liste = nb fréquence
      rcl%mantisse_directivite = '('//TRIM(ch_num)//TRIM(tfs%mantisse_reel)//')'
      
      rcl%mantisse_pr = '(3'//TRIM(tfs%mantisse_reel)//')' ! 3 colonnes
    end do
  end if    
end do


				
End Subroutine champ_lointain_freq_precalcul


!===========================================================	
! Subroutine champ_lointain_freq_alias_champ
! Calculs avant les itérations temporelles
!===========================================================	
Subroutine champ_lointain_freq_alias_champ(tcl, tc)
!=== Déclaration des arguments ===
type(st_champ_lointain_freq), intent(inout)  :: tcl
type(st_champ), intent(in)   :: tc

!===  Déclaration des variables  ===
integer  :: i, j, ph
type(st_un_champ_lointain), pointer :: cl

!===   Début des traitements   ===
! boucle sur tous les champs lointains

do i=1, tcl%nchl
  cl =>tcl%l_chl(i)
  if (cl%flag_sol) call chl_demi_espace_alias_champ(cl%tcde, tc)  
  if ((.not.cl%flag_sol).or.(cl%flag_pr)) then
    ! boucle sur toutes les surfaces de Huygens
    ! allocation des tableaux poour les courants en fréquence 
    do j=1, 6
      if (cl%tsurfhuyg%flag_surface(j)) then
	if (j==1 .or. j==2) then
	  ph = cl%tpchamp(j)%position
	  if (j == 1) ph = cl%tpchamp(j)%position -1
	  cl%tpchamp(j)%e1 => tc%ex(cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin, &
				    cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin, &
				    cl%tpchamp(j)%position)
	  cl%tpchamp(j)%e2 => tc%ey(cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin, &
				    cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin, &
				    cl%tpchamp(j)%position)
	  cl%tpchamp(j)%h1 => tc%hx(cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin,cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin,ph)
	  cl%tpchamp(j)%h2 => tc%hy(cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin,cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin,ph)
  
	else if (j==3 .or. j==4) then
	  ph = cl%tpchamp(j)%position
	  if (j == 3) ph = cl%tpchamp(j)%position -1
	  cl%tpchamp(j)%e1 => tc%ey(cl%tpchamp(j)%position, &
				    cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin, &
				    cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin)
	  cl%tpchamp(j)%e2 => tc%ez(cl%tpchamp(j)%position, &
				    cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin, &
				    cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin)
	  cl%tpchamp(j)%h1 => tc%hy(ph,cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin,cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin)
	  cl%tpchamp(j)%h2 => tc%hz(ph,cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin,cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin)
  
	else 
	  ph = cl%tpchamp(j)%position
	  if (j == 5) ph = cl%tpchamp(j)%position -1
	  cl%tpchamp(j)%e1 => tc%ez(cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin, &
				    cl%tpchamp(j)%position, &
				    cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin)
	  cl%tpchamp(j)%e2 => tc%ex(cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin, &
				    cl%tpchamp(j)%position, &
				    cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin)
	  cl%tpchamp(j)%h1 => tc%hz(cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin, ph, cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin)
	  cl%tpchamp(j)%h2 => tc%hx(cl%tpchamp(j)%ideb:cl%tpchamp(j)%ifin, ph, cl%tpchamp(j)%jdeb:cl%tpchamp(j)%jfin)
  
	end if
	
      end if
    end do
  end if
end do
				
End Subroutine champ_lointain_freq_alias_champ

!===========================================================	
! Subroutine champ_lointain_freq_calcul
!  Calcul pendant les itérations temporelles
!===========================================================	
Subroutine champ_lointain_freq_calcul(tcl)
!=== Déclaration des arguments ===
type(st_champ_lointain_freq), intent(inout)  :: tcl

!===  Déclaration des variables  ===
integer  :: n,l,lf
type(st_un_champ_lointain), pointer :: cl

!===   Début des traitements   ===
! boucle sur tous les champs lointains
do n=1, tcl%nchl
  cl =>tcl%l_chl(n)
  if (.not. cl%tf%flag_calcul) cycle

  if (cl%flag_sol) call chl_demi_espace_calcul(cl%tcde,cl%tf)  
  
  ! boucle sur toutes les surfaces de Huygens
  ! allocation des tableaux poour les courants en fréquence 
  if ((.not.cl%flag_sol).or.(cl%flag_pr)) then
    do l=1, 6
      if (cl%tsurfhuyg%flag_surface(l)) then
	! boucle sur les fréquences
	do lf = 1, cl%tf%n_liste
	  ! signe correct que pour les faces inférieures
	  cl%tpchamp(l)%J1(:,:,lf) = cl%tpchamp(l)%J1(:,:,lf) + cl%tpchamp(l)%h2(:,:) * cl%tf%expi2pitf_h(lf)
	  cl%tpchamp(l)%J2(:,:,lf) = cl%tpchamp(l)%J2(:,:,lf) - cl%tpchamp(l)%h1(:,:) * cl%tf%expi2pitf_h(lf)
	  cl%tpchamp(l)%M1(:,:,lf) = cl%tpchamp(l)%M1(:,:,lf) - cl%tpchamp(l)%e2(:,:) * cl%tf%expi2pitf(lf)
	  cl%tpchamp(l)%M2(:,:,lf) = cl%tpchamp(l)%M2(:,:,lf) + cl%tpchamp(l)%e1(:,:) * cl%tf%expi2pitf(lf)
	end do
      end if
    end do
  end if
end do
				
End Subroutine champ_lointain_freq_calcul

!===========================================================	
! Subroutine champ_lointain_freq_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine champ_lointain_freq_affichage(tcl)
implicit none
!=== Déclaration des arguments ===
type(st_champ_lointain_freq), intent(inout)  :: tcl

!===  Déclaration des variables  ===
integer :: n
type(st_un_champ_lointain), pointer :: cl

!===   Début des traitements   ===
if (rang == RANG_MAITRE_AFF) then
  print '("==================== CHAMP LOINTAIN ================= ")'
  PRINT '(T5,"Nbre de calcul champ lointain : ",I3)',tcl%nchl
  do n=1, tcl%nchl
    cl =>tcl%l_chl(n)
    PRINT '(T10,"-------- champ lointain :",A," n° ",I2,"-----------")',TRIM(cl%nom),n
    PRINT '(T10,"FlagPr, FlagImage,FlagSol : ",3L3)',cl%flag_pr,cl%flag_image,cl%flag_sol
    if (cl%flag_image) then
      PRINT '(T10,"Plan_Image =",I2," Position = ",I3)',cl%plan_image,cl%position_plan_image
    end if
    PRINT '(T10," Centre de phase (Nbre cellules) = ",3F8.2)',cl%origine
    PRINT '(T10," Nb_resultat = ",I4)',cl%nb_resultat
    PRINT '(T10," Zmilieu, Vphase = ",2ES16.6E2)',cl%zmilieu, cl%vphase
    if ((.not.cl%flag_sol).or.(cl%flag_pr)) then
      PRINT '(T10," Nb   surface Huygens = ",I2)',cl%tsurfhuyg%nbsurface
      PRINT '(T10," Flag surface Huygens = ",6L3)',cl%tsurfhuyg%flag_surface(1:6)
      PRINT '(T10," Coord. surface Huygens = ",6I5)',cl%tsurfhuyg%coord(1:6)
      PRINT '(T10," Coord_ssd. surface Huygens = ",6I5)',cl%tsurfhuyg%coord_ssd(1:6)
    end if
    if (cl%flag_sol) call chl_demi_espace_affichage(cl%tcde)
  end do
end if
End Subroutine champ_lointain_freq_affichage

!===========================================================	
! Subroutine champ_lointain_rectif_J_M
!  Annulation dans les tableaux de sources de courant 
! des valeurs sur certains bords extérieurs.
!  Correction signe pour les faces supérieures
!===========================================================	
Subroutine champ_lointain_rectif_J_M(cl)
implicit none
!=== Déclaration des arguments ===
type(st_un_champ_lointain), intent(inout) :: cl

!===  Déclaration des variables  ===
integer  :: l
type(st_pointeur_champ_surface), pointer  :: pcs

!===   Début des traitements   ===

! boucle sur toutes les faces de la surface de Huygens
do l=2,6,2 ! plans supérieurs
  if (cl%tsurfhuyg%flag_surface(l)) then
    pcs => cl%tpchamp(l)    
    ! correction signe
    pcs%J1 = - pcs%J1
    pcs%J2 = - pcs%J2
    pcs%M1 = - pcs%M1
    pcs%M2 = - pcs%M2
  end if
end do

do l=1, 4
  if (cl%tsurfhuyg%flag_surface(l)) then
    pcs => cl%tpchamp(l)    
    pcs%J1(pcs%n1,:,:) = CMPLX(0.0,0.0)
    pcs%M2(pcs%n1,:,:) = CMPLX(0.0,0.0)
    pcs%J2(:,pcs%n2,:) = CMPLX(0.0,0.0)
    pcs%M1(:,pcs%n2,:) = CMPLX(0.0,0.0)
  end if
end do

do l=5, 6 !plans ZOX, indice non permuté selon la permutation circulaire
  if (cl%tsurfhuyg%flag_surface(l)) then
    pcs => cl%tpchamp(l)    
    pcs%J2(pcs%n1,:,:) = CMPLX(0.0,0.0)
    pcs%M1(pcs%n1,:,:) = CMPLX(0.0,0.0)
    pcs%J1(:,pcs%n2,:) = CMPLX(0.0,0.0)
    pcs%M2(:,pcs%n2,:) = CMPLX(0.0,0.0)
  end if
end do

End Subroutine champ_lointain_rectif_J_M

!===========================================================	
! Subroutine champ_lointain_cor_image_ideal
!  Dans le cas d'une image idéale, les sources de courant sont
! soit doublées soit annulées. Ceci est fonction de l'orientation
! de la surface de huygens par rapport au plan de masse.
!===========================================================	
Subroutine champ_lointain_cor_image_ideal(cl)
!=== Déclaration des arguments ===
type(st_un_champ_lointain), intent(inout) :: cl

!===  Déclaration des variables  ===
integer, parameter  :: n = 3 
integer, dimension(n,n) :: cj1 = RESHAPE((/0,0,2,  2,0,0,  0,2,0/), (/3,3/))
integer, dimension(n,n) :: cj2 = RESHAPE((/0,2,0,  0,0,2,  2,0,0/), (/3,3/))
integer, dimension(n,n) :: cm1 = RESHAPE((/2,2,0,  0,2,2,  2,0,2/), (/3,3/))
integer, dimension(n,n) :: cm2 = RESHAPE((/2,0,2,  2,0,0,  0,2,0/), (/3,3/))

integer, dimension(n,n) :: c1temp,c2temp
integer  :: l, pref
type(st_pointeur_champ_surface), pointer  :: pcs

!===   Début des traitements   ===
!cj1 = RESHAPE((/0,0,2,  2,0,0,  0,2,0/), (/3,3/))
!cj2 = RESHAPE((/0,2,0,  0,0,2,  2,0,0/), (/3,3/))
!cm1 = RESHAPE((/2,2,0,  0,2,2,  2,0,2/), (/3,3/))
!cm2 = RESHAPE((/2,0,2,  2,0,0,  0,2,0/), (/3,3/))
if (cl%type_image == MUR_MAGNETIQUE) then
  c1temp = cj1
  c2temp = cj2
  cj1 = cm1
  cj2 = cm2
  cm1 = c1temp
  cm2 = c2temp
end if

! boucle sur toutes les faces
pref = cl%plan_image
do l=1,NB_SURFACE_HUYGENS_MAX
  if (cl%tsurfhuyg%flag_surface(l)) then
    pcs => cl%tpchamp(l)    
    select case(l) 
    case(1,2) !face xoy inf et sup
      pcs%J1(:,:,:) = pcs%J1(:,:,:) * cj1(PLAN_XOY,pref)
      pcs%J2(:,:,:) = pcs%J2(:,:,:) * cj2(PLAN_XOY,pref)
      pcs%M1(:,:,:) = pcs%M1(:,:,:) * cm1(PLAN_XOY,pref)
      pcs%M2(:,:,:) = pcs%M2(:,:,:) * cm2(PLAN_XOY,pref)
    case(3,4) !face yoz inf et sup
      pcs%J1(:,:,:) = pcs%J1(:,:,:) * cj1(PLAN_YOZ,pref)
      pcs%J2(:,:,:) = pcs%J2(:,:,:) * cj2(PLAN_YOZ,pref)
      pcs%M1(:,:,:) = pcs%M1(:,:,:) * cm1(PLAN_YOZ,pref)
      pcs%M2(:,:,:) = pcs%M2(:,:,:) * cm2(PLAN_YOZ,pref)
 
    case(5,6) !face zox inf et sup
      pcs%J1(:,:,:) = pcs%J1(:,:,:) * cj1(PLAN_XOZ,pref)
      pcs%J2(:,:,:) = pcs%J2(:,:,:) * cj2(PLAN_XOZ,pref)
      pcs%M1(:,:,:) = pcs%M1(:,:,:) * cm1(PLAN_XOZ,pref)
      pcs%M2(:,:,:) = pcs%M2(:,:,:) * cm2(PLAN_XOZ,pref)
    end select
  end if
end do

End Subroutine champ_lointain_cor_image_ideal

!===========================================================	
! Subroutine champ_lointain_freq_postcalcul
!  Calcul après les itérations temporelles
!===========================================================	
Subroutine champ_lointain_freq_postcalcul(tcl,tv,tmv)
!=== Déclaration des arguments ===
type(st_champ_lointain_freq), intent(inout)  :: tcl
type(st_volume), intent(in) :: tv
type(st_milieu_volume), intent(in) :: tmv


!===  Déclaration des variables  ===
integer  :: n,l,nf,m,i,j
real     :: theta
real     :: pr_normalise
type(st_un_champ_lointain), pointer :: cl
type(st_resultat_champ_directivite), pointer  :: rcd

!===   Début des traitements   ===
! boucle sur tous les champs lointains
do n=1, tcl%nchl
  cl =>tcl%l_chl(n)
  print '(T10," Post-Calcul sur Champ Lointain de nom =",A)',TRIM(cl%nom)
  ! Annulation des valeurs dans les tableaux de sources de courant inutile
  ! Correction signe pour les faces supérieures
  if (cl%flag_sol) call chl_demi_espace_J_M(cl%tcde,tv)
  if ((.not.cl%flag_sol).or.(cl%flag_pr)) then
    print '(T10,"  --> Correctif courant sur les bords des surfaces")'
    call champ_lointain_rectif_J_M(cl)
  end if

  ! boucle sur tous les types de résultats
  do l=1, cl%nb_resultat
    rcd => cl%trcl(l)

    !--- Calcul de la puissance complexe rayonnée ---
    if  (rcd%flag_pr .or. rcd%flag_dth .or. rcd%flag_dph) then
      allocate(rcd%pr(1:cl%tf%n_liste))
      allocate(rcd%pr_dp(1:cl%tf%n_liste))
      rcd%pr(:) = CMPLX(0.0,0.0)
      rcd%pr_dp(:) = CMPLX(0.0_dp,0.0_dp)
      print '(T10,"  --> Calcul puissance rayonnée")'
      call un_champ_lointain_calcul_Pr(cl,rcd,tv)
      !call champ_lointain_calcul_Pr_interp(cl,rcd,tv) ! PR surestimée !
    end if
  end do
  
  ! boucle sur tous les types de résultats
  do l=1, cl%nb_resultat
    rcd => cl%trcl(l)

 
    !--- Calcul du champ lointain ---
    if (rcd%flag_eth .or. rcd%flag_eph .or. &
        rcd%flag_dth .or. rcd%flag_dph) then

      !--- Correction des sources si image idéale
      if (cl%flag_image .and. cl%flag_image_ideale) then
        print '(T10,"  --> Application du principe des images (image parfaite) sur les courants")'
	call champ_lointain_cor_image_ideal(cl)
      end if
      
      ! Allocation mémoire pour Etheta et Ephi
      allocate(rcd%etheta(1:rcd%tza%nbcoupleangle , 1:cl%tf%n_liste))
      allocate(rcd%ephi(1:rcd%tza%nbcoupleangle , 1:cl%tf%n_liste))
      do j=1,cl%tf%n_liste
      do i=1,rcd%tza%nbcoupleangle
        rcd%etheta(i,j) = CMPLX(0.0,0.0);
        rcd%ephi(i,j) = CMPLX(0.0,0.0);
      end do; end do
      !------------------------------------------------------------------------
      !-------------- Calcul des champs lointains Etheta et Ephi---------------
      if (cl%flag_sol) then
        print '(T10,"  --> Calcul champ lointain avec sol")'
       call chl_demi_espace_calcul_eth_eph(cl%tcde,cl%tf,tv,rcd%tza,rcd%etheta,rcd%ephi)
	  !print '(" FLAG SOL")'
      else
          print '(T10,"  --> Calcul champ lointain en espace libre ")'
          call champ_lointain_calcul_eth_eph(cl,rcd,tv)	
      end if
      !--- Calcul du champ lointain  pour une image ---
      if (cl%flag_image .and. (.not.cl%flag_image_ideale)) then
	! Calcul du champ lointain du aux sources virtuelles
	print '(T10,"  --> Calcul champ lointain avec principe des images")'
	call champ_lointain_modif_surf_J_M(cl)   
	call champ_lointain_image_eth_eph(cl,rcd,tv)	
	call champ_lointain_modif_surf_J_M(cl)  ! retour au valeur initiale 
      end if

    end if

      
    !--- Calcul de la directivité ---
    if (rcd%flag_dth .or. rcd%flag_dph) then
      allocate(rcd%dtheta(1:rcd%tza%nbcoupleangle , 1:cl%tf%n_liste))
      allocate(rcd%dphi(1:rcd%tza%nbcoupleangle , 1:cl%tf%n_liste))
      rcd%dtheta(:,:) = 0.0
      rcd%dphi(:,:) = 0.0
      if (cl%flag_sol) then
	print '(T10,"  --> Calcul directivité avec présence sol")'
	call chl_demi_espace_calcul_dth_dph(cl%tcde,cl%tf%n_liste,rcd%tza,rcd%pr,&
	                                 rcd%etheta,rcd%ephi,rcd%dtheta,rcd%dphi)
      else
          print '(T10,"  --> Calcul directivité avec présence ou non d''une image")'
	  do nf = 1, cl%tf%n_liste
	    if (REAL(rcd%pr(nf)) == 0.0) then
	      rcd%dtheta(:,nf) =   1e-37
	      rcd%dphi(:,nf)   =   1e-37
	    else
	      if (cl%flag_image) then
		pr_normalise = PI /(cl%zmilieu * REAL(rcd%pr(nf)))
	      else
		pr_normalise = 2 * PI /(cl%zmilieu * REAL(rcd%pr(nf)))
	      end if
	    
	      rcd%dtheta(:,nf) =   pr_normalise* CABS(rcd%etheta(:,nf))**2
	      rcd%dphi(:,nf) =   pr_normalise* CABS(rcd%ephi(:,nf))**2
	      rcd%dtheta(:,nf) = MAX(rcd%dtheta(:,nf),1e-37)
	      rcd%dphi(:,nf)   = MAX(rcd%dphi(:,nf),1e-37)
	    end if
          end do
      end if
    end if

  end do
  
end do
				
End Subroutine champ_lointain_freq_postcalcul


!===========================================================	
! Subroutine champ_lointain_modif_surf_J_M
!  Dans le cas d'une image, des sources de courant virtuelles
! sont calculées sur la base des originaux en changant les signe
! de ceux-ci.
! Les surfaces sont aussi changées
!===========================================================	
Subroutine champ_lointain_modif_surf_J_M(cl)
implicit none
!=== Déclaration des arguments ===
type(st_un_champ_lointain), intent(inout) :: cl

!===  Déclaration des variables  ===
integer, parameter  :: n = 3 
integer, dimension(0:1) :: inverse =(/1,0/) 
integer, dimension(n,n) :: cj1 = RESHAPE((/-1,-1,1,  1,-1,-1,  -1,1,-1/), (/3,3/))
integer, dimension(n,n) :: cj2 = RESHAPE((/-1,1,-1,  -1,-1,1,  1,-1,-1/), (/3,3/))
integer, dimension(n,n) :: cm1 = RESHAPE((/ 1,1,-1,  -1,1,1,   1,-1,1/), (/3,3/))
integer, dimension(n,n) :: cm2 = RESHAPE((/ 1,-1,1,  1,1,-1,  -1,1,1/), (/3,3/))
integer  :: l, pref
type(st_pointeur_champ_surface), pointer  :: pcs

!===   Début des traitements   ===
if (cl%type_image==MUR_MAGNETIQUE) then
  cj1 = -cj1; cj2 = -cj2
  cm1 = -cm1; cm2 = -cm2  
end if

! boucle sur toutes les faces
pref = cl%plan_image
do l=1,NB_SURFACE_HUYGENS_MAX
  if (cl%tsurfhuyg%flag_surface(l)) then
    pcs => cl%tpchamp(l)    

    !initialisation
    if (l==XOY_INF .or. l==YOZ_INF .or.l==XOZ_INF) then
      pcs%dec_position = 1
    else
      pcs%dec_position = 0
    end if
    pcs%signe_i = 1
    pcs%signe_j = 1
    
    select case(l) 
    case(1,2) !face xoy inf et sup
      
      pcs%J1(:,:,:) = pcs%J1(:,:,:) * cj1(PLAN_XOY,pref)
      pcs%J2(:,:,:) = pcs%J2(:,:,:) * cj2(PLAN_XOY,pref)
      pcs%M1(:,:,:) = pcs%M1(:,:,:) * cm1(PLAN_XOY,pref)
      pcs%M2(:,:,:) = pcs%M2(:,:,:) * cm2(PLAN_XOY,pref)
      if (pref == PLAN_XOY) then
        pcs%position = 2* cl%position_plan_image -pcs%position
        pcs%dec_position = inverse(pcs%dec_position)
      end if
      if (pref == PLAN_YOZ) then !x/sym
        pcs%ideb     = 2* cl%position_plan_image - pcs%ideb
        pcs%ifin     = 2* cl%position_plan_image - pcs%ifin
        pcs%signe_i = -1
      end if
      if (pref == PLAN_XOZ) then !y/sym
        pcs%jdeb     = 2* cl%position_plan_image - pcs%jdeb
        pcs%jfin     = 2* cl%position_plan_image - pcs%jfin
        pcs%signe_j = -1
      end if
      print '(T10,"Face n°",I2,", (ideb,ifin,jdeb,jfin) =",4I5,", position, dec_position =",2I4,", signe i et j=",2I3)',&
           l,pcs%ideb,pcs%ifin,pcs%jdeb,pcs%jfin,pcs%position,pcs%dec_position,pcs%signe_i,pcs%signe_j
    case(3,4) !face yoz inf et sup

      pcs%J1(:,:,:) = pcs%J1(:,:,:) * cj1(PLAN_YOZ,pref)
      pcs%J2(:,:,:) = pcs%J2(:,:,:) * cj2(PLAN_YOZ,pref)
      pcs%M1(:,:,:) = pcs%M1(:,:,:) * cm1(PLAN_YOZ,pref)
      pcs%M2(:,:,:) = pcs%M2(:,:,:) * cm2(PLAN_YOZ,pref)
      if (pref == PLAN_XOY) then !z/sym
        pcs%jdeb     = 2* cl%position_plan_image - pcs%jdeb
        pcs%jfin     = 2* cl%position_plan_image - pcs%jfin
        pcs%signe_j = -1
      end if
      if (pref == PLAN_YOZ) then
        pcs%position = 2* cl%position_plan_image -pcs%position
        pcs%dec_position = inverse(pcs%dec_position)
      end if
      if (pref == PLAN_XOZ) then !y/sym
        pcs%ideb     = 2* cl%position_plan_image - pcs%ideb
        pcs%ifin     = 2* cl%position_plan_image - pcs%ifin
        pcs%signe_i = -1
      end if
      print '(T10,"Face n°",I2,", (ideb,ifin,jdeb,jfin) =",4I5,", position, dec_position =",2I4,", signe i et j=",2I3)',&
           l,pcs%ideb,pcs%ifin,pcs%jdeb,pcs%jfin,pcs%position,pcs%dec_position,pcs%signe_i,pcs%signe_j
 
    case(5,6) !face zox inf et sup
      
      pcs%J1(:,:,:) = pcs%J1(:,:,:) * cj1(PLAN_XOZ,pref)
      pcs%J2(:,:,:) = pcs%J2(:,:,:) * cj2(PLAN_XOZ,pref)
      pcs%M1(:,:,:) = pcs%M1(:,:,:) * cm1(PLAN_XOZ,pref)
      pcs%M2(:,:,:) = pcs%M2(:,:,:) * cm2(PLAN_XOZ,pref)
      if (pref == PLAN_XOY) then !z/sym
        pcs%jdeb     = 2* cl%position_plan_image - pcs%jdeb
        pcs%jfin     = 2* cl%position_plan_image - pcs%jfin
        pcs%signe_j = -1
      end if
      if (pref == PLAN_XOZ) then 
        pcs%position = 2* cl%position_plan_image -pcs%position
        pcs%dec_position = inverse(pcs%dec_position)	
      end if
      if (pref == PLAN_YOZ) then !x/sym
        pcs%ideb     = 2* cl%position_plan_image - pcs%ideb
        pcs%ifin     = 2* cl%position_plan_image - pcs%ifin
        pcs%signe_i = -1
      end if
      print '(T10,"FACE n°",I2,", (ideb,ifin,jdeb,jfin) =",4I5,", position, dec_position =",2I4,", signe i et j=",2I3)',&
           l,pcs%ideb,pcs%ifin,pcs%jdeb,pcs%jfin,pcs%position,pcs%dec_position,pcs%signe_i,pcs%signe_j
    end select

  end if
end do

End Subroutine champ_lointain_modif_surf_J_M


!===========================================================	
! Subroutine calcul_knum_meth_newton
! calcul du knumerique par un processus itératif de newton
!===========================================================	
function calcul_knum_meth_newton(nf,f,vphase,theta,phi,dxyz,dt) result(jknum)
!=== Déclaration des arguments ===
integer, intent(in) :: nf
real,dimension(1:nf),intent(in) :: f
real,intent(in)                :: vphase,theta,phi,dxyz(3),dt
!===  Déclaration des variables  ===
complex, dimension(1:nf)  :: jknum
!===   Début des traitements   ===
integer :: i,j
real :: d_dx2,d_dy2,d_dz2,Ax,Ay,Az,B
real :: ki,ki1,ecart
real :: sintetacosfi,sintetasinfi,costeta

Ax = sin(theta)*cos(phi)*dxyz(1)/2
Ay = sin(theta)*sin(phi)*dxyz(2)/2
Az = cos(theta)*dxyz(3)/2

do i=1,nf
  B  = ( (1/(vphase*dt))*sin(pi*f(i)*dt))**2
  ki = 2*pi*f(i)/vphase
  
  d_dx2 = (1/dxyz(1))**2;d_dy2 = (1/dxyz(2))**2;d_dz2 = (1/dxyz(3))**2;
  ecart = 1.0
  do j=1,30
    ki1 = ki - (d_dx2*(sin(Ax*ki))**2 + d_dy2*(sin(Ay*ki))**2 + d_dz2*(sin(Az*ki))**2 -B) / &
       (Ax*d_dx2*sin(2*Ax*ki) + Ay*d_dy2*sin(2*Ay*ki) + Az*d_dz2*sin(2*Az*ki) )      
    ! Evaluation de l'erreur
    ecart = abs(ki1 - ki)/ki
    ki=ki1
    if (ecart<1e-6) exit
  end do
  !print '(T10,"f,theta,phi=",3ES12.2,"  (ki1-ki)/ki=",ES15.2)',f(i),theta,phi,ecart/ki
  jknum(i)=CMPLX(0.0,1.0)*ki1
end do

end function calcul_knum_meth_newton
!===========================================================	
! Subroutine champ_lointain_calcul_eth_eph
!  Calcul des champ Etheta et Ephi en fonction de la fréquence
! A partir des sources de courant de la surface de Huygens
! L'algorithme est mise en oeuvre de manière particulière
! afin que la vectorisation puisse s'appliquer.
!===========================================================	
Subroutine champ_lointain_calcul_eth_eph(cl,rcd,tv)
implicit none
!=== Déclaration des arguments ===
type(st_un_champ_lointain), intent(inout) :: cl
type(st_resultat_champ_directivite), intent(inout) :: rcd
type(st_volume), intent(in) :: tv

!===  Déclaration des variables  ===
integer   :: i,j,k,l,nf,n
real      :: unsur4pi
complex, allocatable, dimension(:)  :: jk
complex, allocatable, dimension(:)  :: Ntheta, Nphi, Ltheta, Lphi
complex   :: Nteta, Nfi, Lteta, Lfi
real      :: dxdy, dydz, dxdz
real      :: theta, phi
real	  :: dxsintetacosfi, dysintetasinfi, dzcosteta
real	  :: cosfi,sinteta,sinfi,costetacosfi, costetasinfi
real      :: position_J, position_M
integer      :: isrc, jsrc, ksrc
real, dimension(2) :: rj1, rj2, rm1, rm2
type(st_pointeur_champ_surface), pointer  :: pcs
real      :: dr

!===   Début des traitements   ===
unsur4pi = 1 / (4 * PI)
! Calcul des surfaces élémentaires
dxdy = tv%celx*tv%cely
dydz = tv%cely*tv%celz
dxdz = tv%celx*tv%celz
if (rang == RANG_MAITRE_AFF) print '(T10,"Centre de phase =",3F10.2)',cl%origine
! Calcul de j*k sur la bande de fréquence
allocate(jk(1:cl%tf%n_liste))
jk(:) = CMPLX(0.0,1.0) * 2 * PI * cl%tf%liste(:) / cl%vphase

!  Composantes des vecteurs de phase
allocate(Ntheta(1:cl%tf%n_liste))
allocate(Ltheta(1:cl%tf%n_liste))
allocate(Nphi(1:cl%tf%n_liste))
allocate(Lphi(1:cl%tf%n_liste))
! Boucle sur tous les couples d'angle (theta,phi)
do n = 1, rcd%tza%nbcoupleangle
  Ntheta(:) = CMPLX(0.0,0.0);  Ltheta(:) = CMPLX(0.0,0.0);
  Nphi(:)   = CMPLX(0.0,0.0);  Lphi(:)   = CMPLX(0.0,0.0);
  phi   = rcd%tza%liste_couple_angle(C_PHI,n)
  theta = rcd%tza%liste_couple_angle(C_THETA,n)
  
  cosfi = cos(phi);
  sinteta        = sin(theta)
  sinfi          = sin(phi);
  costetacosfi   = cos(theta)* cos(phi);
  costetasinfi   = cos(theta)* sin(phi);  
  dxsintetacosfi = tv%celx * sin(theta)* cos(phi);
  dysintetasinfi = tv%cely * sin(theta)* sin(phi);
  dzcosteta      = tv%celz * cos(theta)
  ! Dispersion numérique de prise en compte
  if (cl%flag_dispersion) then
    dr = sqrt(tv%dxyz(1)**2 *(sinteta*cosfi)**4+tv%dxyz(2)**2*(sinteta*sinfi)**4+tv%dxyz(3)**2*cos(theta)**4)
    jk(:) = CMPLX(0.0,1.0)*(2/dr)*asin(dr/(cl%vphase*tv%dt)*sin(PI*cl%tf%liste(:)*tv%dt))
    !jk = calcul_knum_meth_newton(cl%tf%n_liste,cl%tf%liste,cl%vphase,theta,phi,tv%dxyz,tv%dt)
  end if
 ! boucle sur toutes les faces de la surface de Huygens
  do l=1, 6
    if (cl%tsurfhuyg%flag_surface(l)) then
      pcs => cl%tpchamp(l)    
      select case(l) 

      case(1,2) !face xoy inf et sup
! Par défaut les variables sont partagées	
!!$OMP PARALLEL PRIVATE(i,j,nf,rj1,rm1,rj2,rm2,position_J,position_M,jsrc,isrc) &
!!       !$OMP PRIVATE(rscosfiJ1,rscosfiJ2,rscosfiM1,rscosfiM2,Nteta,Lteta,Nfi,Lfi)
	Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);

	! position surface J et M suivant z
        position_J = pcs%position - cl%origine(3) ! comme E
        position_M = position_J                   ! comme H  
	if (l == 1) position_M = position_M -1
	
	rj1(1) = position_J * dzcosteta ; rj2(1) = rj1(1)
	rm1(1) = (position_M + 0.5) * dzcosteta ; rm2(1) = rm1(1)
	! boucle suivant y
!!$OMP DO SCHEDULE(RUNTIME)
        do j = 1,pcs%n2
	  jsrc = j -1 + pcs%jdeb - cl%origine(2)
	  
	  rj1(2) = jsrc * dysintetasinfi
	  rm2(2) = jsrc * dysintetasinfi
	  rj2(2) = (jsrc+0.5) * dysintetasinfi
	  rm1(2) = (jsrc+0.5) * dysintetasinfi
	  
	  ! boucle suivant x
          do i = 1,pcs%n1
	    isrc = i -1 + pcs%ideb - cl%origine(1)
	    pcs%rscosfiJ1(i) = rj1(1) + rj1(2) + (isrc+0.5) * dxsintetacosfi
	    pcs%rscosfiM2(i) = rm2(1) + rm2(2) + (isrc+0.5) * dxsintetacosfi
	    pcs%rscosfiJ2(i) = rj2(1) + rj2(2) + isrc * dxsintetacosfi
	    pcs%rscosfiM1(i) = rm1(1) + rm1(2) + isrc * dxsintetacosfi
          end do
	  	  
          ! Boucle sur toutes les fréquences
          do nf = 1, cl%tf%n_liste
	    ! boucle suivant x
            do i = 1,pcs%n1
		 ! Calcul de exp(2pikr'cos(FI))
	       !Calcul des vecteurs de phase pour chaque fréquence
	       Nteta = Nteta + pcs%J1(i,j,nf) * costetacosfi * CEXP(jk(nf) * pcs%rscosfiJ1(i)) &
	                     + pcs%J2(i,j,nf) * costetasinfi * CEXP(jk(nf) * pcs%rscosfiJ2(i))   
	       Lteta = Lteta + pcs%M1(i,j,nf) * costetacosfi * CEXP(jk(nf) * pcs%rscosfiM1(i)) &
	                     + pcs%M2(i,j,nf) * costetasinfi * CEXP(jk(nf) * pcs%rscosfiM2(i))   
			     
	       Nfi   = Nfi   - pcs%J1(i,j,nf) * sinfi * CEXP(jk(nf) * pcs%rscosfiJ1(i))  &
	                     + pcs%J2(i,j,nf) * cosfi * CEXP(jk(nf) * pcs%rscosfiJ2(i))   
	       Lfi   = Lfi   - pcs%M1(i,j,nf) * sinfi * CEXP(jk(nf) * pcs%rscosfiM1(i)) &
	                     + pcs%M2(i,j,nf) * cosfi * CEXP(jk(nf) * pcs%rscosfiM2(i))   

	     end do
	     Ntheta(nf) = Ntheta(nf) + Nteta * dxdy
	     Ltheta(nf) = Ltheta(nf) + Lteta * dxdy
	     Nphi(nf)   = Nphi(nf)   + Nfi * dxdy
 	     Lphi(nf)   = Lphi(nf)   + Lfi * dxdy
	     Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	     Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);
	   end do
	 end do
!!$OMP END DO
!!$OMP END PARALLEL

      case(3,4) !face yoz inf et sup
!!$OMP PARALLEL PRIVATE(i,j,nf,rj1,rm1,rj2,rm2,position_J,position_M,jsrc,isrc,ksrc) &
!       !$OMP PRIVATE(rscosfiJ1,rscosfiJ2,rscosfiM1,rscosfiM2,Nteta,Lteta,Nfi,Lfi)
	
	Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);

	! position surface J et M suivant x
        position_J = pcs%position - cl%origine(1) ! comme E
        position_M = position_J                   ! comme H  
	if (l == 3) position_M = position_M -1
	
	rj1(1) = position_J * dxsintetacosfi ; rj2(1) = rj1(1)
	rm1(1) = (position_M + 0.5) * dxsintetacosfi ; rm2(1) = rm1(1)
        
        ! boucle suivant z
!!$OMP DO SCHEDULE(RUNTIME)
        do j = 1,pcs%n2
	  ksrc = j -1 + pcs%jdeb - cl%origine(3)
	  
	  rj1(2) = ksrc * dzcosteta
	  rm2(2) = ksrc * dzcosteta
	  rj2(2) = (ksrc+0.5) * dzcosteta
	  rm1(2) = (ksrc+0.5) * dzcosteta
	  
	  ! boucle suivant y
          do i = 1,pcs%n1
	    jsrc = i -1 + pcs%ideb - cl%origine(2)
	    pcs%rscosfiJ1(i) = rj1(1) + rj1(2) + (jsrc+0.5) * dysintetasinfi
	    pcs%rscosfiM2(i) = rm2(1) + rm2(2) + (jsrc+0.5) * dysintetasinfi
	    pcs%rscosfiJ2(i) = rj2(1) + rj2(2) + jsrc * dysintetasinfi
	    pcs%rscosfiM1(i) = rm1(1) + rm1(2) + jsrc * dysintetasinfi
          end do

          ! Boucle sur toutes les fréquences
          do nf = 1, cl%tf%n_liste
	    ! boucle suivant x
            do i = 1,pcs%n1
		 ! Calcul de exp(2pikr'cos(FI))
	       !Calcul des vecteurs de phase pour chaque fréquence
	       Nteta = Nteta + pcs%J1(i,j,nf) * costetasinfi * CEXP(jk(nf)* pcs%rscosfiJ1(i)) &
	                     - pcs%J2(i,j,nf) * sinteta      * CEXP(jk(nf)* pcs%rscosfiJ2(i))   
	       Lteta = Lteta + pcs%M1(i,j,nf) * costetasinfi * CEXP(jk(nf)* pcs%rscosfiM1(i)) &
	                     - pcs%M2(i,j,nf) * sinteta      * CEXP(jk(nf)* pcs%rscosfiM2(i))
			     
	       Nfi   = Nfi   + pcs%J1(i,j,nf) * cosfi * CEXP(jk(nf)*pcs%rscosfiJ1(i))
	       Lfi   = Lfi   + pcs%M1(i,j,nf) * cosfi * CEXP(jk(nf)*pcs%rscosfiM1(i))
	     end do
	     Ntheta(nf) = Ntheta(nf) + Nteta * dydz
	     Ltheta(nf) = Ltheta(nf) + Lteta * dydz
	     Nphi(nf)   = Nphi(nf)   + Nfi * dydz
	     Lphi(nf)   = Lphi(nf)   + Lfi * dydz
	     Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	     Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);
	   end do
	  
        end do
!!$OMP END DO
!!$OMP END PARALLEL

      case(5,6) !face xoz inf et sup
!!$OMP PARALLEL PRIVATE(i,j,nf,rj1,rm1,rj2,rm2,position_J,position_M,jsrc,isrc,ksrc) &
!       !$OMP PRIVATE(rscosfiJ1,rscosfiJ2,rscosfiM1,rscosfiM2,Nteta,Lteta,Nfi,Lfi)
	
	Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);

	! position surface J et M suivant y
        position_J = pcs%position - cl%origine(2) ! comme E
        position_M = position_J                   ! comme H  
	if (l == 5) position_M = position_M -1
	
	rj1(1) = position_J * dysintetasinfi ; 
	rj2(1) = rj1(1)
	rm1(1) = (position_M + 0.5) * dysintetasinfi ; 
	rm2(1) = rm1(1)

        ! boucle suivant z
!!$OMP DO SCHEDULE(RUNTIME)
        do j = 1,pcs%n2
	  ksrc = j -1 + pcs%jdeb - cl%origine(3)
	  
	  rj1(2) = (ksrc+0.5) * dzcosteta
	  rm2(2) = (ksrc+0.5) * dzcosteta
	  rj2(2) = ksrc * dzcosteta
	  rm1(2) = ksrc * dzcosteta
	  
	  ! boucle suivant x
          do i = 1,pcs%n1
	    isrc = i -1 + pcs%ideb - cl%origine(1)
	    pcs%rscosfiJ1(i) = rj1(1) + rj1(2) + isrc * dxsintetacosfi
	    pcs%rscosfiM2(i) = rm2(1) + rm2(2) + isrc * dxsintetacosfi
	    pcs%rscosfiJ2(i) = rj2(1) + rj2(2) + (isrc+0.5) * dxsintetacosfi
	    pcs%rscosfiM1(i) = rm1(1) + rm1(2) + (isrc+0.5) * dxsintetacosfi
          end do
	  
          ! Boucle sur toutes les fréquences
          do nf = 1, cl%tf%n_liste
	    ! boucle suivant x
            do i = 1,pcs%n1
		 ! Calcul de exp(2pikr'cos(FI))
	       !Calcul des vecteurs de phase pour chaque fréquence	       
	       Nteta = Nteta - pcs%J1(i,j,nf) * sinteta      * CEXP(jk(nf)*pcs%rscosfiJ1(i)) &
	                     + pcs%J2(i,j,nf) * costetacosfi * CEXP(jk(nf)*pcs%rscosfiJ2(i))
	       Lteta = Lteta - pcs%M1(i,j,nf) * sinteta      * CEXP(jk(nf)*pcs%rscosfiM1(i)) &
	                     + pcs%M2(i,j,nf) * costetacosfi * CEXP(jk(nf)*pcs%rscosfiM2(i))
			     
	       Nfi   = Nfi   - pcs%J2(i,j,nf) * sinfi * CEXP(jk(nf)*pcs%rscosfiJ2(i))
	       Lfi   = Lfi   - pcs%M2(i,j,nf) * sinfi * CEXP(jk(nf)*pcs%rscosfiM2(i))
	     end do
	     Ntheta(nf) = Ntheta(nf) + Nteta * dxdz
	     Ltheta(nf) = Ltheta(nf) + Lteta * dxdz
	     Nphi(nf)   = Nphi(nf)   + Nfi * dxdz
	     Lphi(nf)   = Lphi(nf)   + Lfi * dxdz
	     Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	     Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);
	   end do
         end do
!!$OMP END DO
!!$OMP END PARALLEL

      end select
    end if
  end do


  !---- Calcul de Etheta et Ephi normalisé, ----
  ! le terme exp(-jkr) / r n'est pas inclu !
  do nf = 1, cl%tf%n_liste
    rcd%etheta(n,nf) =  - jk(nf) * unsur4pi * (Lphi(nf) + cl%zmilieu * Ntheta(nf))
    rcd%ephi(n,nf)   =    jk(nf) * unsur4pi * (Ltheta(nf) - cl%zmilieu * Nphi(nf))
    if (CABS(rcd%etheta(n,nf)) == 0.0) rcd%etheta(n,nf) =   1e-37
    if (CABS(rcd%ephi(n,nf)) == 0.0) rcd%ephi(n,nf) =   1e-37
  end do

end do

deallocate(Ntheta, Nphi, Ltheta, Lphi)
deallocate(jk)
  				
End Subroutine champ_lointain_calcul_eth_eph

!===========================================================	
! Subroutine champ_lointain_image_eth_eph
!  Calcul des champ Etheta et Ephi en fonction de la fréquence
! A partir des sources de courant de la surface de Huygens
! virtuelle, symétrique de la surface réelle par rapport au plan de
! masse. Principe des images appliquées
! L'algorithme est mise en oeuvre de manière particulière
! afin que la vectorisation puisse s'appliquer.
!===========================================================	
Subroutine champ_lointain_image_eth_eph(cl,rcd,tv)
implicit none
!=== Déclaration des arguments ===
type(st_un_champ_lointain), intent(inout) :: cl
type(st_resultat_champ_directivite), intent(inout) :: rcd
type(st_volume), intent(in) :: tv

!===  Déclaration des variables  ===
integer   :: i,j,k,l,nf,n
real      :: unsur4pi
complex, allocatable, dimension(:)  :: jk
complex, allocatable, dimension(:)  :: Ntheta, Nphi, Ltheta, Lphi
complex   :: Nteta, Nfi, Lteta, Lfi
real      :: dxdy, dydz, dxdz
real      :: theta, phi
real	  :: dxsintetacosfi, dysintetasinfi, dzcosteta
real	  :: cosfi,sinteta,sinfi,costetacosfi, costetasinfi
real      :: position_J, position_M
real      :: s_0_5
integer      :: isrc, jsrc, ksrc
real, dimension(2) :: rj1, rj2, rm1, rm2
real     :: dr
type(st_pointeur_champ_surface), pointer  :: pcs

!===   Début des traitements   ===
unsur4pi = 1 / (4 * PI)
! Calcul des surfaces élémentaires
dxdy = tv%celx*tv%cely
dydz = tv%cely*tv%celz
dxdz = tv%celx*tv%celz

! Calcul de j*k sur la bande de fréquence
allocate(jk(1:cl%tf%n_liste))
jk(:) = CMPLX(0.0,1.0) * 2 * PI * cl%tf%liste(:) / cl%vphase

!  Composantes des vecteurs de phase
allocate(Ntheta(1:cl%tf%n_liste))
allocate(Ltheta(1:cl%tf%n_liste))
allocate(Nphi(1:cl%tf%n_liste))
allocate(Lphi(1:cl%tf%n_liste))
! Boucle sur tous les couples d'angle (theta,phi)
do n = 1, rcd%tza%nbcoupleangle
  Ntheta(:) = CMPLX(0.0,0.0);  Ltheta(:) = CMPLX(0.0,0.0);
  Nphi(:)   = CMPLX(0.0,0.0);  Lphi(:)   = CMPLX(0.0,0.0);
  phi   = rcd%tza%liste_couple_angle(C_PHI,n)
  theta = rcd%tza%liste_couple_angle(C_THETA,n)
  
  cosfi = cos(phi);
  sinteta        = sin(theta)
  sinfi          = sin(phi);
  costetacosfi   = cos(theta)* cos(phi);
  costetasinfi   = cos(theta)* sin(phi);  
  dxsintetacosfi = tv%celx * sin(theta)* cos(phi);
  dysintetasinfi = tv%cely * sin(theta)* sin(phi);
  dzcosteta      = tv%celz * cos(theta)
  
  ! Dispersion numérique de prise en compte
  if (cl%flag_dispersion) then
    dr = sqrt(tv%dxyz(1)**2 *(sinteta*cosfi)**4+tv%dxyz(2)**2*(sinteta*sinfi)**4+tv%dxyz(3)**2*cos(theta)**4)
    jk(:) = CMPLX(0.0,1.0)*(2/dr)*asin(dr/(cl%vphase*tv%dt)*sin(PI*cl%tf%liste(:)*tv%dt))
    !jk = calcul_knum_meth_newton(cl%tf%n_liste,cl%tf%liste,cl%vphase,theta,phi,tv%dxyz,tv%dt)
    
  end if
  
 ! boucle sur toutes les faces de la surface de Huygens
  do l=1, 6
    if (cl%tsurfhuyg%flag_surface(l)) then
      pcs => cl%tpchamp(l)    
      select case(l) 

      case(1,2) !face xoy inf et sup
	
!!$OMP PARALLEL PRIVATE(i,j,nf,rj1,rm1,rj2,rm2,position_J,position_M,jsrc,isrc,s_0_5) &
!!       !$OMP PRIVATE(rscosfiJ1,rscosfiJ2,rscosfiM1,rscosfiM2,Nteta,Lteta,Nfi,Lfi)
	Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);

	! position surface J et M
        position_J = pcs%position - cl%origine(3) ! comme E
	position_M = position_J -pcs%dec_position ! comme H
	
	rj1(1) = position_J * dzcosteta ; rj2(1) = rj1(1)
	rm1(1) = (position_M + 0.5) * dzcosteta ; rm2(1) = rm1(1)
	! boucle suivant y
!!$OMP DO SCHEDULE(RUNTIME)
        do j = 1,pcs%n2
	  jsrc = (j -1) * pcs%signe_j + pcs%jdeb - cl%origine(2)
	  s_0_5 = 0.5 * pcs%signe_j
	  rj1(2) = jsrc * dysintetasinfi
	  rm2(2) = jsrc * dysintetasinfi
	  rj2(2) = (jsrc+s_0_5) * dysintetasinfi
	  rm1(2) = (jsrc+s_0_5) * dysintetasinfi
	  
	  ! boucle suivant x
          do i = 1,pcs%n1
	    isrc = (i -1) * pcs%signe_i + pcs%ideb - cl%origine(1)
            s_0_5 = 0.5 * pcs%signe_i
	    pcs%rscosfiJ1(i) = rj1(1) + rj1(2) + (isrc+s_0_5) * dxsintetacosfi
	    pcs%rscosfiM2(i) = rm2(1) + rm2(2) + (isrc+s_0_5) * dxsintetacosfi
	    pcs%rscosfiJ2(i) = rj2(1) + rj2(2) + isrc * dxsintetacosfi
	    pcs%rscosfiM1(i) = rm1(1) + rm1(2) + isrc * dxsintetacosfi
          end do
	  	  
          ! Boucle sur toutes les fréquences
          do nf = 1, cl%tf%n_liste
	    ! boucle suivant x
            do i = 1,pcs%n1
		 ! Calcul de exp(2pikr'cos(FI))
	       !Calcul des vecteurs de phase pour chaque fréquence
	       Nteta = Nteta + pcs%J1(i,j,nf) * costetacosfi * CEXP(jk(nf) * pcs%rscosfiJ1(i)) &
	                     + pcs%J2(i,j,nf) * costetasinfi * CEXP(jk(nf) * pcs%rscosfiJ2(i))   
	       Lteta = Lteta + pcs%M1(i,j,nf) * costetacosfi * CEXP(jk(nf) * pcs%rscosfiM1(i)) &
	                     + pcs%M2(i,j,nf) * costetasinfi * CEXP(jk(nf) * pcs%rscosfiM2(i))   
			     
	       Nfi   = Nfi   - pcs%J1(i,j,nf) * sinfi * CEXP(jk(nf) * pcs%rscosfiJ1(i))  &
	                     + pcs%J2(i,j,nf) * cosfi * CEXP(jk(nf) * pcs%rscosfiJ2(i))   
	       Lfi   = Lfi   - pcs%M1(i,j,nf) * sinfi * CEXP(jk(nf) * pcs%rscosfiM1(i)) &
	                     + pcs%M2(i,j,nf) * cosfi * CEXP(jk(nf) * pcs%rscosfiM2(i))   

	     end do
	     Ntheta(nf) = Ntheta(nf) + Nteta * dxdy
	     Ltheta(nf) = Ltheta(nf) + Lteta * dxdy
	     Nphi(nf)   = Nphi(nf)   + Nfi * dxdy
 	     Lphi(nf)   = Lphi(nf)   + Lfi * dxdy
	     Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	     Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);
	   end do
	 end do
!!$OMP END DO
!!$OMP END PARALLEL
        
      case(3,4) !face yoz inf et sup
!!$OMP PARALLEL PRIVATE(i,j,nf,rj1,rm1,rj2,rm2,position_J,position_M,jsrc,isrc,s_0_5) &
!!       !$OMP PRIVATE(rscosfiJ1,rscosfiJ2,rscosfiM1,rscosfiM2,Nteta,Lteta,Nfi,Lfi)
	
	Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);

	! position surface J et M
        position_J = pcs%position - cl%origine(1) ! comme E
	position_M = position_J -pcs%dec_position ! comme H
	
	rj1(1) = position_J * dxsintetacosfi ; rj2(1) = rj1(1)
	rm1(1) = (position_M + 0.5) * dxsintetacosfi ; rm2(1) = rm1(1)
        
        ! boucle suivant z
!!$OMP DO SCHEDULE(RUNTIME)
        do j = 1,pcs%n2
	  ksrc = (j -1) * pcs%signe_j + pcs%jdeb - cl%origine(3)
	  s_0_5 = 0.5 * pcs%signe_j
	  
	  rj1(2) = ksrc * dzcosteta
	  rm2(2) = ksrc * dzcosteta
	  rj2(2) = (ksrc+s_0_5) * dzcosteta
	  rm1(2) = (ksrc+s_0_5) * dzcosteta
	  
	  ! boucle suivant y
          do i = 1,pcs%n1
	    jsrc = (i -1) * pcs%signe_i + pcs%ideb - cl%origine(2)
            s_0_5 = 0.5 * pcs%signe_i
	    pcs%rscosfiJ1(i) = rj1(1) + rj1(2) + (jsrc+s_0_5) * dysintetasinfi
	    pcs%rscosfiM2(i) = rm2(1) + rm2(2) + (jsrc+s_0_5) * dysintetasinfi
	    pcs%rscosfiJ2(i) = rj2(1) + rj2(2) + jsrc * dysintetasinfi
	    pcs%rscosfiM1(i) = rm1(1) + rm1(2) + jsrc * dysintetasinfi
          end do

          ! Boucle sur toutes les fréquences
          do nf = 1, cl%tf%n_liste
	    ! boucle suivant x
            do i = 1,pcs%n1
		 ! Calcul de exp(2pikr'cos(FI))
	       !Calcul des vecteurs de phase pour chaque fréquence
	       Nteta = Nteta + pcs%J1(i,j,nf) * costetasinfi * CEXP(jk(nf)* pcs%rscosfiJ1(i)) &
	                     - pcs%J2(i,j,nf) * sinteta      * CEXP(jk(nf)* pcs%rscosfiJ2(i))   
	       Lteta = Lteta + pcs%M1(i,j,nf) * costetasinfi * CEXP(jk(nf)* pcs%rscosfiM1(i)) &
	                     - pcs%M2(i,j,nf) * sinteta      * CEXP(jk(nf)* pcs%rscosfiM2(i))
			     
	       Nfi   = Nfi   + pcs%J1(i,j,nf) * cosfi * CEXP(jk(nf)*pcs%rscosfiJ1(i))
	       Lfi   = Lfi   + pcs%M1(i,j,nf) * cosfi * CEXP(jk(nf)*pcs%rscosfiM1(i))
	     end do
	     Ntheta(nf) = Ntheta(nf) + Nteta * dydz
	     Ltheta(nf) = Ltheta(nf) + Lteta * dydz
	     Nphi(nf)   = Nphi(nf)   + Nfi * dydz
	     Lphi(nf)   = Lphi(nf)   + Lfi * dydz
	     Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	     Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);
	   end do
	  
        end do
!!!$OMP END DO
!!$OMP END PARALLEL

      case(5,6) !face xoz inf et sup
!!$OMP PARALLEL PRIVATE(i,j,nf,rj1,rm1,rj2,rm2,position_J,position_M,jsrc,isrc,s_0_5) &
!!       !$OMP PRIVATE(rscosfiJ1,rscosfiJ2,rscosfiM1,rscosfiM2,Nteta,Lteta,Nfi,Lfi)
	
	Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);

	! position surface J et M
        position_J = pcs%position - cl%origine(2) ! comme E
	position_M = position_J -pcs%dec_position ! comme H
	
	rj1(1) = position_J * dysintetasinfi ; 
	rj2(1) = rj1(1)
	rm1(1) = (position_M + 0.5) * dysintetasinfi ; 
	rm2(1) = rm1(1)

        ! boucle suivant z
!!$OMP DO SCHEDULE(RUNTIME)
        do j = 1,pcs%n2
	  ksrc = (j -1) * pcs%signe_j + pcs%jdeb - cl%origine(3)
	  s_0_5 = 0.5 * pcs%signe_j
	  
	  rj1(2) = (ksrc+s_0_5) * dzcosteta
	  rm2(2) = (ksrc+s_0_5) * dzcosteta
	  rj2(2) = ksrc * dzcosteta
	  rm1(2) = ksrc * dzcosteta
	  
	  ! boucle suivant x
          do i = 1,pcs%n1
	    isrc = (i -1) * pcs%signe_i + pcs%ideb - cl%origine(1)
            s_0_5 = 0.5 * pcs%signe_i
	    pcs%rscosfiJ1(i) = rj1(1) + rj1(2) + isrc * dxsintetacosfi
	    pcs%rscosfiM2(i) = rm2(1) + rm2(2) + isrc * dxsintetacosfi
	    pcs%rscosfiJ2(i) = rj2(1) + rj2(2) + (isrc+s_0_5) * dxsintetacosfi
	    pcs%rscosfiM1(i) = rm1(1) + rm1(2) + (isrc+s_0_5) * dxsintetacosfi
          end do
	  
          ! Boucle sur toutes les fréquences
          do nf = 1, cl%tf%n_liste
	    ! boucle suivant x
            do i = 1,pcs%n1
		 ! Calcul de exp(2pikr'cos(FI))
	       !Calcul des vecteurs de phase pour chaque fréquence	       
	       Nteta = Nteta - pcs%J1(i,j,nf) * sinteta      * CEXP(jk(nf)*pcs%rscosfiJ1(i)) &
	                     + pcs%J2(i,j,nf) * costetacosfi * CEXP(jk(nf)*pcs%rscosfiJ2(i))
	       Lteta = Lteta - pcs%M1(i,j,nf) * sinteta      * CEXP(jk(nf)*pcs%rscosfiM1(i)) &
	                     + pcs%M2(i,j,nf) * costetacosfi * CEXP(jk(nf)*pcs%rscosfiM2(i))
			     
	       Nfi   = Nfi   - pcs%J2(i,j,nf) * sinfi * CEXP(jk(nf)*pcs%rscosfiJ2(i))
	       Lfi   = Lfi   - pcs%M2(i,j,nf) * sinfi * CEXP(jk(nf)*pcs%rscosfiM2(i))
	     end do
	     Ntheta(nf) = Ntheta(nf) + Nteta * dxdz
	     Ltheta(nf) = Ltheta(nf) + Lteta * dxdz
	     Nphi(nf)   = Nphi(nf)   + Nfi * dxdz
	     Lphi(nf)   = Lphi(nf)   + Lfi * dxdz
	     Nteta = CMPLX(0.0,0.0);  Lteta = CMPLX(0.0,0.0);
	     Nfi   = CMPLX(0.0,0.0);  Lfi   = CMPLX(0.0,0.0);
	   end do
         end do
!!$OMP END DO
!!$OMP END PARALLEL
        
      end select
    end if
  end do


  !---- Calcul de Etheta et Ephi normalisé, ----
  ! le terme exp(-jkr) / r n'est pas inclu !
  ! tient compte de la contribution du champ lointain de la surface de 
  ! huygens réelle calculée auparavant.
  do nf = 1, cl%tf%n_liste
    rcd%etheta(n,nf) = rcd%etheta(n,nf) - jk(nf) * unsur4pi * (Lphi(nf) + cl%zmilieu * Ntheta(nf))
    rcd%ephi(n,nf)   = rcd%ephi(n,nf) + jk(nf) * unsur4pi * (Ltheta(nf) - cl%zmilieu * Nphi(nf))
    if (CABS(rcd%etheta(n,nf)) == 0.0) rcd%etheta(n,nf) =   1e-37
    if (CABS(rcd%ephi(n,nf)) == 0.0) rcd%ephi(n,nf) =   1e-37
  end do

end do

deallocate(Ntheta, Nphi, Ltheta, Lphi)
deallocate(jk)
  				
End Subroutine champ_lointain_image_eth_eph

!===========================================================	
! Subroutine champ_lointain_Calcul_Pr
!  Calcul de la puissance complexe en fonction de la fréquence
! A partir du vecteur de Poynting sommé d
!===========================================================	
Subroutine un_champ_lointain_calcul_Pr(cl,rcd,tv)
implicit none
!=== Déclaration des arguments ===
type(st_un_champ_lointain), intent(inout) :: cl
type(st_resultat_champ_directivite), intent(inout) :: rcd
type(st_volume), intent(in) :: tv

!===  Déclaration des variables  ===
integer   :: i,j,l,lf
integer   :: infsup
real(dp)  :: signe_S
type(st_pointeur_champ_surface), pointer :: pcs
!===   Début des traitements   ===
  ! boucle sur toutes les surfaces de Huygens
  ! allocation des tableaux poour les courants en fréquence 
  do l=1, 6
    if (cl%tsurfhuyg%flag_surface(l)) then
      if (MODULO(l,2)==0) then
        infsup = -1 ! pour les faces supérieures
      else
        infsup = 1
      end if
      ! surface élémentaire
      select case (l)
      case(1,2) 
        signe_S = infsup * tv%celx * tv%cely *1.0_dp
      case(3,4) 
        signe_S = infsup * tv%cely * tv%celz *1.0_dp
      case(5,6) 
        signe_S = infsup * tv%celz * tv%celx *1.0_dp
      end select      
      
      
      pcs => cl%tpchamp(l) 
      ! boucle sur les fréquences
      do lf = 1, cl%tf%n_liste
        ! boucle sur toute la surface de huygens
        do j=1, pcs%n2
        do i=1, pcs%n1
	
          rcd%pr_dp(lf) = rcd%pr_dp(lf) +signe_S * & 
	                   ( - CONJG(pcs%J1(i,j,lf)) * pcs%M2(i,j,lf) &
	                     + CONJG(pcs%J2(i,j,lf)) * pcs%M1(i,j,lf))
	 end do; end do
      end do
    end if
  end do
  rcd%pr_dp(:) = rcd%pr_dp(:) * 0.5
  rcd%pr(:) = rcd%pr_dp(:) 
End Subroutine un_champ_lointain_calcul_Pr

!===========================================================	
! Subroutine champ_lointain_calcul_Pr_interp
!  Calcul de la puissance complexe en fonction de la fréquence
! A partir du vecteur de Poynting sommé
! Les champs sont interpolés au centre des faces de chaque cellule
!===========================================================	
Subroutine champ_lointain_calcul_Pr_interp(cl,rcd,tv)
implicit none
!=== Déclaration des arguments ===
type(st_un_champ_lointain), intent(inout) :: cl
type(st_resultat_champ_directivite), intent(inout) :: rcd
type(st_volume), intent(in) :: tv

!===  Déclaration des variables  ===
integer   :: i,j,l,lf
integer   :: infsup
real      :: signe_S
type(st_pointeur_champ_surface), pointer :: pcs
!===   Début des traitements   ===
  ! boucle sur toutes les surfaces de Huygens
  ! allocation des tableaux poour les courants en fréquence 
  do l=1, 6
    if (cl%tsurfhuyg%flag_surface(l)) then
      if (MODULO(l,2)==0) then
        infsup = -1 ! pour les faces supérieures
      else
        infsup = 1
      end if
      ! surface élémentaire
      select case (l)
      case(1,2) 
        signe_S = infsup * tv%celx * tv%cely
      case(3,4) 
        signe_S = infsup * tv%cely * tv%celz
      case(5,6) 
        signe_S = infsup * tv%celz * tv%celx
      end select      
      
      
      pcs => cl%tpchamp(l) 
      ! boucle sur les fréquences
      select case (l)
      case(1,2,3,4) 
	do lf = 1, cl%tf%n_liste ! boucle sur toutes les fréquences
	  ! boucle sur toute la surface de huygens
	  do j=1, pcs%n2-1
	  do i=1, pcs%n1-1
	    rcd%pr(lf) = rcd%pr(lf) +signe_S * 0.25 * & 
			     ( - CONJG(pcs%J1(i,j,lf)+pcs%J1(i,j+1,lf)) * &
			              (pcs%M2(i,j,lf)+pcs%M2(i,j+1,lf)) )
	   end do; end do
	  do j=1, pcs%n2-1
	  do i=1, pcs%n1-1
	    rcd%pr(lf) = rcd%pr(lf) +signe_S * 0.25 * & 
			       ( CONJG(pcs%J2(i,j,lf)+pcs%J2(i+1,j,lf)) * &
				      (pcs%M1(i,j,lf)+pcs%M1(i+1,j,lf))  )
	   end do; end do
	end do
      case(5,6) 
	do lf = 1, cl%tf%n_liste ! boucle sur toutes les fréquences
	  ! boucle sur toute la surface de huygens
	  do j=1, pcs%n2-1
	  do i=1, pcs%n1-1
	    rcd%pr(lf) = rcd%pr(lf) +signe_S * 0.25 * & 
			     ( - CONJG(pcs%J1(i,j,lf)+pcs%J1(i+1,j,lf)) * &
			              (pcs%M2(i,j,lf)+pcs%M2(i+1,j,lf)) )
	   end do; end do
	  do j=1, pcs%n2-1
	  do i=1, pcs%n1-1
	    rcd%pr(lf) = rcd%pr(lf) +signe_S * 0.25 * & 
			       ( CONJG(pcs%J2(i,j,lf)+pcs%J2(i,j+1,lf)) * &
				      (pcs%M1(i,j,lf)+pcs%M1(i,j+1,lf))  )
	   end do; end do
	end do
      end select  
      
    end if
  end do
  rcd%pr(:) = rcd%pr(:) * 0.5
  				
End Subroutine champ_lointain_calcul_Pr_interp



!===========================================================	
! Subroutine champ_lointain_freq_impression
!  Ecriture des résultats dans les fichiers de sortie
!===========================================================	
Subroutine champ_lointain_freq_impression(tcl,tfs,t_gene)
!=== Déclaration des arguments ===
type(st_champ_lointain_freq), intent(inout) :: tcl
type(st_fichier_sortie), intent(in) :: tfs
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer  :: n,l,j
integer  :: com_format
type(st_un_champ_lointain), pointer :: cl
type(st_resultat_champ_directivite), pointer  :: rcd
complex, dimension(:), allocatable   :: tnorm_f


!===   Début des traitements   ===
! boucle sur tous les champs lointains
do n=1, tcl%nchl
  cl =>tcl%l_chl(n)

  ! -- stockage ascii --
  if (tfs%format_fichier == FORMAT_ASCII) then
    ! boucle sur toutes les types de résultats
    do l=1, cl%nb_resultat
      rcd => cl%trcl(l)
      !---------- Normalisation ?-----------------
      if (rcd%sn%flag_normalisation) then
	allocate(tnorm_f(1:cl%tf%n_liste))
	call Normalise_Reponse_Freq(t_gene,cl%tf,rcd%sn,tnorm_f)
      end if

      ! etheta
      if (rcd%flag_eth) then
	!---------- Normalisation ?-----------------
	if (rcd%sn%flag_normalisation) then
	  ! normalisation pour chaque composante
	  do j=1,rcd%tza%nbcoupleangle
	    rcd%etheta(j,:) = rcd%etheta(j,:) / tnorm_f(:)
	  end do
	end if
	!---------- Ecriture -----------------
        call impression_ascii_complexe_2x(rcd%unite_eth, &
	     rcd%mantisse_champ, rcd%com_format_tf, &
	     rcd%tza%liste_couple_angle_degre(C_THETA,:), &  !abscisse 1
	     rcd%tza%liste_couple_angle_degre(C_PHI,:), &    !abscisse 2
	     rcd%etheta(:,:), &	                     !ordonnées 
	     rcd%tza%nbcoupleangle, cl%tf%n_liste)
	close(rcd%unite_eth)
      end if
      ! ephi
      if (rcd%flag_eph) then
	!---------- Normalisation ?-----------------
	if (rcd%sn%flag_normalisation) then
	  ! normalisation pour chaque composante
	  do j=1,rcd%tza%nbcoupleangle
	    rcd%ephi(j,:) = rcd%ephi(j,:) / tnorm_f(:)
	  end do
	end if
	!---------- Ecriture -----------------
        call impression_ascii_complexe_2x(rcd%unite_eph, &
	     rcd%mantisse_champ, rcd%com_format_tf, &
	     rcd%tza%liste_couple_angle_degre(C_THETA,:), &  !abscisse 1
	     rcd%tza%liste_couple_angle_degre(C_PHI,:), &    !abscisse 2
	     rcd%ephi(:,:), &	                     !ordonnées 
	     rcd%tza%nbcoupleangle, cl%tf%n_liste)
	close(rcd%unite_eph)
      
      end if
      com_format = rcd%com_format_tf
      if ((rcd%com_format_tf == MODULE_DB).or.(rcd%com_format_tf == MODULE_DB_PHASE)) then
        com_format = MODULE_DBW
      end if	
      ! dtheta
      if (rcd%flag_dth) then
        call impression_ascii_reel_2x(rcd%unite_dth, &
	     rcd%mantisse_directivite, com_format, &
	     rcd%tza%liste_couple_angle_degre(C_THETA,:), &  !abscisse 1
	     rcd%tza%liste_couple_angle_degre(C_PHI,:), &    !abscisse 2
	     rcd%dtheta(:,:), &	                     !ordonnées 
	     rcd%tza%nbcoupleangle, cl%tf%n_liste)
	close(rcd%unite_dth)
      
      end if
      ! dphi
      if (rcd%flag_dph) then
        call impression_ascii_reel_2x(rcd%unite_dph, &
	     rcd%mantisse_directivite, com_format, &
	     rcd%tza%liste_couple_angle_degre(C_THETA,:), &  !abscisse 1
	     rcd%tza%liste_couple_angle_degre(C_PHI,:), &    !abscisse 2
	     rcd%dphi(:,:), &	                     !ordonnées 
	     rcd%tza%nbcoupleangle, cl%tf%n_liste)
	close(rcd%unite_dph)
      
      end if
      ! puissance rayonnée
      if (rcd%flag_pr) then
	!---------- Normalisation ?-----------------
	if (rcd%sn%flag_normalisation) rcd%pr(:) = rcd%pr(:) / tnorm_f(:)
	!---------- Ecriture -----------------
        call impression_ascii_complexe(rcd%unite_pr, &
	     rcd%mantisse_pr, REELLE_IMAGINAIRE, &
	     cl%tf%liste, rcd%pr(:), cl%tf%n_liste,1)
	close(rcd%unite_pr)
      
      end if
      ! Libération mémoire pour la normalisation
      if (rcd%sn%flag_normalisation) then
	deallocate(tnorm_f)
      end if
      
    end do    
  end if
end do

				
End Subroutine champ_lointain_freq_impression

!===========================================================	
! Subroutine champ_lointain_freq_deallocate
!  Libération de la mémoire
!===========================================================	
Subroutine champ_lointain_freq_deallocate(tcl)
implicit none
!=== Déclaration des arguments ===
type(st_champ_lointain_freq), intent(inout) :: tcl

!===  Déclaration des variables  ===
integer  :: i, j,l
type(st_un_champ_lointain), pointer :: cl
type(st_resultat_champ_directivite), pointer  :: rcd

!===   Début des traitements   ===
! boucle sur tous les champs lointains

do i=1, tcl%nchl
  cl =>tcl%l_chl(i)
  if (cl%flag_sol) call chl_demi_espace_deallocate(cl%tcde)
  ! boucle sur toutes les surfaces de Huygens
  ! allocation des tableaux poour les courants en fréquence
  if ((.not.cl%flag_sol).or.(cl%flag_pr)) then
    do j=1,6
      if (cl%tsurfhuyg%flag_surface(j)) then
	deallocate(cl%tpchamp(j)%J1)
	deallocate(cl%tpchamp(j)%J2)
	deallocate(cl%tpchamp(j)%M1)
	deallocate(cl%tpchamp(j)%M2)
	deallocate(cl%tpchamp(j)%rscosfiJ1, cl%tpchamp(j)%rscosfiJ2)
	deallocate(cl%tpchamp(j)%rscosfiM1, cl%tpchamp(j)%rscosfiM2)
	
      end if
    end do
    deallocate(cl%tpchamp)
  end if
  
  ! boucle sur toutes les types de résultats
  do l=1, cl%nb_resultat
    rcd => cl%trcl(l)
    nullify(rcd%tza)
    if (rcd%flag_eth .or. rcd%flag_eph .or. &
        rcd%flag_dth .or. rcd%flag_dph) then
	deallocate(rcd%etheta, rcd%ephi)
    end if
    if  (rcd%flag_pr .or. rcd%flag_dth .or. rcd%flag_dph) then
      deallocate(rcd%pr)
    end if
    if ((rcd%flag_dth .or. rcd%flag_dph).and.rang==RANG_MAITRE_AFF) then
      deallocate(rcd%dtheta, rcd%dphi)
    end if
  end do
  
  deallocate(cl%trcl)
  nullify(cl%tf) ; nullify(cl%tsurfhuyg)
end do

deallocate(tcl%l_chl)
				
End Subroutine champ_lointain_freq_deallocate



End Module mfdtd_champ_lointain_freq
