!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_compress.f90
!// Purpose:     module de compression des géométries
!//   maillées se trouvant dans le bloc structure 
!// Concerne le solveur FDTD
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Avril 2008
!// Copyright:   (c) Christophe Guiffaut, équipe CNRS IRCOM/DEM
!//              Utilisation des sources réservées à l'IRCOM
!// Licence  :
!/////////////////////////////////////////////////////////////////////////////

module  mfdtd_compress
use m_constantes, only : LEN_CHAINE

use mfdtd_structure
use mfdtd_volume
use m_liste_milieu, only : st_liste_milieu_simple,liste_milieu_nu,liste_milieu_nom

implicit none
integer, parameter :: IP_1= selected_int_kind(2) ! jusqu'à +-10^2


contains

! ==========================================================
! == Subroutine compress_structure_volume                 ==
! == dans le fichier d'entrée                             ==
! ==========================================================
function compress_structure_volume(ts,v,lm) result(nbmil)
type (st_structure),intent(inout) :: ts
type(st_volume), intent(in)           :: v
type(st_liste_milieu_simple), intent(in) :: lm

integer  :: nbmil

integer  :: cbmin(3),cbmax(3)
integer  :: l,nu,i,j,k,i1,j1,k1,m
integer(IP_1), allocatable,dimension(:,:,:)  :: MilCell
integer(IP_16B),allocatable, dimension(:,:) :: tmilnew

!---------- Recherche des bornes des volumes ---------------
cbmin = 1000
cbmax = 1
do l = 1,ts%nbmilieu
  cbmin(1) = min(cbmin(1),ts%t_milieu(1,l))
  cbmax(1) = max(cbmax(1),ts%t_milieu(2,l))
  cbmin(2) = min(cbmin(2),ts%t_milieu(3,l))
  cbmax(2) = max(cbmax(2),ts%t_milieu(4,l))
  cbmin(3) = min(cbmin(3),ts%t_milieu(5,l))
  cbmax(3) = max(cbmax(3),ts%t_milieu(6,l))
end do
print '(T5,"Bornes (Imin,Jmin,Kmin)=",3I5,"  (Imax,Jmax,Kmax)= ",3I5)',cbmin,cbmax

!------------- Remplissage du tableau de cellule -----------
allocate(MilCell(cbmin(1):cbmax(1)+1,cbmin(2):cbmax(2)+1,cbmin(3):cbmax(3)+1))
MilCell = 0
do l = 1,ts%nbmilieu
  nu = liste_milieu_nu(lm,ts%nommilieu(l))
  if (nu>0) then
    do k = ts%t_milieu(5,l),ts%t_milieu(6,l)-1
    do j = ts%t_milieu(3,l),ts%t_milieu(4,l)-1
    do i = ts%t_milieu(1,l),ts%t_milieu(2,l)-1
      MilCell(i,j,k) = nu
    end do;end do;end do  
  else
    print *,'ERREUR, Nom de milieu non reconnu dans le bloc structure :',ts%nommilieu(l)
    stop 'A mon commandement, REPOS !!!'  
  end if
end do

print '(T5,"Nombre de volumes avant la compression",I7)',ts%nbmilieu
!-------------- Compression des données --------------
allocate(tmilnew(1:7,ts%nbmilieu))
tmilnew = 0
m=0 ! indice nouveau tableau milieu
do k=cbmin(3),cbmax(3)
do j=cbmin(2),cbmax(2)
do i=cbmin(1),cbmax(1)
  if (MilCell(i,j,k)>0) then
    nu = MilCell(i,j,k)
    ! Compression suivant x
    i1=i+1
    do while (MilCell(i1,j,k)==nu)
      i1=i1+1
    end do
    ! Compression suivant y
    j1=j+1
    do while (all(MilCell(i:i1-1,j1,k)==nu))
      j1=j1+1
    end do
    ! Compression suivant z
    k1=k+1
    do while (all(MilCell(i:i1-1,j:j1-1,k1)==nu))
      k1=k1+1
    end do
    
    ! Volume compressé récupéré
    m=m+1; 
    if (m>ts%nbmilieu) then
      nbmil=-1
      print '(T5,"Attention, Pas de gain de compression sur les milieux volumiques !!!")'
      return
    end if
    tmilnew(1,m) = i ;tmilnew(3,m) =j ;tmilnew(5,m) =k;
    tmilnew(2,m) = i1;tmilnew(4,m) =j1;tmilnew(6,m) =k1;
    tmilnew(7,m) = nu
    MilCell(i:i1-1,j:j1-1,k:k1-1) = 0
  end if
end do;end do; end do;
i=sum(MilCell);
if (i>0) then
  print '("Erreur, tous les volumes n''ont pas été traités !!!")'
  stop 'Pas de compressions réalisés, veuillez contacter le SAV agréé, Tél 10euros/min !!!'
end if
print '(T5,"Nombre de volume après la compression",I7)',m

!-------------------------------------------------------------------
! Correction des informations dans la structure ts :
ts%nbmilieu = m
deallocate(ts%t_milieu)
deallocate(ts%nommilieu)

allocate(ts%t_milieu(1:7,1:m))
allocate(ts%nommilieu(1:m))
ts%t_milieu=0
ts%t_milieu(7,:)=1 ! type d'objet --> par défaut
ts%t_milieu(1:6,1:m) = tmilnew(1:6,1:m)
do i=1,m
  j = tmilnew(7,i)
  ts%nommilieu(i) = liste_milieu_nom(lm,j)
end do

deallocate(MilCell);deallocate(tmilnew)
end function compress_structure_volume


! ==========================================================
! == Subroutine compress_metal                            ==
! == Compression des surfaces de type métal par plan      ==
! ==========================================================
function compress_metal(ts,v) result(nbmil)
type (st_structure),intent(inout) :: ts
type(st_volume), intent(in)           :: v

integer  :: nbmil

integer  :: cbmin(3),cbmax(3)
integer  :: imin,imax,jmin,jmax,kmin,kmax
integer  :: l,nu,i,j,k,i1,j1,k1,ii,jj,kk
integer  :: m,mxy,myz,mxz
integer(IP_16B),allocatable, dimension(:,:)    :: tmetxy,tmetyz,tmetxz
integer(IP_1), allocatable,dimension(:,:,:)  :: MilCell

!---------- Recherche des bornes des volumes ---------------
cbmin = 1000
cbmax = 1
do l = 1,ts%nbmetal
  cbmin(1) = min(cbmin(1),ts%t_metal(1,l))
  cbmax(1) = max(cbmax(1),ts%t_metal(2,l))
  cbmin(2) = min(cbmin(2),ts%t_metal(3,l))
  cbmax(2) = max(cbmax(2),ts%t_metal(4,l))
  cbmin(3) = min(cbmin(3),ts%t_metal(5,l))
  cbmax(3) = max(cbmax(3),ts%t_metal(6,l))
end do
print '(T5,"Bornes (Imin,Jmin,Kmin)=",3I5,"  (Imax,Jmax,Kmax)= ",3I5)',cbmin,cbmax

allocate(MilCell(cbmin(1):cbmax(1)+1,cbmin(2):cbmax(2)+1,cbmin(3):cbmax(3)+1))

print '(T5,"Nombre de structures de type métal avant la compression",I7)',ts%nbmetal

!------------- Remplissage du tableau de cellule -----------
MilCell = 0
do l = 1,ts%nbmetal
  imin = ts%t_metal(1,l);imax = ts%t_metal(2,l) 
  jmin = ts%t_metal(3,l);jmax = ts%t_metal(4,l) 
  kmin = ts%t_metal(5,l);kmax = ts%t_metal(6,l) 
  if (ts%t_metal(7,l)==1) then
    ! plan xoy
    do j=jmin,jmax-1
    do i=imin,imax-1
      MilCell(i,j,kmin)=IBSET(MilCell(i,j,kmin),0)
      MilCell(i,j,kmax)=IBSET(MilCell(i,j,kmax),0)
    end do; end do
    ! plan yoz
    do k=kmin,kmax-1
    do j=jmin,jmax-1
      MilCell(imin,j,k)=IBSET(MilCell(imin,j,k),1)
      MilCell(imax,j,k)=IBSET(MilCell(imax,j,k),1)
    end do; end do
    ! plan xoz
    do k=kmin,kmax-1
    do i=imin,imax-1
      MilCell(i,jmin,k)=IBSET(MilCell(i,jmin,k),2)
      MilCell(i,jmax,k)=IBSET(MilCell(i,jmax,k),2)
    end do; end do
  else if (ts%t_metal(7,l)==0) then
    ! plan xoy
    do j=jmin,jmax-1
    do i=imin,imax-1
      MilCell(i,j,kmin)=IBCLR(MilCell(i,j,kmin),0)
      MilCell(i,j,kmax)=IBCLR(MilCell(i,j,kmax),0)
    end do; end do;
    ! plan yoz
    do k=kmin,kmax-1
    do j=jmin,jmax-1
      MilCell(imin,j,k)=IBCLR(MilCell(imin,j,k),1)
      MilCell(imax,j,k)=IBCLR(MilCell(imax,j,k),1)
    end do; end do
    ! plan xoz
    do k=kmin,kmax-1
    do i=imin,imax-1
      MilCell(i,jmin,k)=IBCLR(MilCell(i,jmin,k),2)
      MilCell(i,jmax,k)=IBCLR(MilCell(i,jmax,k),2)
    end do; end do
  end if
end do

!==========================================================
!================ Compression des données =================
!==========================================================

!------------- Traitement plan XOY -------------------------
allocate(tmetxy(1:6,ts%nbmetal+10))
tmetxy = 0
m=0 ! indice nouveau tableau milieu
do k=cbmin(3),cbmax(3)
do j=cbmin(2),cbmax(2)
do i=cbmin(1),cbmax(1)
  if (btest(MilCell(i,j,k),0)) then
    ! Compression suivant x
    i1=i+1
    do while (btest(MilCell(i1,j,k),0))
      i1=i1+1
    end do
    ! Compression suivant y
    j1=j+1
    do while (all(btest(MilCell(i:i1-1,j1,k),0)))
      j1=j1+1
    end do
    
    ! Volume compressé récupéré
    m=m+1; 
    if (m>ts%nbmetal+10) then
      nbmil=-1
      print '(T5,"Attention, Pas de gain de compression sur les surface metal XY!!!")'
      return
    end if
    tmetxy(1,m) = i ;tmetxy(3,m) =j ;tmetxy(5,m) =k;
    tmetxy(2,m) = i1;tmetxy(4,m) =j1;tmetxy(6,m) =k;
    !print '("m=",I6,"  tmetxy(1:6)=",6I4)',m,tmetxy(1:6,m)
    do jj=j,j1-1
    do ii=i,i1-1
      MilCell(ii,jj,k)=IBCLR(MilCell(ii,jj,k),0)
    end do; end do;
    
  end if
end do;end do; end do;
mxy=m
print '(T5,"Nombre de surfaces metal après la compression plan xy",I7)',mxy

!------------- Traitement plan YOZ -------------------------
allocate(tmetyz(1:6,ts%nbmetal+10))
tmetyz = 0
m=0 ! indice nouveau tableau milieu
do i=cbmin(1),cbmax(1)
do k=cbmin(3),cbmax(3)
do j=cbmin(2),cbmax(2)
  if (btest(MilCell(i,j,k),1)) then
    ! Compression suivant y
    j1=j+1
    do while (btest(MilCell(i,j1,k),1))
      j1=j1+1
    end do
    ! Compression suivant z
    k1=k+1
    do while (all(btest(MilCell(i,j:j1-1,k1),1)))
      k1=k1+1
    end do
    
    ! Volume compressé récupéré
    m=m+1; 
    if (m>ts%nbmetal+10) then
      nbmil=-1
      print '(T5,"Attention, Pas de gain de compression sur les surface metal YZ!!!")'
      return
    end if
    tmetyz(1,m) = i;tmetyz(3,m) =j ;tmetyz(5,m) =k;
    tmetyz(2,m) = i;tmetyz(4,m) =j1;tmetyz(6,m) =k1;
    !print '("m=",I6,"  tmetyz(1:6)=",6I4)',m,tmetyz(1:6,m)
    do kk=k,k1-1
    do jj=j,j1-1
      MilCell(i,jj,kk)=IBCLR(MilCell(i,jj,kk),1)
    end do; end do;
    
  end if
end do;end do; end do;

myz=m

print '(T5,"Nombre de surfaces métal après la compression plan yz",I7)',myz

!------------- Traitement plan XOZ -------------------------
allocate(tmetxz(1:6,ts%nbmetal+10))
tmetxz = 0
m=0 ! indice nouveau tableau milieu
do j=cbmin(2),cbmax(2)
do k=cbmin(3),cbmax(3)
do i=cbmin(1),cbmax(1)
  if (btest(MilCell(i,j,k),2)) then
    ! Compression suivant x
    i1=i+1
    do while (btest(MilCell(i1,j,k),2))
      i1=i1+1
    end do
    ! Compression suivant z
    k1=k+1
    do while (all(btest(MilCell(i:i1-1,j,k1),2)))
      k1=k1+1
    end do
    
    ! Volume compressé récupéré
    m=m+1; 
    if (m>ts%nbmetal+10) then
      nbmil=-1
      print '(T5,"Attention, Pas de gain de compression sur les surface metal XZ!!!")'
      return
    end if
    tmetxz(1,m) = i ;tmetxz(3,m) =j ;tmetxz(5,m) =k;
    tmetxz(2,m) = i1;tmetxz(4,m) =j ;tmetxz(6,m) =k1;
    !print '("m=",I6,"  tmetxz(1:6)=",6I4)',m,tmetxz(1:6,m)
    do kk=k,k1-1
    do ii=i,i1-1
      MilCell(ii,j,kk)=IBCLR(MilCell(ii,j,kk),2)
    end do; end do;
    
  end if
end do;end do; end do;
mxz=m

print '(T5,"Nombre de surfaces métal après la compression plan xz",I7)',mxz

i=sum(MilCell);
if (i>0) then
  print '("Erreur, toutes les surfaces de type métal n''ont pas été traités !!!")'
  stop 'Pas de compressions réalisés, veuillez contacter le SAV agréé, Tél 10euros/min !!!'
end if


!-------------------------------------------------------------------
! Correction des informations dans la structure ts :
i=ts%nbmetal
ts%nbmetal = mxy+myz+mxz
print '(T5,"Nombre de surfaces métal total = ",I7)',ts%nbmetal
print '(T5,"Facteur de compression de",F7.1,"  soit",F7.1,"%")',1.0*i/ts%nbmetal,100*(1.0-1.0*ts%nbmetal/i)
deallocate(ts%t_metal)

allocate(ts%t_metal(1:8,1:ts%nbmetal))
ts%t_metal=0
ts%t_metal(7,:)=1 ! métallisation
ts%t_metal(8,:)=1 ! type d'objet --> par défaut
ts%t_metal(1:6,1:mxy) = tmetxy(1:6,1:mxy)
ts%t_metal(1:6,mxy+1:mxy+myz) = tmetyz(1:6,1:myz)
ts%t_metal(1:6,mxy+myz+1:mxy+myz+mxz) = tmetxz(1:6,1:mxz)

deallocate(MilCell);deallocate(tmetxy);deallocate(tmetyz);deallocate(tmetxz)
end function compress_metal


! ==========================================================
! == Subroutine compress_plaque_mince                     ==
! == Compression des surfaces de type métal par plan      ==
! ==========================================================
function compress_plaque_mince(ts,v) result(nbmil)
type (st_structure),intent(inout) :: ts
type(st_volume), intent(in)       :: v

!---- Déclaration des types de structure locals ---
type st_mil_plaque_mince
  character(LEN=LEN_MOT_CLE) :: nommilieu
  real :: d
end type st_mil_plaque_mince
!---- Déclaration des variables locales ---
type (st_mil_plaque_mince), dimension(0:255) :: tpm
integer  :: nbmilieu_pm ! nbre de milieu de type plaque mince
logical  :: flag
integer  :: nbmil

integer  :: cbmin(3),cbmax(3)
integer  :: imin,imax,jmin,jmax,kmin,kmax
integer  :: l,nu,i,j,k,i1,j1,k1,ii,jj,kk
integer  :: m,mxy,myz,mxz
integer(IP_1), allocatable,dimension(:,:,:) :: MilCell
integer(IP_16B),allocatable, dimension(:,:) :: tmilold
integer(IP_16B),allocatable, dimension(:,:) :: tmilxy,tmilyz,tmilxz

!---------- Recherche des bornes des volumes ---------------

cbmin = 1000
cbmax = 1
do l = 1,ts%nbplaque
  cbmin(1) = min(cbmin(1),ts%t_plaque(l)%coord(1))
  cbmax(1) = max(cbmax(1),ts%t_plaque(l)%coord(2))
  cbmin(2) = min(cbmin(2),ts%t_plaque(l)%coord(3))
  cbmax(2) = max(cbmax(2),ts%t_plaque(l)%coord(4))
  cbmin(3) = min(cbmin(3),ts%t_plaque(l)%coord(5))
  cbmax(3) = max(cbmax(3),ts%t_plaque(l)%coord(6))
end do
print '(T5,"Bornes (Imin,Jmin,Kmin)=",3I5,"  (Imax,Jmax,Kmax)= ",3I5)',cbmin,cbmax

allocate(MilCell(cbmin(1):cbmax(1)+1,cbmin(2):cbmax(2)+1,cbmin(3):cbmax(3)+1))
allocate(tmilold(1:7,ts%nbplaque))
tmilold = 0;

!----- On compte le nombre de milieu de type plaque mince ----------
!--> on tient compte de l'épaisseur !
nbmilieu_pm = 0
tpm(0)%d = 0.0 ! gomme plaque mince
tpm(0)%nommilieu = "" 

do l = 1,ts%nbplaque
flag = .true.
  if (ts%t_plaque(l)%d==0.0) then
    i=0;
    flag = .false.
!    print '("rep0")'
  else
    do i=1,nbmilieu_pm
      if (ts%t_plaque(l)%d == tpm(i)%d .and. ts%t_plaque(l)%nommilieu == tpm(i)%nommilieu) then
        flag = .false.
!        print '("l=",I7,"  nom milieu=",A," d=",ES15.5)',l,trim(ts%t_plaque(l)%nommilieu),ts%t_plaque(l)%d
        exit
      end if
    end do
  end if
  if (flag) then
!    print '("rep2")'
    nbmilieu_pm = nbmilieu_pm+1
    i=nbmilieu_pm
    tpm(nbmilieu_pm)%d        =ts%t_plaque(l)%d
    tpm(nbmilieu_pm)%nommilieu=ts%t_plaque(l)%nommilieu
  end if
  tmilold(1:6,l) =ts%t_plaque(l)%coord(1:6)
  tmilold(7,l)   =i
end do
print '(T5,"Nombre de milieu =",I7)',nbmilieu_pm
print '(T5,"Nombre de structures de type plaque mince avant la compression",I7)',ts%nbplaque


!==========================================================
!================ Compression des données =================
!==========================================================

!------------- Traitement plan XOY -------------------------
!------------- Remplissage du tableau de cellule -----------
MilCell = 0
do l = 1,ts%nbplaque
  imin = tmilold(1,l);imax = tmilold(2,l) 
  jmin = tmilold(3,l);jmax = tmilold(4,l) 
  kmin = tmilold(5,l);kmax = tmilold(6,l) 
  if (imin==imax .or. jmin==jmax) cycle
  
  MilCell(imin:imax-1,jmin:jmax-1,kmin)=tmilold(7,l)
  if (kmax>kmin) MilCell(imin:imax-1,jmin:jmax-1,kmax)=tmilold(7,l)
end do

!--------------- COMPRESSION XOY --------------------
allocate(tmilxy(1:7,ts%nbplaque+10))
tmilxy = 0
m=0 ! indice nouveau tableau milieu
do k=cbmin(3),cbmax(3)
do j=cbmin(2),cbmax(2)
do i=cbmin(1),cbmax(1)
  if (MilCell(i,j,k)>0) then
    nu = MilCell(i,j,k)
    ! Compression suivant x
    i1=i+1
    do while (MilCell(i1,j,k)==nu)
      i1=i1+1
    end do
    ! Compression suivant y
    j1=j+1
    do while (all(MilCell(i:i1-1,j1,k)==nu))
      j1=j1+1
    end do
    
    ! Volume compressé récupéré
    m=m+1; 
    if (m>ts%nbplaque+10) then
      nbmil=-1
      print '(T5,"Attention, Pas de gain de compression sur les surface metal XY!!!")'
      return
    end if
    tmilxy(1,m) = i ;tmilxy(3,m) =j ;tmilxy(5,m) =k;
    tmilxy(2,m) = i1;tmilxy(4,m) =j1;tmilxy(6,m) =k;
    tmilxy(7,m) = nu
    MilCell(i:i1-1,j:j1-1,k)=0
    
  end if
end do;end do; end do;
mxy=m
print '(T5,"Nombre de surfaces plaque mince après la compression plan xy",I7)',mxy

i=sum(MilCell);
if (i>0) then
  print '("Erreur, toutes les surfaces de type plaque mince XOY n''ont pas été traités !!!")'
  stop 'Pas de compressions réalisés, veuillez contacter le SAV agréé, Tél 10euros/min !!!'
end if

!------------- Traitement plan YOZ -------------------------
!------------- Remplissage du tableau de cellule -----------
MilCell = 0
do l = 1,ts%nbplaque
  imin = tmilold(1,l);imax = tmilold(2,l) 
  jmin = tmilold(3,l);jmax = tmilold(4,l) 
  kmin = tmilold(5,l);kmax = tmilold(6,l) 
  if (jmin==jmax .or. kmin==kmax) cycle
  MilCell(imin,jmin:jmax-1,kmin:kmax-1)=tmilold(7,l)
  if (imax>imin) MilCell(imax,jmin:jmax-1,kmin:kmax-1)=tmilold(7,l)
end do

!--------------- COMPRESSION YOZ --------------------
allocate(tmilyz(1:7,ts%nbplaque+10))
tmilyz = 0
m=0 ! indice nouveau tableau milieu
do i=cbmin(1),cbmax(1)
do k=cbmin(3),cbmax(3)
do j=cbmin(2),cbmax(2)
  if (MilCell(i,j,k)>0) then
    nu = MilCell(i,j,k)
    ! Compression suivant y
    j1=j+1
    do while (MilCell(i,j1,k)==nu)
      j1=j1+1
    end do
    ! Compression suivant z
    k1=k+1
    do while (all(MilCell(i,j:j1-1,k1)==nu))
      k1=k1+1
    end do
    
    ! Volume compressé récupéré
    m=m+1; 
    if (m>ts%nbplaque+10) then
      nbmil=-1
      print '(T5,"Attention, Pas de gain de compression sur les surface metal YZ!!!")'
      return
    end if
    tmilyz(1,m) = i ;tmilyz(3,m) =j ;tmilyz(5,m) =k;
    tmilyz(2,m) = i ;tmilyz(4,m) =j1;tmilyz(6,m) =k1;
    tmilyz(7,m) = nu
    MilCell(i,j:j1-1,k:k1-1)=0
  end if
end do;end do; end do;
myz=m
print '(T5,"Nombre de surfaces plaque mince après la compression plan yz",I7)',myz

i=sum(MilCell);
if (i>0) then
  print '("Erreur, toutes les surfaces de type plaque mince YOZ n''ont pas été traités !!!")'
  stop 'Pas de compressions réalisés, veuillez contacter le SAV agréé, Tél 10euros/min !!!'
end if

!------------- Traitement plan XOZ -------------------------
!------------- Remplissage du tableau de cellule -----------
MilCell = 0
do l = 1,ts%nbplaque
  imin = tmilold(1,l);imax = tmilold(2,l) 
  jmin = tmilold(3,l);jmax = tmilold(4,l) 
  kmin = tmilold(5,l);kmax = tmilold(6,l) 
  if (imin==imax .or. kmin==kmax) cycle
  MilCell(imin:imax-1,jmin,kmin:kmax-1)=tmilold(7,l)
  if (jmax>jmin) MilCell(imin:imax-1,jmax,kmin:kmax-1)=tmilold(7,l)
end do

!--------------- COMPRESSION XOZ --------------------
allocate(tmilxz(1:7,ts%nbplaque+10))
tmilxz = 0
m=0 ! indice nouveau tableau milieu
do j=cbmin(2),cbmax(2)
do k=cbmin(3),cbmax(3)
do i=cbmin(1),cbmax(1)
  if (MilCell(i,j,k)>0) then
    nu = MilCell(i,j,k)
    ! Compression suivant x
    i1=i+1
    do while (MilCell(i1,j,k)==nu)
      i1=i1+1
    end do
    ! Compression suivant z
    k1=k+1
    do while (all(MilCell(i:i1-1,j,k1)==nu))
      k1=k1+1
    end do
    
    ! Volume compressé récupéré
    m=m+1; 
    if (m>ts%nbplaque+10) then
      nbmil=-1
      print '(T5,"Attention, Pas de gain de compression sur les surface metal XZ!!!")'
      return
    end if
    tmilxz(1,m) = i ;tmilxz(3,m) =j ;tmilxz(5,m) =k;
    tmilxz(2,m) = i1;tmilxz(4,m) =j ;tmilxz(6,m) =k1;
    tmilxz(7,m) = nu
    MilCell(i:i1-1,j,k:k1-1)=0
  end if
end do;end do; end do;
mxz=m
print '(T5,"Nombre de surfaces plaque mince après la compression plan xz",I7)',myz

i=sum(MilCell);
if (i>0) then
  print '("Erreur, toutes les surfaces de type plaque mince XOZ n''ont pas été traités !!!")'
  stop 'Pas de compressions réalisés, veuillez contacter le SAV agréé, Tél 10euros/min !!!'
end if


!-------------------------------------------------------------------
! Correction des informations dans la structure de plaques minces :
i=ts%nbplaque
ts%nbplaque = mxy +myz+mxz
print '(T5,"Nombre de surfaces plaque mince total = ",I7)',ts%nbplaque
print '(T5,"Facteur de compression de",F7.1,"  soit",F7.1,"%")',1.0*i/ts%nbplaque,100*(1.0-1.0*ts%nbplaque/i)
deallocate(ts%t_plaque)

allocate(ts%t_plaque(1:ts%nbplaque))
do l=1,ts%nbplaque
  ts%t_plaque(l)%nom = 'plaque_mince'
  ts%t_plaque(l)%nuobj = 1  
end do

do l=1,mxy
  nu = tmilxy(7,l)
  ts%t_plaque(l)%d = tpm(nu)%d 
  ts%t_plaque(l)%nommilieu=tpm(nu)%nommilieu 
  ts%t_plaque(l)%coord(1:6)=tmilxy(1:6,l)
end do
i=1
do l=mxy+1,mxy+myz
  nu = tmilyz(7,i)
  ts%t_plaque(l)%d = tpm(nu)%d 
  ts%t_plaque(l)%nommilieu=tpm(nu)%nommilieu 
  ts%t_plaque(l)%coord(1:6)=tmilyz(1:6,i)
  i=i+1
end do

i=1
do l=mxy+myz+1,mxy+myz+mxz
  nu = tmilxz(7,i)
  ts%t_plaque(l)%d = tpm(nu)%d 
  ts%t_plaque(l)%nommilieu=tpm(nu)%nommilieu 
  ts%t_plaque(l)%coord(1:6)=tmilxz(1:6,i)
  i=i+1
end do

deallocate(MilCell);deallocate(tmilxy);deallocate(tmilyz);deallocate(tmilxz)
deallocate(tmilold)
end function compress_plaque_mince






end module  mfdtd_compress
