!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_cpml.f90
!// Purpose:     Gestion des PMLs
!// Author:      Cyril Giraudon, Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Traitement des couches CPML                          ==
! == Module module_cpml                                   ==
! == Type st_cpml                                         ==
! == Subroutine cpml_volume                               ==
! == Subroutine cpml_lecture                              ==
! == Subroutine cpml_init                       ==
! == Subroutine cpml_precalcul                            ==
! == Subroutine cpml_calcul_h                             ==
! == Subroutine cpml_calcul_ye                            ==
! == Subroutine cpml_calcul_e                             ==
! == Subroutine cpml_calcul_yh                            ==
! == Subroutine cpml_calcul                               ==
! == Subroutine cpml_affichage                            ==
! == Subroutine cpml_deallocate                           ==
! ==========================================================

module mfdtd_cpml

use m_constantes, only : LEN_MOT_CLE,rang,RANG_MAITRE_AFF,CPML
use m_constantes_scientifiques, only : EPSILON_0,PI

use m_entree, only : st_fichier, &
                                 lineread, &
				 lecture_stop

use mfdtd_volume, only : st_volume
use mfdtd_champ , only : st_champ
use mfdtd_cpml_x, only : st_cpml_x, cpml_x_precalcul, cpml_x_init, &
                	  cpml_x_calcul_e,& 
                	  cpml_x_calcul_e1,& 
			  cpml_x_calcul_ye, &
                	  cpml_x_calcul_h, cpml_x_calcul_yh, &
			  cpml_x_deallocate, cpml_x_affichage
			  
use mfdtd_cpml_y, only : st_cpml_y, cpml_y_precalcul, cpml_y_init, &
                	  cpml_y_calcul_e, cpml_y_calcul_ye, &
                	  cpml_y_calcul_h, cpml_y_calcul_yh, &
			  cpml_y_deallocate, cpml_y_affichage
			  
use mfdtd_cpml_z, only : st_cpml_z, cpml_z_precalcul, cpml_z_init, &
                	  cpml_z_calcul_e, cpml_z_calcul_ye, &
                	  cpml_z_calcul_h, cpml_z_calcul_yh, &
			  cpml_z_deallocate, cpml_z_affichage

implicit none

integer, parameter  :: PROFIL_GEOMETRIQUE_AUTO = 0
integer, parameter  :: PROFIL_GEOMETRIQUE_MANUEL = 1


! Paramètres pour un profil géométrique de la conductivité
type st_cpml_profil_geometrique
  real  :: c_reflexion   ! coefficient de réflexion en incidence normale
  real  :: c_geom        ! coefficient géométrique
  real  :: c_amort       ! coefficient d'amortissement
  integer  :: ncell      ! nombre de cellules
end type st_cpml_profil_geometrique

! ==========================================================
! == type st_cpml                                         ==
! == Defintion du type st_cpml                            ==
! ==========================================================
type st_cpml
  logical :: flag_allocate
  integer :: ncell  ! Nbre de cellules des couches absorbantes
  integer :: iprecisg
  real :: dxinf, dyinf, dzinf ! pas spatiaux dans les PML
  real :: dxsup, dysup, dzsup ! pas spatiaux dans les PML
  real :: epsilon_defaut,vitesse, dt 
  real :: absdon, p, q, cell
  real :: tmaxi, geom, eta, aux, fcc, tc, ic
  real, dimension(:), pointer :: abseinf, abshinf, abseeinf, abshhinf
  real, dimension(:), pointer :: absesup, abshsup, abseesup, abshhsup
  integer  :: type_profil_sigma
  real  :: alpha ! coefficient cpml 
  type(st_cpml_profil_geometrique)  :: t_pg

  type(st_cpml_x), pointer :: xinf, xsup
  type(st_cpml_y), pointer :: yinf, ysup
  type(st_cpml_z), pointer :: zinf, zsup

  logical :: flag_xinf, flag_xsup
  logical :: flag_yinf, flag_ysup
  logical :: flag_zinf, flag_zsup

  !-------- Taille mémoire et Nbre d'opération par seconde ----
  real      :: taille_mem
  real      :: nb_oper
  integer   :: nb_cell_pml
end type st_cpml

contains


! ==========================================================
! == Suborutine cpml_initdefaut                           ==
! ==========================================================
subroutine cpml_profil_initdefaut(t_pg)

type(st_cpml_profil_geometrique), intent(inout) :: t_pg

t_pg%c_reflexion = 1e-3;
t_pg%c_geom = 2.2    
t_pg%c_amort = 1.0 
t_pg%ncell   = 10  
end subroutine cpml_profil_initdefaut

! ==========================================================
! == Suborutine cpml_initdefaut                           ==
! ==========================================================
subroutine cpml_initdefaut(t_cpml)

type(st_cpml), intent(inout) :: t_cpml

t_cpml%flag_allocate = .false.
t_cpml%flag_xinf = .true.
t_cpml%flag_xsup = .true.
t_cpml%flag_yinf = .true.
t_cpml%flag_ysup = .true.
t_cpml%flag_zinf = .true.
t_cpml%flag_zsup = .true.
t_cpml%absdon    = 1e-3
t_cpml%iprecisg  = 2 ! A revoir

t_cpml%alpha = 0;
t_cpml%epsilon_defaut = EPSILON_0
t_cpml%type_profil_sigma = PROFIL_GEOMETRIQUE_MANUEL
call cpml_profil_initdefaut(t_cpml%t_pg)

t_cpml%taille_mem = 0.0
t_cpml%nb_oper    = 0.0
t_cpml%nb_cell_pml=0
end subroutine cpml_initdefaut

! ==========================================================
! == Subroutine cpml_copy                                 ==
! ==========================================================	
subroutine cpml_copy(vin,vout)
! === Déclaration des arguments ===
type(st_cpml ), intent(in)       :: vin
type(st_cpml ), intent(out)      :: vout

! ===   Début des traitements   ===
vout%t_pg              = vin%t_pg
vout%alpha             = vin%alpha
vout%type_profil_sigma = vin%type_profil_sigma
vout%absdon            = vin%absdon
vout%iprecisg          = vin%iprecisg
vout%flag_allocate     = vin%flag_allocate

vout%taille_mem = vin%taille_mem
vout%nb_oper    = vin%nb_oper
vout%nb_cell_pml=vin%nb_cell_pml
end subroutine cpml_copy


! ==========================================================
! == Subroutine cpml_lecture                              ==
! == Lecture des donnees pour les cpml                    ==
! ==========================================================
subroutine cpml_lecture(t_fichier,t_cpml)


type(st_fichier), intent(inout) :: t_fichier
type(st_cpml), intent(inout) :: t_cpml

character(LEN_MOT_CLE) :: mot_cle

! ===   Début des traitements   ===
select case (t_fichier%premiermot)
  case ('cpml_alpha')
    read(t_fichier%chaine,*) mot_cle, t_cpml%alpha
  
  case ('profil_pml')
    read(t_fichier%chaine,*) mot_cle, t_cpml%absdon, t_cpml%iprecisg
    t_cpml%type_profil_sigma = PROFIL_GEOMETRIQUE_AUTO
  
  case ('profil_pml_geometrique')

    read(t_fichier%chaine,*) mot_cle, t_cpml%t_pg%ncell, &
                                   t_cpml%t_pg%c_reflexion, &
                                   t_cpml%t_pg%c_geom
    t_cpml%type_profil_sigma = PROFIL_GEOMETRIQUE_MANUEL
    t_cpml%t_pg%c_amort = 1.0

    if (t_cpml%t_pg%ncell<2) then
       write(6,*)' Erreur : nombre de cellules PML doit etre supérieur à 2'
       call lecture_stop(t_fichier)
    endif


    if (t_cpml%t_pg%c_geom<=1.) then
       write(6,*)' Erreur : progression géométrique dans PML doit etre supérieure à 1'
       call lecture_stop(t_fichier)
    endif

    if (t_cpml%t_pg%c_geom>10.) then
       write(6,*)' Erreur : progression géométrique doit etre inférieure à 10'
       call lecture_stop(t_fichier)
    endif

end select

end subroutine cpml_lecture
 

! ==========================================================
! == Subroutine cpml_init                       ==
! ==========================================================
subroutine cpml_init(c,xinf,xsup,yinf,ysup,zinf,zsup, &
                               ivold,ivolf,jvold,jvolf,kvold,kvolf)

type(st_cpml), intent(inout) :: c
integer, intent(in) :: xinf, xsup, yinf, ysup, zinf, zsup
integer, intent(in) :: ivold, ivolf, jvold, jvolf, kvold, kvolf

integer :: xinf_sup, xsup_inf
integer :: yinf_sup, ysup_inf
integer :: zinf_sup, zsup_inf

integer,dimension(6) :: borne_volume
! ===   Début des traitements   ===

borne_volume(1)=ivold; borne_volume(2)=ivolf; 
borne_volume(3)=jvold; borne_volume(4)=jvolf; 
borne_volume(5)=kvold; borne_volume(6)=kvolf; 

allocate(c%xinf)
if (xinf/=CPML) then
  c%flag_xinf = .false.
  deallocate(c%xinf)
endif

allocate(c%xsup)
if (xsup/=CPML) then
  c%flag_xsup = .false.
  deallocate(c%xsup)
endif

allocate(c%yinf)
if (yinf/=CPML) then
  c%flag_yinf = .false.
  deallocate(c%yinf)
endif

allocate(c%ysup)
if (ysup/=CPML) then
  c%flag_ysup = .false.
  deallocate(c%ysup)
endif

allocate(c%zinf)
if (zinf/=CPML) then
  c%flag_zinf = .false.
  deallocate(c%zinf)
endif

allocate(c%zsup)
if (zsup/=CPML) then
  c%flag_zsup = .false.
  deallocate(c%zsup)
endif

xinf_sup = ivold+c%ncell-1; xsup_inf = ivolf-c%ncell+1
yinf_sup = jvold+c%ncell-1; ysup_inf = jvolf-c%ncell+1
zinf_sup = kvold+c%ncell-1; zsup_inf = kvolf-c%ncell+1

if (c%flag_xinf) call cpml_x_init(c%xinf,'inf',ivold,xinf_sup)
if (c%flag_xsup) call cpml_x_init(c%xsup,'sup',xsup_inf,ivolf)
if (c%flag_yinf) call cpml_y_init(c%yinf,'inf',jvold,yinf_sup)
if (c%flag_ysup) call cpml_y_init(c%ysup,'sup',ysup_inf,jvolf)
if (c%flag_zinf) call cpml_z_init(c%zinf,'inf',kvold,zinf_sup)
if (c%flag_zsup) call cpml_z_init(c%zsup,'sup',zsup_inf,kvolf)
c%flag_allocate = .true.

call cpml_mem_nboper(c,borne_volume)
!print '("Bornes volumes=",6I5)',borne_volume
end subroutine cpml_init


! ==========================================================
! == Subroutine cpml_volume_et_profil                     ==
! == Calcul du nombre de couches cpml                     ==
! == Calcul du profil de conductivité                     ==
! == Caclul des coefficients cpml multiplicatif des champs==
! ==========================================================
integer function cpml_volume_et_profil(t_cpml,v)
           
type(st_cpml), target, intent(inout) :: t_cpml
type(st_volume), intent(inout) :: v
  
if (t_cpml%type_profil_sigma == PROFIL_GEOMETRIQUE_MANUEL) then
  cpml_volume_et_profil = cpml_profil_geom_manuel(t_cpml)
else
  cpml_volume_et_profil = cpml_volume(t_cpml,v)  
end if
t_cpml%dxinf = v%celx;t_cpml%dxsup = v%celx
t_cpml%dyinf = v%cely;t_cpml%dysup = v%cely
t_cpml%dzinf = v%celz;t_cpml%dzsup = v%celz

if (v%flag_maillage_nu) then
  t_cpml%dxinf = v%mnu(1)%pas_h(1);t_cpml%dxsup = v%mnu(1)%pas_h(v%nxyz(1));
  t_cpml%dyinf = v%mnu(2)%pas_h(1);t_cpml%dysup = v%mnu(2)%pas_h(v%nxyz(2));
  t_cpml%dzinf = v%mnu(3)%pas_h(1);t_cpml%dzsup = v%mnu(3)%pas_h(v%nxyz(3));
end if

t_cpml%dt = v%dt
t_cpml%vitesse = 1 /SQRT(v%epsilon*v%mu)
t_cpml%epsilon_defaut = v%epsilon
end function cpml_volume_et_profil

! ==========================================================
! == Subroutine cpml_nbcouche                             ==
! == Calcul du nombre de couches cpml                     ==
! ==========================================================
integer function cpml_nbcouche(t_cpml,t_volume)
           
type(st_cpml), target, intent(inout) :: t_cpml
type(st_volume), intent(inout) :: t_volume

integer :: nlong
real    :: geom20,geom100,geom,tmaxi,tc,sig0,sig,dxcouch,aux

if (t_cpml%type_profil_sigma == PROFIL_GEOMETRIQUE_MANUEL) then
  cpml_nbcouche  = t_cpml%t_pg%ncell
else
  nlong = MAXval(t_volume%nxyz)
  if (nlong>100) nlong = 100
  if (nlong<20) nlong = 20

  geom20 = 4.64
  geom100 = 2.15
  geom = geom20 - (geom20-geom100)*(nlong-20) / 80.

  if(t_cpml%iprecisg==-2) geom = geom**1.6
  if(t_cpml%iprecisg==-1) geom = geom**1.3
  if(t_cpml%iprecisg==+1) geom = geom**0.8
  if(t_cpml%iprecisg==+2) geom = geom**0.6

  ! -- PMLA, coupure a 3 tmaxi --
  tmaxi = t_volume%dt * t_volume%ntinc
  tc = 3. * tmaxi
  if(t_cpml%iprecisg==1) tc = tc*2.
  if(t_cpml%iprecisg==2) tc = tc*4.

  ! -- Determination de sigma0 et sigma(0) en fonction de tc --
  ! -  sig0 = sig(0) = conductivite FDTD dans interface
  ! -  sig  = sigo   = conductivite theorique dans interface
  sig0 = 1. / (1.8e10*tc)
  sig = sig0 * LOG(geom) / (SQRT(geom) - 1.)
  
  !-- Calcul nombre de cellules necessaires pour realiser (geom,sig0) --
  dxcouch = MIN(t_volume%celx,t_volume%cely,t_volume%celz)
  
  aux = (SQRT(geom) - 1.) * LOG(t_cpml%absdon) / (754. * dxcouch * sig0)
	
  cpml_nbcouche = 1 + INT(LOG(1.-aux) / LOG(geom))
  if (cpml_nbcouche<2) then
     print '(" ERREUR, Le nombre de cellules PML est inferieur à 2 !!!")'
     stop
  endif
end if
end function cpml_nbcouche



! ==========================================================
! == Subroutine cpml_mem_nboper                           ==
! ==   Evaluation de la taille mémoire en octets dans la
! ==    structure et du nbre d'opération flottante si impactant
! ==     sur le temps de calcul
!===========================================================	
subroutine cpml_mem_nboper(c,bv)
type(st_cpml), intent(inout) :: c
integer,dimension(6) :: bv

integer :: nbcell1,nbcell2
real :: test
test=0.0

if (c%flag_xinf) then
  nbcell1 = (c%xinf%ifin-c%xinf%idebe+1)*(bv(4)-bv(3)  )*(bv(6)-bv(5)+1)
  nbcell2 = (c%xinf%ifin-c%xinf%idebe+1)*(bv(4)-bv(3)+1)*(bv(6)-bv(5)  )
  
  c%taille_mem=c%taille_mem+(nbcell1+nbcell2)*2*kind(test)
  ! tableau additionnel pour la vectorisation 
  c%taille_mem =c%taille_mem+(c%xinf%ifin-c%xinf%idebe+1)*(bv(4)-bv(3)+1)*(bv(6)-bv(5)+1)*2*kind(test)
  c%nb_cell_pml=c%nb_cell_pml+c%ncell*(bv(4)-bv(3)+1)*(bv(6)-bv(5)+1)
  !nbre d'opérations flottantes
  c%nb_oper   =c%nb_oper+14*(nbcell1+nbcell2) ! sept opérations flottantes en moyenne par inconnu
end if

if (c%flag_xsup) then
  nbcell1 = (c%xsup%ifin-c%xsup%idebe+1)*(bv(4)-bv(3)  )*(bv(6)-bv(5)+1)
  nbcell2 = (c%xsup%ifin-c%xsup%idebe+1)*(bv(4)-bv(3)+1)*(bv(6)-bv(5)  )
  c%taille_mem=c%taille_mem+(nbcell1+nbcell2)*2*kind(test)
  ! tableau additionnel pour la vectorisation 
  c%taille_mem=c%taille_mem+(c%xsup%ifin-c%xsup%idebe+1)*(bv(4)-bv(3)+1)*(bv(6)-bv(5)+1)*2*kind(test)
  
  c%nb_cell_pml=c%nb_cell_pml+c%ncell*(bv(4)-bv(3)+1)*(bv(6)-bv(5)+1)
  !nbre d'opérations flottantes
  c%nb_oper   =c%nb_oper+14*(nbcell1+nbcell2) ! sept opérations flottantes en moyenne par inconnu  
end if

if (c%flag_yinf) then
  nbcell1 = (c%yinf%jfin-c%yinf%jdebe+1)*(bv(2)-bv(1)  )*(bv(6)-bv(5)+1)
  nbcell2 = (c%yinf%jfin-c%yinf%jdebe+1)*(bv(2)-bv(1)+1)*(bv(6)-bv(5)  )
  c%taille_mem=c%taille_mem+(nbcell1+nbcell2)*2*kind(test)
  c%nb_cell_pml=c%nb_cell_pml+c%ncell*(bv(2)-bv(1)+1)*(bv(6)-bv(5)+1)
  !nbre d'opérations flottantes
  c%nb_oper   =c%nb_oper+12*(nbcell1+nbcell2) ! six opérations flottantes en moyenne par inconnu  
end if

if (c%flag_ysup) then
  nbcell1 = (c%ysup%jfin-c%ysup%jdebe+1)*(bv(2)-bv(1)  )*(bv(6)-bv(5)+1)
  nbcell2 = (c%ysup%jfin-c%ysup%jdebe+1)*(bv(2)-bv(1)+1)*(bv(6)-bv(5)  )
  c%taille_mem=c%taille_mem+(nbcell1+nbcell2)*2*kind(test)
  c%nb_cell_pml=c%nb_cell_pml+c%ncell*(bv(2)-bv(1)+1)*(bv(6)-bv(5)+1)
  !nbre d'opérations flottantes
  c%nb_oper   =c%nb_oper+12*(nbcell1+nbcell2) ! six opérations flottantes en moyenne par inconnu  
end if

if (c%flag_zinf) then
  nbcell1 = (c%zinf%kfin-c%zinf%kdebe+1)*(bv(2)-bv(1)  )*(bv(4)-bv(3)+1)
  nbcell2 = (c%zinf%kfin-c%zinf%kdebe+1)*(bv(2)-bv(1)+1)*(bv(4)-bv(3)  )
  c%taille_mem=c%taille_mem+(nbcell1+nbcell2)*2*kind(test)
  c%nb_cell_pml=c%nb_cell_pml+c%ncell*(bv(2)-bv(1)+1)*(bv(4)-bv(3)+1)
  !nbre d'opérations flottantes
  c%nb_oper   =c%nb_oper+12*(nbcell1+nbcell2) ! six opérations flottantes en moyenne par inconnu    
end if

if (c%flag_zsup) then
  nbcell1 = (c%zsup%kfin-c%zsup%kdebe+1)*(bv(2)-bv(1)  )*(bv(4)-bv(3)+1)
  nbcell2 = (c%zsup%kfin-c%zsup%kdebe+1)*(bv(2)-bv(1)+1)*(bv(4)-bv(3)  )
  c%taille_mem=c%taille_mem+(nbcell1+nbcell2)*2*kind(test)
  c%nb_cell_pml=c%nb_cell_pml+c%ncell*(bv(2)-bv(1)+1)*(bv(4)-bv(3)+1)
  !nbre d'opérations flottantes
  c%nb_oper   =c%nb_oper+12*(nbcell1+nbcell2) ! six opérations flottantes en moyenne par inconnu    
end if

end subroutine cpml_mem_nboper

! ==========================================================
! == Subroutine cpml_precalcul                            ==
! == Calcul a effectuer avant la boucle sur le temps      ==
! ==========================================================
subroutine cpml_precalcul(c, ch)

type(st_cpml), intent(inout) :: c
type(st_champ), intent(inout)  :: ch

!--> PML suivant x

if (c%flag_xinf) then  
  print '(T5,"--> Traitement PML XINF, dxinf=",ES11.2)',c%dxinf
  if (c%type_profil_sigma == PROFIL_GEOMETRIQUE_MANUEL) then
    call cpml_absorbant_geom_manuel(c, c%dxinf)
  end if
  call cpml_x_precalcul(c%xinf,ch,c%ncell,c%abseinf,c%abseeinf,c%abshinf,c%abshhinf)
endif

if (c%flag_xsup) then
  print '(T5,"--> Traitement PML XSUP, dxsup=",ES11.2)',c%dxsup
  if (c%type_profil_sigma == PROFIL_GEOMETRIQUE_MANUEL) then
    call cpml_absorbant_geom_manuel(c, c%dxsup)
  end if
  call cpml_x_precalcul(c%xsup,ch,c%ncell,c%absesup,c%abseesup,c%abshsup,c%abshhsup)
endif

!--> PML suivant y
if (c%flag_yinf) then
  print '(T5,"--> Traitement PML YINF, dyinf=",ES11.2)',c%dyinf
  if (c%type_profil_sigma == PROFIL_GEOMETRIQUE_MANUEL) then
    call cpml_absorbant_geom_manuel(c, c%dyinf)
  end if
  call cpml_y_precalcul(c%yinf,ch,c%ncell,c%abseinf,c%abseeinf,c%abshinf,c%abshhinf)
endif

if (c%flag_ysup) then
  print '(T5,"--> Traitement PML YUP, dysup=",ES11.2)',c%dysup
  if (c%type_profil_sigma == PROFIL_GEOMETRIQUE_MANUEL) then
    call cpml_absorbant_geom_manuel(c, c%dysup)
  end if
  call cpml_y_precalcul(c%ysup,ch,c%ncell,c%absesup,c%abseesup,c%abshsup,c%abshhsup)
endif

!--> PML suivant z
if (c%flag_zinf) then
  print '(T5,"--> Traitement PML ZINF, dzinf=",ES11.2)',c%dzinf
  if (c%type_profil_sigma == PROFIL_GEOMETRIQUE_MANUEL) then
    call cpml_absorbant_geom_manuel(c, c%dzinf)
  end if
  call cpml_z_precalcul(c%zinf,ch,c%ncell,c%abseinf,c%abseeinf,c%abshinf,c%abshhinf)
endif

if (c%flag_zsup) then
  print '(T5,"--> Traitement PML ZUP, dzsup=",ES11.2)',c%dzsup
  if (c%type_profil_sigma == PROFIL_GEOMETRIQUE_MANUEL) then
    call cpml_absorbant_geom_manuel(c, c%dzsup)
  end if
  call cpml_z_precalcul(c%zsup,ch,c%ncell,c%absesup,c%abseesup,c%abshsup,c%abshhsup)
endif

end subroutine cpml_precalcul


! ==========================================================
! == Subroutine cpml_calcul_h                             ==
! == Calcul des champs h dans les CPML                    ==
! ==========================================================
subroutine cpml_calcul_h(t_cpml,t_champ)


type(st_cpml), intent(in) :: t_cpml
type(st_champ), intent(inout) :: t_champ

if (t_cpml%flag_xinf) call cpml_x_calcul_h(t_cpml%xinf,t_champ)
if (t_cpml%flag_xsup) call cpml_x_calcul_h(t_cpml%xsup,t_champ)
if (t_cpml%flag_yinf) call cpml_y_calcul_h(t_cpml%yinf,t_champ)
if (t_cpml%flag_ysup) call cpml_y_calcul_h(t_cpml%ysup,t_champ)
if (t_cpml%flag_zinf) call cpml_z_calcul_h(t_cpml%zinf,t_champ)
if (t_cpml%flag_zsup) call cpml_z_calcul_h(t_cpml%zsup,t_champ)

end subroutine cpml_calcul_h


! ==========================================================
! == Subroutine cpml_calcul_ye                            ==
! == Mise a jour des ye                                   ==
! ==========================================================
subroutine cpml_calcul_ye(t_cpml,t_champ)

type(st_cpml), intent(inout) :: t_cpml
type(st_champ), intent(in) :: t_champ

if (t_cpml%flag_xinf) call cpml_x_calcul_ye(t_cpml%xinf,t_champ)
if (t_cpml%flag_xsup) call cpml_x_calcul_ye(t_cpml%xsup,t_champ)
if (t_cpml%flag_yinf) call cpml_y_calcul_ye(t_cpml%yinf,t_champ)
if (t_cpml%flag_ysup) call cpml_y_calcul_ye(t_cpml%ysup,t_champ)
if (t_cpml%flag_zinf) call cpml_z_calcul_ye(t_cpml%zinf,t_champ)
if (t_cpml%flag_zsup) call cpml_z_calcul_ye(t_cpml%zsup,t_champ)

end subroutine cpml_calcul_ye


! ==========================================================
! == Subrotuine cpml_calcul_e                             ==
! == Calcul des e dans les CPML                           ==
! ==========================================================
subroutine cpml_calcul_e(t_cpml,t_champ)

type(st_cpml), intent(in) :: t_cpml
type(st_champ), intent(inout) :: t_champ

!type(st_cpml_x), pointer :: cx
!integer  :: i

!nullify(cx)
!do i=1,2
!  if ((i==1).and.(t_cpml%flag_xinf)) then
!    cx => t_cpml%xinf
!  else if ((i==2).and.(t_cpml%flag_xsup)) then
!    cx => t_cpml%xsup
!  else
!    cycle
!  end if
  
!  if (t_champ%flag_ce2) then
!    cx%ey(:,:,:) = t_champ%ey  (cx%idebe:cx%ifine,t_champ%jdey:t_champ%jfey,t_champ%kdey:t_champ%kfey)
!    cx%ez(:,:,:) = t_champ%ez  (cx%idebe:cx%ifine,t_champ%jdez:t_champ%jfez,t_champ%kdez:t_champ%kfez)
!    call cpml_x_calcul_e1(cx%taille_yey,cx%taille_yez, cx%ey, cx%ez, cx%yey,cx%yez,&
!	t_champ%cey2(cx%idebe:cx%ifine,t_champ%jdey:t_champ%jfey,t_champ%kdey:t_champ%kfey),&
!	t_champ%cez2(cx%idebe:cx%ifine,t_champ%jdez:t_champ%jfez,t_champ%kdez:t_champ%kfez))  
!    t_champ%ey  (cx%idebe:cx%ifine,t_champ%jdey:t_champ%jfey,t_champ%kdey:t_champ%kfey) =cx%ey(:,:,:)
!    t_champ%ez  (cx%idebe:cx%ifine,t_champ%jdez:t_champ%jfez,t_champ%kdez:t_champ%kfez) =cx%ez(:,:,:)
!	
!  else
!    call cpml_x_calcul_e(cx,t_champ)
!  end if
! 
!end do

if (t_cpml%flag_xinf) call cpml_x_calcul_e(t_cpml%xinf,t_champ)
if (t_cpml%flag_xsup) call cpml_x_calcul_e(t_cpml%xsup,t_champ)
if (t_cpml%flag_yinf) call cpml_y_calcul_e(t_cpml%yinf,t_champ)
if (t_cpml%flag_ysup) call cpml_y_calcul_e(t_cpml%ysup,t_champ)
if (t_cpml%flag_zinf) call cpml_z_calcul_e(t_cpml%zinf,t_champ)
if (t_cpml%flag_zsup) call cpml_z_calcul_e(t_cpml%zsup,t_champ)

end subroutine cpml_calcul_e


! ==========================================================
! == Subroutine cpml_calcul_yh                            ==
! == Mise a jour des yh                                   ==
! ==========================================================
subroutine cpml_calcul_yh(t_cpml,t_champ)

type(st_cpml), intent(inout) :: t_cpml
type(st_champ), intent(in) :: t_champ

if (t_cpml%flag_xinf) call cpml_x_calcul_yh(t_cpml%xinf,t_champ)
if (t_cpml%flag_xsup) call cpml_x_calcul_yh(t_cpml%xsup,t_champ)
if (t_cpml%flag_yinf) call cpml_y_calcul_yh(t_cpml%yinf,t_champ)
if (t_cpml%flag_ysup) call cpml_y_calcul_yh(t_cpml%ysup,t_champ)
if (t_cpml%flag_zinf) call cpml_z_calcul_yh(t_cpml%zinf,t_champ)
if (t_cpml%flag_zsup) call cpml_z_calcul_yh(t_cpml%zsup,t_champ)

end subroutine cpml_calcul_yh


! ==========================================================
! == Subroutine cpml_calcul_h_et_ye                       ==
! == Mise des champs h dans les cpml                      ==
! ==========================================================
subroutine cpml_calcul_h_et_ye(t_cpml,t_champ)

type(st_cpml), intent(inout) :: t_cpml
type(st_champ), intent(inout) :: t_champ

call cpml_calcul_h (t_cpml,t_champ)
call cpml_calcul_ye(t_cpml,t_champ)

end subroutine cpml_calcul_h_et_ye


! ==========================================================
! == Subroutine cpml_calcul_e_et_yh                       ==
! == Mise des champs e dans les cpml                      ==
! ==========================================================
subroutine cpml_calcul_e_et_yh(t_cpml,t_champ)

type(st_cpml), intent(inout) :: t_cpml
type(st_champ), intent(inout) :: t_champ

call cpml_calcul_e (t_cpml,t_champ)
call cpml_calcul_yh(t_cpml,t_champ)

end subroutine cpml_calcul_e_et_yh

! ==========================================================
! == Subroutine cpml_profil_geom_manuel                   ==
! == Calcul du profil géométrique                         ==
! ==========================================================
integer function cpml_profil_geom_manuel(t_cpml)

type(st_cpml), target, intent(inout) :: t_cpml

integer, pointer :: ncell

ncell => t_cpml%ncell
ncell        = t_cpml%t_pg%ncell
t_cpml%geom  = t_cpml%t_pg%c_geom

! -- Calcul des coefficients absorbants --
allocate(t_cpml%abseinf(ncell) , t_cpml%abshinf(ncell) )
allocate(t_cpml%abseeinf(ncell), t_cpml%abshhinf(ncell))
allocate(t_cpml%absesup(ncell) , t_cpml%abshsup(ncell) )
allocate(t_cpml%abseesup(ncell), t_cpml%abshhsup(ncell))
t_cpml%abseinf = 0; t_cpml%abshinf = 0; t_cpml%abseeinf = 0; t_cpml%abshhinf = 0
t_cpml%absesup = 0; t_cpml%abshsup = 0; t_cpml%abseesup = 0; t_cpml%abshhsup = 0

cpml_profil_geom_manuel = ncell

end function cpml_profil_geom_manuel

! ==========================================================
! == Subroutine cpml_absorbant_geom_manuel          ==
! == Calcul du profil géométrique et des coefficients     ==
! == Calcul des coefficients absorbants                   ==
! ==========================================================
subroutine cpml_absorbant_geom_manuel(t_cpml, delta)

type(st_cpml), target, intent(inout) :: t_cpml
real, intent(in)  :: delta           

integer :: i
integer :: ncell
real ::  geom, sig0
real :: refl
real :: omega_i   ! = sigma_i/epsilon
real :: alpha_eps ! = alpha  /epsilon
real,dimension(:,:),allocatable :: profil_om

ncell = t_cpml%t_pg%ncell
refl  = t_cpml%t_pg%c_reflexion
ncell = t_cpml%t_pg%ncell
geom  = t_cpml%t_pg%c_geom
alpha_eps = t_cpml%alpha/t_cpml%epsilon_defaut

! -- Calcul des coefficients absorbants --
t_cpml%abseinf = 0; t_cpml%abshinf = 0; t_cpml%abseeinf = 0; t_cpml%abshhinf = 0
t_cpml%absesup = 0; t_cpml%abshsup = 0; t_cpml%abseesup = 0; t_cpml%abshhsup = 0


! -- Determination de sigma0
sig0 = - LOG(geom)*LOG(refl)*t_cpml%vitesse/(2*delta*(geom**ncell -1))
omega_i = sig0 

if (rang==RANG_MAITRE_AFF) then
  print '(t10,"Progression géométrique : Ncell=",I2," ; g=",F6.3," ; R(0)=",ES12.3)',ncell,geom,refl
  print '(t10,"                          Fc=",ES15.6,", Sig0=",ES15.6)',&
               omega_i/(2*PI),omega_i*t_cpml%epsilon_defaut
end if
allocate(profil_om(1:ncell,2))
do i=1,ncell
  t_cpml%absesup(i) = EXP(-t_cpml%dt* (omega_i+alpha_eps))
  t_cpml%abseesup(i) = (t_cpml%absesup(i)-1.)*omega_i/(omega_i+alpha_eps)
  profil_om(i,1) =  omega_i 
  omega_i = omega_i * SQRT(geom)
  profil_om(i,2) =  omega_i 

  t_cpml%abshsup(i) = EXP(-t_cpml%dt* (omega_i+alpha_eps))
  t_cpml%abshhsup(i) = (t_cpml%abshsup(i)-1.)*omega_i/(omega_i+alpha_eps)

  omega_i = omega_i * SQRT(geom)
  t_cpml%abseinf (ncell-i+1) = t_cpml%absesup (i)
  t_cpml%abshinf (ncell-i+1) = t_cpml%abshsup (i)
  t_cpml%abseeinf(ncell-i+1) = t_cpml%abseesup(i)
  t_cpml%abshhinf(ncell-i+1) = t_cpml%abshhsup(i)
enddo
if (rang==RANG_MAITRE_AFF) then
  print '(T10,"E : Sig(i)/Sig(0) =",20ES9.2)',profil_om(1:ncell,1)/sig0
  print '(T10,"H : Sig(i)/Sig(0) =",20ES9.2)',profil_om(1:ncell,2)/sig0
end if
deallocate(profil_om)
end subroutine cpml_absorbant_geom_manuel


! ==========================================================
! == Subroutine cpml_volume                               ==
! == Calcul du nombre de couches cpml                     ==
! == et nouvelles limites du volume a cause des pml       ==
! ==========================================================
integer function cpml_volume(t_cpml,t_volume)

type(st_cpml), target, intent(inout) :: t_cpml
type(st_volume), intent(inout) :: t_volume

real, allocatable, dimension (:) :: tsig, tsigh
integer :: i, nlong, ncelm, ivolu, jvolu, kvolu
integer, pointer :: ncell
real :: geom20, geom100, geom, tmaxi, tc, sig0, sig, aux
real :: dxcouch, cell, sgeom, fcc, ss, aa, eta, sigma

ncell => t_cpml%ncell

! -- GENERATION AUTOMATIQUE DE LA COUCHE ABSORBANTE         
! -- Valeur de la progression geometrique, pour une structure
! -- comprise entre 20 et 100 elements             
ivolu = t_volume%nx
jvolu = t_volume%ny
kvolu = t_volume%nz
nlong = MAX(ivolu,jvolu,kvolu)
if (nlong>100) nlong = 100
if (nlong<20) nlong = 20

geom20 = 4.64
geom100 = 2.15
geom = geom20 - (geom20-geom100)*(nlong-20) / 80.

if(t_cpml%iprecisg==-2) geom = geom**1.6
if(t_cpml%iprecisg==-1) geom = geom**1.3
if(t_cpml%iprecisg==+1) geom = geom**0.8
if(t_cpml%iprecisg==+2) geom = geom**0.6

! -- PMLA, coupure a 3 tmaxi --
tmaxi = t_volume%dt * t_volume%ntinc
tc = 3. * tmaxi
if(t_cpml%iprecisg==1) tc = tc*2.
if(t_cpml%iprecisg==2) tc = tc*4.

! -- Determination de sigma0 et sigma(0) en fonction de tc --
! -  sig0 = sig(0) = conductivite FDTD dans interface
! -  sig  = sigo   = conductivite theorique dans interface
sig0 = 1. / (1.8e10*tc)
sig = sig0 * LOG(geom) / (SQRT(geom) - 1.)

!-- Calcul nombre de cellules necessaires pour realiser (geom,sig0) --
dxcouch = MIN(t_volume%celx,t_volume%cely,t_volume%celz)

aux = (SQRT(geom) - 1.) * LOG(t_cpml%absdon) / (754. * dxcouch * sig0)
      
cell = LOG(1.-aux) / LOG(geom)
ncell = 1 + INT(cell)

! -- Calcul sig interface en fonction absorption --
sgeom = SQRT(geom)
aux = 1. - geom**ncell
sig = 1. / (2.*377.) * LOG(t_cpml%absdon) * LOG(geom) / (dxcouch*aux)

! -- Parametres de la methode PLMA --
t_cpml%p = 0.8888
t_cpml%q = 1. - t_cpml%p
eta = 0.4

t_cpml%tmaxi = tmaxi
!print '(//t10,"Couche absorbante generee automatiquement",//t10,"Duree du calcul demande :",i7, &
!     &" iterations, soit ",f10.2," ns")', t_volume%ntinc, tmaxi*1.e9

t_cpml%tc = tc
!print '(//t10,"Temps de calcul induit par la precision",f10.2," ns")', tc*1.e9

!print '(/t10,"Precision demandee : ",i5)', t_cpml%iprecisg

t_cpml%cell = cell
!print'(/t10,"Nombre de cellules de la couche absorbante :",i4,2x,"(",f4.1,")")', ncell, cell
 
if (ncell<2) then
   write(6,*)' Nombre de cellules inferieur a 2'
   stop
endif

ncelm = ncell 
 
!print '(/t10,"Croissance conductivite : Geometrique")'

t_cpml%geom = geom
!print '(/t10,"Progression geometrique conductivite :",f8.5)', geom

if (geom<1.and.geom>0.) then
   write(6,*)' progression inferieure a 1'
   stop
endif

if (geom>100.) then
   write(6,*)' progression superieure a 100'
   stop
endif

t_cpml%eta = eta
!print '(/t10,"Facteurs P et eta de la PMLA :",f9.5,f9.3)', t_cpml%p, eta

! -- Calcul des facteurs multiplicatifs de la conductivite --
allocate(tsig(ncell+1), tsigh(ncell+1))
tsig = 0; tsigh = 0

tsig(1) = (sgeom-1. / sgeom) / LOG(geom)

do i=1,ncell
  tsigh(i) = tsig(i) * sgeom
  tsig(i+1) = tsigh(i) * sgeom
enddo

tsig(1) = (sgeom - 1.) / LOG(geom)

! -- Loi empirique : pour PMLA sig(0)=sig(1/2)*0.4 --
tsig(1) = tsigh(1) * eta

aux = tsig(1)*sig
fcc = aux*1.8e10
tc = (1./fcc)*1.0e9
t_cpml%aux = aux
t_cpml%fcc = fcc
t_cpml%ic  = tc
!print '(/t10,"Premiere conductivite : ",e10.3,/,t10,"Frequence de coupure : ",e10.2," Hz",/, &
!      &t10,"Instant de coupure : ",f10.1," ns")', aux, fcc, tc

! -- Calcul des coefficients absorbants --
allocate(t_cpml%abseinf(ncell) , t_cpml%abshinf(ncell) )
allocate(t_cpml%abseeinf(ncell), t_cpml%abshhinf(ncell))
allocate(t_cpml%absesup(ncell) , t_cpml%abshsup(ncell) )
allocate(t_cpml%abseesup(ncell), t_cpml%abshhsup(ncell))
t_cpml%abseinf = 0; t_cpml%abshinf = 0; t_cpml%abseeinf = 0; t_cpml%abshhinf = 0
t_cpml%absesup = 0; t_cpml%abshsup = 0; t_cpml%abseesup = 0; t_cpml%abshhsup = 0

ss = sig*t_volume%dt/t_volume%epsilon

do i=1,ncell
  sigma = sig*tsig(i)
  aa = ss*tsig(i)
  t_cpml%absesup(i) = EXP(-aa)
  t_cpml%abseesup(i) = (t_cpml%absesup(i)-1.)

  sigma = sig*tsigh(i)
  aa = ss*tsigh(i)
  t_cpml%abshsup(i) = EXP(-aa)
  t_cpml%abshhsup(i) = (t_cpml%abshsup(i)-1.)

  t_cpml%abseinf (ncell-i+1) = t_cpml%absesup (i)
  t_cpml%abshinf (ncell-i+1) = t_cpml%abshsup (i)
  t_cpml%abseeinf(ncell-i+1) = t_cpml%abseesup(i)
  t_cpml%abshhinf(ncell-i+1) = t_cpml%abshhsup(i)
enddo


cpml_volume = ncell

end function cpml_volume


! ==========================================================
! == Subroutine cpml_affichage                            ==
! ==========================================================
subroutine cpml_affichage(c)

type(st_cpml), intent(in) :: c

if (rang==RANG_MAITRE_AFF) then
  print '(/,"Couche absorbante generee automatiquement:")'
  print '(t5,"Temps de calcul induit par la precision",ES13.3," ns")', c%tc*1.0e9
  print '(t5,"Precision demandee : ",i7)', c%iprecisg
  print '(t5,"Nombre de cellules de la couche :",i4,2x,"(",f4.1,")")', c%ncell, c%cell
  print '(t5,"Croissance conductivite : Geometrique")'
  print '(t5,"Progression geometrique conductivite :",f8.5)', c%geom
  print '(t5,"Facteurs P et eta de la PMLA :",f9.5,f9.5)', c%p, c%eta
  print '(t5,"Premiere conductivite : ",e10.3)', c%aux
  print '(t5,"Frequence de coupure : ",e10.2," Hz")', c%fcc
  print '(t5,"Instant de coupure : ",f10.1," ns")', c%ic
  
  if (c%flag_xinf) call cpml_x_affichage(c%xinf)
  if (c%flag_xsup) call cpml_x_affichage(c%xsup)
  if (c%flag_yinf) call cpml_y_affichage(c%yinf)
  if (c%flag_ysup) call cpml_y_affichage(c%ysup)
  if (c%flag_zinf) call cpml_z_affichage(c%zinf)
  if (c%flag_zsup) call cpml_z_affichage(c%zsup)
end if
end subroutine cpml_affichage


! ==========================================================
! == Subroutine cpml_deallocate                           ==
! == Desalloue la memoire des champs dynamiques           ==
! ==========================================================
subroutine cpml_deallocate(t_cpml)

type(st_cpml), intent(inout) :: t_cpml

if (.not.t_cpml%flag_allocate) return
deallocate(t_cpml%abseinf, t_cpml%abshinf, t_cpml%abseeinf, t_cpml%abshhinf)
deallocate(t_cpml%absesup, t_cpml%abshsup, t_cpml%abseesup, t_cpml%abshhsup)


if (t_cpml%flag_xinf) then
  call cpml_x_deallocate(t_cpml%xinf)
  deallocate(t_cpml%xinf)
endif

if (t_cpml%flag_xsup) then
  call cpml_x_deallocate(t_cpml%xsup)
  deallocate(t_cpml%xsup)
endif

if (t_cpml%flag_yinf) then
  call cpml_y_deallocate(t_cpml%yinf)
  deallocate(t_cpml%yinf)
endif

if (t_cpml%flag_ysup) then
  call cpml_y_deallocate(t_cpml%ysup)
  deallocate(t_cpml%ysup)
endif

if (t_cpml%flag_zinf) then
  call cpml_z_deallocate(t_cpml%zinf)
  deallocate(t_cpml%zinf)
endif

if (t_cpml%flag_zsup) then 
  call cpml_z_deallocate(t_cpml%zsup)
  deallocate(t_cpml%zsup)
endif

end subroutine cpml_deallocate


! ==========================================================
! ==                     Fin du module                    ==
! ==========================================================
end module mfdtd_cpml






