!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_cpml_x.f90
!// Purpose:     Traitement des PMLs Xinf et Xsup
!// Author:      Cyril Giraudon, Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! ==========================================================
! == type st_cpml_x                                       ==
! == subroutine cpml_x_init                               ==
! == subroutine cpml_x_calcul_h                           ==
! == subroutine cpml_x_calcul_yh                          ==
! == subroutine cpml_x_calcul_e                           ==
! == subroutine cpml_x_calcul_ye                          ==
! == subroutine cpml_x_affichage                          ==
! == subroutine cpml_x_deallocate                         ==
! ==========================================================

module mfdtd_cpml_x

use mfdtd_champ, only : st_champ

implicit none

! ==========================================================
! == Type st_cpml_x                                       ==
! ==========================================================
type st_cpml_x
  character(LEN=3) :: ou ! -- ou = 'inf' | 'sup' --
!  integer  :: taille_yey, taille_yez
  integer :: ideb, ifin
  integer :: idebe, ifine
  integer :: idebh, ifinh
  integer :: imin,imax,jmin,jmax,kmin,kmax

  real, dimension(:), pointer :: abse, absh, absee, abshh
  real, dimension(:), pointer :: abse2,absh2 ! coeff indépendant du milieu
  real, dimension(:,:,:), pointer :: yey, yhy
  real, dimension(:,:,:), pointer :: yez, yhz
  real, dimension(:,:,:), pointer :: eh1,eh2
  real, dimension(:,:), pointer   :: chy2,chz2,cey2,cez2
end type st_cpml_x

contains

! ==========================================================
! == Subroutine cpml_x_initdefaut                         ==
! ==========================================================
subroutine cpml_x_initdefaut(c)

implicit none

type(st_cpml_x), intent(inout) :: c

c%ou = ''

c%ideb = 0; c%ifin = 0
c%idebe = 0; c%ifine = 0
c%idebh = 0; c%ifinh = 0

nullify(c%abse,c%absh,c%absee,c%abshh)
nullify(c%yey,c%yhy)
nullify(c%yez,c%yhz)

end subroutine cpml_x_initdefaut

! ==========================================================
! == Subroutine cpml_x_init                     ==
! == Alloue les tableaux pour les cpml selon l'axe x      ==
! ==========================================================
subroutine cpml_x_init(t_cpml_x,ou,ideb,ifin)

implicit none

type(st_cpml_x), intent(inout) :: t_cpml_x
character(LEN=3), intent(in) :: ou
integer, intent(in) :: ideb, ifin

t_cpml_x%ideb = ideb
t_cpml_x%ifin = ifin
t_cpml_x%ou = ou
if (ou=='inf') then
  t_cpml_x%idebe = ideb + 1; t_cpml_x%ifine = ifin + 1
  t_cpml_x%idebh = ideb    ; t_cpml_x%ifinh = ifin 
elseif (ou=='sup') then
  t_cpml_x%idebe = ideb    ; t_cpml_x%ifine = ifin
  t_cpml_x%idebh = ideb    ; t_cpml_x%ifinh = ifin
else
  stop 'erreur, cpml_x_initilisation'
endif

end subroutine cpml_x_init


! ==========================================================
! == Subroutine cpml_x_precalcul                          ==
! == Calcul a effectuer avant la boucle sur le temps      ==
! ==========================================================
subroutine cpml_x_precalcul(c,ch,ncell,abse,absee,absh,abshh)

implicit none

type(st_cpml_x), intent(inout) :: c
type(st_champ), intent(inout) :: ch
integer, intent(in) :: ncell
real, dimension(ncell), intent(in) :: abse, absee, absh, abshh

integer :: i,j,k

allocate(c%yey(ch%jdey:ch%jfey,c%idebe:c%ifine,ch%kdey:ch%kfey)); c%yey = 0
allocate(c%yez(ch%jdez:ch%jfez,c%idebe:c%ifine,ch%kdez:ch%kfez)); c%yez = 0
allocate(c%yhy(ch%jdhy:ch%jfhy,c%idebh:c%ifinh,ch%kdhy:ch%kfhy)); c%yhy = 0
allocate(c%yhz(ch%jdhz:ch%jfhz,c%idebh:c%ifinh,ch%kdhz:ch%kfhz)); c%yhz = 0


allocate(c%abse (c%idebe:c%ifine));  
allocate(c%absee(c%idebe:c%ifine)); 
allocate(c%abse2(c%idebe:c%ifine)); 
allocate(c%absh (c%idebh:c%ifinh)); 
allocate(c%abshh(c%idebh:c%ifinh)); 
allocate(c%absh2(c%idebh:c%ifinh)); 
c%abse  = abse
c%absh  = absh 
c%abse2(c%idebe:c%ifine) = absee(:) 
c%absh2(c%idebh:c%ifinh) = abshh(:)

c%absee(c%idebe:c%ifine) = absee(:) * ch%cer2dt_epsodx(c%idebe:c%ifine)
c%abshh(c%idebh:c%ifinh) = abshh(:) * ch%chr2dt_muodxo(c%idebh:c%ifinh)

c%imin=MIN(c%idebe,c%idebh)
c%imax=MAX(c%ifine,c%ifinh)
c%jmin=MIN(ch%jdey,ch%jdez,ch%jdhy,ch%jdhz)
c%jmax=MAX(ch%jfey,ch%jfez,ch%jfhy,ch%jfhz)
c%kmin=MIN(ch%kdey,ch%kdez,ch%kdhy,ch%kdhz)
c%kmax=MAX(ch%kfey,ch%kfez,ch%kfhy,ch%kfhz)
allocate(c%eh1(c%jmin:c%jmax,c%imin:c%imax,c%kmin:c%kmax))
allocate(c%eh2(c%jmin:c%jmax,c%imin:c%imax,c%kmin:c%kmax))
c%eh1=0.0
c%eh2=0.0
nullify(c%chy2)
nullify(c%chz2)
nullify(c%cey2)
nullify(c%cez2)
if (ch%flag_ch2) then
  allocate(c%chy2(c%jmin:c%jmax,c%kmin:c%kmax))
  allocate(c%chz2(c%jmin:c%jmax,c%kmin:c%kmax))
  do k=c%kmin,c%kmax
    do j=c%jmin,c%jmax
      c%chy2(j,k) = ch%chy2(c%idebh,j,k)
      c%chz2(j,k) = ch%chz2(c%idebh,j,k)
    end do
  end do
end if


if (ch%flag_ce2) then
  allocate(c%cey2(c%jmin:c%jmax,c%kmin:c%kmax))
  allocate(c%cez2(c%jmin:c%jmax,c%kmin:c%kmax))
  do k=c%kmin,c%kmax
    do j=c%jmin,c%jmax
      c%cey2(j,k) = ch%cey2(c%idebe,j,k)
      c%cez2(j,k) = ch%cez2(c%idebe,j,k)
    end do
  end do
end if

end subroutine cpml_x_precalcul

 
! ==========================================================
! == Subroutine cpml_x_calcul_hyz                         ==      
! == Mise a jour de hy et hz dans les cpml x              ==
! ==========================================================
subroutine cpml_x_calcul_h(cp,ch)

implicit none

type(st_cpml_x), intent(in) :: cp
type(st_champ), intent(inout) :: ch
integer :: i, j, k
real, dimension(:,:,:),pointer :: hy,hz,yhz,yhy
real, dimension(:,:),  pointer :: chz2,chy2


hy =>cp%eh1
hz =>cp%eh2
!$OMP PARALLEL
!$OMP DO SCHEDULE(RUNTIME)
do k=cp%kmin,cp%kmax
do i=cp%idebh,cp%ifinh
  hy(cp%jmin:cp%jmax,i,k) = ch%hy(i,cp%jmin:cp%jmax,k)
  hz(cp%jmin:cp%jmax,i,k) = ch%hz(i,cp%jmin:cp%jmax,k)
end do
end do
!$OMP END DO
!$OMP END PARALLEL


!hy=>ch%hy
!hz=>ch%hz
yhy=>cp%yhy
yhz=>cp%yhz
if (ch%flag_ch2) then
  chy2=>cp%chy2
  chz2=>cp%chz2
! introduction des boucles pour la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdhz,ch%kfhz
    do i=cp%idebh,cp%ifinh
      do j=ch%jdhz,ch%jfhz   ! inversion de boucle pour la vectorisation
	hz(j,i,k) = hz(j,i,k) -yhz(j,i,k) * chz2(j,k)
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdhy,ch%kfhy
    do i=cp%idebh,cp%ifinh
      do j=ch%jdhy,ch%jfhy ! inversion de boucle pour la vectorisation
	hy(j,i,k) = hy(j,i,k) + yhy(j,i,k) * chy2(j,k) 
      end do
    end do
  end do  
  !$OMP END DO
!$OMP END PARALLEL

else
! introduction des boucles pour la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdhz,ch%kfhz
    do i=cp%idebh,cp%ifinh
      do j=ch%jdhz,ch%jfhz
	hz(j,i,k) = hz(j,i,k) -yhz(j,i,k)
      end do
    end do
  end do  

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdhy,ch%kfhy
    do i=cp%idebh,cp%ifinh
      do j=ch%jdhy,ch%jfhy ! inversion de boucle pour le calcul vectoriel
	hy(j,i,k) = hy(j,i,k) + yhy(j,i,k)

      end do
    end do
  end do  
  !$OMP END DO
!$OMP END PARALLEL
endif


!$OMP PARALLEL
!$OMP DO SCHEDULE(RUNTIME)
do k=cp%kmin,cp%kmax
!CDIR NOVECTOR
do i=cp%idebh,cp%ifinh
  ch%hy(i,cp%jmin:cp%jmax,k) = hy(cp%jmin:cp%jmax,i,k)
  ch%hz(i,cp%jmin:cp%jmax,k) = hz(cp%jmin:cp%jmax,i,k)
end do
end do
!$OMP END DO
!$OMP END PARALLEL


end subroutine cpml_x_calcul_h


! ==========================================================
! == Subroutine cpml_x_calcul_yeyz                        ==
! == Mise a jour des champs yey et yez                    ==
! ==========================================================
subroutine cpml_x_calcul_ye(cp,ch)

implicit none

type(st_cpml_x), intent(inout) :: cp
type(st_champ), intent(in) :: ch

integer :: i, j, k
real, dimension(:,:,:),pointer :: hy,hz,yez,yey
real, dimension(:),pointer     :: abse,absee

hy=>ch%hy
hz=>ch%hz
yey=>cp%yey
yez=>cp%yez
abse =>cp%abse
absee=>cp%absee

!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
do k=ch%kdez,ch%kfez
  do i=cp%idebe,cp%ifine
  do j=ch%jdez,ch%jfez
      yez(j,i,k) = yez(j,i,k)*abse(i) + (hy(i,j,k)-hy(i-1,j,k))*absee(i)
    enddo
  enddo
enddo

  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
do k=ch%kdey,ch%kfey
  do i=cp%idebe,cp%ifine
  do j=ch%jdey,ch%jfey
      yey(j,i,k) = yey(j,i,k)*abse(i) + (hz(i,j,k)-hz(i-1,j,k))*absee(i) 
    enddo
  enddo
enddo
  !$OMP END DO
!$OMP END PARALLEL

end subroutine cpml_x_calcul_ye


! ==========================================================
! == subroutine cpml_x_calcul_e                           ==
! == Calcul des e dans les CPML                           ==
! ==========================================================
subroutine cpml_x_calcul_e(cp,ch)

implicit none

type(st_cpml_x), intent(in) :: cp
type(st_champ), intent(inout) :: ch

integer :: i,j,k
real, dimension(:,:,:),pointer :: ey,ez,yez,yey
real, dimension(:,:),  pointer :: cez2,cey2

ey =>cp%eh1
ez =>cp%eh2
!$OMP PARALLEL
!$OMP DO SCHEDULE(RUNTIME)
do k=cp%kmin,cp%kmax
do i=cp%idebe,cp%ifine
  ey(cp%jmin:cp%jmax,i,k) = ch%ey(i,cp%jmin:cp%jmax,k)
  ez(cp%jmin:cp%jmax,i,k) = ch%ez(i,cp%jmin:cp%jmax,k)
end do
end do
!$OMP END DO
!$OMP END PARALLEL


!ey=>ch%ey
!ez=>ch%ez
yey=>cp%yey
yez=>cp%yez

! introduction des boucles pour la vectorisation
if (ch%flag_ce2) then
  cey2=>cp%cey2
  cez2=>cp%cez2
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdez,ch%kfez
    do i=cp%idebe,cp%ifine
      do j=ch%jdez,ch%jfez
	ez(j,i,k) = ez(j,i,k) + yez(j,i,k) * cez2(j,k)
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdey,ch%kfey
    do i=cp%idebe,cp%ifine
      do j=ch%jdey,ch%jfey ! inversion de boucle pour la vectorisation
	ey(j,i,k) = ey(j,i,k) - yey(j,i,k) * cey2(j,k) 
      end do
    end do
  end do  
  !$OMP END DO
!$OMP END PARALLEL

!  ch%ey   (cp%idebe:cp%ifine,ch%jdey:ch%jfey,ch%kdey:ch%kfey) = &
!  ch%ey   (cp%idebe:cp%ifine,ch%jdey:ch%jfey,ch%kdey:ch%kfey) - &
!  cp%yey  (cp%idebe:cp%ifine,ch%jdey:ch%jfey,ch%kdey:ch%kfey) * &
!  ch%cey2 (cp%idebe:cp%ifine,ch%jdey:ch%jfey,ch%kdey:ch%kfey)

!  ch%ez   (cp%idebe:cp%ifine,ch%jdez:ch%jfez,ch%kdez:ch%kfez) = &
!  ch%ez   (cp%idebe:cp%ifine,ch%jdez:ch%jfez,ch%kdez:ch%kfez) + &
!  cp%yez  (cp%idebe:cp%ifine,ch%jdez:ch%jfez,ch%kdez:ch%kfez) * &
!  ch%cez2 (cp%idebe:cp%ifine,ch%jdez:ch%jfez,ch%kdez:ch%kfez)

else

! introduction des boucles pour la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdez,ch%kfez
    do i=cp%idebe,cp%ifine
      do j=ch%jdez,ch%jfez
	ez(j,i,k) = ez(j,i,k) + yez(j,i,k)
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdey,ch%kfey
    do i=cp%idebe,cp%ifine
      do j=ch%jdey,ch%jfey ! inversion de boucle pour la vectorisation
	ey(j,i,k) = ey(j,i,k) - yey(j,i,k)
      end do
    end do
  end do  
  !$OMP END DO
!$OMP END PARALLEL

!  ch%ey   (cp%idebe:cp%ifine,ch%jdey:ch%jfey,ch%kdey:ch%kfey) = &
!  ch%ey   (cp%idebe:cp%ifine,ch%jdey:ch%jfey,ch%kdey:ch%kfey) - &
!  cp%yey  (cp%idebe:cp%ifine,ch%jdey:ch%jfey,ch%kdey:ch%kfey)
  
!  ch%ez   (cp%idebe:cp%ifine,ch%jdez:ch%jfez,ch%kdez:ch%kfez) = &
!  ch%ez   (cp%idebe:cp%ifine,ch%jdez:ch%jfez,ch%kdez:ch%kfez) + &
!  cp%yez  (cp%idebe:cp%ifine,ch%jdez:ch%jfez,ch%kdez:ch%kfez)
endif

!$OMP PARALLEL
!$OMP DO SCHEDULE(RUNTIME)
do k=cp%kmin,cp%kmax
!!CDIR NOVECTOR
do i=cp%idebe,cp%ifine
  ch%ey(i,cp%jmin:cp%jmax,k) = ey(cp%jmin:cp%jmax,i,k)
  ch%ez(i,cp%jmin:cp%jmax,k) = ez(cp%jmin:cp%jmax,i,k)
end do
end do
!$OMP END DO
!$OMP END PARALLEL

end subroutine cpml_x_calcul_e

! ==========================================================
! == subroutine cpml_x_calcul_e1                           ==
! == Calcul des e dans les CPML                           ==
! == cas test
! ==========================================================
subroutine cpml_x_calcul_e1(taille_yey,taille_yez,ey,ez,yey,yez,cey2,cez2)

implicit none
integer, intent(in)  :: taille_yey, taille_yez
real, dimension(1:taille_yey),intent(inout) :: ey
real, dimension(1:taille_yey),intent(in) :: yey,cey2
real, dimension(1:taille_yez),intent(inout) :: ez
real, dimension(1:taille_yez),intent(in) :: yez,cez2


integer :: i

do i=1,taille_yey
  ey(i) = ey(i) - yey(i) * cey2(i)
end do 

do i=1,taille_yez
  ez(i) = ez(i) + yez(i) * cez2(i)
end do 
end subroutine cpml_x_calcul_e1


! =========================================================
! == Subroutine cpml_x_calcul_yh                         ==
! == Mise a jour des champs yhy yhz                      ==
! =========================================================
subroutine cpml_x_calcul_yh(cp,ch)

implicit none

type(st_cpml_x), intent(inout) :: cp
type(st_champ), intent(in) :: ch

integer :: i, j, k

real, dimension(:,:,:),pointer :: ey,ez,yhz,yhy
real, dimension(:),pointer     :: absh,abshh

ey=>ch%ey
ez=>ch%ez
yhy=>cp%yhy
yhz=>cp%yhz
absh =>cp%absh
abshh=>cp%abshh

!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
do k=ch%kdhz,ch%kfhz
!!!!CDIR COLLAPSE  !fusion ne marche pas correctement!!!
  do i=cp%idebh,cp%ifinh
    do j=ch%jdhz,ch%jfhz ! inversion de boucle pour la vectorisation
      yhz(j,i,k) = yhz(j,i,k)*absh(i) + (ey(i+1,j,k)-ey(i,j,k))*abshh(i)
    enddo
  enddo
enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)

do k=ch%kdhy,ch%kfhy
!!!CDIR COLLAPSE ! fusion ne marche pas correctement !!!
  do i=cp%idebh,cp%ifinh
    do j=ch%jdhy,ch%jfhy ! inversion de boucle pour la vectorisation
      yhy(j,i,k) = yhy(j,i,k)*absh(i) + (ez(i+1,j,k)-ez(i,j,k))*abshh(i) 
    enddo
  enddo
enddo
  !$OMP END DO
!$OMP END PARALLEL

end subroutine cpml_x_calcul_yh


! ========================================================
! == Subroutine cpml_x_affichage                        ==
! == Affichage des caracteristiques d'une cpml_x        ==
! ========================================================
subroutine cpml_x_affichage(c)

implicit none

type(st_cpml_x), intent(in) :: c

print '(/,"Couche PML yoz", a3, ":")', c%ou
print '(t5,"Selon X, de i=", i5, " à i=", i5)', c%ideb, c%ifin

end subroutine cpml_x_affichage


! ========================================================
! == Subroutine cpml_x_deallocate                       ==
! == Desalloue la memoire pour les champs dynamiques    ==
! ========================================================
subroutine cpml_x_deallocate(c)

implicit none

type(st_cpml_x), intent(inout) :: c

deallocate(c%abse,c%absh,c%absee,c%abse2,c%abshh,c%absh2)

deallocate(c%yey,c%yhy,c%yez,c%yhz)
deallocate(c%eh1,c%eh2)

if (associated(c%chy2))  deallocate(c%chy2)
if (associated(c%chz2))  deallocate(c%chz2)
if (associated(c%cey2))  deallocate(c%cey2)
if (associated(c%cez2))  deallocate(c%cez2)


end subroutine cpml_x_deallocate


! ========================================================
! ==                     Fin du module                  ==
! ========================================================
end module mfdtd_cpml_x


