!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_cpml_y.f90
!// Purpose:     Traitement des PMLs Yinf et Ysup
!// Author:      Cyril Giraudon, Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

module mfdtd_cpml_y

use mfdtd_champ, only : st_champ

implicit none

! ==========================================================
! == Type st_cpml_y                                       ==
! ==========================================================
type st_cpml_y
  character(LEN=3) :: ou ! -- ou = 'inf' | 'sup' --

  integer :: jdeb, jfin
  integer :: jdebe, jfine
  integer :: jdebh, jfinh

  real, dimension(:), pointer :: abse, absh, absee, abshh
  real, dimension(:), pointer :: abse2, absh2 ! coeff indépendant du milieu
  real, dimension(:,:,:), pointer :: yex, yhx
  real, dimension(:,:,:), pointer :: yez, yhz
end type st_cpml_y

contains

! ==========================================================
! == Subroutine cpml_y_initdefaut                         ==
! ==========================================================
subroutine cpml_y_initdefaut(c)

implicit none

type(st_cpml_y), intent(inout) :: c

c%ou = ''

c%jdeb = 0; c%jfin = 0
c%jdebe = 0; c%jfine = 0
c%jdebh = 0; c%jfinh = 0

nullify(c%abse,c%absh,c%absee,c%abshh)
nullify(c%yex,c%yhx)
nullify(c%yez,c%yhz)

end subroutine cpml_y_initdefaut


! ==========================================================
! == Subroutine cpml_y_init                     ==
! == Alloue les tableaux pour les cpml selon l'axe y      ==
! ==========================================================
subroutine cpml_y_init(t_cpml_y,ou,jdeb,jfin)

implicit none

type(st_cpml_y), intent(inout) :: t_cpml_y
character(LEN=3), intent(in) :: ou
integer, intent(in) :: jdeb, jfin 

t_cpml_y%jdeb = jdeb
t_cpml_y%jfin = jfin
t_cpml_y%ou = ou
if (ou=='inf') then
  t_cpml_y%jdebe = jdeb + 1; t_cpml_y%jfine = jfin + 1
  t_cpml_y%jdebh = jdeb    ; t_cpml_y%jfinh = jfin
elseif (ou=='sup') then
  t_cpml_y%jdebe = jdeb    ; t_cpml_y%jfine = jfin
  t_cpml_y%jdebh = jdeb    ; t_cpml_y%jfinh = jfin
else
  stop 'erreur, cpml_y_initilisation'
endif

end subroutine cpml_y_init


! ==========================================================
! == Subroutine cpml_y_precalcul                          ==
! == Calcul a effectuer avant la boucle sur le temps      ==
! ==========================================================
subroutine cpml_y_precalcul(c,ch,ncell,abse,absee,absh,abshh)

implicit none

type(st_cpml_y), intent(inout) :: c
type(st_champ), intent(inout)  :: ch

integer, intent(in) :: ncell
real, dimension(ncell), intent(in) :: abse, absee, absh, abshh


allocate(c%yex(ch%idex:ch%ifex,c%jdebe:c%jfine,ch%kdex:ch%kfex)); c%yex = 0
allocate(c%yez(ch%idez:ch%ifez,c%jdebe:c%jfine,ch%kdez:ch%kfez)); c%yez = 0
allocate(c%yhx(ch%idhx:ch%ifhx,c%jdebh:c%jfinh,ch%kdhx:ch%kfhx)); c%yhx = 0
allocate(c%yhz(ch%idhz:ch%ifhz,c%jdebh:c%jfinh,ch%kdhz:ch%kfhz)); c%yhz = 0

allocate(c%abse (c%jdebe:c%jfine)) 
allocate(c%absee(c%jdebe:c%jfine)) 
allocate(c%abse2(c%jdebe:c%jfine)) 
allocate(c%absh (c%jdebh:c%jfinh)) 
allocate(c%abshh(c%jdebh:c%jfinh)) 
allocate(c%absh2(c%jdebh:c%jfinh)) 
c%abse  = abse
c%absh  = absh
c%abse2(c%jdebe:c%jfine) = absee(:)
c%absh2(c%jdebh:c%jfinh) = abshh(:)

c%absee(c%jdebe:c%jfine) = absee(:)*ch%cer2dt_epsody(c%jdebe:c%jfine)
c%abshh(c%jdebh:c%jfinh) = abshh(:)*ch%chr2dt_muodyo(c%jdebh:c%jfinh)

end subroutine cpml_y_precalcul

 
! ==========================================================
! == Subroutine cpml_y_calcul_hyz                         ==      
! == Mise a jour de hy et hz dans les cpml                ==
! ==========================================================
subroutine cpml_y_calcul_h(cp,ch)

implicit none

type(st_cpml_y), intent(in) :: cp
type(st_champ), intent(inout) :: ch

integer  :: i,j,k
real, dimension(:,:,:),pointer :: hx,hz,yhz,yhx,chz2,chx2

hx=>ch%hx
hz=>ch%hz
yhx=>cp%yhx
yhz=>cp%yhz

if (ch%flag_ch2) then
  chx2=>ch%chx2
  chz2=>ch%chz2

  ! boucle pour plus de performances à la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdhx,ch%kfhx
    do j=cp%jdebh,cp%jfinh 
      do i=ch%idhx,ch%ifhx
	hx(i,j,k) = hx(i,j,k) - yhx(i,j,k) * chx2(i,j,k) 
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdhz,ch%kfhz
    do j=cp%jdebh,cp%jfinh 
      do i=ch%idhz,ch%ifhz
	hz(i,j,k) = hz(i,j,k) + yhz(i,j,k) * chz2(i,j,k) 
      end do
    end do
  end do  
 !$OMP END DO
!$OMP END PARALLEL

!  ch%hx   (ch%idhx:ch%ifhx,cp%jdebh:cp%jfinh,ch%kdhx:ch%kfhx) = &
!  ch%hx   (ch%idhx:ch%ifhx,cp%jdebh:cp%jfinh,ch%kdhx:ch%kfhx) - &
!  cp%yhx  (ch%idhx:ch%ifhx,cp%jdebh:cp%jfinh,ch%kdhx:ch%kfhx) * &
!  ch%chx2 (ch%idhx:ch%ifhx,cp%jdebh:cp%jfinh,ch%kdhx:ch%kfhx)

!  ch%hz   (ch%idhz:ch%ifhz,cp%jdebh:cp%jfinh,ch%kdhz:ch%kfhz) = &
!  ch%hz   (ch%idhz:ch%ifhz,cp%jdebh:cp%jfinh,ch%kdhz:ch%kfhz) + &
!  cp%yhz  (ch%idhz:ch%ifhz,cp%jdebh:cp%jfinh,ch%kdhz:ch%kfhz) * &
!  ch%chz2 (ch%idhz:ch%ifhz,cp%jdebh:cp%jfinh,ch%kdhz:ch%kfhz)

else
  ! boucle pour plus de performances à la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdhx,ch%kfhx
    do j=cp%jdebh,cp%jfinh 
      do i=ch%idhx,ch%ifhx
	hx(i,j,k) = hx(i,j,k) - yhx(i,j,k)
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdhz,ch%kfhz
    do j=cp%jdebh,cp%jfinh 
      do i=ch%idhz,ch%ifhz
	hz(i,j,k) = hz(i,j,k) + yhz(i,j,k)
      end do
    end do
  end do  
 !$OMP END DO
!$OMP END PARALLEL

!  ch%hx   (ch%idhx:ch%ifhx,cp%jdebh:cp%jfinh,ch%kdhx:ch%kfhx) = &
!  ch%hx   (ch%idhx:ch%ifhx,cp%jdebh:cp%jfinh,ch%kdhx:ch%kfhx) - &
!  cp%yhx  (ch%idhx:ch%ifhx,cp%jdebh:cp%jfinh,ch%kdhx:ch%kfhx)

!  ch%hz   (ch%idhz:ch%ifhz,cp%jdebh:cp%jfinh,ch%kdhz:ch%kfhz) = &
!  ch%hz   (ch%idhz:ch%ifhz,cp%jdebh:cp%jfinh,ch%kdhz:ch%kfhz) + &
!  cp%yhz  (ch%idhz:ch%ifhz,cp%jdebh:cp%jfinh,ch%kdhz:ch%kfhz)
endif

end subroutine cpml_y_calcul_h


! ==========================================================
! == Subroutine cpml_y_calcul_yexz                        ==
! == Mise a jour des champs yex et yez                    ==
! ==========================================================
subroutine cpml_y_calcul_ye(cp,ch)

implicit none

type(st_cpml_y), intent(inout) :: cp
type(st_champ), intent(in) :: ch

integer :: i, j, k
real, dimension(:,:,:),pointer :: hx,hz,yez,yex
real, dimension(:),pointer     :: abse,absee

hx=>ch%hx
hz=>ch%hz
yex=>cp%yex
yez=>cp%yez
abse =>cp%abse
absee=>cp%absee

!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
do k=ch%kdez,ch%kfez
  do j=cp%jdebe,cp%jfine
    do i=ch%idez,ch%ifez
      yez(i,j,k) = yez(i,j,k)*abse(j) + (hx(i,j,k)-hx(i,j-1,k))*absee(j)
    enddo
  enddo
enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
do k=ch%kdex,ch%kfex
  do j=cp%jdebe,cp%jfine
    do i=ch%idex,ch%ifex
      yex(i,j,k) = yex(i,j,k)*abse(j) + (hz(i,j,k)-hz(i,j-1,k))*absee(j) 
    enddo
  enddo
enddo
 !$OMP END DO
!$OMP END PARALLEL

end subroutine cpml_y_calcul_ye


! ==========================================================
! == subroutine cpml_y_calcul_e                           ==
! == Calcul des e dans les CPML                           ==
! ==========================================================
subroutine cpml_y_calcul_e(cp,ch)

implicit none

type(st_cpml_y), intent(in) :: cp
type(st_champ), intent(inout) :: ch

integer :: i,j,k

real, dimension(:,:,:),pointer :: ex,ez,yex,yez,cex2,cez2

ex=>ch%ex
ez=>ch%ez
yex=>cp%yex
yez=>cp%yez

if (ch%flag_ce2) then
  cex2=>ch%cex2
  cez2=>ch%cez2
  ! boucle pour plus de performances à la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdex,ch%kfex
    do j=cp%jdebe,cp%jfine
      do i=ch%idex,ch%ifex
	ex(i,j,k) = ex(i,j,k) + yex(i,j,k) * cex2(i,j,k) 
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdez,ch%kfez
    do j=cp%jdebe,cp%jfine
      do i=ch%idez,ch%ifez
	ez(i,j,k) = ez(i,j,k) - yez(i,j,k) * cez2(i,j,k) 
      end do
    end do
  end do  
 !$OMP END DO
!$OMP END PARALLEL
!  ch%ex   (ch%idex:ch%ifex,cp%jdebe:cp%jfine,ch%kdex:ch%kfex) = &
!  ch%ex   (ch%idex:ch%ifex,cp%jdebe:cp%jfine,ch%kdex:ch%kfex) + &
!  cp%yex  (ch%idex:ch%ifex,cp%jdebe:cp%jfine,ch%kdex:ch%kfex) * &
!  ch%cex2 (ch%idex:ch%ifex,cp%jdebe:cp%jfine,ch%kdex:ch%kfex)

!  ch%ez   (ch%idez:ch%ifez,cp%jdebe:cp%jfine,ch%kdez:ch%kfez) = &
!  ch%ez   (ch%idez:ch%ifez,cp%jdebe:cp%jfine,ch%kdez:ch%kfez) - &
!  cp%yez  (ch%idez:ch%ifez,cp%jdebe:cp%jfine,ch%kdez:ch%kfez) * &
!  ch%cez2 (ch%idez:ch%ifez,cp%jdebe:cp%jfine,ch%kdez:ch%kfez)

else
  ! boucle pour plus de performances à la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdex,ch%kfex
    do j=cp%jdebe,cp%jfine
      do i=ch%idex,ch%ifex
	ex(i,j,k) = ex(i,j,k) + yex(i,j,k) 
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
  do k=ch%kdez,ch%kfez
    do j=cp%jdebe,cp%jfine
      do i=ch%idez,ch%ifez
	ez(i,j,k) = ez(i,j,k) - yez(i,j,k) 
      end do
    end do
  end do  
 !$OMP END DO
!$OMP END PARALLEL

!  ch%ex   (ch%idex:ch%ifex,cp%jdebe:cp%jfine,ch%kdex:ch%kfex) = &
!  ch%ex   (ch%idex:ch%ifex,cp%jdebe:cp%jfine,ch%kdex:ch%kfex) + &
!  cp%yex  (ch%idex:ch%ifex,cp%jdebe:cp%jfine,ch%kdex:ch%kfex)

!  ch%ez   (ch%idez:ch%ifez,cp%jdebe:cp%jfine,ch%kdez:ch%kfez) = &
!  ch%ez   (ch%idez:ch%ifez,cp%jdebe:cp%jfine,ch%kdez:ch%kfez) - &
!  cp%yez  (ch%idez:ch%ifez,cp%jdebe:cp%jfine,ch%kdez:ch%kfez)
endif

end subroutine cpml_y_calcul_e


! =========================================================
! == Subroutine cpml_y_calcul_yhxz                       ==
! == Mise a jour des champs yhx yhz                      ==
! =========================================================
subroutine cpml_y_calcul_yh(cp,ch)

implicit none

type(st_cpml_y), intent(inout) :: cp
type(st_champ), intent(in) :: ch

integer :: i, j, k

real, dimension(:,:,:),pointer :: ex,ez,yhx,yhz
real, dimension(:),pointer     :: absh,abshh

ex=>ch%ex
ez=>ch%ez
yhx=>cp%yhx
yhz=>cp%yhz
absh =>cp%absh
abshh=>cp%abshh

!$OMP PARALLEL
  !$OMP DO SCHEDULE(RUNTIME)
do k=ch%kdhz,ch%kfhz
  do j=cp%jdebh,cp%jfinh
    do i=ch%idhz,ch%ifhz
      yhz(i,j,k) = yhz(i,j,k)*absh(j) + (ex(i,j+1,k)-ex(i,j,k))*abshh(j)
    enddo
  enddo
enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(RUNTIME)
do k=ch%kdhx,ch%kfhx
  do j=cp%jdebh,cp%jfinh
    do i=ch%idhx,ch%ifhx
      yhx(i,j,k) = yhx(i,j,k)*absh(j) + (ez(i,j+1,k)-ez(i,j,k))*abshh(j) 
    enddo
  enddo
enddo
 !$OMP END DO
!$OMP END PARALLEL

end subroutine cpml_y_calcul_yh


! ========================================================
! == Subroutine cpml_y_affichage                        ==
! == Affichage des caracteristiques d'une cpml_y        ==
! ========================================================
subroutine cpml_y_affichage(c)

implicit none

type(st_cpml_y), intent(in) :: c

print '(/,"Couche PML xoz", a3, ":")', c%ou
print '(t5,"Selon Y, de j=", i5, " à j=", i5)', c%jdeb, c%jfin

end subroutine cpml_y_affichage


! ========================================================
! == Subroutine cpml_y_deallocate                       ==
! == Desalloue la memoire pour les champs dynamiques    ==
! ========================================================
subroutine cpml_y_deallocate(t_cpml_y)

implicit none

type(st_cpml_y), intent(inout) :: t_cpml_y

deallocate(t_cpml_y%abse,  &
           t_cpml_y%absh,  &
	   t_cpml_y%absee, &
	   t_cpml_y%abshh, &
	   t_cpml_y%abse2, &
	   t_cpml_y%absh2)

deallocate(t_cpml_y%yex, &
           t_cpml_y%yhx, &
	   t_cpml_y%yez, &
	   t_cpml_y%yhz)

end subroutine cpml_y_deallocate


! ========================================================
! ==                     Fin du module                  ==
! ========================================================
end module mfdtd_cpml_y


