!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_cpml_z.f90
!// Purpose:     Traitement des PMLs Zinf et Zsup
!// Author:      Cyril Giraudon, Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Novembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
! ==========================================================
! == type st_cpml_z                                       ==
! == subroutine cpml_z_init                     ==
! == subroutine cpml_z_calcul_h                           ==
! == subroutine cpml_z_calcul_yh                          ==
! == subroutine cpml_z_calcul_e                           ==
! == subroutine cpml_z_calcul_ye                          ==
! == subroutine cpml_z_affichage                          ==
! == suborutine cpml_z_deallocate                         ==
! ==========================================================

module mfdtd_cpml_z

use mfdtd_champ, only : st_champ

implicit none

! ==========================================================
! == Type st_cpml_z                                       ==
! ==========================================================
type st_cpml_z
  character(LEN=3) :: ou ! -- ou = 'inf' | 'sup' --

  integer :: kdeb, kfin
  integer :: kdebe, kfine
  integer :: kdebh, kfinh

  real, dimension(:), pointer :: abse, absh, absee, abshh
  real, dimension(:), pointer :: abse2, absh2
  real, dimension(:,:,:), pointer :: yex, yhx
  real, dimension(:,:,:), pointer :: yey, yhy
end type st_cpml_z

contains

! ==========================================================
! == Subroutine cpml_z_initdefaut                         ==
! ==========================================================
subroutine cpml_z_initdefaut(c)

implicit none

type(st_cpml_z), intent(inout) :: c

c%ou = ''

c%kdeb = 0; c%kfin = 0
c%kdebe = 0; c%kfine = 0
c%kdebh = 0; c%kfinh = 0

nullify(c%abse,c%absh,c%absee,c%abshh)
nullify(c%yex,c%yhx)
nullify(c%yey,c%yhy)

end subroutine cpml_z_initdefaut


! ==========================================================
! == Subroutine cpml_z_init                     ==
! == Alloue les tableaux pour les cpml selon l'axe x      ==
! ==========================================================
subroutine cpml_z_init(t_cpml_z,ou,kdeb,kfin)

implicit none

type(st_cpml_z), intent(inout) :: t_cpml_z
character(LEN=3), intent(in) :: ou
integer, intent(in) :: kdeb, kfin 

t_cpml_z%kdeb = kdeb
t_cpml_z%kfin = kfin
t_cpml_z%ou = ou
if (ou=='inf') then
  t_cpml_z%kdebe = kdeb + 1; t_cpml_z%kfine = kfin + 1
  t_cpml_z%kdebh = kdeb    ; t_cpml_z%kfinh = kfin
elseif (ou=='sup') then
  t_cpml_z%kdebe = kdeb    ; t_cpml_z%kfine = kfin
  t_cpml_z%kdebh = kdeb    ; t_cpml_z%kfinh = kfin
else
  stop 'erreur, cpml_z_initilisation'
endif

end subroutine cpml_z_init


! ==========================================================
! == Subroutine cpml_z_precalcul                          ==
! == Calcul a effectuer avant la boucle sur le temps      ==
! ==========================================================
subroutine cpml_z_precalcul(c,ch,ncell,abse,absee,absh,abshh)

implicit none

type(st_cpml_z), intent(inout) :: c
type(st_champ), intent(inout)  :: ch

integer, intent(in) :: ncell
real, dimension(ncell), intent(in) :: abse, absee, absh, abshh

allocate(c%yex(ch%idex:ch%ifex,ch%jdex:ch%jfex,c%kdebe:c%kfine)); c%yex = 0
allocate(c%yey(ch%idey:ch%ifey,ch%jdey:ch%jfey,c%kdebe:c%kfine)); c%yey = 0
allocate(c%yhx(ch%idhx:ch%ifhx,ch%jdhx:ch%jfhx,c%kdebh:c%kfinh)); c%yhx = 0
allocate(c%yhy(ch%idhy:ch%ifhy,ch%jdhy:ch%jfhy,c%kdebh:c%kfinh)); c%yhy = 0

allocate(c%abse (c%kdebe:c%kfine)); 
allocate(c%absee(c%kdebe:c%kfine)); 
allocate(c%absh (c%kdebh:c%kfinh)); 
allocate(c%abshh(c%kdebh:c%kfinh)); 
allocate(c%abse2(c%kdebe:c%kfine)); 
allocate(c%absh2(c%kdebh:c%kfinh)); 

c%abse  = abse
c%absh  = absh
c%abse2(c%kdebe:c%kfine) = absee(:)
c%absh2(c%kdebh:c%kfinh) = abshh(:)
c%absee(c%kdebe:c%kfine) = absee(:) * ch%cer2dt_epsodz(c%kdebe:c%kfine)
c%abshh(c%kdebh:c%kfinh) = abshh(:) * ch%chr2dt_muodzo(c%kdebh:c%kfinh)

end subroutine cpml_z_precalcul

 
! ==========================================================
! == Subroutine cpml_z_calcul_hyz                         ==      
! == Mise a jour de hy et hz dans les cpml                ==
! ==========================================================
subroutine cpml_z_calcul_h(cp,ch)

implicit none

type(st_cpml_z), intent(in) :: cp
type(st_champ), intent(inout) :: ch

integer  :: i,j,k
real, dimension(:,:,:),pointer :: hx,hy,yhx,yhy,chx2,chy2

hx=>ch%hx
hy=>ch%hy
yhx=>cp%yhx
yhy=>cp%yhy

if (ch%flag_ch2) then
  chx2=>ch%chx2
  chy2=>ch%chy2

  ! boucle pour plus de performances à la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(DYNAMIC,2)
  do k=cp%kdebh,cp%kfinh
    do j=ch%jdhx,ch%jfhx 
      do i=ch%idhx,ch%ifhx
	hx(i,j,k) = hx(i,j,k) + yhx(i,j,k) * chx2(i,j,k) 
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(DYNAMIC,2)
  do k=cp%kdebh,cp%kfinh
    do j=ch%jdhy,ch%jfhy 
      do i=ch%idhy,ch%ifhy
	hy(i,j,k) = hy(i,j,k) - yhy(i,j,k) * chy2(i,j,k) 
      end do
    end do
  end do  
  !$OMP END DO
!$OMP END PARALLEL


!  ch%hx   (ch%idhx:ch%ifhx,ch%jdhx:ch%jfhx,cp%kdebh:cp%kfinh) = &
!  ch%hx   (ch%idhx:ch%ifhx,ch%jdhx:ch%jfhx,cp%kdebh:cp%kfinh) + &
!  cp%yhx  (ch%idhx:ch%ifhx,ch%jdhx:ch%jfhx,cp%kdebh:cp%kfinh) * &
!  ch%chx2 (ch%idhx:ch%ifhx,ch%jdhx:ch%jfhx,cp%kdebh:cp%kfinh)

!  ch%hy   (ch%idhy:ch%ifhy,ch%jdhy:ch%jfhy,cp%kdebh:cp%kfinh) = &
!  ch%hy   (ch%idhy:ch%ifhy,ch%jdhy:ch%jfhy,cp%kdebh:cp%kfinh) - &
!  cp%yhy  (ch%idhy:ch%ifhy,ch%jdhy:ch%jfhy,cp%kdebh:cp%kfinh) * &
!  ch%chy2 (ch%idhy:ch%ifhy,ch%jdhy:ch%jfhy,cp%kdebh:cp%kfinh)

else
  ! boucle pour plus de performances à la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(DYNAMIC,2)
  do k=cp%kdebh,cp%kfinh
    do j=ch%jdhx,ch%jfhx 
      do i=ch%idhx,ch%ifhx
	hx(i,j,k) = hx(i,j,k) + yhx(i,j,k)
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(DYNAMIC,2)
  do k=cp%kdebh,cp%kfinh
    do j=ch%jdhy,ch%jfhy 
      do i=ch%idhy,ch%ifhy
	hy(i,j,k) = hy(i,j,k) - yhy(i,j,k) 
      end do
    end do
  end do  
  !$OMP END DO
!$OMP END PARALLEL
!  ch%hx   (ch%idhx:ch%ifhx,ch%jdhx:ch%jfhx,cp%kdebh:cp%kfinh) = &
!  ch%hx   (ch%idhx:ch%ifhx,ch%jdhx:ch%jfhx,cp%kdebh:cp%kfinh) + &
!  cp%yhx  (ch%idhx:ch%ifhx,ch%jdhx:ch%jfhx,cp%kdebh:cp%kfinh)

!  ch%hy   (ch%idhy:ch%ifhy,ch%jdhy:ch%jfhy,cp%kdebh:cp%kfinh) = &
!  ch%hy   (ch%idhy:ch%ifhy,ch%jdhy:ch%jfhy,cp%kdebh:cp%kfinh) - &
!  cp%yhy  (ch%idhy:ch%ifhy,ch%jdhy:ch%jfhy,cp%kdebh:cp%kfinh)
endif

end subroutine cpml_z_calcul_h

! ==========================================================
! == Subroutine cpml_z_calcul_yexz                        ==
! == Mise a jour des champs yex et yey                    ==
! ==========================================================
subroutine cpml_z_calcul_ye(cp,ch)

implicit none

type(st_cpml_z), intent(inout) :: cp
type(st_champ), intent(in) :: ch

integer :: i, j, k
real, dimension(:,:,:),pointer :: hx,hy,yex,yey
real, dimension(:),pointer     :: abse,absee

hx=>ch%hx
hy=>ch%hy
yex=>cp%yex
yey=>cp%yey
abse =>cp%abse
absee=>cp%absee

!$OMP PARALLEL
  !$OMP DO SCHEDULE(DYNAMIC,2)
do k=cp%kdebe,cp%kfine
  do j=ch%jdey,ch%jfey
    do i=ch%idey,ch%ifey
      yey(i,j,k) = yey(i,j,k)*abse(k) + (hx(i,j,k)-hx(i,j,k-1))*absee(k)
    enddo
  enddo
enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(DYNAMIC,2)
do k=cp%kdebe,cp%kfine
  do j=ch%jdex,ch%jfex
    do i=ch%idex,ch%ifex
      yex(i,j,k) = yex(i,j,k)*abse(k) + (hy(i,j,k)-hy(i,j,k-1))*absee(k) 
    enddo
  enddo
enddo
  !$OMP END DO
!$OMP END PARALLEL
end subroutine cpml_z_calcul_ye

! ==========================================================
! == subroutine cpml_z_calcul_e                           ==
! == Calcul des e dans les CPML                           ==
! ==========================================================
subroutine cpml_z_calcul_e(cp,ch)

implicit none

type(st_cpml_z), intent(in) :: cp
type(st_champ), intent(inout) :: ch

integer  :: i,j,k
real, dimension(:,:,:),pointer :: ex,ey,yex,yey,cex2,cey2

ex=>ch%ex
ey=>ch%ey
yex=>cp%yex
yey=>cp%yey

if (ch%flag_ce2) then
  cex2=>ch%cex2
  cey2=>ch%cey2
  
  ! boucle pour plus de performances à la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(DYNAMIC,2)
  do k=cp%kdebe,cp%kfine
    do j=ch%jdex,ch%jfex 
      do i=ch%idex,ch%ifex
	ex(i,j,k) = ex(i,j,k) - yex(i,j,k) * cex2(i,j,k) 
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(DYNAMIC,2)
  do k=cp%kdebe,cp%kfine
    do j=ch%jdey,ch%jfey 
      do i=ch%idey,ch%ifey
	ey(i,j,k) = ey(i,j,k) + yey(i,j,k) * cey2(i,j,k) 
      end do
    end do
  end do  
  !$OMP END DO
!$OMP END PARALLEL

!  ch%ex   (ch%idex:ch%ifex,ch%jdex:ch%jfex,cp%kdebe:cp%kfine) = &
!  ch%ex   (ch%idex:ch%ifex,ch%jdex:ch%jfex,cp%kdebe:cp%kfine) - &
!  cp%yex  (ch%idex:ch%ifex,ch%jdex:ch%jfex,cp%kdebe:cp%kfine) * &
!  ch%cex2 (ch%idex:ch%ifex,ch%jdex:ch%jfex,cp%kdebe:cp%kfine)

!  ch%ey   (ch%idey:ch%ifey,ch%jdey:ch%jfey,cp%kdebe:cp%kfine) = &
!  ch%ey   (ch%idey:ch%ifey,ch%jdey:ch%jfey,cp%kdebe:cp%kfine) + &
!  cp%yey  (ch%idey:ch%ifey,ch%jdey:ch%jfey,cp%kdebe:cp%kfine) * &
!  ch%cey2 (ch%idey:ch%ifey,ch%jdey:ch%jfey,cp%kdebe:cp%kfine)

else
  ! boucle pour plus de performances à la vectorisation
!$OMP PARALLEL
  !$OMP DO SCHEDULE(DYNAMIC,2)
  do k=cp%kdebe,cp%kfine
    do j=ch%jdex,ch%jfex 
      do i=ch%idex,ch%ifex
	ex(i,j,k) = ex(i,j,k) - yex(i,j,k)  
      end do
    end do
  end do  
  !$OMP END DO
  !$OMP DO SCHEDULE(DYNAMIC,2)
  do k=cp%kdebe,cp%kfine
    do j=ch%jdey,ch%jfey 
      do i=ch%idey,ch%ifey
	ey(i,j,k) = ey(i,j,k) + yey(i,j,k)  
      end do
    end do
  end do  
  !$OMP END DO
!$OMP END PARALLEL

!  ch%ex   (ch%idex:ch%ifex,ch%jdex:ch%jfex,cp%kdebe:cp%kfine) = &
!  ch%ex   (ch%idex:ch%ifex,ch%jdex:ch%jfex,cp%kdebe:cp%kfine) - &
!  cp%yex  (ch%idex:ch%ifex,ch%jdex:ch%jfex,cp%kdebe:cp%kfine)

!  ch%ey   (ch%idey:ch%ifey,ch%jdey:ch%jfey,cp%kdebe:cp%kfine) = &
!  ch%ey   (ch%idey:ch%ifey,ch%jdey:ch%jfey,cp%kdebe:cp%kfine) + &
!  cp%yey  (ch%idey:ch%ifey,ch%jdey:ch%jfey,cp%kdebe:cp%kfine)
endif

end subroutine cpml_z_calcul_e

! =========================================================
! == Subroutine cpml_z_calcul_yhxz                       ==
! == Mise a jour des champs yhx yhy                      ==
! =========================================================
subroutine cpml_z_calcul_yh(cp,ch)

implicit none

type(st_cpml_z), intent(inout) :: cp
type(st_champ), intent(in) :: ch

integer :: i, j, k
real, dimension(:,:,:),pointer :: ex,ey,yhx,yhy
real, dimension(:),pointer     :: absh,abshh

ex=>ch%ex
ey=>ch%ey
yhx=>cp%yhx
yhy=>cp%yhy
absh =>cp%absh
abshh=>cp%abshh

!$OMP PARALLEL
  !$OMP DO SCHEDULE(STATIC,3)
do k=cp%kdebh,cp%kfinh
  do j=ch%jdhy,ch%jfhy
    do i=ch%idhy,ch%ifhy
      yhy(i,j,k) = yhy(i,j,k)*absh(k) + (ex(i,j,k+1)-ex(i,j,k))*abshh(k)
    enddo
  enddo
enddo
  !$OMP END DO
  !$OMP DO SCHEDULE(STATIC,3)
do k=cp%kdebh,cp%kfinh
  do j=ch%jdhx,ch%jfhx
    do i=ch%idhx,ch%ifhx
      yhx(i,j,k) = yhx(i,j,k)*absh(k) + (ey(i,j,k+1)-ey(i,j,k))*abshh(k) 
    enddo
  enddo
enddo
  !$OMP END DO
!$OMP END PARALLEL

end subroutine cpml_z_calcul_yh

! ========================================================
! == Subroutine cpml_z_affichage                        ==
! == Affichage des caracteristiques d'une cpml_z        ==
! ========================================================
subroutine cpml_z_affichage(c)

implicit none

type(st_cpml_z), intent(in) :: c

print '(/,"Couche PML xoy", a3, ":")', c%ou
print '(t5,"Selon Z, de k=", i5, " à k=", i5)', c%kdeb, c%kfin

end subroutine cpml_z_affichage


! ========================================================
! == Subroutine cpml_z_deallocate                       ==
! == Desalloue la memoire pour les champs dynamiques    ==
! ========================================================
subroutine cpml_z_deallocate(t_cpml_z)

implicit none

type(st_cpml_z), intent(inout) :: t_cpml_z

deallocate(t_cpml_z%abse,  &
           t_cpml_z%absh,  &
	   t_cpml_z%absee, &
	   t_cpml_z%abshh, &
	   t_cpml_z%abse2, &
	   t_cpml_z%absh2)

deallocate(t_cpml_z%yex, &
           t_cpml_z%yhx, &
	   t_cpml_z%yey, &
	   t_cpml_z%yhy)

end subroutine cpml_z_deallocate


! ========================================================
! ==                     Fin du module                  ==
! ========================================================
end module mfdtd_cpml_z


