!////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_filhol_commun.f90
!// Purpose:     Traitement du formalisme des fils minces de Holland  ==

!// Author:      Christophe Guiffaut
!// Modified by:
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Avril 2007
!// Copyright:   (c) Christophe Guiffaut, département CNRS XLIM/OSA
!//              
!// Licence  : Utilisation des sources réservées à XLIM
!////////////////////////////////////////////////////////////////////////


module mfdtd_filhol_commun

use m_constantes

use m_constantes_scientifiques, only : PI, EPSILON_0, MU_0, CVIDE


implicit none

integer,  parameter :: flag_schema_exponentiel = 1
integer,  parameter :: X = 1
integer,  parameter :: Y = 2
integer,  parameter :: Z = 3
integer,  parameter :: NB_ECH_THETA = 400
integer,  parameter :: NB_ECH_R = 100
integer,  parameter :: NB_MAILLE_DISQUE = 100  ! NOMBRE PAIR !
integer,  parameter :: CO = 0   ! circuit ouvert
integer,  parameter :: CC = 1   ! court-circuit
integer,  parameter :: JPML = 2 ! jonction PML
integer,  parameter :: JNF = 4  ! jonction noeud multi-fil
integer,  parameter :: JFF = 3  ! jonction fil-fil
integer,  parameter :: JFC = 5  ! jonction fil-coaxial
integer,  parameter :: JFQ = 6  ! jonction fil-fil avec quadripole
integer,  parameter :: JFI = 7  ! jonction d'au moins un fil 
integer,  parameter :: EXTR_DEBUT = 0  ! extremite  debut
integer,  parameter :: EXTR_FIN = 1  ! extremite  fin

type st_unfil_1D_pml
  real, pointer,dimension(:)  :: cyI1,cyI2
  real, pointer,dimension(:)  :: cyQ1,cyQ2
  real, pointer,dimension(:)  :: yI, yQ
  integer  :: debI, finI
  integer  :: debQ, finQ
end type st_unfil_1D_pml

type st_fil_coaxial
  real :: r0,epsr,sige ! rayon de lame du coaxial, permittivité relative et conductivité substrat
  real :: Lp,Cp
  real, dimension(:), pointer :: I,Q
  real, dimension(:), pointer :: cI1,cIq,cQ1,cQ2,cIvsrc
  logical :: flag_gv
end type st_fil_coaxial

! caractéristique de chaque segment du fil, tableaux des courants et char
type st_fil_segment
  real  :: c_cur, c_qc, c_emoy, c_vsrc
  real, pointer    :: cur
  real, pointer    :: qc,qc1
  real    :: cq_q,cq_cur
  logical :: flag_noeud
  logical :: flag_c ! -- présence d'une capacité
  real, pointer    :: emoy,vsrc,vc, c_vc ! capacité
  real, dimension(4)  :: cp_j
  real    :: v,dt_epsilon,sige_eps
end type st_fil_segment


type st_alias_champ_fil
  real, dimension(:), pointer    :: e
end type st_alias_champ_fil

!---------- GÉNÉRATEUR SUR UN FIL ------------------
type st_generateur_localise
  character(LEN=LEN_MOT_CLE) :: nom, nomgene
  integer :: seg
  real   :: e0, retard, phase, r0
end type st_generateur_localise


!---------- LES DONNÉES POUR UN FIL ------------------
type st_unfil
  character(LEN=LEN_MOT_CLE)  :: nom
  integer :: coord(1:6)
  integer :: ldeb, lfin ! coordonnées du fil début et fin dans le volume
  integer :: ldeb_frontiere, lfin_frontiere !coordonnées du fil début et fin avec les frontieres
  integer :: nseg ! -- Nombre de segments composant le fil
  integer, dimension(2) :: coord_rel ! coordonnées relative au début du fil
  integer :: nsegtot
  integer :: dir ! direction
  integer :: jonction_inf,jonction_sup ! type d'extrémité
  real    :: rayon,epsilonr   !rayon du fil et la permittivité relative
  real, dimension(2) :: eps_extr,sige_extr
  logical :: flag_epsr
  real, dimension(2) :: decal_cell, position_fil
  real, dimension(3) :: delta3  !décalage dans la cellule
  real, dimension(:), pointer  :: ro, lo, co
  real    :: R_lineique
  
!  real, dimension(:), pointer  :: zpo ! coefficient de PADE
  
  real, dimension(:), pointer :: tabgene !contient les formes d'excitation
  logical :: flag_gv  ! -- Generateur de tension --
  type(st_generateur_localise), pointer :: gv
  real    :: lp,cp ! inductance et capa propre

  ! -- Si présence d'une interface diélectrique à proximité
  real                  :: h_interface
  real, dimension(2)    :: epsr_interface
  logical :: flag_interface
  ! -- Pas spatiaux --
  real, dimension(2) :: d3d_t ! pas spatiaux 3d transverses
  real  :: surface_rect

  !-------------------------------------------------------------
  ! -- Concerne uniquement le courant distribué sur un disque --
  real :: s_disque, s_discret  ! surface réelle pour J
  integer, dimension(2) :: uc, vc
  real, dimension(:,:), pointer :: surface
  ! Alias vers n composantes de champ qui entoure le fil (cas du disque)
  integer  :: nbcomposante
  type(st_alias_champ_fil), dimension(:), pointer :: p

  real, dimension(:), pointer :: cp_j, cp_e
  
  type(st_fil_segment), dimension(:), pointer :: l_seg
  real, dimension(:), pointer    :: dl ! pas spatial suivant la direction
  ! Alias vers les 4 composantes de champ qui entoure le fil
  real, dimension(:), pointer    :: e1,e2,e3,e4
  real, dimension(4)  :: cp_emoy ! formalisme Holland
  logical  :: moyenne_e_rectangle

  !-------- Variable supplémentaire concernant la distribution
  !            du courant sur un disque                       ---------
  logical  :: flag_disque
  real     :: r_disque  ! rayon du disque
  real :: eps_moy   ! ne sert pas pour l'instant !

  !--- Traitement d'un cable coaxial ---  
  logical :: flag_coaxial
  type(st_fil_coaxial), pointer :: t_coax !cable coaxial

  !---  Structure pour les CPML aux extrémités ---
  type(st_unfil_1D_pml), pointer  :: pml_deb,pml_fin
  
  ! Présence d'un plan de masse
  real :: p_pm
  integer :: ir_pm
  real,dimension(:,:), pointer :: I_avant  
  logical :: flag_plan_masse
end type st_unfil




contains

! ==========================================================
! == Initialisation du type st_unfil                  ==
! ==========================================================
subroutine structure_unfil_initdefaut(f)

type(st_unfil), intent(inout) :: f

nullify(f%gv)
NULLIFY(f%pml_deb) ; NULLIFY(f%pml_fin);
f%nom = ' '
f%decal_cell(:) = 0.0
f%flag_gv = .false.
f%nseg = 0
f%epsr_interface = 0.0
f%h_interface    = 0.0
f%flag_interface = .false.
f%flag_coaxial = .false.
f%flag_plan_masse = .false.
! par défaut, les extremites du fil sont en circuit ouvert
f%jonction_inf = CO
f%jonction_sup = CO
f%R_lineique   = 0.0

end subroutine structure_unfil_initdefaut



end module mfdtd_filhol_commun
