!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_frontiere.f90
!// Purpose:     Gestion de tous les types de frontières
!// Author:      Cyril Giraudon, Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Novembre 2001
!//                 24 Octobre 2005 : ajout FLOQUET en incidence normale
!// Copyright:   (c), équipe CNRS IRCOM/DEM
!//              Utilisation des sources réservées à l'IRCOM
!// Licence  :
!/////////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Type st_frontiere                                    ==
! == Subroutine frontiere_initdefaut                      ==
! == Subroutine frontiere_lecture                         ==
! == Subroutine frontiere_init                  ==
! == Subroutine frontiere_precalcul                       ==
! == Subroutine frontiere_calcul                          ==
! == Subroutine frontiere_affichage                       ==
! == Subroutine frontiere_postcalcul                      ==
! == Subroutine frontiere_deallocate                      ==
! ==========================================================
module mfdtd_frontiere

use m_constantes
use m_entree, only : st_fichier, lineread, &
                                  lecture_stop, &
				  nb_mot_chaine, &
				  ctrl_nb_mot_chaine
use mfdtd_volume  !, only : st_volume,maillage_NU_init
use mfdtd_champ, only : st_champ
use mfdtd_cpml, only : st_cpml, PROFIL_GEOMETRIQUE_AUTO,&
                        cpml_initdefaut, &
			cpml_copy, &
			cpml_lecture, &
			cpml_init, &
                        cpml_precalcul, cpml_volume_et_profil, & 
			cpml_calcul_h_et_ye, &
			cpml_calcul_e_et_yh, &
			cpml_deallocate, &
			cpml_nbcouche, &
			cpml_affichage

implicit none
  
! ==========================================================
! == Type st_frontiere                                    ==
! ==========================================================
type st_frontiere
  type(st_cpml), pointer :: t_cpml
  logical :: flag_cpml
  integer, dimension(6) :: type_face
  integer :: xinf
  integer :: xsup
  integer :: yinf
  integer :: ysup
  integer :: zinf
  integer :: zsup

  !-------- Taille mémoire et Nbre d'opération par seconde ----
  real      :: taille_mem
  real      :: nb_oper
end type st_frontiere
 
contains

! ==========================================================	
! == Subroutine frontiere_initdefaut                      ==
! == Initialisation par défaut des variables contenant    ==
! == les paramètres définis par l'utilisateur             ==
! == dans le fichier d'entrée                             == 
! ==========================================================
subroutine frontiere_initdefaut(t_frontiere)
type(st_frontiere), intent(inout) :: t_frontiere

t_frontiere%xinf = CPML !
t_frontiere%xsup = CPML !
t_frontiere%yinf = CPML ! -- Par defaut toute les     --
t_frontiere%ysup = CPML ! -- frontieres sont des cpml --
t_frontiere%zinf = CPML !
t_frontiere%zsup = CPML !
t_frontiere%type_face(:) = CPML

allocate(t_frontiere%t_cpml)
call cpml_initdefaut(t_frontiere%t_cpml)
t_frontiere%flag_cpml = .true.

t_frontiere%taille_mem=0.0
t_frontiere%nb_oper   = 0.0
end subroutine frontiere_initdefaut

! ==========================================================
! == Subroutine frontiere_copy                               ==
! ==========================================================	
subroutine frontiere_copy(vin,vout)
! === Déclaration des arguments ===
type(st_frontiere ), intent(in)       :: vin
type(st_frontiere ), intent(out)      :: vout
! ===   Début des traitements   ===
vout%type_face = vin%type_face
vout%flag_cpml = vin%flag_cpml

vout%xinf = vin%xinf
vout%xsup = vin%xsup
vout%yinf = vin%yinf
vout%ysup = vin%ysup
vout%zinf = vin%zinf
vout%zsup = vin%zsup
call cpml_copy(vin%t_cpml,vout%t_cpml)

vout%taille_mem = vin%taille_mem
vout%nb_oper    = vin%nb_oper

end subroutine frontiere_copy
! ==========================================================
! == Subroutine frontiere_deallocate                      ==
! ==========================================================
subroutine frontiere_deallocate(f)
type(st_frontiere), intent(inout) :: f

if (f%flag_cpml) then
  call cpml_deallocate(f%t_cpml)
  deallocate(f%t_cpml)
endif

end subroutine frontiere_deallocate


! ==========================================================
! == Subroutine frontiere_lecture                         ==
! == Lecture dans le fichier d'entrée...                  ==
! ==========================================================
subroutine frontiere_lecture(t_fichier,t_frontiere)

type(st_fichier), intent(inout) :: t_fichier
type(st_frontiere), intent(inout) :: t_frontiere

!--- DÉCLARATION DES VARIABLES LOCALES ---
integer :: nb_mot,status
character(len=LEN_MOT_CLE) :: premiermot
character (len=LEN_MOT_CLE), dimension(1:6) :: face

do
  premiermot = lineread(t_fichier)
  nb_mot = nb_mot_chaine(t_fichier%chaine)
  select case(premiermot)
    case('yozinf')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2)
      t_frontiere%type_face(YOZ_INF) = frontiere_type(t_fichier)
    case('yozsup')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2)
      t_frontiere%xsup = frontiere_type(t_fichier)
      t_frontiere%type_face(YOZ_SUP) = frontiere_type(t_fichier)

    case('xozinf')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2)
      t_frontiere%type_face(XOZ_INF) = frontiere_type(t_fichier)

    case('xozsup')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2)
      t_frontiere%type_face(XOZ_SUP) = frontiere_type(t_fichier)

    case('xoyinf')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2)
      t_frontiere%type_face(XOY_inf) = frontiere_type(t_fichier)

    case('xoysup')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2)
      t_frontiere%type_face(XOY_SUP) = frontiere_type(t_fichier)

    case ('cpml_alpha')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2)
      call cpml_lecture(t_fichier,t_frontiere%t_cpml)
      
    case('profil_pml')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,3)
      call cpml_lecture(t_fichier,t_frontiere%t_cpml)

    case('profil_pml_geometrique')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,4)
      call cpml_lecture(t_fichier,t_frontiere%t_cpml)

    case('pml')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2,7)
      read(t_fichier%chaine,*) premiermot, face(1:nb_mot-1)
      call frontieres_types(t_fichier,t_frontiere,face,nb_mot-1,CPML)
    case('mur_magnetique')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2,7)
      read(t_fichier%chaine,*) premiermot, face(1:nb_mot-1)
      call frontieres_types(t_fichier,t_frontiere,face,nb_mot-1,MUR_MAGNETIQUE)
    case('mur_electrique')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2,7)
      read(t_fichier%chaine,*) premiermot, face(1:nb_mot-1)
      call frontieres_types(t_fichier,t_frontiere,face,nb_mot-1,MUR_ELECTRIQUE)

    case('sans_frontiere')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2,7)
      read(t_fichier%chaine,*) premiermot, face(1:nb_mot-1)
      call frontieres_types(t_fichier,t_frontiere,face,nb_mot-1,SANS_FRONTIERE)

    case('floquet_incidence_normale')
      status = ctrl_nb_mot_chaine(t_fichier,t_fichier%chaine,2,7)
      read(t_fichier%chaine,*) premiermot, face(1:nb_mot-1)
      call frontieres_types(t_fichier,t_frontiere,face,nb_mot-1,FLOQUET)

    case default
      if (t_fichier%nomfinbloc == t_fichier%premiermot) exit ! fin du bloc
      print *,'Mot clé non reconnu dans le bloc frontiere : ', TRIM(t_fichier%premiermot)
      call lecture_stop(t_fichier)    
  end select
enddo

t_frontiere%xinf = t_frontiere%type_face(YOZ_INF)
t_frontiere%xsup = t_frontiere%type_face(YOZ_SUP)
t_frontiere%yinf = t_frontiere%type_face(XOZ_INF)
t_frontiere%ysup = t_frontiere%type_face(XOZ_SUP)
t_frontiere%zinf = t_frontiere%type_face(XOY_INF)
t_frontiere%zsup = t_frontiere%type_face(XOY_SUP)

  
end subroutine frontiere_lecture

! ==========================================================
! == Subroutine frontiere_init                  ==
! == Initialisation des variables, lorsque                ==
! == qu'il n'y a pas de lecture du fichier d'entrée       ==
! ==========================================================
subroutine frontiere_preinit(t_frontiere,v)

type(st_frontiere), intent(inout) :: t_frontiere
type(st_volume), intent(inout) :: v ! Pour calcul du nombre de mailles totales

integer  :: ncellpml

!----------------------------------------------------------------
! Pré-initialisation pour connaitre le volume, y compris les PMLs
!----------------------------------------------------------------
! -- Cas mur electrique ou mur magnetique ou sans frontiere --
v%ivold = 1
v%ivolf = v%nx
v%jvold = 1
v%jvolf = v%ny
v%kvold = 1
v%kvolf = v%nz

if (ALL(t_frontiere%type_face(:)/=CPML)) then
  t_frontiere%flag_cpml = .false.
else
  ncellpml=cpml_nbcouche(t_frontiere%t_cpml,v)
  ! -- Calcul des limites du volume de calcul, pml comprises --
  if (t_frontiere%xinf==CPML) then
    v%ivold = 1 - ncellpml
  endif

  if (t_frontiere%xsup==CPML) then
    v%ivolf = v%nx + ncellpml
  endif

  if (t_frontiere%yinf==CPML) then
    v%jvold = 1 - ncellpml
  endif

  if (t_frontiere%ysup==CPML) then
    v%jvolf = v%ny + ncellpml
  endif

  if (t_frontiere%zinf==CPML) then
    v%kvold = 1 - ncellpml
  endif

  if (t_frontiere%zsup==CPML) then
    v%kvolf = v%nz + ncellpml
  endif
  
end if
v%borne_volume(XMIN) = v%ivold ; v%borne_volume(XMAX) = v%ivolf
v%borne_volume(YMIN) = v%jvold ; v%borne_volume(YMAX) = v%jvolf
v%borne_volume(ZMIN) = v%kvold ; v%borne_volume(ZMAX) = v%kvolf

!print '(T10,"BORNES X =",2I6)',v%ivold,v%ivolf
!print '(T10,"BORNES Y =",2I6)',v%jvold,v%kvolf
!print '(T10,"BORNES Z =",2I6)',v%kvold,v%kvolf
  print '(T5, "Rang=",I5," ; Bornes du volume de calcul : ")',rang
  print '(T10,"Rang=",I5," ; BORNES X =",2I6)',rang,v%ivold,v%ivolf
  print '(T10,"Rang=",I5," ; BORNES Y =",2I6)',rang,v%jvold,v%jvolf
  print '(T10,"Rang=",I5," ; BORNES Z =",2I6)',rang,v%kvold,v%kvolf

end subroutine frontiere_preinit

! ==========================================================
! == Subroutine frontiere_ecriture                        ==
! == Ecriture dans le fichier d'entrée...                 ==
! ==========================================================
subroutine frontiere_ecriture(f,t_frontiere)

type(st_fichier), intent(inout) :: f
type(st_frontiere), intent(inout) :: t_frontiere

integer :: i
character(LEN=LEN_CHAINE) :: ch
character(LEN=10),dimension(1:6) :: ch1



if (ANY(t_frontiere%type_face == CPML)) then
  if (t_frontiere%t_cpml%alpha/=0.0) then
    write(f%unite,'(T3,A,3x,F7.4)')  'cpml_alpha',t_frontiere%t_cpml%alpha
  end if
  
  if (t_frontiere%t_cpml%type_profil_sigma==PROFIL_GEOMETRIQUE_AUTO) then
    write(f%unite,'(T3,A,3x,ES10.3E2,I4)')  'profil_pml',t_frontiere%t_cpml%absdon, &
						  t_frontiere%t_cpml%iprecisg
  else 
    write(f%unite,'(T3,A,3x,I2,3x,ES10.3E2,3x,F6.3)')  'profil_pml_geometrique', &
		  t_frontiere%t_cpml%t_pg%ncell,t_frontiere%t_cpml%t_pg% c_reflexion, &
		  t_frontiere%t_cpml%t_pg%c_geom
  end if
end if


if (ANY(t_frontiere%type_face /= CPML)) then
  ch1(XOY_INF) = 'zinf' ; ch1(XOY_SUP) = 'zsup'
  ch1(YOZ_INF) = 'xinf' ; ch1(YOZ_SUP) = 'xsup'
  ch1(XOZ_INF) = 'yinf' ; ch1(XOZ_SUP) = 'ysup'
  
  if (ANY(t_frontiere%type_face == CPML)) then
    ch = '';
    do i=1,6
      if (t_frontiere%type_face(i) == CPML) ch = TRIM(ch)//'  '//TRIM(ch1(i))
    end do
    write(f%unite,'(T3,A,3x,A)')  'pml',TRIM(ch)
  end if
  
  if (ANY(t_frontiere%type_face == MUR_MAGNETIQUE)) then
    ch = '';
    do i=1,6
      if (t_frontiere%type_face(i) == MUR_MAGNETIQUE) ch = TRIM(ch)//'  '//TRIM(ch1(i))
    end do
    write(f%unite,'(T3,A,3x,A)')  'mur_magnetique',TRIM(ch)
  end if

  if (ANY(t_frontiere%type_face == MUR_ELECTRIQUE)) then
    ch = '';
    do i=1,6
      if (t_frontiere%type_face(i) == MUR_ELECTRIQUE) ch = TRIM(ch)//'  '//TRIM(ch1(i))
    end do
    write(f%unite,'(T3,A,3x,A)')  'mur_electrique',TRIM(ch)
  end if

  if (ANY(t_frontiere%type_face == SANS_FRONTIERE)) then
    ch = '';
    do i=1,6
      if (t_frontiere%type_face(i) == SANS_FRONTIERE) ch = TRIM(ch)//'  '//TRIM(ch1(i))
    end do
    write(f%unite,'(T3,A,3x,A)')  'sans_frontiere',TRIM(ch)
  end if

  if (ANY(t_frontiere%type_face == FLOQUET)) then
    ch = '';
    do i=1,6
      if (t_frontiere%type_face(i) == FLOQUET) ch = TRIM(ch)//'  '//TRIM(ch1(i))
    end do
    write(f%unite,'(T3,A,3x,A)')  'floquet_incidence_normale',TRIM(ch)
  end if
end if

end subroutine frontiere_ecriture

! ==========================================================
! == Fonction frontiere-type                              ==
! == Determine le type de frontiere                       ==
! ==========================================================
integer function frontiere_type(t_fichier)

type(st_fichier), intent(in) :: t_fichier

character(len=LEN_CHAINE):: premiermot
character(len=LEN_CHAINE):: chaine_frontiere

read(t_fichier%chaine,*) premiermot, chaine_frontiere
select case(chaine_frontiere)
  case('pml')
    frontiere_type = CPML
  case('mur_magnetique')
    frontiere_type = MUR_MAGNETIQUE
  case('mur_electrique')
    frontiere_type = MUR_ELECTRIQUE
  case('sans_frontiere')
    frontiere_type = SANS_FRONTIERE
  case('floquet_incidence_normale')
    frontiere_type = FLOQUET
  case default
    print *, 'Type de frontiere inconnu => stop'
    stop 'Arret dans frontiere_lecture'
end select

end function frontiere_type


! ==========================================================
! == subroutine frontieres-types                              ==
! == Determine le type de frontiere pour plusieurs
! == type de faces externes. Arret si le type de frontière
! == n'est pas reconnu.
! ==========================================================
subroutine frontieres_types(t_fichier,t_frontiere,face,nbface,type_frontiere)

!--- DÉCLARATION DES ARGUMENTS ---
type(st_fichier), intent(in)      :: t_fichier
type(st_frontiere), intent(inout) :: t_frontiere
character (len=LEN_MOT_CLE), dimension(1:6), intent(in) :: face
integer, intent(in)          :: nbface,type_frontiere
!--- DÉCLARATION DES VARIABLES LOCALES ---
integer  :: i

do i=1,nbface
  select case (face(i))
    case ('xoyinf','zinf')
       t_frontiere%type_face(XOY_INF) = type_frontiere	    
    case ('xoysup','zsup')
       t_frontiere%type_face(XOY_SUP) = type_frontiere	    
    case ('yozinf','xinf')
       t_frontiere%type_face(YOZ_INF) = type_frontiere	    
    case ('yozsup','xsup')
       t_frontiere%type_face(YOZ_SUP) = type_frontiere	    
    case ('xozinf','zoxinf','yinf')
       t_frontiere%type_face(XOZ_INF) = type_frontiere	    
    case ('xozsup','zoxsup','ysup')
       t_frontiere%type_face(XOZ_SUP) = type_frontiere	    
    case default
      print '("Type de frontiere inconnu => stop :",A)',TRIM(face(i))
      call lecture_stop(t_fichier)
  end select
end do  

end subroutine frontieres_types
! ==========================================================
! == Subroutine frontiere_init                  ==
! == Initialisation des variables, lorsque                ==
! == qu'il n'y a pas de lecture du fichier d'entrée       ==
! ==========================================================
subroutine frontiere_init(f,v)

type(st_frontiere), intent(inout) :: f
type(st_volume), intent(inout) :: v ! Pour calcul du nombre de mailles totales

integer :: ncell

!if (f%xinf/=1.and.f%xsup/=1.and.f%yinf/=1.and.f%ysup/=1.and.f%zinf/=1.and.f%zsup/=1) then
if (ALL(f%type_face(:)/=CPML)) then
!  deallocate(f%t_cpml)
!  NULLIFY(f%t_cpml)
  f%flag_cpml = .false.
else
  ! -- On calcul le nombre de couches pml --
  ncell = cpml_volume_et_profil(f%t_cpml,v)
endif

				
!if (f%flag_cpml) then
  ! on le fait dans tous les cas pour les milieux stratifiés (injection d'ondes planes)
  call cpml_init(f%t_cpml,f%xinf,f%xsup,f%yinf,f%ysup,f%zinf,f%zsup, &
                           v%ivold,v%ivolf,v%jvold,v%jvolf,v%kvold,v%kvolf)
!endif

call frontiere_mem_nboper(f)
end subroutine frontiere_init

! ==========================================================
! == Subroutine frontiere_mem_nboper                      ==
! ==   Evaluation de la taille mémoire en octets dans la
! ==    structure et du nbre d'opération flottante si impactant
! ==     sur le temps de calcul
!===========================================================	
subroutine frontiere_mem_nboper(f)
type(st_frontiere), intent(inout) :: f

f%taille_mem = f%t_cpml%taille_mem
f%nb_oper    = f%t_cpml%nb_oper
end subroutine frontiere_mem_nboper

! ==========================================================
! == Subroutine frontiere_precalcul                       ==
! == Calculs avant les itérations temporelles             ==
!===========================================================	
subroutine frontiere_precalcul(t_frontiere,ch)

implicit none

type(st_frontiere), intent(inout) :: t_frontiere
type(st_champ), intent(inout)  :: ch

if (t_frontiere%flag_cpml) call cpml_precalcul(t_frontiere%t_cpml,ch)
				
end Subroutine frontiere_precalcul

! ==========================================================
! == Subroutine frontiere_calcul_h                        ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine frontiere_calcul_h(t_frontiere,t_champ)

implicit none

type(st_frontiere), intent(inout) :: t_frontiere
type(st_champ), intent(inout) :: t_champ

integer :: i, j, k

if (t_frontiere%flag_cpml) call cpml_calcul_h_et_ye(t_frontiere%t_cpml,t_champ)

!---------------------------------- XINF ---------------------------------------
if (t_frontiere%xinf==MUR_MAGNETIQUE) then
  i = t_champ%idhy
  do k=t_champ%kdhy,t_champ%kfhy
  do j=t_champ%jdhy,t_champ%jfhy
      t_champ%hy(i-1,j,k) = -t_champ%hy(i,j,k)
  enddo
  enddo

  i = t_champ%idhz
  do k=t_champ%kdhz,t_champ%kfhz
  do j=t_champ%jdhz,t_champ%jfhz
      t_champ%hz(i-1,j,k) = -t_champ%hz(i,j,k)
  enddo
  enddo
else if (t_frontiere%xinf==FLOQUET) then
  do k=t_champ%kdhy,t_champ%kfhy
  do j=t_champ%jdhy,t_champ%jfhy
      t_champ%hy(t_champ%idhy-1,j,k) = t_champ%hy(t_champ%ifhy,j,k)
  enddo
  enddo

  do k=t_champ%kdhz,t_champ%kfhz
  do j=t_champ%jdhz,t_champ%jfhz
      t_champ%hz(t_champ%idhz-1,j,k) = t_champ%hz(t_champ%ifhz,j,k)
  enddo
  enddo

endif

!---------------------------------- XSUP ---------------------------------------
if (t_frontiere%xsup==MUR_MAGNETIQUE) then
  i = t_champ%ifhy
  do k=t_champ%kdhy,t_champ%kfhy
  do j=t_champ%jdhy,t_champ%jfhy
      t_champ%hy(i+1,j,k) = -t_champ%hy(i,j,k)
  enddo
  enddo

  i = t_champ%ifhz
  do k=t_champ%kdhz,t_champ%kfhz
  do j=t_champ%jdhz,t_champ%jfhz
      t_champ%hz(i+1,j,k) = -t_champ%hz(i,j,k)
  enddo
  enddo
else if (t_frontiere%xsup==FLOQUET) then  
  do k=t_champ%kdhy,t_champ%kfhy
  do j=t_champ%jdhy,t_champ%jfhy
      t_champ%hy(t_champ%ifhy+1,j,k) = t_champ%hy(t_champ%idhy,j,k)
  enddo
  enddo

  do k=t_champ%kdhz,t_champ%kfhz
  do j=t_champ%jdhz,t_champ%jfhz
      t_champ%hz(t_champ%ifhz+1,j,k) = t_champ%hz(t_champ%idhz,j,k)
  enddo
  enddo
endif

!---------------------------------- YINF ---------------------------------------
if (t_frontiere%yinf==MUR_MAGNETIQUE) then
  j = t_champ%jdhx
  do k=t_champ%kdhx,t_champ%kfhx
  do i=t_champ%idhx,t_champ%ifhx
      t_champ%hx(i,j-1,k) = -t_champ%hx(i,j,k)
  enddo
  enddo

  j = t_champ%jdhz
  do k=t_champ%kdhz,t_champ%kfhz
  do i=t_champ%idhz,t_champ%ifhz
      t_champ%hz(i,j-1,k) = -t_champ%hz(i,j,k)
  enddo
  enddo
else if (t_frontiere%yinf==FLOQUET) then  
  do k=t_champ%kdhx,t_champ%kfhx
  do i=t_champ%idhx,t_champ%ifhx
      t_champ%hx(i,t_champ%jdhx-1,k) = t_champ%hx(i,t_champ%jfhx,k)
  enddo
  enddo

  do k=t_champ%kdhz,t_champ%kfhz
  do i=t_champ%idhz,t_champ%ifhz
      t_champ%hz(i,t_champ%jdhz-1,k) = t_champ%hz(i,t_champ%jfhz,k)
  enddo
  enddo
endif

!---------------------------------- YSUP ---------------------------------------
if (t_frontiere%ysup==MUR_MAGNETIQUE) then
  j = t_champ%jfhx
  do k=t_champ%kdhx,t_champ%kfhx
  do i=t_champ%idhx,t_champ%ifhx
      t_champ%hx(i,j+1,k) = -t_champ%hx(i,j,k)
    enddo
  enddo

  j = t_champ%jfhz
  do k=t_champ%kdhz,t_champ%kfhz
  do i=t_champ%idhz,t_champ%ifhz
      t_champ%hz(i,j+1,k) = -t_champ%hz(i,j,k)
    enddo
  enddo
else if (t_frontiere%ysup==FLOQUET) then  
  do k=t_champ%kdhx,t_champ%kfhx
  do i=t_champ%idhx,t_champ%ifhx
      t_champ%hx(i,t_champ%jfhx+1,k) = t_champ%hx(i,t_champ%jdhx,k)
  enddo
  enddo

  do k=t_champ%kdhz,t_champ%kfhz
  do i=t_champ%idhz,t_champ%ifhz
      t_champ%hz(i,t_champ%jfhz+1,k) = t_champ%hz(i,t_champ%jdhz,k)
  enddo
  enddo
endif

!---------------------------------- ZINF ---------------------------------------
if (t_frontiere%zinf==MUR_MAGNETIQUE) then
  k = t_champ%kdhx
  do j=t_champ%jdhx,t_champ%jfhx
  do i=t_champ%idhx,t_champ%ifhx
      t_champ%hx(i,j,k-1) = -t_champ%hx(i,j,k)
  enddo
  enddo

  k = t_champ%kdhy
  do j=t_champ%jdhy,t_champ%jfhy
  do i=t_champ%idhy,t_champ%ifhy
      t_champ%hy(i,j,k-1) = -t_champ%hy(i,j,k)
  enddo
  enddo
else if (t_frontiere%zinf==FLOQUET) then  
  do j=t_champ%jdhx,t_champ%jfhx
  do i=t_champ%idhx,t_champ%ifhx
      t_champ%hx(i,j,t_champ%kdhx-1) = t_champ%hx(i,j,t_champ%kfhx)
  enddo
  enddo

  do j=t_champ%jdhy,t_champ%jfhy
  do i=t_champ%idhy,t_champ%ifhy
      t_champ%hy(i,j,t_champ%kdhy-1) = t_champ%hy(i,j,t_champ%kfhy)
  enddo
  enddo
endif

!---------------------------------- ZSUP ---------------------------------------
if (t_frontiere%zsup==MUR_MAGNETIQUE) then
  k = t_champ%kfhx
  do j=t_champ%jdhx,t_champ%jfhx
  do i=t_champ%idhx,t_champ%ifhx
      t_champ%hx(i,j,k+1) = -t_champ%hx(i,j,k)
  enddo
  enddo

  k = t_champ%kfhy
  do j=t_champ%jdhy,t_champ%jfhy
  do i=t_champ%idhy,t_champ%ifhy
      t_champ%hy(i,j,k+1) = -t_champ%hy(i,j,k)
  enddo
  enddo
else if (t_frontiere%zsup==FLOQUET) then  
  do j=t_champ%jdhx,t_champ%jfhx
  do i=t_champ%idhx,t_champ%ifhx
      t_champ%hx(i,j,t_champ%kfhx+1) = t_champ%hx(i,j,t_champ%kdhx)
  enddo
  enddo

  do j=t_champ%jdhy,t_champ%jfhy
  do i=t_champ%idhy,t_champ%ifhy
      t_champ%hy(i,j,t_champ%kfhy+1) = t_champ%hy(i,j,t_champ%kdhy)
  enddo
  enddo
endif
!------------------------------------------------------------------------------

end subroutine frontiere_calcul_h

! ==========================================================
! == Subroutine frontiere_calcul_h                        ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine frontiere_calcul_e(f,t_champ)

implicit none

type(st_frontiere), intent(inout) :: f
type(st_champ), intent(inout) :: t_champ

if (f%flag_cpml) call cpml_calcul_e_et_yh(f%t_cpml,t_champ)

end subroutine frontiere_calcul_e


! ==========================================================
! == Subroutine frontiere_sans_frontiere                  ==
! == Gestion des valeurs aux frontieres                   ==
! ==========================================================
subroutine frontiere_sans_frontiere(f1,f2)

implicit none

type(st_frontiere), intent(inout) :: f1, f2

end subroutine frontiere_sans_frontiere


! ==========================================================
! == Subroutine frontiere_affichage                       ==
! == Affichage de résultats dans le terminal              ==
! ==========================================================	
subroutine frontiere_affichage(f)

!--- DÉCLARATION DES ARGUMENTS ---
type(st_frontiere), intent(inout) :: f

!--- DÉCLARATION DES VARIABLES LOCALES ---
integer   :: i,j1,j2
character(len=LEN_MOT_CLE),dimension(6)  :: tfront
character(len=4),dimension(6)  :: tface

!--- DÉBUT DES TRAITEMENTS
tfront(CPML) = 'CPML' 
tfront(MUR_ELECTRIQUE) = 'Mur Électrique' 
tfront(MUR_MAGNETIQUE) = 'Mur Magnétique' 
tfront(SANS_FRONTIERE) = 'Sans Frontiere' 
tfront(FLOQUET) = 'Floquet' 
tfront(FRONTIERE_MPI) = 'Frontière MPI' 
tface = (/"zinf","zsup","xinf","xsup","yinf","ysup"/)

if (rang==RANG_MAITRE_AFF) &
  print '(/,"=============== LES FRONTIÈRES ================")'
  do i=1,5,2 
    j1= f%type_face(i)
    j2= f%type_face(i+1)
    print '(t5,"Rang =",I5," - FRONTIERE : ",A," = ",A," ; ",A," = ",A )',rang,tface(i),trim(tfront(j1)),&
                    tface(i+1),trim(tfront(j2))
  end do
if (f%flag_cpml) call cpml_affichage(f%t_cpml)
				
end subroutine frontiere_affichage

! ==========================================================
! == Subroutine frontiere_postcalcul                      ==
! == Calcul après les itérations temporelles              ==
! ==========================================================	
subroutine frontiere_postcalcul(f)

implicit none
				
type(st_frontiere), intent(inout) :: f

end Subroutine frontiere_postcalcul

! ==========================================================
! == Subroutine frontiere_impression                      ==
! == Ecriture des résultats dans les fichiers de sortie   ==
! ==========================================================	
subroutine frontiere_impression(f)

implicit none

type(st_frontiere), intent(inout) :: f
				
end subroutine frontiere_impression


! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module mfdtd_frontiere	



