!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_onde_plane.f90
!// Purpose:      Injection d'ondes planes en espace libre, gestion d'un plan de masse
!//               parfait
!//               Injecteur axial en milieu stratifié
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Novembre 2001
!// Copyright:   (c) Christophe Guiffaut, équipe CNRS IRCOM/DEM
!//              Utilisation des sources réservées à l'IRCOM
!// Licence  :
!/////////////////////////////////////////////////////////////////////////////

!================================================================
!Injecteur d'ondes planes par la méthode de la règle 1D 
!================================================================

Module mfdtd_onde_plane
use m_constantes
use m_constantes_scientifiques, only : PI,EPSILON_0,MU_0,CVIDE
use m_entree, only : st_fichier, lineread,multibackspace, &
                                 unitefichier, numbackspace_set, &
				 lecture_stop, nb_mot_chaine, &
				 ctrl_nb_mot_chaine
use m_sortie
use m_math_operation, only : atan2d_cor
use m_verif_data
use m_temps_frequence, only : st_temps_frequence, &
                                	 st_liste_temps_freq, &
					 liste_temps_freq_pointe, &
					 temps_freq_compte, &
					 identifie_deux_nom
use m_generateur,only : st_generateur, generateur_calcul_t,&
                        generateur_existe_actif
			 
use m_maillage

use mfdtd_identifiant
use mfdtd_volume, only : st_volume
use mfdtd_champ, only : st_champ
use mfdtd_cpml,   only : st_cpml
use mfdtd_cpml_x, only : st_cpml_x
use mfdtd_cpml_y, only : st_cpml_y
use mfdtd_cpml_z, only : st_cpml_z 
use mfdtd_milieu_volume, only : st_milieu_volume,cherche_epsilon_cellule,cherche_mu_cellule


implicit none
!=== Déclaration des types ===
integer, PARAMETER  :: INJECTEUR_SIMPLE  = 0
integer, PARAMETER  :: INJECTEUR_AXIAL   = 1
integer, PARAMETER  :: INJECTEUR_OBLIQUE_TM = 2
integer, PARAMETER  :: INJECTEUR_OBLIQUE_TE = 3
integer, parameter  :: DEBUT_REGLE_1D = -4
integer, parameter  :: AXE_X = 1
integer, parameter  :: AXE_Y = 2
integer, parameter  :: AXE_Z = 3
integer, dimension(3), parameter :: PYZX=(/3,1,2/)
integer, dimension(3), parameter :: PXZY=(/1,3,2/)
integer, dimension(6), parameter :: P6XYZ1=(/3,4,5,6,1,2/)

!Structure pour le pre-stockage des retards
type st_prestock_tr_ondeplane
  real, dimension(:,:), pointer       :: tde1,tde2
  real, dimension(:,:), pointer       :: tdh1,tdh2
  integer (kind=2), dimension(:,:), pointer    :: tdie1,tdie2
  integer (kind=2), dimension(:,:), pointer    :: tdih1,tdih2 
end type st_prestock_tr_ondeplane

type st_opl_surface_huygens
  real, dimension(:,:), pointer       :: e1inc,e2inc
  real, dimension(:,:), pointer       :: h1inc,h2inc
end type st_opl_surface_huygens

!--- Structure pour le stockage d'un point de référence ---
type st_ptref_onde_plane
  character (LEN=LEN_MOT_CLE)  :: mantisse,interv_t,interv_f
  integer            :: unite
  real, dimension(3) :: pt
  logical            :: flag_t,flag_f
  type(st_temps_frequence), pointer :: tf_t,tf_f  
  
  real, dimension(:), pointer   :: einc,hinc
  real, dimension(:), pointer   :: psi 
  real, dimension(:,:), pointer :: p
  ! -- Tableaux pour le stockage des composantes  en frequence --
  complex, dimension(:,:), pointer :: ehinc_f

end type st_ptref_onde_plane


! Structure propre à chaque injecteur
type st_injecteurondeplane
  integer   :: type_injecteur
  character (len=LEN_MOT_CLE) :: nomgenerateur
  real,dimension(:), pointer  :: tabgene
  real                :: theta, phi, anglepolar
  real                :: thetarad, phirad, anglepolarrad
  real                :: eo, phase, tr
  real                :: dr
  integer             :: origine(1:3), coordinjection
  integer             :: longueurregle, debutpml
  real, pointer, dimension(:)     :: einc,hinc
  real, pointer, dimension(:)     :: eaccu_pml, haccu_pml
  real, pointer, dimension(:)     :: ce1_pml, ce2_pml, ch1_pml, ch2_pml
  real                            :: cre1, cre2, crh1, crh2
  real,dimension(1:6)             :: conve1, conve2, convh1,convh2
  type(st_prestock_tr_ondeplane), dimension(:), pointer :: tdr
  
  !injecteur de l'onde réfléchie
  integer             :: ip
  real                :: p
  ! injecteur axial en milieu stratifié
  integer             :: debutregle,finpml1,debutpml2, finregle
  integer             :: p1, p2 ! position min et max des surfaces des Huygens sur les injecteurs
  logical             :: flagpml3d
  real,dimension(:), pointer      :: ce1, ce2, ch1, ch2
  real,dimension(:,:), pointer      :: cvh_e1, cvh_e2, cve_h1, cve_h2
  
end type st_injecteurondeplane
   
! Structure décrivant les propriétés des surfaces
type st_onde_plane
  integer                 :: nbsurface, nufacemur,typeplanmur
  integer, dimension(1:6) :: taille,taille_ssd ! taille et taille en sous-domaine
  logical, dimension(1:6) :: flagsurface
  character (len=LEN_MOT_CLE), dimension(1:6) :: plan
  character (len=LEN_MOT_CLE) :: plandumur
  logical             :: flagmurelectrique
  logical             :: flag_ordre2
  integer             :: nbinjecteur, nbinjecteur_tot
  integer             :: nbcpml1d
  real                :: gpml,rpml,apml,alphapml !coefficients pour les pml
  logical             :: flaginjecteuraxial
  integer             :: dir_injecteur_axial
  type(st_injecteurondeplane), dimension(:), pointer :: tinj1d
  
  type(st_opl_surface_huygens),dimension(6) :: tshuyg
  logical             :: flag_ptref
  type (st_ptref_onde_plane),pointer  :: t_ptref

end type st_onde_plane


contains

!===========================================================
! Subroutine onde_plane_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine onde_plane_initdefaut(t_opl)

implicit none
!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout)  :: t_opl
!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_opl%flagsurface(:)     = .true.
t_opl%taille(:)     = 0
t_opl%taille_ssd(:) = 0
t_opl%plan(1)  = BOP_XOYINF
t_opl%plan(2)  = BOP_XOYSUP
t_opl%plan(3)  = BOP_YOZINF
t_opl%plan(4)  = BOP_YOZSUP
t_opl%plan(5)  = BOP_XOZINF
t_opl%plan(6)  = BOP_XOZSUP

t_opl%plandumur = '';
t_opl%flagmurelectrique  = .false.
t_opl%flaginjecteuraxial = .false.
t_opl%flag_ordre2 = .false.
t_opl%nbsurface   = 6
t_opl%nbinjecteur = 1
t_opl%nbinjecteur_tot = 1

t_opl%nbcpml1d    = 25
t_opl%gpml        = 2.0
t_opl%rpml        = 1.0e-6
t_opl%apml        = 1
t_opl%alphapml    = 0.0

t_opl%flag_ptref  = .false.
nullify(t_opl%t_ptref)
end subroutine onde_plane_initdefaut

!===========================================================	
! Subroutine onde_plane_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine onde_plane_lecture(t_fich, t_opl,ltf,tm,g)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_onde_plane), intent(inout)  :: t_opl
type(st_liste_temps_freq), intent(inout) :: ltf
type(st_maillage), intent(in) :: tm
type(st_generateur), intent(inout)    :: g

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)          :: premiermot, corbeille, plandumur,nom1,nom2,nom_t,nom_f
character (len=LEN_MOT_CLE), dimension(1:6) :: plan
integer  :: lchaine, nbligne,i, nbmot, status
integer  :: nbinjecteur, iinj
real     :: phase, retard
logical  :: flag_injecteur_axial
type(st_injecteurondeplane), pointer  :: tinj1d 
!===   Début des traitements   ===
plan = ''
! Initialisation par défaut
call onde_plane_initdefaut(t_opl)
flag_injecteur_axial = .false.

! On compte le nombre d'injecteur
nbinjecteur = 0
call numbackspace_set(t_fich,0)
do
  premiermot = lineread(t_fich,1)
  if (premiermot == BOP_INJECTEUR) then
    nbinjecteur = nbinjecteur +1 
  else if (premiermot == BOP_MUR_ELECTRIQUE) then
    t_opl%flagmurelectrique = .true.
  else if (premiermot == BOP_MILIEU_STRATIFIE_X .or. &
	   premiermot == BOP_MILIEU_STRATIFIE_Y .or. &
	   premiermot == BOP_MILIEU_STRATIFIE_Z) then
    flag_injecteur_axial = .true.
  else if (premiermot == t_fich%nomfinbloc) then
    ! le pointeur de fichier est remonté au début du bloc
    nbligne = multibackspace(t_fich)
    exit 
  end if
end do
if (nbinjecteur == 0) then
  print *,'Pas d''injecteur 1D de défini dans le bloc :',t_fich%nombloc 
  call lecture_stop(t_fich)
end if

! Allocation de la structure injecteur 1D
t_opl%nbinjecteur = nbinjecteur
t_opl%nbinjecteur_tot = nbinjecteur
if (t_opl%flagmurelectrique .and.(.not.flag_injecteur_axial)) then ! nombre d'injecteur doublé
  t_opl%nbinjecteur_tot = nbinjecteur * 2
  allocate(t_opl%tinj1d(1:2*nbinjecteur))
else
  allocate(t_opl%tinj1d(1:nbinjecteur))
end if
phase = 0.0
retard = 0.0

! Lecture des informations sur l'onde plane
iinj = 1
do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)

  select case(premiermot(1:lchaine))
    case('phase')
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,ERR=999) corbeille,phase
    case('retard') 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,ERR=999) corbeille,retard
      
    case('pt_reference') 
      read(t_fich%chaine,*,ERR=999) corbeille, nom1, nom2
      if (.not.t_opl%flag_ptref) allocate(t_opl%t_ptref)
      t_opl%flag_ptref  = .true.
      if (.not.identifie_deux_nom(nom1,nom2,t_opl%t_ptref%flag_t,t_opl%t_ptref%flag_f,nom_t,nom_f)) then
	print '(/,t10,"Nom du type de stockage erroné !!!")'
	call lecture_stop(t_fich)
      end if
      t_opl%t_ptref%interv_t = nom_t
      t_opl%t_ptref%interv_f = nom_f
      if (t_opl%t_ptref%flag_t) then
        nullify(t_opl%t_ptref%tf_t)
	t_opl%t_ptref%tf_t => liste_temps_freq_pointe(ltf,nom_t)
	if (.not.associated(t_opl%t_ptref%tf_t)) then
	  print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
	  call lecture_stop(t_fich)      
	end if
      end if      
      if (t_opl%t_ptref%flag_f) then
        nullify(t_opl%t_ptref%tf_f)
	t_opl%t_ptref%tf_f => liste_temps_freq_pointe(ltf,nom_f)
	if (.not.associated(t_opl%t_ptref%tf_f)) then
	  print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
	  call lecture_stop(t_fich)      
	end if
      end if      
      ! -- Calcul du nombre de mot, lecture des coordonnées --
      nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,5,6)      
      if (nbmot == 5) then
        read(t_fich%chaine,*,ERR=999) corbeille, nom1,t_opl%t_ptref%pt(1:3)
      else
        read(t_fich%chaine,*,ERR=999) corbeille, nom1, nom2,t_opl%t_ptref%pt(1:3)      
      end if
      t_opl%t_ptref%pt= origine_cellule3(tm,t_opl%t_ptref%pt)
      if (rang==RANG_MAITRE_AFF) &
        print '(t10,"Point ref ds le repère de calcul = ",3ES15.6)',t_opl%t_ptref%pt
    !--------------------------------------------------------------------------
    case(BOP_DIM_HUYGS)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,7)
      read(t_fich%chaine,*) corbeille, t_opl%taille(1:6)
      t_opl%taille= origine_cellule6(tm,t_opl%taille(1:6))
      if (rang==RANG_MAITRE_AFF) &
        print '(t10,"Dimensions Huygens ds le repère de calcul = ",6I6)',t_opl%taille
      t_opl%taille_ssd = t_opl%taille ! pour MPI
    case(BOP_FLAG_SURFACE) 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,7)
      t_opl%nbsurface = nb_mot_chaine(t_fich%chaine)-1
      read(t_fich%chaine,*) corbeille, plan(1:t_opl%nbsurface)
      t_opl%flagsurface(:) = .false.
      t_opl%plan(:) = plan
      do i=1,t_opl%nbsurface
        select case (plan(i))
	  case (BOP_XOYINF,'zinf')
             t_opl%flagsurface(1) = .true.	    
	  case (BOP_XOYSUP,'zsup')
             t_opl%flagsurface(2) = .true.	    
	  case (BOP_YOZINF,'xinf')
             t_opl%flagsurface(3) = .true.	    
	  case (BOP_YOZSUP,'xsup')
             t_opl%flagsurface(4) = .true.	    
	  case (BOP_XOZINF,'yinf')
             t_opl%flagsurface(5) = .true.	    
	  case (BOP_XOZSUP,'ysup')
             t_opl%flagsurface(6) = .true.	    
	  case default
            print *,'Mot clé non reconnu dans le bloc ',t_fich%nombloc,' : ', plan(i)
            call lecture_stop(t_fich)
	end select
      end do  
      
    case(BOP_MUR_ELECTRIQUE) 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille, plandumur
      t_opl%plandumur = plandumur
        select case (plandumur)
	  case (BOP_XOYINF)
             t_opl%nufacemur = 1
	     t_opl%typeplanmur = PLAN_XOY	    
	  case (BOP_XOYSUP)
             t_opl%nufacemur = 2	    
	     t_opl%typeplanmur = PLAN_XOY	    
	  case (BOP_YOZINF)
             t_opl%nufacemur = 3	    
	     t_opl%typeplanmur = PLAN_YOZ	    
	  case (BOP_YOZSUP)
             t_opl%nufacemur = 4	    
	     t_opl%typeplanmur = PLAN_YOZ	    
	  case (BOP_XOZINF)
             t_opl%nufacemur = 5	    
	     t_opl%typeplanmur = PLAN_XOZ	    
	  case (BOP_XOZSUP)
             t_opl%nufacemur = 6	    
	     t_opl%typeplanmur = PLAN_XOZ	    
	  case default
            print *,'Mur électrique non reconnu dans le bloc ',t_fich%nombloc,' : ', plandumur
            call lecture_stop(t_fich)
	end select

    case(BOP_INJECTEUR) 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,6,8)
      nbmot = nb_mot_chaine(t_fich%chaine)
      if (nbmot == 8) then
        read(t_fich%chaine,*) corbeille, t_opl%tinj1d(iinj)%nomgenerateur, &
               t_opl%tinj1d(iinj)%theta,t_opl%tinj1d(iinj)%phi, &
               t_opl%tinj1d(iinj)%anglepolar, t_opl%tinj1d(iinj)%eo, & 
               t_opl%tinj1d(iinj)%phase, t_opl%tinj1d(iinj)%tr
      else if (nbmot == 7) then
        read(t_fich%chaine,*) corbeille, t_opl%tinj1d(iinj)%nomgenerateur, &
               t_opl%tinj1d(iinj)%theta,t_opl%tinj1d(iinj)%phi, &
               t_opl%tinj1d(iinj)%anglepolar, t_opl%tinj1d(iinj)%eo, & 
               t_opl%tinj1d(iinj)%phase
	t_opl%tinj1d(iinj)%tr = retard
	
      else if (nbmot == 6) then
        read(t_fich%chaine,*) corbeille, t_opl%tinj1d(iinj)%nomgenerateur, &
               t_opl%tinj1d(iinj)%theta,t_opl%tinj1d(iinj)%phi, &
               t_opl%tinj1d(iinj)%anglepolar, t_opl%tinj1d(iinj)%eo
	t_opl%tinj1d(iinj)%phase = phase
	t_opl%tinj1d(iinj)%tr = retard
      end if
      
      if (.not.generateur_existe_actif(g,t_opl%tinj1d(iinj)%nomgenerateur)) then
	call lecture_stop(t_fich)
      end if
      
      iinj = iinj + 1
    case(BOP_MILIEU_STRATIFIE_X) 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_opl%flaginjecteuraxial = .true.
      t_opl%dir_injecteur_axial = AXE_X
    case(BOP_MILIEU_STRATIFIE_Y) 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_opl%flaginjecteuraxial = .true.
      t_opl%dir_injecteur_axial = AXE_Y
    case(BOP_MILIEU_STRATIFIE_Z) 
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,1)
      t_opl%flaginjecteuraxial = .true.
      t_opl%dir_injecteur_axial = AXE_Z
    case ('interpolation')
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,ERR=999) corbeille,nom1
      if (nom1=='ordre2') then
        t_opl%flag_ordre2 = .true.
      else if (nom1=='ordre4') then
        t_opl%flag_ordre2 = .false.
      end if
    case default
      if (t_fich%nomfinbloc == premiermot) exit ! fin du bloc      
      print *,'Mot clé non reconnu dans le bloc : ',t_fich%nombloc
      call lecture_stop(t_fich)
 
  end select 
end do				
do i=1, t_opl%nbinjecteur
  tinj1d => t_opl%tinj1d(i)
  if ((MOD(INT(tinj1d%anglepolar),90) /= 0).and.t_opl%flaginjecteuraxial) then
    print *,'Injecteur axial, l''angle de polarisation doit etre un modulo de 90°.'
    call lecture_stop(t_fich)
  end if
end do

if (t_opl%flag_ptref) then
  if (.not.verif_pt_ds_volume('pt_reference',real(t_opl%taille(1:6)),t_opl%t_ptref%pt(1:3))) then
    call lecture_stop(t_fich)
  end if 
end if

return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(t_fich)
	
end subroutine onde_plane_lecture

! ==========================================================
! == Subroutine resultat_champ_ecriture                   ==
! ==========================================================
subroutine onde_plane_ecriture(f,topl,tm)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_onde_plane), intent(inout) :: topl
type(st_maillage), intent(in) :: tm

integer  :: i
character(LEN=LEN_CHAINE) :: ch
real     :: phase, retard,point(3)
integer  :: taille(6)
type(st_injecteurondeplane), pointer  :: tinj1d 


!----- Ecriture des données ---------

taille= origine_cellule6(tm,topl%taille(1:6),.true.)

write(f%unite,'(T3,A,1x,6I5)') trim(BOP_DIM_HUYGS),taille(1:6)
ch = ' ';
do i=1,topl%nbsurface
  ch = trim(ch)//' '//TRIM(topl%plan(i))
end do
write(f%unite,'(T3,A,2x,A)') 'surface',trim(ch)

if (topl%flaginjecteuraxial) then
  if (topl%dir_injecteur_axial == AXE_X) then
    write(f%unite,'(T3,A)') BOP_MILIEU_STRATIFIE_X
  else if (topl%dir_injecteur_axial == AXE_Y) then
    write(f%unite,'(T3,A)') BOP_MILIEU_STRATIFIE_Y
  else
    write(f%unite,'(T3,A)') BOP_MILIEU_STRATIFIE_Z
  end if
end if

if (topl%flagmurelectrique) then
  write(f%unite,'(T3,A,2x,A)') 'mur_electrique',trim(topl%plandumur) 
end if

if (topl%flag_ptref) then
  ch = '  ';
  if (topl%t_ptref%flag_t) ch = trim(ch)//'  '//trim(topl%t_ptref%interv_t)
  if (topl%t_ptref%flag_f) ch = trim(ch)//'  '//trim(topl%t_ptref%interv_f)
  
  point= origine_cellule3(tm,topl%t_ptref%pt,.true.)
  write(f%unite,'(T3,A,2x,A,1x,3G14.6)') 'pt_reference',trim(ch),point(1:3)
end if

phase = 0.0; retard = 0.0;
do i=1, topl%nbinjecteur
  tinj1d => topl%tinj1d(i)
  if (tinj1d%phase /= phase) then
    phase = tinj1d%phase
    write(f%unite,'(T3,A,3x,G14.6)') 'phase',tinj1d%phase
  end if
  if (tinj1d%tr /= retard) then
    retard = tinj1d%tr
    write(f%unite,'(T3,A,3x,G14.6)') 'retard',tinj1d%tr
  end if
  
  write(f%unite,'(T3,A,1x,A,1x,3F10.3,1x,G14.6)') 'injecteur',trim(tinj1d%nomgenerateur), &
                                  tinj1d%theta,tinj1d%phi,tinj1d%anglepolar,tinj1d%eo  

end do

end subroutine onde_plane_ecriture


!===========================================================	
! Subroutine onde_plane_init
! Initialisation des variables, lorsque qu'il n'y a pas de lecture du fichier
! d'entrée
!===========================================================	
Subroutine onde_plane_init()
implicit none
!=== Déclaration des arguments ===


!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine onde_plane_init

!===========================================================	
! Subroutine passpatial1d
! Calcul du pas spatial 1d
!===========================================================	
real function passpatial1d(dx,dy,dz,theta,phi)
implicit none
!=== Déclaration des arguments ===
real, intent(in)    ::dx,dy,dz,theta,phi
!=== Déclaration des variables ===

!===   Début des traitements   ===
passpatial1d = SQRT( dx**2 * (COS(phi)*SIN(theta))**4 + &
                     dy**2 * (SIN(phi)*SIN(theta))**4 + &
		     dz**2 * (COS(theta))**4 )
		     
end function passpatial1d

!===========================================================	
! subroutine transpose_angle0_360
! Transpose l'angle en degré entre 0° et 360° si ce n'est pas le cas
!===========================================================	
subroutine transpose_angle0_360(angle)
implicit none
!=== Déclaration des arguments ===
real, intent(inout)    ::angle
!=== Déclaration des variables ===
logical             :: flagpositif 

!===   Début des traitements   ===
flagpositif = .true.
do while (flagpositif)
 if (angle>=0.0) then
   flagpositif = .false.
 else 
   angle = angle + 360
 end if   
end do
flagpositif = .true.
do while (flagpositif)
 if (angle<360.0) then
   flagpositif = .false.
 else 
   angle = angle - 360
 end if   
end do
end subroutine transpose_angle0_360

!===========================================================	
! function origine3d
! L'origine dans le volume 3d de l'onde plane dépend de 
! l'angle d'incidence
! phi et theta doivent etre compris entre 0° et 360°
!===========================================================	
function origine3d(taille, theta,phi)
implicit none
!=== Déclaration des arguments ===
integer, dimension(1:6),intent(in)   :: taille
real, intent(in)        ::theta,phi
integer, dimension(1:3) :: origine3d

!=== Déclaration des variables ===

!===   Début des traitements   ===

if ((theta>=0.0).and.(theta<=180.0)) then
  origine3d(1) = taille(1)  !imin  
  origine3d(2) = taille(3)  !jmin
  origine3d(3) = taille(5)  !kmin

  if (phi>90.0)   origine3d(1)= taille(2)  !imax
  if (phi>180.0) origine3d(2)= taille(4)  !jmax
  if (phi>270.0) origine3d(1)= taille(1)  !imin
  
  if ((theta>90.0).and.(theta<=180.0)) origine3d(3)= taille(6)  !kmax
else if ((theta>180.0).and.(theta<360.0)) then
  origine3d(1) = taille(2)  !imax  
  origine3d(2) = taille(4)  !jmax
  origine3d(3) = taille(6)  !kmax

  if (phi>90.0)   origine3d(1)= taille(1)  !imin
  if (phi>180.0) origine3d(2)= taille(3)  !jmin
  if (phi>270.0) origine3d(1)= taille(2)  !imax
  
  if ((theta>270.0).and.(theta<360.0)) origine3d(3)= taille(5)  !kminax
end if   
end function origine3d

!===========================================================	
! subroutine angles_reflexion
! Détermine les angles de la réflexion sur un plan conducteur
! d'une onde incidente d'angles (theta_inc,phi_inc)
! Les angles sont en degré
!===========================================================	
subroutine angles_reflexion(type_plan,theta_ref, phi_ref, angle_polar_ref, &
                                    theta_inc, phi_inc, angle_polar_inc)
implicit none
!=== Déclaration des arguments ===
integer, intent(in)   :: type_plan
real, intent(out)        ::theta_ref,phi_ref,angle_polar_ref
real, intent(in)        ::theta_inc,phi_inc,angle_polar_inc

!=== Déclaration des variables ===
integer, dimension(1:3), parameter :: cex = (/-1,1,-1/)
integer, dimension(1:3), parameter :: cey = (/-1,-1,1/)
integer, dimension(1:3), parameter :: cez = (/1,-1,-1/)
real  :: exr,eyr,ezr,ethr,ephr
real  :: rtheta_inc, rphi_inc, rangle_polar_inc
real  :: rtheta_ref, rphi_ref
complex :: e_sph
!===   Début des traitements   ===
select case(type_plan)
case(PLAN_XOY)
  theta_ref = 180.0 - theta_inc
  phi_ref   = phi_inc
case(PLAN_YOZ)
  theta_ref = - theta_inc
  phi_ref   = - phi_inc
case(PLAN_XOZ)
  theta_ref =  theta_inc
  phi_ref   = - phi_inc
end select
! Champ réfléchi normalisé en coordonnées cartésiennes
rtheta_inc       = theta_inc * PI /180.0
rphi_inc         = phi_inc * PI /180.0
rangle_polar_inc = angle_polar_inc * PI /180.0
rtheta_ref       = theta_ref * PI /180.0
rphi_ref         = phi_ref * PI /180.0

exr = cex(type_plan) * ( cos(rangle_polar_inc)*cos(rtheta_inc)*cos(rphi_inc) - &
                         sin(rangle_polar_inc)*sin(rphi_inc) )
eyr = cey(type_plan) * ( cos(rangle_polar_inc)*cos(rtheta_inc)*sin(rphi_inc) + &
                         sin(rangle_polar_inc)*cos(rphi_inc) )
ezr = cez(type_plan) * ( - cos(rangle_polar_inc)*sin(rtheta_inc) )

! Champ réfléchi normalisé en coordonnées sphériques
ethr = +exr*cos(rtheta_ref)*cos(rphi_ref) + eyr*cos(rtheta_ref)*sin(rphi_ref) - ezr*sin(rtheta_ref)
ephr = -exr*sin(rphi_ref) + eyr*cos(rphi_ref)
! Calcul de l'angle de polarisation en réflexion
e_sph = CMPLX(ethr,ephr)
angle_polar_ref = atan2d_cor(e_sph)

end subroutine angles_reflexion

!===========================================================	
! function longueurregle1d
! L'origine dans le volume 3d de l'onde plane dépend de 
! l'angle d'incidence
! phi et theta doivent etre compris entre 0° et 360°
!===========================================================	
integer function longueurregle1d(nbcpml1d,theta,phi,dr,dx,dy,dz,taille)
implicit none
!=== Déclaration des arguments ===
real, intent(in)    ::  theta,phi,dr,dx,dy,dz
integer,intent(in)  :: nbcpml1d
integer,dimension(1:6), intent(in) :: taille
!=== Déclaration des variables ===
real           :: lx, ly, lz
!===   Début des traitements   ===
lx = (taille(2) - taille(1)) * SIN(theta)*COS(phi)*dx 
ly = (taille(4) - taille(3)) * SIN(theta)*SIN(phi)*dy 
lz = (taille(6) - taille(5)) * COS(theta)         *dz 
longueurregle1d = INT((ABS(lx)+ABS(ly)+ABS(lz)) /dr) + 2 + nbcpml1d
end function longueurregle1d

!===========================================================	
! subroutine onde_plane_calcul_injecteur_1d
! Algorithme d'insertion du champ E incident sur les surfaces 
! de Huygens.  Mise à jour du champ H 3D
!===========================================================	
subroutine onde_plane_impress_1d_oblique(t_opl)
implicit none
!=== Déclaration des arguments ===
type(st_onde_plane), intent(in) :: t_opl

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer     :: i,j,k,l,n
real,dimension(:), pointer        :: einc, hinc, ce1, ce2 

!===   Début des traitements   ===
200 FORMAT(100ES19.10E3)

!boucle sur tous les injecteurs
!do l=1,t_opl%nbinjecteur

! Ecriture des données en fréquence
n =  unitefichier() 
open(n, file='excit1de.txt', status = 'replace',action='write')
open(n+1, file='excit1dh.txt', status = 'replace',action='write')

  tinj1d=>t_opl%tinj1d(1)
  einc=>tinj1d%einc
  hinc=>tinj1d%hinc
  
  !calcul de Einc sur la règle 1d
  do i = 1, 600
    !insertion de l'excitation
    einc(DEBUT_REGLE_1D) = tinj1d%tabgene(i)
    !calcul de Einc sur la règle 1d
    !print *,'i =', i
    do j=DEBUT_REGLE_1D+1,tinj1d%longueurregle-1
       einc(j) = tinj1d%cre1 * einc(j) + tinj1d%cre2*(hinc(j)-hinc(j-1))
    end do

    ! mise à jour dans les pmls
    do j = tinj1d%debutpml, tinj1d%longueurregle-1
       tinj1d%eaccu_pml(j) = tinj1d%ce1_pml(j) * tinj1d%eaccu_pml(j) + tinj1d%ce2_pml(j) * (hinc(j)-hinc(j-1))
       einc(j) = einc(j) + tinj1d%cre2 * tinj1d%eaccu_pml(j)
    end do
    
    !calcul de Hinc sur la règle 1d
    do j=DEBUT_REGLE_1D,tinj1d%longueurregle-1
       hinc(j) = tinj1d%crh1 * hinc(j) + tinj1d%crh2*(einc(j+1)-einc(j))
    end do

    ! mise à jour dans les pmls
    do j = tinj1d%debutpml, tinj1d%longueurregle-1
       tinj1d%haccu_pml(j) = tinj1d%ch1_pml(j) * tinj1d%haccu_pml(j) + tinj1d%ch2_pml(j) * (einc(j+1)-einc(j))
       hinc(j) = hinc(j) + tinj1d%crh2 * tinj1d%haccu_pml(j)
    end do
    write(n,200) einc(0), einc(20), einc(50)
    write(n+1,200) hinc(0), hinc(20), hinc(50)
  end do
close(n)
close(n+1)
end subroutine onde_plane_impress_1d_oblique

!===========================================================	
! subroutine onde_plane_impress_inj1d_axial
!===========================================================	
subroutine onde_plane_impress_inj1d_axial(t_opl)
implicit none
!=== Déclaration des arguments ===
type(st_onde_plane), intent(in) :: t_opl

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer     :: i,j,k,l,n
real,dimension(:), pointer        :: einc, hinc

!===   Début des traitements   ===
200 FORMAT(100ES19.10E3)

!boucle sur tous les injecteurs
!do l=1,t_opl%nbinjecteur

! Ecriture des données en fréquence
n =  unitefichier() 
open(n, file='excit1de.txt', status = 'replace',action='write')
open(n+1, file='excit1dh.txt', status = 'replace',action='write')

  tinj1d=>t_opl%tinj1d(1)
  einc=>tinj1d%einc
  hinc=>tinj1d%hinc
  
  !calcul de Einc sur la règle 1d
  do i = 1, 1200
    !calcul de Einc sur la règle 1d
    !print *,'i =', i
    !calcul de Einc sur la règle 1d
    do j=tinj1d%debutregle+1,tinj1d%finregle-1
       einc(j) = tinj1d%ce1(j) * einc(j) - tinj1d%ce2(j)*(hinc(j)-hinc(j-1))
    end do

    !insertion de l'excitation
    einc(tinj1d%coordinjection) = einc(tinj1d%coordinjection) + tinj1d%tabgene(i)

    ! mise à jour dans les pmls inf et sup
    do j = tinj1d%debutregle+1, tinj1d%finpml1
       tinj1d%eaccu_pml(j) = tinj1d%ce1_pml(j) * tinj1d%eaccu_pml(j) + tinj1d%ce2_pml(j) * (hinc(j)-hinc(j-1))
       einc(j) = einc(j)  - tinj1d%ce2(j)*tinj1d%eaccu_pml(j)
    end do

    do j = tinj1d%debutpml2, tinj1d%finregle-1
       tinj1d%eaccu_pml(j) = tinj1d%ce1_pml(j) * tinj1d%eaccu_pml(j) + tinj1d%ce2_pml(j) * (hinc(j)-hinc(j-1))
       einc(j) = einc(j)  - tinj1d%ce2(j)*tinj1d%eaccu_pml(j)
    end do

     !calcul de Hinc sur la règle 1d
     do j=tinj1d%debutregle,tinj1d%finregle-1
	hinc(j) = tinj1d%ch1(j) * hinc(j) - tinj1d%ch2(j)*(einc(j+1)-einc(j))
     end do

     ! mise à jour dans les pmls inf et sup
     do j = tinj1d%debutregle, tinj1d%finpml1-1
	tinj1d%haccu_pml(j) = tinj1d%ch1_pml(j) * tinj1d%haccu_pml(j) + tinj1d%ch2_pml(j) * (einc(j+1)-einc(j))
	hinc(j) = hinc(j)  - tinj1d%ch2(j)*tinj1d%haccu_pml(j)
     end do
     do j = tinj1d%debutpml2, tinj1d%finregle-1
	tinj1d%haccu_pml(j) = tinj1d%ch1_pml(j) * tinj1d%haccu_pml(j) + tinj1d%ch2_pml(j) * (einc(j+1)-einc(j))
	hinc(j) = hinc(j)  - tinj1d%ch2(j)*tinj1d%haccu_pml(j)
     end do

    write(n,200) einc(50), einc(60), einc(130),  einc(200)
    write(n+1,200) hinc(50), hinc(60), hinc(130), hinc(200)
  end do
close(n)
close(n+1)
end subroutine onde_plane_impress_inj1d_axial

!===========================================================	
! Subroutine injecteur_axial_precalcul
! Precalcul dans le cas d'un injecteur axial
!===========================================================	
subroutine injecteur_axial_precalcul(t_opl, t_vol, t_gene, t_c, t_pml)

implicit none

!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout) :: t_opl
type(st_volume), intent(in) :: t_vol
type(st_generateur), intent(inout) :: t_gene
type(st_champ), intent(in) :: t_c
type(st_cpml), intent(in) :: t_pml
!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer        :: l,n,signe,sign_eh
integer, target:: i,j,k
integer        :: imin,imax,jmin,jmax,kmin,kmax
integer        :: pimin,pimax,pjmin,pjmax,pkmin,pkmax
integer        :: p1,p2, infsup
real           :: dervaleurgene
real           :: theta,phi,psi
real           :: dx, dy, dz,dt
real           :: eps, mu, sigmae, sigmah
real,dimension(2)    :: drpml(2),epspml(2), mupml(2)
real           :: sigman
logical        :: flagcoherence, flagpml3d(1:2)
type(st_cpml_x), pointer :: pmlx
type(st_cpml_y), pointer :: pmly
type(st_cpml_z), pointer :: pmlz
real, dimension(:,:,:), pointer :: c21,c22
integer, pointer :: p
!===   Début des traitements   ===
dt = t_vol%dt

imin = t_opl%taille(1);jmin = t_opl%taille(3)
imax = t_opl%taille(2);jmax = t_opl%taille(4)
kmin = t_opl%taille(5);kmax = t_opl%taille(6)


do l=1, t_opl%nbinjecteur
  tinj1d => t_opl%tinj1d(l)
  tinj1d%type_injecteur = INJECTEUR_AXIAL
  ! Calcul du générateur  
  allocate(tinj1d%tabgene(0:t_vol%ntinc))
  tinj1d%tabgene(:) = 0.0;
  dervaleurgene = generateur_calcul_t(t_gene, tinj1d%nomgenerateur, &
         tinj1d%eo, tinj1d%phase, tinj1d%tr, dt, t_vol%ntinc, tinj1d%tabgene)
 
  ! transposition des angles theta et phi entre 0 et 360, pour le calcul de l'origine
  call transpose_angle0_360(tinj1d%theta)
  call transpose_angle0_360(tinj1d%phi)

  ! origine
  tinj1d%origine(1:3) = origine3d(t_opl%taille,tinj1d%theta,tinj1d%phi)
  ! origine décalé de une cellule vers l'extérieur des surfaces de Huygens
  sign_eh = 1
  tinj1d%coordinjection = tinj1d%origine(t_opl%dir_injecteur_axial)-1
  if (tinj1d%origine(t_opl%dir_injecteur_axial) == t_opl%taille(t_opl%dir_injecteur_axial*2)) then
     tinj1d%coordinjection = tinj1d%origine(t_opl%dir_injecteur_axial)+1
    sign_eh = -1  ! pour préserver la bonne polarisation 
  end if
 
        
  ! Controle de la cohérence entre l'axe de propagation et l'angle d'incidence
  ! Initialisation des extrémités et choix de l'origine
  flagpml3d(:)  = .false.
  flagcoherence = .false.
  if (t_opl%dir_injecteur_axial == AXE_X) then
    if ((tinj1d%theta == 90.0) .or.(tinj1d%theta == 270.0)) then
      if ((tinj1d%phi == 0.0) .or.(tinj1d%phi == 180.0)) flagcoherence = .true.
    end if
   write (*,'(A24,I2)') 'Injecteur onde plane n°:', l, 'Injecteur axial X'

    p1 = imin-2; p2 = imax+2 ! décalage de deux cellule par rapport à l'origine
        
    ! injecteur dans les frontieres ?
    if (imin == 1) then !alors corespondance avec le volume 3d
      t_opl%nbcpml1d = 1 - t_vol%ivold
      flagpml3d(1) = .true.
      imin = t_vol%ivold
      p1 = 1
    end if
    if (imax == t_vol%nx+1) then !alors corespondance avec le volume 3d
      t_opl%nbcpml1d = t_vol%ivolf-t_vol%nx
      flagpml3d(2) = .true.
      p2 = imax      
      imax = t_vol%ivolf+1
    end if
    pimin = imin-1; pimax = imax
    pjmin = jmin; pjmax = jmin
    pkmin = kmin; pkmax = kmin

  else if  (t_opl%dir_injecteur_axial == AXE_Y) then 
    write (*,'(A24,I2)') 'Injecteur onde plane n°:', l, 'Injecteur axial Y'
    if ((tinj1d%theta == 90.0) .or.(tinj1d%theta == 270.0)) then
      if ((tinj1d%phi == 90.0) .or.(tinj1d%phi == 270.0)) flagcoherence = .true.
    end if
    p1 = jmin-2; p2 = jmax+2 ! décalage de deux cellules par rapport à l'origine

    ! injecteur dans les frontieres ?
    if (jmin == 1) then !alors corespondance avec le volume 3d
      t_opl%nbcpml1d = 1 - t_vol%jvold
      flagpml3d(1) = .true.
      jmin = t_vol%jvold
      p1 = 1
    end if
    if (jmax == t_vol%ny+1) then !alors corespondance avec le volume 3d
      t_opl%nbcpml1d = t_vol%jvolf-t_vol%ny
      flagpml3d(2) = .true.
      p2 = jmax    
      jmax = t_vol%jvolf+1  
    end if

    pimin = imin; pimax = imin
    pjmin = jmin-1; pjmax = jmax
    pkmin = kmin; pkmax = kmin
  else 
    flagcoherence = .false.
  end if
   
  ! cohérence ?  
  if (.not.flagcoherence) then
    print *,'Incohérence entre l''angle d''incidence et la direction de l''injecteur axial' 
    stop 'Arret ...'
  end if 
  
  
  ! rectification des surfaces de Huygens survenant si elles pénètrent dans
  ! les pmls
  t_opl%taille(1) = imin ; t_opl%taille(3) = jmin
  t_opl%taille(2) = imax ; t_opl%taille(4) = jmax
  t_opl%taille(5) = kmin ; t_opl%taille(6) = kmax
  ! position min et max des surfaces de Huygens suivant l'axe de l'injecteur
  tinj1d%p1 = t_opl%taille((t_opl%dir_injecteur_axial-1)*2+1)
  tinj1d%p2 = t_opl%taille(t_opl%dir_injecteur_axial*2)
    
  !longueur de la règle 1d et allocation
  tinj1d%debutregle = p1 - t_opl%nbcpml1d  
  tinj1d%finpml1 = p1
  tinj1d%debutpml2 = p2
  tinj1d%finregle = p2 + t_opl%nbcpml1d

  !conversion en radian
  tinj1d%thetarad = tinj1d%theta * PI/180 
  tinj1d%phirad   = tinj1d%phi   * PI/180
  tinj1d%anglepolarrad = tinj1d%anglepolar * PI/180
  theta = tinj1d%thetarad
  phi = tinj1d%phirad
  psi = tinj1d%anglepolarrad

  !boucle sur les 6 surfaces de Huygens
  do n=1,6 
    if (.not.t_opl%flagsurface(n)) cycle
    signe = 1
    if (MOD(n,2) == 0) signe = -1    
    !coefficients de conversion du champ des coordonnées sphériques
    !vers les coordonnées cartésiennes
    select case(n)
    case(1,2) !plan xoyinf et sup
       tinj1d%conve1(n) = +signe*( COS(theta)*COS(phi)*COS(psi) - SIN(phi)*SIN(psi))   
       tinj1d%convh1(n) = -signe*(-COS(theta)*COS(phi)*SIN(psi) - SIN(phi)*COS(psi))   
       tinj1d%conve2(n) = -signe*( COS(theta)*SIN(phi)*COS(psi) + COS(phi)*SIN(psi))   
       tinj1d%convh2(n) = +signe*(-COS(theta)*SIN(phi)*SIN(psi) + COS(phi)*COS(psi))   
    case(3,4) !plan yozinf et sup
       tinj1d%conve1(n) = +signe*( COS(theta)*SIN(phi)*COS(psi) + COS(phi)*SIN(psi))   
       tinj1d%convh1(n) = -signe*(-COS(theta)*SIN(phi)*SIN(psi) + COS(phi)*COS(psi))   
       tinj1d%conve2(n) = -signe*(-SIN(theta)*COS(psi))   
       tinj1d%convh2(n) = +signe*( SIN(theta)*SIN(psi))   
    case(5,6) !plan zoxinf et sup
       tinj1d%conve1(n) = +signe*(-SIN(theta)*COS(psi))   
       tinj1d%convh1(n) = -signe*( SIN(theta)*SIN(psi))   
       tinj1d%conve2(n) = -signe*( COS(theta)*COS(phi)*COS(psi) - SIN(phi)*SIN(psi))   
       tinj1d%convh2(n) = +signe*(-COS(theta)*COS(phi)*SIN(psi) - SIN(phi)*COS(psi))   
    end select
  end do
  
  !allocation des tableaux des champs de la règle 1d et de leurs coefficients
  allocate(tinj1d%einc(tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%hinc(tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%ce1 (tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%ce2 (tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%ch1 (tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%ch2 (tinj1d%debutregle:tinj1d%finregle));
  tinj1d%einc(:) = 0.0
  tinj1d%hinc(:) = 0.0
  tinj1d%ce1(:) = 1.0
  tinj1d%ce2(:) = 0.0
  tinj1d%ch1(:) = 1.0
  tinj1d%ch2(:) = 0.0
  

  ! Calcul des coefficients de l'injecteur, fonction des milieux 
  eps = EPSILON_0 * t_vol%epsilon_r; 
  mu      = MU_0      * t_vol%mu_r
  sigmae = t_vol%sigma_e; sigmah = t_vol%sigma_h; 
  
  ! coefficient multiplicatif pour e
  ! le choix des coefficients se fait en fonction de conve1 et conve2 (=0 ou 1)
  if (t_c%flag_dielectrique_inhomogene .or. t_c%flag_dielectrique_quelconque) then

    if (t_opl%dir_injecteur_axial == AXE_X) then
      if (t_c%flag_dielectrique_quelconque) &
        tinj1d%ce1(p1:p2) = t_c%cey1(p1:p2,jmin,kmin)*ABS(tinj1d%conve1(3)) + &
	                    t_c%cez1(p1:p2,jmin,kmin)*ABS(tinj1d%conve2(3))
      tinj1d%ce2(p1:p2) = sign_eh*t_c%dt_epsodx(p1:p2) * (&
                            t_c%cey2(p1:p2,jmin,kmin)*ABS(tinj1d%conve1(3)) + &
			    t_c%cez2(p1:p2,jmin,kmin)*ABS(tinj1d%conve2(3)))
			    
    else if (t_opl%dir_injecteur_axial == AXE_Y) then
      if (t_c%flag_dielectrique_quelconque) &
        tinj1d%ce1(p1:p2) = t_c%cez1(imin,p1:p2,kmin)*ABS(tinj1d%conve1(5)) + &
	                    t_c%cex1(imin,p1:p2,kmin)*ABS(tinj1d%conve2(5))
      tinj1d%ce2(p1:p2) = sign_eh*t_c%dt_epsody(p1:p2) * (&
                            t_c%cez2(imin,p1:p2,kmin)*ABS(tinj1d%conve1(5)) + &
			    t_c%cex2(imin,p1:p2,kmin)*ABS(tinj1d%conve2(5)))
      
    else
      if (t_c%flag_dielectrique_quelconque) &
        tinj1d%ce1(p1:p2) = t_c%cex1(imin,jmin,p1:p2)*ABS(tinj1d%conve1(1)) + &
	                    t_c%cey1(imin,jmin,p1:p2)*ABS(tinj1d%conve2(1))
      tinj1d%ce2(p1:p2) = sign_eh*t_c%dt_epsodz(p1:p2) * (&
                            t_c%cex2(imin,jmin,p1:p2)*ABS(tinj1d%conve1(1)) + &
			    t_c%cey2(imin,jmin,p1:p2)*ABS(tinj1d%conve2(1)))
    end if
    
  else if (t_c%flag_dielectrique_homogene .or.t_c%flag_diel_pertes_homogene) then
    if (t_opl%dir_injecteur_axial == AXE_X) then
      tinj1d%ce2(p1:p2) = sign_eh*t_c%cer2dt_epsodx(p1:p2)
    else if (t_opl%dir_injecteur_axial == AXE_Y) then
      tinj1d%ce2(p1:p2) = sign_eh*t_c%cer2dt_epsody(p1:p2)
    else
      tinj1d%ce2(p1:p2) = sign_eh*t_c%cer2dt_epsodz(p1:p2)
    end if
    
  else if (t_c%flag_diel_pertes_homogene) then
    tinj1d%ce1(:) = (1-sigmae*dt / (eps*2))	/ (1+sigmae*dt / (eps*2))

  end if
  
  tinj1d%ce1(tinj1d%debutregle:p1-1) = tinj1d%ce1(p1) 
  tinj1d%ce1(p2+1:tinj1d%finregle)   = tinj1d%ce1(p2) 
  tinj1d%ce2(tinj1d%debutregle:p1-1) = tinj1d%ce2(p1) 
  tinj1d%ce2(p2+1:tinj1d%finregle)   = tinj1d%ce2(p2) 

  ! coefficient multiplicatif pour h
  if (t_c%flag_magnetique_inhomogene .or.t_c%flag_magnetique_quelconque) then
    if (t_opl%dir_injecteur_axial == AXE_X) then
      if  (t_c%flag_magnetique_quelconque) &
        tinj1d%ch1(p1:p2) = t_c%chy1(p1:p2,jmin,kmin)*ABS(tinj1d%convh1(3)) + &
	                    t_c%chz1(p1:p2,jmin,kmin)*ABS(tinj1d%convh2(3))
			    
      tinj1d%ch2(p1:p2) = sign_eh*t_c%dt_muodxo(p1:p2) * &
                            (t_c%chy2(p1:p2,jmin,kmin)*ABS(tinj1d%convh1(3)) + &
			    t_c%chz2(p1:p2,jmin,kmin)*ABS(tinj1d%convh2(3)))
			    
    else if (t_opl%dir_injecteur_axial == AXE_Y) then
      if  (t_c%flag_magnetique_quelconque) &
        tinj1d%ch1(p1:p2) = t_c%chz1(imin,p1:p2,kmin)*ABS(tinj1d%convh1(5)) + &
	                    t_c%chx1(imin,p1:p2,kmin)*ABS(tinj1d%convh2(5))
			    
      tinj1d%ch2(p1:p2) = sign_eh*t_c%dt_muodyo(p1:p2) * (&
                            t_c%chz2(imin,p1:p2,kmin)*ABS(tinj1d%convh1(5)) + &
			    t_c%chx2(imin,p1:p2,kmin)*ABS(tinj1d%convh2(5)))
			    
    else
      if  (t_c%flag_magnetique_quelconque) &
        tinj1d%ch1(p1:p2) = t_c%chx1(imin,jmin,p1:p2)*ABS(tinj1d%convh1(1)) + &
	                    t_c%chy1(imin,jmin,p1:p2)*ABS(tinj1d%convh2(1))
      tinj1d%ch2(p1:p2) = sign_eh*t_c%dt_muodzo(p1:p2) * (&
                            t_c%chx2(imin,jmin,p1:p2)*ABS(tinj1d%convh1(1)) + &
			    t_c%chy2(imin,jmin,p1:p2)*ABS(tinj1d%convh2(1)))
    end if
          
  else if (t_c%flag_magnetique_homogene .or. t_c%flag_magnet_pertes_homogene) then
    if (t_opl%dir_injecteur_axial == AXE_X) then
      tinj1d%ch2(p1:p2) = sign_eh*t_c%chr2dt_muodxo(p1:p2)
    else if (t_opl%dir_injecteur_axial == AXE_Y) then
      tinj1d%ch2(p1:p2) = sign_eh*t_c%chr2dt_muodyo(p1:p2)
    else
      tinj1d%ch2(p1:p2) = sign_eh*t_c%chr2dt_muodzo(p1:p2)
    end if
        
  else if (t_c%flag_magnet_pertes_homogene) then
    tinj1d%ch1(:) = (1-sigmah*dt / (mu*2))	/ (1+sigmah*dt / (mu*2))
  end if

  tinj1d%ch1(tinj1d%debutregle:p1-1) = tinj1d%ch1(p1) 
  tinj1d%ch1(p2+1:tinj1d%finregle)   = tinj1d%ch1(p2) 
  tinj1d%ch2(tinj1d%debutregle:p1-1) = tinj1d%ch2(p1) 
  tinj1d%ch2(p2+1:tinj1d%finregle)   = tinj1d%ch2(p2) 
  
  ! *** Allocation et Calcul des coefficients dans les pmls ***
  allocate(tinj1d%eaccu_pml(tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%haccu_pml(tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%ce1_pml(tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%ce2_pml(tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%ch1_pml(tinj1d%debutregle:tinj1d%finregle));
  allocate(tinj1d%ch2_pml(tinj1d%debutregle:tinj1d%finregle));
  tinj1d%eaccu_pml = 0.0; tinj1d%haccu_pml = 0.0;
  tinj1d%ce1_pml = 0.0; tinj1d%ce2_pml = 0.0;
  tinj1d%ch1_pml = 0.0; tinj1d%ch2_pml = 0.0;
  

  ! pas spatial 1d début et fin pml
  if (t_opl%dir_injecteur_axial == AXE_X) then
    drpml(1) = t_vol%dxyz(1)
    drpml(2) = t_vol%dxyz(1)    
  else if (t_opl%dir_injecteur_axial == AXE_Y) then
    drpml(1) = t_vol%dxyz(2)
    drpml(2) = t_vol%dxyz(2)    
  else 
    drpml(1) = t_vol%dxyz(3)
    drpml(2) = t_vol%dxyz(3)    
  end if
  
  epspml(1) = (t_vol%dt/drpml(1))*0.5*(1.0 + tinj1d%ce1(p1))/abs(tinj1d%ce2(p1))
  epspml(2) = (t_vol%dt/drpml(2))*0.5*(1.0 + tinj1d%ce1(p2))/abs(tinj1d%ce2(p2))
  mupml(1) = (t_vol%dt/drpml(1))*0.5*(1.0 + tinj1d%ch1(p1))/abs(tinj1d%ch2(p1))
  mupml(2) = (t_vol%dt/drpml(2))*0.5*(1.0 + tinj1d%ch1(p2))/abs(tinj1d%ch2(p2))
  if (rang==RANG_MAITRE_AFF) then
    print '(T10,"eps_r_1D(deb,fin) = ",2F8.2)', epspml(1:2)/EPSILON_0
    print '(T10," mu_r_1D(deb,fin) = ",2F8.2)',  mupml(1:2)/MU_0
  end if
  ! PML inférieur et supérieur
  do i = 1,2  
    if (flagpml3d(i)) then ! on récupère les coefficients des pmls
      if (t_opl%dir_injecteur_axial == AXE_X) then
	if (i == 1) then 
	  pmlx => t_pml%xinf      
	else 
	  pmlx => t_pml%xsup
	end if      
	tinj1d%ce1_pml(pmlx%idebe:pmlx%ifine)  = pmlx%abse(pmlx%idebe:pmlx%ifine)
	tinj1d%ce2_pml(pmlx%idebe:pmlx%ifine)  = pmlx%absee(pmlx%idebe:pmlx%ifine) 
	tinj1d%ch1_pml(pmlx%idebh:pmlx%ifinh)  = pmlx%absh(pmlx%idebh:pmlx%ifinh)
	tinj1d%ch2_pml(pmlx%idebh:pmlx%ifinh)  = pmlx%abshh(pmlx%idebh:pmlx%ifinh) 
      else if (t_opl%dir_injecteur_axial == AXE_Y) then
	if (i == 1) then 
	  pmly => t_pml%yinf      
	else 
	  pmly => t_pml%ysup
	end if      
	tinj1d%ce1_pml(pmly%jdebe:pmly%jfine)  = pmly%abse(pmly%jdebe:pmly%jfine)
	tinj1d%ce2_pml(pmly%jdebe:pmly%jfine)  = pmly%absee(pmly%jdebe:pmly%jfine) 
	tinj1d%ch1_pml(pmly%jdebh:pmly%jfinh)  = pmly%absh(pmly%jdebh:pmly%jfinh)
	tinj1d%ch2_pml(pmly%jdebh:pmly%jfinh)  = pmly%abshh(pmly%jdebh:pmly%jfinh) 
      else 
	if (i == 1) then 
	  pmlz => t_pml%zinf      
	else 
	  pmlz => t_pml%zsup
	end if      
	tinj1d%ce1_pml(pmlz%kdebe:pmlz%kfine)  = pmlz%abse(pmlz%kdebe:pmlz%kfine)
	tinj1d%ce2_pml(pmlz%kdebe:pmlz%kfine)  = pmlz%absee(pmlz%kdebe:pmlz%kfine) 
	tinj1d%ch1_pml(pmlz%kdebh:pmlz%kfinh)  = pmlz%absh(pmlz%kdebh:pmlz%kfinh)
	tinj1d%ch2_pml(pmlz%kdebh:pmlz%kfinh)  = pmlz%abshh(pmlz%kdebh:pmlz%kfinh)       
      end if
      
    else ! on calcule les coefficients des pmls
      sigman = - EPSILON_0*LOG(t_opl%gpml)*LOG(t_opl%rpml)/ &
          (2*drpml(i)*(t_opl%gpml**t_opl%nbcpml1d -1)* SQRT(epspml(i)*mupml(i)))
      if (i ==1) then ! inf
        p1 = tinj1d%debutregle+t_opl%nbcpml1d
	p2 = tinj1d%debutregle+1
	infsup = -1
      else ! sup
        p1 = tinj1d%finregle - t_opl%nbcpml1d
	p2 = tinj1d%finregle-1 
	infsup = 1  
      end if
      do j = p1,p2,infsup
	tinj1d%ce1_pml(j) = EXP(-dt/(t_opl%apml*EPSILON_0) * (t_opl%apml*t_opl%alphapml + sigman))
	tinj1d%ce2_pml(j) = (tinj1d%ce1_pml(j)-1) * (sigman / &
                             (t_opl%apml*(t_opl%apml*t_opl%alphapml + sigman)))

	sigman = sigman * SQRT(t_opl%gpml)

	tinj1d%ch1_pml(j+i-2) = EXP(-dt/(t_opl%apml*EPSILON_0) * (t_opl%apml*t_opl%alphapml + sigman))
	tinj1d%ch2_pml(j+i-2) = (tinj1d%ch1_pml(j+i-2)-1) * (sigman / &
                             (t_opl%apml*(t_opl%apml*t_opl%alphapml + sigman)))
	sigman = sigman * SQRT(t_opl%gpml)
      end do
    end if
  end do
  
! Coefficient multiplicatif du champ incident pour la maj du champ 3d
  p1 = tinj1d%p1-1
  p2 = tinj1d%p2  
  allocate(tinj1d%cvh_e1(p1:p2,1:6))
  allocate(tinj1d%cvh_e2(p1:p2,1:6))
  allocate(tinj1d%cve_h1(p1:p2,1:6))
  allocate(tinj1d%cve_h2(p1:p2,1:6))
  do n=1,6 
    if (.not.t_opl%flagsurface(n)) cycle
    select case(n)
    case(1,2) !plan xoyinf et sup
       if (t_opl%dir_injecteur_axial == AXE_Z) then
         tinj1d%cvh_e1(:,n) = t_c%cer2dt_epsodz(p1:p2) * tinj1d%convh2(n)    
         tinj1d%cvh_e2(:,n) = t_c%cer2dt_epsodz(p1:p2) * tinj1d%convh1(n)    
         tinj1d%cve_h1(:,n) = t_c%chr2dt_muodzo(p1:p2) * tinj1d%conve2(n)    
         tinj1d%cve_h2(:,n) = t_c%chr2dt_muodzo(p1:p2) * tinj1d%conve1(n)
       else
         tinj1d%cvh_e1(:,n) = t_c%cer2dt_epsodz(t_opl%taille(n+2)) * tinj1d%convh2(n)    
         tinj1d%cvh_e2(:,n) = t_c%cer2dt_epsodz(t_opl%taille(n+2)) * tinj1d%convh1(n)    
         tinj1d%cve_h1(:,n) = t_c%chr2dt_muodzo(t_opl%taille(n+2)) * tinj1d%conve2(n)    
         tinj1d%cve_h2(:,n) = t_c%chr2dt_muodzo(t_opl%taille(n+2)) * tinj1d%conve1(n)         
       end if  
    case(3,4) !plan yozinf et sup
       if (t_opl%dir_injecteur_axial == AXE_X) then
	 tinj1d%cvh_e1(:,n) = t_c%cer2dt_epsodx(p1:p2) * tinj1d%convh2(n)    
	 tinj1d%cvh_e2(:,n) = t_c%cer2dt_epsodx(p1:p2) * tinj1d%convh1(n)    
	 tinj1d%cve_h1(:,n) = t_c%chr2dt_muodxo(p1:p2) * tinj1d%conve2(n)    
	 tinj1d%cve_h2(:,n) = t_c%chr2dt_muodxo(p1:p2) * tinj1d%conve1(n)
       else
	 tinj1d%cvh_e1(:,n) = t_c%cer2dt_epsodx(t_opl%taille(n-2)) * tinj1d%convh2(n)    
	 tinj1d%cvh_e2(:,n) = t_c%cer2dt_epsodx(t_opl%taille(n-2)) * tinj1d%convh1(n)    
	 tinj1d%cve_h1(:,n) = t_c%chr2dt_muodxo(t_opl%taille(n-2)) * tinj1d%conve2(n)    
	 tinj1d%cve_h2(:,n) = t_c%chr2dt_muodxo(t_opl%taille(n-2)) * tinj1d%conve1(n)
       end if  
    case(5,6) !plan zoxinf et sup
       if (t_opl%dir_injecteur_axial == AXE_Y) then
	 tinj1d%cvh_e1(:,n) = t_c%cer2dt_epsody(p1:p2) * tinj1d%convh2(n)    
	 tinj1d%cvh_e2(:,n) = t_c%cer2dt_epsody(p1:p2) * tinj1d%convh1(n)    
	 tinj1d%cve_h1(:,n) = t_c%chr2dt_muodyo(p1:p2) * tinj1d%conve2(n)    
	 tinj1d%cve_h2(:,n) = t_c%chr2dt_muodyo(p1:p2) * tinj1d%conve1(n)  
       else
	 tinj1d%cvh_e1(:,n) = t_c%cer2dt_epsody(t_opl%taille(n-2)) * tinj1d%convh2(n)    
	 tinj1d%cvh_e2(:,n) = t_c%cer2dt_epsody(t_opl%taille(n-2)) * tinj1d%convh1(n)    
	 tinj1d%cve_h1(:,n) = t_c%chr2dt_muodyo(t_opl%taille(n-2)) * tinj1d%conve2(n)    
	 tinj1d%cve_h2(:,n) = t_c%chr2dt_muodyo(t_opl%taille(n-2)) * tinj1d%conve1(n)  
       end if  
    end select
  end do
  ! correction pour e
  if (t_opl%dir_injecteur_axial == AXE_X) then
    p => i
  else if (t_opl%dir_injecteur_axial == AXE_Y) then
    p => j
  else
    p => k
  end if
  if (t_c%flag_dielectrique_inhomogene .or. t_c%flag_dielectrique_quelconque) then       
    do n=1,6 
      if (.not.t_opl%flagsurface(n)) cycle
      select case(n)
      case(1,2) !plan xoyinf et sup
        c21 => t_c%cex2
        c22 => t_c%cey2
      case(3,4) !plan yozinf et sup
        c21 => t_c%cey2
        c22 => t_c%cez2
      case(5,6) !plan zoxinf et sup
        c21 => t_c%cez2
        c22 => t_c%cex2
      end select
      do k = pkmin,pkmax
      do j = pjmin,pjmax
      do i = pimin,pimax
        tinj1d%cvh_e1(p,n) = tinj1d%cvh_e1(p,n) * c21(i,j,k)
	tinj1d%cvh_e2(p,n) = tinj1d%cvh_e2(p,n) * c22(i,j,k)
      end do; end do; end do
    end do
  end if
  ! correction pour h
  if (t_c%flag_magnetique_inhomogene .or. t_c%flag_magnetique_quelconque) then       
    do n=1,6 
      if (.not.t_opl%flagsurface(n)) cycle
      select case(n)
      case(1,2) !plan xoyinf et sup
        c21 => t_c%chx2
        c22 => t_c%chy2
      case(3,4) !plan yozinf et sup
        c21 => t_c%chy2
        c22 => t_c%chz2
      case(5,6) !plan zoxinf et sup
        c21 => t_c%chz2
        c22 => t_c%chx2
      end select
      do k = pkmin,pkmax
      do j = pjmin,pjmax
      do i = pimin,pimax
        tinj1d%cve_h1(p,n) = tinj1d%cve_h1(p,n) * c21(i,j,k)
	tinj1d%cve_h2(p,n) = tinj1d%cve_h2(p,n) * c22(i,j,k)
      end do; end do; end do
    end do
  end if

end do

end subroutine injecteur_axial_precalcul

	
!===========================================================	
! Subroutine injecteur_oblique_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
subroutine injecteur_oblique_precalcul(t_opl, t_vol, t_gene,t_mv)

implicit none

!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout) :: t_opl
type(st_volume), intent(in) :: t_vol
type(st_generateur), intent(inout) :: t_gene
type(st_milieu_volume), intent(in) :: t_mv

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer        :: l,i,j,k,n,signe
integer        :: imin,imax,jmin,jmax,kmin,kmax
real           :: d,le1(1:3),le2(1:3),lh1(1:3),lh2(1:3)
real           :: dervaleurgene
real           :: theta,phi,psi
real           :: dx, dy, dz,dt
real           :: eps, mu, sigmae, sigmah
real           :: epsr(3),mur(3)
real           :: ce1,ch1
real           :: sigma0,sigman
integer        :: debutpml
real           :: costheta, sinthetasinphi,sinthetacosphi


!===   Début des traitements   ===
! récupération des pas spatiaux 3D au point imin,jmin,kmin
dx = t_vol%dxyz(1)
dy = t_vol%dxyz(2)
dz = t_vol%dxyz(3)
dt = t_vol%dt

!vérification que le maillage est uniforme au sein de la surface de Huygens
!...

!--> Allocation tableau pour le champ incident mémorisé sur chaque
!--  surface de huygens
!do n=1,6 
!  if (.not.t_opl%flagsurface(n)) cycle
!  ! borne calculé en permutation circulaire
!  i=(n-1)/2+1
!  imin = t_opl%taille(PC21(i));imax = t_opl%taille(PC21(i)+1)
!  jmin = t_opl%taille(PC23(i));jmax = t_opl%taille(PC23(i)+1)
!  if (n>=5) then ! cas inversé pour xoz
!    imin = t_opl%taille(PC23(i));imax = t_opl%taille(PC23(i)+1)
!    jmin = t_opl%taille(PC21(i));jmax = t_opl%taille(PC21(i)+1)
!  end if
!  print '(T15,"N° de la surface : ",I2," ; bornes = ",4I6)',i,imin,imax,jmin,jmax
!  allocate(t_opl%tshuyg(n)%e1inc(imin:imax,jmin:jmax))
!  allocate(t_opl%tshuyg(n)%e2inc(imin:imax,jmin:jmax))
!  allocate(t_opl%tshuyg(n)%h1inc(imin:imax,jmin:jmax))
!  allocate(t_opl%tshuyg(n)%h2inc(imin:imax,jmin:jmax))
!  t_opl%tshuyg(n)%e1inc = 0.0
!  t_opl%tshuyg(n)%e2inc = 0.0
!  t_opl%tshuyg(n)%h1inc = 0.0
!  t_opl%tshuyg(n)%h2inc = 0.0
!end do

!-------------------------------------------------------------------------------
!---------------- boucle sur tous les injecteurs -------------------------------
!-------------------------------------------------------------------------------
write (*,'(A21,I4)') 'Nombre d''injecteur :', t_opl%nbinjecteur

do l=1, t_opl%nbinjecteur_tot
  tinj1d => t_opl%tinj1d(l)
  write (*,'(A24,I2)') 'Injecteur onde plane n°:', l
  ! Calcul du générateur  
  allocate(tinj1d%tabgene(0:t_vol%ntinc))
  tinj1d%tabgene(:) = 0.0;

  ! Cas ou l'injecteur représente une onde réfléchie
  ! Récupération des informations sur les 3 angles
  if (l>t_opl%nbinjecteur) then
    i = l - t_opl%nbinjecteur ! règle liée à la n°i
    tinj1d%tabgene(:) = 0.0; ! pas de générateur
    call angles_reflexion(t_opl%typeplanmur, tinj1d%theta, tinj1d%phi, &
                   tinj1d%anglepolar,t_opl%tinj1d(i)%theta, t_opl%tinj1d(i)%phi, &
                   t_opl%tinj1d(i)%anglepolar)
  else
  dervaleurgene = generateur_calcul_t(t_gene, tinj1d%nomgenerateur, &
      tinj1d%eo, tinj1d%phase, tinj1d%tr, t_vol%dt, t_vol%ntinc, tinj1d%tabgene)
  
  end if

  ! transposition des angles theta et phi, pour le calcul de l'origine
  call transpose_angle0_360(tinj1d%theta)
  call transpose_angle0_360(tinj1d%phi)
  
  !origine
  tinj1d%origine(1:3) = origine3d(t_opl%taille,tinj1d%theta,tinj1d%phi)
  
  !conversion en radian
  tinj1d%thetarad = tinj1d%theta * PI/180.0 
  tinj1d%phirad   = tinj1d%phi   * PI/180.0
  tinj1d%anglepolarrad = tinj1d%anglepolar * PI/180.0
  !pas spatial 1d 
  tinj1d%dr = passpatial1d(dx,dy,dz,tinj1d%thetarad,tinj1d%phirad)
  if (rang==RANG_MAITRE_AFF) then
    print '(T5,"Origine : ",3I4)', tinj1d%origine(1:3)		   
    print '(T5,"Angle theta de réflexion : ",F7.3)', tinj1d%theta		   
    print '(T5,"Angle phi de réflexion : ",F7.3)', tinj1d%phi		   
    print '(T5,"Angle polarisation de réflexion : ",F7.3)', tinj1d%anglepolar		   
    print '(T5,"pas spatial 1d = ",F9.5)', tinj1d%dr		   
  end if
  !longueur de la règle 1d et allocation
  tinj1d%longueurregle = longueurregle1d(t_opl%nbcpml1d,tinj1d%thetarad, &
                                              tinj1d%phirad, tinj1d%dr,dx,dy,dz, &
					      t_opl%taille)

  !allocation des tableaux des champs de la règle 1d et de leurs coefficients
  allocate(tinj1d%einc(DEBUT_REGLE_1D:tinj1d%longueurregle));
  allocate(tinj1d%hinc(DEBUT_REGLE_1D:tinj1d%longueurregle));
  tinj1d%einc(:) = 0.0; tinj1d%hinc(:) = 0.0;  

  !Calcul des coefficients de l'injecteur, fonction du milieu 
  !défini par défaut.
  epsr = cherche_epsilon_cellule(t_mv,t_opl%taille(1),t_opl%taille(3),t_opl%taille(5))
  mur  = cherche_mu_cellule(t_mv,t_opl%taille(1),t_opl%taille(3),t_opl%taille(5))
  !eps = EPSILON_0 * t_vol%epsilon_r; 
  !mu      = MU_0      * t_vol%mu_r
  eps = EPSILON_0 * epsr(1)
  mu      = MU_0      * mur(1)
  sigmae = t_vol%sigma_e; sigmah = t_vol%sigma_h; 
  tinj1d%cre1 = (1-sigmae*dt / (eps*2))   / (1+sigmae*dt / (eps*2))
  tinj1d%crh1 = (1-sigmah*dt / (mu*2))	     / (1+sigmah*dt / (mu*2))
  tinj1d%cre2 = -(dt / (eps*tinj1d%dr)) / (1+sigmae*dt / (eps*2))
  tinj1d%crh2 = -(dt / (mu*tinj1d%dr))	/ (1+sigmah*dt / (mu*2))
  if (rang==RANG_MAITRE_AFF) then
    print '(T5,"cre1,cre2 = ",2ES16.6)',tinj1d%cre1,tinj1d%cre2
    print '(T5,"crh1,crh2 = ",2ES16.6)',tinj1d%crh1,tinj1d%crh2
  end if
  !Allocation et Calcul des coefficients dans les pmls
  debutpml = tinj1d%longueurregle - t_opl%nbcpml1d
  tinj1d%debutpml = debutpml
  allocate(tinj1d%eaccu_pml(debutpml:tinj1d%longueurregle));
  allocate(tinj1d%haccu_pml(debutpml:tinj1d%longueurregle));
  allocate(tinj1d%ce1_pml(debutpml:tinj1d%longueurregle));
  allocate(tinj1d%ce2_pml(debutpml:tinj1d%longueurregle));
  allocate(tinj1d%ch1_pml(debutpml:tinj1d%longueurregle));
  allocate(tinj1d%ch2_pml(debutpml:tinj1d%longueurregle));
  tinj1d%eaccu_pml = 0.0; tinj1d%haccu_pml = 0.0;
  tinj1d%ce1_pml = 0.0; tinj1d%ce2_pml = 0.0;
  tinj1d%ch1_pml = 0.0; tinj1d%ch2_pml = 0.0;
  
  sigma0 = - EPSILON_0*LOG(t_opl%gpml)*LOG(t_opl%rpml)/(2*tinj1d%dr*(t_opl%gpml**t_opl%nbcpml1d -1)* &
	    SQRT(eps*mu))
  
  sigman = sigma0
  
  if (rang==RANG_MAITRE_AFF) then
    print '(T5,"debutpml = ",I5)', debutpml
    print '(T5,"longueur regle 1D = ",I5)', tinj1d%longueurregle
  end if
  
  do i = debutpml,tinj1d%longueurregle-1
    tinj1d%ce1_pml(i) = EXP(-dt/(t_opl%apml*EPSILON_0) * (t_opl%apml*t_opl%alphapml + sigman))
    tinj1d%ce2_pml(i) = (tinj1d%ce1_pml(i)-1) * (sigman / &
                         (t_opl%apml*(t_opl%apml*t_opl%alphapml + sigman)))

    sigman = sigman * SQRT(t_opl%gpml)

    tinj1d%ch1_pml(i) = EXP(-dt/(t_opl%apml*EPSILON_0) * (t_opl%apml*t_opl%alphapml + sigman))
    tinj1d%ch2_pml(i) = (tinj1d%ch1_pml(i)-1) * (sigman / &
                         (t_opl%apml*(t_opl%apml*t_opl%alphapml + sigman)))
    sigman = sigman * SQRT(t_opl%gpml)
!    print '(T5,"ce1_pml,ce2_pml,ch1_pml, ch2_pml (",I2,")= ",4ES16.6E3)',i, &
!             tinj1d%ce1_pml(i),tinj1d%ce2_pml(i),tinj1d%ch1_pml(i),tinj1d%ch2_pml(i)
  end do


  ! Cas ou l'injecteur représente une onde réfléchie
  ! Calcul du retard en distance
  if (l>t_opl%nbinjecteur) then
    i = l - t_opl%nbinjecteur ! règle liée à la n°i
    !
    theta = t_opl%tinj1d(i)%thetarad
    phi = t_opl%tinj1d(i)%phirad
    costheta       = COS(theta)
    sinthetasinphi = SIN(theta)*SIN(phi)
    sinthetacosphi = SIN(theta)*COS(phi)  
    le1(1) = dx * (tinj1d%origine(1) - t_opl%tinj1d(i)%origine(1)) * sinthetacosphi
    le1(2) = dy * (tinj1d%origine(2) - t_opl%tinj1d(i)%origine(2)) * sinthetasinphi
    le1(3) = dz * (tinj1d%origine(3) - t_opl%tinj1d(i)%origine(3)) * costheta
    tinj1d%ip = FLOOR((le1(1) + le1(2) + le1(3))/tinj1d%dr) 
    tinj1d%p = (le1(1) + le1(2) + le1(3))/tinj1d%dr - tinj1d%ip 
    ! décalage 
    tinj1d%ip = tinj1d%ip + DEBUT_REGLE_1D
    if (rang==RANG_MAITRE_AFF) then
      print '(T5,"Distance retard arrondi : ",I3)', tinj1d%ip		   
      print '(T5,"Distance retard cellule : ",F9.5)', tinj1d%p		   
    end if
  end if  

  !*coefficients de conversion du champ des coordonnées sphériques
  !vers les coordonnées cartésiennes
  !*calcul des retards
  ce1 = (dt/eps)/(1+sigmae*dt/(2*eps)) 
  ch1 = (dt/mu)/(1+sigmah*dt/(2*mu)) 
  theta = tinj1d%thetarad
  phi = tinj1d%phirad
  psi = tinj1d%anglepolarrad
  costheta       = COS(theta)
  sinthetasinphi = SIN(theta)*SIN(phi)
  sinthetacosphi = SIN(theta)*COS(phi)  

  allocate(tinj1d%tdr(1:6))
  
  if (rang==RANG_MAITRE_AFF) then
    print '(T5,"Pour Einc = 1.0, on a :")'
    print '(T10,"Etheta = ",F9.6," ; Ephi = ",F9.6)',COS(psi),SIN(psi)
    print '(T10,"Ex = ",F9.6," ; Ey = ",F9.6," ; Ez = ",F9.6)',&
                        COS(theta)*COS(phi)*COS(psi) - SIN(phi)*SIN(psi),&
			COS(theta)*SIN(phi)*COS(psi) + COS(phi)*SIN(psi),&
			-SIN(theta)*COS(psi)
    print '(T10,"Kx =K0*",F9.6," ; Ky = K0*",F9.6," ; Kz = K0*",F9.6)',&
                        sin(theta)*COS(phi),sin(theta)*SIN(phi),cos(theta)
  end if
  !boucle sur les 6 surfaces de Huygens
  do n=1,6 
    if (.not.t_opl%flagsurface(n)) cycle
    signe = 1
    if (MOD(n,2) == 0) signe = -1    
    !coefficients de conversion du champ des coordonnées sphériques
    !vers les coordonnées cartésiennes
    select case(n)
    case(1,2) !plan xoyinf et sup
       tinj1d%conve1(n) = +signe*ch1*( COS(theta)*COS(phi)*COS(psi) - SIN(phi)*SIN(psi))/dz   
       tinj1d%convh1(n) = -signe*ce1*(-COS(theta)*COS(phi)*SIN(psi) - SIN(phi)*COS(psi))/dz  
       tinj1d%conve2(n) = -signe*ch1*( COS(theta)*SIN(phi)*COS(psi) + COS(phi)*SIN(psi))/dz   
       tinj1d%convh2(n) = +signe*ce1*(-COS(theta)*SIN(phi)*SIN(psi) + COS(phi)*COS(psi))/dz   
    case(3,4) !plan yozinf et sup
       tinj1d%conve1(n) = +signe*ch1*( COS(theta)*SIN(phi)*COS(psi) + COS(phi)*SIN(psi))/dx   
       tinj1d%convh1(n) = -signe*ce1*(-COS(theta)*SIN(phi)*SIN(psi) + COS(phi)*COS(psi))/dx   
       tinj1d%conve2(n) = -signe*ch1*(-SIN(theta)*COS(psi))/dx   
       tinj1d%convh2(n) = +signe*ce1*( SIN(theta)*SIN(psi))/dx   
    case(5,6) !plan zoxinf et sup
       tinj1d%conve1(n) = +signe*ch1*(-SIN(theta)*COS(psi))/dy   
       tinj1d%convh1(n) = -signe*ce1*( SIN(theta)*SIN(psi))/dy   
       tinj1d%conve2(n) = -signe*ch1*( COS(theta)*COS(phi)*COS(psi) - SIN(phi)*SIN(psi))/dy   
       tinj1d%convh2(n) = +signe*ce1*(-COS(theta)*COS(phi)*SIN(psi) - SIN(phi)*COS(psi))/dy   
    end select

!    write (*,*)tinj1d%conve1(n),tinj1d%conve2(n),tinj1d%convh1(n),tinj1d%convh2(n)     
    !calcul des positions des champs 3d sur la règle 1d
    select case(n)
    case(1,2) !plan xoyinf et sup
       imin = t_opl%taille(1);jmin = t_opl%taille(3)
       imax = t_opl%taille(2);jmax = t_opl%taille(4)
       k    = t_opl%taille(4+n)
       allocate(tinj1d%tdr(n)%tde1(imin:imax,jmin:jmax))
       allocate(tinj1d%tdr(n)%tde2(imin:imax,jmin:jmax))
       allocate(tinj1d%tdr(n)%tdh1(imin:imax,jmin:jmax))
       allocate(tinj1d%tdr(n)%tdh2(imin:imax,jmin:jmax))
       tinj1d%tdr(n)%tde1(:,:) = 0.0
       tinj1d%tdr(n)%tde2(:,:) = 0.0
       tinj1d%tdr(n)%tdh1(:,:) = 0.0
       tinj1d%tdr(n)%tdh2(:,:) = 0.0
       allocate(tinj1d%tdr(n)%tdie1(imin:imax,jmin:jmax))
       allocate(tinj1d%tdr(n)%tdie2(imin:imax,jmin:jmax))
       allocate(tinj1d%tdr(n)%tdih1(imin:imax,jmin:jmax))
       allocate(tinj1d%tdr(n)%tdih2(imin:imax,jmin:jmax))
       tinj1d%tdr(n)%tdie1(:,:) = 0
       tinj1d%tdr(n)%tdie2(:,:) = 0
       tinj1d%tdr(n)%tdih1(:,:) = 0
       tinj1d%tdr(n)%tdih2(:,:) = 0
       
       le1(3) = (k-tinj1d%origine(3))*costheta*dz
       le2(3) = (k-tinj1d%origine(3))*costheta*dz
       lh1(3) = (k-signe*0.5-tinj1d%origine(3))*costheta*dz
       lh2(3) = (k-signe*0.5-tinj1d%origine(3))*costheta*dz
       do j=jmin,jmax
         le1(2) = (j    -tinj1d%origine(2))*sinthetasinphi*dy
         le2(2) = (j+0.5-tinj1d%origine(2))*sinthetasinphi*dy
         lh1(2) = (j+0.5-tinj1d%origine(2))*sinthetasinphi*dy
         lh2(2) = (j    -tinj1d%origine(2))*sinthetasinphi*dy
         do i=imin,imax
           le1(1) = (i+0.5-tinj1d%origine(1))*sinthetacosphi*dx
           le2(1) = (i    -tinj1d%origine(1))*sinthetacosphi*dx
           lh1(1) = (i    -tinj1d%origine(1))*sinthetacosphi*dx
           lh2(1) = (i+0.5-tinj1d%origine(1))*sinthetacosphi*dx
	   d = (le1(1)+le1(2)+le1(3))/tinj1d%dr
	   tinj1d%tdr(n)%tdie1(i,j) = FLOOR(d)+1
	   tinj1d%tdr(n)%tde1(i,j)  = d-FLOOR(d)
	   
	   d = (le2(1)+le2(2)+le2(3))/tinj1d%dr
	   tinj1d%tdr(n)%tdie2(i,j) = FLOOR(d)+1
	   tinj1d%tdr(n)%tde2(i,j)  = d-FLOOR(d)
	   
	   d = (lh1(1)+lh1(2)+lh1(3))/tinj1d%dr+0.5
	   tinj1d%tdr(n)%tdih1(i,j) = FLOOR(d)
	   tinj1d%tdr(n)%tdh1(i,j)  = d-FLOOR(d)
	   
	   d = (lh2(1)+lh2(2)+lh2(3))/tinj1d%dr+0.5
	   tinj1d%tdr(n)%tdih2(i,j) = FLOOR(d)
	   tinj1d%tdr(n)%tdh2(i,j)  = d-FLOOR(d)
	 end do
       end do
!    write (*,'(40I2)')tinj1d%tdr(n)%tdih2(:,:)
!    write (*,*)tinj1d%tdr(n)%tdh2(:,:)
!     stop

    case(3,4) !plan yozinf et sup
       kmin = t_opl%taille(5);jmin = t_opl%taille(3)
       kmax = t_opl%taille(6);jmax = t_opl%taille(4)
       i    = t_opl%taille(n-2)
       allocate(tinj1d%tdr(n)%tde1(jmin:jmax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tde2(jmin:jmax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdh1(jmin:jmax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdh2(jmin:jmax,kmin:kmax))
       tinj1d%tdr(n)%tde1(:,:) = 0.0
       tinj1d%tdr(n)%tde2(:,:) = 0.0
       tinj1d%tdr(n)%tdh1(:,:) = 0.0
       tinj1d%tdr(n)%tdh2(:,:) = 0.0
       allocate(tinj1d%tdr(n)%tdie1(jmin:jmax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdie2(jmin:jmax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdih1(jmin:jmax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdih2(jmin:jmax,kmin:kmax))
       tinj1d%tdr(n)%tdie1(:,:) = 0
       tinj1d%tdr(n)%tdie2(:,:) = 0
       tinj1d%tdr(n)%tdih1(:,:) = 0
       tinj1d%tdr(n)%tdih2(:,:) = 0

       le1(1) = (i-tinj1d%origine(1))*sinthetacosphi*dx
       le2(1) = (i-tinj1d%origine(1))*sinthetacosphi*dx
       lh1(1) = (i-signe*0.5-tinj1d%origine(1))*sinthetacosphi*dx
       lh2(1) = (i-signe*0.5-tinj1d%origine(1))*sinthetacosphi*dx
       do k=kmin,kmax
         le1(3) = (k    -tinj1d%origine(3))*costheta*dz
         le2(3) = (k+0.5-tinj1d%origine(3))*costheta*dz
         lh1(3) = (k+0.5-tinj1d%origine(3))*costheta*dz
         lh2(3) = (k    -tinj1d%origine(3))*costheta*dz
         do j=jmin,jmax
           le1(2) = (j+0.5-tinj1d%origine(2))*sinthetasinphi*dy
           le2(2) = (j    -tinj1d%origine(2))*sinthetasinphi*dy
           lh1(2) = (j    -tinj1d%origine(2))*sinthetasinphi*dy
           lh2(2) = (j+0.5-tinj1d%origine(2))*sinthetasinphi*dy
	   d = (le1(1)+le1(2)+le1(3))/tinj1d%dr
	   tinj1d%tdr(n)%tdie1(j,k) = FLOOR(d)+1
	   tinj1d%tdr(n)%tde1(j,k)  = d-FLOOR(d)
	   
	   d = (le2(1)+le2(2)+le2(3))/tinj1d%dr
	   tinj1d%tdr(n)%tdie2(j,k) = FLOOR(d)+1 
	   tinj1d%tdr(n)%tde2(j,k)  = d-FLOOR(d)
	   
	   d = (lh1(1)+lh1(2)+lh1(3))/tinj1d%dr+0.5
	   tinj1d%tdr(n)%tdih1(j,k) = FLOOR(d)
	   tinj1d%tdr(n)%tdh1(j,k)  = d-FLOOR(d)
	   
	   d = (lh2(1)+lh2(2)+lh2(3))/tinj1d%dr+0.5
	   tinj1d%tdr(n)%tdih2(j,k) = FLOOR(d)
	   tinj1d%tdr(n)%tdh2(j,k)  = d-FLOOR(d)
	 end do
       end do
!    write (*,'(40I2)')tinj1d%tdr(n)%tdih1(:,:)
!    write (*,*)tinj1d%tdr(n)%tdh1(:,:)
!    if (n==4) stop
    case(5,6) !plan zoxinf et sup
       kmin = t_opl%taille(5);imin = t_opl%taille(1)
       kmax = t_opl%taille(6);imax = t_opl%taille(2)
       j    = t_opl%taille(n-2)
       allocate(tinj1d%tdr(n)%tde1(imin:imax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tde2(imin:imax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdh1(imin:imax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdh2(imin:imax,kmin:kmax))
       tinj1d%tdr(n)%tde1(:,:) = 0.0
       tinj1d%tdr(n)%tde2(:,:) = 0.0
       tinj1d%tdr(n)%tdh1(:,:) = 0.0
       tinj1d%tdr(n)%tdh2(:,:) = 0.0
       allocate(tinj1d%tdr(n)%tdie1(imin:imax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdie2(imin:imax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdih1(imin:imax,kmin:kmax))
       allocate(tinj1d%tdr(n)%tdih2(imin:imax,kmin:kmax))
       tinj1d%tdr(n)%tdie1(:,:) = 0
       tinj1d%tdr(n)%tdie2(:,:) = 0
       tinj1d%tdr(n)%tdih1(:,:) = 0
       tinj1d%tdr(n)%tdih2(:,:) = 0

       le1(2) = (j-tinj1d%origine(2))*sinthetasinphi*dy
       le2(2) = (j-tinj1d%origine(2))*sinthetasinphi*dy
       lh1(2) = (j-signe*0.5-tinj1d%origine(2))*sinthetasinphi*dy
       lh2(2) = (j-signe*0.5-tinj1d%origine(2))*sinthetasinphi*dy

       do k=kmin,kmax
         le1(3) = (k+0.5-tinj1d%origine(3))*costheta*dz
         le2(3) = (k    -tinj1d%origine(3))*costheta*dz
         lh1(3) = (k    -tinj1d%origine(3))*costheta*dz
         lh2(3) = (k+0.5-tinj1d%origine(3))*costheta*dz
         do i=imin,imax
           le1(1) = (i    -tinj1d%origine(1))*sinthetacosphi*dx
           le2(1) = (i+0.5-tinj1d%origine(1))*sinthetacosphi*dx
           lh1(1) = (i+0.5-tinj1d%origine(1))*sinthetacosphi*dx
           lh2(1) = (i    -tinj1d%origine(1))*sinthetacosphi*dx
	   d = (le1(1)+le1(2)+le1(3))/tinj1d%dr
	   tinj1d%tdr(n)%tdie1(i,k) = FLOOR(d)+1
	   tinj1d%tdr(n)%tde1(i,k)  = d-FLOOR(d)
	   
	   d = (le2(1)+le2(2)+le2(3))/tinj1d%dr
	   tinj1d%tdr(n)%tdie2(i,k) = FLOOR(d)+1
	   tinj1d%tdr(n)%tde2(i,k)  = d-FLOOR(d)
	   
	   d = (lh1(1)+lh1(2)+lh1(3))/tinj1d%dr+0.5
	   tinj1d%tdr(n)%tdih1(i,k) = FLOOR(d)
	   tinj1d%tdr(n)%tdh1(i,k)  = d-FLOOR(d)
	   
	   d = (lh2(1)+lh2(2)+lh2(3))/tinj1d%dr+0.5
	   tinj1d%tdr(n)%tdih2(i,k) = FLOOR(d)
	   tinj1d%tdr(n)%tdh2(i,k)  = d-FLOOR(d)
	 end do
       end do
!    write (*,'(40I2)')tinj1d%tdr(n)%tdie1(:,:)
!    write (*,*)tinj1d%tdr(n)%tde1(:,:)
!    stop

    end select
  end do ! boucle sur les 6 surfaces de Huygens  
end do ! boucle sur les injecteurs	

!call onde_plane_calcul_injecteur_1d(t_opl)
!stop 'arret onde plane'			

end subroutine injecteur_oblique_precalcul

!===========================================================	
! Subroutine onde_plane_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
subroutine onde_plane_precalcul(t_opl, t_vol, t_gene, t_c, t_pml,t_mv,tfs)

implicit none

!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout) :: t_opl
type(st_volume), intent(in) :: t_vol
type(st_generateur), intent(inout) :: t_gene
type(st_champ), intent(in) :: t_c
type(st_cpml), intent(in) :: t_pml
type(st_milieu_volume), intent(in) :: t_mv
type(st_fichier_sortie), intent(in)   :: tfs

!=== Déclaration des variables ===
integer, dimension(6) :: ncell
integer  :: i

! mur électrique ...
if (t_opl%flagmurelectrique) then
  t_opl%flagsurface(t_opl%nufacemur) = .false.
end if

! surface de huygens sur une frontière
ncell(2) = t_vol%nx
ncell(4) = t_vol%ny
ncell(6) = t_vol%nz
do i = 1,5,2
  if (t_opl%taille(i) <= 1) t_opl%flagsurface(P6XYZ1(i)) = .false.
  if (t_opl%taille(i+1) >= (ncell(i+1)+1)) t_opl%flagsurface(P6XYZ1(i+1)) = .false.  
end do

! precalcul injecteur ...
if (t_opl%flaginjecteuraxial) then
  call injecteur_axial_precalcul(t_opl, t_vol, t_gene, t_c, t_pml)
! pour le deboggage
!  call onde_plane_impress_inj1d_axial(t_opl)
!  stop
else
  call injecteur_oblique_precalcul(t_opl, t_vol, t_gene,t_mv)  
! pour le deboggage
!  call onde_plane_impress_1d_oblique(t_opl)
!  stop
end if

if (t_opl%flag_ptref) call onde_plane_precalcul_ptref(t_opl, t_vol,tfs)
end subroutine onde_plane_precalcul

!===========================================================	
! Subroutine onde_plane_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
subroutine onde_plane_precalcul_ptref(t_opl, t_vol, tfs)

implicit none

!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout) :: t_opl
type(st_volume), intent(in) :: t_vol
type(st_fichier_sortie), intent(in)   :: tfs

!=== Déclaration des variables ===
integer  :: i,ipe,iph,origine(3)
real     :: lx,ly,lz,d
type (st_ptref_onde_plane),pointer :: tpt
type(st_injecteurondeplane), pointer :: t1d
character(LEN=LEN_MOT_CLE) :: nomfichier


! allocation dynamique
tpt =>t_opl%t_ptref
allocate(tpt%psi(t_opl%nbinjecteur_tot))
allocate(tpt%p(1:2,t_opl%nbinjecteur_tot))


do i=1,t_opl%nbinjecteur_tot
  t1d =>t_opl%tinj1d(i)
  origine(3) = 0
  if (.not.t_opl%flaginjecteuraxial) origine = t1d%origine
  tpt%psi(i) = t1d%anglepolarrad
  lx = (tpt%pt(1)-origine(1))*sin(t1d%thetarad)*cos(t1d%phirad)*t_vol%dxyz(1)
  ly = (tpt%pt(2)-origine(2))*sin(t1d%thetarad)*sin(t1d%phirad)*t_vol%dxyz(2)
  lz = (tpt%pt(3)-origine(3))*cos(t1d%thetarad)                *t_vol%dxyz(3)
  
  
  d = (lx+ly+lz)/t1d%dr
  ipe = floor(d)
  iph = Floor(d-0.5)
  tpt%p(1,i)  = d - floor(d)
  tpt%p(2,i)  = d-0.5 - floor(d-0.5)
!  if (rang==RANG_MAITRE_AFF) &
    print '(T5," Point de référence : (ipe,iph) =",2I5,"  (pe,ph) =",2F8.5)',ipe,iph,tpt%p(1:2,i)
  tpt%einc => t1d%einc(ipe:ipe+1)
  tpt%hinc => t1d%hinc(iph:iph+1)
end do

!-- Stockage en temps, ouverture du fichier --
!if (rang==RANG_MAITRE_AFF) &
  print '(T15,"flag_t et flag_f =",2L2)',tpt%flag_t,tpt%flag_f 
if (tpt%flag_t) then
  nomfichier = fichier_sortie_nom_temps(tfs,'pt_reference')
  tpt%unite = ouvre_en_ecriture(nomfichier)
  tpt%mantisse = '(' //'10'//trim(tfs%mantisse_reel)//')'
!  if (rang==RANG_MAITRE_AFF) &
    print '(T15,"mantisse pour le stockage temporel",A)',TRIM(tpt%mantisse)
end if

! -- Tableaux pour le stockage des composantes  en frequence --
! -- Stockage en frequence --
if (tpt%flag_f) then
  allocate(tpt%ehinc_f(1:tpt%tf_f%n_liste,1:4))          
  tpt%ehinc_f = 0.0          
end if

end subroutine onde_plane_precalcul_ptref


!===========================================================	
! subroutine injecteur_oblique_champ_surface
! Algorithme de mise à jour d'une des 6 composantes du champ
! sur une quelconque surface de Huygens 
! La mise à jour est effectuée a partir du champ incident 
! calculé sur la règle 1D
!===========================================================	
subroutine injecteur_oblique_champ_surface(champ3d,champinc,conv,d,di,l1,l2,flag_ordre2)

!=== Déclaration des arguments ===
real,dimension(0:,0:), intent(inout) :: champ3d
real,dimension(0:,0:), intent(in)     :: d
integer(kind=2),dimension(0:,0:), intent(in)  :: di
real,dimension(DEBUT_REGLE_1D:), intent(in)     :: champinc
real, intent(in)   :: conv
integer,intent(in) :: l1,l2
logical, intent(in) :: flag_ordre2

!===  Déclaration des variables  ===
integer  :: i,j
real   :: champ_incident,p, c1_6
integer:: ip

!===   Début des traitements   ===
! boucle sur la surface plan xoy
if (flag_ordre2) then
  do j= 0, l2
  do i= 0, l1
    p = d(i,j)
    ip = di(i,j)
    ! interpolation linéaire
   !champ_incident =  conv *((1.0-d(i,j))*champinc(ip-1) + d(i,j)*champinc(ip))
   
   champ3d(i,j)  = champ3d(i,j) + conv *(&
                   (1-p)*champinc(ip-1) + p*champinc(ip) )
	  
    ! mise à jour de la composante du champ
   ! champ3d(i,j) = champ3d(i,j) +  champ_incident
    
  !  if (champ_incident>1000.0) then
  !    print '("!!! injecteur_oblique_champ_surface : Champ incident =",ES16.6)',champ_incident
  !  end if
  end do
  end do
else
  c1_6 = 1.0 / 6.0 ! coefficient pour interpolation ordre 4
  ! boucle sur la surface plan xoy
  do j= 0, l2
  do i= 0, l1
    p = d(i,j)
    ip = di(i,j)
    ! interpolation Lagrange à l'ordre 4
    champ_incident = - p*(p-1)*(p-2)*c1_6*champinc(ip-2)  &
		     + (p**2-1)*(p-2)*0.5*champinc(ip-1)  &
		     - p*(p+1)*(p-2)*0.5 *champinc(ip) &
		     + p*(p**2-1)*c1_6   *champinc(ip+1)
	  
    ! mise à jour de la composante du champ
    champ3d(i,j) = champ3d(i,j) +  conv *champ_incident
    
  end do
  end do
end if
end subroutine injecteur_oblique_champ_surface

!===========================================================	
! subroutine injecteur_oblique_ehinc_huyg
! Algorithme de calcul du champ incident 
! sur une quelconque surface de Huygens 
!===========================================================	
subroutine injecteur_oblique_ehinc_huyg(champinc,chinc1d,d,di,l1,l2)

!=== Déclaration des arguments ===
real,dimension(0:,0:), intent(inout) :: champinc
real,dimension(0:,0:), intent(in)     :: d
integer(kind=2),dimension(0:,0:), intent(in)  :: di
real,dimension(DEBUT_REGLE_1D:), intent(in)     :: chinc1d
integer,intent(in) :: l1,l2

!===  Déclaration des variables  ===
integer  :: i,j
real   :: p, c1_6
integer:: ip

!===   Début des traitements   ===
c1_6 = 1.0 / 6.0 ! coefficient pour interpolation ordre 4
! boucle sur la surface plan xoy
do j= 0, l2
do i= 0, l1
  p = d(i,j)
  ip = di(i,j)
  ! interpolation linéaire
 champinc(i,j) = champinc(i,j)+ (1-p)*chinc1d(ip-1) + p*chinc1d(ip)
 
  ! interpolation Lagrange à l'ordre 4
!  champ_incident = - p*(p-1)*(p-2)*c1_6*chinc1d(ip-2)  &
!	           + (p**2-1)*(p-2)*0.5*chinc1d(ip-1)  &
!	           - p*(p+1)*(p-2)*0.5 *chinc1d(ip) &
!	           + p*(p**2-1)*c1_6   *chinc1d(ip+1)
	
end do
end do

end subroutine injecteur_oblique_ehinc_huyg


!===========================================================	
! subroutine injecteur_oblique_maj_eh
! Algorithme de mise à jour d'une des 6 composantes du champ
! sur une quelconque surface de Huygens 
! La mise à jour est effectuée a partir du champ incident 
! calculé auparavant sur la surface de huygens
!===========================================================	
subroutine injecteur_oblique_maj_eh(champ3d,champinc,conv,l1,l2)

!=== Déclaration des arguments ===
real,dimension(0:,0:), intent(inout) :: champ3d
real,dimension(0:,0:), intent(inout) :: champinc
real, intent(in)   :: conv
integer,intent(in) :: l1,l2

!===  Déclaration des variables  ===
integer  :: i,j

!===   Début des traitements   ===
!--> Mise à jour
do j= 0, l2
do i= 0, l1
  champ3d(i,j) = champ3d(i,j) + conv * champinc(i,j)
end do
end do
!--> Réinitialisation du champ incident
champinc = 0.0
end subroutine injecteur_oblique_maj_eh

!===========================================================	
! function injecteur_oblique_champ_ref
! calcul du champ réfléchi à partir du champ incident
!===========================================================	
real function injecteur_oblique_champ_ref(champinc,p,ip)

implicit none
!=== Déclaration des arguments ===
real,dimension(DEBUT_REGLE_1D:), intent(in)     :: champinc
real, intent(in)     :: p
integer, intent(in)  :: ip

!===  Déclaration des variables  ===
real   :: c1_6

!===   Début des traitements   ===
c1_6 = 1.0 / 6.0 ! coefficient pour interpolation ordre 4

! interpolation Lagrange à l'ordre 4
  injecteur_oblique_champ_ref = &
        - p*(p-1)*(p-2)*c1_6*champinc(ip-1)  &
	+ (p**2-1)*(p-2)*0.5*champinc(ip)  &
	- p*(p+1)*(p-2)*0.5 *champinc(ip+1) &
	+ p*(p**2-1)*c1_6   *champinc(ip+2)
	
end function injecteur_oblique_champ_ref

!===========================================================	
! subroutine injecteur_oblique_calcul_champe
! Algorithme d'insertion du champ E incident sur les surfaces 
! de Huygens.  Mise à jour du champ E 3D
!===========================================================	
subroutine injecteur_oblique_calcul_champe(itertemporel,t_opl,t_ch)
implicit none
!=== Déclaration des arguments ===
integer,intent(in)          :: itertemporel
type(st_onde_plane), intent(inout) :: t_opl
type(st_champ), intent(inout) :: t_ch

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer     :: i,j,k,l,n
integer     :: imin,jmin,kmin,imax,jmax,kmax
real,dimension(:), pointer        :: einc, hinc, ce1, ce2 

!===   Début des traitements   ===

!boucle sur tous les injecteurs
do l=1,t_opl%nbinjecteur_tot

  tinj1d=>t_opl%tinj1d(l)
  einc=>tinj1d%einc
  hinc=>tinj1d%hinc
  
  
  !insertion de l'excitation
  if (l>t_opl%nbinjecteur) then
    i = l - t_opl%nbinjecteur ! règle 1d en réflexion liée à la n°i
    einc(DEBUT_REGLE_1D) = injecteur_oblique_champ_ref(t_opl%tinj1d(i)%einc, &
                                    tinj1d%p, tinj1d%ip)
  else
    einc(DEBUT_REGLE_1D) = tinj1d%tabgene(itertemporel)
  end if
  !calcul de Einc sur la règle 1d
  do j=DEBUT_REGLE_1D+1,tinj1d%longueurregle-1
     einc(j) = tinj1d%cre1 * einc(j) + tinj1d%cre2*(hinc(j)-hinc(j-1))
  end do
  
  ! mise à jour dans les pmls
  do j = tinj1d%debutpml, tinj1d%longueurregle-1
     tinj1d%eaccu_pml(j) = tinj1d%ce1_pml(j) * tinj1d%eaccu_pml(j) + tinj1d%ce2_pml(j) * (hinc(j)-hinc(j-1))
  end do
  
  do j = tinj1d%debutpml, tinj1d%longueurregle-1
     einc(j) = einc(j) + tinj1d%cre2 * tinj1d%eaccu_pml(j)
  end do
  

  !boucle sur toutes les surfaces de Huygens
  do n=1,6 
    if (.not.t_opl%flagsurface(n)) cycle
    select case(n)
    case(1,2) !plan xoyinf et sup
	! bornes
       imin = t_opl%taille(1);jmin = t_opl%taille(3)
       imax = t_opl%taille(2);jmax = t_opl%taille(4)
       k    = t_opl%taille(4+n)
       
       !=== composante Ex ===
       !call injecteur_oblique_ehinc_huyg(t_opl%tshuyg(n)%h1inc(imin:imax,jmin:jmax),hinc, &
        !    tinj1d%tdr(n)%tdh2(imin:imax,jmin:jmax), &
	!    tinj1d%tdr(n)%tdih2(imin:imax,jmin:jmax),imax-1-imin,jmax-jmin)
	    
       call injecteur_oblique_champ_surface(t_ch%ex(imin:imax,jmin:jmax,k),hinc,tinj1d%convh2(n), &
            tinj1d%tdr(n)%tdh2(imin:imax,jmin:jmax), &
	    tinj1d%tdr(n)%tdih2(imin:imax,jmin:jmax),imax-1-imin,jmax-jmin,t_opl%flag_ordre2)
       !=== composante Ey ===
       !call injecteur_oblique_ehinc_huyg(t_opl%tshuyg(n)%h2inc(imin:imax,jmin:jmax),hinc, &
        !    tinj1d%tdr(n)%tdh1(imin:imax,jmin:jmax), &
	!    tinj1d%tdr(n)%tdih1(imin:imax,jmin:jmax),imax-imin,jmax-jmin-1)
       call injecteur_oblique_champ_surface(t_ch%ey(imin:imax,jmin:jmax,k),hinc,tinj1d%convh1(n), &
            tinj1d%tdr(n)%tdh1(imin:imax,jmin:jmax), &
	    tinj1d%tdr(n)%tdih1(imin:imax,jmin:jmax),imax-imin,jmax-jmin-1,t_opl%flag_ordre2)

    case(3,4) !plan yozinf et sup
	! bornes
       kmin = t_opl%taille(5);jmin = t_opl%taille(3)
       kmax = t_opl%taille(6);jmax = t_opl%taille(4)
       i    = t_opl%taille(n-2)	

       !=== composante Ey ===
       !call injecteur_oblique_ehinc_huyg(t_opl%tshuyg(n)%h1inc(jmin:jmax,kmin:kmax),hinc, &
        !    tinj1d%tdr(n)%tdh2(jmin:jmax,kmin:kmax), &
        !    tinj1d%tdr(n)%tdih2(jmin:jmax,kmin:kmax),jmax-1-jmin,kmax-kmin)
       call injecteur_oblique_champ_surface(t_ch%ey(i,jmin:jmax,kmin:kmax),hinc,tinj1d%convh2(n), &
            tinj1d%tdr(n)%tdh2(jmin:jmax,kmin:kmax), &
            tinj1d%tdr(n)%tdih2(jmin:jmax,kmin:kmax),jmax-1-jmin,kmax-kmin,t_opl%flag_ordre2)

       !=== composante Ez ===
       !call injecteur_oblique_ehinc_huyg(t_opl%tshuyg(n)%h2inc(jmin:jmax,kmin:kmax),hinc, &
        !    tinj1d%tdr(n)%tdh1(jmin:jmax,kmin:kmax), &
        !    tinj1d%tdr(n)%tdih1(jmin:jmax,kmin:kmax),jmax-jmin,kmax-1-kmin)
       call injecteur_oblique_champ_surface(t_ch%ez(i,jmin:jmax,kmin:kmax),hinc,tinj1d%convh1(n), &
            tinj1d%tdr(n)%tdh1(jmin:jmax,kmin:kmax), &
            tinj1d%tdr(n)%tdih1(jmin:jmax,kmin:kmax),jmax-jmin,kmax-1-kmin,t_opl%flag_ordre2)
       
    case(5,6) !plan zoxinf et sup
       kmin = t_opl%taille(5);imin = t_opl%taille(1)
       kmax = t_opl%taille(6);imax = t_opl%taille(2)
       j    = t_opl%taille(n-2)

       !=== composante Ez ===
       !call injecteur_oblique_ehinc_huyg(t_opl%tshuyg(n)%h1inc(imin:imax,kmin:kmax),hinc, &
        !    tinj1d%tdr(n)%tdh2(imin:imax,kmin:kmax), &
        !    tinj1d%tdr(n)%tdih2(imin:imax,kmin:kmax),imax-imin,kmax-1-kmin)
       call injecteur_oblique_champ_surface(t_ch%ez(imin:imax,j,kmin:kmax),hinc,tinj1d%convh2(n), &
            tinj1d%tdr(n)%tdh2(imin:imax,kmin:kmax), &
            tinj1d%tdr(n)%tdih2(imin:imax,kmin:kmax),imax-imin,kmax-1-kmin,t_opl%flag_ordre2)

       !=== composante Ex ===
       !call injecteur_oblique_ehinc_huyg(t_opl%tshuyg(n)%h2inc(imin:imax,kmin:kmax),hinc, &
        !    tinj1d%tdr(n)%tdh1(imin:imax,kmin:kmax), &
        !    tinj1d%tdr(n)%tdih1(imin:imax,kmin:kmax),imax-1-imin,kmax-kmin)
       call injecteur_oblique_champ_surface(t_ch%ex(imin:imax,j,kmin:kmax),hinc,tinj1d%convh1(n), &
            tinj1d%tdr(n)%tdh1(imin:imax,kmin:kmax), &
            tinj1d%tdr(n)%tdih1(imin:imax,kmin:kmax),imax-1-imin,kmax-kmin,t_opl%flag_ordre2)

    end select
  end do  ! boucle sur les surfaces
end do !boucle sur les injecteurs


end subroutine injecteur_oblique_calcul_champe


!===========================================================	
! subroutine injecteur_oblique_calcul_champh
! Algorithme d'insertion du champ E incident sur les surfaces 
! de Huygens.  Mise à jour du champ H 3D
!===========================================================	
subroutine injecteur_oblique_calcul_champh(itertemporel,t_opl,t_ch)
implicit none
!=== Déclaration des arguments ===
integer,intent(in)          :: itertemporel
type(st_onde_plane), intent(inout) :: t_opl
type(st_champ), intent(inout) :: t_ch

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer     :: i,j,k,l,n
integer     :: imin,jmin,kmin,imax,jmax,kmax
real,dimension(:), pointer        :: einc, hinc, ch1, ch2 

!===   Début des traitements   ===

!boucle sur tous les injecteurs
do l=1,t_opl%nbinjecteur_tot

  tinj1d=>t_opl%tinj1d(l)
  einc=>tinj1d%einc
  hinc=>tinj1d%hinc
  
  !calcul de Hinc sur la règle 1d
  do j=DEBUT_REGLE_1D,tinj1d%longueurregle-1
     hinc(j) = tinj1d%crh1 * hinc(j) + tinj1d%crh2*(einc(j+1)-einc(j))
  end do

  ! mise à jour dans les pmls
  do j = tinj1d%debutpml, tinj1d%longueurregle-1
     tinj1d%haccu_pml(j) = tinj1d%ch1_pml(j) * tinj1d%haccu_pml(j) + tinj1d%ch2_pml(j) * (einc(j+1)-einc(j))
  end do
  do j = tinj1d%debutpml, tinj1d%longueurregle-1
     hinc(j) = hinc(j) + tinj1d%crh2 * tinj1d%haccu_pml(j)
  end do
!  print '("hinc(0),hinc(60),einc(0),einc(60) = ",4ES16.6)',hinc(0),hinc(60),einc(0),einc(60)
  !boucle sur toutes les surfaces de Huygens
  do n=1,6 
    if (.not.t_opl%flagsurface(n)) cycle

    select case(n)
    case(1,2) !plan xoyinf et sup
	! bornes
       imin = t_opl%taille(1);jmin = t_opl%taille(3)
       imax = t_opl%taille(2);jmax = t_opl%taille(4)
       k    = t_opl%taille(4+n)
       if (n==1) k = k-1
       !=== composante Hx ===
       call injecteur_oblique_champ_surface(t_ch%hx(imin:imax,jmin:jmax,k),einc,tinj1d%conve2(n), &
            tinj1d%tdr(n)%tde2(:,:),tinj1d%tdr(n)%tdie2(:,:),imax-imin,jmax-1-jmin,t_opl%flag_ordre2)
       !=== composante Hy ===
       call injecteur_oblique_champ_surface(t_ch%hy(imin:imax,jmin:jmax,k),einc,tinj1d%conve1(n), &
            tinj1d%tdr(n)%tde1(:,:),tinj1d%tdr(n)%tdie1(:,:),imax-1-imin,jmax-jmin,t_opl%flag_ordre2)

    case(3,4) !plan yozinf et sup
	! bornes
       kmin = t_opl%taille(5);jmin = t_opl%taille(3)
       kmax = t_opl%taille(6);jmax = t_opl%taille(4)
       i    = t_opl%taille(n-2)	
       if (n==3) i = i-1

       !=== composante Hy ===
       call injecteur_oblique_champ_surface(t_ch%hy(i,jmin:jmax,kmin:kmax),einc,tinj1d%conve2(n), &
            tinj1d%tdr(n)%tde2(:,:), tinj1d%tdr(n)%tdie2(:,:),jmax-jmin,kmax-1-kmin,t_opl%flag_ordre2)

       !=== composante Hz ===
       call injecteur_oblique_champ_surface(t_ch%hz(i,jmin:jmax,kmin:kmax),einc,tinj1d%conve1(n), &
            tinj1d%tdr(n)%tde1(:,:), tinj1d%tdr(n)%tdie1(:,:),jmax-jmin-1,kmax-kmin,t_opl%flag_ordre2)
       
    case(5,6) !plan zoxinf et sup
       kmin = t_opl%taille(5);imin = t_opl%taille(1)
       kmax = t_opl%taille(6);imax = t_opl%taille(2)
       j    = t_opl%taille(n-2)
       if (n==5) j = j-1

       !=== composante Hz ===
       call injecteur_oblique_champ_surface(t_ch%hz(imin:imax,j,kmin:kmax),einc,tinj1d%conve2(n), &
            tinj1d%tdr(n)%tde2(:,:), tinj1d%tdr(n)%tdie2(:,:),imax-imin-1,kmax-kmin,t_opl%flag_ordre2)

       !=== composante Hx ===
       call injecteur_oblique_champ_surface(t_ch%hx(imin:imax,j,kmin:kmax),einc,tinj1d%conve1(n), &
            tinj1d%tdr(n)%tde1(:,:), tinj1d%tdr(n)%tdie1(:,:),imax-imin,kmax-kmin-1,t_opl%flag_ordre2)

    end select
  end do  ! boucle sur les surfaces
end do !boucle sur les injecteurs

end subroutine injecteur_oblique_calcul_champh

!===========================================================	
! subroutine calcul_axial_champ_surface
! Algorithme de mise à jour d'une des 12 composantes du champ
! sur une quelconque surface de Huygens 
! La mise à jour est effectuée a partir du champ incident 
! calculé sur la règle 1D orienté suivant un axe du repère 
! cartésien.
!===========================================================	
subroutine calcul_axial_champ_surface(champ3d,champinc,c2,l1,l2,ind3d,ind1d,pointe)

implicit none
!=== Déclaration des arguments ===
real,dimension(0:,0:), intent(inout) :: champ3d
real,dimension(-1:), intent(in)     :: champinc, c2
integer,intent(in) :: l1,l2, pointe
integer, intent(in), target :: ind3d,ind1d

!===  Déclaration des variables  ===
integer,target  :: i,j 
integer, pointer :: p3d,p1d
!===   Début des traitements   ===

if (pointe == 1) then
  p3d => i
  p1d => i
else if (pointe == 2) then
  p3d => j
  p1d => j
else
  p3d => ind3d
  p1d => ind1d
end if

do j= 0, l2
do i= 0, l1
  ! mise à jour de la composante du champ 3d
  champ3d(i,j) = champ3d(i,j) + c2(p3d) * champinc(p1d)	     
end do
end do

end subroutine calcul_axial_champ_surface



!===========================================================	
! subroutine injecteur_axial_calcul_champe
! Algorithme d'insertion du champ H incident sur les surfaces 
! de Huygens.  Mise à jour du champ E 3D
!===========================================================	
subroutine injecteur_axial_calcul_champe(itertemporel,t_opl,t_ch)
implicit none
!=== Déclaration des arguments ===
integer,intent(in)          :: itertemporel
type(st_onde_plane), intent(inout) :: t_opl
type(st_champ), intent(inout) :: t_ch

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer     :: i,j,k,l,n,ph,pe
integer     :: imin,jmin,kmin,imax,jmax,kmax
integer     :: axe
real,dimension(:), pointer        :: einc, hinc, ce1, ce2 

!===   Début des traitements   ===

axe  = t_opl%dir_injecteur_axial
imin = t_opl%taille(1)
imax = t_opl%taille(2)
jmin = t_opl%taille(3)
jmax = t_opl%taille(4)
kmin = t_opl%taille(5)
kmax = t_opl%taille(6)

!boucle sur tous les injecteurscoordinjection
do l=1,t_opl%nbinjecteur

  tinj1d=>t_opl%tinj1d(l)
  
  
  einc=>tinj1d%einc
  hinc=>tinj1d%hinc
  
  
  !calcul de Einc sur la règle 1d
  do j=tinj1d%debutregle+1,tinj1d%finregle-1
     einc(j) = tinj1d%ce1(j) * einc(j) - tinj1d%ce2(j)*(hinc(j)-hinc(j-1))
  end do

!  if (itertemporel == 2) then
!    do j=tinj1d%debutregle,tinj1d%finregle
!      write(100,'("n°cel, ce1_pml, ce2_pml, ce2 = ",I4,3ES14.5e2)') j,tinj1d%ce1_pml(j),tinj1d%ce2_pml(j),tinj1d%ce2(j)
!    end do
!  end if
  !insertion de l'excitation
  einc(tinj1d%coordinjection) = einc(tinj1d%coordinjection) + tinj1d%tabgene(itertemporel)
!  einc(tinj1d%coordinjection) =  tinj1d%tabgene(itertemporel)
  
 ! write(100,'(I4,2ES16.5e2)') itertemporel,einc(tinj1d%coordinjection+2),einc(tinj1d%coordinjection-2)
  
  ! mise à jour dans les pmls inf et sup
  do j = tinj1d%debutregle+1, tinj1d%finpml1
     tinj1d%eaccu_pml(j) = tinj1d%ce1_pml(j) * tinj1d%eaccu_pml(j) + tinj1d%ce2_pml(j) * (hinc(j)-hinc(j-1))
     einc(j) = einc(j)  - tinj1d%ce2(j)*tinj1d%eaccu_pml(j)
  end do
  
  do j = tinj1d%debutpml2, tinj1d%finregle-1
     tinj1d%eaccu_pml(j) = tinj1d%ce1_pml(j) * tinj1d%eaccu_pml(j) + tinj1d%ce2_pml(j) * (hinc(j)-hinc(j-1))
     einc(j) = einc(j)  - tinj1d%ce2(j)*tinj1d%eaccu_pml(j)
  end do

  !boucle sur toutes les surfaces de Huygens

  do n=1,6 

    if (.not.t_opl%flagsurface(n)) cycle
    ph  = -1; pe = 0
    if (MOD(n,2) == 0) then    
      ph = tinj1d%p2-tinj1d%p1
      pe = ph
    end if 

    select case(n)
    case(1,2) !plan xoyinf et sup
	! bornes
       k  = t_opl%taille(4+n)
       !=== composante Ex ===
       if (axe /= AXE_Y) call calcul_axial_champ_surface( &
                        t_ch%ex(imin:imax,jmin:jmax,k), &
                        hinc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cvh_e1(tinj1d%p1-1:tinj1d%p2,n), &
			imax-1-imin,jmax-jmin,pe,ph,axe)
       !=== composante Ey ===
       if (axe /= AXE_X) call calcul_axial_champ_surface( &
                        t_ch%ey(imin:imax,jmin:jmax,k), &
                        hinc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cvh_e2(tinj1d%p1-1:tinj1d%p2,n), &
			imax-imin,jmax-jmin-1,pe,ph,axe)

    case(3,4) !plan yozinf et sup
	! bornes
       i    = t_opl%taille(n-2)	
       !=== composante Ey ===
       if (axe /= AXE_Z) call calcul_axial_champ_surface( &
                        t_ch%ey(i,jmin:jmax,kmin:kmax), &
                        hinc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cvh_e1(tinj1d%p1-1:tinj1d%p2,n), &
			jmax-1-jmin,kmax-kmin,pe,ph,PYZX(axe))

       !=== composante Ez ===
       if (axe /= AXE_Y) call calcul_axial_champ_surface( &
                        t_ch%ez(i,jmin:jmax,kmin:kmax), &
                        hinc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cvh_e2(tinj1d%p1-1:tinj1d%p2,n), &
			jmax-jmin,kmax-kmin-1,pe,ph,PYZX(axe))
       
    case(5,6) !plan zoxinf et sup
       j    = t_opl%taille(n-2)

       !=== composante Ez ===
       if (axe /= AXE_X) call calcul_axial_champ_surface( &
                        t_ch%ez(imin:imax,j,kmin:kmax), &
                        hinc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cvh_e1(tinj1d%p1-1:tinj1d%p2,n), &
			imax-imin,kmax-1-kmin,pe,ph,PXZY(axe))

       !=== composante Ex ===
       if (axe /= AXE_Z) call calcul_axial_champ_surface( &
                        t_ch%ex(imin:imax,j,kmin:kmax), &
                        hinc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cvh_e2(tinj1d%p1-1:tinj1d%p2,n), &
			imax-imin-1,kmax-kmin,pe,ph,PXZY(axe))

    end select
  end do  ! boucle sur les surfaces
end do !boucle sur les injecteurs

end subroutine injecteur_axial_calcul_champe

!===========================================================	
! subroutine injecteur_axial_calcul_champh
! Algorithme d'insertion du champ E incident sur les surfaces 
! de Huygens.  Mise à jour du champ H 3D
!===========================================================	
subroutine injecteur_axial_calcul_champh(itertemporel,t_opl,t_ch)
implicit none
!=== Déclaration des arguments ===
integer,intent(in)          :: itertemporel
type(st_onde_plane), intent(inout) :: t_opl
type(st_champ), intent(inout) :: t_ch

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer     :: i,j,k,l,n,pe,ph
integer     :: imin,jmin,kmin,imax,jmax,kmax
integer     :: axe
real,dimension(:), pointer        :: einc, hinc, ce1, ce2 

!===   Début des traitements   ===

axe  = t_opl%dir_injecteur_axial
imin = t_opl%taille(1)
imax = t_opl%taille(2)
jmin = t_opl%taille(3)
jmax = t_opl%taille(4)
kmin = t_opl%taille(5)
kmax = t_opl%taille(6)

!boucle sur tous les injecteurs
do l=1,t_opl%nbinjecteur

  tinj1d=>t_opl%tinj1d(l)
  
  
  einc=>tinj1d%einc
  hinc=>tinj1d%hinc
  
!  if (itertemporel == 2) then
!    write(100,'("debutregle,finpml1 = ",2I4)') tinj1d%debutregle,tinj1d%finpml1
!    write(100,'("debutpml2,finregle = ",2I4)') tinj1d%debutpml2,tinj1d%finregle
!    do j=tinj1d%debutregle,tinj1d%finregle
!      write(100,'("n°cel,ch1_pml,ch2_pml,ch2 = ",I4,3ES14.5e2)') j,tinj1d%ch1_pml(j),tinj1d%ch2_pml(j),tinj1d%ch2(j)
!    end do
!  end if
  
  !calcul de Hinc sur la règle 1d
  do j=tinj1d%debutregle,tinj1d%finregle-1
     hinc(j) = tinj1d%ch1(j) * hinc(j) - tinj1d%ch2(j)*(einc(j+1)-einc(j))
  end do

  ! mise à jour dans les pmls inf et sup
  do j = tinj1d%debutregle, tinj1d%finpml1-1
     tinj1d%haccu_pml(j) = tinj1d%ch1_pml(j) * tinj1d%haccu_pml(j) + tinj1d%ch2_pml(j) * (einc(j+1)-einc(j))
     hinc(j) = hinc(j)  - tinj1d%ch2(j)*tinj1d%haccu_pml(j)
  end do
  do j = tinj1d%debutpml2, tinj1d%finregle-1
     tinj1d%haccu_pml(j) = tinj1d%ch1_pml(j) * tinj1d%haccu_pml(j) + tinj1d%ch2_pml(j) * (einc(j+1)-einc(j))
     hinc(j) = hinc(j)  - tinj1d%ch2(j)*tinj1d%haccu_pml(j)
  end do
  

  !boucle sur toutes les surfaces de Huygens

  do n=1,6 
    if (.not.t_opl%flagsurface(n)) cycle
    ph  = -1; pe = 0
    if (MOD(n,2) == 0) then    
      ph = tinj1d%p2-tinj1d%p1
      pe = ph
    end if 

    select case(n)
    case(1,2) !plan xoyinf et sup
	! bornes
       k  = t_opl%taille(4+n)
       if (n==1) k = k-1
       
       !=== composante Hx ===
       if (axe /= AXE_Y) call calcul_axial_champ_surface( &
                        t_ch%hx(imin:imax,jmin:jmax,k), &
                        einc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cve_h1(tinj1d%p1-1:tinj1d%p2,n), &
			imax-imin,jmax-jmin-1,ph,pe,axe)
       !=== composante Hy ===
       if (axe /= AXE_X) call calcul_axial_champ_surface( &
                        t_ch%hy(imin:imax,jmin:jmax,k), &
                        einc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cve_h2(tinj1d%p1-1:tinj1d%p2,n), &
			imax-imin-1,jmax-jmin,ph,pe,axe)

    case(3,4) !plan yozinf et sup
	! bornes
       i    = t_opl%taille(n-2)	
       if (n==3) i = i-1

       !=== composante Hy ===
       if (axe /= AXE_Z) call calcul_axial_champ_surface( &
                        t_ch%hy(i,jmin:jmax,kmin:kmax), &
                        einc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cve_h1(tinj1d%p1-1:tinj1d%p2,n), &
			jmax-jmin,kmax-kmin-1,ph,pe,PYZX(axe))

       !=== composante Hz ===
       if (axe /= AXE_Y) call calcul_axial_champ_surface( &
                        t_ch%hz(i,jmin:jmax,kmin:kmax), &
                        einc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cve_h2(tinj1d%p1-1:tinj1d%p2,n), &
			jmax-jmin-1,kmax-kmin,ph,pe,PYZX(axe))
       
    case(5,6) !plan zoxinf et sup
       j    = t_opl%taille(n-2)
       if (n==5) j = j-1

       !=== composante Hz ===
       if (axe /= AXE_X) call calcul_axial_champ_surface( &
                        t_ch%hz(imin:imax,j,kmin:kmax), &
                        einc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cve_h1(tinj1d%p1-1:tinj1d%p2,n), &
			imax-imin-1,kmax-kmin,ph,pe,PXZY(axe))

       !=== composante Hx ===
       if (axe /= AXE_Z) call calcul_axial_champ_surface( &
                        t_ch%hx(imin:imax,j,kmin:kmax), &
                        einc(tinj1d%p1-1:tinj1d%p2), &
	       tinj1d%cve_h2(tinj1d%p1-1:tinj1d%p2,n), &
			imax-imin,kmax-kmin-1,ph,pe,PXZY(axe))

    end select
  end do  ! boucle sur les surfaces
end do !boucle sur les injecteurs

end subroutine injecteur_axial_calcul_champh

!===========================================================	
! Subroutine onde_plane_calcul_champh
!  Calcul pendant les itérations temporelles
! Mise à jour du champ électrique
!===========================================================	
subroutine onde_plane_calcul_champh(itertemporel,t_opl,t_ch)
implicit none
!=== Déclaration des arguments ===
integer,intent(in)          :: itertemporel
type(st_onde_plane), intent(inout) :: t_opl
type(st_champ), intent(inout) :: t_ch


!===  Déclaration des variables  ===

!===   Début des traitements   ===
if (t_opl%flaginjecteuraxial) then
  call injecteur_axial_calcul_champh(itertemporel,t_opl,t_ch)
else
  call injecteur_oblique_calcul_champh(itertemporel,t_opl,t_ch)
end if

				
end subroutine onde_plane_calcul_champh

!===========================================================	
! Subroutine onde_plane_calcul_champe
!  Calcul pendant les itérations temporelles
! Mise à jour du champ électrique
!===========================================================	
subroutine onde_plane_calcul_champe(itertemporel,t_opl,t_ch)
implicit none
!=== Déclaration des arguments ===
integer,intent(in)          :: itertemporel
type(st_onde_plane), intent(inout) :: t_opl
type(st_champ), intent(inout) :: t_ch
!===  Déclaration des variables  ===

!===   Début des traitements   ===
if (t_opl%flaginjecteuraxial) then
  call injecteur_axial_calcul_champe(itertemporel,t_opl,t_ch)
else
  call injecteur_oblique_calcul_champe(itertemporel,t_opl,t_ch)
end if
    
end subroutine onde_plane_calcul_champe

!===========================================================	
! Subroutine onde_plane_calcul_ptref
!  Calcul pendant les itérations temporelles
! Calcul du point de référence et stockage
!===========================================================	
subroutine onde_plane_calcul_ptref(t_opl)
implicit none
!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout) :: t_opl

!===  Déclaration des variables  ===
integer  :: i
real :: einc,hinc,etheta,ephi,htheta,hphi
type (st_ptref_onde_plane),pointer :: tpt

!===   Début des traitements   ===

if (t_opl%flag_ptref) then
  tpt =>t_opl%t_ptref
  !----------------------- RÉSULTATS EN TEMPOREL ------------------------------
  if (t_opl%t_ptref%flag_t) then
    if (tpt%tf_t%flag_calcul) then
      etheta = 0.0; ephi = 0.0
      htheta = 0.0; hphi = 0.0
      
      do i=1,t_opl%nbinjecteur_tot
        einc =  (1-tpt%p(1,i)) * tpt%einc(1) + tpt%p(1,i) *tpt%einc(2)
        hinc =  (1-tpt%p(2,i)) * tpt%hinc(1) + tpt%p(2,i) *tpt%hinc(2)
	etheta =  etheta + einc * cos(tpt%psi(i))
	ephi   =  ephi   + einc * sin(tpt%psi(i))
	htheta = -htheta + hinc * sin(tpt%psi(i))
	hphi   =  hphi   + hinc * cos(tpt%psi(i))
      end do
      write (tpt%unite,tpt%mantisse) tpt%tf_t%liste(tpt%tf_t%curseur), etheta,ephi,htheta,hphi
    end if
  end if
  !----------------------- RÉSULTATS EN FRÉQUENCE ------------------------------
  if (t_opl%t_ptref%flag_f) then
    if (tpt%tf_f%flag_calcul) then
      etheta = 0.0; ephi = 0.0
      htheta = 0.0; hphi = 0.0
      
      do i=1,t_opl%nbinjecteur_tot
        einc =  (1-tpt%p(1,i)) * tpt%einc(1) + tpt%p(1,i) *tpt%einc(2)
        hinc =  (1-tpt%p(2,i)) * tpt%hinc(1) + tpt%p(2,i) *tpt%hinc(2)
	etheta =  etheta + einc * cos(tpt%psi(i))
	ephi   =  ephi   + einc * sin(tpt%psi(i))
	htheta = -htheta + hinc * sin(tpt%psi(i))
	hphi   =  hphi   + hinc * cos(tpt%psi(i))
      end do
      tpt%ehinc_f(:,1) = tpt%ehinc_f(:,1) + etheta * tpt%tf_f%expi2pitf(:)
      tpt%ehinc_f(:,2) = tpt%ehinc_f(:,2) + ephi   * tpt%tf_f%expi2pitf(:)
      tpt%ehinc_f(:,3) = tpt%ehinc_f(:,3) + htheta * tpt%tf_f%expi2pitf(:)
      tpt%ehinc_f(:,4) = tpt%ehinc_f(:,4) + hphi   * tpt%tf_f%expi2pitf(:)
    end if
  end if
end if
    
end subroutine onde_plane_calcul_ptref


!===========================================================	
! Subroutine onde_plane_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine onde_plane_affichage()
implicit none
!=== Déclaration des arguments ===

!===   Début des traitements   ===
				
end subroutine onde_plane_affichage

!===========================================================	
! Subroutine onde_plane_postcalcul
!  Calcul après les itérations temporelles
!===========================================================	
subroutine onde_plane_impression(t_opl,tfs)
implicit none
!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout) :: t_opl
type(st_fichier_sortie), intent(in)   :: tfs


!===  Déclaration des variables  ===
integer  :: i, unite
type (st_ptref_onde_plane),pointer :: tpt
character(LEN=LEN_MOT_CLE) :: nomfichier

!===   Début des traitements   ===
if (t_opl%flag_ptref) then
  tpt =>t_opl%t_ptref
  !----------------------- RÉSULTATS EN TEMPOREL ------------------------------
  if (t_opl%t_ptref%flag_t) close(tpt%unite)
  
  !----------------------- RÉSULTATS EN FRÉQUENCE ------------------------------
  if (tpt%flag_f) then
    nomfichier = fichier_sortie_nom_freq(tfs,'pt_reference')
    tpt%unite = ouvre_en_ecriture(nomfichier)
    tpt%mantisse = '(' //'20'//trim(tfs%mantisse_reel)//')'
    
    call impression_ascii_complexe(tpt%unite,tpt%mantisse, REELLE_IMAGINAIRE, &
              tpt%tf_f%liste, tpt%ehinc_f,tpt%tf_f%n_liste,4)
    close(tpt%unite)    				    
  end if
end if
				
End Subroutine onde_plane_impression

!===========================================================	
! Subroutine onde_plane_deallocate
!  Libération de la mémoire allouée
!===========================================================	
subroutine onde_plane_deallocate(t_opl)
implicit none
!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout) :: t_opl

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer :: l,n

!===   Début des traitements   ===
if (t_opl%flaginjecteuraxial) then
  call injecteur_axial_deallocate(t_opl)
else
  call injecteur_oblique_deallocate(t_opl)
end if

deallocate(t_opl%tinj1d);			

if (t_opl%flag_ptref) then
  NULLIFY(t_opl%t_ptref%einc); NULLIFY(t_opl%t_ptref%hinc)
  deallocate(t_opl%t_ptref%psi) 
  deallocate(t_opl%t_ptref%p) 
  
  if (t_opl%t_ptref%flag_f) deallocate(t_opl%t_ptref%ehinc_f) 
end if


end subroutine onde_plane_deallocate

!===========================================================	
! Subroutine injecteur_oblique_deallocate
!  Libération de la mémoire allouée
!===========================================================	
subroutine injecteur_oblique_deallocate(t_opl)
implicit none
!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout) :: t_opl

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer :: l,n

!===   Début des traitements   ===
!do n=1,6 
!  if (.not.t_opl%flagsurface(n)) cycle
!  deallocate(t_opl%tshuyg(n)%e1inc)
!  deallocate(t_opl%tshuyg(n)%e2inc)
!  deallocate(t_opl%tshuyg(n)%h1inc)
!  deallocate(t_opl%tshuyg(n)%h2inc)
!end do

do l=1, t_opl%nbinjecteur_tot
  tinj1d => t_opl%tinj1d(l)
  !boucle sur les 6 surfaces de Huygens
  do n=1,6 
    if (.not.t_opl%flagsurface(n)) cycle
    select case(n)
    case(1,2) !plan xoyinf et sup
       deallocate(tinj1d%tdr(n)%tde1)
       deallocate(tinj1d%tdr(n)%tde2)
       deallocate(tinj1d%tdr(n)%tdh1)
       deallocate(tinj1d%tdr(n)%tdh2)
       deallocate(tinj1d%tdr(n)%tdie1)
       deallocate(tinj1d%tdr(n)%tdie2)
       deallocate(tinj1d%tdr(n)%tdih1)
       deallocate(tinj1d%tdr(n)%tdih2)
       

    case(3,4) !plan yozinf et sup
       deallocate(tinj1d%tdr(n)%tde1)
       deallocate(tinj1d%tdr(n)%tde2)
       deallocate(tinj1d%tdr(n)%tdh1)
       deallocate(tinj1d%tdr(n)%tdh2)
       deallocate(tinj1d%tdr(n)%tdie1)
       deallocate(tinj1d%tdr(n)%tdie2)
       deallocate(tinj1d%tdr(n)%tdih1)
       deallocate(tinj1d%tdr(n)%tdih2)
    case(5,6) !plan zoxinf et sup
       deallocate(tinj1d%tdr(n)%tde1)
       deallocate(tinj1d%tdr(n)%tde2)
       deallocate(tinj1d%tdr(n)%tdh1)
       deallocate(tinj1d%tdr(n)%tdh2)
       deallocate(tinj1d%tdr(n)%tdie1)
       deallocate(tinj1d%tdr(n)%tdie2)
       deallocate(tinj1d%tdr(n)%tdih1)
       deallocate(tinj1d%tdr(n)%tdih2)
    end select
  end do ! boucle sur les 6 surfaces de Huygens  

  deallocate(tinj1d%tdr)
  deallocate(tinj1d%tabgene)

  deallocate(tinj1d%einc);
  deallocate(tinj1d%hinc);
  deallocate(tinj1d%eaccu_pml);
  deallocate(tinj1d%haccu_pml);
  deallocate(tinj1d%ce1_pml );
  deallocate(tinj1d%ce2_pml );
  deallocate(tinj1d%ch1_pml );
  deallocate(tinj1d%ch2_pml );

end do ! boucle sur les injecteurs	
				
end subroutine injecteur_oblique_deallocate


!===========================================================	
! Subroutine injecteur_axial_deallocate
!  Libération de la mémoire allouée
!===========================================================	
subroutine injecteur_axial_deallocate(t_opl)
implicit none
!=== Déclaration des arguments ===
type(st_onde_plane), intent(inout) :: t_opl

!===  Déclaration des variables  ===
type(st_injecteurondeplane), pointer :: tinj1d
integer :: l,n

!===   Début des traitements   ===

do l=1, t_opl%nbinjecteur
  tinj1d => t_opl%tinj1d(l)
  !boucle sur les 6 surfaces de Huygens
  deallocate(tinj1d%tabgene)
  deallocate(tinj1d%einc);
  deallocate(tinj1d%hinc);
  deallocate(tinj1d%ce1 );
  deallocate(tinj1d%ce2 );
  deallocate(tinj1d%ch1 );
  deallocate(tinj1d%ch2 );

  deallocate(tinj1d%cvh_e1 );
  deallocate(tinj1d%cvh_e2 );
  deallocate(tinj1d%cve_h1 );
  deallocate(tinj1d%cve_h2 );

  deallocate(tinj1d%eaccu_pml);
  deallocate(tinj1d%haccu_pml);
  deallocate(tinj1d%ce1_pml );
  deallocate(tinj1d%ce2_pml );
  deallocate(tinj1d%ch1_pml );
  deallocate(tinj1d%ch2_pml );
end do ! boucle sur les injecteurs	
				
end subroutine injecteur_axial_deallocate
       
end module mfdtd_onde_plane
