!////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_resultat_champ.f90
!// Purpose:     Résultat local au volume sur l'une des 6 composantes de 
!//                champ
!// Author:      Christophe Guiffaut
!// Modified by: Christophe Guiffaut
!// Created  :     Décembre 2001
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Stockage de points d'observation, de surfaces et de  ==
! == volumes en champ                                     ==
! ==========================================================

module mfdtd_resultat_champ

use m_constantes
use m_es_identifiant, only : MANTISSE_REEL,MANTISSE_ENTIER,FORMAT_ASCII,REELLE_IMAGINAIRE

use m_math_operation, only : atan2d_cor
use m_entree, only : st_fichier, &
                                  lineread, &
				  ligne_precedente, &
				  multibackspace, &
				  numbackspace_set, &
				  lecture_stop, &
				  compteur_mot_cle, &
				  nb_mot_chaine, &
				  ctrl_nb_mot_chaine, &
				  unitefichier
				  
use m_sortie, only : st_fichier_sortie, &
		    fichier_sortie_lect_format_freq,&
                    impression_ascii_complexe, &
		    fichier_sortie_nom_temps, &
		    fichier_sortie_ecrit_format, &
		    ouvre_en_ecriture,&
		    fichier_sortie_nom_freq

use m_temps_frequence, only : st_temps_frequence, &
                              st_liste_temps_freq, &
			      liste_temps_freq_pointe, &
			      temps_freq_compte, &
			      identifie_deux_nom


use m_generateur, only : st_generateur
use m_sortie_normalise, only : st_sortie_normalise, &
                              sortie_normalise_initdefaut,&
			      sortie_normalise_lecture   ,&
			      sortie_normalise_ecriture ,&
			      sortie_normalise_compare, &
			      Normalise_Reponse_Freq

use m_commun_identifiant
use m_verif_data
use m_maillage

use mfdtd_champ, only : st_champ
use mfdtd_milieu_volume, only : st_milieu_volume, &
                                 cherche_epsilon_cellule, &
				 cherche_mu_cellule
use mfdtd_volume,         only : st_volume
use mfdtd_structure,      only : st_structure, &
				structure_surfacique_cellule,&
                                  structure_metal_cellule
implicit none

integer, parameter :: E_X   = 1
integer, parameter :: E_Y   = 2
integer, parameter :: E_Z   = 3
integer, parameter :: H_X   = 4
integer, parameter :: H_Y   = 5
integer, parameter :: H_Z   = 6
integer, parameter :: E_V   = 7
integer, parameter :: H_V   = 8
integer, dimension(3),parameter :: PC211=(/2,1,1/)
integer, dimension(3),parameter :: PC332=(/3,3,2/)

!=== Déclaration des types ==
type st_pointeur_reel
  real, pointer  :: eh
end type st_pointeur_reel

type st_pointeur_centrepoint
  ! pointeur
  real, dimension(:,:),pointer  :: Ec,Em,Ep
  real, dimension(:), pointer   :: Hc,H1m,H1p,H2m,H2p
  
  real                          :: pas  
  real                          :: eps_c,mu_c
  real                          :: eps_inf,eps_sup
  real, dimension(2)            :: mu1, mu2
  real, dimension(1:2,1:2)      :: pond  ! pondération
  ! tableau pour E
  real, dimension(:), pointer :: Ecentre, gradE1,gradE2,Einf,Esup
  ! tableau pour H
  real, dimension(:), pointer :: Hcentre, gradH,H1inf,H1sup,H2inf,H2sup
end type st_pointeur_centrepoint 


!---   STRUCTURE POUR LES POINTS D'OBSERVATION AVEC LES COMPOSANTES À LEURS
!                POSITIONS NATURELS DANS LA CELLULE DE YEE                    ---
type st_resultat_champ_point
  integer  :: ix,iy,iz       ! coordonnees du point de stockage
  integer  :: nbc            ! nombre de composantes
  integer  :: unite          ! unité du fichier
  integer  :: com_format_tf  ! type de formatage des données en fréquence
  logical  :: flag_eval_fin_simu ! capteur pour l'évaluation de la fin de la simulation
  type(st_pointeur_reel), dimension(1:8)  :: p ! tableau d'alias vers les composantes à stocker
  character(LEN=LEN_MOT_CLE), dimension(1:8) :: nom_eh
  integer,dimension(1:8) :: flag_compo
  complex, dimension(:,:),pointer :: exyz,hxyz
  
  character(LEN=LEN_MOT_CLE) :: nomfichier
  character(LEN=LEN_MOT_CLE) :: mantisse
  type(st_temps_frequence), pointer :: tf
  ! -- Tableaux pour le stockage des composantes  en frequence --
  complex, dimension(:,:), pointer :: eh_f

  ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
end type st_resultat_champ_point

!---   STRUCTURE POUR LES POINTS D'OBSERVATION AVEC LES COMPOSANTES À LA
!                POSITION SPÉCIFIIÉE PAR LES COORDONNÉES (x,y,z)           ---
type st_resultat_champ_centrepoint
  integer  :: normale
  integer  :: nbech
  REAL, dimension(3)     :: xyz,xyzcell   ! coordonnees du point de stockage
  REAL, dimension(3)     :: dxyz       ! décalage dans la cellule par rapport à son barycentre
  REAL, dimension(3)     :: dxdydz     ! Les trois pas spatiaux
  integer, dimension(3)  :: ixyz       ! coordonnees du point de stockage en nombre de cellule
  integer, dimension(1:8):: nu_eh
  logical, dimension(1:6):: flagstsurf_face
  integer  :: nbc            ! nombre de composantes
  integer  :: com_format_tf  ! type de formatage des données en fréquence
  logical  :: flag_eval_fin_simu ! capteur pour l'évaluation de la fin de la simulation
  type(st_pointeur_centrepoint), dimension(1:8)  :: p ! tableau d'alias vers les composantes à stocker
  character(LEN=LEN_MOT_CLE), dimension(1:8)     :: nom_eh
  type(st_pointeur_centrepoint), dimension(1:3)  :: pe,ph ! tableau d'alias vers les composantes à stocker
  integer,dimension(1:8) :: flag_compo
  character(LEN=LEN_MOT_CLE) :: nomfichier
  character(LEN=LEN_MOT_CLE) :: mantisse
  type(st_temps_frequence), pointer :: tf
  
  ! Normalisation en fréquence
  type(st_sortie_normalise) :: sn
end type st_resultat_champ_centrepoint

!------------------------------------------------------------------------------
type st_resultat_champ_ES ! données directes pour les entrées sorties
  character(LEN=LEN_MOT_CLE) :: nom,type_observ
  character(LEN=LEN_MOT_CLE) :: interv1,interv2  
  integer  :: nbc            ! nombre de composantes
  character(LEN=LEN_MOT_CLE), dimension(1:8) :: nom_eh
  integer   :: tf_commutateur
  logical  :: flag_eval_fin_simu ! capteur pour l'évaluation de la fin de la simulation  
  REAL, dimension(3)     :: xyz        ! coordonnees du point de stockage
  integer, dimension(3)  :: ixyz       ! coordonnees du point de stockage en nombre de cellule
  real   , dimension(3)  :: xyzcell       ! coordonnees du point de stockage en nombre de cellule
   ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
   
end type st_resultat_champ_ES

!------------------------------------------------------------------------------

!=== structure principale ===
type st_resultat_champ
  logical                    :: flag_data
  character(LEN=LEN_MOT_CLE) :: mantisse_entier, mantisse_reel
  real                       :: dt
  ! -- Structure de points de stockage en temps et en frequence --
  integer  ::  nb_pt_f, nb_pt_t  ! Nombre de points de stockage en temps et en frequence --
  type(st_resultat_champ_point), dimension(:), pointer        :: t_lpt_t, t_lpt_f  
  
  integer  ::  nb_ptcp_f, nb_ptcp_t 
  type(st_resultat_champ_centrepoint), dimension(:), pointer  :: lptcp_t, lptcp_f  
  !--  données directes pour les entrées sorties --
  integer :: n_res
  type(st_resultat_champ_ES), pointer, dimension(:) :: l_res
  
  real, dimension(:,:,:) ,pointer :: ex,ey,ez,hx,hy,hz
end type st_resultat_champ
 
! Pas de variables globales...

contains
!===========================================================	
! Subroutine resultat_champ_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine resultat_champ_initdefaut(tic)

!=== Déclaration des arguments ===
type(st_resultat_champ), intent(inout) :: tic

!===  Déclaration des variables  ===

!===   Début des traitements   ===
tic%flag_data = .false.
tic%nb_pt_f = 0; tic%nb_pt_t = 0
tic%mantisse_reel = MANTISSE_REEL
tic%mantisse_entier = MANTISSE_ENTIER

tic%nb_ptcp_f = 0; tic%nb_ptcp_t = 0 
nullify(tic%t_lpt_t)
nullify(tic%t_lpt_f)
nullify(tic%lptcp_t)
nullify(tic%lptcp_f)

end subroutine resultat_champ_initdefaut

! ==========================================================
! == Subroutine resultat_champ_es_initdefaut     ==
! ==========================================================
subroutine resultat_champ_es_initdefaut(rf)

type(st_resultat_champ_ES), intent(inout) :: rf

rf%nom         = ''
rf%type_observ = ''
rf%interv1 = ''
rf%interv2 = ''
rf%tf_commutateur = 0;
rf%nbc = 0
rf%nom_eh(:) = ''
rf%xyz = 0.0
rf%ixyz=0 
rf%flag_eval_fin_simu = .false.
call sortie_normalise_initdefaut(rf%sn)

end subroutine resultat_champ_es_initdefaut

! ==========================================================
! == Subroutine resultat_champ_unpoint_copy     ==
! ==========================================================
subroutine resultat_champ_unpoint_copy(cpin,cpout)
!=== Déclaration des arguments ===
type(st_resultat_champ_point), intent(in)  :: cpin
type(st_resultat_champ_point), intent(out) :: cpout
!===  Déclaration des variables  ===

!===   Début des traitements   ===
cpout%ix=cpin%ix;cpout%iy=cpin%iy;cpout%iz=cpin%iz
cpout%nbc=cpin%nbc
cpout%unite=cpin%unite
cpout%com_format_tf=cpin%com_format_tf
cpout%flag_eval_fin_simu=cpin%flag_eval_fin_simu
cpout%nom_eh=cpin%nom_eh
cpout%flag_compo=cpin%flag_compo
cpout%nomfichier=cpin%nomfichier
cpout%mantisse=cpin%mantisse
cpout%tf=>cpin%tf
cpout%sn=cpin%sn
!nullify(cpout%p)
nullify(cpout%exyz);nullify(cpout%hxyz);
nullify(cpout%eh_f)

end subroutine resultat_champ_unpoint_copy

! ==========================================================
! == Subroutine resultat_champ_unpointR_copy     ==
! ==========================================================
subroutine resultat_champ_unpointR_copy(cpin,cpout)
!=== Déclaration des arguments ===
type(st_resultat_champ_centrepoint), intent(in)  :: cpin
type(st_resultat_champ_centrepoint), intent(out) :: cpout
!===  Déclaration des variables  ===

!===   Début des traitements   ===
cpout%normale=cpin%normale
cpout%nbech=cpin%nbech
cpout%xyz=cpin%xyz
cpout%xyzcell=cpin%xyzcell
cpout%dxyz=cpin%dxyz
cpout%dxdydz=cpin%dxdydz
cpout%ixyz=cpin%ixyz
cpout%nu_eh=cpin%nu_eh
cpout%flagstsurf_face=cpin%flagstsurf_face
cpout%nbc=cpin%nbc
cpout%com_format_tf=cpin%com_format_tf
cpout%flag_eval_fin_simu=cpin%flag_eval_fin_simu
cpout%nom_eh=cpin%nom_eh
cpout%flag_compo=cpin%flag_compo
cpout%nomfichier=cpin%nomfichier
cpout%mantisse=cpin%mantisse
cpout%tf=>cpin%tf
cpout%sn=cpin%sn
!nullify(cpout%p)

end subroutine resultat_champ_unpointR_copy
  

!===========================================================	
! Subroutine resultat_champ_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
subroutine resultat_champ_lecture(f,tic,ltf,tm,borne_volume)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_resultat_champ), intent(inout) :: tic
type(st_liste_temps_freq), intent(inout) :: ltf
type(st_maillage), intent(in) :: tm
integer,dimension(6)  :: borne_volume ! (y compris les pmls)

!===  Déclaration des variables  ===
integer  :: status
integer  :: i, i_pt_t, i_pt_f, i_pcpt_t,i_pcpt_f
integer  :: ires
integer  :: nu
integer  :: nbmot, nbcomposante, com_format_tf
integer, dimension(1:3)  ::entier3
real, dimension(1:3)     ::reel3,xyzcell
logical  :: flag_f, flag_t
logical  :: flag_reel
logical  :: flag_eval_fin_simu
character(LEN=LEN_MOT_CLE) :: corbeille, nom, nom1, nom2, premiermot,ch
character(LEN=LEN_MOT_CLE) :: nom_t, nom_f
character(LEN=LEN_MOT_CLE), dimension(1:8) :: nom_eh
type(st_resultat_champ_point), pointer :: tcp
type(st_resultat_champ_centrepoint), pointer :: tcpcp
type(st_resultat_champ_ES),pointer :: pres

! Normalisation en fréquence
type(st_sortie_normalise) :: sn
!===   Début des traitements   ===

!--- On compte le nombre de points de stockage en temps et en frequence---
i_pt_t = 0;
i_pt_f = 0;
i    =   compteur_mot_cle(f,'pcpt')
ires = i+compteur_mot_cle(f,'pt')
call temps_freq_compte(f,'pt',i_pt_t,i_pt_f)
i_pcpt_t = 0;
i_pcpt_f = 0;
call temps_freq_compte(f,'pcpt',i_pcpt_t,i_pcpt_f)

if (i_pt_t == 0 .and. i_pt_f == 0 .and. i_pcpt_t == 0 .and. i_pcpt_f == 0) then
  print '(/,t10,"Avertissement, Pas de points d''observation en champ de définis !!!")'
  tic%flag_data = .false.
  return
end if
tic%flag_data = .true.
! -- Allocation des structures en temps et en frequence... --
tic%nb_pt_t = i_pt_t
if (i_pt_t>0) allocate(tic%t_lpt_t(i_pt_t))
tic%nb_pt_f = i_pt_f
if (i_pt_f>0) allocate(tic%t_lpt_f(i_pt_f))
tic%nb_ptcp_t = i_pcpt_t
if (i_pcpt_t>0) allocate(tic%lptcp_t(i_pcpt_t))
tic%nb_ptcp_f = i_pcpt_f
if (i_pcpt_f>0) allocate(tic%lptcp_f(i_pcpt_f))

tic%n_res = ires
allocate(tic%l_res(tic%n_res))
do i=1,ires
  call resultat_champ_es_initdefaut(tic%l_res(i))
end do

i_pt_t   = 1;
i_pt_f   = 1;
i_pcpt_t = 1;
i_pcpt_f = 1;
com_format_tf = REELLE_IMAGINAIRE    
ires = 0
flag_reel = .false.
flag_eval_fin_simu = .false.

! Init normalisation
call sortie_normalise_initdefaut(sn)
! -- Lecture des donnees --
do
  premiermot = lineread(f)
  nom_eh = (/'ex','ey','ez','hx','hy','hz','  ','  '/);    
  !----------------------------------------------------------------------------
  if (premiermot(1:2) == 'pt') then 
    read(f%chaine,*) corbeille, nom1, nom2
    
    if (.not.identifie_deux_nom(nom1,nom2,flag_t,flag_f,nom_t,nom_f)) then
      print '(/,t10,"Nom du type de stockage erroné !!!")'
      call lecture_stop(f)
    end if
    
    ! -- Calcul du nombre de composantes de champ dans la chaine --
    status = ctrl_nb_mot_chaine(f,f%chaine,5,14)      
    nbmot = nb_mot_chaine(f%chaine) 
    reel3 = 0.0
    if (flag_t .and. flag_f) then
      nbcomposante = nbmot - 6;
      if (nbcomposante>8) then
        print '(" ERREUR, trop de composantes")'
	call lecture_stop(f)
      end if
      if (nbcomposante >0) then
        read(f%chaine,*) nom,nom1,nom2,nom_eh(1:nbcomposante), reel3(1:3)
      else
        read(f%chaine,*) nom,nom1,nom2, reel3(1:3)
      end if
    else
      nbcomposante = nbmot - 5;
      if (nbcomposante>8) then
        print '(" ERREUR, trop de composantes")'
	call lecture_stop(f)
      end if
      if (nbcomposante >0) then
        read(f%chaine,*) nom,nom1,nom_eh(1:nbcomposante), reel3(1:3)
      else
        read(f%chaine,*) nom,nom1, reel3(1:3)
      end if
    end if  
    
    if (nbcomposante == 0) nbcomposante = 6
    ! -- Controle l'existence des composantes --
    do i=1, nbcomposante
      if (ALL(LISTE_EH(:) /= nom_eh(i))) then
        print '(/,t10,"Nom de la composante de  champ erronée : ", A)', TRIM(nom_eh(i))
        call lecture_stop(f)
      end if
    end do
    
    ! Gestion du chgt d'origine dans le repère de calcul
    if (flag_reel) then
      entier3=maillage_Unif_ConvToCell3(tm,reel3)
    else
      entier3= origine_cellule3(tm,int(reel3))
    end if
    
    ! Vérification si point dans le volume (y compris les pmls)
    if (.not. verif_pt_ds_volume(nom,real(borne_volume),real(entier3))) then
      call lecture_stop(f)
    end if
    
    if (rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",3I6)',trim(nom),entier3
    
    ! Conversion en nbre de cellules, origine de calcul déjà converti au dessus
    xyzcell = maillage_Unif_ConvToCellR3(tm,reel3,.false.)
    
    !--- Structure en temps ---
    if (flag_t) then
      tcp => tic%t_lpt_t(i_pt_t)
      ! -- Recupere les pointeurs sur les structures temps-frequence --
      tcp%tf => liste_temps_freq_pointe(ltf,nom_t)
      if (.not.associated(tcp%tf)) then
        print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
        call lecture_stop(f)      
      end if
    
      ! -- Type de composantes a stocker, coordonnees --
      tcp%nomfichier = nom
      tcp%flag_eval_fin_simu = flag_eval_fin_simu
      tcp%nom_eh = nom_eh
      tcp%nbc = nbcomposante
      tcp%ix = entier3(1);
      tcp%iy = entier3(2);
      tcp%iz = entier3(3);      
      i_pt_t = i_pt_t + 1
    end if
    
    ! -- Structure en frequence --
    if (flag_f) then
      tcp => tic%t_lpt_f(i_pt_f)
      tcp%tf => liste_temps_freq_pointe(ltf,nom_f)
      if (.not.associated(tcp%tf)) then
        print '(/,t10,"Nom du type de stockage en frequence erroné !!!")'
        call lecture_stop(f)      
      end if
      ! -- Type de composantes a stocker, coordonnees --
      tcp%nomfichier = nom
      tcp%flag_eval_fin_simu = flag_eval_fin_simu
      tcp%nbc = nbcomposante
      tcp%nom_eh = nom_eh
      tcp%ix = entier3(1);
      tcp%iy = entier3(2);
      tcp%iz = entier3(3);      
      tcp%com_format_tf = com_format_tf
      
      ! Information pour la normalisation ou pas
      tcp%sn = sn;
      
      i_pt_f = i_pt_f + 1
    end if        

  !----------------------------------------------------------------------------
  else if (premiermot(1:4) == 'pcpt') then 
    read(f%chaine,*) corbeille, nom1, nom2
    
    if (.not.identifie_deux_nom(nom1,nom2,flag_t,flag_f,nom_t,nom_f)) then
      print '(/,t10,"Nom du type de stockage erroné !!!")'
      call lecture_stop(f)
    end if
    
    ! -- Calcul du nombre de composantes de champ dans la chaine --
    status = ctrl_nb_mot_chaine(f,f%chaine,6,15)      
    nbmot = nb_mot_chaine(f%chaine) 
    if (flag_t .and. flag_f) then
      nbcomposante = nbmot - 7;
      if (nbcomposante>8) then
        print '(" ERREUR, trop de composantes")'
	call lecture_stop(f)
      else if (nbcomposante >0) then
        read(f%chaine,*) nom,nom1,nom2,nom_eh(1:nbcomposante), reel3(1:3),ch
      else
        read(f%chaine,*) nom,nom1,nom2, reel3(1:3),ch
      end if
    else
      nbcomposante = nbmot - 6;
      if (nbcomposante>8) then
        print '(" ERREUR, trop de composantes")'
	call lecture_stop(f)
      else if (nbcomposante >0) then
        read(f%chaine,*) nom,nom1,nom_eh(1:nbcomposante), reel3(1:3),ch
      else
        read(f%chaine,*) nom,nom1, reel3(1:3),ch
      end if
    end if 
    
    ! Gestion du chgt d'origine dans le repère de calcul
    if (flag_reel) then ! on garde les coordonnées réelles
      reel3= origine_reelle_abs3(tm,reel3)
    else
      reel3 = maillage_Unif_RcellConvToReal3(tm,reel3)
    end if
    
    ! Conversion en nbre de cellules, origine de calcul déjà converti au dessus
    xyzcell = maillage_Unif_ConvToCellR3(tm,reel3,.false.)
    ! Vérification si point dans le volume (y compris les pmls)
    if (.not. verif_pt_ds_volume(nom,real(borne_volume),xyzcell)) then
      call lecture_stop(f)
    end if


    if (rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords réelles ds le repère de calcul = ",3ES13.3E2)',&
               trim(nom),reel3
      print '(t10,A," : Coord cell     ds le repère de calcul =",3G15.4)',trim(nom),xyzcell

    if (nbcomposante == 0) nbcomposante = 6
    ! -- Controle l'existence des composantes --
    do i=1, nbcomposante
      if (ALL(LISTE_EH(:) /= nom_eh(i))) then
        print '(/,t10,"Nom de la composante de  champ erronée : ", A)', TRIM(nom_eh(i))
        call lecture_stop(f)
      end if
    end do
    
    ! traitement de la normale pour le point d'obervation :
    select case (ch)
      case ('cell')
        nu = 0
      case ('zinf')
        nu = 5
      case ('zsup') 
        nu = 6
      case ('xinf') 
        nu = 1
      case ('xsup') 
        nu = 2
      case ('yinf') 
        nu = 3
      case ('ysup') 
        nu = 4
      case default
	print *,'ERREUR, Manque une indication sur le point d''observation (cell,xinf,xsup...) !!! ',f%nombloc
	call lecture_stop(f)
    end select
    
    !--- Structure en temps ---
    if (flag_t) then
      tcpcp => tic%lptcp_t(i_pcpt_t)
      ! -- Recupere les pointeurs sur les structures temps-frequence --
      tcpcp%tf => liste_temps_freq_pointe(ltf,nom_t)
      if (.not.associated(tcpcp%tf)) then
        print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
        call lecture_stop(f)      
      end if
    
      ! -- Type de composantes a stocker, coordonnees --
      tcpcp%nomfichier = nom
      tcpcp%flag_eval_fin_simu = flag_eval_fin_simu
      tcpcp%normale    = nu
      tcpcp%nom_eh     = nom_eh
      tcpcp%nbc        = nbcomposante
      tcpcp%xyz(1:3)   = reel3(1:3);
      tcpcp%xyzcell       = xyzcell
      
      i_pcpt_t = i_pcpt_t + 1
    end if
    
    ! -- Structure en fréquence --
    if (flag_f) then
      tcpcp => tic%lptcp_f(i_pcpt_f)
      tcpcp%tf => liste_temps_freq_pointe(ltf,nom_f)
      if (.not.associated(tcpcp%tf)) then
        print '(/,t10,"Nom du type de stockage en frequence erroné !!!")'
        call lecture_stop(f)      
      end if
      ! -- Type de composantes a stocker, coordonnees --
      tcpcp%nomfichier = nom
      tcpcp%flag_eval_fin_simu = flag_eval_fin_simu
      tcpcp%normale    = nu
      tcpcp%nbc        = nbcomposante
      tcpcp%nom_eh     = nom_eh
      tcpcp%xyz(1:3)   = reel3(1:3)
      tcpcp%xyzcell       = xyzcell
      
      tcpcp%com_format_tf = com_format_tf
            ! Information pour la normalisation ou pas
      tcpcp%sn = sn;

      i_pcpt_f = i_pcpt_f + 1
    end if        

  !----------------------------------------------------------------------------
  else if (premiermot==ID_COORD_REEL) then
    flag_reel = .true.
  else if (premiermot==ID_COORD_CELL) then
    flag_reel = .false.
  !----------------------------------------------------------------------------
  else if (premiermot==ID_EVAL_TMAX) then
    flag_eval_fin_simu = .true.
  else if (premiermot==ID_FIN_EVAL_TMAX) then
    flag_eval_fin_simu = .false.
  !----------------------------------------------------------------------------
  else if (sortie_normalise_lecture(f,premiermot,sn)) then
    cycle
  !----------------------------------------------------------------------------
  else if (f%nomfinbloc == premiermot) then
    exit ! -- Fin du bloc --
  else if (.not.fichier_sortie_lect_format_freq(premiermot,com_format_tf)) then
    print '(/,t10,"Mot clé non reconnu  :",A )', TRIM(premiermot)
    call lecture_stop(f)
    
  end if 
  
  if ((premiermot(1:2) == 'pt') .or.(premiermot(1:4) == 'pcpt')) then
      ! Remplissage structure entrée/sortie
    ires=ires+1
    pres=>tic%l_res(ires)
    pres%nom  = nom
    pres%interv1 = nom1
    if (flag_t .and. flag_f) pres%interv2 = nom2
    pres%tf_commutateur = com_format_tf;
    pres%flag_eval_fin_simu = flag_eval_fin_simu
    pres%nbc = nbcomposante
    pres%nom_eh(1:nbcomposante) = nom_eh(1:nbcomposante)
    pres%ixyz=entier3(1:3)
    pres%sn  = sn
    if (premiermot(1:2) == 'pt') then
      pres%ixyz=entier3(1:3)
    else
      pres%xyz=reel3(1:3)
      pres%xyzcell=xyzcell
      pres%type_observ = ch
    end if
  end if
  
enddo

end subroutine resultat_champ_lecture


! ==========================================================
! == Subroutine resultat_champ_ecriture                   ==
! ==========================================================
subroutine resultat_champ_ecriture(f,tc,tm)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_resultat_champ), intent(inout) :: tc
type(st_maillage), intent(in) :: tm

integer  :: i,j,commutateur
logical  :: flag_eval_fin_simu
integer  :: entier3(3)
real     :: coord3(3)
character(LEN=LEN_MOT_CLE) :: ch1
type(st_resultat_champ_ES),pointer :: pres
type(st_sortie_normalise) :: sn


commutateur = REELLE_IMAGINAIRE
flag_eval_fin_simu = .false.
call sortie_normalise_initdefaut(sn)
!write(f%unite,'(T3,A, " # Uniquement pour les identifiants pcpt...")') ID_COORD_REEL

do i=1,tc%n_res
  pres =>tc%l_res(i)
  if (.not.sortie_normalise_compare(sn,pres%sn)) then
    call sortie_normalise_ecriture(f,pres%sn)
    sn = pres%sn
  end if
  
  if (flag_eval_fin_simu.neqv.pres%flag_eval_fin_simu) then
    flag_eval_fin_simu = pres%flag_eval_fin_simu
    if (flag_eval_fin_simu) then
      write(f%unite,'(T3,A)') ID_EVAL_TMAX
    else
      write(f%unite,'(T3,A)') ID_FIN_EVAL_TMAX
    end if
  end if

  if (commutateur/=pres%tf_commutateur) then
    call fichier_sortie_ecrit_format(f,pres%tf_commutateur)
    commutateur = pres%tf_commutateur
  end if

  ch1 = ''
  do j=1,pres%nbc
    ch1 = trim(ch1)//' '//TRIM(pres%nom_eh(j))
  end do
  if (pres%nom(1:2) == 'pt') then
      entier3= origine_cellule3(tm,pres%ixyz(1:3),.true.)
    
    if (pres%interv2 /='') then
      write(f%unite,'(T3,A,3x,A,3x,A,3x,A,3x,3I5)') trim(pres%nom),trim(pres%interv1),&
                                             trim(pres%interv2),trim(ch1),entier3
    else
      write(f%unite,'(T3,A,3x,A,3x,A,3x,3I5)') trim(pres%nom),trim(pres%interv1),&
                                             trim(ch1),entier3
    end if
  else if (pres%nom(1:4) == 'pcpt') then
    !coord3= origine_reelle_abs3(tm,pres%xyz,.true.)
    coord3 = origine_cellule3(tm,pres%xyzcell,.true.)
    if (pres%interv2 /='') then
      write(f%unite,'(T3,A,3x,A,3x,A,3x,A,3x,3G14.6,3x,A)') trim(pres%nom),trim(pres%interv1),&
			    trim(pres%interv2),trim(ch1),coord3,TRIM(pres%type_observ)
    else
      write(f%unite,'(T3,A,3x,A,3x,A,3x,3G14.6,3x,A)') trim(pres%nom),trim(pres%interv1),&
                                             trim(ch1),coord3,trim(pres%type_observ)
    end if
  end if
end do

end subroutine resultat_champ_ecriture


!===========================================================	
! Subroutine resultat_champ_init
! Initialisation des variables, lorsque qu'il n'y a pas de lecture du fichier
! d'entrée
!===========================================================	
subroutine resultat_champ_init(tic)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_champ), intent(inout) :: tic

!===  Déclaration des variables  ===

!===   Début des traitements   ===

				
end subroutine resultat_champ_init
	

!===========================================================	
! Subroutine resultat_champ_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
subroutine resultat_champ_precalcul(tic,tc,v,s,tfs,tm)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_champ), intent(inout) :: tic
type(st_champ), intent(in)    :: tc
type(st_volume), intent(in)   :: v
type(st_structure), intent(in)   :: s
type(st_fichier_sortie), intent(in)   :: tfs
type(st_milieu_volume), intent(in) :: tm

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nomfichier
character(LEN=2) :: c
integer  :: i,j,k,n
real     :: ixyz(3)
type(st_resultat_champ_point), pointer :: tcp
type(st_resultat_champ_centrepoint), pointer :: tcpcp

!===   Début des traitements   ===

! -- Boucle sur tous les points de stockage --
tic%mantisse_reel = tfs%mantisse_reel
tic%mantisse_entier = tfs%mantisse_entier
tic%dt = v%dt
tic%ex=>tc%ex;tic%ey=>tc%ey;tic%ez=>tc%ez
tic%hx=>tc%hx;tic%hy=>tc%hy;tic%hz=>tc%hz


! -- Stockage en temps --
do i = 1,tic%nb_pt_t
  tcp => tic%t_lpt_t(i)
  ! -- Creation des alias sur les composantes de champ --
  call resultat_champ_alias(tcp,tc)
  
  ! -- Creation du format d'ecriture en temps --
  write(c,'(I1)') tcp%nbc+1  
  tcp%mantisse = '(' //c(1:1)//tic%mantisse_reel(1:LEN_TRIM(tic%mantisse_reel))//')'
  
  ! -- ouverture du fichier si format ascii --
  if (tfs%format_fichier == FORMAT_ASCII) then   
    nomfichier = fichier_sortie_nom_temps(tfs,tcp%nomfichier)
    tcp%unite = ouvre_en_ecriture(nomfichier)
  end if
end do				

! -- Stockage en frequence --
do i = 1,tic%nb_pt_f
  tcp => tic%t_lpt_f(i)
  ! -- Creation des alias sur les composantes de champ --
  call resultat_champ_alias(tcp,tc)
  
  ! -- Creation du format d'ecriture en frequence --
  write(c,'(I2)') (tcp%nbc*2+1)  
  tcp%mantisse = '(' //c(1:LEN_TRIM(c))//tic%mantisse_reel &
                 (1:LEN_TRIM(tic%mantisse_reel))//')'

  ! -- Stockage en frequence --
  allocate(tcp%eh_f(1:tcp%tf%n_liste,1:tcp%nbc))          
  tcp%eh_f = 0.0          
  
end do				

! -- Stockage en temps --
!print '(T5,"--> Stockage en temps pour les points d''observation localisés")'
do i = 1,tic%nb_ptcp_t
  tcpcp => tic%lptcp_t(i)
  if (rang==RANG_MAITRE_AFF) print '(T10," Point n°",I4," Nom= ",A)',i,TRIM(tcpcp%nomfichier)
  ! alias vers les champs
  call resultat_champ_cp_precalcul(tcpcp,v,s,tm,0)
  call resultat_champ_alias_cp(tcpcp,tc)  
end do
! -- Stockage en fréquence --
!print '(T5,"--> Stockage en fréquence pour les points d''observation localisés")'
do i = 1,tic%nb_ptcp_f
  tcpcp => tic%lptcp_f(i)
  if (rang==RANG_MAITRE_AFF) print '(T10," Point n°",I4," Nom= ",A)',i,TRIM(tcpcp%nomfichier)
  ! alias vers les champs
  call resultat_champ_cp_precalcul(tcpcp,v,s,tm,1)
  call resultat_champ_alias_cp(tcpcp,tc)  
end do

end subroutine resultat_champ_precalcul


!===========================================================	
! Subroutine resultat_champ_alias
! 
!===========================================================	
subroutine resultat_champ_alias(tcp,tc)

implicit none

!=== Declaration des arguments ===
type(st_resultat_champ_point), intent(inout) :: tcp
type(st_champ), intent(in)   :: tc

!=== Declaration des variables ===
integer :: j, ix, iy, iz

!===   Debut des traitements   ===
ix = tcp%ix
iy = tcp%iy
iz = tcp%iz
tcp%flag_compo = 0

do j = 1,tcp%nbc
  ! -- Tableau d'alias vers les composantes a stocker --
  if (tcp%nom_eh(j) == LISTE_EH(1)) then
    tcp%p(j)%eh => tc%ex(ix,iy,iz) 
    tcp%flag_compo(j) = E_X    
  else if (tcp%nom_eh(j) == LISTE_EH(2)) then
    tcp%p(j)%eh => tc%ey(ix,iy,iz)      
    tcp%flag_compo(j) = E_Y
  else if (tcp%nom_eh(j) == LISTE_EH(3)) then
    tcp%p(j)%eh => tc%ez(ix,iy,iz)      
    tcp%flag_compo(j) = E_Z
  else if (tcp%nom_eh(j) == LISTE_EH(4)) then
    tcp%p(j)%eh => tc%hx(ix,iy,iz)      
    tcp%flag_compo(j) = H_X
  else if (tcp%nom_eh(j) == LISTE_EH(5)) then
    tcp%p(j)%eh => tc%hy(ix,iy,iz)      
    tcp%flag_compo(j) = H_Y
  else if (tcp%nom_eh(j) == LISTE_EH(6)) then
    tcp%p(j)%eh => tc%hz(ix,iy,iz)
    tcp%flag_compo(j) = H_Z
  else if (tcp%nom_eh(j) == LISTE_EH(7)) then
    tcp%flag_compo(j) = E_V
    allocate(tcp%exyz(tcp%tf%n_liste,3))
    tcp%exyz = CMPLX(0.0,0.0)
  else if (tcp%nom_eh(j) == LISTE_EH(8)) then
    tcp%flag_compo(j) = H_V
    allocate(tcp%hxyz(tcp%tf%n_liste,3))
    tcp%hxyz = CMPLX(0.0,0.0)
  end if
end do

end subroutine resultat_champ_alias


!===========================================================	
! Subroutine resultat_champ_alias_cp
! 
!===========================================================	
subroutine resultat_champ_cp_precalcul(tcp,v,s,tm,itype_res)

implicit none

!=== Declaration des arguments ===
type(st_resultat_champ_centrepoint), intent(inout) :: tcp
type(st_volume), intent(in)   :: v
type(st_structure), intent(in)   :: s
type(st_milieu_volume), intent(in) :: tm
integer, intent(in)           :: itype_res ! type de résultat (temps=0 ou fréquence = 1)

!=== Declaration des variables ===
integer :: j,i, ix, iy, iz,n
real    :: reel3(3),reste(3)
real     :: ixyz(3),p(3)
!===   Debut des traitements   ===

tcp%dxdydz = v%dxyz
ixyz = tcp%xyz/v%dxyz+1.0
tcp%ixyz = int(ixyz)
if (tcp%normale > 0) then
  ! Vérification que le point est bien sur une face
  reste = ixyz-1.0*NINT(ixyz)
  n = (tcp%normale+1) / 2 ! direction normale à la face (1=x,2=y,3=z)
  if (abs(reste(n)) > 0.0001) then
    print '("ERREUR Sur le point d''observation de nom :",A)',trim(tcp%nomfichier)
    print '(" Direction normale à la face= ", I2)',n
    print '(" Coord en nbre de cell= ", 3ES15.7E2)',ixyz
    PRINT '(" Le point ne se trouve pas sur une face de cellule comme spécifié !!!! ")'
    stop
  end if
  
  ! choix de la cellule si le point d'observation est sur une face
  tcp%ixyz(n) = NINT(ixyz(n)) + tcp%normale - 2*n
end if  

! décalage par rapport au centre de la cellule
tcp%dxyz = tcp%xyz - ((tcp%ixyz-1.0)*1.0+0.5)*v%dxyz
! pondération 
p = tcp%xyz/v%dxyz - (tcp%ixyz-1.0) ! p=0 si coin cell inf gauche 

! Recherche des faces métalliques
!tcp%flagstsurf_face = structure_metal_cellule(s,int(tcp%ixyz))
tcp%flagstsurf_face = structure_surfacique_cellule(s,int(tcp%ixyz))
if (rang==RANG_MAITRE_AFF) then
  print '(T10," Position cellule =",3I5)',tcp%ixyz
  print '(T10," Décalage dans la cellule =",3ES13.4)',tcp%dxyz
  print '(T10," pondération à partir du coin de la cellule =",3F10.5)',p
  print '(T10," Flags faces métals =",6L3)',tcp%flagstsurf_face
end if
ix = tcp%ixyz(1)
iy = tcp%ixyz(2)
iz = tcp%ixyz(3)
if (itype_res==0) then
  n = tcp%tf%n_liste ! nombre d'échantillons
else
  n = v%ntinc /tcp%tf%isaut+1  ! pour le domaine des fréquences
end if
tcp%nbech = n
tcp%flag_compo = 0
do j = 1,tcp%nbc
  select case(tcp%nom_eh(j))
    case(LISTE_EH(1),LISTE_EH(2),LISTE_EH(3))
      if (LISTE_EH(1)==tcp%nom_eh(j)) then
        tcp%flag_compo(j) = E_X
      else if (LISTE_EH(2)==tcp%nom_eh(j)) then
        tcp%flag_compo(j) = E_Y
      else
        tcp%flag_compo(j) = E_Z
      end if
      
      allocate(tcp%p(j)%Ecentre(1:n))          
      !allocate(tcp%p(j)%gradE1(1:n))          
      !allocate(tcp%p(j)%gradE2(1:n))          
      allocate(tcp%p(j)%Einf(1:n))          
      allocate(tcp%p(j)%Esup(1:n))
      tcp%p(j)%Ecentre = 0.0
      !tcp%p(j)%gradE1  = 0.0;tcp%p(j)%gradE2  = 0.0
      tcp%p(j)%Einf    = 0.0;tcp%p(j)%Esup    = 0.0
    case(LISTE_EH(4),LISTE_EH(5),LISTE_EH(6))
      if (LISTE_EH(4)==tcp%nom_eh(j)) then
        tcp%flag_compo(j) = H_X
      else if (LISTE_EH(5)==tcp%nom_eh(j)) then
        tcp%flag_compo(j) = H_Y
      else
        tcp%flag_compo(j) = H_Z
      end if
      allocate(tcp%p(j)%Hcentre(1:n))          
      allocate(tcp%p(j)%gradH(1:n))          
      allocate(tcp%p(j)%H1inf(1:n))          
      allocate(tcp%p(j)%H1sup(1:n))          
      allocate(tcp%p(j)%H2inf(1:n))          
      allocate(tcp%p(j)%H2sup(1:n)) 
      tcp%p(j)%Hcentre = 0.0;
      tcp%p(j)%gradH   = 0.0;
      tcp%p(j)%H1inf   = 0.0; tcp%p(j)%H1sup = 0.0
      tcp%p(j)%H2inf   = 0.0; tcp%p(j)%H2sup = 0.0
    case (LISTE_EH(7))
      tcp%flag_compo(j) = E_V
      do i=1,3
        allocate(tcp%pe(i)%Ecentre(1:n))          
        allocate(tcp%pe(i)%Einf(1:n))          
        allocate(tcp%pe(i)%Esup(1:n))
        tcp%p(i)%Ecentre = 0.0
        tcp%p(i)%Einf    = 0.0;tcp%p(i)%Esup    = 0.0
      end do
    case (LISTE_EH(8))
      tcp%flag_compo(j) = H_V
      do i=1,3
	allocate(tcp%ph(i)%Hcentre(1:n))          
	allocate(tcp%ph(i)%gradH(1:n))          
	allocate(tcp%ph(i)%H1inf(1:n))          
	allocate(tcp%ph(i)%H1sup(1:n))          
	allocate(tcp%ph(i)%H2inf(1:n))          
	allocate(tcp%ph(i)%H2sup(1:n)) 
	tcp%ph(i)%Hcentre = 0.0;
	tcp%ph(i)%gradH   = 0.0;
	tcp%ph(i)%H1inf   = 0.0; tcp%ph(i)%H1sup = 0.0
	tcp%ph(i)%H2inf   = 0.0; tcp%ph(i)%H2sup = 0.0
      end do
  end select
  
  ! -- Tableau d'alias vers les composantes à stocker --
  if (tcp%nom_eh(j) == LISTE_EH(1)) then !ex
    tcp%nu_eh(j) = 1
    tcp%p(j)%pond(1,1) = (1-p(2))*(1-p(3)) ! pondération 2D transverse y,z
    tcp%p(j)%pond(2,1) =     p(2)*(1-p(3)) 
    tcp%p(j)%pond(1,2) = (1-p(2))*p(3)     
    tcp%p(j)%pond(2,2) =     p(2)*p(3)  
    
    tcp%p(j)%pas = v%dxyz(1)
    reel3 = cherche_epsilon_cellule(tm,ix,iy,iz) 
    tcp%p(j)%eps_c = reel3(1)    
    reel3 = cherche_epsilon_cellule(tm,ix-1,iy,iz) 
    tcp%p(j)%eps_inf = reel3(1)    
    reel3 = cherche_epsilon_cellule(tm,ix+1,iy,iz) 
    tcp%p(j)%eps_sup = reel3(1)    
  else if (tcp%nom_eh(j) == LISTE_EH(2)) then !ey
    tcp%nu_eh(j) = 2
    tcp%p(j)%pond(1,1) = (1-p(1))*(1-p(3)) ! pondération 2D transverse x,z
    tcp%p(j)%pond(2,1) =     p(1)*(1-p(3)) 
    tcp%p(j)%pond(1,2) = (1-p(1))*p(3)     
    tcp%p(j)%pond(2,2) =     p(1)*p(3)  
    
    tcp%p(j)%pas = v%dxyz(2)
    reel3 = cherche_epsilon_cellule(tm,ix,iy,iz) 
    tcp%p(j)%eps_c = reel3(2)    
    reel3 = cherche_epsilon_cellule(tm,ix,iy-1,iz) 
    tcp%p(j)%eps_inf = reel3(2)    
    reel3 = cherche_epsilon_cellule(tm,ix,iy+1,iz) 
    tcp%p(j)%eps_sup = reel3(2)    
  else if (tcp%nom_eh(j) == LISTE_EH(3)) then !ez
    tcp%nu_eh(j) = 3
    tcp%p(j)%pond(1,1) = (1-p(1))*(1-p(2)) ! pondération 2D transverse x,y
    tcp%p(j)%pond(2,1) =     p(1)*(1-p(2)) 
    tcp%p(j)%pond(1,2) = (1-p(1))*p(2)     
    tcp%p(j)%pond(2,2) =     p(1)*p(2)  
    
    tcp%p(j)%pas = v%dxyz(3)
    reel3 = cherche_epsilon_cellule(tm,ix,iy,iz) 
    tcp%p(j)%eps_c = reel3(3)    
    reel3 = cherche_epsilon_cellule(tm,ix,iy,iz-1) 
    tcp%p(j)%eps_inf = reel3(3)    
    reel3 = cherche_epsilon_cellule(tm,ix,iy,iz+1) 
    tcp%p(j)%eps_sup = reel3(3)  
  !-----------------------------------------------------    
  else if (tcp%nom_eh(j) == LISTE_EH(4)) then !hx
    tcp%nu_eh(j) = 4
    tcp%p(j)%pas = v%dxyz(1)
    reel3 = cherche_mu_cellule(tm,ix,iy,iz) 
    tcp%p(j)%mu_c = reel3(1)    
    reel3 = cherche_mu_cellule(tm,ix,iy-1,iz) 
    tcp%p(j)%mu1(1) = reel3(1)    
    reel3 = cherche_mu_cellule(tm,ix,iy+1,iz) 
    tcp%p(j)%mu1(2) = reel3(1)    
    reel3 = cherche_mu_cellule(tm,ix,iy,iz-1) 
    tcp%p(j)%mu2(1) = reel3(1)    
    reel3 = cherche_mu_cellule(tm,ix,iy,iz+1) 
    tcp%p(j)%mu2(2) = reel3(1)    
  else if (tcp%nom_eh(j) == LISTE_EH(5)) then !hy
    tcp%nu_eh(j) = 5
    tcp%p(j)%pas = v%dxyz(2)
    reel3 = cherche_mu_cellule(tm,ix,iy,iz) 
    tcp%p(j)%mu_c = reel3(2)    
    reel3 = cherche_mu_cellule(tm,ix,iy,iz-1) 
    tcp%p(j)%mu1(1) = reel3(2)    
    reel3 = cherche_mu_cellule(tm,ix,iy,iz+1) 
    tcp%p(j)%mu1(2) = reel3(2)    
    reel3 = cherche_mu_cellule(tm,ix-1,iy,iz) 
    tcp%p(j)%mu2(1) = reel3(2)    
    reel3 = cherche_mu_cellule(tm,ix+1,iy,iz) 
    tcp%p(j)%mu2(2) = reel3(2)    
  else if (tcp%nom_eh(j) == LISTE_EH(6)) then !hz
    tcp%nu_eh(j) = 6
    tcp%p(j)%pas = v%dxyz(3)
    reel3 = cherche_mu_cellule(tm,ix,iy,iz) 
    tcp%p(j)%mu_c = reel3(3)    
    reel3 = cherche_mu_cellule(tm,ix-1,iy,iz) 
    tcp%p(j)%mu1(1) = reel3(3)    
    reel3 = cherche_mu_cellule(tm,ix+1,iy,iz) 
    tcp%p(j)%mu1(2) = reel3(3)    
    reel3 = cherche_mu_cellule(tm,ix,iy-1,iz) 
    tcp%p(j)%mu2(1) = reel3(3)    
    reel3 = cherche_mu_cellule(tm,ix,iy+1,iz) 
    tcp%p(j)%mu2(2) = reel3(3)    
  !----------------------------------------------------------------------------
  else if (tcp%flag_compo(j) == E_V) then
    do i=1,3
      tcp%pe(i)%pond(1,1) = (1- p(PC211(i)))*(1-p(PC332(i))) ! pondération 2D transverse y,z
      tcp%pe(i)%pond(2,1) =     p(PC211(i)) *(1-p(PC332(i))) 
      tcp%pe(i)%pond(1,2) = (1- p(PC211(i)))*p(PC332(i))     
      tcp%pe(i)%pond(2,2) =     p(PC211(i))*p(PC332(i))  
      tcp%pe(i)%pas = v%dxyz(i)
      reel3 = cherche_epsilon_cellule(tm,ix,iy,iz) ; tcp%pe(i)%eps_c = reel3(i)
      
      if (i==1) then
        reel3 = cherche_epsilon_cellule(tm,ix-1,iy,iz) ; tcp%pe(i)%eps_inf = reel3(i)    
        reel3 = cherche_epsilon_cellule(tm,ix+1,iy,iz) ; tcp%pe(i)%eps_sup = reel3(i)    
      else if (i==2) then
	reel3 = cherche_epsilon_cellule(tm,ix,iy-1,iz) ; tcp%pe(i)%eps_inf = reel3(i)    
	reel3 = cherche_epsilon_cellule(tm,ix,iy+1,iz) ; tcp%pe(i)%eps_sup = reel3(i)    
      else
	reel3 = cherche_epsilon_cellule(tm,ix,iy,iz-1) ; tcp%pe(i)%eps_inf = reel3(i)    
	reel3 = cherche_epsilon_cellule(tm,ix,iy,iz+1) ; tcp%pe(i)%eps_sup = reel3(i)  
      end if
      
    end do
  !----------------------------------------------------------------------------
  else if (tcp%flag_compo(j) == H_V) then
    do i=1,3
      tcp%ph(i)%pas = v%dxyz(i)
      reel3 = cherche_mu_cellule(tm,ix,iy,iz) ; tcp%ph(i)%mu_c = reel3(i)
      if (i==1) then      
        reel3 = cherche_mu_cellule(tm,ix,iy-1,iz);  tcp%ph(i)%mu1(1) = reel3(i)    
        reel3 = cherche_mu_cellule(tm,ix,iy+1,iz);  tcp%ph(i)%mu1(2) = reel3(i)    
        reel3 = cherche_mu_cellule(tm,ix,iy,iz-1);  tcp%ph(i)%mu2(1) = reel3(i)    
        reel3 = cherche_mu_cellule(tm,ix,iy,iz+1);  tcp%ph(i)%mu2(2) = reel3(i)    
      else if (i==2) then      
        reel3 = cherche_mu_cellule(tm,ix,iy,iz-1);  tcp%ph(i)%mu1(1) = reel3(i)    
        reel3 = cherche_mu_cellule(tm,ix,iy,iz+1);  tcp%ph(i)%mu1(2) = reel3(i)    
        reel3 = cherche_mu_cellule(tm,ix-1,iy,iz);  tcp%ph(i)%mu2(1) = reel3(i)    
        reel3 = cherche_mu_cellule(tm,ix+1,iy,iz);  tcp%ph(i)%mu2(2) = reel3(i)    
      else
        reel3 = cherche_mu_cellule(tm,ix-1,iy,iz);  tcp%ph(i)%mu1(1) = reel3(i)    
        reel3 = cherche_mu_cellule(tm,ix+1,iy,iz);  tcp%ph(i)%mu1(2) = reel3(i)    
        reel3 = cherche_mu_cellule(tm,ix,iy-1,iz);  tcp%ph(i)%mu2(1) = reel3(i)    
        reel3 = cherche_mu_cellule(tm,ix,iy+1,iz);  tcp%ph(i)%mu2(2) = reel3(i)    
      end if
      
    end do     
  end if
!  print '(T15," composantes n°",I2," Nom = ",A)',tcp%nu_eh(j),TRIM(tcp%nom_eh(j))

!  if (tcp%nu_eh(j)<=3) then
!    print '(T15," eps_c =",ES13.4)',tcp%p(j)%eps_c
!    print '(T15," eps_inf, eps_sup =",2ES13.4)',tcp%p(j)%eps_inf,tcp%p(j)%eps_sup
!  end if
!  if (tcp%nu_eh(j)>3) then
!    print '(T15," mu_c =",ES13.4)',tcp%p(j)%mu_c
!    print '(T15," mu1 =",2ES13.4)',tcp%p(j)%mu1
!    print '(T15," mu2 =",2ES13.4)',tcp%p(j)%mu2
!  end if
  
end do

end subroutine resultat_champ_cp_precalcul
!===========================================================	
! Subroutine resultat_champ_alias_cp
! 
!===========================================================	
subroutine resultat_champ_alias_cp(tcp,tc)

implicit none

!=== Declaration des arguments ===
type(st_resultat_champ_centrepoint), intent(inout) :: tcp
type(st_champ), intent(in)   :: tc

!=== Declaration des variables ===
integer :: j, ix, iy, iz

!===   Debut des traitements   ===
ix = tcp%ixyz(1)
iy = tcp%ixyz(2)
iz = tcp%ixyz(3)
do j = 1,tcp%nbc
  ! -- Tableau d'alias vers les composantes a stocker --
  if (tcp%nom_eh(j) == LISTE_EH(1)) then !ex
    tcp%p(j)%Ec => tc%ex(ix  ,iy:iy+1,iz:iz+1)      
    tcp%p(j)%Em => tc%ex(ix-1,iy:iy+1,iz:iz+1)      
    tcp%p(j)%Ep => tc%ex(ix+1,iy:iy+1,iz:iz+1)      
  else if (tcp%nom_eh(j) == LISTE_EH(2)) then !ey
    tcp%p(j)%Ec => tc%ey(ix:ix+1,iy  ,iz:iz+1)      
    tcp%p(j)%Em => tc%ey(ix:ix+1,iy-1,iz:iz+1)      
    tcp%p(j)%Ep => tc%ey(ix:ix+1,iy+1,iz:iz+1)      
  else if (tcp%nom_eh(j) == LISTE_EH(3)) then !ez
    tcp%p(j)%Ec => tc%ez(ix:ix+1,iy:iy+1,iz)      
    tcp%p(j)%Em => tc%ez(ix:ix+1,iy:iy+1,iz-1)      
    tcp%p(j)%Ep => tc%ez(ix:ix+1,iy:iy+1,iz+1)      
  else if (tcp%nom_eh(j) == LISTE_EH(4)) then !hx
    tcp%p(j)%Hc  => tc%hx(ix:ix+1,iy,iz)      
    tcp%p(j)%H1m => tc%hx(ix:ix+1,iy-1,iz)      
    tcp%p(j)%H1p => tc%hx(ix:ix+1,iy+1,iz)      
    tcp%p(j)%H2m => tc%hx(ix:ix+1,iy  ,iz-1)      
    tcp%p(j)%H2p => tc%hx(ix:ix+1,iy  ,iz+1)      
  else if (tcp%nom_eh(j) == LISTE_EH(5)) then !hy
    tcp%p(j)%Hc  => tc%hy(ix  ,iy:iy+1,iz)      
    tcp%p(j)%H1m => tc%hy(ix  ,iy:iy+1,iz-1)      
    tcp%p(j)%H1p => tc%hy(ix  ,iy:iy+1,iz+1)      
    tcp%p(j)%H2m => tc%hy(ix-1,iy:iy+1,iz  )      
    tcp%p(j)%H2p => tc%hy(ix+1,iy:iy+1,iz  )      
  else if (tcp%nom_eh(j) == LISTE_EH(6)) then !hz
    tcp%p(j)%Hc  => tc%hz(ix  ,iy  ,iz:iz+1)  
    tcp%p(j)%H1m => tc%hz(ix-1,iy  ,iz:iz+1)  
    tcp%p(j)%H1p => tc%hz(ix+1,iy  ,iz:iz+1)  
    tcp%p(j)%H2m => tc%hz(ix  ,iy-1,iz:iz+1)  
    tcp%p(j)%H2p => tc%hz(ix  ,iy+1,iz:iz+1) 
  !----------------------------------------------------------------------------
  else if (tcp%flag_compo(j) == E_V) then
    tcp%pe(1)%Ec => tc%ex(ix  ,iy:iy+1,iz:iz+1)      
    tcp%pe(1)%Em => tc%ex(ix-1,iy:iy+1,iz:iz+1)      
    tcp%pe(1)%Ep => tc%ex(ix+1,iy:iy+1,iz:iz+1) 
    
    tcp%pe(2)%Ec => tc%ey(ix:ix+1,iy  ,iz:iz+1)      
    tcp%pe(2)%Em => tc%ey(ix:ix+1,iy-1,iz:iz+1)      
    tcp%pe(2)%Ep => tc%ey(ix:ix+1,iy+1,iz:iz+1) 
    
    tcp%pe(3)%Ec => tc%ez(ix:ix+1,iy:iy+1,iz)      
    tcp%pe(3)%Em => tc%ez(ix:ix+1,iy:iy+1,iz-1)      
    tcp%pe(3)%Ep => tc%ez(ix:ix+1,iy:iy+1,iz+1)      
  
  !----------------------------------------------------------------------------
  else if (tcp%flag_compo(j) == H_V) then
    tcp%ph(1)%Hc  => tc%hx(ix:ix+1,iy,iz)      
    tcp%ph(1)%H1m => tc%hx(ix:ix+1,iy-1,iz)      
    tcp%ph(1)%H1p => tc%hx(ix:ix+1,iy+1,iz)      
    tcp%ph(1)%H2m => tc%hx(ix:ix+1,iy  ,iz-1)      
    tcp%ph(1)%H2p => tc%hx(ix:ix+1,iy  ,iz+1)      
    
    tcp%ph(2)%Hc  => tc%hy(ix  ,iy:iy+1,iz)      
    tcp%ph(2)%H1m => tc%hy(ix  ,iy:iy+1,iz-1)      
    tcp%ph(2)%H1p => tc%hy(ix  ,iy:iy+1,iz+1)      
    tcp%ph(2)%H2m => tc%hy(ix-1,iy:iy+1,iz  )      
    tcp%ph(2)%H2p => tc%hy(ix+1,iy:iy+1,iz  )      

    tcp%ph(3)%Hc  => tc%hz(ix  ,iy  ,iz:iz+1)  
    tcp%ph(3)%H1m => tc%hz(ix-1,iy  ,iz:iz+1)  
    tcp%ph(3)%H1p => tc%hz(ix+1,iy  ,iz:iz+1)  
    tcp%ph(3)%H2m => tc%hz(ix  ,iy-1,iz:iz+1)  
    tcp%ph(3)%H2p => tc%hz(ix  ,iy+1,iz:iz+1) 
    
  end if
end do

end subroutine resultat_champ_alias_cp


!===========================================================	
! Subroutine resultat_champ_calcul
!  Ecriture des résultats à chaque iteration temporelle
! dans l'intervalle temporel spéccifié
!===========================================================	
subroutine resultat_champ_calcul(tic,tfs)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_champ), intent(inout)   :: tic
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
integer :: i,j,k,n,ix,iy,iz
real    :: ex,ey,ez,hx,hy,hz
real, dimension(1:8)  :: teh 
complex :: ctfd
type(st_resultat_champ_point), pointer :: tcp
type(st_resultat_champ_centrepoint), pointer :: tcpcp
type(st_pointeur_centrepoint), pointer   :: p
!===   Début des traitements   ===

! -- Boucle sur tous les points de stockage en temps --
do i = 1,tic%nb_pt_t
  tcp => tic%t_lpt_t(i)
!  print '("flag temps",L )',tcp%tf%flag_calcul
  if (.not.tcp%tf%flag_calcul) cycle ! -- Cas ou on ne stocke pas --
    
  ! -- Placement dans tableau  des composantes a stocker --
!  print '("flag temps",I3 )',tcp%nbc
  do j=1,tcp%nbc
    if (tcp%flag_compo(j)<E_V) then
      teh(j) = tcp%p(j)%eh 
    else if (tcp%flag_compo(j)==E_V) then
      ix = tcp%ix
      iy = tcp%iy
      iz = tcp%iz
      ex = 0.25*(tic%ex(ix,iy,iz)+tic%ex(ix,iy+1,iz)+tic%ex(ix,iy,iz+1)+tic%ex(ix,iy+1,iz+1))
      ey = 0.25*(tic%ey(ix,iy,iz)+tic%ey(ix,iy,iz+1)+tic%ey(ix+1,iy,iz)+tic%ey(ix+1,iy,iz+1))
      ez = 0.25*(tic%ez(ix,iy,iz)+tic%ez(ix+1,iy,iz)+tic%ez(ix,iy+1,iz)+tic%ez(ix+1,iy+1,iz))
      teh(j) = sqrt(ex**2+ey**2+ez**2)
    else if (tcp%flag_compo(j)==H_V) then
      ix = tcp%ix
      iy = tcp%iy
      iz = tcp%iz
      hx = 0.5*(tic%hx(ix,iy,iz)+tic%hx(ix+1,iy,iz))
      hy = 0.5*(tic%hy(ix,iy,iz)+tic%hy(ix,iy+1,iz))
      hz = 0.5*(tic%hz(ix,iy,iz)+tic%hz(ix,iy,iz+1))
      teh(j) = sqrt(hx**2+hy**2+hz**2)    
    end if
  end do    

  !if (tfs%format_fichier == FORMAT_ASCII) then
    write (tcp%unite,tcp%mantisse) &
          tcp%tf%liste(tcp%tf%curseur), teh(1:tcp%nbc)
  !end if
end do


!------------------------------------------------------------------------------
! -- Boucle sur les points de stockage en frequence --
do i = 1,tic%nb_pt_f
  tcp => tic%t_lpt_f(i)
  if (.not.tcp%tf%flag_calcul) cycle
  do j = 1, tcp%nbc  ! -- Boucle sur les composantes --
    if (tcp%flag_compo(j)<E_V) then
      teh(j) = tcp%p(j)%eh 
      tcp%eh_f(:,j) = tcp%eh_f(:,j) + teh(j) * tcp%tf%expi2pitf(:)
    else if (tcp%flag_compo(j)==E_V) then
      ix = tcp%ix
      iy = tcp%iy
      iz = tcp%iz
      ex = 0.25*(tic%ex(ix,iy,iz)+tic%ex(ix,iy+1,iz)+tic%ex(ix,iy,iz+1)+tic%ex(ix,iy+1,iz+1))
      ey = 0.25*(tic%ey(ix,iy,iz)+tic%ey(ix,iy,iz+1)+tic%ey(ix+1,iy,iz)+tic%ey(ix+1,iy,iz+1))
      ez = 0.25*(tic%ez(ix,iy,iz)+tic%ez(ix+1,iy,iz)+tic%ez(ix,iy+1,iz)+tic%ez(ix+1,iy+1,iz))
      tcp%exyz(:,1) = tcp%exyz(:,1)+ex*tcp%tf%expi2pitf(:)
      tcp%exyz(:,2) = tcp%exyz(:,2)+ey*tcp%tf%expi2pitf(:)
      tcp%exyz(:,3) = tcp%exyz(:,3)+ez*tcp%tf%expi2pitf(:)
    else if (tcp%flag_compo(j)==H_V) then
      ix = tcp%ix
      iy = tcp%iy
      iz = tcp%iz
      hx = 0.5*(tic%hx(ix,iy,iz)+tic%hx(ix+1,iy,iz))
      hy = 0.5*(tic%hy(ix,iy,iz)+tic%hy(ix,iy+1,iz))
      hz = 0.5*(tic%hz(ix,iy,iz)+tic%hz(ix,iy,iz+1))
      tcp%hxyz(:,1) = tcp%hxyz(:,1)+hx*tcp%tf%expi2pitf(:)
      tcp%hxyz(:,2) = tcp%hxyz(:,2)+hy*tcp%tf%expi2pitf(:)
      tcp%hxyz(:,3) = tcp%hxyz(:,3)+hz*tcp%tf%expi2pitf(:)
    end if
  end do
end do

!------------------------------------------------------------------------------
! ---  résultats temporels pour les Points en position centre point ---
do i = 1,tic%nb_ptcp_t
  tcpcp => tic%lptcp_t(i)
  if (.not.tcpcp%tf%flag_calcul) cycle ! -- Cas ou on ne stocke pas --
  n = tcpcp%tf%curseur
  do j=1,tcpcp%nbc
    !print '("tcpcp%flag_compo=",I5)',tcpcp%flag_compo(j)
    if (tcpcp%flag_compo(j)<=E_Z) then
      p=>tcpcp%p(j)
      p%Ecentre(n) = p%Ec(1,1)*p%pond(1,1)+p%Ec(1,2)*p%pond(1,2)+ &
                     p%Ec(2,1)*p%pond(2,1)+p%Ec(2,2)*p%pond(2,2)
		     
      p%Einf(n)    = p%Em(1,1)*p%pond(1,1)+p%Em(1,2)*p%pond(1,2)+ &
                     p%Em(2,1)*p%pond(2,1)+p%Em(2,2)*p%pond(2,2)
		     
      p%Esup(n)    =  p%Ep(1,1)*p%pond(1,1)+p%Ep(1,2)*p%pond(1,2)+ &
                      p%Ep(2,1)*p%pond(2,1)+p%Ep(2,2)*p%pond(2,2)
      !p%gradE1(n)  = 0.5 *(p%Ec(2,1)+p%Ec(2,2)-p%Ec(1,1)-p%Ec(1,2))
      !p%gradE2(n)  = 0.5 *(p%Ec(1,2)+p%Ec(2,2)-p%Ec(1,1)-p%Ec(2,1))
    else if (tcpcp%flag_compo(j)<=H_Z) then
    
      p=>tcpcp%p(j)
      p%Hcentre(n) = 0.5*(p%Hc(1) + p%Hc(2))
      p%gradH(n)   = p%Hc(2) - p%Hc(1)
      p%H1inf(n)   = 0.5*(p%H1m(1) + p%H1m(2))
      p%H1sup(n)   = 0.5*(p%H1p(1) + p%H1p(2))
      p%H2inf(n)   = 0.5*(p%H2m(1) + p%H2m(2))
      p%H2sup(n)   = 0.5*(p%H2p(1) + p%H2p(2))
    else if (tcpcp%flag_compo(j)==E_V) then
      do k=1,3
        p=>tcpcp%pe(k)
	p%Ecentre(n) = p%Ec(1,1)*p%pond(1,1)+p%Ec(1,2)*p%pond(1,2)+ &
		       p%Ec(2,1)*p%pond(2,1)+p%Ec(2,2)*p%pond(2,2)
		       
	p%Einf(n)    = p%Em(1,1)*p%pond(1,1)+p%Em(1,2)*p%pond(1,2)+ &
		       p%Em(2,1)*p%pond(2,1)+p%Em(2,2)*p%pond(2,2)
		       
	p%Esup(n)    =  p%Ep(1,1)*p%pond(1,1)+p%Ep(1,2)*p%pond(1,2)+ &
			p%Ep(2,1)*p%pond(2,1)+p%Ep(2,2)*p%pond(2,2)	
      end do
    else if (tcpcp%flag_compo(j)==H_V) then
      do k=1,3
        p=>tcpcp%ph(k)
	p%Hcentre(n) = 0.5*(p%Hc(1) + p%Hc(2))
	p%gradH(n)   = p%Hc(2) - p%Hc(1)
	p%H1inf(n)   = 0.5*(p%H1m(1) + p%H1m(2))
	p%H1sup(n)   = 0.5*(p%H1p(1) + p%H1p(2))
	p%H2inf(n)   = 0.5*(p%H2m(1) + p%H2m(2))
	p%H2sup(n)   = 0.5*(p%H2p(1) + p%H2p(2))
      end do
    end if
  end do
end do

!------------------------------------------------------------------------------
! ---  résultats pour le domaine des fréquences pour les Points en position centre point ---
do i = 1,tic%nb_ptcp_f
  tcpcp => tic%lptcp_f(i)
  if (.not.tcpcp%tf%flag_calcul) cycle ! -- Cas ou on ne stocke pas --
  n = tcpcp%tf%curseur
  do j=1,tcpcp%nbc
    if (tcpcp%flag_compo(j)<=E_Z) then
      p=>tcpcp%p(j)
      p%Ecentre(n) = p%Ec(1,1)*p%pond(1,1)+p%Ec(1,2)*p%pond(1,2)+ &
                     p%Ec(2,1)*p%pond(2,1)+p%Ec(2,2)*p%pond(2,2)
		     
      p%Einf(n)    = p%Em(1,1)*p%pond(1,1)+p%Em(1,2)*p%pond(1,2)+ &
                     p%Em(2,1)*p%pond(2,1)+p%Em(2,2)*p%pond(2,2)
		     
      p%Esup(n)    =  p%Ep(1,1)*p%pond(1,1)+p%Ep(1,2)*p%pond(1,2)+ &
                      p%Ep(2,1)*p%pond(2,1)+p%Ep(2,2)*p%pond(2,2)
      !p%gradE1(n)  = 0.5 *(p%Ec(2,1)+p%Ec(2,2)-p%Ec(1,1)-p%Ec(1,2))
      !p%gradE2(n)  = 0.5 *(p%Ec(1,2)+p%Ec(2,2)-p%Ec(1,1)-p%Ec(2,1))
    else if (tcpcp%flag_compo(j)<=H_Z) then
      p=>tcpcp%p(j)
      p%Hcentre(n) = 0.5*(p%Hc(1) + p%Hc(2))
      p%gradH(n)   = p%Hc(2) - p%Hc(1)
      p%H1inf(n)   = 0.5*(p%H1m(1) + p%H1m(2))
      p%H1sup(n)   = 0.5*(p%H1p(1) + p%H1p(2))
      p%H2inf(n)   = 0.5*(p%H2m(1) + p%H2m(2))
      p%H2sup(n)   = 0.5*(p%H2p(1) + p%H2p(2))
    else if (tcpcp%flag_compo(j)==E_V) then
      do k=1,3
        p=>tcpcp%pe(k)
	p%Ecentre(n) = p%Ec(1,1)*p%pond(1,1)+p%Ec(1,2)*p%pond(1,2)+ &
		       p%Ec(2,1)*p%pond(2,1)+p%Ec(2,2)*p%pond(2,2)
		       
	p%Einf(n)    = p%Em(1,1)*p%pond(1,1)+p%Em(1,2)*p%pond(1,2)+ &
		       p%Em(2,1)*p%pond(2,1)+p%Em(2,2)*p%pond(2,2)
		       
	p%Esup(n)    =  p%Ep(1,1)*p%pond(1,1)+p%Ep(1,2)*p%pond(1,2)+ &
			p%Ep(2,1)*p%pond(2,1)+p%Ep(2,2)*p%pond(2,2)	
      end do
    else if (tcpcp%flag_compo(j)==H_V) then
      do k=1,3
        p=>tcpcp%ph(k)
	p%Hcentre(n) = 0.5*(p%Hc(1) + p%Hc(2))
	p%gradH(n)   = p%Hc(2) - p%Hc(1)
	p%H1inf(n)   = 0.5*(p%H1m(1) + p%H1m(2))
	p%H1sup(n)   = 0.5*(p%H1p(1) + p%H1p(2))
	p%H2inf(n)   = 0.5*(p%H2m(1) + p%H2m(2))
	p%H2sup(n)   = 0.5*(p%H2p(1) + p%H2p(2))
      end do
    end if
  end do
end do



end Subroutine resultat_champ_calcul


!===========================================================	
! Subroutine resultat_champ_impression
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_champ_impression(tic,tfs,t_gene)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_champ), intent(inout) :: tic
type(st_fichier_sortie), intent(in) :: tfs
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer  :: i,j,unite
type(st_resultat_champ_point), pointer :: tcp
character(LEN=2) :: c
character(LEN=LEN_MOT_CLE) :: nomfichier
character(LEN=LEN_MOT_CLE) :: mantisse

real, dimension(3)    :: gradient
real, dimension(:,:), allocatable :: teh
complex, dimension(:,:), allocatable :: teh_f
complex, dimension(:), allocatable   :: tnorm_f

!===   Début des traitements   ===

! -- Boucle sur tous les points de stockage en temps --
do i = 1,tic%nb_pt_t
  tcp => tic%t_lpt_t(i)
  ! -- Stockage ascii --
  if (tfs%format_fichier == FORMAT_ASCII) then
    close(tcp%unite)    				    
  end if
end do 

! -- Boucle sur tous les points de stockage en frequence --
do i = 1,tic%nb_pt_f
  tcp => tic%t_lpt_f(i)
  ! -- stockage ascii --
  do j=1,tcp%nbc
    if (tcp%flag_compo(j)==E_V) then
      tcp%eh_f(:,j) = sqrt(abs(tcp%exyz(:,1))**2+abs(tcp%exyz(:,2))**2+abs(tcp%exyz(:,3))**2)
    else if (tcp%flag_compo(j)==H_V) then
      tcp%eh_f(:,j) = sqrt(abs(tcp%hxyz(:,1))**2+abs(tcp%hxyz(:,2))**2+abs(tcp%hxyz(:,3))**2)    
    end if
  end do
  if (tfs%format_fichier == FORMAT_ASCII) then
  
    !---------- Normalisation ?-----------------
    if (tcp%sn%flag_normalisation) then
      allocate(tnorm_f(1:tcp%tf%n_liste))
      call Normalise_Reponse_Freq(t_gene,tcp%tf,tcp%sn,tnorm_f)
      ! normalisation pour chaque composante
      do j=1,tcp%nbc
        tcp%eh_f(:,j) = tcp%eh_f(:,j) / tnorm_f(:)
      end do
      deallocate(tnorm_f)
    end if
    
    nomfichier = fichier_sortie_nom_freq(tfs,tcp%nomfichier)

    tcp%unite = ouvre_en_ecriture(nomfichier)    				    
    call impression_ascii_complexe(tcp%unite,tcp%mantisse, tcp%com_format_tf, &
              tcp%tf%liste, tcp%eh_f,tcp%tf%n_liste,tcp%nbc)
    close(tcp%unite)    				    
  end if
end do

if (tic%nb_ptcp_t>0 .or.tic%nb_ptcp_f>0) call resultat_postcalcul_pcpt(tic,tfs,t_gene)
				
end subroutine resultat_champ_impression


!===========================================================	
! Subroutine resultat_postcalcul_pcpt
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_postcalcul_pcpt(tic,tfs,t_gene)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_champ), intent(inout) :: tic
type(st_fichier_sortie), intent(in) :: tfs
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer  :: i,j,k,n,m,unite
character(LEN=2) :: c
character(LEN=LEN_MOT_CLE) :: nomfichier
character(LEN=LEN_MOT_CLE) :: mantisse

real, dimension(:,:), allocatable :: teh,teh1
complex, dimension(:,:), allocatable :: teh_f,teh_f1
complex, dimension(:), allocatable   :: expi2pitf

complex, dimension(:), allocatable   :: tnorm_f

type(st_resultat_champ_centrepoint), pointer :: tcpcp
type(st_pointeur_centrepoint), pointer   :: p


!------------------------------------------------------------------------------
!--- Boucle sur tous les résultats centrepoint en temps ---
!------------------------------------------------------------------------------
do i = 1,tic%nb_ptcp_t
  tcpcp => tic%lptcp_t(i)
  allocate(teh(tcpcp%tf%n_liste,tcpcp%nbc));teh= 0.0
  
!  call resultat_postcalcul_interpoEH(tcpcp,tcpcp%tf%n_liste,tcpcp%nbc,teh)
  do j=1,tcpcp%nbc
    if (tcpcp%flag_compo(j)<=E_Z) then
      call resultat_postcalcul_interpoEXYZ(tcpcp,tcpcp%tf%n_liste,teh(:,j),j)
    else if (tcpcp%flag_compo(j)<=H_Z) then
      call resultat_postcalcul_interpoHXYZ(tcpcp,tcpcp%tf%n_liste,teh(:,j),j)
    else if (tcpcp%flag_compo(j)==E_V) then
      allocate(teh1(tcpcp%tf%n_liste,3));teh1= 0.0
      call resultat_postcalcul_interpoE(tcpcp,tcpcp%tf%n_liste,teh1)
      teh(:,j) = sqrt(teh1(:,1)**2+teh1(:,2)**2+teh1(:,3)**2)
      deallocate(teh1)
    else if (tcpcp%flag_compo(j)==H_V) then
      allocate(teh1(tcpcp%tf%n_liste,3));teh1= 0.0
      call resultat_postcalcul_interpoH(tcpcp,tcpcp%tf%n_liste,teh1)
      teh(:,j) = sqrt(teh1(:,1)**2+teh1(:,2)**2+teh1(:,3)**2)
      deallocate(teh1)
    end if
  end do
  
  !--- ECRITURE DES RÉSULTATS DANS LE FICHIER ---
  nomfichier = fichier_sortie_nom_temps(tfs,tcpcp%nomfichier)
  unite = ouvre_en_ecriture(nomfichier)    				    
  write(c,'(I1)') tcpcp%nbc+1  
  mantisse = '(' //c(1:1)//trim(tic%mantisse_reel)//')'
  do n = 1,tcpcp%tf%n_liste-1 !boucle en temps
    write (unite,mantisse) tcpcp%tf%liste(n), teh(n,1:tcpcp%nbc)
  end do
  close (unite)
  deallocate(teh)
end do

!------------------------------------------------------------------------------
!--- Boucle sur tous les résultats centrepoint en fréquence  ---
!------------------------------------------------------------------------------
do i = 1,tic%nb_ptcp_f
  tcpcp => tic%lptcp_f(i)
  allocate(teh(tcpcp%nbech,tcpcp%nbc));teh= 0.0
  allocate(teh_f(tcpcp%tf%n_liste,tcpcp%nbc))
  teh_f = CMPLX(0.0,0.0)
  allocate(expi2pitf(tcpcp%tf%n_liste))
!  call resultat_postcalcul_interpoEH(tcpcp,tcpcp%nbech,tcpcp%nbc,teh)
  do j=1,tcpcp%nbc
    !--------------------------------------------------------------------------
    if (tcpcp%flag_compo(j)<=E_Z) then
      call resultat_postcalcul_interpoEXYZ(tcpcp,tcpcp%nbech,teh(:,j),j)
    !--------------------------------------------------------------------------
    else if (tcpcp%flag_compo(j)<=H_Z) then
      call resultat_postcalcul_interpoHXYZ(tcpcp,tcpcp%nbech,teh(:,j),j)
    !--------------------------------------------------------------------------
    else if (tcpcp%flag_compo(j)==E_V) then
      allocate(teh1(tcpcp%tf%n_liste,3));teh1= 0.0
      allocate(teh_f1(tcpcp%tf%n_liste,3))
      teh_f1 = CMPLX(0.0,0.0)
      call resultat_postcalcul_interpoE(tcpcp,tcpcp%tf%n_liste,teh1)
      !--- Calcul de la Transformée Discrète ---
      expi2pitf(:) = tic%dt * tcpcp%tf%isaut / tcpcp%tf%expi2pidtf(:) 
      do n = 1,tcpcp%nbech
        expi2pitf(:) = expi2pitf(:) * tcpcp%tf%expi2pidtf(:)
        do k = 1, 3  ! -- Boucle sur les composantes --
          teh_f1(:,k) = teh_f1(:,k) + teh1(n,k) * expi2pitf(:)
        end do
      end do
      !do n=1,tcpcp%nbech
      ! write (100,'(3ES15.6)') teh1(n,1:3)
      !end do

      teh_f(:,j) = sqrt(abs(teh_f1(:,1))**2+abs(teh_f1(:,2))**2+abs(teh_f1(:,3))**2)
      deallocate(teh1,teh_f1)
    !--------------------------------------------------------------------------
    else if (tcpcp%flag_compo(j)==H_V) then
      allocate(teh1(tcpcp%tf%n_liste,3));teh1= 0.0
      allocate(teh_f1(tcpcp%tf%n_liste,3))
      teh_f1 = CMPLX(0.0,0.0)
      call resultat_postcalcul_interpoH(tcpcp,tcpcp%tf%n_liste,teh1)
      !--- Calcul de la Transformée Discrète ---
      expi2pitf(:) = tic%dt * tcpcp%tf%isaut / tcpcp%tf%expi2pidtf(:) 
      do n = 1,tcpcp%nbech
        expi2pitf(:) = expi2pitf(:) * tcpcp%tf%expi2pidtf(:)
        do k = 1, 3  ! -- Boucle sur les composantes --
          teh_f1(:,k) = teh_f1(:,k) + teh1(n,k) * expi2pitf(:)
        end do
      end do
      teh_f(:,j) = sqrt(abs(teh_f1(:,1))**2+abs(teh_f1(:,2))**2+abs(teh_f1(:,3))**2)
      deallocate(teh1,teh_f1)
    end if
  end do
  
  
  !--- Calcul de la Transformée Discrète ---
  expi2pitf(:) = tic%dt * tcpcp%tf%isaut / tcpcp%tf%expi2pidtf(:) 
  do n = 1,tcpcp%nbech
    expi2pitf(:) = expi2pitf(:) * tcpcp%tf%expi2pidtf(:)
    do j = 1, tcpcp%nbc  ! -- Boucle sur les composantes --
      if (tcpcp%flag_compo(j)>H_Z) cycle
      teh_f(:,j) = teh_f(:,j) + teh(n,j) * expi2pitf(:)
    end do
  end do
  
  !---------- NORMALISATION ?-----------------
  if (tcpcp%sn%flag_normalisation) then
    allocate(tnorm_f(1:tcpcp%tf%n_liste))
    call Normalise_Reponse_Freq(t_gene,tcpcp%tf,tcpcp%sn,tnorm_f)
    ! normalisation pour chaque composante
    do j=1,tcpcp%nbc
      teh_f(:,j) = teh_f(:,j) / tnorm_f(:)
    end do
    deallocate(tnorm_f)
  end if
  
  !--- ECRITURE DES RÉSULTATS EN FRÉQUENCE DANS LE FICHIER ---
  nomfichier = fichier_sortie_nom_freq(tfs,tcpcp%nomfichier)
  unite = ouvre_en_ecriture(nomfichier)	    
  write(c,'(I2)') (tcpcp%nbc*2+1)
  mantisse = '(' //trim(c)//trim(tic%mantisse_reel)//')'
  call impression_ascii_complexe(unite,mantisse, tcpcp%com_format_tf, &
              tcpcp%tf%liste, teh_f,tcpcp%tf%n_liste,tcpcp%nbc)
  deallocate(teh,teh_f,expi2pitf)
end do


end subroutine resultat_postcalcul_pcpt

!===========================================================	
! Subroutine resultat_postcalcul_interpoEXYZ
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_postcalcul_interpoEXYZ(tcpcp,nbech,teh,j)
!=== Déclaration des arguments ===
type(st_resultat_champ_centrepoint),target, intent(inout) :: tcpcp
integer, intent(in) :: nbech,j  ! n° de la composante
real, dimension(1:nbech),intent(out) :: teh

!===  Déclaration des variables  ===
integer  :: n,m
real                  :: grad,fact
type(st_pointeur_centrepoint), pointer   :: p

p=>tcpcp%p(j)
m = tcpcp%nu_eh(j)
! Traitement des métallisations sur les faces pour E
fact = 0.5;
if (tcpcp%flagstsurf_face(PC23(m))) then ! fonctionne en permutation circulaire, face inf
  p%Einf = p%Ecentre
  p%eps_inf = p%eps_c
  fact = 1.0
end if
if (tcpcp%flagstsurf_face(PC23(m)+1)) then  ! fonctionne en permutation circulaire, face sup
  p%Esup = p%Ecentre
  p%eps_sup = p%eps_c
  fact = 1.0
end if
do n = 1,nbech !boucle en temps
  grad = fact*(p%Esup(n)*p%eps_sup - p%Einf(n)*p%eps_inf)/(p%eps_c*p%pas)
  teh(n) = p%Ecentre(n) + tcpcp%dxyz(m)*grad
end do
end subroutine resultat_postcalcul_interpoEXYZ

!===========================================================	
! Subroutine resultat_postcalcul_interpoHXYZ
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_postcalcul_interpoHXYZ(tcpcp,nbech,teh,j)
!=== Déclaration des arguments ===
type(st_resultat_champ_centrepoint),target, intent(inout) :: tcpcp
integer, intent(in) :: nbech,j
real, dimension(1:nbech),intent(out) :: teh

!===  Déclaration des variables  ===
integer  :: n,m
real                  :: fact1,fact2
real, dimension(3)    :: gradient
type(st_pointeur_centrepoint), pointer   :: p

p=>tcpcp%p(j)
fact1 = 0.5;fact2=0.5;
m = tcpcp%nu_eh(j)-3
!--- correction sur présence de faces métalliques ---
if (tcpcp%flagstsurf_face(PC21(m))) then ! fonctionne en permutation circulaire
  p%H2inf = p%Hcentre
  p%mu2(1) = p%mu_c
  fact2 = 1.0
end if
if (tcpcp%flagstsurf_face(PC21(m)+1)) then  ! fonctionne en permutation circulaire
  p%H2sup = p%Hcentre
  p%mu2(2) = p%mu_c
  fact2 = 1.0
end if  
if (tcpcp%flagstsurf_face(PC25(m))) then ! fonctionne en permutation circulaire
  p%H1inf = p%Hcentre
  p%mu1(1) = p%mu_c
  fact1 = 1.0
end if
if (tcpcp%flagstsurf_face(PC25(m)+1)) then  ! fonctionne en permutation circulaire
  p%H1sup = p%Hcentre
  p%mu1(2) = p%mu_c
  fact1 = 1.0
end if  
!print '("j,m,p%pas=",2I5,ES15.5)',j,m,p%pas
do n = 1,nbech !boucle en temps
  gradient(m) = p%gradH(n) / p%pas
  gradient(PC1(m)) = fact1*(p%H1sup(n)*p%mu1(2) -p%H1inf(n)*p%mu1(1)) / (p%mu_c*tcpcp%dxdydz(PC1(m)))
  gradient(PC2(m)) = fact2*(p%H2sup(n)*p%mu2(2) -p%H2inf(n)*p%mu2(1)) / (p%mu_c*tcpcp%dxdydz(PC2(m)))
  teh(n) = p%Hcentre(n) + dot_product(tcpcp%dxyz,gradient)
end do
end subroutine resultat_postcalcul_interpoHXYZ


!===========================================================	
! Subroutine resultat_postcalcul_interpoE
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_postcalcul_interpoE(tcpcp,nbech,teh)

!=== Déclaration des arguments ===
type(st_resultat_champ_centrepoint),target, intent(inout) :: tcpcp
integer, intent(in) :: nbech
real, dimension(1:nbech,1:3),intent(out) :: teh
!===  Déclaration des variables  ===
integer  :: n,m
real                  :: grad,fact
type(st_pointeur_centrepoint), pointer   :: p
!=== DÉBUT DES TRAITEMENTS ===
teh= 0.0
! Traitement des métallisations sur les faces pour E
do m=1,3
  fact = 0.5;
  p=>tcpcp%pe(m)
  if (tcpcp%flagstsurf_face(PC23(m))) then ! fonctionne en permutation circulaire, face inf
    p%Einf = p%Ecentre
    p%eps_inf = p%eps_c
    fact = 1.0
  end if
  if (tcpcp%flagstsurf_face(PC23(m)+1)) then  ! fonctionne en permutation circulaire, face sup
    p%Esup = p%Ecentre
    p%eps_sup = p%eps_c
    fact = 1.0
  end if
  do n = 1,nbech !boucle en temps
    grad = fact*(p%Esup(n)*p%eps_sup - p%Einf(n)*p%eps_inf)/(p%eps_c*p%pas)
    teh(n,m) = p%Ecentre(n) + tcpcp%dxyz(m)*grad
  end do
end do

end subroutine resultat_postcalcul_interpoE

!===========================================================	
! Subroutine resultat_postcalcul_interpoH
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_postcalcul_interpoH(tcpcp,nbech,teh)

!=== Déclaration des arguments ===
type(st_resultat_champ_centrepoint),target, intent(inout) :: tcpcp
integer, intent(in) :: nbech
real, dimension(1:nbech,1:3),intent(out) :: teh

!===  Déclaration des variables  ===
integer  :: n,m
real                  :: fact1,fact2
real, dimension(3)    :: gradient
type(st_pointeur_centrepoint), pointer   :: p

teh= 0.0
do m=1,3
  fact1 = 0.5;fact2=0.5;
  p=>tcpcp%ph(m)
  !--- correction sur présence de faces métalliques ---
  if (tcpcp%flagstsurf_face(PC21(m))) then ! fonctionne en permutation circulaire
    p%H2inf = p%Hcentre
    p%mu2(1) = p%mu_c
    fact2 = 1.0
  end if
  if (tcpcp%flagstsurf_face(PC21(m)+1)) then  ! fonctionne en permutation circulaire
    p%H2sup = p%Hcentre
    p%mu2(2) = p%mu_c
    fact2 = 1.0
  end if  
  if (tcpcp%flagstsurf_face(PC25(m))) then ! fonctionne en permutation circulaire
    p%H1inf = p%Hcentre
    p%mu1(1) = p%mu_c
    fact1 = 1.0
  end if
  if (tcpcp%flagstsurf_face(PC25(m)+1)) then  ! fonctionne en permutation circulaire
    p%H1sup = p%Hcentre
    p%mu1(2) = p%mu_c
    fact1 = 1.0
  end if  
  do n = 1,nbech !boucle en temps
    gradient(m) = p%gradH(n) / p%pas
    gradient(PC1(m)) = fact1*(p%H1sup(n)*p%mu1(2) -p%H1inf(n)*p%mu1(1)) / (p%mu_c*tcpcp%dxdydz(PC1(m)))
    gradient(PC2(m)) = fact2*(p%H2sup(n)*p%mu2(2) -p%H2inf(n)*p%mu2(1)) / (p%mu_c*tcpcp%dxdydz(PC2(m)))
    teh(n,m) = p%Hcentre(n) + dot_product(tcpcp%dxyz,gradient)
  end do
end do

end subroutine resultat_postcalcul_interpoH

!===========================================================	
! Subroutine resultat_postcalcul_interpo
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_postcalcul_interpoXYZ(tcpcp,nbech,nbc,teh)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_champ_centrepoint),target, intent(inout) :: tcpcp
integer, intent(in) :: nbech,nbc
real, dimension(1:nbech,1:nbc),intent(out) :: teh

!===  Déclaration des variables  ===
integer  :: i,j,n,m

real                  :: grad,fact,fact1,fact2
real, dimension(3)    :: gradient
type(st_pointeur_centrepoint), pointer   :: p


!------------------------------------------------------------------------------
!--- Boucle sur tous les résultats centrepoint en temps ---
!------------------------------------------------------------------------------
  teh= 0.0
  do j=1,tcpcp%nbc
    p=>tcpcp%p(j)
    m = tcpcp%nu_eh(j)
    ! Traitement des métallisations sur les faces pour E
    if (tcpcp%flag_compo(j)<=E_Z) then
      fact = 0.5;
      if (tcpcp%flagstsurf_face(PC23(m))) then ! fonctionne en permutation circulaire, face inf
	p%Einf = p%Ecentre
	p%eps_inf = p%eps_c
	fact = 1.0
      end if
      if (tcpcp%flagstsurf_face(PC23(m)+1)) then  ! fonctionne en permutation circulaire, face sup
	p%Esup = p%Ecentre
	p%eps_sup = p%eps_c
	fact = 1.0
      end if
      do n = 1,nbech !boucle en temps
        grad = fact*(p%Esup(n)*p%eps_sup - p%Einf(n)*p%eps_inf)/(p%eps_c*p%pas)
	teh(n,j) = p%Ecentre(n) + tcpcp%dxyz(m)*grad
      end do
    !--------------------------------------------------------------------------
    else if (tcpcp%flag_compo(j)<=H_Z) then
      fact1 = 0.5;fact2=0.5;
      m = tcpcp%nu_eh(j)-3
      !--- correction sur présence de faces métalliques ---
      if (tcpcp%flagstsurf_face(PC21(m))) then ! fonctionne en permutation circulaire
	p%H2inf = p%Hcentre
	p%mu2(1) = p%mu_c
	fact2 = 1.0
      end if
      if (tcpcp%flagstsurf_face(PC21(m)+1)) then  ! fonctionne en permutation circulaire
	p%H2sup = p%Hcentre
	p%mu2(2) = p%mu_c
	fact2 = 1.0
      end if  
      if (tcpcp%flagstsurf_face(PC25(m))) then ! fonctionne en permutation circulaire
	p%H1inf = p%Hcentre
	p%mu1(1) = p%mu_c
	fact1 = 1.0
      end if
      if (tcpcp%flagstsurf_face(PC25(m)+1)) then  ! fonctionne en permutation circulaire
	p%H1sup = p%Hcentre
	p%mu1(2) = p%mu_c
	fact1 = 1.0
      end if  
      !print '("j,m,p%pas=",2I5,ES15.5)',j,m,p%pas
      do n = 1,nbech !boucle en temps
        gradient(m) = p%gradH(n) / p%pas
	gradient(PC1(m)) = fact1*(p%H1sup(n)*p%mu1(2) -p%H1inf(n)*p%mu1(1)) / (p%mu_c*tcpcp%dxdydz(PC1(m)))
	gradient(PC2(m)) = fact2*(p%H2sup(n)*p%mu2(2) -p%H2inf(n)*p%mu2(1)) / (p%mu_c*tcpcp%dxdydz(PC2(m)))
	teh(n,j) = p%Hcentre(n) + dot_product(tcpcp%dxyz,gradient)
      end do
      
    !--------------------------------------------------------------------------
    ! ------------  RECONSTRUCTION DU VECTEUR E -------------------------------
    else if  (tcpcp%flag_compo(j)==E_V) then
      do m=1,3
        fact = 0.5;
        p=>tcpcp%pe(m)
	if (tcpcp%flagstsurf_face(PC23(m))) then ! fonctionne en permutation circulaire, face inf
	  p%Einf = p%Ecentre
	  p%eps_inf = p%eps_c
	  fact = 1.0
	end if
	if (tcpcp%flagstsurf_face(PC23(m)+1)) then  ! fonctionne en permutation circulaire, face sup
	  p%Esup = p%Ecentre
	  p%eps_sup = p%eps_c
	  fact = 1.0
	end if
	do n = 1,nbech !boucle en temps
	  grad = fact*(p%Esup(n)*p%eps_sup - p%Einf(n)*p%eps_inf)/(p%eps_c*p%pas)
	  teh(n,j) = teh(n,j)+(p%Ecentre(n) + tcpcp%dxyz(m)*grad)**2
	end do
      end do
      teh(:,j) = sqrt(teh(:,j))
    !--------------------------------------------------------------------------
    ! ------------  RECONSTRUCTION DU VECTEUR H -------------------------------
    else if  (tcpcp%flag_compo(j)==H_V) then
      do m=1,3
        fact1 = 0.5;fact2=0.5;
        p=>tcpcp%ph(m)
	!--- correction sur présence de faces métalliques ---
	if (tcpcp%flagstsurf_face(PC21(m))) then ! fonctionne en permutation circulaire
	  p%H2inf = p%Hcentre
	  p%mu2(1) = p%mu_c
	  fact2 = 1.0
	end if
	if (tcpcp%flagstsurf_face(PC21(m)+1)) then  ! fonctionne en permutation circulaire
	  p%H2sup = p%Hcentre
	  p%mu2(2) = p%mu_c
	  fact2 = 1.0
	end if  
	if (tcpcp%flagstsurf_face(PC25(m))) then ! fonctionne en permutation circulaire
	  p%H1inf = p%Hcentre
	  p%mu1(1) = p%mu_c
	  fact1 = 1.0
	end if
	if (tcpcp%flagstsurf_face(PC25(m)+1)) then  ! fonctionne en permutation circulaire
	  p%H1sup = p%Hcentre
	  p%mu1(2) = p%mu_c
	  fact1 = 1.0
	end if  
	do n = 1,nbech !boucle en temps
	  gradient(m) = p%gradH(n) / p%pas
	  gradient(PC1(m)) = fact1*(p%H1sup(n)*p%mu1(2) -p%H1inf(n)*p%mu1(1)) / (p%mu_c*tcpcp%dxdydz(PC1(m)))
	  gradient(PC2(m)) = fact2*(p%H2sup(n)*p%mu2(2) -p%H2inf(n)*p%mu2(1)) / (p%mu_c*tcpcp%dxdydz(PC2(m)))
	  teh(n,j) = teh(n,j) + (p%Hcentre(n) + dot_product(tcpcp%dxyz,gradient))**2
	end do
      end do
      teh(:,j) = sqrt(teh(:,j))
      
    end if
  end do
end subroutine resultat_postcalcul_interpoXYZ

!===========================================================	
! Subroutine resultat_postcalcul_gradient
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_postcalcul_gradientEH(tcpcp,nbech,nbc,teh)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_champ_centrepoint),target, intent(inout) :: tcpcp
integer, intent(in) :: nbech,nbc
real, dimension(1:nbech,1:nbc),intent(out) :: teh


!===  Déclaration des variables  ===
integer  :: i,j,n,m
real, dimension(3)    :: gradient
type(st_pointeur_centrepoint), pointer   :: p

!------------------------------------------------------------------------------
!--- Boucle sur tous les résultats centrepoint en temps ---
!------------------------------------------------------------------------------
  teh= 0.0
  do j=1,tcpcp%nbc
    p=>tcpcp%p(j)
    m = tcpcp%nu_eh(j)
    ! Traitement des métallisations sur les faces pour E
    if (m<=3) then
      if (tcpcp%flagstsurf_face(PC23(m))) then ! fonctionne en permutation circulaire
	p%Einf = p%Ecentre*2
	p%Esup = p%Esup*2
	p%eps_inf = p%eps_c
      end if
      if (tcpcp%flagstsurf_face(PC23(m)+1)) then  ! fonctionne en permutation circulaire
	p%Esup = p%Ecentre*2
	p%Einf = p%Einf*2
	p%eps_sup = p%eps_c
      end if
    end if
    if (tcpcp%nu_eh(j) == E_X) then !EX      
      do n = 1,tcpcp%tf%n_liste !boucle en temps
        gradient(1) = 0.5*(p%Esup(n)*p%eps_sup - p%Einf(n)*p%eps_inf)/(p%eps_c*p%pas)
	gradient(2) = p%gradE1(n)/tcpcp%dxdydz(2)
	gradient(3) = p%gradE2(n)/tcpcp%dxdydz(3)
	teh(n,j) = p%Ecentre(n) + dot_product(tcpcp%dxyz,gradient)
      end do
    else if (tcpcp%nu_eh(j) == E_Y) then 
      do n = 1,nbech !boucle en temps
        gradient(2) = 0.5*(p%Esup(n)*p%eps_sup - p%Einf(n)*p%eps_inf)/(p%eps_c*p%pas)
	gradient(1) = p%gradE1(n)/tcpcp%dxdydz(1)
	gradient(3) = p%gradE2(n)/tcpcp%dxdydz(3)
	teh(n,j) = p%Ecentre(n) + dot_product(tcpcp%dxyz,gradient)
      end do
    else if (tcpcp%nu_eh(j) == E_Z) then 
      do n = 1,nbech  !boucle en temps
        gradient(3) = 0.5*(p%Esup(n)*p%eps_sup - p%Einf(n)*p%eps_inf)/(p%eps_c*p%pas)
	gradient(1) = p%gradE1(n)/tcpcp%dxdydz(1)
	gradient(2) = p%gradE2(n)/tcpcp%dxdydz(2)
	teh(n,j) = p%Ecentre(n) + dot_product(tcpcp%dxyz,gradient)
      end do
    !--------------------------------------------------------------------------
    else if (tcpcp%nu_eh(j) >= H_X) then ! cas Hx,Hy,Hz
      m = tcpcp%nu_eh(j)-3
      !--- correction su présence de faces métalliques ---
      if (tcpcp%flagstsurf_face(PC21(m))) then ! fonctionne en permutation circulaire
	p%H2inf = p%Hcentre*2
	p%H2sup = p%H2sup*2
	p%mu2(1) = p%mu_c
      end if
      if (tcpcp%flagstsurf_face(PC21(m)+1)) then  ! fonctionne en permutation circulaire
	p%H2sup = p%Hcentre*2
	p%H2inf = p%H2inf*2
	p%mu2(2) = p%mu_c
      end if  
      if (tcpcp%flagstsurf_face(PC25(m))) then ! fonctionne en permutation circulaire
	p%H1inf = p%Hcentre*2
	p%H1sup = p%H1sup*2
	p%mu1(1) = p%mu_c
      end if
      if (tcpcp%flagstsurf_face(PC25(m)+1)) then  ! fonctionne en permutation circulaire
	p%H1sup = p%Hcentre*2
	p%H1inf = p%H1inf*2
	p%mu1(2) = p%mu_c
      end if  
      
      do n = 1,nbech !boucle en temps
        gradient(m) = p%gradH(n) / p%pas
	gradient(PC1(m)) = 0.5*(p%H1sup(n)*p%mu1(2) -p%H1inf(n)*p%mu1(1)) / (p%mu_c*tcpcp%dxdydz(PC1(m)))
	gradient(PC2(m)) = 0.5*(p%H2sup(n)*p%mu2(2) -p%H2inf(n)*p%mu2(1)) / (p%mu_c*tcpcp%dxdydz(PC2(m)))
	teh(n,j) = p%Hcentre(n) + dot_product(tcpcp%dxyz,gradient)
      end do
    end if

  end do
  

end subroutine resultat_postcalcul_gradientEH

!===========================================================	
! Subroutine resultat_champ_deallocate
!  Libération de toute la mémoire allouée
!===========================================================	
subroutine resultat_champ_deallocate(tic)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_champ), intent(inout) :: tic

!===  Déclaration des variables  ===
integer :: i

!=== Debut des traitements ===
do i =1, tic%nb_pt_f
  if (ASSOCIATED(tic%t_lpt_f(i)%eh_f)) deallocate(tic%t_lpt_f(i)%eh_f)
end do			

if (tic%nb_pt_t>0) deallocate(tic%t_lpt_t)
if (tic%nb_pt_f>0) deallocate(tic%t_lpt_f)

end subroutine resultat_champ_deallocate	


! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module mfdtd_resultat_champ


