!////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_resultat_champ_surface.f90
!// Purpose:     Résultat local au volume sur l'une des 6 composantes de 
!//                champ, stockage surfacique en temps ou en fréquence
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Mai 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Module mfdtd_resultat_champ_surf.f90             ==
! == Stockage des composantes de champ sur des surfaces   ==
!===========================================================
module mfdtd_resultat_champ_surf
use m_constantes
use m_math_operation, only : atan2d_cor

use m_es_identifiant, only : MANTISSE_REEL,MANTISSE_ENTIER,FORMAT_ASCII,REELLE_IMAGINAIRE
use m_chaine_operation, only : extrait_sous_chaine,nb_mot_chaine
use m_entree, only : st_fichier, &
                lineread, &
		ligne_precedente, &
		multibackspace, &
		numbackspace_set, &
		lecture_stop, &
		ctrl_nb_mot_chaine, &
		unitefichier, &
	  apres_fin_bloc, &
    compteur_N_mot_cle,&
		compteur_mot_cle
		
				  
use m_sortie, only : st_fichier_sortie, &
                fichier_sortie_lect_format_freq, &
		impression_ascii_surface, &
		fichier_sortie_nom_temps, &
		fichier_sortie_ecrit_format, &				  
		fichier_sortie_nom_freq,&
		ouvre_en_ecriture

use m_temps_frequence, only : st_temps_frequence, &
                st_liste_temps_freq, &
		liste_temps_freq_pointe, &
 		temps_freq_compte, &
		identifie_deux_nom

use m_maillage
use mfdtd_champ, only : st_champ
use mfdtd_volume, only : st_volume
use m_generateur, only : st_generateur
use m_sortie_normalise, only : st_sortie_normalise, &
                                  sortie_normalise_initdefaut,&
				  sortie_normalise_lecture   ,&
				  sortie_normalise_ecriture ,&
				  sortie_normalise_compare, &
				  Normalise_Reponse_Freq

implicit none

!character(LEN=2), dimension(8), parameter :: LISTE_EH = &
!                    (/'ex','ey','ez','hx','hy','hz','e ','h '/) 
integer,parameter  :: TYPE_RES_SURF_YEE = 0
integer,parameter  :: TYPE_RES_SURF_CENTRE = 1
type st_pointeur_champ_surface
  REAL(SDP), dimension(:,:), pointer  :: eh
end type st_pointeur_champ_surface 

type st_pointeur_vectchamp_surface
  REAL(SDP), dimension(:,:,:), pointer  :: px,py,pz
end type st_pointeur_vectchamp_surface 

!=== Déclaration des types ===
! liste des point de stockage 
type st_resultat_champ_une_surface
  integer                :: type_sortie
  integer, dimension(6)  :: coord ! coordonnées de la surface de stockage
  integer, dimension(3)  :: pas
  integer                :: nb_pt1, nb_pt2   ! incrément spatial pour le stockage
  integer                :: deb_pt1, deb_pt2   ! debut des surfaces par rapport à l'origine 
  integer                :: fin_pt1, fin_pt2   ! debut des surfaces par rapport à l'origine 
  integer                :: pas_pt1, pas_pt2   ! saut des surfaces par rapport à l'origine 
  integer                :: type_plan
  integer  :: nbc            ! nombre de composantes
  integer  :: unite          ! unité du fichier
  integer  :: com_format_tf  ! type de formatage des données en fréquence
  type(st_pointeur_champ_surface), dimension(1:8)  :: p ! tableau d'alias vers les composantes à stocker
  character(LEN=LEN_MOT_CLE), dimension(1:8) :: nom_eh

  type(st_pointeur_vectchamp_surface) ::e,h
  logical  :: flag_e,flag_h
  logical,dimension(1:8) :: flag_compo
  character(LEN=LEN_MOT_CLE) :: nomfichier
  character(LEN=LEN_MOT_CLE) :: mantisse

  type(st_temps_frequence), pointer :: tf
  ! tableaux pour le stockage des composantes  en fréquence
  COMPLEX(SDP), dimension(:,:,:,:), pointer :: eh_f

! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
end type st_resultat_champ_une_surface

!------------------------------------------------------------------------------
type st_resultat_champ_surf_ES ! données directes pour les entrées sorties
  character(LEN=LEN_MOT_CLE) :: nom,type_observ
  character(LEN=LEN_MOT_CLE) :: interv1,interv2  
  integer  :: nbc            ! nombre de composantes
  character(LEN=LEN_MOT_CLE), dimension(1:8) :: nom_eh
  integer   :: tf_commutateur
  !REAL(SDP), dimension(3)     :: xyz        ! coordonnees du point de stockage
  integer, dimension(6)  :: icoord
  integer, dimension(3)  :: ipas
   ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
   
end type st_resultat_champ_surf_ES


!=== structure principale ===
type st_resultat_champ_surf
  logical  :: flag_data
! Nombre de points de stockage en temps et en fréquence
  integer  ::  nb_s_f, nb_s_t 
  REAL(SDP)     ::  dt
  character(LEN=LEN_MOT_CLE) :: mantisse_entier, mantisse_reel
  ! structure de points de stockage en temps et en fréquence   
  type(st_resultat_champ_une_surface), dimension(:), pointer  :: ls_t, ls_f  
  
    !--  données directes pour les entrées sorties --
  integer :: n_res
  type(st_resultat_champ_surf_ES), pointer, dimension(:) :: l_res

end type st_resultat_champ_surf
 
! Pas de variables globales...

contains

!===========================================================	
! Subroutine resultat_champ_surf_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine resultat_champ_surf_initdefaut(tcs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_champ_surf), intent(inout) :: tcs

!===  Déclaration des variables  ===

!===   Début des traitements   ===
tcs%nb_s_f = 0; tcs%nb_s_t = 0
tcs%dt = 0.0_SDP
tcs%mantisse_reel = MANTISSE_REEL
tcs%mantisse_entier = MANTISSE_ENTIER
tcs%flag_data = .false.
				
End Subroutine resultat_champ_surf_initdefaut

! ==========================================================
! == Subroutine resultat_chsurf_es_initdefaut             ==
! ==========================================================
subroutine resultat_chsurf_es_initdefaut(rf)

type(st_resultat_champ_surf_ES), intent(inout) :: rf

rf%nom         = ''
rf%type_observ = ''
rf%interv1 = ''
rf%interv2 = ''
rf%tf_commutateur = 0;
rf%nbc = 0
rf%nom_eh(:) = ''
rf%icoord=0 
rf%ipas=0 
call sortie_normalise_initdefaut(rf%sn)

end subroutine resultat_chsurf_es_initdefaut

!===========================================================	
! Subroutine resultat_champ_surf_copy
!===========================================================	
Subroutine resultat_champ_surf_copy(rsin,rsout)
!=== Déclaration des arguments ===
type(st_resultat_champ_une_surface), intent(in)  :: rsin
type(st_resultat_champ_une_surface), intent(out) :: rsout

!===   Début des traitements   ===
rsout%type_sortie = rsin%type_sortie
rsout%coord  =rsin%coord
rsout%pas    =rsin%pas
rsout%nb_pt1 =rsin%nb_pt1 ; rsout%nb_pt2=rsin%nb_pt2
rsout%deb_pt1=rsin%deb_pt1;rsout%deb_pt2=rsin%deb_pt2
rsout%fin_pt1=rsin%fin_pt1;rsout%fin_pt2=rsin%fin_pt2
rsout%pas_pt1=rsin%pas_pt1;rsout%pas_pt2=rsin%pas_pt2
rsout%type_plan=rsin%type_plan
rsout%nbc    =rsin%nbc
rsout%unite  =rsin%unite
rsout%com_format_tf=rsin%com_format_tf
rsout%nom_eh =rsin%nom_eh
rsout%flag_e =rsin%flag_e;rsout%flag_h=rsin%flag_h
rsout%flag_compo=rsin%flag_compo
rsout%nomfichier=rsin%nomfichier
rsout%mantisse=rsin%mantisse
rsout%tf=>rsin%tf
rsout%sn=rsin%sn
nullify(rsout%eh_f)

end Subroutine resultat_champ_surf_copy

!===========================================================	
! Subroutine resultat_champ_surf_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine resultat_champ_surf_lecture(f, tcs,ltf,tm)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_resultat_champ_surf), intent(inout) :: tcs
type(st_liste_temps_freq), intent(inout) :: ltf
type(st_maillage), intent(in) :: tm

!===  Déclaration des variables  ===
integer  :: status
integer  :: i,ic(2),i_s_t, i_s_f, nbmot, nbcomposante, type_plan, com_format_tf
integer  :: ires
integer, dimension(1:3)  ::pas3, nb_pt
integer, dimension(1:6)  ::coordcell6
logical  :: flag_f, flag_t
character(LEN=LEN_MOT_CLE) :: corbeille, nom, nom1, nom2, premiermot
character(LEN=LEN_MOT_CLE) :: nom_t, nom_f
character(LEN=LEN_MOT_CLE), dimension(1:8) :: nom_eh

type(st_resultat_champ_une_surface), pointer :: tcp
type(st_resultat_champ_surf_ES), pointer ::pres

! Normalisation en fréquence
type(st_sortie_normalise) :: sn

!===   Début des traitements   ===

call resultat_champ_surf_initdefaut(tcs)
!--- On compte le nombre de surface de stockage en temps et en frequence---
i_s_t = 0;
i_s_f = 0;
com_format_tf = REELLE_IMAGINAIRE    
ires = 0
!ires = compteur_mot_cle(f,'surf')
ic=compteur_N_mot_cle(f,2, (/'surf','surc'/))
ires = ic(1)+ic(2)
!print '(T10,"ires=",I4)',ires
call temps_freq_compte(f,'surf',i_s_t,i_s_f)
call temps_freq_compte(f,'surc',ic(1),ic(2))
i_s_t=i_s_t+ic(1);
i_s_f=i_s_f+ic(2);

if (i_s_t == 0 .and. i_s_f == 0) then
  call apres_fin_bloc(f)
  if (rang==RANG_MAITRE_AFF) print '("Avertissement : Aucune donnée dans le bloc ", A)', TRIM(f%nombloc)
  return
end if
tcs%flag_data = .true.

! allocation des structures en temps et en frequence...
tcs%nb_s_t = i_s_t
allocate(tcs%ls_t(i_s_t))
tcs%nb_s_f = i_s_f
allocate(tcs%ls_f(i_s_f))

tcs%n_res = ires
allocate(tcs%l_res(tcs%n_res))
do i=1,ires
  call resultat_chsurf_es_initdefaut(tcs%l_res(i))
end do

i_s_t = 1;
i_s_f = 1;
ires = 0;
! Init normalisation
call sortie_normalise_initdefaut(sn)

!******* lecture des données **********
do
  premiermot = lineread(f)
  if (premiermot(1:4) == 'surf'.or.premiermot(1:4) == 'surc') then 
    read(f%chaine,*) corbeille,nom1,nom2
    
    if (.not.identifie_deux_nom(nom1,nom2,flag_t,flag_f,nom_t,nom_f)) then
      print '(/,t10,"Nom du type de stockage erroné !!!")'
      call lecture_stop(f)
    end if
    
    ! Calcul du nombre de composantes de champ dans la chaine
    nbmot = nb_mot_chaine(f%chaine)        
    if (flag_t .and. flag_f) then
      nbcomposante = nbmot - 12; !12 mots dans la chaine hors composantes
      if (nbcomposante > 0) then
        read(f%chaine,*) nom,nom1,nom2,nom_eh(1:nbcomposante), coordcell6, pas3
      else 
        read(f%chaine,*) nom,nom1,nom2,coordcell6, pas3
      end if
    else
      nbcomposante = nbmot - 11; !11 mots dans la chaine hors composantes
      if (nbcomposante > 0) then
        read(f%chaine,*) nom,nom1,nom_eh(1:nbcomposante), coordcell6, pas3
      else 
        read(f%chaine,*) nom,nom1,coordcell6, pas3
      end if
    end if
    
    coordcell6 = origine_cellule6(tm,coordcell6)
    if (rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(nom),coordcell6
    
    ! Controle l'existence des composantes
    if (nbcomposante == 0) then 
      nbcomposante = 6
      nom_eh = (/'ex','ey','ez','hx','hy','hz','  ','  '/);    
    end if
    do i=1, nbcomposante
      if (ALL(LISTE_EH(:) /= nom_eh(i))) then
        print '(/,t10,"Nom de la composante de  champ erronée : ",A)', TRIM(nom_eh(i))
        call lecture_stop(f)
      end if
    end do

    ! Controle des coordonnées
    if (.not.verif_surface(nom,coordcell6,type_plan)) call lecture_stop(f)
    if (.not.ALL(pas3>0)) then 
        print '(/,t10,"Les increments spatiaux doivent etre > 0")'
        call lecture_stop(f)      
    end if    
    
    ! nombre de points dans chaque direction
    nb_pt(:)  = (coordcell6(2:6:2) - coordcell6(1:5:2)) / pas3(:) +1
    
    do i=1,2
      if (i == 1) then
        !--- Structure en temps ---
	if (flag_t) then
	  tcp => tcs%ls_t(i_s_t)
	  ! Récupère les pointeurs sur les structures temps-frequence
	  tcp%tf => liste_temps_freq_pointe(ltf,nom_t)
	  if (.not.associated(tcp%tf)) then
            print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
            call lecture_stop(f)      
	  end if
          i_s_t = i_s_t + 1
	else 
	  cycle
  end if
      else 
        !--- structure en frequence
	if (flag_f) then
	  tcp => tcs%ls_f(i_s_f)
	  tcp%tf => liste_temps_freq_pointe(ltf,nom_f)
	  if (.not.associated(tcp%tf)) then
            print '(/,t10,"Nom du type de stockage en frequence erroné !!!")'
            call lecture_stop(f)      
	  end if
	  ! Information pour la normalisation ou pas
          tcp%sn = sn;

          i_s_f = i_s_f + 1
	else
	  cycle;
        end if
      end if
      ! type de composantes à stocker, coordonnées...
      if (premiermot(1:4) == 'surf') then
        tcp%type_sortie = TYPE_RES_SURF_YEE
      else
        tcp%type_sortie = TYPE_RES_SURF_CENTRE
        nb_pt(:)  = (coordcell6(2:6:2)-1 - coordcell6(1:5:2)) / pas3(:)+1
      end if
      tcp%nomfichier  = nom
      tcp%nom_eh      = nom_eh
      tcp%nbc = nbcomposante
      tcp%pas(:)      = pas3(:);
      tcp%coord(:)    = coordcell6(:);
      tcp%nb_pt1 = nb_pt(1)
      tcp%nb_pt2 = nb_pt(3)
      tcp%com_format_tf = com_format_tf
      tcp%type_plan = type_plan
      if (tcp%type_plan == PLAN_YOZ) tcp%nb_pt1 =NB_PT(2)
      if (tcp%type_plan == PLAN_XOY) tcp%nb_pt2 =NB_PT(2)
    end do    
    !--- Récupère les données pour l'écriture ---
    ires=ires+1
    pres=>tcs%l_res(ires)
    pres%nom  = nom
    pres%interv1 = nom1
    if (flag_t .and. flag_f) pres%interv2 = nom2
    pres%tf_commutateur = com_format_tf;
    pres%nbc = nbcomposante
    pres%nom_eh(1:nbcomposante) = nom_eh(1:nbcomposante)
    pres%icoord=coordcell6
    pres%ipas  =pas3
    pres%sn  = sn
	
    
  !----------------------------------------------------------------------------
  else if (sortie_normalise_lecture(f,premiermot,sn)) then
    cycle
  !----------------------------------------------------------------------------
  else if (f%nomfinbloc == premiermot) then
    exit ! fin du bloc    
  else if (.not.fichier_sortie_lect_format_freq(premiermot,com_format_tf)) then
    print *,'(/,t10,"Mot clé non reconnu  : ",A )', TRIM(premiermot)
    call lecture_stop(f)
  end if 
enddo
			
end subroutine resultat_champ_surf_lecture

! ==========================================================
! == Subroutine resultat_champ_surf_ecriture              ==
! ==========================================================
subroutine resultat_champ_surf_ecriture(f,tc,tm)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_resultat_champ_surf), intent(inout) :: tc
type(st_maillage), intent(in) :: tm

integer  :: i,j,commutateur
integer  :: coordcell6(1:6)
character(LEN=LEN_MOT_CLE) :: ch1
type(st_resultat_champ_surf_ES),pointer :: pres
type(st_sortie_normalise) :: sn

if (.not.tc%flag_data) return

commutateur = REELLE_IMAGINAIRE

call sortie_normalise_initdefaut(sn)

do i=1,tc%n_res
  pres =>tc%l_res(i)
  if (.not.sortie_normalise_compare(sn,pres%sn)) then
    call sortie_normalise_ecriture(f,pres%sn)
    sn = pres%sn
  end if
  
  if (commutateur/=pres%tf_commutateur) then
    call fichier_sortie_ecrit_format(f,pres%tf_commutateur)
    commutateur = pres%tf_commutateur
  end if
  
  ch1 = ''
  do j=1,pres%nbc
    ch1 = trim(ch1)//' '//TRIM(pres%nom_eh(j))
  end do
  if (pres%nom(1:4) == 'surf') then
    coordcell6 = origine_cellule6(tm,pres%icoord,.true.)
!    print '("icoord=",6I5)',pres%icoord
!    print '("coordcell=",6I5)',coordcell6
    if (pres%interv2 /='') then
      write(f%unite,'(T2,A,3x,A,3x,A,3x,A,3x,6I5,3I4)') trim(pres%nom),trim(pres%interv1),&
                                             trim(pres%interv2),trim(ch1),coordcell6(1:6),&
					     pres%ipas(1:3)
    else
      write(f%unite,'(T2,A,3x,A,3x,A,3x,6I5,3I4)') trim(pres%nom),trim(pres%interv1),&
                                             trim(ch1),coordcell6(1:6),pres%ipas(1:3)
    end if
  end if
end do

end subroutine resultat_champ_surf_ecriture

! ==========================================================
! == Subroutine verif_surface                             ==
! == Verifie que les six entiers forment une surface      ==
! == Déduit le type de plan      			  ==
! ==========================================================
logical function verif_surface(nom,coord6,type_plan)

implicit none

character(LEN=LEN_MOT_CLE), intent(in) :: nom
integer, dimension(1:6), intent(in)    :: coord6
integer, intent(out)  :: type_plan

integer  :: i,j
logical  :: flagsup

verif_surface = .true.
type_plan = -1

! vérifie que la surface en est une : coordonnées égales dans 1 direction
j=0
do i=1,3
 if (coord6(i*2-1) == coord6(i*2)) then
   j = j+1
   type_plan = PC1(i)   
 end if
end do

if (j==0) then
  verif_surface = .false.
  print '(/,t10,"ATTENTION, ", A, " ne represente pas une surface !!!")', TRIM(nom)
  return
endif

! vérifie que les coordonnées sont correctes (imax>imin)
do i=1,3
 if (coord6(i*2) < coord6(i*2-1)) then
   verif_surface = .false.
   print '(/,t10,"Veillez a ce que indices de debut < indices de fin")'
   return 
 end if
end do
end function verif_surface


!===========================================================	
! Subroutine resultat_champ_surf_init
! Initialisation des variables, lorsque qu'il n'y a pas de lecture du fichier
! d'entrée
!===========================================================	
Subroutine resultat_champ_surf_init(tcs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_champ_surf), intent(inout) :: tcs

!===  Déclaration des variables  ===

!===   Début des traitements   ===

				
End Subroutine resultat_champ_surf_init
	
!===========================================================	
! Subroutine resultat_champ_surf_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
subroutine resultat_champ_surf_precalcul(tcs,tvol,tc,tfs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_champ_surf), intent(inout) :: tcs
type(st_volume), intent(in)  :: tvol
type(st_champ), intent(in)   :: tc
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nomfichier
character(LEN=5) :: c
integer  :: i,j,k,n,m,unite
type(st_resultat_champ_une_surface), pointer :: tcp
!===   Début des traitements   ===

tcs%dt = tvol%dt
tcs%mantisse_reel   = tfs%mantisse_reel
tcs%mantisse_entier = tfs%mantisse_entier

!--- stockage en temps ---
do i = 1,tcs%nb_s_t
  tcp => tcs%ls_t(i)
  
  ! Creation des alias sur les composantes de champ
  call resultat_champ_surf_alias(tcp,tc)
  
  ! création du format d'écriture en temps
  write(c,'(I5)') tcp%nb_pt1  
  tcp%mantisse = '(' //c(1:LEN_TRIM(c))// &
             tcs%mantisse_reel(1:LEN_TRIM(tcs%mantisse_reel))//')'
  
  ! ouverture du fichier si format asccii
  if (tfs%format_fichier == FORMAT_ASCII) then   
    nomfichier = fichier_sortie_nom_temps(tfs,tcp%nomfichier)
    tcp%unite = ouvre_en_ecriture(nomfichier)    				    
    
  end if
  
end do				

!--- stockage en frequence ---
do i = 1,tcs%nb_s_f
  tcp => tcs%ls_f(i)
  ! Creation des alias sur les composantes de champ
  call resultat_champ_surf_alias(tcp,tc)
  
  ! création du format d'écriture en fréquence
  write(c,'(I5)') tcp%nb_pt1*2  
  tcp%mantisse = '(' //c(1:LEN_TRIM(c))//tcs%mantisse_reel &
                 (1:LEN_TRIM(tcs%mantisse_reel))//')'
  !-- stockage en fréquence
  allocate(tcp%eh_f(1:tcp%nb_pt1, 1:tcp%nb_pt2, 1:tcp%nbc, 1:tcp%tf%n_liste))          
  ! Initialisation du tableau sans boucle peut ne pas passer sur 
  ! certaine machine tel que Uqbar
  do n=1,tcp%tf%n_liste
  do k=1,tcp%nbc
  do j=1,tcp%nb_pt2
  do m=1,tcp%nb_pt1
    tcp%eh_f(m,j,k,n) = CMPLX(0.0,0.0,SDP)          
  end do;  end do;end do;end do
  
end do				

end Subroutine resultat_champ_surf_precalcul

!===========================================================	
! Subroutine resultat_champ_surf_alias
! 
!===========================================================	
subroutine resultat_champ_surf_alias(tcp,tc)

!=== Déclaration des arguments ===
type(st_resultat_champ_une_surface), intent(inout) :: tcp
type(st_champ), intent(in)   :: tc

!=== Déclaration des variables ===
integer :: j
integer, dimension(3) :: i
integer, dimension(6) :: crd
REAL(SDP), dimension(:,:,:), pointer :: peh
REAL(SDP), dimension(:,:,:), pointer :: peh1,peh2,peh3
!===   Début des traitements   ===

crd = tcp%coord
i = tcp%pas
tcp%flag_e = .false.
tcp%flag_h = .false.
tcp%flag_compo = .true.
do j = 1,tcp%nbc
  ! tableau d'alias vers les composantes à stocker
  if (tcp%nom_eh(j) == LISTE_EH(1)) then
     peh => tc%ex      
  else if (tcp%nom_eh(j) == LISTE_EH(2)) then
     peh => tc%ey      
  else if (tcp%nom_eh(j) == LISTE_EH(3)) then
     peh => tc%ez      
  else if (tcp%nom_eh(j) == LISTE_EH(4)) then
     peh => tc%hx      
  else if (tcp%nom_eh(j) == LISTE_EH(5)) then
     peh => tc%hy      
  else if (tcp%nom_eh(j) == LISTE_EH(6)) then
     peh => tc%hz      
  else if (tcp%nom_eh(j) == LISTE_EH(7)) then
      tcp%e%px => tc%ex
      tcp%e%py => tc%ey
      tcp%e%pz => tc%ez
     tcp%flag_e = .true.
     tcp%flag_compo(j) = .false.
     cycle
  else if (tcp%nom_eh(j) == LISTE_EH(8)) then
      tcp%h%px => tc%hx
      tcp%h%py => tc%hy
      tcp%h%pz => tc%hz
     tcp%flag_h = .true.
     tcp%flag_compo(j) = .false.
     cycle
  end if

  if (tcp%type_plan == PLAN_XOY) then
    tcp%p(j)%eh => peh(crd(1):crd(2) , crd(3):crd(4) , crd(5))
  else if (tcp%type_plan == PLAN_YOZ) then
    tcp%p(j)%eh => peh(crd(1) , crd(3):crd(4) , crd(5):crd(6))
  else
    tcp%p(j)%eh => peh(crd(1):crd(2) , crd(3) , crd(5):crd(6))
  end if
end do

if (tcp%type_plan == PLAN_XOY) then
    tcp%deb_pt1 = crd(1);
    tcp%deb_pt2 = crd(3);
    tcp%fin_pt1 = crd(2);
    tcp%fin_pt2 = crd(4);
    tcp%pas_pt1 = i(1);
    tcp%pas_pt2 = i(2);
else if (tcp%type_plan == PLAN_YOZ) then
    tcp%deb_pt1 = crd(3);
    tcp%deb_pt2 = crd(5);
    tcp%fin_pt1 = crd(4);
    tcp%fin_pt2 = crd(6);
    tcp%pas_pt1 = i(2);
    tcp%pas_pt2 = i(3);
else
    tcp%deb_pt1 = crd(1);
    tcp%deb_pt2 = crd(5);
    tcp%fin_pt1 = crd(2);
    tcp%fin_pt2 = crd(6);    
    tcp%pas_pt1 = i(1);
    tcp%pas_pt2 = i(3);
end if

end subroutine resultat_champ_surf_alias

!===========================================================	
! Subroutine resultat_champ_surf_calcul
!  Ecriture des résultats à chaque iteration temporelle
! dans l'intervalle temporel spéccifié
!===========================================================	
subroutine resultat_champ_surf_calcul(tcs,tfs)
!=== Déclaration des arguments ===
type(st_resultat_champ_surf), intent(inout)   :: tcs
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
integer :: i,j,k, m,n
REAL(SDP), dimension(:,:), pointer  :: peh 
type(st_resultat_champ_une_surface), pointer :: tcp

!===   Début des traitements   ===

!--- boucle sur toutes les surfaces de stockage en temps ---
do i = 1,tcs%nb_s_t
  tcp => tcs%ls_t(i)
  if (.not.tcp%tf%flag_calcul) cycle ! cas ou on ne stocke pas      
    
  ! boucle sur toutes les composantes à stocker
  if (tcp%type_sortie==TYPE_RES_SURF_YEE) then  
    do j=1,tcp%nbc
      !    if (tfs%format_fichier == FORMAT_ASCII) then
      if (tcp%flag_compo(j)) then
        do k=1,tcp%nb_pt2
          m = 1 + (k-1) * tcp%pas_pt2
          write (tcp%unite,tcp%mantisse)  (tcp%p(j)%eh(n,m), &
	                        n=1,tcp%nb_pt1*tcp%pas_pt1,tcp%pas_pt1)
        end do
      end if
    end do 
  else
    do j=1,tcp%nbc
      !    if (tfs%format_fichier == FORMAT_ASCII) then
      if (tcp%flag_compo(j)) then
        do k=1,tcp%nb_pt2
          m = 1 + (k-1) * tcp%pas_pt2
          write (tcp%unite,tcp%mantisse)  (tcp%p(j)%eh(n,m), &
	                        n=1,tcp%nb_pt1*tcp%pas_pt1,tcp%pas_pt1)
        end do
      else if (tcp%flag_e) then
  	
      else if (tcp%flag_h) then
      
      end if
    end do     
  end if
end do
    
!--- boucle sur les surfaces de stockage en fréquence ---
do i = 1,tcs%nb_s_f
  tcp => tcs%ls_f(i)
  if (.not.tcp%tf%flag_calcul) cycle
  do j = 1, tcp%nbc  ! boucle sur les composantes
    if (tcp%flag_compo(j)) then
      do k = 1, tcp%tf%n_liste  ! boucle sur les frequences
      do m = 1, tcp%nb_pt2  ! boucle sur la surface
      do n = 1, tcp%nb_pt1  ! boucle sur la surface

      tcp%eh_f(n,m,j,k) = tcp%eh_f(n,m,j,k) + tcp%tf%expi2pitf(k) * &
                          tcp%p(j)%eh(1 + (n-1)*tcp%pas_pt1, &
                                      1 + (m-1)*tcp%pas_pt2) 
      end do; end do                                       
      end do
    else if (tcp%flag_e) then
    
    else if (tcp%flag_h) then
    
    end if

  end do
end do

end Subroutine resultat_champ_surf_calcul



!===========================================================	
! Subroutine resultat_champ_surf_impression
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_champ_surf_impression(tcs,tfs,t_gene)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_champ_surf), intent(inout) :: tcs
type(st_fichier_sortie), intent(in) :: tfs
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer  :: i,j,k,n,m
type(st_resultat_champ_une_surface), pointer :: tcp
character(LEN=LEN_MOT_CLE) :: nomfichier
COMPLEX(SDP), dimension(:), allocatable   :: tnorm_f

!===   Début des traitements   ===

!--- boucle sur les surfaces de stockage en temps ---
do i = 1,tcs%nb_s_t
  tcp => tcs%ls_t(i)
  ! stockage ascii
!  if (tfs%format_fichier == FORMAT_ASCII) then
    close(tcp%unite)    				    
!  end if
end do  

!--- boucle sur les surfaces de stockage en frequence ---
do i = 1,tcs%nb_s_f
  tcp => tcs%ls_f(i)
  !if (tfs%format_fichier == FORMAT_ASCII) then
    nomfichier = fichier_sortie_nom_freq(tfs,tcp%nomfichier)
    tcp%unite = ouvre_en_ecriture(nomfichier)    				    
  !end if   				    

  !---------- Normalisation ?-----------------
  if (tcp%sn%flag_normalisation) then
    allocate(tnorm_f(1:tcp%tf%n_liste))
    call Normalise_Reponse_Freq(t_gene,tcp%tf,tcp%sn,tnorm_f)
    ! normalisation pour chaque fréquence
    do k=1,tcp%tf%n_liste
      do j = 1, tcp%nbc  ! boucle sur les composantes
        do m = 1, tcp%nb_pt2  ! boucle sur la surface
        do n = 1, tcp%nb_pt1  ! boucle sur la surface
          tcp%eh_f(n,m,j,k) = tcp%eh_f(n,m,j,k) / tnorm_f(k)
	end do;end do
      end do
    end do
    deallocate(tnorm_f)
  end if
  
  ! stockage ascii      
!  if (tfs%format_fichier == FORMAT_ASCII) then
    call impression_ascii_surface(tcp%unite, &
             tcp%mantisse,tcp%com_format_tf, tcp%tf%liste, tcp%eh_f, &
	     tcp%nb_pt1, tcp%nb_pt2, tcp%tf%n_liste,tcp%nbc)  
!    print '("mantisse =",A)', TRIM(tcp%mantisse)
!    print '("commutateur =",I2)', tcp%com_format_tf
!    print '("taille eh_f =",2I3)', size(tcp%eh_f,1), size(tcp%eh_f,2)
!    print '("nb_pt1 et 2 =",2I3)', tcp%nb_pt1,tcp%nb_pt2
!    print '("nb_freq et nb_composante =",2I4)', tcp%tf%n_liste,tcp%nbc
    close(tcp%unite)    				    
!  end if
end do				

				
End Subroutine resultat_champ_surf_impression

!===========================================================	
! Subroutine resultat_champ_surf_deallocate
!  Libération de toute la mémoire allouée
!===========================================================	
subroutine resultat_champ_surf_deallocate(tcs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_champ_surf), intent(inout) :: tcs

!===  Déclaration des variables  ===
integer :: i

!=== Debut des traitements ===

do i =1, tcs%nb_s_f
  if (ASSOCIATED(tcs%ls_f(i)%eh_f)) deallocate(tcs%ls_f(i)%eh_f)
end do			

if (tcs%nb_s_t>0) deallocate(tcs%ls_t)
if (tcs%nb_s_f>0) deallocate(tcs%ls_f)
if (tcs%n_res>0 ) deallocate(tcs%l_res)

end subroutine resultat_champ_surf_deallocate	

end module mfdtd_resultat_champ_surf


