!////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_resultat_champ_volume.f90
!// Purpose:      Stockage Eps et SigmaE sur un volume donné
!// Author:      Christophe Guiffaut
!// Modified by:
!// Created  :     Avril 2003
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!////////////////////////////////////////////////////////////////////////
module mfdtd_resultat_champ_volume
use m_constantes
use m_math_operation, only : atan2d_cor,dp

use m_es_identifiant, only : MANTISSE_REEL,MANTISSE_ENTIER,FORMAT_ASCII,REELLE_IMAGINAIRE
use m_chaine_operation, only : extrait_sous_chaine,nb_mot_chaine
use m_entree, only : st_fichier, &
                                  lineread, &
				  ligne_precedente, &
				  multibackspace, &
				  numbackspace_set, &
				  lecture_stop, &
				  ctrl_nb_mot_chaine, &
				  compteur_mot_cle, &
				  apres_fin_bloc, &
				  unitefichier
				  
use m_sortie, only : st_fichier_sortie, &
                fichier_sortie_lect_format_freq, &
		impression_ascii_surface, &
		fichier_sortie_nom_temps, &
		fichier_sortie_ecrit_format, &				  
		fichier_sortie_nom_freq,&
		ouvre_en_ecriture

use m_temps_frequence, only : st_temps_frequence, &
                              st_liste_temps_freq, &
			      liste_temps_freq_pointe, &
			      temps_freq_compte, &
			      identifie_deux_nom
use m_maillage
use mfdtd_champ, only : st_champ
use mfdtd_volume, only : st_volume
use mfdtd_milieu_volume, only : st_milieu_volume				  
use m_generateur, only : st_generateur
use m_sortie_normalise

implicit none

integer, parameter :: VOL_CALC_MOY    = 0
integer, parameter :: VOL_CALC_MOYC   = 1
integer, parameter :: VOL_CALC_SUM    = 2
integer, parameter :: VOL_CALC_SUMC   = 3
integer, parameter :: VOL_CALC_MAX    = 4
integer, parameter :: VOL_CALC_VAR    = 5
integer, parameter :: VOL_CALC_SIGMA  = 6
integer, parameter :: VOL_CALC_POYNTING  = 7

!=========================== Déclaration des types ===========================

type st_pointeur_champ_volume
  real, dimension(:,:,:), pointer  :: eh
end type st_pointeur_champ_volume 

!===================  STRUCTURE POUR LES PARAMÈTRES EPSILON, SIGMA ============
type st_resultat_param_volume
  integer, dimension(6)  :: coord ! coordonnées de la surface de stockage
  integer, dimension(3)  :: pas
  logical, dimension(3) :: flag_dir
  character(LEN=LEN_MOT_CLE) :: nom
end type st_resultat_param_volume

!======================= STRUCTURE DE TRAITEMENT DE VOLUME ====================
type st_resultat_champ_un_volume
  character(LEN=LEN_MOT_CLE) :: nom
  integer, dimension(6)  :: coord ! coordonnées de la surface de stockage
  integer, dimension(3)  :: pas
  integer  :: nbc,nb_ech      !nbre de composantes et nbre d'échantillons
  integer, dimension(3)  :: nb_pt
  integer  :: com_format_tf  ! type de formatage des données en fréquence
  
  ! type de traitement à appliquer sur chaque composante
  integer :: nb_type_calcul,nb_type_calcul_fmoy
  integer, dimension(:,:),pointer  :: type_calcul,type_calcul_fmoy  
  
  integer  :: unite          ! unité du fichier en temporel
  type(st_pointeur_champ_volume), dimension(1:6)  :: p ! tableau d'alias vers les composantes à stocker
  character(LEN=LEN_MOT_CLE), dimension(1:6) :: nom_eh
  
  character(LEN=LEN_MOT_CLE) :: mantisse

  type(st_temps_frequence), pointer :: tf
  
  real,dimension(:,:,:), pointer      :: res_t ! résultat temporel
  ! tableaux pour le stockage des composantes  en fréquence
  
  logical  :: flag_f,flag_fmoy
  complex, dimension(:,:,:,:,:), pointer :: eh_f
  complex, dimension(:,:), pointer :: eh_fmoy

  ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn

end type st_resultat_champ_un_volume

!------------------------------------------------------------------------------
type st_resultat_champ_vol_ES ! données directes pour les entrées sorties
  character(LEN=LEN_MOT_CLE) :: nom,type_observ
  character(LEN=LEN_MOT_CLE) :: interv1,interv2 
  character(LEN=LEN_CHAINE)  :: ch_type_calcul  
  integer  :: nbc            ! nombre de composantes
  character(LEN=LEN_MOT_CLE), dimension(1:6) :: nom_eh
  integer   :: tf_commutateur
  integer, dimension(6)  :: icoord
  integer, dimension(3)  :: ipas
   ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
   
end type st_resultat_champ_vol_ES


!======================= STRUCTURE PRINCIPALE =============================
type st_resultat_champ_volume
  logical :: flag_data
  character(LEN=LEN_MOT_CLE) :: mantisse_entier, mantisse_reel
  integer  :: nb_eps, nb_sige
  type(st_resultat_param_volume), dimension(:), pointer  :: l_eps, l_sige

  integer  ::  nb_v_f, nb_v_t 
  ! structure de points de stockage en temps et en fréquence   
  type(st_resultat_champ_un_volume), dimension(:), pointer  :: lv_t, lv_f  
  
    !--  données directes pour les entrées sorties --
  integer :: n_res
  type(st_resultat_champ_vol_ES), pointer, dimension(:) :: l_res
  real, dimension(:,:,:), pointer  :: ex,ey,ez,hx,hy,hz
end type st_resultat_champ_volume
 
! Pas de variables globales...

contains

!===========================================================	
! Subroutine resultat_champ_surf_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine resultat_champ_vol_initdefaut(v)
!=== Déclaration des arguments ===
type(st_resultat_champ_volume), intent(inout) :: v

!===  Déclaration des variables  ===

!===   Début des traitements   ===
v%nb_eps = 0; v%nb_sige = 0
v%mantisse_reel = MANTISSE_REEL
v%mantisse_entier = MANTISSE_ENTIER
v%flag_data = .false.	

nullify(v%l_eps);nullify(v%l_sige)
!nullify(v%ex);nullify(v%ey);nullify(v%ez)
!nullify(v%hx);nullify(v%hy);nullify(v%hz) ! pointeur vers les composantes de champ

v%nb_v_f=0;v%nb_v_t =0
v%n_res=0
nullify(v%l_res)

End Subroutine resultat_champ_vol_initdefaut

!===========================================================	
! Subroutine resultat_champ_unvol_initdefaut
!===========================================================	
Subroutine resultat_champ_unvol_initdefaut(vout)
!=== Déclaration des arguments ===
type(st_resultat_champ_un_volume), intent(out) :: vout

!===   Début des traitements   ===
vout%nom   = ''
vout%coord = 0; vout%pas = 0
vout%nbc   = 0; vout%nb_ech = 0
vout%nb_pt = 0
vout%com_format_tf  = REELLE_IMAGINAIRE
vout%nb_type_calcul = 0
vout%nb_type_calcul_fmoy = 0
vout%unite = 0
nullify(vout%type_calcul)
nullify(vout%type_calcul_fmoy)
vout%nom_eh = ''
!vout%flag_e = .false.
!vout%flag_h = .false.
!vout%flag_compo = .false.
vout%flag_f = .false.
vout%flag_fmoy = .false.

vout%mantisse = ''

nullify(vout%res_t)
nullify(vout%tf)
nullify(vout%eh_f)
nullify(vout%eh_fmoy)
call sortie_normalise_initdefaut(vout%sn)
end Subroutine resultat_champ_unvol_initdefaut


!===========================================================	
! Subroutine st_resultat_param_volume
! Copy de la structure
! Les tableaux dynamiques pour la partie calcul ne sont pas
!  alloués
!===========================================================	
Subroutine resultat_volume_param_copy(vin,vout)
!=== Déclaration des arguments ===
type(st_resultat_param_volume), intent(in)  :: vin
type(st_resultat_param_volume), intent(out) :: vout

!===  Déclaration des variables  ===

!===   Début des traitements   ===
vout%coord=vin%coord
vout%pas  =vin%pas
vout%nom  =vin%nom
vout%flag_dir=vin%flag_dir

end Subroutine resultat_volume_param_copy


!===========================================================	
! Subroutine resultat_champ_unvol_copy
! Copy de la structure
! Les tableaux dynamiques pour la partie calcul ne sont pas
!  alloués
!===========================================================	
Subroutine resultat_champ_unvol_copy(vin,vout)
!=== Déclaration des arguments ===
type(st_resultat_champ_un_volume), intent(in)  :: vin
type(st_resultat_champ_un_volume), intent(out) :: vout

!===  Déclaration des variables  ===

!===   Début des traitements   ===
vout%nom   = vin%nom
vout%coord = vin%coord; vout%pas  = vin%pas
vout%nbc   = vin%nbc; vout%nb_ech = vin%nb_ech
vout%nb_pt = vin%nb_pt
vout%com_format_tf  = vin%com_format_tf
vout%nb_type_calcul = vin%nb_type_calcul
vout%nb_type_calcul_fmoy = vin%nb_type_calcul_fmoy

if (vin%nb_type_calcul>0) then
  allocate(vout%type_calcul(vin%nb_type_calcul,2))
  vout%type_calcul = vin%type_calcul
end if
vout%unite = vin%unite
vout%nom_eh = vin%nom_eh

!vout%flag_e = vin%flag_e
!vout%flag_h = vin%flag_h
!vout%flag_compo = vin%flag_compo

vout%flag_f = vin%flag_f
vout%flag_fmoy = vin%flag_fmoy
vout%mantisse = vin%mantisse

vout%tf=>vin%tf
nullify(vout%eh_f)
nullify(vout%eh_fmoy)
nullify(vout%res_t)

vout%sn = vin%sn
end Subroutine resultat_champ_unvol_copy


! ==========================================================
! == Subroutine resultat_chvol_es_initdefaut              ==
! ==========================================================
subroutine resultat_chvol_es_initdefaut(rf)

type(st_resultat_champ_vol_ES), intent(inout) :: rf

rf%nom         = ''
rf%interv1 = ''
rf%interv2 = ''
rf%tf_commutateur = 0;
rf%nbc = 0
rf%nom_eh(:) = ''
rf%icoord=0 
rf%ipas=0 
rf%ch_type_calcul = ''
call sortie_normalise_initdefaut(rf%sn)

end subroutine resultat_chvol_es_initdefaut

!===========================================================	
! Subroutine resultat_champ_surf_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine resultat_champ_vol_lecture(f, tcv,ltf,tm)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_resultat_champ_volume), intent(inout) :: tcv
type(st_liste_temps_freq), intent(inout) :: ltf
type(st_maillage), intent(in)   :: tm
!===  Déclaration des variables  ===
integer  :: status,nbmot
integer  :: ires, i_v_t,i_v_f
integer  :: i,j,i_eps,i_sige
integer  :: coordcell(6)
character(LEN=LEN_MOT_CLE) :: premiermot,nom1,nom2,nom_t,nom_f
character(LEN=LEN_CHAINE) :: chaine,ch_type_calcul
character(LEN=1), dimension(1:3) :: cxyz
type(st_resultat_param_volume), pointer :: tpv
logical  :: flag_t, flag_f

type(st_resultat_champ_un_volume) :: vin
type(st_resultat_champ_vol_ES), pointer :: pres

!===   Début des traitements   ===

call resultat_champ_vol_initdefaut(tcv)
!------ On compte le nombre de volume de stockage --------

tcv%nb_eps  = compteur_mot_cle(f,'volepsr')
tcv%nb_sige = compteur_mot_cle(f,'volsige')
tcv%n_res = compteur_mot_cle(f,'VU')
call temps_freq_compte(f,'VU',tcv%nb_v_t,tcv%nb_v_f)


if (tcv%nb_eps==0 .and.tcv%nb_sige==0.and.tcv%n_res==0)  then
 call apres_fin_bloc(f)
 if (rang==RANG_MAITRE_AFF) &
   print '("Avertissement : Aucune donnée dans le bloc ", A)', TRIM(f%nombloc)
 return
end if
tcv%flag_data = .true.

!-------- Allocation mémoire des structures --------------
if (tcv%nb_eps>0)  allocate(tcv%l_eps(tcv%nb_eps))
if (tcv%nb_sige>0)  allocate(tcv%l_sige(tcv%nb_sige))

if (tcv%nb_v_t>0) allocate(tcv%lv_t(tcv%nb_v_t))
if (tcv%nb_v_f>0) allocate(tcv%lv_f(tcv%nb_v_f))


allocate(tcv%l_res(tcv%n_res))
do i=1,tcv%n_res
  call resultat_chvol_es_initdefaut(tcv%l_res(i))
end do

!--------------- Initialisation ------------------
i_eps = 1;
i_sige = 1;
ires=0;
i_v_t=1;i_v_f=1
ch_type_calcul = ''
call resultat_champ_unvol_initdefaut(vin)
!******* lecture des données **********
do
  premiermot = lineread(f)
  if ((premiermot(1:7) == 'volepsr').or.(premiermot(1:7) == 'volsige')) then 
    if (premiermot(1:7) == 'volepsr') then
      tpv => tcv%l_eps(i_eps)
      i_eps = i_eps+1
    else
      tpv => tcv%l_sige(i_sige)
      i_sige = i_sige+1
    end if
    cxyz(:) = ' ';
    status = ctrl_nb_mot_chaine(f,f%chaine,11,13)
    nbmot = nb_mot_chaine(f%chaine)        
    read(f%chaine,*) tpv%nom,cxyz(1:nbmot-10), coordcell(1:6),tpv%pas(1:3)
    tpv%flag_dir(:)  = .false.
    tpv%coord(1:6) = origine_cellule6(tm,coordcell(1:6))
    if (rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(tpv%nom),tpv%coord(1:6)

    if (ANY(cxyz=='x')) tpv%flag_dir(1) = .true.
    if (any(cxyz=='y')) tpv%flag_dir(2) = .true.
    if (any(cxyz=='z')) tpv%flag_dir(3) = .true.
    !print '(" flag_dir = ",3L)', tpv%flag_dir(:)
    IF (.not.ANY(tpv%flag_dir)) then
      print '("Erreur, Manque l''indication de la composante x, y ou z")'
      print '("Petit coquin !!!!! J''arrete !!!!")'
      call lecture_stop(f)
    end if    
  !----------------------------------------------------------------------------
  else if (premiermot(1:2) == 'VU') then
    if (vin%nb_type_calcul==0) then
      print '(/,t10,"Erreur, le type de résultat doit etre défini au préalable !!!")'
      call lecture_stop(f)
    end if
    read(f%chaine,*) vin%nom,nom1,nom2
    if (.not.identifie_deux_nom(nom1,nom2,flag_t,flag_f,nom_t,nom_f)) then
      print '(/,t10,"Nom du type de stockage erroné !!!")'
      call lecture_stop(f)
    end if
    nbmot = nb_mot_chaine(f%chaine);
    i=3; if (flag_t .and.flag_f) i=4;    
    chaine = extrait_sous_chaine(f%chaine,i,nbmot)
    nbmot = nbmot-i+1
    vin%nbc=nbmot-9
    if (vin%nbc>0) then
      read(chaine,*) vin%nom_eh(1:vin%nbc),vin%coord,vin%pas
    else if (vin%nbc==0) then
      read(chaine,*) vin%coord,vin%pas    
    else
      print '("Erreur, Manque des informations !!!")'
      call lecture_stop(f)      
    end if
    vin%coord = origine_cellule6(tm,vin%coord)
    if (rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(vin%nom),vin%coord
    
    !------- Controle l'existence des composantes  -------------
    if (vin%nbc == 0) then 
      vin%nbc = 6
      vin%nom_eh = (/'ex','ey','ez','hx','hy','hz'/);    
    end if
    do i=1, vin%nbc
      if (ALL(COMPOSANTE_EH(:) /= vin%nom_eh(i))) then
        print '(/,t10,"Nom de la composante de  champ erronée : ",A)', TRIM(vin%nom_eh(i))
        call lecture_stop(f)
      end if
    end do
    
    !--> nombre de points dans chaque direction
    vin%nb_pt(:)  = (vin%coord(2:6:2) - vin%coord(1:5:2)) / vin%pas(:) +1
    if (any(vin%nb_pt<=0)) then
        print '(/,t10,"ERREUR, erreur sur les coordonnées!!! ")'
        call lecture_stop(f)    
    end if
    !--- Remplissage Structure en temps ---
    if (flag_t) then
      call resultat_champ_unvol_copy(vin,tcv%lv_t(i_v_t))
      ! Récupère les pointeurs sur la structure temps-frequence
      tcv%lv_t(i_v_t)%tf => liste_temps_freq_pointe(ltf,nom_t)
      if (.not.associated(tcv%lv_t(i_v_t)%tf)) then
	print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
	call lecture_stop(f)      
      end if
      i_v_t = i_v_t + 1
    end if
    !--- Remplissage Structure en temps ---
    if (flag_f) then
      call resultat_champ_unvol_copy(vin,tcv%lv_f(i_v_f))
      ! Récupère les pointeurs sur la structure temps-frequence
      tcv%lv_f(i_v_f)%tf => liste_temps_freq_pointe(ltf,nom_f)
      if (.not.associated(tcv%lv_f(i_v_f)%tf)) then
	print '(/,t10,"Nom du type de stockage en fréquence erroné !!!")'
	call lecture_stop(f)      
      end if
      i_v_f = i_v_f + 1
    end if
    !--- Récupère les données pour l'écriture ---
    ires=ires+1
    pres=>tcv%l_res(ires)
    pres%nom  = vin%nom
    pres%interv1 = nom1
    if (flag_t .and. flag_f) pres%interv2 = nom2
    pres%tf_commutateur = vin%com_format_tf;
    pres%nbc = vin%nbc
    pres%nom_eh(:) = vin%nom_eh(:)
    pres%icoord= vin%coord;pres%ipas  = vin%pas
    pres%sn    = vin%sn
    pres%ch_type_calcul = ch_type_calcul
    if (rang==RANG_MAITRE_AFF) &
      print '(T10, "  --> nbre d''échantillons = ",3I5)',product(vin%nb_pt)
  !----------------------------------------------------------------------------
  else if (premiermot=='type') then
    nbmot = nb_mot_chaine(f%chaine);
    ch_type_calcul = extrait_sous_chaine(f%chaine,2,nbmot)
    vin%nb_type_calcul = nbmot-1
    allocate(vin%type_calcul(vin%nb_type_calcul,2))
    vin%type_calcul = 0
    chaine = ch_type_calcul
    do i=1,nbmot-1
      read(chaine,*) nom1
      nom1=adjustl(nom1)
      select case (nom1)
        case('moy','moy-1','moy1','moy2','moy3','moy4')
	  vin%type_calcul(i,1)=VOL_CALC_MOY
	  vin%type_calcul(i,2)=1
	  if (nom1(4:5)/='') read(nom1(4:5),*) vin%type_calcul(i,2)
        case('sum','sum-1','sum1','sum2','sum3','sum4')
	  vin%type_calcul(i,1)=VOL_CALC_SUM
	  vin%type_calcul(i,2)=1
	  if (nom1(4:5)/='') read(nom1(4:5),*) vin%type_calcul(i,2)
	case('moyc')
	  vin%type_calcul(i,1)=VOL_CALC_MOYC
	  vin%type_calcul(i,2)=1
	case('sumc')
	  vin%type_calcul(i,1)=VOL_CALC_SUMC
	  vin%type_calcul(i,2)=1
        case('max')
	  vin%type_calcul(i,1)=VOL_CALC_MAX
        case('var')
	  vin%type_calcul(i,1)=VOL_CALC_VAR
        case('sigma')
	  vin%type_calcul(i,1)=VOL_CALC_SIGMA
	case default
	    print '(/,t10,"Erreur, nom=",A," ce type de résultats est erroné !!!")',trim(nom1)
	    call lecture_stop(f)      	  
      end select
      chaine=extrait_sous_chaine(chaine,2,nbmot-i)
    end do

  !----------------------------------------------------------------------------
  else if (sortie_normalise_lecture(f,premiermot,vin%sn)) then
    cycle
  !else if (fichier_sortie_lect_format_freq(premiermot,vin%com_format_tf)) then
  !  cycle
  !----------------------------------------------------------------------------
  else if (f%nomfinbloc == premiermot) then
    exit ! fin du bloc    
  else
    print *,'(/,t10,"Mot clé non reconnu  : ",A )', TRIM(premiermot)
    call lecture_stop(f)
  end if 
enddo

return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(f)

end subroutine resultat_champ_vol_lecture


! ==========================================================
! == Subroutine resultat_champ_vol                        ==
! ==========================================================
subroutine resultat_champ_vol_ecriture(f,tcv,tm)

type(st_fichier), intent(inout) :: f
type(st_resultat_champ_volume), intent(inout) :: tcv
type(st_maillage), intent(in)   :: tm

integer :: i,j,commutateur
integer :: coordcell(6)
character(LEN=1),dimension(1:3) :: ch1
character(LEN=LEN_MOT_CLE) :: ch,ch2
character(LEN=LEN_CHAINE) :: ch_type_calcul

type(st_resultat_param_volume), pointer :: tpv
type(st_resultat_champ_vol_ES), pointer :: pres
type(st_sortie_normalise) :: sn


if (.not.tcv%flag_data) return
commutateur = REELLE_IMAGINAIRE
call sortie_normalise_initdefaut(sn)
ch_type_calcul=''

!--------- Ecriture pour les sorties basées sur les composantes de champ ------

do i=1,tcv%n_res
  pres =>tcv%l_res(i)
  if (.not.sortie_normalise_compare(sn,pres%sn)) then
    call sortie_normalise_ecriture(f,pres%sn)
    sn = pres%sn
  end if
  
  if (commutateur/=pres%tf_commutateur) then
    call fichier_sortie_ecrit_format(f,pres%tf_commutateur)
    commutateur = pres%tf_commutateur
  end if
  if (ch_type_calcul/=pres%ch_type_calcul) then
    write(f%unite,'(T3,A,3x,A)') 'type',trim(pres%ch_type_calcul)
    ch_type_calcul=pres%ch_type_calcul
  end if
  ch2 = '';
  do j=1,pres%nbc
    ch2 = trim(ch2)//' '//TRIM(pres%nom_eh(j))
  end do
  ch = trim(pres%interv1)//' '//trim(pres%interv2)
  if (pres%nom(1:2) == 'VU') then
      coordcell = origine_cellule6(tm,pres%icoord,.true.)
      write(f%unite,'(T5,A,3x,A,3x,A,3x,6I5,3I4)') trim(pres%nom),trim(ch),&
                                             trim(ch2),coordcell(1:6),pres%ipas(1:3)
  end if
end do

write(f%unite,*)
!--------------------- Ecriture pour les paramètres epsilon et sigma ----------
ch1(1) = 'x' ; ch1(2) = 'y' ; ch1(3) = 'z'

do i=1,tcv%nb_eps
  tpv =>tcv%l_eps(i)
  ch = ''
  do j=1,3
    if (tpv%flag_dir(j)) ch = trim(ch)//' '//trim(ch1(j))
  end do
  coordcell = origine_cellule6(tm,tpv%coord,.true.)
  write(f%unite,'(T3,A,1x,A,1x,6I5,1x,3I4)')  trim(tpv%nom),trim(ch),coordcell(1:6),tpv%pas(1:3)  
end do

do i=1,tcv%nb_sige
  tpv =>tcv%l_sige(i)
  ch = ''
  do j=1,3
    if (tpv%flag_dir(j)) ch = trim(ch)//' '//trim(ch1(j))
  end do
  coordcell = origine_cellule6(tm,tpv%coord,.true.)
  write(f%unite,'(T3,A,1x,A,1x,6I5,1x,3I4)')  trim(tpv%nom),trim(ch),coordcell(1:6),tpv%pas(1:3)  
end do

end subroutine resultat_champ_vol_ecriture

!===========================================================	
! Subroutine resultat_champ_vol_init
! Initialisation des variables, 
!===========================================================	
Subroutine resultat_champ_vol_init(tcv)
!=== Déclaration des arguments ===
type(st_resultat_champ_volume), intent(inout) :: tcv

!===  Déclaration des variables  ===

!===   Début des traitements   ===

				
End Subroutine resultat_champ_vol_init

!===========================================================	
! Subroutine resultat_champ_vol_precalcul1
! Calculs avant les itérations temporelles
!===========================================================	
subroutine resultat_champ_vol_precalcul1(tcv,tmv,tfs)
!=== Déclaration des arguments ===
type(st_resultat_champ_volume), intent(inout) :: tcv
type(st_milieu_volume), intent(in)  :: tmv
type(st_fichier_sortie), intent(in) :: tfs

!===  Déclaration des variables  ===
type(st_resultat_param_volume), pointer :: tpv

integer :: i
!===   Début des traitements   ===

tcv%mantisse_reel   = tfs%mantisse_reel
tcv%mantisse_entier = tfs%mantisse_entier

!----------------- gestion des sorties sur epsilon et sigma -------------------
do i = 1,tcv%nb_eps
  tpv => tcv%l_eps(i)
  call resultat_vol_ecrit_param(tpv,tmv,tfs,1)
end do
do i = 1,tcv%nb_sige
  tpv => tcv%l_sige(i)
  call resultat_vol_ecrit_param(tpv,tmv,tfs,2)
end do
  
end subroutine resultat_champ_vol_precalcul1

!===========================================================	
! Subroutine resultat_champ_vol_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
subroutine resultat_champ_vol_precalcul2(tcv,tc,tfs)
!=== Déclaration des arguments ===
type(st_resultat_champ_volume), intent(inout) :: tcv
type(st_champ), intent(in)   :: tc
type(st_fichier_sortie), intent(in) :: tfs

!===  Déclaration des variables  ===
type(st_resultat_champ_un_volume),pointer :: tcp

integer :: i,j,k,n
integer :: n1,n2
integer,dimension(20,2) :: type1,type2
character(LEN=2) :: c
character(LEN=LEN_MOT_CLE) :: nomfichier
!===   Début des traitements   ===

tcv%mantisse_reel   = tfs%mantisse_reel
tcv%mantisse_entier = tfs%mantisse_entier

! Alias
tcv%ex=>tc%ex
tcv%ey=>tc%ey
tcv%ez=>tc%ez
tcv%hx=>tc%hx
tcv%hy=>tc%hy
tcv%hz=>tc%hz

!-------------------------- stockage en temps ----------------------------------
do i = 1,tcv%nb_v_t
  tcp => tcv%lv_t(i)
  !print '(T10,A," temp, type =",10I2)',trim(tcp%nom),tcp%type_calcul(1:tcp%nb_type_calcul,1:2)
  ! Creation des alias sur les composantes de champ
  call resultat_champ_vol_alias(tcp,tc)
  allocate(tcp%res_t(tcp%nb_type_calcul,tcp%nbc,2))
  tcp%res_t=0.0
  tcp%nb_ech = product(tcp%nb_pt)
  ! création du format d'écriture en temps
  write(c,'(I2)') tcp%nb_type_calcul*tcp%nbc+1
  tcp%mantisse = '(' //trim(c)//trim(tcv%mantisse_reel)//')'
  
  do j=1,tcp%nb_type_calcul
    if (tcp%type_calcul(j,1)==VOL_CALC_MOYC) tcp%type_calcul(j,1)=VOL_CALC_MOY
    if (tcp%type_calcul(j,1)==VOL_CALC_SUMC) tcp%type_calcul(j,1)=VOL_CALC_SUM
  end do
  !print '(T10,A,"  , type =",10I2)',trim(tcp%nom),tcp%type_calcul(1:tcp%nb_type_calcul,1:2)
  ! ouverture du fichier si format asccii
  if (tfs%format_fichier == FORMAT_ASCII) then   
    nomfichier = fichier_sortie_nom_temps(tfs,tcp%nom)
    tcp%unite = ouvre_en_ecriture(nomfichier)    				    
  end if
  
end do				

!--------------------------- stockage en frequence ----------------------------

do i = 1,tcv%nb_v_f
  tcp => tcv%lv_f(i)
  ! Creation des alias sur les composantes de champ
  !print '(T10,A,"  freq, type =",10I2)',trim(tcp%nom),tcp%type_calcul(1:tcp%nb_type_calcul,1:2)
  call resultat_champ_vol_alias(tcp,tc)
  
  ! création du format d'écriture en fréquence
  tcp%nb_ech = product(tcp%nb_pt)
  ! création du format d'écriture en temps
  write(c,'(I2)') tcp%nb_type_calcul*tcp%nbc*2+1
  tcp%mantisse = '(' //trim(c)//trim(tcv%mantisse_reel)//')'

  n1 = 0;n2=0
  do j=1,tcp%nb_type_calcul
    if (tcp%type_calcul(j,1)==VOL_CALC_MOYC .or.&
        tcp%type_calcul(j,1)==VOL_CALC_SUMC) then
      n2=n2+1
      type2(n2,1:2) = tcp%type_calcul(j,1:2)
      tcp%flag_fmoy = .true.
    else
      n1=n1+1
      type1(n1,1:2) = tcp%type_calcul(j,1:2)
      tcp%flag_f = .true.
    end if
  end do
  tcp%nb_type_calcul= n1
  tcp%nb_type_calcul_fmoy= n2
  


  deallocate(tcp%type_calcul)
  !-- stockage en fréquence
  if (tcp%flag_f) then
    allocate(tcp%type_calcul(1:n1,1:2))
    tcp%type_calcul(1:n1,1:2) = type1(1:n1,1:2)
    allocate(tcp%eh_f(1:tcp%nb_pt(1),1:tcp%nb_pt(2),1:tcp%nb_pt(3),1:tcp%tf%n_liste,1:tcp%nbc))   
    ! Initialisation du tableau sans boucle peut ne pas passer sur 
    ! certaine machine tel que Uqbar
    do k=1,tcp%nbc
    do j=1,tcp%tf%n_liste
      tcp%eh_f(:,:,:,j,k) = CMPLX(0.0,0.0)          
    end do;end do;
  end if
  if (tcp%flag_fmoy) then
    allocate(tcp%type_calcul_fmoy(1:n2,1:2))
    tcp%type_calcul_fmoy(1:n2,1:2) = type2(1:n2,1:2)
    allocate(tcp%eh_fmoy(1:tcp%tf%n_liste,1:tcp%nbc))
    tcp%eh_fmoy = cmplx(0.0,0.0)
  end if
end do				
 
end subroutine resultat_champ_vol_precalcul2

!===========================================================	
! Subroutine resultat_champ_vol_alias
! 
!===========================================================	
subroutine resultat_champ_vol_alias(tcp,tc)
!=== Déclaration des arguments ===
type(st_resultat_champ_un_volume), intent(inout) :: tcp
type(st_champ), intent(in)   :: tc

!=== Déclaration des variables ===
integer :: j
!===   Début des traitements   ===
!tcp%flag_e = .false.
!tcp%flag_h = .false.
!tcp%flag_compo = .true.
do j = 1,tcp%nbc
  ! tableau d'alias vers les composantes à stocker
  if (tcp%nom_eh(j) == LISTE_EH(1)) then
     tcp%p(j)%eh => tc%ex      
  else if (tcp%nom_eh(j) == LISTE_EH(2)) then
     tcp%p(j)%eh => tc%ey      
  else if (tcp%nom_eh(j) == LISTE_EH(3)) then
     tcp%p(j)%eh => tc%ez      
  else if (tcp%nom_eh(j) == LISTE_EH(4)) then
     tcp%p(j)%eh => tc%hx      
  else if (tcp%nom_eh(j) == LISTE_EH(5)) then
     tcp%p(j)%eh => tc%hy      
  else if (tcp%nom_eh(j) == LISTE_EH(6)) then
     tcp%p(j)%eh => tc%hz      
!  else if (tcp%nom_eh(j) == LISTE_EH(7)) then
!     tcp%flag_e = .true.
!     tcp%flag_compo(j) = .false.
!     cycle
!  else if (tcp%nom_eh(j) == LISTE_EH(8)) then
!     tcp%flag_h = .true.
!     tcp%flag_compo(j) = .false.
!     cycle
  end if
end do
end subroutine resultat_champ_vol_alias


!===========================================================	
! Subroutine resultat_vol_ecrit_param
! 
!===========================================================	
subroutine resultat_vol_ecrit_param(tpv,tmv,tfs,flagparam)
!=== Déclaration des arguments ===
type(st_resultat_param_volume), intent(inout) :: tpv
type(st_milieu_volume), intent(in)  :: tmv
type(st_fichier_sortie), intent(in)   :: tfs
integer, intent(in)  :: flagparam

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nomfichier
character(LEN=LEN_MOT_CLE) :: mantisse
character(LEN=5) :: c
integer, dimension(3) :: nbpt
integer  :: i,j,k,n,unite
real, dimension(:,:,:), pointer :: param
!===   Début des traitements   ===

! création du format d'écriture
nbpt(1:3) = (tpv%coord(2:6:2)-tpv%coord(1:5:2))/tpv%pas(1:3)+1
if (nbpt(1) > 1) then
  write(c,'(I5)') nbpt(1)
else
  write(c,'(I5)') nbpt(2)  
end if
mantisse = '(' //c(1:LEN_TRIM(c))// tfs%mantisse_reel(1:LEN_TRIM(tfs%mantisse_reel))//')'
!print '("mantisse = ",A)',mantisse

! ouverture du fichier si format asccii
if (tfs%format_fichier == FORMAT_ASCII) then   
  nomfichier = TRIM(tpv%nom) // TRIM(tfs%extension)
  
  unite = ouvre_en_ecriture(nomfichier)  
  
  ! Ecriture dans le fichier    
  do n=1,3
    if (tpv%flag_dir(n)) then
      select case (n)
	case(1)
	  if (flagparam==1) param =>tmv%epsx
	  if (flagparam==2) param =>tmv%sigex
	case(2)
	  if (flagparam==1) param =>tmv%epsy
	  if (flagparam==2) param =>tmv%sigey
	case(3)
	  if (flagparam==1) param =>tmv%epsz
	  if (flagparam==2) param =>tmv%sigez
      end select
    else
      cycle
    end if
!    print '("coord = ",6I)',tpv%coord
!    print '("pas = ",3I)',tpv%pas
!    print '("nbpt = ",3I)',nbpt
    do k=tpv%coord(5),tpv%coord(6),tpv%pas(3)
      if (nbpt(1)>1) then
	do j=tpv%coord(3),tpv%coord(4),tpv%pas(2)
!          print '("param = ",50F)',param(tpv%coord(1):tpv%coord(2):tpv%pas(1),j,k)
	  write(unite,mantisse) param(tpv%coord(1):tpv%coord(2):tpv%pas(1),j,k)
	end do
      else
	i = tpv%coord(1)
	write(unite,mantisse) param(i,tpv%coord(3):tpv%coord(4):tpv%pas(2),k)
      end if
    end do
  end do
  
  close(unite)
end if
  

end Subroutine resultat_vol_ecrit_param

!===========================================================	
! Subroutine resultat_champ_vol_calcul
!  Ecriture des résultats à chaque iteration temporelle
! dans l'intervalle temporel spéccifié
!===========================================================	
subroutine resultat_champ_vol_calcul(tcv,tfs)
!=== Déclaration des arguments ===
type(st_resultat_champ_volume), intent(inout) :: tcv
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
integer :: i,j,k,m1,m2,n,p
integer :: i1,j1,k1
integer :: type_calcul
real    :: somme,a
real, dimension(:,:), pointer  :: peh 
type(st_resultat_champ_un_volume),pointer :: tcp

!===   Début des traitements   ===

!--- boucle sur toutes les volume de stockage en temps ---
do n = 1,tcv%nb_v_t
  tcp => tcv%lv_t(n)
  if (.not.tcp%tf%flag_calcul) cycle ! cas ou on ne stocke pas      
  ! boucle sur toutes les composantes à stocker
  tcp%res_t=0.0
  do m1=1,tcp%nb_type_calcul
    p=tcp%type_calcul(m1,2)
    type_calcul =tcp%type_calcul(m1,1) 
    select case(type_calcul)
    !------------------------------------------------------
    case (VOL_CALC_MOY,VOL_CALC_SUM)
      do m2=1,tcp%nbc
	!if (tcp%flag_compo(m2)) then
	  do k=tcp%coord(5),tcp%coord(6),tcp%pas(3)
	  do j=tcp%coord(3),tcp%coord(4),tcp%pas(2)
	  do i=tcp%coord(1),tcp%coord(2),tcp%pas(1)
	    tcp%res_t(m1,m2,1) = tcp%res_t(m1,m2,1)+(tcp%p(m2)%eh(i,j,k)*tcp%tf%filtre%c_t)**p
	  end do;end do;end do
	!end if
      end do 
      if (type_calcul==VOL_CALC_MOY) then
        tcp%res_t(m1,:,1)=(tcp%res_t(m1,:,1)/tcp%nb_ech)**(1.0/p)
      end if
    !------------------------------------------------------
    case (VOL_CALC_MAX)
      do m2=1,tcp%nbc
	!if (tcp%flag_compo(m2)) then
	  do k=tcp%coord(5),tcp%coord(6),tcp%pas(3)
	  do j=tcp%coord(3),tcp%coord(4),tcp%pas(2)
	  do i=tcp%coord(1),tcp%coord(2),tcp%pas(1)
	    tcp%res_t(m1,m2,1) = MAX(tcp%res_t(m1,m2,1),tcp%p(m2)%eh(i,j,k)*tcp%tf%filtre%c_t)
	  end do;end do;end do
	!end if
      end do 
    !------------------------------------------------------
    case (VOL_CALC_VAR,VOL_CALC_SIGMA)
      do m2=1,tcp%nbc
	!if (tcp%flag_compo(m2)) then
	  do k=tcp%coord(5),tcp%coord(6),tcp%pas(3)
	  do j=tcp%coord(3),tcp%coord(4),tcp%pas(2)
	  do i=tcp%coord(1),tcp%coord(2),tcp%pas(1)
	    tcp%res_t(m1,m2,1) = tcp%res_t(m1,m2,1)+tcp%p(m2)%eh(i,j,k) *tcp%tf%filtre%c_t
	    tcp%res_t(m1,m2,2) = tcp%res_t(m1,m2,2)+(tcp%p(m2)%eh(i,j,k)*tcp%tf%filtre%c_t)**2
	  end do;end do;end do
	!end if
      end do 
      tcp%res_t(m1,:,1:2)=tcp%res_t(m1,:,1:2)/tcp%nb_ech
      tcp%res_t(m1,:,1)  =tcp%res_t(m1,:,2)-tcp%res_t(m1,:,1)**2
      if (type_calcul==VOL_CALC_SIGMA) then
        do m2=1,tcp%nbc
          tcp%res_t(m1,m2,1) = sqrt(abs(tcp%res_t(m1,m2,1)))
	end do
      end if
    end select
  end do
  !------------------ Ecriture des résultats dans le fichier ------------------
  write (tcp%unite,tcp%mantisse)  tcp%tf%liste(tcp%tf%curseur),&
             tcp%res_t(1:tcp%nb_type_calcul,1:tcp%nbc,1)          

end do 
 
!--- boucle sur les surfaces de stockage en fréquence ---
do n = 1,tcv%nb_v_f
  tcp => tcv%lv_f(n)
  if (.not.tcp%tf%flag_calcul) cycle
  
  !----------------------------------------------------------------------------
  if (tcp%flag_f) then
    do m2 = 1, tcp%nbc  ! boucle sur les composantes
      !if (tcp%flag_compo(m2)) then
	do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
	    k1=0
	    do k=tcp%coord(5),tcp%coord(6),tcp%pas(3)
	      k1=k1+1;j1=0
	    do j=tcp%coord(3),tcp%coord(4),tcp%pas(2)
	      j1=j1+1;i1=0
	    do i=tcp%coord(1),tcp%coord(2),tcp%pas(1)
	      i1=i1+1
	      tcp%eh_f(i1,j1,k1,m1,m2) = tcp%eh_f(i1,j1,k1,m1,m2) + tcp%tf%expi2pitf(m1) * &
			    tcp%p(m2)%eh(i,j,k) 
	      !print '("(i1,j1,k1)=",3I5)',i1,j1,k1
	  end do; end do;end do
	end do
      !end if
  
    end do
  end if
  !----------------------------------------------------------------------------
  if (tcp%flag_fmoy) then
    do m2 = 1, tcp%nbc  ! boucle sur les composantes
      !if (tcp%flag_compo(m2)) then
        somme =0.0
	do k=tcp%coord(5),tcp%coord(6),tcp%pas(3)
	do j=tcp%coord(3),tcp%coord(4),tcp%pas(2)
	do i=tcp%coord(1),tcp%coord(2),tcp%pas(1)
	  somme = somme+tcp%p(m2)%eh(i,j,k)
	end do;end do;end do
	do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
	      tcp%eh_fmoy(m1,m2) = tcp%eh_fmoy(m1,m2) + tcp%tf%expi2pitf(m1) * somme 
	end do
      !end if  
    end do
  end if
  !----------------------------------------------------------------------------  
end do

end Subroutine resultat_champ_vol_calcul



!===========================================================	
! Subroutine resultat_champ_surf_impression
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_champ_vol_impression(tcv,tfs,t_gene)

!=== Déclaration des arguments ===
type(st_resultat_champ_volume), intent(inout) :: tcv
type(st_fichier_sortie), intent(in) :: tfs
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer  :: i,j,k,n,m,m1,m2,p
integer  :: type_calcul
type(st_resultat_champ_un_volume), pointer :: tcp
character(LEN=LEN_MOT_CLE) :: nomfichier
complex, dimension(:), allocatable   :: tnorm_f
real,  dimension(:,:,:,:), allocatable      :: res_f1
complex,  dimension(:,:,:,:), allocatable   :: res_f2
!===   Début des traitements   ===

!--- boucle sur les surfaces de stockage en temps ---
do i = 1,tcv%nb_v_t
  tcp => tcv%lv_t(i)
  close(tcp%unite)    				    
end do  

!--- boucle sur les surfaces de stockage en frequence ---
do n = 1,tcv%nb_v_f
  tcp => tcv%lv_f(n)
  !---------- Normalisation ?-----------------
  if (tcp%sn%flag_normalisation) then
    allocate(tnorm_f(1:tcp%tf%n_liste))
    call Normalise_Reponse_Freq(t_gene,tcp%tf,tcp%sn,tnorm_f)
    ! normalisation pour chaque fréquence
    if (tcp%flag_f) then
      do m2 = 1, tcp%nbc  ! boucle sur les composantes
      do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
	do k=1,tcp%nb_pt(3)
	do j=1,tcp%nb_pt(2)
	do i=1,tcp%nb_pt(1)
	    tcp%eh_f(i,j,k,m1,m2) = tcp%eh_f(i,j,k,m1,m2) / tnorm_f(m1)      
	end do;end do;end do;
      end do;end do;
    end if
    if (tcp%flag_fmoy) then
      do m2 = 1, tcp%nbc  ! boucle sur les composantes
      do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
	    tcp%eh_fmoy(m1,m2) = tcp%eh_fmoy(m1,m2) / tnorm_f(m1)      
      end do;end do;
    end if
    deallocate(tnorm_f)
  end if
  
  !----------------------------------------------------------------------------
  !----------------------------------------------------------------------------
  if (tcp%flag_f) then
    allocate(res_f1(tcp%nb_type_calcul,tcp%nbc,tcp%tf%n_liste,2))
    res_f1=0.0
    ! boucle sur toutes les composantes à stocker
    do m=1,tcp%nb_type_calcul
      p=tcp%type_calcul(m,2)
      type_calcul =tcp%type_calcul(m,1) 
      select case(type_calcul)
      !------------------------------------------------------
      case (VOL_CALC_MOY,VOL_CALC_SUM)
	do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
	do m2=1,tcp%nbc
	  do k=1,tcp%nb_pt(3)
	  do j=1,tcp%nb_pt(2)
	  do i=1,tcp%nb_pt(1)
	      res_f1(m,m2,m1,1) = res_f1(m,m2,m1,1)+abs(tcp%eh_f(i,j,k,m1,m2))**p
	  end do;end do;end do
	end do; end do
	if (type_calcul==VOL_CALC_MOY) then
	  res_f1(m,:,:,1)=(res_f1(m,:,:,1)/tcp%nb_ech)**(1.0/p)
	end if
      !------------------------------------------------------
      case (VOL_CALC_MAX)
	do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
	do m2=1,tcp%nbc
	  do k=1,tcp%nb_pt(3)
	  do j=1,tcp%nb_pt(2)
	  do i=1,tcp%nb_pt(1)
	      res_f1(m,m2,m1,1) = max(res_f1(m,m2,m1,1),abs(tcp%eh_f(i,j,k,m1,m2)))
	  end do;end do;end do
	end do; end do
      !------------------------------------------------------
      case (VOL_CALC_VAR,VOL_CALC_SIGMA)
	do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
	do m2=1,tcp%nbc
	  do k=1,tcp%nb_pt(3)
	  do j=1,tcp%nb_pt(2)
	  do i=1,tcp%nb_pt(1)
	      res_f1(m,m2,m1,1) = res_f1(m,m2,m1,1)+abs(tcp%eh_f(i,j,k,m1,m2))
	      res_f1(m,m2,m1,2) = res_f1(m,m2,m1,2)+abs(tcp%eh_f(i,j,k,m1,m2))**2
	  end do;end do;end do
	end do; end do
	res_f1(m,:,:,1:2)=res_f1(m,:,:,1:2)/tcp%nb_ech
	res_f1(m,:,:,1)  =res_f1(m,:,:,2)-res_f1(m,:,:,1)**2
	if (type_calcul==VOL_CALC_SIGMA) res_f1(m,:,:,1) = sqrt(abs(res_f1(m,:,:,1)))
      end select
    end do
  end if
  
  !----------------------------------------------------------------------------
  !----------------------------------------------------------------------------
  if (tcp%flag_fmoy) then
    allocate(res_f2(tcp%nb_type_calcul_fmoy,tcp%nbc,tcp%tf%n_liste,2))
    res_f2=cmplx(0.0,0.0)
    ! boucle sur toutes les composantes à stocker
    do m=1,tcp%nb_type_calcul_fmoy
      p=tcp%type_calcul_fmoy(m,2)
      type_calcul =tcp%type_calcul_fmoy(m,1) 
      select case(type_calcul)
      !------------------------------------------------------
      case (VOL_CALC_MOYC,VOL_CALC_SUMC)
	do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
	do m2=1,tcp%nbc
	      res_f2(m,m2,m1,1) = res_f2(m,m2,m1,1)+tcp%eh_fmoy(m1,m2)
	end do; end do
	if (type_calcul==VOL_CALC_MOYC) then
	  res_f2(m,:,:,1)=res_f2(m,:,:,1)/tcp%nb_ech
	end if
      end select
    end do
  end if
  
  !------------ Ecriture dans le fichier ------------------------
  if (tcp%flag_f) then
    nomfichier = fichier_sortie_nom_freq(tfs,tcp%nom)
    tcp%unite = ouvre_en_ecriture(nomfichier)    				    
    do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
      write (tcp%unite,tcp%mantisse)  tcp%tf%liste(m1),&
	    real(res_f1(1:tcp%nb_type_calcul,1:tcp%nbc,m1,1))
	!    abs(tcp%eh_f(1,1,1,m1,1:tcp%nbc))
    end do  
    deallocate(res_f1)
    close(tcp%unite)    				    
  end if
  !------------ Ecriture dans le fichier ------------------------
  if (tcp%flag_fmoy) then
    !--> ouverture du fichier
    nomfichier=trim(tcp%nom)//'C';
    nomfichier = fichier_sortie_nom_freq(tfs,nomfichier)
    tcp%unite = ouvre_en_ecriture(nomfichier)    				    
    do m1 = 1, tcp%tf%n_liste  ! boucle sur les frequences
      write (tcp%unite,tcp%mantisse)  tcp%tf%liste(m1),&
	     res_f2(1:tcp%nb_type_calcul_fmoy,1:tcp%nbc,m1,1)
    end do
    deallocate(res_f2)
    close(tcp%unite) 
  end if    
end do				

				
End Subroutine resultat_champ_vol_impression

!===========================================================	
! Subroutine resultat_champ_vol_deallocate
!  Libération de toute la mémoire allouée
!===========================================================	
subroutine resultat_champ_vol_deallocate(tcv)
!=== Déclaration des arguments ===
type(st_resultat_champ_volume), intent(inout) :: tcv

!===  Déclaration des variables  ===
integer :: i,j
type(st_resultat_champ_un_volume),pointer :: tcp

!=== Debut des traitements ===

if (tcv%nb_eps>0) deallocate(tcv%l_eps)
if (tcv%nb_sige>0) deallocate(tcv%l_sige)
tcv%nb_eps = 0
tcv%nb_sige = 0

do i =1, tcv%nb_v_t
  tcp => tcv%lv_t(i)
  deallocate(tcp%type_calcul)
  deallocate(tcp%res_t)
  do j=1,6
    nullify(tcp%p(j)%eh)
  end do
end do			
do i =1, tcv%nb_v_f
  tcp => tcv%lv_f(i)
  if (ASSOCIATED(tcp%eh_f)) deallocate(tcp%eh_f)
  if (tcp%flag_f) deallocate(tcp%type_calcul)  
  if (tcp%flag_fmoy) then
    deallocate(tcp%type_calcul_fmoy)
    deallocate(tcp%eh_fmoy)
  end if
  do j=1,6
    nullify(tcp%p(j)%eh)
  end do
end do			

if (tcv%nb_v_t>0) deallocate(tcv%lv_t)
if (tcv%nb_v_f>0) deallocate(tcv%lv_f)
if (tcv%n_res>0 ) deallocate(tcv%l_res)
end subroutine resultat_champ_vol_deallocate	

end module mfdtd_resultat_champ_volume


