!////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_resultat_structure_filf90
!// Purpose:      Résultat sur les structures filaires
!// Author:      Christophe Guiffaut
!// Modified by:
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :   Février 2004
!// Copyright:   (c) Christophe Guiffaut, équipe CNRS IRCOM/DEM
!//              Utilisation des sources réservées à l'IRCOM
!// Licence  :
!////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Module resultat_structure_fil                    ==
! == Calcul et stockage des resultats correspondant aux   ==
! == structures structure_filaires                                  ==
! ==========================================================

module mfdtd_resultat_structure_fil


use m_constantes, only : LEN_CHAINE,LEN_MOT_CLE
use m_es_identifiant, only : MANTISSE_REEL,MANTISSE_ENTIER,FORMAT_ASCII,&
                             REELLE_IMAGINAIRE,SUFFIXE_TEMP,SUFFIXE_FREQ
use m_entree, only : st_fichier, &
                    lineread, &
		    ligne_precedente, &
		    lecture_stop, &
		    len_int, &
		    nb_mot_chaine, &
		    ctrl_nb_mot_chaine, &
		    unitefichier, &
		    multibackspace, &
		    numbackspace_set, &
		    entier_chaine, &
		    extrait_sous_chaine
			
				 
use m_math_operation, only : atan2d_cor
use m_sortie, only : st_fichier_sortie, &
		    fichier_sortie_lect_format_freq, &
		    impression_ascii_complexe, &
		    fichier_sortie_nom_temps, &
		    fichier_sortie_ecrit_format, &
		    fichier_sortie_nom_freq, &
		    ouvre_en_ecriture
		

use m_temps_frequence, only : st_temps_frequence, &
                              st_liste_temps_freq, &
			      liste_temps_freq_pointe, &
			      temps_freq_compteur_incr, &
			      identifie_deux_nom, &
			      INTERVALLE_TEMPOREL, &
			      INTERVALLE_FREQUENTIEL, &
			      LISTE_TEMPORELLE, &
			      LISTE_FREQUENTIELLE

use m_generateur, only : st_generateur
use m_sortie_normalise, only : st_sortie_normalise, &
                                  sortie_normalise_initdefaut,&
				  sortie_normalise_lecture   ,&
				  sortie_normalise_ecriture ,&
				  sortie_normalise_compare, &
				  Normalise_Reponse_Freq

use m_commun_identifiant
use mfdtd_volume, only : st_volume
use mfdtd_structure_fil, only : st_structure_fil, &
                                 st_fil_segment, &
				 st_unfil, &
                                 verif_segment_fil, &
                                 verif_nom_fil, &
				 verif_nom_gene_fil


implicit none

! ==========================================================
! == Definition des differents types                      ==
! ==                                                      ==
! == Type st_lieu : nom du fil, position sur le fil       ==
! == Type st_resultat_structure_fil : caracteristiques          ==
! ==  d'un resultat fil                               ==
! == Type st_resultat_structure_fil : ensemble des resultat     ==
! ==  toron_filaire                                             ==
! ==========================================================
type st_pointe_param
  real, pointer  :: valeur
  real, pointer  :: c_val
end type st_pointe_param

type st_lieu
  character(LEN=LEN_MOT_CLE) :: nomfil
  ! -- pointeur sur cur ou qc correspondant suivant "i_ou_q" --
  type(st_pointe_param), dimension(:), pointer :: p
  ! -- numfil : numero du fil dans la liste des fils --
  ! -- deb : segment de debut --
  ! -- fin : segment de fin --
  ! -- pas : pas entre deb fin --
  ! -- nb_segment : le nombre de segment ---
  integer :: numfil, deb, fin, pas
  integer :: nb_segment 

end type st_lieu

type st_resultat_un_fil
  ! -- nom : nom de la commande lue dans le fichier de donnees --
  character(LEN=LEN_MOT_CLE) :: nom
  ! -- nom_fichier : nom du fichier de resultat --
  character(LEN=LEN_MOT_CLE) :: nom_fichier

  ! -- ou : numero de fil, de (debut fin pas) segments --
  type (st_lieu) :: ou
  
  integer :: id_fichier ! unité fichier en temps ou en fréquence
  
  ! -- formate : chaine de formatage pour les sortie fichiers ascii --
  character(LEN=LEN_MOT_CLE) :: formate
  ! -- tf_commutateur : commutateur pour les tf 
  integer :: tf_commutateur
   
  ! -- lieu temporel ou frequentiel --
  type(st_temps_frequence), pointer :: tf

  logical :: flag_cor
  logical :: flag_eval_fin_simu
  ! -- tableaux pour le stockage des composantes en frequence --
  real  :: rg
  real,    dimension(:),   pointer :: val_t, val_tprecedent
  complex, dimension(:,:), pointer :: val_f
    ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn

end type st_resultat_un_fil

!------------------------------------------------------------------------------
type st_resultat_un_fil_ES ! données directes pour les entrées sorties
   character(LEN=LEN_MOT_CLE) :: nom
   character(LEN=LEN_MOT_CLE) :: interv1,interv2
   character(LEN=LEN_MOT_CLE) :: nom_objet ! fil ou générateur
   character(LEN=LEN_MOT_CLE) :: nom_objet2 
   integer   :: deb,fin,pas
   integer   :: tf_commutateur
   logical   :: flag_eval_fin_simu
   ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
   
end type st_resultat_un_fil_ES

!------------------------------------------------------------------------------
type st_resultat_structure_fil
  logical  :: flag_data
  real, pointer :: dt
  ! -- Resultat en temporel --
  ! -- n_it : nombre de resultats en courant --
  ! -- n_qt : nombre de resultats en charges --
  integer :: n_it, n_qt, n_vt
  ! -- Liste des resultats temporels --
  type(st_resultat_un_fil), dimension(:), pointer :: l_it, l_qt, l_vt
  ! -- Resultat en frequentiel --
  ! -- n_if : nombre de resultats en courant --
  ! -- n_qf : nombre de resultats en charges --
  ! -- n_zf : nombre de resultats en impedance --
  integer :: n_if, n_qf, n_vf, n_zf
  ! -- Listes des resultats frequentiels --
  type(st_resultat_un_fil), dimension(:), pointer :: l_if, l_qf, l_vf, l_zf

  ! -- Chaines de formatage dans le cas ascii --
  character(LEN=LEN_MOT_CLE) :: mantisse_entier, mantisse_reel
  
  !--  données directes pour les entrées sorties --
  integer :: n_res
  type(st_resultat_un_fil_ES), pointer, dimension(:) :: l_res
end type st_resultat_structure_fil

contains


! ==========================================================
! == Subroutine resultat_structure_fil_initdefaut     ==
! ==                                                      ==
! == Positionne a zero/null/'' les variables              ==
! ==========================================================
subroutine resultat_fil_initdefaut(rtf)

implicit none

type(st_resultat_structure_fil), intent(inout) :: rtf

!rtf%flag_data = .false. ! pas utile ici
nullify(rtf%dt)

rtf%n_it = 0; rtf%n_qt = 0; rtf%n_vt = 0; 
nullify(rtf%l_it)
nullify(rtf%l_qt)
nullify(rtf%l_vt)

rtf%n_if = 0;rtf%n_qf = 0; rtf%n_vf = 0; rtf%n_zf = 0
nullify(rtf%l_if)
nullify(rtf%l_qf)
nullify(rtf%l_zf)
nullify(rtf%l_vf)

rtf%mantisse_entier = MANTISSE_ENTIER
rtf%mantisse_reel = MANTISSE_REEL

end subroutine resultat_fil_initdefaut

! ==========================================================
! == Traitrement d'un lieu                                ==
! ==========================================================
subroutine lieu_initdefaut(lieu)

implicit none

type(st_lieu), intent(inout) :: lieu

NULLIFY(lieu%p)
lieu%numfil = 0
lieu%deb = 0
lieu%fin = 0
lieu%pas = 0
lieu%nb_segment = 0

end subroutine lieu_initdefaut


! ==========================================================
! == Traitement sur un fil                                ==
! ==========================================================
subroutine resultat_un_fil_initdefaut(rf)

implicit none

type(st_resultat_un_fil), intent(inout) :: rf

rf%nom = ''
rf%nom_fichier = ''
rf%id_fichier = 0
call lieu_initdefaut(rf%ou)
rf%formate = '(' // TRIM(MANTISSE_REEL) // ')'
nullify(rf%tf)
nullify(rf%val_t)
nullify(rf%val_tprecedent)
nullify(rf%val_f)
rf%tf_commutateur = 0
rf%rg = 0.0
rf%flag_cor = .false.
rf%flag_eval_fin_simu = .false.
end subroutine resultat_un_fil_initdefaut


! ==========================================================
! == Subroutine resultat_structure_fil_initdefaut     ==
! ==                                                      ==
! == Positionne a zero/null/'' les variables              ==
! ==========================================================
subroutine resultat_unfil_es_initdefaut(rf)

type(st_resultat_un_fil_ES), intent(inout) :: rf

rf%nom = ''
rf%interv1 = ''
rf%interv2 = ''
rf%nom_objet  = ''
rf%nom_objet2  = ''
rf%deb = 0 ; rf%fin = 0 ; rf%pas = 0
rf%tf_commutateur = 0;
rf%flag_eval_fin_simu = .false.
call sortie_normalise_initdefaut(rf%sn)

end subroutine resultat_unfil_es_initdefaut
! ==========================================================
! == Subroutine resultat_structure_fil_lecture()      ==
! == Lecture correspondant aux resultats demandes sur     ==
! == les structures structure_filaires                              ==
! ==========================================================
subroutine resultat_structure_fil_lecture(f,rtf,sf,ltf)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_resultat_structure_fil), intent(inout) :: rtf
type(st_structure_fil), intent(inout) :: sf
type(st_liste_temps_freq), intent(inout) :: ltf

!=== Déclaration des variables ===
logical :: flag_t, flag_f,flag_eval_fin_simu
character(LEN=LEN_MOT_CLE) :: corbeille, nom, nom1, nom2, nom_t, nom_f
character(LEN=LEN_CHAINE)  :: tampon
integer :: tf_commutateur
integer :: i_if, i_it, i_qf, i_qt
integer :: i_zf, i_zt
integer :: i, i_vt, i_vf
integer :: ires,nbmot
type(st_resultat_un_fil), dimension(:), pointer  :: l_zt
! Normalisation en fréquence
type(st_sortie_normalise) :: sn
type(st_resultat_un_fil_ES), pointer :: pres


! -- Mise a zero de f%numbackspace --
nullify(l_zt)
call numbackspace_set(f,0)
ires = 0
! On compte le nombre de paramètres résultats
! en temps et en fréquence
do
  corbeille = lineread(f,1)
  if (corbeille==f%nomfinbloc) then
    exit
  
  ! -- Point de courant --
  elseif (corbeille(1:1) == 'I') then
    read(f%chaine,*,err=999) nom, nom1, nom2
    call temps_freq_compteur_incr(nom1,nom2,rtf%n_it,rtf%n_if)
    ires = ires+1
  ! -- Point de tension --
  elseif (corbeille(1:1) == 'V') then
    read(f%chaine,*,err=999) nom, nom1, nom2
    call temps_freq_compteur_incr(nom1,nom2,rtf%n_vt,rtf%n_vf)
    ires = ires+1
  ! -- Point de charge --
  elseif (corbeille(1:1) == 'Q') then
    read(f%chaine,*,err=999) nom, nom1, nom2
    call temps_freq_compteur_incr(nom1,nom2,rtf%n_qt,rtf%n_qf)
    ires = ires+1
  ! -- Point d'impedance --
  elseif (corbeille(1:1) == 'Z') then
    read(f%chaine,*,err=999) nom, nom1, nom2
    if (identifie_deux_nom(nom1,nom2,flag_t,flag_f,nom_t,nom_f)) then
      if (flag_t) then
	print '(/,t10,"ATTENTION, pas de calcul d''impedance en temporel")'
	call lecture_stop(f)
      endif
      if (flag_f) then
	rtf%n_zf = rtf%n_zf + 1 
	rtf%n_if = rtf%n_if + 1 
	rtf%n_vf = rtf%n_vf + 1
      endif
    end if
    ires = ires+1
  end if
enddo

! -- Les tableaux doivent etre alloue meme a zero --
! -- tableau en temps ---
allocate(rtf%l_it(rtf%n_it))
do i=1,rtf%n_it
  call resultat_un_fil_initdefaut(rtf%l_it(i))
enddo

allocate(rtf%l_qt(rtf%n_qt))
do i=1,rtf%n_qt
  call resultat_un_fil_initdefaut(rtf%l_qt(i))
enddo

allocate(rtf%l_vt(rtf%n_vt))
do i=1,rtf%n_vt
  call resultat_un_fil_initdefaut(rtf%l_vt(i))
enddo

! -- tableau en fréquence --
allocate(rtf%l_if(rtf%n_if))
do i=1,rtf%n_if
  call resultat_un_fil_initdefaut(rtf%l_if(i))
enddo

allocate(rtf%l_qf(rtf%n_qf))
do i=1,rtf%n_qf
  call resultat_un_fil_initdefaut(rtf%l_qf(i))
enddo

allocate(rtf%l_vf(rtf%n_vf))
do i=1,rtf%n_vf
  call resultat_un_fil_initdefaut(rtf%l_vf(i))
enddo

allocate(rtf%l_zf(rtf%n_zf))
do i=1,rtf%n_zf
  call resultat_un_fil_initdefaut(rtf%l_zf(i))
enddo

rtf%n_res = ires
allocate(rtf%l_res(rtf%n_res))
do i=1,ires
  call resultat_unfil_es_initdefaut(rtf%l_res(i))
end do
! -- On remonte au début du bloc --
i = multibackspace(f)
i = 0
i_it = 0
i_if = 0
i_qt = 0
i_qf = 0
i_vt = 0
i_vf = 0
i_zf = 0
i_zt = 0
ires= 0
tf_commutateur = REELLE_IMAGINAIRE
flag_eval_fin_simu = .false.
! Init normalisation
call sortie_normalise_initdefaut(sn)

! ------------ Lecture des donnees --------------
do
  corbeille = lineread(f)
  if (corbeille==f%nomfinbloc) then
    exit
  
  ! -- Commutateur de formatage de TF --

  else if (fichier_sortie_lect_format_freq(corbeille,tf_commutateur)) then
    cycle
  elseif(corbeille(1:2) == 'TF' ) then
    read(f%chaine,*,err=999) nom, nom1
    if (.not. fichier_sortie_lect_format_freq(nom1,tf_commutateur)) then
      print '(/,t10,"ATTENTION, mots clés ", A,"  ",A, " incorrects.")', &
               TRIM(corbeille), TRIM(nom1)
      call lecture_stop(f)
    endif
    cycle
  ! -- Point de courant --
  elseif (corbeille(1:1) == 'I') then
    call lecture_partielle(rtf%l_it,i_it,rtf%l_if,i_if, &
			   f,sf,ltf,tf_commutateur,sn,flag_eval_fin_simu)

  ! -- Point de charge --
  elseif (corbeille(1:1) == 'Q') then
    call lecture_partielle(rtf%l_qt,i_qt,rtf%l_qf,i_qf, &
			   f,sf,ltf,tf_commutateur,sn)

  ! -- Point de tension --
  elseif (corbeille(1:1) == 'V') then
    call lecture_partielle(rtf%l_vt,i_vt,rtf%l_vf,i_vf, &
			   f,sf,ltf,tf_commutateur,sn)

  ! -- Point d'impedance -
  elseif (corbeille(1:1) == 'Z') then
    call lecture_partielle(l_zt,i_zt,rtf%l_zf,i_zf, &
			   f,sf,ltf,tf_commutateur,sn)
    call lecture_partielle(rtf%l_it,i_it, rtf%l_if,i_if, &
			   f,sf,ltf,tf_commutateur,sn)
    call lecture_partielle(rtf%l_vt,i_vt,rtf%l_vf,i_vf, &
			   f,sf,ltf,tf_commutateur,sn)


  !----------------------------------------------------------------------------
  else if (corbeille==ID_EVAL_TMAX) then
    flag_eval_fin_simu = .true.
  else if (corbeille==ID_FIN_EVAL_TMAX) then
    flag_eval_fin_simu = .false.
  
  !----------------------------------------------------------------------------
  elseif (sortie_normalise_lecture(f,corbeille,sn)) then
    cycle
  !----------------------------------------------------------------------------
  else
    print '(/,t10,"ATTENTION, mot cle ", A, " incorrect.")', TRIM(corbeille)
    call lecture_stop(f)
  endif
  
  !----------------------------------------------------------------------------
  !----------------------------------------------------------------------------
  if (corbeille(1:1) == 'V' .or.corbeille(1:1) == 'I'.or.corbeille(1:1) == 'Z'.or. &
      corbeille(1:1) == 'Q' ) then
    ires = ires+1
    pres =>rtf%l_res(ires)
    pres%tf_commutateur = tf_commutateur;
    pres%flag_eval_fin_simu = flag_eval_fin_simu
    pres%sn = sn

    read(f%chaine,*) pres%nom, pres%interv1,pres%interv2
    if (.not.identifie_deux_nom(pres%interv1,pres%interv2,flag_t,flag_f,nom_t,nom_f)) cycle
    nbmot = nb_mot_chaine(f%chaine)
    if (flag_t.and.flag_f) then
      tampon = extrait_sous_chaine(f%chaine,4,nbmot) ! à partir du 4ème mot
    else
      tampon = extrait_sous_chaine(f%chaine,3,nbmot) ! à partir du 3ème mot
      pres%interv2 = ""
    endif
    
    if (corbeille(1:2) == 'VG'.or.corbeille(1:2) == 'ZE'.or.corbeille(1:2) == 'VQ'.or. &
        corbeille(1:3) == 'VIQ') then
	  read(tampon,*,err=999) pres%nom_objet
    else if (corbeille(1:3) == 'VZP'.or.corbeille(1:2) == 'ZP') then
      read(tampon,*,err=999) pres%nom_objet2,pres%nom_objet,pres%deb
    else 
      nbmot = nb_mot_chaine(f%chaine)
      read(tampon,*,err=999) pres%nom_objet,pres%deb
      if (nbmot>=3) tampon = extrait_sous_chaine(tampon,3,nbmot) ! à partir du 3ème mot
      nbmot = nb_mot_chaine(tampon)
      if (nbmot==1) then
        read(tampon,*,err=999) pres%fin
      else if (nbmot==2) then
        read(tampon,*,err=999) pres%fin,pres%pas
      end if      
    end if
  end if

enddo

return
999 print '(/,t10,"ATTENTION, erreur de lecture de la ligne")'
    call lecture_stop(f)

end subroutine resultat_structure_fil_lecture

! ==========================================================
! == Subroutine resultat_structure_fil_ecriture()         ==
! == Lecture correspondant aux resultats demandes sur     ==
! == les structures structure_filaires                    ==
! ==========================================================
subroutine resultat_structure_fil_ecriture(f,rtf)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_resultat_structure_fil), intent(inout) :: rtf
integer  :: i,commutateur
logical  :: flag_eval_fin_simu
character(LEN=LEN_CHAINE)  :: chaine,ch1,ch2
type(st_resultat_un_fil_ES), pointer :: pres
type(st_sortie_normalise) :: sn


commutateur = REELLE_IMAGINAIRE
call sortie_normalise_initdefaut(sn)
do i=1,rtf%n_res
  pres =>rtf%l_res(i)
  if (.not.sortie_normalise_compare(sn,pres%sn)) then
    call sortie_normalise_ecriture(f,pres%sn)
    sn = pres%sn
  end if
  
  if (flag_eval_fin_simu.neqv.pres%flag_eval_fin_simu) then
    flag_eval_fin_simu = pres%flag_eval_fin_simu
    if (flag_eval_fin_simu) then
      write(f%unite,'(T3,A)') ID_EVAL_TMAX
    else
      write(f%unite,'(T3,A)') ID_FIN_EVAL_TMAX
    end if
  end if
  
  if (commutateur/=pres%tf_commutateur) then
    call fichier_sortie_ecrit_format(f,pres%tf_commutateur)
    commutateur = pres%tf_commutateur
  end if
  
  if (pres%nom(1:2) == 'VG'.or.pres%nom(1:3) == 'VZP'.or.pres%nom(1:2) == 'VQ'.or. &
        pres%nom(1:3) == 'VIQ') then
    if (pres%interv2 /='') then
      write(f%unite,'(T3,3(A,3x),A)') trim(pres%nom),trim(pres%interv1),&
                                             trim(pres%interv2),trim(pres%nom_objet)
    else
      write(f%unite,'(T3,A,3x,A,3x,A)') trim(pres%nom),trim(pres%interv1),&
				      trim(pres%nom_objet)
    end if
  else if (pres%nom(1:2) == 'ZE') then
    write(f%unite,'(T3,2(A,3x),A)') trim(pres%nom),trim(pres%interv1),&
					    trim(pres%nom_objet)
					    
  else if (pres%nom(1:2) == 'ZP'.or.pres%nom(1:3) == 'VZP') then
    write(f%unite,'(T3,5(A,3x),I5)') trim(pres%nom),trim(pres%interv1),&
                      trim(pres%interv1),trim(pres%nom_objet2),trim(pres%nom_objet),pres%deb
					   
  else 
    if (pres%interv2 /='') then
      chaine =  trim(pres%nom)//' '//trim(pres%interv1)//' '//trim(pres%interv2)&
                      //' '//trim(pres%nom_objet)
    else
      chaine =  trim(pres%nom)//' '//trim(pres%interv1)//' '//trim(pres%nom_objet)
    end if
    write(ch1,'(I5)') pres%deb 
    ch2 = ' ';
    if (pres%fin>0) then
      if (pres%pas>0) then
        write(ch2,'(2I5)') pres%fin,pres%pas
      else
        write(ch2,'(I4)') pres%fin      
      end if
    end if
    write(f%unite,'(T3,2(A,3x),A)') trim(chaine),trim(ch1),trim(ch2)
  end if
end do

end subroutine resultat_structure_fil_ecriture

! ==========================================================
! == Subroutine lecture_partielle                         ==
! ==========================================================
subroutine lecture_partielle(l_it,i_it, l_if,i_if, &
			     f,stf,ltf,tf_commutateur,sn,flag_eval_fin_simu)


integer, intent(in) :: tf_commutateur
integer, intent(inout) :: i_it, i_if
type(st_resultat_un_fil), dimension(:), pointer :: l_it
type(st_resultat_un_fil), dimension(:), pointer :: l_if
type(st_fichier), intent(inout) :: f
type(st_structure_fil), intent(inout) :: stf
type(st_liste_temps_freq), intent(inout) :: ltf
type(st_sortie_normalise),intent(in) :: sn
logical,intent(in),optional :: flag_eval_fin_simu

character(LEN=LEN_CHAINE) :: tampon, tampon1
character(LEN=LEN_MOT_CLE) :: nom, nom1, nom2, nom_t, nom_f
logical :: flag_t, flag_f,flag_cor
integer :: nbmot
type(st_lieu) :: lieu

read(f%chaine,*) nom, nom1, nom2

! Identifie le ou les noms de stockage temps - fréquence
if (.not.identifie_deux_nom(nom1,nom2,flag_t,flag_f,nom_t,nom_f)) then
  print '(/,t10,"Nom du type de stockage erroné !!!")'
  call lecture_stop(f)
end if

! -- Lecture des points de stockage dans la chaine --
nbmot = nb_mot_chaine(f%chaine)
if (flag_t.and.flag_f) then
  tampon = extrait_sous_chaine(f%chaine,4,nbmot) ! à partir du 4ème mot
else
  tampon = extrait_sous_chaine(f%chaine,3,nbmot) ! à partir du 3ème mot
endif
!print '("nbmot = ",I)', nbmot
!print '("tampon = ",A)', TRIM(tampon)
!print '("flag_t et flag_f = ",L,L)', flag_t,flag_f

lieu = cherche_lieu(f,tampon,stf)
if (lieu%numfil==0) call lecture_stop(f)

flag_cor = .false.
if (f%premiermot(1:1) =='I' .or. f%premiermot(1:3) =='VZP' .or. f%premiermot(1:3) =='VIQ'.or. &
    f%premiermot(1:2) =='VQ'.or. f%premiermot(1:2) =='ZP') flag_cor = .true.



!--- Structure en temps ---
if (flag_t) then
  i_it = i_it + 1
  l_it(i_it)%nom = nom
  l_it(i_it)%flag_cor = flag_cor
  l_it(i_it)%nom = TRIM(l_it(i_it)%nom) // SUFFIXE_TEMP
  l_it(i_it)%ou = lieu
  if (present(flag_eval_fin_simu))  l_it(i_it)%flag_eval_fin_simu = flag_eval_fin_simu

  ! -- Recupere le pointeur sur les structures temps-frequence --
  l_it(i_it)%tf => liste_temps_freq_pointe(ltf,nom_t)
  if (.not.associated(l_it(i_it)%tf)) then
    print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
    call lecture_stop(f)      
  end if
endif

! -- Structure en frequence --
if (flag_f) then
  i_if = i_if + 1
  l_if(i_if)%nom = nom
  l_if(i_if)%flag_cor = flag_cor
  l_if(i_if)%nom = TRIM(l_if(i_if)%nom) // SUFFIXE_FREQ
  l_if(i_if)%ou = lieu
  if (present(flag_eval_fin_simu))  l_if(i_if)%flag_eval_fin_simu = flag_eval_fin_simu

  ! -- Recupere le pointeur sur les structures temps-frequence --
  l_if(i_if)%tf => liste_temps_freq_pointe(ltf,nom_f)
!  print '("nom,nom_f=",A," ",A)',TRIM(nom),TRim(nom_f)
  if (.not.associated(l_if(i_if)%tf)) then
    print '(/,t10,"Nom du type de stockage en frequence erroné !!!")'
    call lecture_stop(f)      
  end if
  l_if(i_if)%tf_commutateur = tf_commutateur
  l_if(i_if)%sn = sn
  
endif    

end subroutine lecture_partielle


! ==========================================================
! == subroutine cherche_lieu()                            ==
! == rempli la structure st_lieu décrivant les lieux ou   ==
! == un paramètre doit etre retenu.                       ==
! ==========================================================
function cherche_lieu(f,chaine,stf) result(coord)

implicit none

type(st_fichier), intent(inout) :: f
character(LEN=LEN_CHAINE), intent(inout) :: chaine
type(st_structure_fil), intent(inout) :: stf

type(st_lieu) :: coord

character(LEN=LEN_MOT_CLE) :: nomfil,nomobjet
integer :: numseg1, numseg2, passeg, numfil, nbmot

call lieu_initdefaut(coord)

! lecture de segment debut, segment fin et le pas
nbmot = nb_mot_chaine(chaine)
numseg1 = 1;
numseg2 = 1
nomfil   = ""
nomobjet = ""
!print '("chaine = ",A)', TRIM(chaine)
!print '("nbmot = ",I)', nbmot

select case(nbmot)
  case(4)
    read(chaine,*,err=998) nomfil, numseg1, numseg2, passeg
    numfil = verif_nom_fil(stf,nomfil)
    if (numfil<=0) call lecture_stop(f)
  case(3)
      read(chaine,*,err=998) nomfil, numseg1, numseg2
      passeg = 1
      numfil   = verif_nom_fil(stf,nomfil)
      if (numfil<=0) call lecture_stop(f)
  case(2)
    read(chaine,*,err=998) nomfil, numseg1
    numseg2 = numseg1
    passeg  = 1
    numfil   = verif_nom_fil(stf,nomfil)
    if (numfil<=0) call lecture_stop(f)
  !----------------------------------------------------------------------------
  case(1) ! stockage tension du générateur ou impédance d'entrée
    read(chaine,*,err=998) nomobjet
    nomfil = nomobjet
    if (f%premiermot(1:2) == 'VG'.or.f%premiermot(1:2) == 'ZE') then
      numfil   = verif_nom_gene_fil(stf,nomobjet)
      if (numfil<=0) call lecture_stop(f)
      numseg1  = stf%l_fil(numfil)%gv%seg
      numseg2  = numseg1
    else
      print '(" Erreur, Type de résultat non reconnu !!! nom=",A)',trim(f%premiermot)
      call lecture_stop(f)
    end if
  !----------------------------------------------------------------------------
  case default
    print '(" Erreur, le nombre de mot est incorrect dans la chaine :", A)',TRIM(chaine)
    call lecture_stop(f)
end select


if (nbmot>1 .and. numfil>0.and. &
    verif_segment_fil(nomfil,stf%l_fil(numfil)%nseg,numseg1).and. &
    verif_segment_fil(nomfil,stf%l_fil(numfil)%nseg,numseg2)) then

  coord%deb = numseg1
  coord%fin = numseg2
  coord%pas = passeg
  coord%nb_segment = nbr_seg(coord)
  coord%numfil = numfil
  coord%nomfil = nomfil
  
else if (nbmot==1 .and. numfil>0)  then
  coord%deb = numseg1 ;  coord%fin = numseg2
  coord%pas = 1 ;  
  coord%nb_segment = nbr_seg(coord)
  coord%numfil = numfil
  coord%nomfil = nomfil
!  print '("nomobjet=",A,"  nb_seg=",I2)',trim(nomfil),coord%nb_segment
endif

return
998 print '(/,t10,"ATTENTION, erreur de lecture des paramètres de la ligne")'
    call lecture_stop(f)

end function cherche_lieu                               


! ==========================================================
! == Function nbr_seg(lieu)                               ==
! == lieu : st_lieu                                       ==
! ==                                                      ==
! == Calcul le nombre de segments a stocker               ==
! ==========================================================
integer function nbr_seg(lieu)

implicit none

type(st_lieu), intent(in) :: lieu

nbr_seg = INT((lieu%fin - lieu%deb) / lieu%pas) + 1

end function nbr_seg


! ==========================================================
! == resultat_structure_fil_init()          ==
! == Initialisation de la structure                       ==
! ==========================================================
subroutine resultat_structure_fil_init(rtf,vol,fs)

implicit none

type(st_resultat_structure_fil), intent(inout) :: rtf
type(st_volume), intent(in) :: vol
type(st_fichier_sortie), intent(in) :: fs

integer :: i

! -- Si le format des fichiers de sortie est ascii --
if (fs%format_fichier==FORMAT_ASCII) then
  do i=1,rtf%n_it
    call resultat_un_fil_init_ascii(rtf%l_it(i),fs)
  enddo  

  do i=1,rtf%n_qt
    call resultat_un_fil_init_ascii(rtf%l_qt(i),fs)
  enddo  

  do i=1,rtf%n_vt
    call resultat_un_fil_init_ascii(rtf%l_vt(i),fs)
  enddo  

  do i=1,rtf%n_if
    if (rtf%l_if(i)%nom(1:1)=='Z') cycle
    call resultat_un_fil_init_ascii(rtf%l_if(i),fs)
  enddo  

  do i=1,rtf%n_qf
    call resultat_un_fil_init_ascii(rtf%l_qf(i),fs)
  enddo  

  do i=1,rtf%n_vf
    if (rtf%l_vf(i)%nom(1:1)=='Z') cycle
    call resultat_un_fil_init_ascii(rtf%l_vf(i),fs)
  enddo  

  do i=1,rtf%n_zf
    call resultat_un_fil_init_ascii(rtf%l_zf(i),fs)
  enddo  
end if
end subroutine resultat_structure_fil_init

! ==========================================================
! == Subroutine resultat_un_fil_init_ascii()    ==
!! ==                                                      ==
! == Creation de la chaine de formatage pour l'ecriture   ==
! == dans le fichier de sortie                            ==
! == Creation des noms de fichier de sortie               ==
! == Ouverture des fichiers de sortie dans le cas ascii   ==
! ==========================================================
subroutine resultat_un_fil_init_ascii(rf,fs)

implicit none

type(st_resultat_un_fil), intent(inout) :: rf
type(st_fichier_sortie), intent(in) :: fs

integer :: nb, len_nb
character(LEN=LEN_MOT_CLE) :: chaine

!nb = nbr_seg(rf%ou)
nb = rf%ou%nb_segment

! -- Creation du format d'ecriture en temporel --
!if (rf%tf%itype==LISTE_TEMPORELLE.or.rf%tf%itype==INTERVALLE_TEMPOREL) then
!  nb = nb + 1
! -- Creation du format d'ecriture en frequence --
!elseif (rf%tf%itype==LISTE_FREQUENTIELLE.or.rf%tf%itype==INTERVALLE_FREQUENTIEL) then
!  nb = 2*nb + 1
!endif

if (rf%tf%flag_freq) then
  nb = 2*nb + 1
else
  nb = nb + 1
end if

chaine = entier_chaine(nb)
rf%formate = '(' // TRIM(chaine) // TRIM(fs%mantisse_reel) // ')'

rf%nom_fichier = set_nom(rf%nom,fs%extension)
rf%id_fichier = ouvre_en_ecriture(rf%nom_fichier)

end subroutine resultat_un_fil_init_ascii

! ==========================================================
! == Function set_nom(nom,ext)                            ==
! == nom : character(LEN=LEN_MOT_CLE)                     ==
! == ext : character(LEN=LEN_EXTENSION)                   ==
! ==                                                      ==
! == Determination des nom de fichiers de sortie          ==
! ==========================================================
character(LEN=LEN_MOT_CLE) function set_nom(nom,ext)

implicit none

character(LEN=LEN_MOT_CLE), intent(in) :: nom
character(LEN=LEN_MOT_CLE), intent(in) :: ext

set_nom = TRIM(nom) // TRIM(ext)

end function set_nom

! ==========================================================
! == Ouverture des fichiers de stockage                   ==
! ==========================================================
subroutine resultat_fil_precalcul(rtf,sf,fs)

implicit none

type(st_resultat_structure_fil), intent(inout) :: rtf
type(st_structure_fil), intent(inout) :: sf
type(st_fichier_sortie), intent(in) :: fs

integer :: i

do i=1,rtf%n_it
  call resultat_un_fil_precalcul(rtf%l_it(i),'I',sf)
enddo

do i=1,rtf%n_qt
  call resultat_un_fil_precalcul(rtf%l_qt(i),'Q',sf)
enddo

do i=1,rtf%n_vt
  call resultat_un_fil_precalcul(rtf%l_vt(i),'V',sf)
enddo

do i=1,rtf%n_if
  call resultat_un_fil_precalcul(rtf%l_if(i),'I',sf)
!  if (rtf%l_if(i)%nom(1:1)=='Z') cycle
enddo

do i=1,rtf%n_qf
  call resultat_un_fil_precalcul(rtf%l_qf(i),'Q',sf)
enddo

do i=1,rtf%n_vf
  call resultat_un_fil_precalcul(rtf%l_vf(i),'V',sf)
!  if (rtf%l_vf(i)%nom(1:1)=='Z') cycle
enddo

do i=1,rtf%n_zf
  call resultat_un_fil_precalcul(rtf%l_zf(i),'Z',sf)
enddo

end subroutine resultat_fil_precalcul

! ==========================================================
! == Subroutine resultat_un_fil_precalcul                 ==
! ==                                                      ==
! == Association des pointeurs                            ==
! ==========================================================
subroutine resultat_un_fil_precalcul(rf,i_ou_q,stf)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_un_fil), intent(inout) :: rf
character, intent(in) :: i_ou_q
type(st_structure_fil), intent(inout) :: stf

!=== Déclaration des variables ===
integer :: numfil
integer :: i,j
type(st_fil_segment), pointer  :: tseg

!=== Début des traitements ===
numfil = rf%ou%numfil
allocate(rf%val_t(rf%ou%nb_segment))
allocate(rf%val_tprecedent(rf%ou%nb_segment))
rf%val_t = 0
rf%val_tprecedent = 0
rf%rg = 0.0
!print '("nomobjet=",A,"  nb_seg=",I2)',trim(rf%ou%nomfil),rf%ou%nb_segment
allocate(rf%ou%p(1:rf%ou%nb_segment))
! coefficient multiplicatif de la composante de stockage
do i = 1, rf%ou%nb_segment
  allocate(rf%ou%p(i)%c_val)
  rf%ou%p(i)%c_val = 1.0
end do

j = 0
!------------------------------------------------------------------------------
! Création des alias
!------------------------------------------------------------------------------
if (i_ou_q=='I'.or.i_ou_q=='i') then
    do i = rf%ou%deb,rf%ou%fin,rf%ou%pas 
      j = j+1
      tseg => stf%l_fil(numfil)%l_seg(i)
      rf%ou%p(j)%valeur => tseg%cur
    end do
!------------------------------------------------------------------------------
else if (i_ou_q=='Q'.or.i_ou_q=='q') then
  do i = rf%ou%deb,rf%ou%fin,rf%ou%pas 
    j = j+1
    rf%ou%p(j)%valeur => stf%l_fil(numfil)%l_seg(i)%qc
  end do
!------------------------------------------------------------------------------
else if (i_ou_q=='V'.or.i_ou_q=='v') then
  if (rf%nom(1:2) == 'VG' .or. rf%nom(1:2) == 'ZE') then ! tension du générateur
    rf%ou%p(1)%valeur => stf%l_fil(numfil)%l_seg(rf%ou%deb)%vsrc
    ! on récupère la valeur de la résistance du générateur
    rf%rg = stf%l_fil(numfil)%gv%r0
  else  ! tension au borne de la cellule
    do i = rf%ou%deb,rf%ou%fin,rf%ou%pas 
      j = j+1
      rf%ou%p(j)%valeur => stf%l_fil(numfil)%l_seg(i)%emoy
      rf%ou%p(j)%c_val = - stf%l_fil(numfil)%dl(i)
    end do  
  end if
end if
!------------------------------------------------------------------------------

if (rf%tf%flag_freq) then
  allocate(rf%val_f(rf%tf%n_liste,rf%ou%nb_segment))
  rf%val_f = 0
!------------------------------------------------------------------------------
endif

end subroutine resultat_un_fil_precalcul

! ==========================================================
! == Subroutine resultat_structure_fil_calcul         ==
! ==                                                      ==
! ==========================================================
subroutine resultat_structure_fil_calcul(rtf,fs,iter)

implicit none

!=== Déclaration des arguments ===
type(st_resultat_structure_fil), intent(inout) :: rtf
type(st_fichier_sortie), intent(in) :: fs
integer, intent(in) :: iter

!=== Déclaration des variables ===
integer :: i

! -- Si format ascii --
if (fs%format_fichier==FORMAT_ASCII) then
  do i=1,rtf%n_it
    call resultat_ecriture_t_ascii_cor(rtf%l_it(i))
  enddo

  do i=1,rtf%n_qt
    call resultat_ecriture_temp_ascii(rtf%l_qt(i))
  enddo

  do i=1,rtf%n_vt
    if (rtf%l_vt(i)%flag_cor) then
      call resultat_ecriture_t_ascii_cor(rtf%l_vt(i))
    else
      call resultat_ecriture_temp_ascii(rtf%l_vt(i))    
    end if
  enddo


endif

! -- Calcul de TF --
do i=1,rtf%n_if
  call resultat_un_fil_calcul_tf_cor(rtf%l_if(i))
enddo

do i=1,rtf%n_qf
  call resultat_un_fil_calcul_tf(rtf%l_qf(i))
enddo

do i=1,rtf%n_vf
  if (rtf%l_vf(i)%flag_cor) then
    call resultat_un_fil_calcul_tf_cor(rtf%l_vf(i))
  else
    call resultat_un_fil_calcul_tf(rtf%l_vf(i))
  end if
enddo

end subroutine resultat_structure_fil_calcul

! ==========================================================
! == Subroutine resultat_un_fil_calcul_tf(rf)   ==
! == rf : st_resultat_structure_fil                       ==
! ==                                                      ==
! == Calcul des TF partielles pour l'iteration donnee     == 
! ==========================================================
subroutine resultat_un_fil_calcul_tf(rf)

implicit none

type(st_resultat_un_fil), intent(inout) :: rf

integer ::  j
type(st_temps_frequence), pointer :: tf

tf => rf%tf
! -- On verifie si on doit faire le calcul a cette iteration --
if (tf%flag_calcul) then
  do j=1,rf%ou%nb_segment
    rf%val_f(:,j) = rf%val_f(:,j) + rf%ou%p(j)%valeur * rf%ou%p(j)%c_val * tf%expi2pitf(:)
  enddo
endif

end subroutine resultat_un_fil_calcul_tf

! ==========================================================
! == Subroutine resultat_un_fil_calcul_tf_cor(rf)   ==
! == rf : st_resultat_structure_fil                       ==
! ==                                                      ==
! == Calcul des TF partielles pour l'iteration donnee     == 
! == Correction arithmétique ...
! == (Valeur à n+1/2 + Valeur à n-1/2) / 2 ==> Valeur à n
! ==========================================================
subroutine resultat_un_fil_calcul_tf_cor(rf)

implicit none

type(st_resultat_un_fil), intent(inout) :: rf

integer :: i, j
real    :: val_t
type(st_temps_frequence), pointer :: tf

tf => rf%tf
! -- On verifie si on doit faire le calcul a cette iteration --
if (tf%flag_calcul) then
  do j=1,rf%ou%nb_segment
    val_t = (rf%ou%p(j)%valeur + rf%val_tprecedent(j))*0.5
    rf%val_f(:,j) = rf%val_f(:,j) + val_t * tf%expi2pitf(:)
  enddo
endif
do j=1,rf%ou%nb_segment ! préserve l'échantillon en cours
  rf%val_tprecedent(j) = rf%ou%p(j)%valeur
enddo

end subroutine resultat_un_fil_calcul_tf_cor


! ==========================================================
! == Subroutine resultat_ecriture_temp_ascii()        ==
! ==========================================================
subroutine resultat_ecriture_temp_ascii(rf)

implicit none

type(st_resultat_un_fil), intent(inout) :: rf

integer  :: i
type(st_temps_frequence), pointer :: tf

tf => rf%tf
! -- On verifie si on doit stocker a cette iteration --
if (tf%flag_calcul) then 
  do i=1, rf%ou%nb_segment
    rf%val_t(i) = rf%ou%p(i)%valeur * rf%ou%p(i)%c_val
  end do
  write(rf%id_fichier,rf%formate) tf%liste(tf%curseur), rf%val_t(:)
endif

end subroutine resultat_ecriture_temp_ascii

! ==========================================================
! == Subroutine resultat_ecriture_temp_ascii()        ==
! == correction arithmétique sur le temps
! == (Valeur à n+1/2 + Valeur à n-1/2) / 2 ==> Valeur à n
! ==========================================================
subroutine resultat_ecriture_t_ascii_cor(rf)

implicit none

type(st_resultat_un_fil), intent(inout) :: rf

integer  :: i
type(st_temps_frequence), pointer :: tf

tf => rf%tf
! -- On verifie si on doit stocker a cette iteration --
if (tf%flag_calcul) then 
  do i=1, rf%ou%nb_segment
    ! moyenne arithmétique
    rf%val_t(i) = (rf%ou%p(i)%valeur + rf%val_tprecedent(i)) * 0.5
  end do
  write(rf%id_fichier,rf%formate) tf%liste(tf%curseur), rf%val_t(:)
endif
do i=1,rf%ou%nb_segment
  rf%val_tprecedent(i) = rf%ou%p(i)%valeur
enddo

end subroutine resultat_ecriture_t_ascii_cor


! ==========================================================
! == Subroutine resultat_fil_impression(rtf)==
! == rtf : st_resultat_structure_fil                  ==
! ==                                                      ==
! == Calcul des divers impedances.                        ==
! == Affichage des calculs (post/in)iterations temporelles==
! == Tf par exemple.                                      ==
! ==========================================================
subroutine resultat_fil_impression(rtf,fs,t_gene)
!=== Déclaration des arguments ===
type(st_resultat_structure_fil), intent(inout) :: rtf
type(st_fichier_sortie), intent(in) :: fs
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer :: i,j
type(st_resultat_un_fil), pointer :: rf
complex, dimension(:), allocatable   :: tnorm_f

!===   Début des traitements   ===
call resultat_structure_fil_Z(rtf)

if (fs%format_fichier==FORMAT_ASCII) then
  do i=1,rtf%n_if
    if (rtf%l_if(i)%nom(1:1)=='Z') cycle
    rf => rtf%l_if(i)
    !---------- Normalisation ?-----------------
    if (rf%sn%flag_normalisation) then
      allocate(tnorm_f(1:rf%tf%n_liste))
      call Normalise_Reponse_Freq(t_gene,rf%tf,rf%sn,tnorm_f)
      ! normalisation pour chaque composante
      do j=1,rf%ou%nb_segment
        rf%val_f(:,j) = rf%val_f(:,j) / tnorm_f(:)
      end do
      deallocate(tnorm_f)
    end if
    call impression_ascii_complexe(rf%id_fichier, &
             rf%formate, rf%tf_commutateur, &
	     rf%tf%liste, rf%val_f, rf%tf%n_liste,SIZE(rf%val_f,2))      
  enddo

  do i=1,rtf%n_qf
    rf => rtf%l_qf(i)
    !---------- Normalisation ?-----------------
    if (rf%sn%flag_normalisation) then
      allocate(tnorm_f(1:rf%tf%n_liste))
      call Normalise_Reponse_Freq(t_gene,rf%tf,rf%sn,tnorm_f)
      ! normalisation pour chaque composante
      do j=1,rf%ou%nb_segment
        rf%val_f(:,j) = rf%val_f(:,j) / tnorm_f(:)
      end do
      deallocate(tnorm_f)
    end if
    
    call impression_ascii_complexe(rf%id_fichier, &
             rf%formate, rf%tf_commutateur, &
	     rf%tf%liste, rf%val_f, rf%tf%n_liste,SIZE(rf%val_f,2))      
  enddo

  do i=1,rtf%n_vf
    if (rtf%l_vf(i)%nom(1:1)=='Z') cycle
    rf => rtf%l_vf(i)
    !---------- Normalisation ?-----------------
    if (rf%sn%flag_normalisation) then
      allocate(tnorm_f(1:rf%tf%n_liste))
      call Normalise_Reponse_Freq(t_gene,rf%tf,rf%sn,tnorm_f)
      ! normalisation pour chaque composante
      do j=1,rf%ou%nb_segment
        rf%val_f(:,j) = rf%val_f(:,j) / tnorm_f(:)
      end do
      deallocate(tnorm_f)
    end if
    call impression_ascii_complexe(rf%id_fichier, &
             rf%formate, rf%tf_commutateur, &
	     rf%tf%liste, rf%val_f, rf%tf%n_liste,SIZE(rf%val_f,2))      
  enddo

  do i=1,rtf%n_zf
    rf => rtf%l_zf(i)
    call impression_ascii_complexe(rf%id_fichier, &
             rf%formate, rf%tf_commutateur, &
	     rf%tf%liste, rf%val_f, rf%tf%n_liste,SIZE(rf%val_f,2))      
  enddo
endif

end subroutine resultat_fil_impression


! ==========================================================
! == Subroutine resultat_structure_fil_Z(rtf) ==
! == rtf : st_resultat_structure_fil                  ==
! ==                                                      ==
! == Calcul des impedances aux endroits demandes          ==
! ==========================================================
subroutine resultat_structure_fil_Z(rtf)

implicit none

type(st_resultat_structure_fil), intent(inout) :: rtf

integer :: i, j, k, i_if, i_vf

do i=1,rtf%n_zf
  i_if = 0
  do j=1,rtf%n_if
    if (rtf%l_if(j)%nom==rtf%l_zf(i)%nom) i_if = j
  enddo
  if (i_if==0) then
    print '(/,t10,"Erreur 1 dans resultat_structure_fil_Z")'
    stop
  endif
  
  i_vf = 0
  do j=1,rtf%n_vf
    if (rtf%l_vf(j)%nom==rtf%l_zf(i)%nom) i_vf = j
  enddo
  if (i_vf==0) then
    print '(/,t10,"Erreur 2 dans resultat_structure_fil_Z")'
    stop
  endif
  
  where(rtf%l_if(i_if)%val_f /= 0 )
    rtf%l_zf(i)%val_f = rtf%l_vf(i_vf)%val_f / rtf%l_if(i_if)%val_f &
                         - rtf%l_vf(i_vf)%rg
  elsewhere

  end where
enddo

end subroutine resultat_structure_fil_Z

! ==========================================================
! == Subroutine resultat_fil_deallocate(rtf) ==
! == rtf : st_resultat_structure_fil                  ==
! ==                                                      ==
! == LIBERATION DE LA MEMOIRE ALLOUÉE                     ==
! ==========================================================
subroutine resultat_fil_deallocate(rtf)

implicit none

type(st_resultat_structure_fil), intent(inout) :: rtf

integer :: ok
if (rtf%n_it>0) call resultat_un_fil_deallocate(rtf%n_it,rtf%l_it)
if (rtf%n_qt>0) call resultat_un_fil_deallocate(rtf%n_qt,rtf%l_qt)
if (rtf%n_vt>0) call resultat_un_fil_deallocate(rtf%n_vt,rtf%l_vt)
if (rtf%n_if>0) call resultat_un_fil_deallocate(rtf%n_if,rtf%l_if)
if (rtf%n_qf>0) call resultat_un_fil_deallocate(rtf%n_qf,rtf%l_qf)
if (rtf%n_vf>0) call resultat_un_fil_deallocate(rtf%n_vf,rtf%l_vf)
if (rtf%n_zf>0) call resultat_un_fil_deallocate(rtf%n_zf,rtf%l_zf)
deallocate(rtf%l_it,STAT=ok)
deallocate(rtf%l_qt,STAT=ok)
deallocate(rtf%l_vt,STAT=ok)
deallocate(rtf%l_if,STAT=ok)
deallocate(rtf%l_qf,STAT=ok)
deallocate(rtf%l_vf,STAT=ok)
deallocate(rtf%l_zf,STAT=ok)
end subroutine resultat_fil_deallocate

! ==========================================================
! == Subroutine resultat_un_fil_deallocate(rf) ==
! ==                                                      ==
! == LIBERATION DE LA MEMOIRE ALLOUÉE                     ==
! ==========================================================
subroutine resultat_un_fil_deallocate(nb,rf)

implicit none
integer, intent(in) :: nb
type(st_resultat_un_fil),dimension(1:nb),intent(inout) :: rf

integer :: i,n,ok

do n=1,nb
  if (associated(rf(n)%val_t)) deallocate(rf(n)%val_t)
  if (associated(rf(n)%val_tprecedent)) deallocate(rf(n)%val_tprecedent)
  if (associated(rf(n)%val_f)) deallocate(rf(n)%val_f)
  
  do i=1,rf(n)%ou%nb_segment
    deallocate(rf(n)%ou%p(i)%c_val,STAT=ok)
  end do
  deallocate(rf(n)%ou%p)
  call resultat_un_fil_initdefaut(rf(n))
end do
end subroutine resultat_un_fil_deallocate


! ==========================================================
! == Subroutine resultat_fil_affichage(rtf) ==
! == rtf : st_resultat_structure_ficlaire                 ==
! ==                                                      ==
! == Affichage des caracteristaiques des resultats        ==
! == demandes dans un terminal                            ==
! ==========================================================
subroutine resultat_fil_affichage(rtf)

implicit none

type(st_resultat_structure_fil), intent(in) :: rtf

integer :: i

print *
if (rtf%n_it>0) print '("Stockage du courant en temporel :")'
do i=1,rtf%n_it
  call resultat_un_fil_affichage(i,rtf%l_it(i))
enddo

if (rtf%n_qt>0) print '("Stockage de la charge en temporel :")'
do i=1,rtf%n_qt
  call resultat_un_fil_affichage(i,rtf%l_qt(i))
enddo

if (rtf%n_vt>0) print '("Stockage de la tension en temporel :")'
do i=1,rtf%n_vt
  call resultat_un_fil_affichage(i,rtf%l_vt(i))
enddo

if (rtf%n_if>0) print '("Stockage du courant en frequentiel :")'
do i=1,rtf%n_if
  if (rtf%l_if(i)%nom(1:1)=='Z') cycle
  call resultat_un_fil_affichage(i,rtf%l_if(i))
enddo

if (rtf%n_qf>0) print '("Stockage de la charge en frequentiel :")'
do i=1,rtf%n_qf
  call resultat_un_fil_affichage(i,rtf%l_qf(i))
enddo

if (rtf%n_vf>0) print '("Stockage de la tension en frequentiel :")'
do i=1,rtf%n_vf
  if (rtf%l_vf(i)%nom(1:1)=='Z') cycle
  call resultat_un_fil_affichage(i,rtf%l_vf(i))
enddo

if (rtf%n_zf>0) print '("Stockage de l''impedance en frequentiel :")'
do i=1,rtf%n_zf
  call resultat_un_fil_affichage(i,rtf%l_zf(i))
enddo

end subroutine resultat_fil_affichage



! ==========================================================
! == Subroutine resultat_un_fil_affichage(i,rf)           ==
! == i : integer, numero associe                          ==
! == rf : st_resultat_structure_fil                       ==
! ==                                                      ==
! == Affichage d'un st_resultat                           ==
! ==========================================================
subroutine resultat_un_fil_affichage(i,rf)

implicit none

integer, intent(in) :: i
type(st_resultat_un_fil), intent(in) :: rf

integer :: j

print '(t10, "Resultat ", i3, ", nom du fil :", A, " ; Segment (min,max,pas)= ",3I4)',&
        i, TRIM(rf%ou%nomfil),rf%ou%deb, rf%ou%fin, rf%ou%pas
  
if (rf%tf%itype==INTERVALLE_TEMPOREL) then
  print '(t15, "Sur l''intervalle de temps ", A, ".")', TRIM(rf%tf%nom)
elseif (rf%tf%itype==LISTE_TEMPORELLE) then
  print '(t15, "Aux instants de la liste ", A, ".")', TRIM(rf%tf%nom)
endif 

end subroutine resultat_un_fil_affichage



! ==========================================================
! == Fermeture des fichiers de stockage                   ==
! ==========================================================
subroutine resultat_fil_fermeture(rtf,fs)

implicit none

type(st_resultat_structure_fil), intent(inout) :: rtf
type(st_fichier_sortie), intent(in) :: fs

integer :: i

if (fs%format_fichier==FORMAT_ASCII) then
  do i=1,rtf%n_it
    close(rtf%l_it(i)%id_fichier)
  enddo

  do i=1,rtf%n_qt
    close(rtf%l_qt(i)%id_fichier)
  enddo

  do i=1,rtf%n_vt
    close(rtf%l_vt(i)%id_fichier)
  enddo

  do i=1,rtf%n_if
    if (rtf%l_if(i)%nom(1:1)=='Z') cycle
    close(rtf%l_if(i)%id_fichier)
  enddo

  do i=1,rtf%n_qf
    close(rtf%l_qf(i)%id_fichier)
  enddo

  do i=1,rtf%n_vf
    if (rtf%l_vf(i)%nom(1:1)=='Z') cycle
    close(rtf%l_vf(i)%id_fichier)
  enddo

  do i=1,rtf%n_zf
    close(rtf%l_zf(i)%id_fichier)
  enddo
endif

end subroutine resultat_fil_fermeture

! ==========================================================
! == Fin du module                                        ==
! ==========================================================
end module mfdtd_resultat_structure_fil
