!////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_resultat_v_i_z.f90
!// Purpose:     Calcul de la tension, du courant et d'une impédance
!//        à partir de la circulation de E et l'intégrale de contour de H ==
!// Author:      Christophe Guiffaut
!// Modified by:
!// Created  :     Janvier 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Module mfdtd_resultat_v_i_z.f90             ==
! == Stockage tension, courant et impedance               ==
!===========================================================
module mfdtd_resultat_v_i_z

use m_constantes
use m_math_operation, only : atan2d_cor
use m_es_identifiant, only : MANTISSE_REEL,FORMAT_ASCII,REELLE_IMAGINAIRE
use m_entree, only : st_fichier, &
                    lineread, &
		    ligne_precedente, &
		    multibackspace, &
		    numbackspace_set, &
		    lecture_stop, &
		    nb_mot_chaine, &
		    ctrl_nb_mot_chaine, &
		    unitefichier
				  
use m_sortie, only : st_fichier_sortie, &
                    fichier_sortie_lect_format_freq,&
                    impression_ascii_complexe, &
		    fichier_sortie_nom_temps, &
		    fichier_sortie_nom_freq, &
		    fichier_sortie_ecrit_format,&
		    ouvre_en_ecriture


use m_temps_frequence, only : st_temps_frequence, &
			      st_liste_temps_freq, &
			      liste_temps_freq_pointe, &
			      temps_freq_compteur_incr, &
			      identifie_deux_nom

use m_generateur, only : st_generateur
use m_sortie_normalise, only : st_sortie_normalise, &
			      sortie_normalise_initdefaut,&
			      sortie_normalise_lecture   ,&
			      sortie_normalise_compare,&
			      sortie_normalise_ecriture,&
			      Normalise_Reponse_Freq

use m_commun_identifiant
use m_maillage
use mfdtd_champ, only : st_champ
use mfdtd_volume, only : st_volume
implicit none

!integer, parameter :: DIR_X       = 1
!integer, parameter :: DIR_Y       = 2
!integer, parameter :: DIR_Z       = 3
 

!=== Déclaration des types ===

type st_pointeur_champ_magnetique
  real, dimension(:), pointer  :: ph
end type st_pointeur_champ_magnetique 

!*** Structure pour un courant *** 
type st_resultat_courant
  integer, dimension(6)  :: coord ! coordonnées du contour d'intégration
  integer  :: nbc1, nbc2   ! nombre de cellules suivant les deux directions du contour
  integer  :: type_plan    ! plan du contour
  integer  :: unite        ! unité du fichier
  integer  :: com_format_tf  ! type de formatage des données en fréquence
  logical  :: flag_eval_fin_simu
  real,dimension(2)     :: pas
  type(st_pointeur_champ_magnetique), dimension(1:4)  :: p ! tableau d'alias vers les composantes à stocker
  character(LEN=LEN_MOT_CLE) :: nomfichier
  type(st_temps_frequence), pointer :: tf
  ! tableaux pour le stockage des composantes  en fréquence
  complex, dimension(:), pointer :: i_f
  ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
  ! Valeur courant en cours
  real, pointer ::Iencours
end type st_resultat_courant

!*** Structure pour une tension *** 
type st_resultat_tension
  integer, dimension(6)  :: coord ! coordonnées de la ligne d'intégration
  integer  :: nbc, direction ! nombre de cellules du circuit d'integration
  integer  :: unite          ! unité du fichier
  integer  :: com_format_tf  ! type de formatage des données en fréquence
  logical  :: flag_eval_fin_simu
  real     :: pas
  real, dimension(:), pointer  :: p_e
  character(LEN=LEN_MOT_CLE) :: nomfichier
  type(st_temps_frequence), pointer :: tf
  ! tableaux pour le stockage des composantes  en fréquence
  complex, dimension(:), pointer :: v_f
  ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
  ! Valeur courant en cours
  real, pointer ::Vencours
end type st_resultat_tension

!*** Structure pour une impedance *** 
type st_resultat_impedance
  integer, dimension(6)  :: coord_v, coord_i ! coordonnées de la surface de stockage
  integer  :: nbc_v           ! nombre de cellules du circuit d'integration
  integer  :: nbc_i1, nbc_i2  ! nombre de cellules suivant les deux directions du contour
  integer  :: type_plan, direction
  integer  :: unite          ! unité du fichier
  integer  :: com_format_tf  ! type de formatage des données en fréquence
  logical  :: flag_eval_fin_simu
  real     :: pasV, pasI(2)
  real, dimension(:), pointer  :: p_e
  type(st_pointeur_champ_magnetique), dimension(1:4)  :: p ! tableau d'alias vers les composantes à stocker
  character(LEN=LEN_MOT_CLE) :: nomfichier
  type(st_temps_frequence), pointer :: tf
  ! tableaux pour le stockage des composantes  en fréquence
  complex, dimension(:), pointer :: i_f, v_f
  ! Valeur courant en cours
  real, pointer ::Iencours,Vencours
end type st_resultat_impedance

!------------------------------------------------------------------------------
type st_resultat_v_i_z_ES ! données directes pour les entrées sorties
  character(LEN=LEN_MOT_CLE) :: nom
  character(LEN=LEN_MOT_CLE) :: interv1,interv2  
  integer   :: tf_commutateur
  logical   :: flag_eval_fin_simu
  integer, dimension(6)  :: coord1,coord2       ! coordonnees 
   ! Normalisation en fréquence
  type(st_sortie_normalise)  :: sn
end type st_resultat_v_i_z_ES
!------------------------------------------------------------------------------

!=== structure principale ===
type st_resultat_v_i_z
! Nombre de points de stockage en temps et en fréquence
  integer  ::  nb_v_f, nb_v_t 
  integer  ::  nb_i_f, nb_i_t 
  integer  ::  nb_z_f ! nombre d'impedance
  character(LEN=LEN_MOT_CLE) :: mantisse_temps
  character(LEN=LEN_MOT_CLE) :: mantisse_frequence
  ! structure de points de stockage en temps et en fréquence   
  type(st_resultat_tension), dimension(:), pointer  :: lv_t, lv_f  
  type(st_resultat_courant), dimension(:), pointer  :: li_t, li_f  
  type(st_resultat_impedance), dimension(:), pointer  :: lz_f  

  integer :: n_res
  type(st_resultat_v_i_z_ES), pointer, dimension(:) :: l_res
end type st_resultat_v_i_z
 
! Pas de variables globales...

contains

!===========================================================	
! Subroutine resultat_v_i_z_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine resultat_v_i_z_initdefaut(tviz)
!=== Déclaration des arguments ===
type(st_resultat_v_i_z), intent(inout) :: tviz

!===  Déclaration des variables  ===

!===   Début des traitements   ===
tviz%nb_v_f = 0; tviz%nb_v_t = 0
tviz%nb_i_f = 0; tviz%nb_i_t = 0
tviz%nb_z_f = 0; 

tviz%mantisse_temps = MANTISSE_REEL
tviz%mantisse_frequence = MANTISSE_REEL
				
End Subroutine resultat_v_i_z_initdefaut

! ==========================================================
! == Subroutine resultat_champ_es_initdefaut     ==
! ==========================================================
subroutine resultat_v_i_z_es_initdefaut(rf)

type(st_resultat_v_i_z_ES), intent(inout) :: rf

rf%nom     = ''
rf%interv1 = ''
rf%interv2 = ''
rf%tf_commutateur = 0;
rf%flag_eval_fin_simu = .false.
rf%coord1 = 0.0
rf%coord2 = 0.0
call sortie_normalise_initdefaut(rf%sn)

end subroutine resultat_v_i_z_es_initdefaut

!===========================================================	
! Subroutine resultat_v_i_z_courant_copy
!===========================================================	
Subroutine resultat_v_i_z_courant_copy(cin,cout)
!=== Déclaration des arguments ===
type(st_resultat_courant), intent(in)  :: cin
type(st_resultat_courant), intent(out) :: cout

!===  Déclaration des variables  ===

!===   Début des traitements   ===
cout%coord=cin%coord
cout%nbc1 =cin%nbc1
cout%nbc2 =cin%nbc2;
cout%type_plan=cin%type_plan
cout%unite=cin%unite
cout%com_format_tf=cin%com_format_tf
cout%flag_eval_fin_simu=cin%flag_eval_fin_simu
cout%pas =cin%pas
cout%nomfichier=cin%nomfichier
cout%tf=>cin%tf
nullify(cout%i_f)
cout%sn  =cin%sn
end Subroutine resultat_v_i_z_courant_copy

!===========================================================	
! Subroutine resultat_v_i_z_tension_copy
!===========================================================	
Subroutine resultat_v_i_z_tension_copy(cin,cout)
!=== Déclaration des arguments ===
type(st_resultat_tension), intent(in)  :: cin
type(st_resultat_tension), intent(out) :: cout

!===  Déclaration des variables  ===

!===   Début des traitements   ===
cout%coord=cin%coord
cout%nbc =cin%nbc
cout%direction =cin%direction
cout%unite=cin%unite
cout%com_format_tf=cin%com_format_tf
cout%flag_eval_fin_simu=cin%flag_eval_fin_simu
cout%pas =cin%pas
cout%nomfichier=cin%nomfichier
cout%tf=>cin%tf
nullify(cout%v_f)
cout%sn  =cin%sn
end Subroutine resultat_v_i_z_tension_copy

!===========================================================	
! Subroutine resultat_v_i_z_impedance_copy
!===========================================================	
Subroutine resultat_v_i_z_impedance_copy(cin,cout)
!=== Déclaration des arguments ===
type(st_resultat_impedance), intent(in)  :: cin
type(st_resultat_impedance), intent(out) :: cout

!===  Déclaration des variables  ===

!===   Début des traitements   ===
cout%coord_v=cin%coord_v
cout%coord_i=cin%coord_i
cout%nbc_v =cin%nbc_v
cout%nbc_i1 =cin%nbc_i1
cout%nbc_i2 =cin%nbc_i2
cout%type_plan =cin%type_plan
cout%direction =cin%direction
cout%unite=cin%unite
cout%com_format_tf=cin%com_format_tf
cout%flag_eval_fin_simu=cin%flag_eval_fin_simu
cout%pasV =cin%pasV
cout%pasI =cin%pasI
cout%nomfichier=cin%nomfichier
cout%tf=>cin%tf
nullify(cout%v_f)
nullify(cout%i_f)
nullify(cout%Iencours)
nullify(cout%Vencours)
end Subroutine resultat_v_i_z_impedance_copy

!===========================================================	
! Subroutine resultat_v_i_z_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine resultat_v_i_z_lecture(f, tviz,ltf,tm)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_resultat_v_i_z), intent(inout) :: tviz
type(st_liste_temps_freq), intent(inout) :: ltf
type(st_maillage), intent(in) :: tm

!===  Déclaration des variables  ===
integer  :: status
integer  :: i,ires, i_v_t, i_v_f, i_i_t, i_i_f,  i_z_f
integer  :: nbmot, direction, type_plan, com_format_tf
integer, dimension(1:6)  ::coord6
logical  :: flag_f, flag_t,flag_eval_fin_simu
character(LEN=LEN_MOT_CLE) :: corbeille,nom_identi, nom, nom1, nom2, premiermot
character(LEN=LEN_MOT_CLE) :: nom_t, nom_f
character(LEN=LEN_MOT_CLE), dimension(1:6) :: nom_eh
type(st_resultat_tension), pointer :: tv
type(st_resultat_courant), pointer :: ti
type(st_resultat_impedance), pointer :: tz
! Normalisation en fréquence
type(st_sortie_normalise) :: sn
type(st_resultat_v_i_z_ES),pointer :: pres

!===   Début des traitements   ===
call resultat_v_i_z_initdefaut(tviz)

!--- On compte le nombre de surface de stockage en temps et en frequence---
i_v_t = 0; i_v_f = 0;
i_i_t = 0; i_i_f = 0;
i_z_f = 0;
ires = 0;
call numbackspace_set(f,0)
do
  corbeille = lineread(f,1)
  if (corbeille==f%nomfinbloc) then
    exit
  else if (corbeille(1:1) == 'V') then
    read(f%chaine,*) nom,nom1,nom2
    call temps_freq_compteur_incr(nom1,nom2,i_v_t,i_v_f)
    ires=ires+1
  else if (corbeille(1:1) == 'I') then
    read(f%chaine,*) nom,nom1,nom2
    call temps_freq_compteur_incr(nom1,nom2,i_i_t,i_i_f)
    ires=ires+1

  else if (corbeille(1:1) == 'Z') then
    i_z_f = i_z_f + 1 ! incrément du nb de points de stockage
    ires=ires+1
   ! saut de 2 lignes utiles
   corbeille = lineread(f,1)
   corbeille = lineread(f,1)
    
  end if
enddo

!on remonte au début du bloc
i = multibackspace(f)

! allocation des structures en temps et en frequence...
tviz%nb_v_t = i_v_t
tviz%nb_i_t = i_i_t
allocate(tviz%lv_t(i_v_t))
allocate(tviz%li_t(i_i_t))

tviz%nb_v_f = i_v_f
tviz%nb_i_f = i_i_f
tviz%nb_z_f = i_z_f
allocate(tviz%lv_f(i_v_f))
allocate(tviz%li_f(i_i_f))
allocate(tviz%lz_f(i_z_f))


tviz%n_res = ires
allocate(tviz%l_res(tviz%n_res))
do i=1,ires
  call resultat_v_i_z_es_initdefaut(tviz%l_res(i))
end do

i_v_t = 1; i_v_f = 1;
i_i_t = 1; i_i_f = 1;
i_z_f = 1;
ires  = 1;
com_format_tf = REELLE_IMAGINAIRE    
flag_eval_fin_simu = .false.
! Init normalisation
call sortie_normalise_initdefaut(sn)

!******* lecture des données **********

do
  premiermot = lineread(f)
  select case(premiermot(1:1))
    case('V')
      read(f%chaine,*) nom_identi,nom1,nom2
    
      if (.not.identifie_deux_nom(nom1,nom2,flag_t,flag_f,nom_t,nom_f)) then
        print '(/,t10,"Nom du type de stockage erroné !!!")'
        call lecture_stop(f)
      end if
    
      ! Lecture des coordonnées
      if (flag_t .and. flag_f) then
        status = ctrl_nb_mot_chaine(f,f%chaine,9)      
        read(f%chaine,*) nom,nom1,nom2, coord6
      else
        status = ctrl_nb_mot_chaine(f,f%chaine,8)      
        read(f%chaine,*) nom,nom1, coord6
      end if  
    

      ! Controle des coordonnées
      if (.not. verif_dir_tension(nom,coord6,direction)) call lecture_stop(f)

      coord6 = origine_cellule6(tm,coord6)
      if (rang==RANG_MAITRE_AFF) &
        Print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(nom),coord6
      !--- Structure en temps ---
      if (flag_t) then
	tv => tviz%lv_t(i_v_t)
	! Récupère les pointeurs sur les structures temps-frequence
	tv%tf => liste_temps_freq_pointe(ltf,nom_t)
	if (.not.associated(tv%tf)) then
          print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
          call lecture_stop(f)      
	end if
    
        tv%nomfichier = nom
	tv%flag_eval_fin_simu = flag_eval_fin_simu
	tv%com_format_tf = com_format_tf
	tv%coord = coord6
	tv%nbc       = coord6(direction*2) - coord6(direction*2-1)
	tv%direction = direction
	
        i_v_t = i_v_t + 1
      end if
      
      if (flag_f) then
	tv => tviz%lv_f(i_v_f)
	! Récupère les pointeurs sur les structures temps-frequence
	tv%tf => liste_temps_freq_pointe(ltf,nom_f)
	if (.not.associated(tv%tf)) then
          print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
          call lecture_stop(f)      
	end if
    
        tv%nomfichier = nom
	tv%com_format_tf = com_format_tf
	tv%flag_eval_fin_simu = flag_eval_fin_simu
	tv%coord(:)  = coord6(:)
	tv%nbc       = coord6(direction*2) - coord6(direction*2-1)
	tv%direction = direction
	tv%sn = sn
        i_v_f = i_v_f + 1
      
      end if
    
    case('I')
      read(f%chaine,*) nom_identi,nom1,nom2
    
      if (.not.identifie_deux_nom(nom1,nom2,flag_t,flag_f,nom_t,nom_f)) then
        print '(/,t10,"Nom du type de stockage erroné !!!")'
        call lecture_stop(f)
      end if
    
      ! Lecture des coordonnées
      if (flag_t .and. flag_f) then
        status = ctrl_nb_mot_chaine(f,f%chaine,9)      
        read(f%chaine,*) nom,nom1,nom2, coord6
      else
        status = ctrl_nb_mot_chaine(f,f%chaine,8)      
        read(f%chaine,*) nom,nom1, coord6
      end if  
 
      ! Controle des coordonnées
      if (.not.verif_surface_contour(nom,coord6,type_plan)) call lecture_stop(f)

      !--- Structure en temps ---
      coord6 = origine_cellule6(tm,coord6)
      if (rang==RANG_MAITRE_AFF) &
        Print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(nom),coord6
      if (flag_t) then
	ti => tviz%li_t(i_i_t)
	! Récupère les pointeurs sur les structures temps-frequence
	ti%tf => liste_temps_freq_pointe(ltf,nom_t)
	if (.not.associated(ti%tf)) then
          print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
          call lecture_stop(f)      
	end if
    
        ti%nomfichier = nom
	ti%com_format_tf = com_format_tf
	ti%flag_eval_fin_simu = flag_eval_fin_simu
	ti%coord(:)  = coord6(:)
	ti%nbc1       = coord6(type_plan*2) - coord6(type_plan*2-1) -1
	ti%nbc2       = coord6(PC1(type_plan)*2) - coord6(PC1(type_plan)*2-1) -1
	ti%type_plan = type_plan
	
        i_i_t = i_i_t + 1
      end if
      
      if (flag_f) then
	ti => tviz%li_f(i_i_f)
	! Récupère les pointeurs sur les structures temps-frequence
	ti%tf => liste_temps_freq_pointe(ltf,nom_f)
	if (.not.associated(ti%tf)) then
          print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
          call lecture_stop(f)      
	end if
    
        ti%nomfichier = nom
	ti%com_format_tf = com_format_tf
	ti%flag_eval_fin_simu = flag_eval_fin_simu
	ti%coord(:)  = coord6(:)
	ti%nbc1       = coord6(type_plan*2) - coord6(type_plan*2-1) -1
	ti%nbc2       = coord6(PC1(type_plan)*2) - coord6(PC1(type_plan)*2-1) -1
	ti%type_plan = type_plan
	ti%sn = sn
	
        i_i_f = i_i_f + 1
      end if
    
    case('Z')
      tz => tviz%lz_f(i_z_f)
      status = ctrl_nb_mot_chaine(f,f%chaine,2)      
      read(f%chaine,*) nom_identi, nom_f
      tz%nomfichier = nom_identi
      ! Récupère les pointeurs sur les structures temps-frequence
      tz%tf => liste_temps_freq_pointe(ltf,nom_f)
      if (.not.associated(tz%tf)) then
        print '(/,t10,"Nom du type de stockage en temps erroné !!!")'
        call lecture_stop(f)      
      end if

      tz%com_format_tf = com_format_tf
      tz%flag_eval_fin_simu = flag_eval_fin_simu
      !--- Lecture tension et courant ---
      do i=1,2
	corbeille = lineread(f)
	select case (corbeille(1:1))
          case ('V')
            status = ctrl_nb_mot_chaine(f,f%chaine,7)      
            read(f%chaine,*) nom, coord6
            coord6 = origine_cellule6(tm,coord6)
	    if (rang==RANG_MAITRE_AFF) &
	      Print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(nom),coord6
            ! Controle des coordonnées
            if (.not. verif_dir_tension(nom,coord6,tz%direction)) call lecture_stop(f)
	    tz%coord_v(:)  = coord6(:)
	    tz%nbc_v       = coord6(tz%direction*2) - coord6(tz%direction*2-1)

          case ('I')
            status = ctrl_nb_mot_chaine(f,f%chaine,7)      
            read(f%chaine,*) nom, coord6
            coord6 = origine_cellule6(tm,coord6)
	    if (rang==RANG_MAITRE_AFF) &
	      Print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(nom),coord6
	    ! Controle des coordonnées
	    if (.not.verif_surface_contour(nom,coord6,tz%type_plan)) call lecture_stop(f)
	    tz%coord_i(:)   = coord6(:)
	    tz%nbc_i1       = coord6(tz%type_plan*2) - coord6(tz%type_plan*2-1) -1
	    tz%nbc_i2       = coord6(PC1(tz%type_plan)*2) - coord6(PC1(tz%type_plan)*2-1) -1
	  case default
            print '(/,t10,"Mot clé non reconnu  : ",A )', TRIM(premiermot)
            call lecture_stop(f)

	end select 
      end do
      i_z_f = i_z_f + 1
    
    case default
	!----------------------------------------------------------------------------
	if (corbeille==ID_EVAL_TMAX) then
	  flag_eval_fin_simu = .true.
	else if (corbeille==ID_FIN_EVAL_TMAX) then
	  flag_eval_fin_simu = .false.
  
	!----------------------------------------------------------------------------
	else if (sortie_normalise_lecture(f,premiermot,sn)) then
	  cycle
	!----------------------------------------------------------------------------
	else if (f%nomfinbloc == premiermot) then
          exit ! fin du bloc    
        else if (.not.fichier_sortie_lect_format_freq(premiermot,com_format_tf)) then
          print '(/,t10,"Mot clé non reconnu  : ",A )', TRIM(premiermot)
          call lecture_stop(f)
	end if 
	       
  end select
  
  !-------------- Remplissage de la structure pour les entrée/sortie ----------------
  if ((premiermot(1:1)=='V') .or.(premiermot(1:1)=='I').or.(premiermot(1:1)=='Z')) then
    pres=>tviz%l_res(ires)
    pres%nom = nom_identi
    pres%coord1 = coord6
    pres%interv1 = nom1
    pres%flag_eval_fin_simu = flag_eval_fin_simu
    if (flag_t .and. flag_f) pres%interv2 = nom2    
    if (premiermot(1:1)=='Z') then
      pres%coord1 = tz%coord_v
      pres%coord2 = tz%coord_i
      pres%interv1 = nom_f;pres%interv2 = ''
    end if
    pres%tf_commutateur = com_format_tf;
    pres%flag_eval_fin_simu = flag_eval_fin_simu
    pres%sn  = sn    
    ires=ires+1
  end if
    	
enddo
			
end subroutine resultat_v_i_z_lecture

! ==========================================================
! == Subroutine resultat_v_i_z_ecriture                   ==
! ==========================================================
subroutine resultat_v_i_z_ecriture(f,tviz,tm)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_resultat_v_i_z), intent(inout) :: tviz
type(st_maillage), intent(in) :: tm

integer  :: i,j,commutateur,coord1(6),coord2(6)
logical  :: flag_eval_fin_simu
character(LEN=LEN_MOT_CLE) :: ch1
type(st_resultat_v_i_z_ES),pointer :: pres
type(st_sortie_normalise) :: sn


commutateur = REELLE_IMAGINAIRE

call sortie_normalise_initdefaut(sn)

do i=1,tviz%n_res
  pres =>tviz%l_res(i)
  if (.not.sortie_normalise_compare(sn,pres%sn)) then
    call sortie_normalise_ecriture(f,pres%sn)
    sn = pres%sn
  end if

  if (flag_eval_fin_simu.neqv.pres%flag_eval_fin_simu) then
    flag_eval_fin_simu = pres%flag_eval_fin_simu
    if (flag_eval_fin_simu) then
      write(f%unite,'(T3,A)') ID_EVAL_TMAX
    else
      write(f%unite,'(T3,A)') ID_FIN_EVAL_TMAX
    end if
  end if

  if (commutateur/=pres%tf_commutateur) then
    call fichier_sortie_ecrit_format(f,pres%tf_commutateur)
    commutateur = pres%tf_commutateur
  end if
  
  ch1 = ''
  if (pres%nom(1:1) == 'V' .or. pres%nom(1:1) == 'I') then
    ch1 = trim(pres%interv1)//' '//trim(pres%interv2)
    coord1 = origine_cellule6(tm,pres%coord1,.true.)
    write(f%unite,'(T3,A,3x,A,3x,6I5)') trim(pres%nom),trim(ch1),coord1
  else if (pres%nom(1:1) == 'Z') then
    write(f%unite,'(T3,A,3x,A)') trim(pres%nom),trim(pres%interv1)
    coord1 = origine_cellule6(tm,pres%coord1,.true.)
    coord2 = origine_cellule6(tm,pres%coord2,.true.)
    write(f%unite,'(T5,A,3x,6I5)') 'V',coord1
    write(f%unite,'(T5,A,3x,6I5)') 'I',coord2
  end if
end do

end subroutine resultat_v_i_z_ecriture

! ==========================================================
! == Subroutine verif_dir_tension                         ==
! == Verifie que les coordonnées sont égales dans deux    ==
! == direction                                            ==
! == Déduit la direction        			  ==
! ==========================================================
logical function verif_dir_tension(nom,coord6,direction)

implicit none

character(LEN=LEN_MOT_CLE), intent(in) :: nom
integer, dimension(1:6), intent(in)    :: coord6
integer, intent(out)  :: direction

integer  :: i,j
logical  :: flagsup

verif_dir_tension = .false.
direction = -1

! coordonnées égales dans 2 directions
j=0
do i=1,3
 if (coord6(i*2-1) == coord6(i*2)) j = j+1
end do

if (j/=2) then
  print '(/,t10,"ATTENTION, ", A, " tension mal définie !!!")', TRIM(nom)
  return
endif

! vérifie que les coordonnées sont correctes dans la direction de la tension
do i=1,3
 if (coord6(i*2) > coord6(i*2-1)) then
   direction = i
   verif_dir_tension = .true.
   return 
 end if
end do

print '(/,t10,"ATTENTION, tension de nom :", A)', TRIM(nom)
print '(/,t10,"Veillez a ce que indices de debut < indices de fin")'

end function verif_dir_tension

! ==========================================================
! == Subroutine verif_surface_contour                     ==
! == Verifie que les six entiers forment une surface      ==
! == Déduit le type de plan      			  ==
! ==========================================================
logical function verif_surface_contour(nom,coord6,type_plan)

implicit none

character(LEN=LEN_MOT_CLE), intent(in) :: nom
integer, dimension(1:6), intent(in)    :: coord6
integer, intent(out)  :: type_plan

integer  :: i,j
logical  :: flagsup

verif_surface_contour = .true.
type_plan = -1

! vérifie que la surface en est une : coordonnées égales dans 1 direction
j=0
do i=1,3
 if (coord6(i*2-1) == coord6(i*2)) then
   j = j+1
   type_plan = PC1(i)   
 end if
end do

if (j==0) then
  verif_surface_contour = .false.
  print '(/,t10,"ATTENTION, ", A, " ne represente pas un contour !!!")', TRIM(nom)
  return
endif

! vérifie que les coordonnées sont correctes (imax>imin)
do i=1,3
 if (coord6(i*2) < coord6(i*2-1)) then
   verif_surface_contour = .false.
   print '(/,t10,"Veillez a ce que indices de debut < indices de fin")'
   return 
 end if
end do
end function verif_surface_contour



!===========================================================	
! Subroutine resultat_v_i_z_init
! Initialisation des variables, lorsque qu'il n'y a pas de lecture du fichier
! d'entrée
!===========================================================	
Subroutine resultat_v_i_z_init(tviz)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_v_i_z), intent(inout) :: tviz

!===  Déclaration des variables  ===

!===   Début des traitements   ===

				
End Subroutine resultat_v_i_z_init
	
!===========================================================	
! Subroutine resultat_v_i_z_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
subroutine resultat_v_i_z_precalcul(tviz,tvol,tc,tfs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_v_i_z), intent(inout) :: tviz
type(st_volume), intent(in)    :: tvol
type(st_champ), intent(in)     :: tc
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nomfichier
character :: c
integer  :: i
type(st_resultat_tension), pointer   :: tv
type(st_resultat_courant), pointer   :: ti
type(st_resultat_impedance), pointer :: tz
!===   Début des traitements   ===

tviz%mantisse_temps     = tfs%mantisse_reel
tviz%mantisse_frequence = tfs%mantisse_entier
write(c,'(I1)') 2  
tviz%mantisse_temps = '(' //c//tfs%mantisse_reel(1:LEN_TRIM(tfs%mantisse_reel))//')'
write(c,'(I1)') 3  
tviz%mantisse_frequence = '(' //c//tfs%mantisse_reel(1:LEN_TRIM(tfs%mantisse_reel))//')'

!--- stockage en temps ---

! tension
do i = 1,tviz%nb_v_t
  tv => tviz%lv_t(i)
  ! Creation des alias sur les composantes de champ
  call resultat_viz_tension_alias(tv%p_e,tv%direction,tv%coord,tc)  
  !call resultat_viz_pas_tension_alias(tv%pas,tv%direction,tv%coord, tvol)
  tv%pas = tvol%dxyz(tv%direction)
  allocate(tv%Vencours)
  call resultat_v_temps_precalcul(tv, tfs)
end do				

! courant
do i = 1,tviz%nb_i_t
  ti => tviz%li_t(i)
  ! Creation des alias sur les composantes de champ
  call resultat_viz_courant_alias(ti%p,ti%type_plan,ti%coord,tc)
  !call resultat_viz_pas_courant_alias(ti%pas,ti%type_plan,ti%coord, tvol)
  ti%pas(1) = tvol%dxyz(ti%type_plan)
  ti%pas(2) = tvol%dxyz(PC1(ti%type_plan))
  allocate(ti%Iencours)
  call resultat_i_temps_precalcul(ti, tfs)
end do				

!--- stockage en frequence ---

! tension
do i = 1,tviz%nb_v_f
  tv => tviz%lv_f(i)
  ! Creation des alias sur les composantes de champ
  call resultat_viz_tension_alias(tv%p_e,tv%direction,tv%coord,tc)
!  call resultat_viz_pas_tension_alias(tv%pas,tv%direction,tv%coord, tvol)
  tv%pas = tvol%dxyz(tv%direction)
  allocate(tv%Vencours)

  call resultat_v_frequence_precalcul(tv, tfs)
end do				

!courant
do i = 1,tviz%nb_i_f
  ti => tviz%li_f(i)
  ! Creation des alias sur les composantes de champ
  call resultat_viz_courant_alias(ti%p,ti%type_plan,ti%coord,tc)
  !call resultat_viz_pas_courant_alias(ti%pas,ti%type_plan,ti%coord, tvol)
  ti%pas(1) = tvol%dxyz(ti%type_plan)
  ti%pas(2) = tvol%dxyz(PC1(ti%type_plan))
  allocate(ti%Iencours)

  call resultat_i_frequence_precalcul(ti, tfs)  
end do				

!impédance
do i = 1,tviz%nb_z_f
  tz => tviz%lz_f(i)
  ! Creation des alias sur les composantes de champ
  call resultat_viz_tension_alias(tz%p_e,tz%direction,tz%coord_v,tc)
  call resultat_viz_courant_alias(tz%p,  tz%type_plan,tz%coord_i,tc)
  !call resultat_viz_pas_tension_alias(tz%pas,tz%direction,tz%coord_v, tvol)
  !call resultat_viz_pas_courant_alias(tz%paso,tz%type_plan,tz%coord_i, tvol)
  tz%pasV = tvol%dxyz(tz%direction)
  tz%pasI(1) = tvol%dxyz(tz%type_plan)
  tz%pasI(2) = tvol%dxyz(PC1(tz%type_plan))
  allocate(tz%Vencours)
  allocate(tz%Iencours)

  call resultat_z_precalcul(tz, tfs)
end do

end Subroutine resultat_v_i_z_precalcul

!===========================================================	
! Subroutine resultat_v_temps_precalcul
! Precalcul pour les tensions
!===========================================================	
subroutine resultat_v_temps_precalcul(tv, tfs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_tension), intent(inout) :: tv
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nomfichier

!===   Début des traitements   ===

if (tfs%format_fichier == FORMAT_ASCII) then  
  nomfichier = fichier_sortie_nom_temps(tfs,tv%nomfichier)

  tv%unite = ouvre_en_ecriture(nomfichier)    				    
end if

end subroutine resultat_v_temps_precalcul

!===========================================================	
! Subroutine resultat_i_temps_precalcul
! Precalcul pour le courant 
!===========================================================	
subroutine resultat_i_temps_precalcul(ti, tfs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_courant), intent(inout) :: ti
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nomfichier

!===   Début des traitements   ===

if (tfs%format_fichier == FORMAT_ASCII) then  
  nomfichier = fichier_sortie_nom_temps(tfs,ti%nomfichier)
  ti%unite = ouvre_en_ecriture(nomfichier)    				    
end if

end subroutine resultat_i_temps_precalcul

!===========================================================	
! Subroutine resultat_v_frequence_precalcul
! Precalcul pour les tensions en frequence
!===========================================================	
subroutine resultat_v_frequence_precalcul(tv, tfs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_tension), intent(inout) :: tv
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nomfichier

!===   Début des traitements   ===

allocate(tv%v_f( 1:tv%tf%n_liste))          
tv%v_f(:) = 0.0          
  
if (tfs%format_fichier == FORMAT_ASCII) then
  nomfichier = fichier_sortie_nom_freq(tfs,tv%nomfichier)
  tv%unite = ouvre_en_ecriture(nomfichier)    				    
end if   				    

end subroutine resultat_v_frequence_precalcul

!===========================================================	
! Subroutine resultat_i_frequence_precalcul
! Precalcul pour les courants en frequence
!===========================================================	
subroutine resultat_i_frequence_precalcul(ti, tfs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_courant), intent(inout) :: ti
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nomfichier

!===   Début des traitements   ===

allocate(ti%i_f( 1:ti%tf%n_liste))          
ti%i_f(:) = 0.0          
  
if (tfs%format_fichier == FORMAT_ASCII) then
  nomfichier = fichier_sortie_nom_freq(tfs,ti%nomfichier)
  ti%unite = ouvre_en_ecriture(nomfichier)    				    
end if   				    

end subroutine resultat_i_frequence_precalcul

!===========================================================	
! Subroutine resultat_z_precalcul
! Precalcul pour les impédance
!===========================================================	
subroutine resultat_z_precalcul(tz, tfs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_impedance), intent(inout) :: tz
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
character(LEN=LEN_MOT_CLE) :: nomfichier

!===   Début des traitements   ===

allocate(tz%v_f( 1:tz%tf%n_liste))          
allocate(tz%i_f( 1:tz%tf%n_liste))          
tz%v_f(:) = 0.0          
tz%i_f(:) = 0.0          
  
if (tfs%format_fichier == FORMAT_ASCII) then
  nomfichier = fichier_sortie_nom_freq(tfs,tz%nomfichier)
  tz%unite = ouvre_en_ecriture(nomfichier)    				    
end if   				    

end subroutine resultat_z_precalcul


!===========================================================	
! Subroutine resultat_viz_tension_alias
! Alias vers les composantes du champ électrique
!===========================================================	
subroutine resultat_viz_tension_alias(p_e,direction,coord, tc)
implicit none
!=== Declaration des arguments ===
real, dimension(:), pointer  :: p_e
integer, intent(in)          :: direction
integer, dimension(6), intent(in) :: coord
type(st_champ), intent(in)   :: tc

!=== Declaration des variables locales ===

!=== debut des traitements ===

! Alias vers les composantes du champ fonction de la direction
if (direction == DIR_X) then
    p_e => tc%ex(coord(XMIN):coord(XMAX)-1,coord(YMIN),coord(ZMIN)) 
else if (direction == DIR_Y) then
    p_e => tc%ey(coord(XMIN),coord(YMIN):coord(YMAX)-1,coord(ZMIN)) 
else  
    p_e => tc%ez(coord(XMIN),coord(YMIN),coord(ZMIN):coord(ZMAX)-1) 
end if

end subroutine resultat_viz_tension_alias


!===========================================================	
! Subroutine resultat_viz_courant_alias
! Alias vers les composantes du champ magnétique
!===========================================================	
subroutine resultat_viz_courant_alias(p,type_plan,coord, tc)
implicit none
!=== Declaration des arguments ===
type(st_pointeur_champ_magnetique), dimension(1:4), intent(inout)  :: p 
integer, intent(in)          :: type_plan
integer, dimension(6), intent(in) :: coord
type(st_champ), intent(in)   :: tc

!=== Declaration des variables locales ===

!=== debut des traitements ===

! Alias vers les composantes du champ fonction de la direction
if (type_plan == PLAN_XOY) then
    p(1)%ph => tc%hx(coord(XMIN)+1:coord(XMAX)-1,coord(YMIN),  coord(ZMIN)) 
    p(2)%ph => tc%hx(coord(XMIN)+1:coord(XMAX)-1,coord(YMAX)-1,coord(ZMIN)) 
    p(3)%ph => tc%hy(coord(XMIN),  coord(YMIN)+1:coord(YMAX)-1,coord(ZMIN)) 
    p(4)%ph => tc%hy(coord(XMAX)-1,coord(YMIN)+1:coord(YMAX)-1,coord(ZMIN)) 

else if (type_plan == PLAN_YOZ) then
    p(1)%ph => tc%hy(coord(XMIN), coord(YMIN)+1:coord(YMAX)-1,coord(ZMIN)) 
    p(2)%ph => tc%hy(coord(XMIN), coord(YMIN)+1:coord(YMAX)-1,coord(ZMAX)-1) 
    p(3)%ph => tc%hz(coord(XMIN), coord(YMIN),  coord(ZMIN)+1:coord(ZMAX)-1) 
    p(4)%ph => tc%hz(coord(XMIN), coord(YMAX)-1,coord(ZMIN)+1:coord(ZMAX)-1) 
else  
    p(1)%ph => tc%hz(coord(XMIN),   coord(YMIN),  coord(ZMIN)+1:coord(ZMAX)-1) 
    p(2)%ph => tc%hz(coord(XMAX)-1, coord(YMIN),  coord(ZMIN)+1:coord(ZMAX)-1) 
    p(3)%ph => tc%hx(coord(XMIN)+1:coord(XMAX)-1, coord(YMIN),  coord(ZMIN)) 
    p(4)%ph => tc%hx(coord(XMIN)+1:coord(XMAX)-1, coord(YMIN),  coord(ZMAX)-1) 
end if

end subroutine resultat_viz_courant_alias

!===========================================================	
! Subroutine resultat_v_i_z_calcul
!  Ecriture des résultats à chaque iteration temporelle
! dans l'intervalle temporel spécifié
!===========================================================	
subroutine resultat_v_i_z_calcul(tviz,tfs)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_v_i_z), intent(inout)   :: tviz
type(st_fichier_sortie), intent(in)   :: tfs

!===  Déclaration des variables  ===
integer :: i,j,k
real    :: tension, courant
type(st_resultat_tension), pointer :: tv
type(st_resultat_courant), pointer :: ti
type(st_resultat_impedance), pointer :: tz

!===   Début des traitements   ===

!--- boucle sur toutes les tensions en temps ---
do i = 1,tviz%nb_v_t
  tv => tviz%lv_t(i)
  if (.not.tv%tf%flag_calcul) cycle ! cas ou on ne stocke pas      
    
  ! Sommation du champ électrique puis stockage
  tension = -SUM(tv%p_e(:))*tv%pas
  !if (tfs%format_fichier == FORMAT_ASCII) then
    write (tv%unite,tviz%mantisse_temps)  tv%tf%liste(tv%tf%curseur), tension
  !end if
  tv%Vencours = tension
end do

!--- boucle sur tous les courants en temps ---
do i = 1,tviz%nb_i_t
  ti => tviz%li_t(i)
  if (.not.ti%tf%flag_calcul) cycle ! cas ou on ne stocke pas      
    
  ! Sommation du champ magnétique puis stockage
  courant = 0.0
  do j = 1, ti%nbc1
    courant =  courant + ti%p(1)%ph(j) * ti%pas(1)
    courant =  courant - ti%p(2)%ph(j) * ti%pas(1)
  end do
  do j = 1, ti%nbc2
    courant =  courant - ti%p(3)%ph(j) * ti%pas(2)
    courant =  courant + ti%p(4)%ph(j) * ti%pas(2)
  end do
  
  !if (tfs%format_fichier == FORMAT_ASCII) then
    write (ti%unite,tviz%mantisse_temps)  ti%tf%liste(ti%tf%curseur), courant
  !end if
  ti%Iencours = courant
  
end do
    
!--- boucle sur les tensions en fréquence ---
do i = 1,tviz%nb_v_f
  tv => tviz%lv_f(i)
  if (.not.tv%tf%flag_calcul) cycle
  ! Sommation du champ électrique
  tension = -SUM(tv%p_e(:))*tv%pas
  ! Transformée de Fourier Discrète
  do k = 1, tv%tf%n_liste  ! boucle sur les frequences
      tv%v_f(k) = tv%v_f(k) + tension * tv%tf%expi2pitf(k)
  end do
  tv%Vencours = tension

end do

!--- boucle sur les courants en fréquence ---
do i = 1,tviz%nb_i_f
  ti => tviz%li_f(i)
  if (.not.ti%tf%flag_calcul) cycle
  ! Sommation du champ magnétique
  courant = 0.0
  do j = 1, ti%nbc1
    courant =  courant + ti%p(1)%ph(j) * ti%pas(1)
    courant =  courant - ti%p(2)%ph(j) * ti%pas(1)
  end do
  do j = 1, ti%nbc2
    courant =  courant - ti%p(3)%ph(j) * ti%pas(2)
    courant =  courant + ti%p(4)%ph(j) * ti%pas(2)
  end do
  
  ! Transformée de Fourier Discrète
  do k = 1, ti%tf%n_liste  ! boucle sur les frequences
      ti%i_f(k) = ti%i_f(k) + courant * ti%tf%expi2pitf(k)
  end do
  ti%Iencours = courant
  
end do

!--- boucle sur toutes les impédances ---
do i = 1,tviz%nb_z_f
  tz => tviz%lz_f(i)
  if (.not.tz%tf%flag_calcul) cycle

  ! Sommation du champ électrique
  tension = -SUM(tz%p_e(:))*tz%pasV

  ! Sommation du champ magnétique
  courant = 0.0
  do j = 1, tz%nbc_i1
    courant =  courant + tz%p(1)%ph(j) * tz%pasI(1)
    courant =  courant - tz%p(2)%ph(j) * tz%pasI(1)
  end do
  do j = 1, tz%nbc_i2
    courant =  courant - tz%p(3)%ph(j) * tz%pasI(2)
    courant =  courant + tz%p(4)%ph(j) * tz%pasI(2)
  end do
  
  ! Transformée de Fourier Discrète
  do k = 1, tz%tf%n_liste  ! boucle sur les frequences
      tz%v_f(k) = tz%v_f(k) + tension * tz%tf%expi2pitf(k)
      tz%i_f(k) = tz%i_f(k) + courant * tz%tf%expi2pitf(k)
  end do
  tz%Iencours=courant
  tz%Vencours=tension
end do

end Subroutine resultat_v_i_z_calcul



!===========================================================	
! Subroutine resultat_v_i_z_impression
!  Calcul après les itérations temporelles
!===========================================================	
subroutine resultat_v_i_z_impression(tviz,tfs,t_gene)
!=== Déclaration des arguments ===
type(st_resultat_v_i_z), intent(inout) :: tviz
type(st_fichier_sortie), intent(in) :: tfs
type(st_generateur), intent(inout) :: t_gene

!===  Déclaration des variables  ===
integer  :: i
complex, dimension(:), pointer :: z_f
type(st_resultat_tension), pointer :: tv
type(st_resultat_courant), pointer :: ti
type(st_resultat_impedance), pointer :: tz
complex, dimension(:), allocatable   :: tnorm_f

!===   Début des traitements   ===

!--- boucle sur les tension en temps ---
do i = 1,tviz%nb_v_t
  tv => tviz%lv_t(i)
  ! stockage ascii
  if (tfs%format_fichier == FORMAT_ASCII) then
    close(tv%unite)    				    
  end if
  
end do  

!--- boucle sur les courants en temps ---
do i = 1,tviz%nb_i_t
  ti => tviz%li_t(i)
  ! stockage ascii
  if (tfs%format_fichier == FORMAT_ASCII) then
    close(ti%unite)    				    
  end if
  
end do  

!--- boucle sur les tensions en frequence ---
do i = 1,tviz%nb_v_f
  tv => tviz%lv_f(i)
  
  !---------- Normalisation ?-----------------
  if (tv%sn%flag_normalisation) then
    allocate(tnorm_f(1:tv%tf%n_liste))
    call Normalise_Reponse_Freq(t_gene,tv%tf,tv%sn,tnorm_f)
    ! normalisation pour chaque composante
    tv%v_f(:) = tv%v_f(:) / tnorm_f(:)
    deallocate(tnorm_f)
  end if
  ! stockage ascii      
  if (tfs%format_fichier == FORMAT_ASCII) then
    call impression_ascii_complexe(tv%unite, &
             tviz%mantisse_frequence, tv%com_format_tf, &
	     tv%tf%liste, tv%v_f, tv%tf%n_liste,1)      
    close(tv%unite)    				    
  end if
end do				

!--- boucle sur les courants en frequence ---
do i = 1,tviz%nb_i_f
  ti => tviz%li_f(i)
  
  !---------- Normalisation ?-----------------
  if (ti%sn%flag_normalisation) then
    allocate(tnorm_f(1:ti%tf%n_liste))
    call Normalise_Reponse_Freq(t_gene,ti%tf,ti%sn,tnorm_f)
    ! normalisation pour chaque composante
    ti%i_f(:) = ti%i_f(:) / tnorm_f(:)
    deallocate(tnorm_f)
  end if
  ! stockage ascii      
  if (tfs%format_fichier == FORMAT_ASCII) then
    call impression_ascii_complexe(ti%unite, &
             tviz%mantisse_frequence, ti%com_format_tf, &
	     ti%tf%liste, ti%i_f, ti%tf%n_liste,1)      
    close(ti%unite)    				    
  end if
end do				

!--- boucle sur les impédances en frequence ---
do i = 1,tviz%nb_z_f
  tz => tviz%lz_f(i)
  ! calcul de l'impédance
  allocate(z_f(1:tz%tf%n_liste))
  z_f = tz%v_f / tz%i_f
  
  ! stockage ascii      
  if (tfs%format_fichier == FORMAT_ASCII) then
    call impression_ascii_complexe(tz%unite, &
             tviz%mantisse_frequence, tz%com_format_tf, &
	     tz%tf%liste, z_f, tz%tf%n_liste,1)      
    close(tz%unite)    				    
  end if
  
  deallocate(z_f) 
end do				
				
End Subroutine resultat_v_i_z_impression


!===========================================================	
! Subroutine resultat_v_i_z_deallocate
!  Libération de toute la mémoire allouée
!===========================================================	
subroutine resultat_v_i_z_deallocate(tviz)
implicit none
!=== Déclaration des arguments ===
type(st_resultat_v_i_z), intent(inout) :: tviz

!===  Déclaration des variables  ===
integer :: i

!=== Debut des traitements ===

do i =1, tviz%nb_v_f
  if (ASSOCIATED(tviz%lv_f(i)%v_f)) deallocate(tviz%lv_f(i)%v_f)
end do			
do i =1, tviz%nb_i_f
  if (ASSOCIATED(tviz%li_f(i)%i_f)) deallocate(tviz%li_f(i)%i_f)
end do			
do i =1, tviz%nb_z_f
  if (ASSOCIATED(tviz%lz_f(i)%i_f)) deallocate(tviz%lz_f(i)%i_f)
  if (ASSOCIATED(tviz%lz_f(i)%v_f)) deallocate(tviz%lz_f(i)%v_f)
end do			

if (tviz%nb_v_t>0) deallocate(tviz%lv_t)
if (tviz%nb_v_f>0) deallocate(tviz%lv_f)
if (tviz%nb_i_t>0) deallocate(tviz%li_t)
if (tviz%nb_i_f>0) deallocate(tviz%li_f)
if (tviz%nb_z_f>0) deallocate(tviz%lz_f)

end subroutine resultat_v_i_z_deallocate	

end module mfdtd_resultat_v_i_z


