!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_source_courant.f90
!// Purpose:      Injection de sources de courant transparentes
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Avril 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
!================================================================
! Traitement des sources de courant cellulaires
!================================================================
module mfdtd_source_courant

use m_constantes
use m_constantes_scientifiques
use m_entree , only : st_fichier, &
                                  lineread, &
				  multibackspace, &
				  nb_mot_chaine, &
				  ctrl_nb_mot_chaine, &
				  lecture_stop, &
				  apres_fin_bloc, &
				  numbackspace_set
				  
use m_verif_data, only : verif_surface
use m_generateur, only : st_generateur, &
                              generateur_calcul_t,&
			      generateur_existe_actif
use m_maillage			      
use mfdtd_volume, only : st_volume, &
                          volume_pasE_local, &
                          volume_pasH_local, &
			  volume_surfaceE_local
use mfdtd_champ, only : st_champ
use mfdtd_milieu_volume, only : st_milieu_volume, &
                                 milieu_volume_extrait_epsilon, &
				 milieu_volume_extrait_sige
				 
implicit none		

!=== Déclaration des constantes ===
real, parameter    :: RESISTANCE_MAX = 1.0e8 
integer, parameter :: ATTENUATION_EXPO       = 1
integer, parameter :: ATTENUATION_EXPO_CARRE = 2
!=== Déclaration des types ===

type st_pointeur_reel
  real, pointer  :: e
  real, pointer, dimension(:)  :: h1, h2
end type st_pointeur_reel

type st_source_canal_foudre
  integer    :: origine(3)   ! origine du canal (origine aussi des phases du canal)
  integer    :: zmin,zmax    ! hauteur min et max du canal
  integer    :: nbcell       ! longueur du canal en nombre de cellules
  real       :: indice_phase ! >=1.0 pondération de la vitesse de propagation
  real       :: E0           ! Amplitude
  real       :: phase, retard
  character(LEN=LEN_MOT_CLE) :: nom, nomgene ! nom du canal, nom de la fonction source
  real, pointer, dimension(:)  :: Isrc  ! contient la fonction source
  integer, pointer, dimension(:)  :: nd ! retard en nombre de dt
  real, pointer, dimension(:)  :: pd    ! facteur d'interpolation temporel
  
  ! atténuation spatiale
  integer    :: type_att                ! type d'atténuation spatial
  real       :: facteur_att             ! facteur d'atténuation spatial
  
  real, pointer, dimension(:)  :: e,ce   ! champ électrique et coefficient pour la mise à jour du champ électrique
end type st_source_canal_foudre


!--------- la source_courant source_courant -------------------
type st_une_source_courant
  logical  :: flag_source_parfaite
  integer  :: direction, plan
  real  :: phase, retard
  real  :: E0, R0
  integer :: nb1, nb2 ! nb sources ds les 2 direction du plan
  integer  :: coord6_ijk(6), saut_ijk(3)
  character(LEN=LEN_MOT_CLE) :: nom, nomgene
  real, pointer, dimension(:)  :: Isrc
  real, pointer, dimension(:)  :: d1,d2
  real, pointer, dimension(:,:)  :: Eprec, ce1, ce2
  type(st_pointeur_reel), pointer, dimension(:,:)  :: p ! tableau d'alias vers les composantes du champ 
end type st_une_source_courant

!---------- La structure principale ------------------------------
type st_source_courant
  logical  :: flag_data
  integer  ::  nbsource
  type(st_une_source_courant), pointer, dimension(:)  :: l_sc
  
  integer  :: nbcanal
  type(st_source_canal_foudre), pointer, dimension(:) :: l_scf
end type st_source_courant
 
! Pas de variables globales...

contains
!===========================================================	
! Subroutine source_courant_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine source_courant_initdefaut(tsc)
!=== Déclaration des arguments ===
type(st_source_courant) :: tsc
!===  Déclaration des variables  ===

!===   Début des traitements   ===
tsc%flag_data = .false.
tsc%nbsource = 0
tsc%nbcanal = 0
				
End Subroutine source_courant_initdefaut

!===========================================================	
! Subroutine source_courant_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine une_source_courant_initdefaut(t1sc)
!=== Déclaration des arguments ===
type(st_une_source_courant)  :: t1sc
!===  Déclaration des variables  ===

!===   Début des traitements   ===
t1sc%direction = 0
t1sc%phase = 0.0
t1sc%retard = 0.0
t1sc%E0= 1.0
t1sc%R0= 0.0
t1sc%coord6_ijk = 0
t1sc%saut_ijk = 	1
t1sc%flag_source_parfaite = .false.
nullify(t1sc%Isrc);nullify(t1sc%d1);nullify(t1sc%d2);
nullify(t1sc%Eprec);
nullify(t1sc%ce1);nullify(t1sc%ce2);
nullify(t1sc%p);
End Subroutine une_source_courant_initdefaut

!===========================================================	
! Subroutine une_source_courant_copy
!===========================================================	
Subroutine une_source_courant_copy(scin,scout)
!=== Déclaration des arguments ===
type(st_une_source_courant),intent(in)  :: scin
type(st_une_source_courant),intent(out)  :: scout
!===  Déclaration des variables  ===

!===   Début des traitements   ===
scout%nom = scin%nom
scout%nomgene = scin%nomgene
scout%direction = scin%direction
scout%plan = scin%plan
scout%phase = scin%phase
scout%retard = scin%retard
scout%E0= scin%E0 ; scout%R0= scin%R0
scout%coord6_ijk = scin%coord6_ijk
scout%saut_ijk = scin%saut_ijk
scout%nb1 = scin%nb1 ;scout%nb2 = scin%nb2
scout%flag_source_parfaite = scin%flag_source_parfaite
nullify(scout%Isrc);nullify(scout%d1);nullify(scout%d2);
nullify(scout%Eprec);
nullify(scout%ce1);nullify(scout%ce2);
nullify(scout%p);
End Subroutine une_source_courant_copy


!===========================================================	
! Subroutine source_courant_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine source_courant_lecture(f,tsc,tm,g)
!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_source_courant), intent(inout)      :: tsc
type(st_maillage), intent(in) :: tm
type(st_generateur), intent(inout)    :: g

!===  Déclaration des variables  ===
integer  :: i, isrc,icanal, nbmot, plan,dir, status
integer :: type_att
real  :: phase, retard, facteur_att
character(LEN=LEN_MOT_CLE) :: corbeille, nom, premiermot
type(st_une_source_courant), pointer  :: t1sc
type(st_source_canal_foudre),pointer  :: t1scf
!===   Début des traitements   ===

!--- On compte le nombre de sources de courant ---
isrc = 0;
icanal = 0;
! -- Mise a zero de f%numbackspace --
call numbackspace_set(f,0)
do
  corbeille = lineread(f,1)
  if (corbeille==f%nomfinbloc) then
    exit
  else if (corbeille(1:1) == 'I') then
     isrc = isrc+1
  else if (corbeille(1:12) == 'canal_foudre') then
     icanal = icanal+1
  end if
enddo

! -- On remonte au début du bloc --
i = multibackspace(f)
if (isrc == 0 .and. icanal == 0) then
  print '(/,t10," !!!! Avertissement : Pas de sources de courant de définies")'
  tsc%flag_data = .false.
  call apres_fin_bloc(f)
  return
end if
tsc%flag_data = .true.

tsc%nbsource = isrc
tsc%nbcanal  = icanal
if (tsc%nbsource>0) then
  allocate(tsc%l_sc(isrc))
end if
if (tsc%nbcanal>0) then
  allocate(tsc%l_scf(icanal))
end if
isrc   = 1
icanal = 1
phase  = 0.0
retard = 0.0
type_att = 0
facteur_att = 0.0
! -- Lecture des donnees --
do
  premiermot = lineread(f)
  if (premiermot(1:2) == 'IX' .or. premiermot(1:2) == 'IY' .or. premiermot(1:2) == 'IZ') then 
    t1sc => tsc %l_sc(isrc)
    isrc = isrc+1
    call une_source_courant_initdefaut(t1sc)
    nbmot = nb_mot_chaine(f%chaine)  
    select case (nbmot)
      case (9)
        read(f%chaine,*,ERR=999)  t1sc%nom, t1sc%nomgene, t1sc%E0, t1sc%coord6_ijk(1:6)
	t1sc%R0 = RESISTANCE_MAX
      case (10)
        read(f%chaine,*,ERR=999)  t1sc%nom, t1sc%nomgene, t1sc%E0, t1sc%R0, t1sc%coord6_ijk(1:6)
      case (12)
        read(f%chaine,*,ERR=999)  t1sc%nom, t1sc%nomgene, t1sc%E0, t1sc%coord6_ijk(1:6), &
                             t1sc%saut_ijk(1:3)
	t1sc%R0 = RESISTANCE_MAX
      case (13)
        read(f%chaine,*,ERR=999)  t1sc%nom, t1sc%nomgene, t1sc%E0, t1sc%R0, t1sc%coord6_ijk(1:6), &
                             t1sc%saut_ijk(1:3)
      case default
        print '(/,t10,"Nbre de mots de la chaine faux !")'
        call lecture_stop(f)      
      end select
    t1sc%phase       = phase
    t1sc%retard      = retard
    
    if (.not.generateur_existe_actif(g,t1sc%nomgene)) then
      call lecture_stop(f)
    end if

    if (.not.verif_surface(t1sc%nom,t1sc%coord6_ijk(1:6),t1sc%plan)) call lecture_stop(f)
    t1sc%coord6_ijk= origine_cellule6(tm,t1sc%coord6_ijk)

      ! direction  
    if (premiermot(1:2) == 'IX') then
      t1sc%direction = DIRECTION_X
    else if (premiermot(1:2) == 'IY') then
      t1sc%direction = DIRECTION_Y    
    else
      t1sc%direction = DIRECTION_Z
    end if
    
    plan = t1sc%plan    
    ! nbre de sources dans le plan
    t1sc%nb1 = (t1sc%coord6_ijk(plan*2) - t1sc%coord6_ijk(plan*2-1)) / t1sc%saut_ijk(plan) + 1
    t1sc%nb2 = (t1sc%coord6_ijk(PC1(plan)*2) - t1sc%coord6_ijk(PC1(plan)*2-1)) / t1sc%saut_ijk(PC1(plan)) + 1 
    ! rectif dans le cas ou la direction du courant est dans le plan de sources de courant
    dir = t1sc%direction
    if (plan == dir.and.(t1sc%saut_ijk(dir)==1)) t1sc%nb1 = t1sc%nb1 - 1
    if (PC1(plan) == dir .and.(t1sc%saut_ijk(dir)==1)) t1sc%nb2 = t1sc%nb2 - 1
    if (rang==RANG_MAITRE_AFF) then
      print '(t10,A," : Plan et Direction =",2I2," ; les coordonnées = ",6I6)',&
              trim(t1sc%nom),plan,t1sc%direction,t1sc%coord6_ijk
      print '(t10,A," : Nbre de sources Nb1*Nb2 = ",I3,"*",I3)',trim(t1sc%nom),t1sc%nb1,t1sc%nb2
    end if
  !-----------------------------------------------------------------------------------  
  else if (premiermot == 'canal_foudre') then
    t1scf => tsc %l_scf(icanal)
    icanal = icanal+1
    status = ctrl_nb_mot_chaine(f,f%chaine,8)
    read(f%chaine,*,ERR=999)  t1scf%nom, t1scf%nomgene, t1scf%E0, t1scf%origine(1:3), &
                              t1scf%nbcell,t1scf%indice_phase
    t1scf%phase       = phase
    t1scf%retard      = retard
    t1scf%facteur_att = facteur_att
    t1scf%type_att    = type_att
    t1scf%origine= origine_cellule3(tm,t1scf%origine)
    if (rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(t1scf%nom),t1scf%origine
    
    if (.not.generateur_existe_actif(g,t1scf%nomgene)) then
      call lecture_stop(f)
    end if

  !-----------------------------------------------------------------------------------  
  else if (premiermot == 'phase') then
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*,ERR=999)  corbeille, phase
  !-----------------------------------------------------------------------------------  
  else if (premiermot == 'attenuation_expo') then
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*,ERR=999)  corbeille, facteur_att
    type_att = ATTENUATION_EXPO
  !-----------------------------------------------------------------------------------  
  else if (premiermot == 'attenuation_expo_carre') then
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*,ERR=999)  corbeille, facteur_att
    type_att = ATTENUATION_EXPO_CARRE
  !-----------------------------------------------------------------------------------  
  else if (premiermot == 'retard') then
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*,ERR=999)  corbeille, retard
  
  !-----------------------------------------------------------------------------------  
  else if (f%nomfinbloc == premiermot) then
    exit ! -- Fin du bloc --
  else     
    print *,'(/,t10,"Mot clé non reconnu  :",A )', TRIM(premiermot) 
    call lecture_stop(f)
  end if 
enddo

return

999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(f)

End Subroutine source_courant_lecture


! ==========================================================
! == Subroutine resultat_champ_ecriture                   ==
! ==========================================================
subroutine source_courant_ecriture(f,tsc,tm)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_source_courant), intent(inout) :: tsc
type(st_maillage), intent(in) :: tm

integer  :: i,pt(3),coord(6)
real     :: phase, retard
type(st_une_source_courant), pointer  :: t1sc
type(st_source_canal_foudre), pointer :: t1scf

if (.not.tsc%flag_data) return

!----- Ecriture des sources de courant ---------
phase = 0.0; retard = 0.0;
do i=1,tsc%nbsource
  t1sc => tsc%l_sc(i)
  if (t1sc%phase /= phase) then
    phase = t1sc%phase
    write(f%unite,'(T3,A,3x,G14.6)') 'phase',t1sc%phase
  end if
  if (t1sc%retard /= retard) then
    retard = t1sc%retard
    write(f%unite,'(T3,A,3x,G14.6)') 'retard',t1sc%retard
  end if

  coord= origine_cellule6(tm,t1sc%coord6_ijk,.true.)

  if (t1sc%R0 /= RESISTANCE_MAX) then
    write(f%unite,'(T3,A,1x,A,1x,2G14.6,1x,6I5,1x,3I4)') trim(t1sc%nom), trim(t1sc%nomgene), &
                        t1sc%E0, t1sc%R0,coord(1:6),t1sc%saut_ijk(1:3)
  else
    write(f%unite,'(T3,A,1x,A,1x,G14.6,1x,6I5,1x,3I4)') trim(t1sc%nom), trim(t1sc%nomgene), &
                        t1sc%E0,coord(1:6),t1sc%saut_ijk(1:3)  
  end if
end do

!----- Ecriture des sources pour canal foudre ---------
phase = 0.0; retard = 0.0;
do i=1,tsc%nbcanal
  t1scf=> tsc%l_scf(i)
  if (t1scf%phase /= phase) then
    phase = t1scf%phase
    write(f%unite,'(T3,A,3x,G14.6)') 'phase',t1scf%phase
  end if
  
  if (t1scf%retard /= retard) then
    retard = t1scf%retard
    write(f%unite,'(T3,A,3x,G14.6)') 'retard',t1scf%retard
  end if
  
  if (t1scf%type_att==ATTENUATION_EXPO) then
    write(f%unite,'(T3,A,3x,G14.6)') 'attenuation_expo',t1scf%facteur_att
  else if (t1scf%type_att==ATTENUATION_EXPO_CARRE) then
    write(f%unite,'(T3,A,3x,G14.6)') 'attenuation_expo_carre',t1scf%facteur_att
  end if
  
  pt= origine_cellule3(tm,t1scf%origine(1:3),.true.)
  write(f%unite,'(T3,A,1x,A,3x,G14.6,1x,4I5,1x,F8.3)')  trim(t1scf%nom), trim(t1scf%nomgene), &
			t1scf%E0, pt, t1scf%nbcell,t1scf%indice_phase

end do
  
end subroutine source_courant_ecriture

!===========================================================	
! Subroutine source_courant_init
! Initialisation des variables, lorsque qu'il n'y a pas de lecture du fichier
! d'entrée
!===========================================================	
Subroutine source_courant_init()
implicit none
!=== Déclaration des arguments ===

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine source_courant_init
	
!===========================================================	
! Subroutine source_courant_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
Subroutine source_courant_precalcul(tsc, v, tm, t_gene)
implicit none
!=== Déclaration des arguments ===
type(st_source_courant)               :: tsc
type(st_volume), intent(in) :: v
type(st_milieu_volume), intent(in)  :: tm
type(st_generateur), intent(inout)     :: t_gene

!===  Déclaration des variables  ===
integer  :: n,i,j, nb1, nb2
integer, dimension(3)  :: c3
real  :: dervaleur
real  :: di, s, eps
real, dimension(:,:,:), pointer  :: epsilon
type(st_une_source_courant), pointer  :: t1sc

!===   Début des traitements   ===

call source_canal_foudre_precalcul(tsc, v, tm, t_gene)

! --- boucle sur toutes les sources ---
do n=1, tsc%nbsource
  t1sc =>tsc%l_sc(n)
  
  ! Calcul du générateur
  allocate(t1sc%Isrc(v%ntinc))
  t1sc%Isrc = 0.0
  dervaleur = generateur_calcul_t(t_gene, t1sc%nomgene, &
         t1sc%E0, t1sc%phase, t1sc%retard, v%dt, v%ntinc, t1sc%Isrc)
  
  if (t1sc%R0>=RESISTANCE_MAX) t1sc%flag_source_parfaite = .true.
  
  !--- Calcul des coefficients multiplicatifs des champs et 
  !     de la source de courant ---
  
  t1sc%saut_ijk(PC2(t1sc%plan)) = 0.0 ! pas normale au plan de courant nul

  allocate(t1sc%d1(t1sc%nb1))
  allocate(t1sc%d2(t1sc%nb2))
  allocate(t1sc%ce1(t1sc%nb1,t1sc%nb2))
  allocate(t1sc%ce2(t1sc%nb1,t1sc%nb2))
  allocate(t1sc%Eprec(t1sc%nb1,t1sc%nb2))
  t1sc%Eprec(:,:) = 0.0
  !print '(T10,"source_courant_precalcul, flag_Iparfait=",L)',t1sc%flag_source_parfaite
  if (t1sc%flag_source_parfaite) then
    t1sc%ce1(:,:) = 1.0
    do j=1, t1sc%nb2
    do i=1, t1sc%nb1
      c3 = source_courant_coord_pt_surf(t1sc, i,j)      
      ! on récupère la surface de la densité de sourant
      s = volume_surfaceE_local(v,c3(1),c3(2),c3(3),PC1(t1sc%direction))
      ! on récupère la permittivité
      eps = milieu_volume_extrait_epsilon(tm,c3(1),c3(2),c3(3),t1sc%direction)
      t1sc%ce2(i,j) = v%dt/(eps*s)
      !print '(T15,"source_courant_precalcul, c3=",3I4)',c3
      !print '(T15,"source_courant_precalcul, s=",ES15.3)',s
      !print '(T15,"source_courant_precalcul, eps=",ES15.3)',eps
    end do; end do
  else
    do j=1, t1sc%nb2
    do i=1, t1sc%nb1
      c3 = source_courant_coord_pt_surf(t1sc, i,j)      
      ! on récupère la permittivité
      eps = milieu_volume_extrait_epsilon(tm,c3(1),c3(2),c3(3),t1sc%direction)
      ! on récupère le pas suivant z à la bonne position
      di = volume_pasE_local(v,c3(1),c3(2),c3(3),t1sc%direction) 
      ! on récupère la surface de la densité de sourant
      s = volume_surfaceE_local(v,c3(1),c3(2),c3(3),PC1(t1sc%direction))

      ! on récupère les pas spatiaux pour la circulation de H --> I
      t1sc%d1(i) = volume_pasH_local(v,c3(1),c3(2),c3(3),PC1(t1sc%direction))
      t1sc%d2(j) = volume_pasH_local(v,c3(1),c3(2),c3(3),PC2(t1sc%direction))
      
      t1sc%ce1(i,j) = ((1-di*v%dt/(2*eps*s*t1sc%R0))/(1+di*v%dt/(2*eps*s*t1sc%R0)))
      t1sc%ce2(i,j) = (v%dt/(eps*s))/(1+di*v%dt/(2*eps*s*t1sc%R0))
      
    end do; end do
  
  end if
end do		

End Subroutine source_courant_precalcul


!===========================================================	
! Subroutine source_canal_foudre_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
Subroutine source_canal_foudre_precalcul(tsc, v, tm, t_gene)
implicit none
!=== Déclaration des arguments ===
type(st_source_courant)               :: tsc
type(st_volume), intent(in) :: v
type(st_milieu_volume), intent(in)  :: tm
type(st_generateur), intent(inout)     :: t_gene

!===  Déclaration des variables  ===
integer  :: n,k,kmin,kmax
real  :: dervaleur
real  :: i_cdt,tr,s, eps,sigma
type(st_source_canal_foudre), pointer  :: t1sc

!===   Début des traitements   ===

! --- boucle sur toutes les canaux foudre ---
do n=1, tsc%nbcanal
  t1sc =>tsc%l_scf(n)
  
  ! Calcul du générateur
  allocate(t1sc%Isrc(-1:v%ntinc))
  t1sc%Isrc = 0.0
  dervaleur = generateur_calcul_t(t_gene, t1sc%nomgene, &
         t1sc%E0, t1sc%phase, t1sc%retard, v%dt, v%ntinc, t1sc%Isrc(1:v%ntinc))
  

  ! Calcul de la hauteur du canal
  kmin = t1sc%origine(3)
  kmax = t1sc%origine(3) + t1sc%nbcell
  if (t1sc%nbcell<0) then 
    kmin = t1sc%origine(3) -t1sc%nbcell
    kmax = t1sc%origine(3)
  end if
  
  if (kmin <= 1)  kmin = v%kvold
  if (kmax >= v%nz+1) kmax = v%kvolf+1
  
  if (rang==RANG_MAITRE_AFF) print '(T10,"(zmin,zmax) = ",2I4)',kmin,kmax
  t1sc%zmin = kmin
  t1sc%zmax = kmax
  !--- Calcul des coefficients multiplicatifs des champs et 
  !     de la source de courant ---
  allocate(t1sc%ce(1:kmax-kmin))
  allocate(t1sc%nd(1:kmax-kmin))
  allocate(t1sc%pd(1:kmax-kmin))
  
  s = v%dxyz(1)*v%dxyz(2)
  i_cdt = t1sc%indice_phase / (CVIDE*v%dt)
  t1sc%ce(:) = 0.0
  do k=1, kmax-kmin
    ! on récupère la permittivité et la conductivité
    eps   = milieu_volume_extrait_epsilon(tm,t1sc%origine(1),t1sc%origine(2),k+kmin-1,DIRECTION_Z)
    sigma = milieu_volume_extrait_sige   (tm,t1sc%origine(1),t1sc%origine(2),k+kmin-1,DIRECTION_Z)
    t1sc%ce(k) = (v%dt/(eps*s) ) / (1+sigma*v%dt/(2*eps))
    
    ! calcul retard en nbre de dt
    tr = (k-1)*v%dxyz(3) *i_cdt
    t1sc%nd(k) = floor(tr)
    t1sc%pd(k) = tr - real(t1sc%nd(k))
  end do
  
  ! Réglage de l'atténuation
  if (t1sc%type_att == ATTENUATION_EXPO) then
    if (rang==RANG_MAITRE_AFF) print '(T10,"Atténuation expo = ",F8.2)',t1sc%facteur_att
    do k=1, kmax-kmin
      t1sc%ce(k) = t1sc%ce(k) * exp(- ((k-1)*v%dxyz(3)/t1sc%facteur_att))
    end do
  else if (t1sc%type_att == ATTENUATION_EXPO_CARRE) then
    if (rang==RANG_MAITRE_AFF) print '(T10,"Atténuation expo carré= ",F8.2)',t1sc%facteur_att
    do k=1, kmax-kmin
      t1sc%ce(k) = t1sc%ce(k) * exp(- ((k-1)*v%dxyz(3)/t1sc%facteur_att)**2)
    end do
  end if
end do		

End Subroutine source_canal_foudre_precalcul


!===========================================================	
! Subroutine source_courant_coord_pt_surf
! Retourne les coordonnées d'un point de la surface de courant
!===========================================================	
function source_courant_coord_pt_surf(t1sc, i,j) result(c3)
implicit none
!=== Déclaration des arguments ===
type(st_une_source_courant)  :: t1sc
integer, intent(in)  :: i,j

integer, dimension(3) :: c3
!===  Déclaration des variables  ===

!===   Début des traitements   ===

if (t1sc%plan == PLAN_XOY) then 
  c3(1) = t1sc%coord6_ijk(XMIN) + (i-1) * t1sc%saut_ijk(DIRECTION_X)
  c3(2) = t1sc%coord6_ijk(YMIN) + (j-1) * t1sc%saut_ijk(DIRECTION_Y)
  c3(3) = t1sc%coord6_ijk(ZMIN)

else if (t1sc%plan == PLAN_YOZ) then 
  c3(1) = t1sc%coord6_ijk(XMIN) 
  c3(2) = t1sc%coord6_ijk(YMIN) + (i-1) * t1sc%saut_ijk(DIRECTION_Y)
  c3(3) = t1sc%coord6_ijk(ZMIN) + (j-1) * t1sc%saut_ijk(DIRECTION_Z)

else
  c3(1) = t1sc%coord6_ijk(XMIN) + (j-1) * t1sc%saut_ijk(DIRECTION_X)
  c3(2) = t1sc%coord6_ijk(YMIN) 
  c3(3) = t1sc%coord6_ijk(ZMIN) + (i-1) * t1sc%saut_ijk(DIRECTION_Z)

end if

end function source_courant_coord_pt_surf

!===========================================================	
! Subroutine source_courant_alias_champ
! Alias vers les composantes du champ électrique et magnétique
!===========================================================	
subroutine source_courant_alias_champ(tsc, tc)
implicit none
!=== Déclaration des arguments ===
type(st_source_courant)               :: tsc
type(st_champ), intent(in) :: tc
!===  Déclaration des variables  ===
integer  :: i,j,n, ix, iy, iz
integer, dimension(3)  :: c3
type(st_une_source_courant), pointer  :: t1sc
type(st_source_canal_foudre), pointer  :: t1scf

!===   Début des traitements   ===

! --- boucle sur toutes les sources ---
do n=1, tsc%nbsource
  t1sc =>tsc%l_sc(n)
  !print '(T10,"source_courant_alias, n=",I2)', n
  allocate(t1sc%p(t1sc%nb1,t1sc%nb2))
  do j=1, t1sc%nb2
  do i=1, t1sc%nb1
    c3 = source_courant_coord_pt_surf(t1sc, i,j)   
    ix = c3(1)
    iy = c3(2)
    iz = c3(3)   
    !print '(T15,"source_courant_alias, (ix,iy,iz)=",3I4)', ix,iy,iz
    if (t1sc%direction == DIRECTION_X) then
      t1sc%p(i,j)%e => tc%ex(ix, iy, iz)
      t1sc%p(i,j)%h1 => tc%hy(ix, iy, iz-1:iz)
      t1sc%p(i,j)%h2 => tc%hz(ix, iy-1:iy, iz)
      
    else if (t1sc%direction == DIRECTION_Y) then
      t1sc%p(i,j)%e => tc%ey(ix, iy, iz)
      t1sc%p(i,j)%h1 => tc%hz(ix-1:ix, iy, iz)
      t1sc%p(i,j)%h2 => tc%hx(ix, iy, iz-1:iz)
    
    else 
      t1sc%p(i,j)%e => tc%ez(ix, iy, iz)
      t1sc%p(i,j)%h1 => tc%hx(ix, iy-1:iy, iz)
      t1sc%p(i,j)%h2 => tc%hy(ix-1:ix, iy, iz)
    
    end if
  end do; end do

end do

! boucle sur la liste des canaux foudre
do n=1, tsc%nbcanal
  t1scf =>tsc%l_scf(n)
  t1scf%e => tc%ez(t1scf%origine(1),t1scf%origine(2),t1scf%zmin:t1scf%zmax-1)
end do

end subroutine source_courant_alias_champ

!===========================================================	
! Subroutine source_courant_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine source_courant_affichage(tsc)
implicit none
!=== Déclaration des arguments ===
type(st_source_courant)               :: tsc

!===  Déclaration des variables  ===

!===   Début des traitements   ===
!if (rang==RANG_MAITRE_AFF) then
if (tsc%nbsource>0.or.tsc%nbcanal>0) then
  print '("============== SOURCE DE COURANT, rang=",I5," =================")',rang
  print '(t5, "Rang=",I5,"Nombre de surfaces de sources de courant : ",I4)',rang,tsc%nbsource 
  print '(t5, "Rang=",I5,"Nombre de canaux foudre : ",I2)',rang,tsc%nbcanal 
end if
!end if				
End Subroutine source_courant_affichage


!===========================================================	
! Subroutine source_courant_calcul
! Calcul pendant les itérations temporelles
! Calcul du courant à partir de H
! Mise à jour du champ électrique à partir des sources de courant
!===========================================================	
Subroutine source_courant_calcul(tsc, iter)
implicit none
!=== Déclaration des arguments ===
type(st_source_courant)               :: tsc
integer, intent(in) :: iter
!===  Déclaration des variables  ===
integer  :: i,j,n,m
real  :: Itot
type(st_une_source_courant), pointer  :: t1sc
type(st_source_canal_foudre),pointer  :: t1scf

!===   Début des traitements   ===

! --- boucle sur toutes les sources ---
do n=1, tsc%nbsource
  t1sc =>tsc%l_sc(n)
  ! cas de la source parfaite
  if (t1sc%flag_source_parfaite) then
    do j=1, t1sc%nb2
    do i=1, t1sc%nb1
      t1sc%p(i,j)%e = t1sc%p(i,j)%e - t1sc%ce2(i,j) * t1sc%Isrc(iter)
    end do; end do
  ! cas de la source avec charge, gene de Norton
  else 
    do j=1, t1sc%nb2
    do i=1, t1sc%nb1
      Itot =   t1sc%d2(j)*(t1sc%p(i,j)%h2(2) - t1sc%p(i,j)%h2(1)) &
             - t1sc%d1(i)*(t1sc%p(i,j)%h1(2) - t1sc%p(i,j)%h1(1)) 
      t1sc%p(i,j)%e = t1sc%ce1(i,j) * t1sc%Eprec(i,j) + &
                      t1sc%ce2(i,j) * (Itot - t1sc%Isrc(iter))
      t1sc%Eprec(i,j) = t1sc%p(i,j)%e     
    end do; end do
  
  end if
end do

!--- Canal foudre ---
do n=1, tsc%nbcanal
  t1scf =>tsc%l_scf(n)
  do i=1,t1scf%zmax-t1scf%zmin
    m = MAX(iter-t1scf%nd(i),0)
    t1scf%e(i) = t1scf%e(i) -t1scf%ce(i) * ( (1-t1scf%pd(i)) * t1scf%Isrc(m) &
                                              + t1scf%pd(i)  * t1scf%Isrc(m-1) )
  end do
end do		

End Subroutine source_courant_calcul


!===========================================================	
! Subroutine source_courant_deallocate
!  Libération mémoire dynamique
!===========================================================	
Subroutine source_courant_deallocate(tsc)
implicit none
!=== Déclaration des arguments ===
type(st_source_courant)               :: tsc

!===  Déclaration des variables  ===
integer  :: n, i,j
type(st_une_source_courant), pointer  :: t1sc
type(st_source_canal_foudre),pointer  :: t1scf
!===   Début des traitements   ===
do n = 1, tsc%nbsource
  t1sc => tsc%l_sc(n)
  deallocate(t1sc%Isrc)
  deallocate(t1sc%Eprec)
  deallocate(t1sc%d1, t1sc%d2)
  deallocate(t1sc%ce1, t1sc%ce2)
  do j=1, t1sc%nb2
  do i=1, t1sc%nb1
    NULLIFY(t1sc%p(i,j)%e)
    NULLIFY(t1sc%p(i,j)%h1)
    NULLIFY(t1sc%p(i,j)%h2)    
  end do; end do
  
  deallocate(t1sc%p)  
end do

do n = 1, tsc%nbcanal
  t1scf => tsc%l_scf(n)
  deallocate(t1scf%Isrc)
  deallocate(t1scf%ce)
  deallocate(t1scf%nd)
  deallocate(t1scf%pd)
  nullify(t1scf%e)
end do

if (tsc%nbsource>0) deallocate(tsc%l_sc)	
if (tsc%nbcanal>0) deallocate(tsc%l_scf)	
			
End Subroutine source_courant_deallocate

!===========================================================	
! Subroutine source_courant_impression
!  Ecriture des résultats dans les fichiers de sortie
!===========================================================	
Subroutine source_courant_impression()
implicit none
!=== Déclaration des arguments ===

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine source_courant_impression

End Module mfdtd_source_courant	



