!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_source_tension.f90
!// Purpose:      gestion de sources de tension
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Avril 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!================================================================
! Traitement des sources de tension cellulaires
!================================================================
module mfdtd_source_tension

use m_constantes
use m_entree, only : st_fichier, &
                                  lineread, &
				  multibackspace, &
				  nb_mot_chaine, &
				  ctrl_nb_mot_chaine, &
				  lecture_stop, &
				  apres_fin_bloc,&
				  numbackspace_set

use m_generateur, only : st_generateur, &
                        generateur_calcul_t,&
			generateur_existe_actif
			      
use m_maillage

use mfdtd_volume, only : st_volume, &
                          volume_pasE_local, &
                          volume_pasH_local, &
			  volume_surfaceE_local
use mfdtd_champ, only : st_champ
use mfdtd_milieu_volume, only : st_milieu_volume, &
                                 milieu_volume_extrait_epsilon

implicit none

!=== Déclaration des constantes ===
real, parameter  :: RESISTANCE_MIN = 0.00001  

!=== Déclaration des types ===

type st_pointeur_reel
  real, pointer  :: e
  real, pointer, dimension(:)  :: h1, h2
end type st_pointeur_reel 

! la source_tension source_tension
type st_une_source_tension
  logical  :: flag_source_parfaite, flag_courant_deplacment
  logical  :: flag_new_gene
  integer  :: direction, duree_iter
  real  :: phase, retard, duree
  real  :: E0, R0
  integer, dimension(1:3)  :: coord_ijk
  character(LEN=LEN_MOT_CLE) :: nom, nomgene
  real, pointer, dimension(:)  :: Vsrc
  real  :: d1,d2
  real  :: Eprec, ce1, ce2
  type(st_pointeur_reel)  :: p ! tableau d'alias vers les composantes du champ 
end type st_une_source_tension

type st_source_tension
  logical  :: flag_data
  integer  ::  nbsource
  type(st_une_source_tension), pointer, dimension(:)  :: l_st

end type st_source_tension
 
! Pas de variables globales...

contains
!===========================================================	
! Subroutine source_tension_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine source_tension_initdefaut(t_v)
!=== Déclaration des arguments ===
type(st_source_tension), intent(inout) :: t_v
!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_v%nbsource = 0
t_v%flag_data = .false.				
End Subroutine source_tension_initdefaut

!===========================================================	
! Subroutine une_source_tension_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine une_source_tension_initdefaut(t1st)
!=== Déclaration des arguments ===
type(st_une_source_tension)  :: t1st
!===  Déclaration des variables  ===

!===   Début des traitements   ===
t1st%direction = 0
t1st%phase  = 0.0
t1st%retard = 0.0
t1st%duree  = 0.0
t1st%E0= 1.0
t1st%R0= 0.0
t1st%coord_ijk = 0
t1st%flag_source_parfaite    = .false.
t1st%flag_courant_deplacment = .true.
t1st%flag_new_gene = .true.
t1st%d1= 0.0; t1st%d2= 0.0;
t1st%Eprec = 0.0
t1st%ce1 = 1.0; t1st%ce2 = 1.0
nullify(t1st%Vsrc)

End Subroutine une_source_tension_initdefaut

!===========================================================	
! Subroutine une_source_tension_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine une_source_tension_copy(vin,vout)
!=== Déclaration des arguments ===
type(st_une_source_tension),intent(in)  :: vin
type(st_une_source_tension),intent(out)  :: vout
!===  Déclaration des variables  ===

!===   Début des traitements   ===
vout%nom = vin%nom
vout%nomgene = vin%nomgene
vout%direction = vin%direction
vout%phase  = vin%phase
vout%retard = vin%retard
vout%duree  = vin%duree
vout%duree_iter  = vin%duree_iter
vout%E0= vin%E0
vout%R0= vin%R0
vout%coord_ijk = vin%coord_ijk
vout%flag_source_parfaite    = vin%flag_source_parfaite 
vout%flag_courant_deplacment = vin%flag_courant_deplacment
vout%flag_new_gene = vin%flag_new_gene
vout%d1= vin%d1; vout%d2= vin%d2;
vout%Eprec = vin%Eprec
vout%ce1 = vin%ce1; vout%ce2 = vin%ce2
nullify(vout%Vsrc)

End Subroutine une_source_tension_copy

!===========================================================	
! Subroutine source_tension_lecture
! Lecture dans le fichier d'entrée...
!===========================================================	
Subroutine source_tension_lecture(f, t_v,tm,g)
!=== Déclaration des arguments ===
type(st_fichier)                  :: f
type(st_source_tension), intent(inout)  :: t_v
type(st_maillage), intent(in) :: tm
type(st_generateur), intent(inout) :: g


!===  Déclaration des variables  ===
integer  :: i, isrc, nbmot, status
real  :: phase, retard, duree
character(LEN=LEN_MOT_CLE) :: corbeille, nom, premiermot
type(st_une_source_tension), pointer  :: t1st

!===   Début des traitements   ===
!--- On compte le nombre de sources de tension ---
isrc = 0; 				
! -- Mise a zero de f%numbackspace --
call numbackspace_set(f,0)
do
  corbeille = lineread(f,1)
  if (corbeille==f%nomfinbloc) then
    exit
  else if (corbeille(1:1) == 'V') then
     isrc = isrc+1
  end if
enddo

! -- On remonte au début du bloc --
i = multibackspace(f)
if (isrc == 0) then
  if (rang==RANG_MAITRE_AFF) print '(/,t10,"AVERTISSEMENT, Pas de sources de tension de définies !!!!")'
  t_v%flag_data = .false.
  call apres_fin_bloc(f)
  return
end if
t_v%flag_data = .true.


t_v%nbsource = isrc
allocate(t_v%l_st(isrc))
isrc   = 1
phase  = 0.0
retard = 0.0
duree = 0.0
! -- Lecture des donnees --

do
  premiermot = lineread(f)
  if (premiermot(1:2) == 'VX' .or. premiermot(1:2) == 'VY' .or. premiermot(1:2) == 'VZ') then 
    t1st => t_v %l_st(isrc)
    isrc = isrc+1
    call une_source_tension_initdefaut(t1st)
    
    status = ctrl_nb_mot_chaine(f,f%chaine,6,7)
    nbmot = nb_mot_chaine(f%chaine)  
    select case (nbmot)
      case (6)
        read(f%chaine,*)  t1st%nom, t1st%nomgene, t1st%E0, t1st%coord_ijk(1:3)
	t1st%R0 = 0.0
      case (7)
        read(f%chaine,*)  t1st%nom, t1st%nomgene, t1st%E0, t1st%R0, t1st%coord_ijk(1:3)
      case default
        print '(/,t10,"Nbre de mots de la chaine faux !")'
        call lecture_stop(f)      
    end select
    
    if (.not.generateur_existe_actif(g,t1st%nomgene)) then
      call lecture_stop(f)
    end if
    
    t1st%coord_ijk= origine_cellule3(tm,t1st%coord_ijk)
    if (rang==RANG_MAITRE_AFF) print '(t10,"Coords de ",A," ds le repère de calcul = ",3I6)',trim(t1st%nom),t1st%coord_ijk
      
    t1st%phase  = phase
    t1st%retard = retard
    t1st%duree  = duree
    
      ! direction  
    if (premiermot(1:2) == 'VX') then
      t1st%direction = DIRECTION_X
    else if (premiermot(1:2) == 'VY') then
      t1st%direction = DIRECTION_Y    
    else
      t1st%direction = DIRECTION_Z
    end if
        
  else if (premiermot == 'phase') then
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*)  corbeille, phase
  else if (premiermot == 'retard') then
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*)  corbeille, retard
  else if (premiermot == 'duree_source') then
    status = ctrl_nb_mot_chaine(f,f%chaine,2)
    read(f%chaine,*)  corbeille, duree
  
  else if (f%nomfinbloc == premiermot) then
    exit ! -- Fin du bloc --
  else     
    print *,'(/,t10,"Mot clé non reconnu  :",A )', TRIM(premiermot) 
    call lecture_stop(f)
  end if 
enddo

End Subroutine source_tension_lecture

! ==========================================================
! == Subroutine source_tension_ecriture                   ==
! ==========================================================
subroutine source_tension_ecriture(f,tst,tm)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout) :: f
type(st_source_tension), intent(inout) :: tst
type(st_maillage), intent(in) :: tm

integer  :: i,coord(3)
real     :: phase, retard,duree
type(st_une_source_tension), pointer  :: t1st

if (.not.tst%flag_data) return

!----- Ecriture des sources de tension ---------
phase = 0.0; retard = 0.0;
duree = 0.0
do i=1,tst%nbsource
  t1st => tst%l_st(i)
  if (t1st%phase /= phase) then
    phase = t1st%phase
    write(f%unite,'(T3,A,3x,G14.6)') 'phase',t1st%phase
  end if
  if (t1st%retard /= retard) then
    retard = t1st%retard
    write(f%unite,'(T3,A,3x,G14.6)') 'retard',t1st%retard
  end if
  if (t1st%duree /= duree) then
    duree = t1st%duree
    write(f%unite,'(T3,A,3x,G14.6)') 'duree_source',t1st%duree
  end if
  
  coord= origine_cellule3(tm,t1st%coord_ijk,.true.)
  
  if (t1st%R0 /= 0.0) then
    write(f%unite,'(T3,A,1x,A,1x,2G14.6,1x,3I5)') trim(t1st%nom), trim(t1st%nomgene), &
                        t1st%E0, t1st%R0,coord
  else
    write(f%unite,'(T3,A,1x,A,1x,G14.6,1x,3I5)') trim(t1st%nom), trim(t1st%nomgene), &
                        t1st%E0,coord
  end if
end do

  
end subroutine source_tension_ecriture


!===========================================================	
! Subroutine source_tension_init
! Initialisation des variables, lorsque qu'il n'y a pas de lecture du fichier
! d'entrée
!===========================================================	
Subroutine source_tension_init()
implicit none
!=== Déclaration des arguments ===

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine source_tension_init
	
!===========================================================	
! Subroutine source_tension_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
Subroutine source_tension_precalcul(t_v, v, tm, t_gene)
implicit none
!=== Déclaration des arguments ===
type(st_source_tension)               :: t_v
type(st_volume), intent(in) :: v
type(st_milieu_volume), intent(in)  :: tm
type(st_generateur), intent(inout)     :: t_gene

!===  Déclaration des variables  ===
integer  :: n, i
real  :: dervaleur
real  :: di, s, eps
real, dimension(:,:,:), pointer  :: epsilon
type(st_une_source_tension), pointer  :: t1st, t1stp

!===   Début des traitements   ===
				
! --- boucle sur toutes les sources ---
do n=1, t_v%nbsource
  t1st =>t_v%l_st(n)

  ! Si le gene existe déjà, je pointe dessus
  do i=2, n-1
    t1stp => t_v%l_st(i)
    if (t1st%phase == t1stp%phase .and. t1st%retard == t1stp%retard .and. &
        t1st%nomgene == t1stp%nomgene) then
      t1st%Vsrc => t1stp%Vsrc
      t1st%flag_new_gene = .false.
      exit
    end if
  end do  
  ! Calcul du générateur
  if (t1st%flag_new_gene) then
    allocate(t1st%Vsrc(v%ntinc))
    t1st%Vsrc = 0.0
    dervaleur = generateur_calcul_t(t_gene, t1st%nomgene, &
           1.0, t1st%phase, t1st%retard, v%dt, v%ntinc, t1st%Vsrc)

  end if
    
  if (t1st%R0<=RESISTANCE_MIN) t1st%flag_source_parfaite = .true.

  t1st%duree_iter = v%ntinc
  if (t1st%duree > 0) t1st%duree_iter = t1st%duree / v%dt
   
  !--- Calcul des coefficients multiplicatifs des champs et 
  !     de la source de tension ---
    
  if (t1st%flag_source_parfaite) then
      di = volume_pasE_local(v,t1st%coord_ijk(1), t1st%coord_ijk(2), &
                        t1st%coord_ijk(3),t1st%direction) 
      t1st%ce2 = t1st%E0 /di

  else
      ! on récupère le pas suivant la direction de V à la bonne position
      di = volume_pasE_local(v,t1st%coord_ijk(1), t1st%coord_ijk(2), &
                        t1st%coord_ijk(3),t1st%direction) 
      ! on récupère la surface transverse à la tension
      s = volume_surfaceE_local(v,t1st%coord_ijk(1), t1st%coord_ijk(2),&
                                  t1st%coord_ijk(3),PC1(t1st%direction))
		    
      ! on récupère la permittivité
      eps = milieu_volume_extrait_epsilon(tm,t1st%coord_ijk(1), t1st%coord_ijk(2), &
                                  t1st%coord_ijk(3),t1st%direction)

      ! on récupère les pas spatiaux pour la circulation de H --> I
      t1st%d1 = volume_pasH_local(v,t1st%coord_ijk(1), t1st%coord_ijk(2),& 
                         t1st%coord_ijk(3), PC1(t1st%direction))
      t1st%d2 = volume_pasH_local(v,t1st%coord_ijk(1), t1st%coord_ijk(2),& 
                         t1st%coord_ijk(3), PC2(t1st%direction))
      
      t1st%ce1 = ((1-di*v%dt/(2*eps*s*t1st%R0))/(1+di*v%dt/(2*eps*s*t1st%R0)))
      t1st%ce2 = (v%dt/(s*eps))/(1+di*v%dt/(2*eps*s*t1st%R0))
      t1st%d1  = t1st%d1
      t1st%d2  = t1st%d2 
      t1st%E0  = t1st%E0 / t1st%R0
  end if

end do

End Subroutine source_tension_precalcul

!===========================================================	
! Subroutine source_tension_alias_champ
! Alias vers les composantes du champ électrique et magnétique
!===========================================================	
subroutine source_tension_alias_champ(t_v, tc)
implicit none
!=== Déclaration des arguments ===
type(st_source_tension)               :: t_v
type(st_champ), intent(in) :: tc
!===  Déclaration des variables  ===
integer  :: n, ix, iy, iz
type(st_une_source_tension), pointer  :: t1st

!===   Début des traitements   ===

! --- boucle sur toutes les sources ---
do n=1, t_v%nbsource
  t1st =>t_v%l_st(n)

  ix = t1st%coord_ijk(1)
  iy = t1st%coord_ijk(2)
  iz = t1st%coord_ijk(3)   
  if (t1st%direction == DIRECTION_X) then
    t1st%p%e  => tc%ex(ix, iy, iz)
    t1st%p%h1 => tc%hy(ix, iy, iz-1:iz)
    t1st%p%h2 => tc%hz(ix, iy-1:iy, iz)

  else if (t1st%direction == DIRECTION_Y) then
    t1st%p%e  => tc%ey(ix, iy, iz)
    t1st%p%h1 => tc%hz(ix-1:ix, iy, iz)
    t1st%p%h2 => tc%hx(ix, iy, iz-1:iz)

  else 
    t1st%p%e  => tc%ez(ix, iy, iz)
    t1st%p%h1 => tc%hx(ix, iy-1:iy, iz)
    t1st%p%h2 => tc%hy(ix-1:ix, iy, iz)

  end if
end do

end subroutine source_tension_alias_champ

!===========================================================	
! Subroutine source_tension_calcul
!  Calcul pendant les itérations temporelles
!===========================================================	
Subroutine source_tension_calcul(t_v, iter)
implicit none
!=== Déclaration des arguments ===
type(st_source_tension)               :: t_v
integer, intent(in) :: iter

!===  Déclaration des variables  ===
integer  :: n
real     :: Jtot
type(st_une_source_tension), pointer  :: t1st

!===   Début des traitements   ===
				
! --- boucle sur toutes les sources ---
do n=1, t_v%nbsource
  t1st =>t_v%l_st(n)
  ! cas de la source parfaite
  if ((t1st%flag_source_parfaite).and.(t1st%duree_iter>=iter)) then
      t1st%p%e =  - t1st%ce2 * t1st%Vsrc(iter)
!      t1st%p%e = -t1st%Eprec - t1st%ce2 * t1st%Vsrc(iter)
!      t1st%Eprec = t1st%p%e     
  ! cas de la source avec charge, gene de Thevenin
  else if (t1st%duree_iter>=iter) then
      Jtot =   t1st%d2*(t1st%p%h2(2) - t1st%p%h2(1)) &
             - t1st%d1*(t1st%p%h1(2) - t1st%p%h1(1)) 
	     
      t1st%p%e = t1st%ce1 * t1st%Eprec + &
                      t1st%ce2 * (Jtot - t1st%E0 *t1st%Vsrc(iter))
      t1st%Eprec = t1st%p%e     
  
  end if
  
  
end do				

End Subroutine source_tension_calcul

!===========================================================	
! Subroutine source_tension_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine source_tension_affichage(t_v)
implicit none
!=== Déclaration des arguments ===
type(st_source_tension)               :: t_v

!===  Déclaration des variables  ===

!===   Début des traitements   ===
if (rang==RANG_MAITRE_AFF) then
  print '("============== SOURCE DE TENSION =================")'
  print '(t5, "Nombre de sources de tension : ",I2)', t_v%nbsource 
end if				
End Subroutine source_tension_affichage

!===========================================================	
! Subroutine source_tension_deallocate
!  Libération mémoire dynamique
!===========================================================	
Subroutine source_tension_deallocate(t_v)
implicit none
!=== Déclaration des arguments ===
type(st_source_tension)               :: t_v

!===  Déclaration des variables  ===
integer  :: n
type(st_une_source_tension), pointer  :: t1st
!===   Début des traitements   ===
do n = 1, t_v%nbsource
  t1st => t_v%l_st(n)
  if (t1st%flag_new_gene) deallocate(t1st%Vsrc)
  NULLIFY(t1st%p%e)
  NULLIFY(t1st%p%h1)
  NULLIFY(t1st%p%h2)    
end do

deallocate(t_v%l_st)	
			
End Subroutine source_tension_deallocate

!===========================================================	
! Subroutine source_tension_postcalcul
!  Calcul après les itérations temporelles
!===========================================================	
Subroutine source_tension_postcalcul()
implicit none
!=== Déclaration des arguments ===

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine source_tension_postcalcul

!===========================================================	
! Subroutine source_tension_impression
!  Ecriture des résultats dans les fichiers de sortie
!===========================================================	
Subroutine source_tension_impression()
implicit none
!=== Déclaration des arguments ===

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine source_tension_impression

End Module mfdtd_source_tension	



