!~ !/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_structure.f90
!// Purpose:     Lecture des données concernant le bloc <structure>
!//         La subroutine init prolonge les structures infinies dans les PMLs
!//           et ajuste les coordonnées en conséquence
!// Author:      Cyril Giraudon, Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Novembre 2001
!// Copyright:   (c) Christophe Guiffaut,  CNRS UMR 6172, Institut XLIM
!//              Utilisation du source réservé à l'institut
!// Licence  :
!/////////////////////////////////////////////////////////////////////////////


module mfdtd_structure

use m_constantes
use m_constantes_scientifiques
use m_entree, only : st_fichier, lineread, &
                    multibackspace, &
		ctrl_nb_mot_chaine, &
		unitefichier, &
		entier_chaine, &
		nb_mot_chaine, &
		nb_mot_chaine_long, &
		lecture_stop, &
		COMMENTAIRE, &
		numbackspace_set
use m_maillage, only : st_maillage,origine_cellule6,conv_chcoord6_cell_int,&
                       conv_chcoord6_cell_reel,&
		       conv_coord6_cell_inttostr
use m_liste_milieu, only : st_liste_milieu_simple
use mfdtd_identifiant
use mfdtd_volume, only : st_volume

implicit none
integer,parameter :: SIBC_INT = 0
integer,parameter :: SIBC_EXT = 1
integer,parameter :: BIBC_INT = -1
integer,parameter :: BIBC_EXT = +1
integer,parameter :: SIBC_INT_EXT = 2
integer,parameter :: SIBC_ALGO_ARETE_PADE = 0
integer,parameter :: SIBC_ALGO_FACE_PADE  = 1
integer,parameter :: SIBC_ALGO_FACE_EXP  = 2
integer, parameter  :: IP_2 = selected_int_kind(1)
integer, parameter  :: IP_8B  = selected_int_kind(2)
integer, parameter  :: IP_16B = selected_int_kind(4)
integer,parameter :: DEC_BIN_METAL= 0
integer,parameter :: DEC_BIN_SIBC = 3
integer,parameter :: DEC_BIN_BIBC = 6


!========================== LES FILS MINCES =====================
! Fils minces traitées par modification des coefficients multiplicatifs
!   des champs
type st_structure_filmince
  integer  :: nuobj
  real     :: ray !rayon des fils
  character (len=LEN_MOT_CLE) :: nom
  real, dimension(6) :: coord_cell
  integer(IP_2)  :: type_fil
end type st_structure_filmince


!========================== LES PLAQUES SIBC =====================
! surface Impedance Boundary Condition
type st_structure_sibc
  integer  :: nuobj
  real     :: d !épaisseur de la plaque mince
  character (len=LEN_MOT_CLE) :: nom_z
  integer, dimension(6) :: coord
  integer(IP_2)  :: type_face,face_int,face_ext
  logical  :: flag_active  ! face active ou pas
end type st_structure_sibc

!========================== LES PLAQUES BIBC =====================
! Bilatéral Impedance Boundary Condition
type st_structure_bibc
  integer  :: nuobj
  real     :: d !épaisseur de la plaque mince
  character (len=LEN_MOT_CLE) :: nom_z
  integer, dimension(6) :: coord
  integer(IP_2)  ::type_face
  logical  :: flag_active  ! face active ou pas
end type st_structure_bibc

!========================== LES PLAQUES MINCES =====================
type st_structure_plaque_mince
  integer  :: nuobj
  real     :: d !largeur de la plaque mince
  character (len=LEN_MOT_CLE) :: nom,nommilieu
  integer, dimension(6) :: coord
  logical, dimension(6) :: flag_face
end type st_structure_plaque_mince

!========================== LES FENTES MINCES =====================
type st_structure_fente
  integer  :: nuobj
  integer  :: dirE, dirH,dirfente
  real     :: epsr
  real     :: w !largeur de la fente mince
  character (len=LEN_MOT_CLE) :: nom,nom_tsf
  integer, dimension(6) :: coord
end type st_structure_fente


!============================ LES OBJETS ==========================
type st_structure_objet
  integer                     :: nbstructure
  character (len=LEN_MOT_CLE) :: nom,typemilieu
  character (LEN=LEN_MOT_CLE) :: typecoord
  integer, dimension(:,:), pointer :: coord
end type st_structure_objet

!======================== SURFACE RUGUEUSE ==========================
type st_surf_rugueuse
  integer  :: nuobj
  character (LEN=LEN_CHAINE)  :: nomfichier
  character (len=LEN_MOT_CLE) :: nommilieu1,nommilieu2 ! milieu inf et sup
  integer :: plan, ind_milieu_avant
  integer, dimension(6) :: coord
  integer, dimension(2) :: origine
  integer, dimension(:,:),pointer :: hauteur
end type st_surf_rugueuse

!======================== STRUCTURE PRINCIPALE ======================
type st_structure
  logical         :: flag_non_metal, flag_grille_metal
  integer         :: nbmetal, nbmilieu !nbre de structures
  integer         :: nbgrille_metal
  
  logical         :: flag_face_cell
  integer(IP_16B), dimension(:,:,:), pointer:: t_face_cell
  !decal :  décalage dans les cellules min et max de la structure
  ! suivant les 3 directions, concerne que les diélectriques
  real, dimension(:,:), pointer  :: decal  	
  integer, dimension(:,:), pointer  :: t_metal,t_grille_metal
  integer, dimension(:,:), pointer  :: t_milieu
  character (len=LEN_MOT_CLE), dimension(:), pointer  :: nommilieu

  !----------------- les objets ----------------------
  integer         :: nbobjet
  type(st_structure_objet), dimension(:), pointer  :: t_objet

  !----------------- les surfaces rugueuses ----------------------
  integer         :: nbsurf_rugueuse
  type(st_surf_rugueuse), dimension(:), pointer  :: t_surf_rugueuse
  
  !----------------- les fentes minces ----------------------
  integer          :: nbfente   
  type(st_structure_fente), dimension(:), pointer :: t_fente
  
  !----------------- les plaques minces ----------------------
  integer          :: nbplaque
  type(st_structure_plaque_mince), dimension(:), pointer :: t_plaque
  
  !----------------- les SIBC --------------------------------
  integer         :: flag_algosibc
  integer         :: nbsibc
  type(st_structure_sibc),dimension(:), pointer :: t_sibc
  !----------------- les BIBC --------------------------------
  integer         :: nbbibc
  type(st_structure_bibc),dimension(:), pointer :: t_bibc

  !----------------- les fils minces --------------------------------
  integer         :: nbfil
  type(st_structure_filmince), dimension(:), pointer :: t_fil
  !-------- type de milieu dans le volume -----------
  integer :: type_milieu_dielec
  integer :: type_milieu_magnet
  !-------- Taille mémoire et Nbre d'opération par seconde ----
  real          :: taille_mem
  integer       :: nb_oper
end type st_structure
 
contains

! ==========================================================
! == Subroutine structure_initdefaut                      ==
! == Initialisation par défaut des variables contenant    ==
! == les paramètres définis par l'utilisateur             ==
! == dans le fichier d'entrée                             ==
! ==========================================================
subroutine structure_initdefaut(t_str)

type(st_structure), intent(inout) :: t_str

t_str%taille_mem = 0.0
t_str%nb_oper    = 0


t_str%nbmetal  = 0
t_str%nbgrille_metal  = 0
t_str%nbmilieu = 0
t_str%nbobjet  = 0
t_str%nbsurf_rugueuse = 0
t_str%nbfente  = 0
t_str%nbplaque = 0
t_str%nbsibc =0
t_str%nbbibc =0
t_str%nbfil  =0
t_str%flag_non_metal = .false.
t_str%flag_grille_metal = .false.
t_str%flag_algosibc= SIBC_ALGO_ARETE_PADE
t_str%flag_face_cell=.false.

t_str%type_milieu_dielec=VOL_DIELEC_HOMOGENE
t_str%type_milieu_magnet=VOL_MAGNET_HOMOGENE

NULLIFY(t_str%t_objet)
NULLIFY(t_str%t_metal)
NULLIFY(t_str%t_milieu)
NULLIFY(t_str%decal)
NULLIFY(t_str%nommilieu)
NULLIFY(t_str%t_surf_rugueuse)
NULLIFY(t_str%t_fente)
NULLIFY(t_str%t_plaque)
nullify(t_str%t_grille_metal)
NULLIFY(t_str%t_sibc)
NULLIFY(t_str%t_bibc)
nullify(t_str%t_fil)
NULLIFY(t_str%t_face_cell)

end subroutine structure_initdefaut

! ==========================================================
! == Subroutine structure_objet_initdefaut                ==
! == Initialisation par défaut des variables              ==
! ==========================================================
subroutine structure_objet_initdefaut(s)
type(st_structure_objet), intent(inout) :: s

s%nbstructure = 0
s%nom=''
s%typemilieu='tout'
s%typecoord ='normal_cell'
nullify(s%coord)

end subroutine structure_objet_initdefaut

! ==========================================================
! == Subroutine structure_pm_initdefaut                   ==
! == Initialisation par défaut des variables              ==
! ==========================================================
subroutine structure_pm_initdefaut(pm)
type(st_structure_plaque_mince), intent(inout) :: pm
pm%nuobj=0
pm%d    =0.0
pm%nom  =""
pm%nommilieu=""
pm%coord=0
pm%flag_face = .false.

end subroutine structure_pm_initdefaut

! ==========================================================
! == Subroutine structure_pm_copy                         ==
! ==========================================================
subroutine structure_pm_copy(cin,cout)
type(st_structure_plaque_mince), intent(in)  :: cin
type(st_structure_plaque_mince), intent(out) :: cout
cout%nuobj=cin%nuobj
cout%d    =cin%d
cout%nom  =cin%nom
cout%nommilieu=cin%nommilieu
cout%coord=cin%coord
cout%flag_face = cin%flag_face

end subroutine structure_pm_copy

! ==========================================================
! == Subroutine structure_fm_copy                         ==
! ==========================================================
subroutine structure_fm_copy(cin,cout)
type(st_structure_fente), intent(in)  :: cin
type(st_structure_fente), intent(out) :: cout
cout=cin
end subroutine structure_fm_copy


! ==========================================================
! == Subroutine structure_lecture                         ==
! == Lecture dans le fichier d'entrée...                  ==
! ==========================================================
subroutine structure_lecture(t_fich,t_str,tm)

type(st_fichier), intent(inout) :: t_fich
type(st_structure), intent(inout) :: t_str
type(st_maillage), intent(in) :: tm

integer  :: nbmot,nbfente
character (len=LEN_MOT_CLE) :: premiermot, corbeille, nom,nom2,type_plan
character (len=LEN_CHAINE)  ::nomfichier
character (len=LEN_MOT_CLE),dimension(300)  ::listeobjet,listeobjet1
integer  :: i,j
integer  :: imetal,igrille_metal,imilieu,iobjet,isr,ifente,iplaque
integer  :: isibc,ibibc,ifil
integer  :: nbmetal,nbgrille_metal,nbmilieu,nbobjet,nuobj
integer  :: lchaine,nbsurf_rugueuse,nbplaque
integer  :: nbligne, status
integer,dimension(2)  :: ncel
integer  :: flagmilieu ! 0 = metal, 1 = milieu (eps,mu)
integer, dimension(3) :: ijk
integer, dimension(6) :: coord
character(LEN=LEN_MOT_CLE),dimension(6)   :: chcoord
logical  :: flag
real, dimension(6)  :: dimensions,coord_reel
type(st_surf_rugueuse), pointer  :: t_sr
type(st_structure_fente), pointer :: tfente
type(st_structure_filmince), pointer   :: tfil
type(st_structure_plaque_mince), pointer :: tplaque
type(st_structure_sibc), pointer ::tsibc
type(st_structure_bibc), pointer ::tbibc

!============ On compte le nombre de structure de différents types ============
nbmetal    = 0
nbgrille_metal = 0
nbfente    = 0
nbmilieu   = 0
nbobjet    = 1
nbplaque   = 0
isibc=0;ibibc=0;
ifil=0;
flagmilieu = 0
nbsurf_rugueuse = 0
listeobjet = ''
listeobjet(1) = BS_OBJET_DEFAUT
call numbackspace_set(t_fich,0) ! compteur retour ligne dans fichier à 0
do
  premiermot = lineread(t_fich,1)
  if ((premiermot(1:5) == 'metal').or.(premiermot(1:9) == 'non_metal')) then
    nbmetal = nbmetal +1
  else if ((premiermot(1:12) == 'grille_metal')) then
    nbmetal = nbmetal +1
    nbgrille_metal =  nbgrille_metal+1 
  else if (premiermot == 'ptminmax') then
    if (flagmilieu == 0) then
      nbmetal = nbmetal +1
    else
      nbmilieu = nbmilieu +1        
    end if
  else if (premiermot == 'OBJET') then
    read(t_fich%chaine,*,err=999) corbeille,nom
    nbobjet=nbobjet+1
    listeobjet(nbobjet) = trim(nom)
  else if (premiermot(1:5) == 'OBJET') then
    read(t_fich%chaine,*,err=999) corbeille,nom
    flagmilieu = 0
    if ((nom /= 'metal').and.(nom /= 'non_metal').and.(nom == 'grille_metal')) flagmilieu = 1  
    if ((nom /= 'metal').and.(nom /= 'non_metal')) flagmilieu = 1  
    nbobjet = nbobjet+1
    listeobjet(nbobjet) = trim(premiermot)
  else if ((premiermot(1:12) == 'plaque_mince').or.(premiermot(1:16) == 'non_plaque_mince')) then
    nbplaque = nbplaque +1  
  else if (premiermot(1:3) == 'fil') then
    ifil = ifil+1  
  else if (premiermot(1:11) == 'fente_mince') then
    nbfente = nbfente +1  
  else if (premiermot(1:16) == 'surface_rugueuse') then
    nbsurf_rugueuse = nbsurf_rugueuse +1
  else if (premiermot == 'algo_sibc_arete_pade'.or. &
           premiermot == 'algo_sibc_face_pade' .or. &
	   premiermot == 'algo_sibc_face_exp') then
    cycle
  else if (premiermot == t_fich%nomfinbloc) then
    ! le pointeur de fichier est remonté au début du bloc
    nbligne = multibackspace(t_fich)
    exit
  else
    nbmilieu = nbmilieu +1  
  end if
end do  

t_str%nbsurf_rugueuse = nbsurf_rugueuse
t_str%nbmetal  = nbmetal
t_str%nbgrille_metal  = nbgrille_metal
t_str%nbmilieu = nbmilieu
t_str%nbfente  = nbfente
t_str%nbplaque = nbplaque
t_str%nbsibc   = isibc
t_str%nbbibc   = ibibc
t_str%nbfil    = ifil
if (rang==RANG_MAITRE_AFF) then
  print '(T10," Nbre de structures de type milieu        :",I8)',nbmilieu
  print '(T10," Nbre de structures de type metal         :",I8)',nbmetal
  print '(T10," Nbre de structures de type grille_metal  :",I8)',nbgrille_metal
  print '(T10," Nbre de structures de type surface_rugueuse  :",I8)',nbsurf_rugueuse
  print '(T10," Nbre de structures de type fente mince   :",I8)',nbfente
  print '(T10," Nbre de structures de type plaque mince  :",I8)',nbplaque
  print '(T10," Nbre de structures de type FIL           :",I8)',ifil
end if
!if ((nbmetal == 0).and.(nbmilieu == 0)) return ! on sort de la fonction

if (nbmetal>0) allocate(t_str%t_metal(1:8,1:nbmetal))
if (nbgrille_metal>0) then
  allocate(t_str%t_grille_metal(1:3,1:nbgrille_metal))
  t_str%t_grille_metal = 0
end if
if (nbmilieu>0) then 
  allocate(t_str%t_milieu(1:7,1:nbmilieu))
  allocate(t_str%decal(1:6,1:nbmilieu))
  allocate(t_str%nommilieu(1:nbmilieu))
end if 
if (nbsurf_rugueuse>0) then
  allocate(t_str%t_surf_rugueuse(1:nbsurf_rugueuse))
end if
if (nbfente>0) then
  allocate(t_str%t_fente(1:nbfente))
end if
if (nbplaque>0) then
  allocate(t_str%t_plaque(1:nbplaque))
end if
if (ifil>0) then
  allocate(t_str%t_fil(1:ifil))
end if

!------------------ Gestion des objets ------------------------------
flag = .true.
listeobjet1(1) = listeobjet(1)
iobjet =1
! on recompte le nombre exact d'objets
do i=2,nbobjet
  do j=1,i-1
    if (listeobjet(i)==listeobjet(j)) then
      flag=.false.
      exit
    end if
  end do
  if (flag) then
    iobjet=iobjet+1
    listeobjet1(iobjet) = listeobjet(i)
  else
    flag =.true.
  end if
end do

allocate(t_str%t_objet(1:iobjet))
nbobjet = iobjet
do i=1,iobjet
  call structure_objet_initdefaut(t_str%t_objet(i))
  t_str%t_objet(i)%nom = listeobjet1(i)
end do
t_str%nbobjet  = iobjet
if (rang==RANG_MAITRE_AFF) print '(T10," Nbre de structures de type objet         :",I4)',iobjet

!--------------------------------------------------------------------
igrille_metal = 1
imetal  = 1
imilieu = 1
iobjet  = 1 ! objet par défaut
isr = 1; ifente = 1; iplaque = 1
isibc = 1;ibibc = 1;
ifil  = 1;
if (rang==RANG_MAITRE_AFF) &
 print '(t5," Affichage des coords dans le repère de calcul pour les 10 1ères structures de chaque type")'
!--------------------------------------------------------------------
! Lecture des informations sur les structures
do
  premiermot = lineread(t_fich)
  !print '(A)', trim(t_fich%chaine)
  lchaine = LEN_TRIM(premiermot)
  if (t_fich%nomfinbloc == premiermot) exit ! fin du bloc      
  if (premiermot(1:5) == 'metal') then
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,7)
      !read(t_fich%chaine,*,err=999) corbeille,coord(1:6)
      !t_str%t_metal(1:6,imetal)= origine_cellule6(tm,coord)
      read(t_fich%chaine,*,err=999) corbeille,chcoord(1:6)
      t_str%t_metal(1:6,imetal)= conv_chcoord6_cell_int(t_fich,tm,chcoord)
      
      if (imetal<=10 .and. rang==RANG_MAITRE_AFF) &
        print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(corbeille),t_str%t_metal(1:6,imetal)
      
      t_str%t_metal(7,imetal) = 1
      t_str%t_metal(8,imetal) = iobjet
      imetal = imetal +1
      cycle
  else if (premiermot(1:9) == 'non_metal') then
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,7)
      !read(t_fich%chaine,*,err=999) corbeille,coord
      !t_str%t_metal(1:6,imetal)= origine_cellule6(tm,coord)
      read(t_fich%chaine,*,err=999) corbeille,chcoord(1:6)
      t_str%t_metal(1:6,imetal)= conv_chcoord6_cell_int(t_fich,tm,chcoord)
      if (imetal<=10.and. rang==RANG_MAITRE_AFF) &
        print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(corbeille),t_str%t_metal(1:6,imetal)

      t_str%t_metal(7,imetal) = 0
      t_str%t_metal(8,imetal) = iobjet
      t_str%flag_non_metal = .true.
      imetal = imetal +1
      cycle
  else if (premiermot(1:12) == 'grille_metal') then
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,10)
      !read(t_fich%chaine,*,err=999) corbeille,coord(1:6), &
      !                                        t_str%t_grille_metal(1:3,igrille_metal)
      !t_str%t_metal(1:6,imetal)= origine_cellule6(tm,coord)
      read(t_fich%chaine,*,err=999) corbeille,chcoord(1:6),t_str%t_grille_metal(1:3,igrille_metal)
      t_str%t_metal(1:6,imetal)= conv_chcoord6_cell_int(t_fich,tm,chcoord)
      if (imetal<=10 .and. rang==RANG_MAITRE_AFF) &
        print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(corbeille),t_str%t_metal(1:6,imetal)
      t_str%t_metal(7,imetal) = 2
      t_str%t_metal(8,imetal) = iobjet
      t_str%flag_grille_metal = .true.
      imetal = imetal +1
      igrille_metal = igrille_metal+1
      cycle
    !--------------------------------------------------------------------------
  else if (premiermot == 'OBJET') then
    ! rien pour l'instant car les informations sont déjà récupérées
    read(t_fich%chaine,*,err=999) corbeille,nom
    iobjet = structure_objet_num(t_str,nom)
    cycle
  else if (premiermot(1:5) == 'OBJET') then
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*,err=999) corbeille,nom
      iobjet = structure_objet_num(t_str,corbeille)
      
      flagmilieu = 0 ! un seul type de milieu par objet, par défaut, c'est du métal
      if ((nom /= 'metal') .and. (nom /= 'non_metal').and.(nom /= 'grille_metal')) flagmilieu = 1        
!      if ((nom /= 'metal') .and. (nom /= 'non_metal')) flagmilieu = 1        
      !--- On compte le nombre de structure de base de l'objet ---
      nbmilieu = 0
      call numbackspace_set(t_fich,0) ! compteur retour ligne dans fichier à 0
      do
        premiermot = lineread(t_fich,1)
	if (premiermot == 'ptminmax') then
          nbmilieu = nbmilieu +1  
        else 
	  ! le pointeur de fichier est remonté au début du bloc
          nbligne = multibackspace(t_fich)
          exit
        end if
      end do  

      t_str%t_objet(iobjet)%nom = corbeille
      t_str%t_objet(iobjet)%typemilieu = nom
      t_str%t_objet(iobjet)%nbstructure = nbmilieu
      if (nbmilieu>0) allocate(t_str%t_objet(iobjet)%coord(1:6,1:nbmilieu))
      ! Lecture des coord. de toutes les structures de base de l'objet
      ! les coordonnées sont récupérées
      do i=1, nbmilieu
        premiermot = lineread(t_fich)
        status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,7)
	if (flagmilieu == 0) then ! du métal
	  !read(t_fich%chaine,*,err=999) corbeille,coord(1:6)
          !t_str%t_metal(1:6,imetal)= origine_cellule6(tm,coord)
          read(t_fich%chaine,*,err=999) corbeille,chcoord(1:6)
          t_str%t_metal(1:6,imetal)= conv_chcoord6_cell_int(t_fich,tm,chcoord)
	  
          if (imetal<=10 .and. rang==RANG_MAITRE_AFF) &
    	    print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(corbeille),t_str%t_metal(1:6,imetal)
	  t_str%t_objet(iobjet)%coord(1:6,i) = t_str%t_metal(1:6,imetal)
	  t_str%t_metal(7,imetal) = 1
          t_str%t_metal(8,imetal) = iobjet
	  
	  if (nom == 'non_metal') then
	    t_str%flag_non_metal = .true.
	    t_str%t_metal(7,imetal) = 0
	  else if (nom == 'grille_metal') then
	    status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,10)
	    !read(t_fich%chaine,*,err=999) corbeille,coord(1:6), &
	    !		    t_str%t_grille_metal(1:3,igrille_metal)
            !t_str%t_metal(1:6,imetal)= origine_cellule6(tm,coord)
            read(t_fich%chaine,*,err=999) corbeille,chcoord(1:6),&
	               t_str%t_grille_metal(1:3,igrille_metal)
            t_str%t_metal(1:6,imetal)= conv_chcoord6_cell_int(t_fich,tm,chcoord)
            if (imetal<=10.and.rang==RANG_MAITRE_AFF) &
	      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(corbeille),t_str%t_metal(1:6,imetal)
	    t_str%t_metal(7,imetal) = 2
	    t_str%flag_grille_metal = .true.
	    igrille_metal = igrille_metal+1
	    cycle
	  end if
	  imetal = imetal+1
	else !un milieu quelconque
!	  read(t_fich%chaine,*,err=999) corbeille, dimensions
          read(t_fich%chaine,*,err=999) corbeille,chcoord(1:6)
          dimensions= conv_chcoord6_cell_reel(t_fich,tm,chcoord)
	  t_str%nommilieu(imilieu) = nom
	  coord(1:6) = NINT(dimensions(1:6))
	  t_str%t_milieu(1:6,imilieu) = coord(1:6)
          !t_str%t_milieu(1:6,imilieu)= origine_cellule6(tm,coord)
          if (imilieu<=10.and.rang==RANG_MAITRE_AFF) &
  	    print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(nom),&
	                              t_str%t_milieu(1:6,imilieu)
          t_str%t_milieu(7,imilieu) = iobjet

	  t_str%decal(1:6,imilieu) = dimensions(1:6) - NINT(dimensions(1:6))
	  t_str%t_objet(iobjet)%coord(1:6,i) = t_str%t_milieu(1:6,imilieu)
	  imilieu = imilieu +1 
	  
	end if
      end do
      iobjet = iobjet+1
      cycle
  end if

  !--------------------------------------------------------------------------
  !--------------------------------------------------------------------------
  !----- FILS MINCES ----
  if (premiermot(1:3) == 'fil') then
    nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,8,8)
    tfil => t_str%t_fil(ifil)
    read(t_fich%chaine,*,err=999) tfil%nom,chcoord(1:6),tfil%ray
    
    tfil%coord_cell(1:6)= conv_chcoord6_cell_reel(t_fich,tm,chcoord)
!    tfente%coord(1:6)= origine_cellule6(tm,tfente%coord(1:6))
    if (ifil<=10 .and. rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",6F10.3)',trim(tfil%nom),tfil%coord_cell(1:6)
    tfil%nuobj = iobjet
    ifil = ifil +1

  !--------------------------------------------------------------------------
  !----- FENTES MINCES ----
  else if (premiermot(1:11) == 'fente_mince') then
    nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,8,9)
    tfente => t_str%t_fente(ifente)
    if (nbmot==8) then
      tfente%nom_tsf = 'tsf_holland'
      read(t_fich%chaine,*,err=999) tfente%nom,tfente%w,chcoord(1:6)
    else
      read(t_fich%chaine,*,err=999) tfente%nom,tfente%w,chcoord(1:6),tfente%nom_tsf
    end if 
    tfente%coord(1:6)= conv_chcoord6_cell_int(t_fich,tm,chcoord)
!    tfente%coord(1:6)= origine_cellule6(tm,tfente%coord(1:6))
    if (ifente<=10.and. rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(tfente%nom),tfente%coord(1:6)
    tfente%nuobj = iobjet
    ijk(1:3) = tfente%coord(2:6:2)-tfente%coord(1:5:2);
    if (.not.((any(ijk==1)).and.(any(ijk==0)))) then
      PRINT '("Erreur, fente mince mal définie !!!")'
      call lecture_stop(t_fich);
    end if
    ifente = ifente +1

  !--------------------------------------------------------------------------
  !----- PLAQUE MINCE ----
  else if ((premiermot(1:12) == 'plaque_mince').or. &
           (premiermot(1:16) == 'non_plaque_mince')) then
    nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,9,9)
    tplaque => t_str%t_plaque(iplaque)
    read(t_fich%chaine,*,err=999) tplaque%nom,tplaque%nommilieu,&
                                  tplaque%d,chcoord(1:6)
				  
    tplaque%coord(1:6)= conv_chcoord6_cell_int(t_fich,tm,chcoord)	
    !tplaque%coord(1:6)= origine_cellule6(tm,tplaque%coord(1:6))
    if (iplaque<=10.and.rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(tplaque%nom),tplaque%coord(1:6)
    ijk(1:3) = tplaque%coord(2:6:2)-tplaque%coord(1:5:2);
    tplaque%nuobj = iobjet
    ! Si pas de plaque mince d = 0 !!!
    if (premiermot(1:16) == 'non_plaque_mince') tplaque%d = 0
    if (.not.(all(ijk>=0))) then
      PRINT '("Erreur, plaque mince mal définie !!!")'
      call lecture_stop(t_fich);
    end if
    iplaque = iplaque +1
    
  !--------------------------------------------------------------------------
  !----- SURFACE RUGUEUSE ----
  else if (premiermot(1:16) == 'surface_rugueuse') then
    nbmot = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,11,13)
    t_sr => t_str%t_surf_rugueuse(isr)
    t_sr%origine = 0
    if (nbmot == 11) then  ! Ne peut pas etre infini!!!
      read(t_fich%chaine,*,err=999) corbeille,nom,nom2,type_plan,coord(1:6),nomfichier
    else if (nbmot == 13) then
      read(t_fich%chaine,*,err=999) corbeille,nom,nom2,type_plan,coord(1:6),nomfichier,t_sr%origine(1:2)
    else
      call lecture_stop(t_fich)
    end if
    t_sr%nuobj=iobjet
    t_sr%nommilieu1 = nom
    t_sr%nommilieu2 = nom2
    t_sr%nomfichier = nomfichier
    if (type_plan == 'xoy') then
      t_sr%coord(1:6) = coord(1:6)
      t_sr%plan = PLAN_XOY
    else
      call lecture_stop(t_fich)    
    end if
    t_sr%coord(1:6)= origine_cellule6(tm,t_sr%coord(1:6))
    if (isr<=10 .and.rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(corbeille),t_sr%coord(1:6)

    ncel(1) = t_sr%coord(2)-t_sr%coord(1)
    ncel(2) = t_sr%coord(4)-t_sr%coord(3)
    allocate(t_sr%hauteur(1:ncel(1),1:ncel(2)))
    t_sr%hauteur = 0
    t_sr%ind_milieu_avant = imilieu-1 ! positionnement de la définition de la surface
                                    ! par rapport aux milieux
    call lecture_surface_rugueuse(nomfichier,t_sr%hauteur,ncel,t_sr%origine)
    
    isr = isr+1
  else if (premiermot == 'algo_sibc_arete_pade') then
    t_str%flag_algosibc = SIBC_ALGO_ARETE_PADE
  else if (premiermot == 'algo_sibc_face_pade') then
    t_str%flag_algosibc = SIBC_ALGO_FACE_PADE
  else if (premiermot == 'algo_sibc_face_exp') then
    t_str%flag_algosibc = SIBC_ALGO_FACE_EXP
  else !un milieu quelconque
    status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,7)
    read(t_fich%chaine,*,err=999) t_str%nommilieu(imilieu), chcoord(1:6)
    dimensions= conv_chcoord6_cell_reel(t_fich,tm,chcoord)
    
    coord = NINT(dimensions(1:6))
    t_str%t_milieu(1:6,imilieu)= coord(1:6)
!    t_str%t_milieu(1:6,imilieu)= origine_cellule6(tm,coord)
    if (imilieu<=10 .and.rang==RANG_MAITRE_AFF) &
      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',&
        trim(t_str%nommilieu(imilieu)),t_str%t_milieu(1:6,imilieu)

    t_str%t_milieu(7,imilieu) = iobjet
    t_str%decal(1:6,imilieu) = dimensions(1:6) - NINT(dimensions(1:6))
    imilieu = imilieu +1 
  end if
end do				

return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(t_fich)

end subroutine structure_lecture

! ==========================================================
! == Subroutine structure_ecriture                        ==
! == Ecriture dans le fichier d'entrée...                 ==
! ==========================================================
subroutine structure_ecriture(f,t_str,tm)

type(st_fichier), intent(inout) :: f
type(st_structure), intent(inout) :: t_str
type(st_maillage), intent(in) :: tm

integer :: i,l,igm
integer :: coord(6)
character(LEN=LEN_MOT_CLE) :: chcoord(6)
type(st_surf_rugueuse), pointer  :: tsr
type(st_structure_fente), pointer:: tf
type(st_structure_sibc), pointer ::tsi
type(st_structure_bibc), pointer ::tbi
type(st_structure_plaque_mince), pointer :: tp

character(LEN=3), dimension(3) :: ch_plan 
character(LEN=10) :: nom,nom1
! ----------- LISTE DES MILIEUX ------------
l=0

do i=1,t_str%nbmilieu
  if (l/=t_str%t_milieu(7,i)) then
    l=t_str%t_milieu(7,i)
    write(f%unite,'(T3,A,2x,A)') 'OBJET',(t_str%t_objet(l)%nom)
  end if
  
  !coord= origine_cellule6(tm,t_str%t_milieu(1:6,i),.true.)
  !write(f%unite,'(T5,A,3x,6I6)')  trim(t_str%nommilieu(i)),coord
  chcoord = conv_coord6_cell_inttostr(tm,t_str%t_milieu(1:6,i))
  write(f%unite,'(T5,A,5x,6A8)')  (t_str%nommilieu(i)),ADJUSTL(chcoord(1:6))
end do
write(f%unite,*)
! ----------- LISTE DES PLAQUES MÉTALLIQUES ------------
igm=1
do i=1,t_str%nbmetal
  if (l/=t_str%t_metal(8,i)) then
    l=t_str%t_metal(8,i)
    write(f%unite,'(T3,A,2x,A)') 'OBJET',(t_str%t_objet(l)%nom)
  end if
  !coord= origine_cellule6(tm,t_str%t_metal(1:6,i),.true.)
  chcoord = conv_coord6_cell_inttostr(tm,t_str%t_metal(1:6,i))
  if (t_str%t_metal(7,i)==1) then
    write(f%unite,'(T5,A9,5x,6A8)')  'metal    ',ADJUSTL(chcoord(1:6))
  else if (t_str%t_metal(7,i)==0) then
    write(f%unite,'(T5,A9,5x,6A8)')  'non_metal',ADJUSTL(chcoord(1:6))
  else if (t_str%t_metal(7,i)==2) then
    write(f%unite,'(T5,A,5x,6A8,3x,3I4)')  'grille_metal',ADJUSTL(chcoord(1:6)),t_str%t_grille_metal(1:3,igm)
    igm=igm+1    
  end if
end do
write(f%unite,*)

! ----------- LISTE DES SURFACES RUGUEUSES ------------
do i=1,t_str%nbsurf_rugueuse
  ch_plan=  (/'xoy','yoz','zox'/) 
  tsr => t_str%t_surf_rugueuse(i)
  if (l/=tsr%nuobj) then
    l=tsr%nuobj
    write(f%unite,'(T3,A,2x,A)') 'OBJET',(t_str%t_objet(l)%nom)
  end if
  coord= origine_cellule6(tm,tsr%coord(1:6),.true.)
  write(f%unite,'(T5,A,3x,A,3x,A,3x,A,3x,6I5,3x,A,2I5)')  'surface_rugueuse',(tsr%nommilieu1), &
	trim(tsr%nommilieu2),ch_plan(tsr%plan),coord(1:6),trim(tsr%nomfichier),tsr%origine(1:2)
end do
write(f%unite,*)


! ----------- LISTE DES FENTES MINCES ------------
do i=1,t_str%nbfente
  tf => t_str%t_fente(i)
  if (l/=tf%nuobj) then
    l=tf%nuobj
    write(f%unite,'(T3,A,2x,A)') 'OBJET',(t_str%t_objet(l)%nom)
  end if
  !coord= origine_cellule6(tm,tf%coord(1:6),.true.)
  chcoord = conv_coord6_cell_inttostr(tm,tf%coord(1:6))
  write(f%unite,'(T5,A,3x,ES12.5,3x,6A8,3x,A)') (tf%nom),tf%w,&
            ADJUSTL(chcoord(1:6)),trim(tf%nom_tsf)
end do

! ----------- LISTE DES PLAQUES MINCES ------------
do i=1,t_str%nbplaque
  tp => t_str%t_plaque(i)
  if (l/=tp%nuobj) then
    l=tp%nuobj
    write(f%unite,'(T3,A,2x,A)') 'OBJET',(t_str%t_objet(l)%nom)
  end if
!  coord= origine_cellule6(tm,tp%coord(1:6),.true.)
  chcoord = conv_coord6_cell_inttostr(tm,tp%coord(1:6))
  write(f%unite,'(T5,A,3x,A,3x,ES12.5,3x,6A8)') trim(tp%nom),(tp%nommilieu),&
             tp%d,ADJUSTL(chcoord(1:6))
end do


end subroutine structure_ecriture

! ==========================================================
! == Subroutine structure_troncature                      ==
! ==========================================================
subroutine structure_troncature(t_str,t_str_new,c6_troncat)

type(st_structure), intent(in)    :: t_str
type(st_structure), intent(inout) :: t_str_new
integer, dimension(6), intent(in) :: c6_troncat

integer :: m,n
integer, dimension(:,:), pointer :: t_coord,t_coordnew
character(LEN=LEN_MOT_CLE), dimension(:), pointer :: listemilieu

!On compte le nombre de structure appartenant au sous-volume
if (associated(t_str_new%t_metal)) then
  deallocate(t_str_new%t_metal)
  nullify(t_str_new%t_metal)
end if
t_str_new%nbmetal = 0

if (t_str%nbmetal>0) then
  t_coord =>t_str%t_metal
  m=0
  do n=1,t_str%nbmetal
    if (t_coord(XMAX,n) <=c6_troncat(XMIN)) cycle
    if (t_coord(YMAX,n) <=c6_troncat(YMIN)) cycle
    if (t_coord(ZMAX,n) <=c6_troncat(ZMIN)) cycle
    if (t_coord(XMIN,n) >=c6_troncat(XMAX)) cycle
    if (t_coord(YMIN,n) >=c6_troncat(YMAX)) cycle
    if (t_coord(ZMIN,n) >=c6_troncat(ZMAX)) cycle
    m = m+1
  end do
  if (m>0) then
    allocate(t_coordnew(1:8,1:m))
    t_coordnew = 0
    m=0
    do n=1,t_str%nbmetal
      if (t_coord(XMAX,n) <=c6_troncat(XMIN)) cycle
      if (t_coord(YMAX,n) <=c6_troncat(YMIN)) cycle
      if (t_coord(ZMAX,n) <=c6_troncat(ZMIN)) cycle
      if (t_coord(XMIN,n) >=c6_troncat(XMAX)) cycle
      if (t_coord(YMIN,n) >=c6_troncat(YMAX)) cycle
      if (t_coord(ZMIN,n) >=c6_troncat(ZMAX)) cycle
      m = m+1
      t_coordnew(:,m) = t_coord(:,n)
      !imin,jmin,kmin
      t_coordnew(1:6:2,m) = t_coord(1:6:2,n) -c6_troncat(1:6:2)+1
      !imax,jmax,kmax
      t_coordnew(2:6:2,m) = t_coord(2:6:2,n) -c6_troncat(1:6:2)+1
      t_coordnew(7:8,m) =  t_coord(7:8,n)
    end do
    allocate(t_str_new%t_metal(1:8,1:m))
    t_str_new%nbmetal = m
    t_str_new%t_metal = t_coordnew
    deallocate(t_coordnew)
  end if
end if

!------------------------------------------------------------------------
!On modification ou troncature des structures de type matériau quelconque
if (associated(t_str_new%t_milieu))  deallocate(t_str_new%t_milieu)
if (associated(t_str_new%nommilieu))  deallocate(t_str_new%nommilieu)
t_str_new%nbmilieu = 0

if (t_str%nbmilieu>0) then
  t_coord =>t_str%t_milieu(:,:)
  m=0
  do n=1,t_str%nbmilieu
    if (t_coord(XMAX,n) <=c6_troncat(XMIN)) cycle
    if (t_coord(YMAX,n) <=c6_troncat(YMIN)) cycle
    if (t_coord(ZMAX,n) <=c6_troncat(ZMIN)) cycle
    if (t_coord(XMIN,n) >=c6_troncat(XMAX)) cycle
    if (t_coord(YMIN,n) >=c6_troncat(YMAX)) cycle
    if (t_coord(ZMIN,n) >=c6_troncat(ZMAX)) cycle
    m = m+1    
  end do
  if (m>0) then
    allocate(t_coordnew(1:7,1:m))
    allocate(listemilieu(1:m))
    t_coordnew = 0
    m=0
    do n=1,t_str%nbmilieu
      if (t_coord(XMAX,n) <=c6_troncat(XMIN)) cycle
      if (t_coord(YMAX,n) <=c6_troncat(YMIN)) cycle
      if (t_coord(ZMAX,n) <=c6_troncat(ZMIN)) cycle
      if (t_coord(XMIN,n) >=c6_troncat(XMAX)) cycle
      if (t_coord(YMIN,n) >=c6_troncat(YMAX)) cycle
      if (t_coord(ZMIN,n) >=c6_troncat(ZMAX)) cycle
      m = m+1
      listemilieu(m) = t_str%nommilieu(n)
      !imin,jmin,kmin
      t_coordnew(1:6:2,m) = t_coord(1:6:2,n) -c6_troncat(1:6:2)+1
      !imax,jmax,kmax
      t_coordnew(2:6:2,m) = t_coord(2:6:2,n) -c6_troncat(1:6:2)+1
      t_coordnew(7,m)     = t_coord(7,m)
    end do
    
    t_str_new%nbmilieu = m
    allocate(t_str_new%t_milieu(1:7,1:m))
    allocate(t_str_new%nommilieu(1:m))
    t_str_new%t_milieu = t_coordnew
    t_str_new%nommilieu = listemilieu
    
    deallocate(t_coordnew)
    deallocate(listemilieu)
  end if
end if

end subroutine structure_troncature

! ==========================================================
! == Subroutine lecture_surface_rugueuse                  ==
!===========================================================	
subroutine lecture_surface_rugueuse(nomfichier,hauteur,ncel,origine)
!--- DECLARATION DES ARGUMENTS ---
character (LEN=LEN_CHAINE), intent(in) :: nomfichier
integer, dimension(2), intent(in)   :: ncel,origine
integer,dimension(1:ncel(1),1:ncel(2)), intent(out)  :: hauteur

!--- DECLARATION DES VARIABLES LOCALES ---
integer  :: unite, statut
integer  :: numligne,nbvaleur
integer  :: i,j
integer,dimension(:), allocatable  :: h
character(len=10000) :: chaine
character(len=LEN_MOT_CLE) :: longueur
!--- DEBUT DES TRAITEMENTS ---
! -- Recupere un descripteur libre le fichier d'entree --

unite =  unitefichier()
if (unite<=0) then
  print '(/,t10,"ATTENTION, pas de descripteur de fichier disponible => STOP !!!")'
  stop
endif
! -- Ouvre le fichier d'entree en lecture --
open(unite,file=nomfichier,status = 'old',action='read',iostat=statut)
if (statut>0) then
  print '(/,t10,"ATTENTION, ouverture de <",A,"> impossible => STOP !!!")', &
    TRIM(nomfichier)
  stop
endif

longueur = entier_chaine(10000) ! 10000 caractères sur une ligne maxi
numligne = 0
i = 0;j=0
!------------------------------------------------------------------------------
! --------------- LECTURE --------------------
do 
  read(unite,'(a'//TRIM(longueur)//')',end=900) chaine
  chaine = ADJUSTL(chaine) !on retire les blancs de début de la chaine
  
  ! compteur ligne 
  numligne = numligne + 1
  ! On ignore les lignes de commentaires et les chaines vides
  if ((chaine=='').or.(chaine(1:1)==COMMENTAIRE)) then
    cycle
  else ! une chaine de données
    i = i+1
    if (origine(2) < i) then
      j = j+1
      if (j>ncel(2)) exit
       ! on suppose que toutes les lignes ont le meme nombre de valeurs
      if (j==1) then
        nbvaleur = nb_mot_chaine_long(chaine,10000)
!        print '("nbvaleur=",I4)',nbvaleur
	allocate(h(1:nbvaleur))
	h = 0
      end if
      read(chaine,*,ERR=901) h(1:nbvaleur)
      hauteur(1:ncel(1),j) = h(origine(1)+1:origine(1)+ncel(1))
!      print '("h=",150I3)',hauteur(1:ncel(1),j)
    end if
  endif
enddo
!------------------------------------------------------------------------------

! fermeture du fichier
900 close(unite)
    return
   
901 print '("Erreur à la lecture de la ligne : ",I4,"dans le fichier de nom ",A)',&
	numligne,TRIM(nomfichier)
    STOP '----> Arret !!!!!!!!!'


end subroutine lecture_surface_rugueuse

!==================================================================
!*********** Fonction qui retourne l'integrale elliptique**********
!*********** d'une valeur. Elle est utilisée dans les *************
!*********************** fentes minces ****************************
!==================================================================
real FUNCTION integral_elliptique(a)
real,intent(in)		:: a

real :: m
m=1-a
integral_elliptique=(1.38629436112+.09666344259*m+&
	.03590092383*m**2+.03742563713*m**3+.01451196212*m**4)+&
	(.5+.12498593597*m+.06880248576*m**2+.03328355346*m**3+&
	.00441787012*m**4)*LOG(1/m)
end FUNCTION integral_elliptique

! ==========================================================
! == Subroutine structure_fente_init                      ==
!===========================================================	
subroutine structure_fente_init(ts,tv)


type(st_structure), intent(inout) :: ts
type(st_volume), intent(in)       :: tv

integer  :: n,i,j
integer, dimension(3) :: ijk
real     :: d1,d2
real     :: r1,r2

type(st_structure_fente), pointer :: tfente

do n=1,ts%nbfente
  tfente => ts%t_fente(n)
  ijk(1:3) = tfente%coord(2:6:2)-tfente%coord(1:5:2)
  d1 = 0.0; d2 = 0.0
  if (ijk(DIRECTION_X) == 1) then
    d1 = tv%celx
    tfente%dirE = DIRECTION_X
    if (ijk(DIRECTION_Z) == 0) then
      d2 = tv%celz
      tfente%dirH     = DIRECTION_Z
      tfente%dirfente = DIRECTION_Y
    else if (ijk(DIRECTION_Y) == 0) then
      d2 = tv%cely
      tfente%dirH     = DIRECTION_Y
      tfente%dirfente = DIRECTION_Z
    end if
    
  else if (ijk(DIRECTION_Y) == 1) then
    d1 = tv%cely
    tfente%dirE = DIRECTION_Y
    if (ijk(DIRECTION_Z) == 0) then
      d2 = tv%celz
      tfente%dirH     = DIRECTION_Z
      tfente%dirfente = DIRECTION_X
    else if (ijk(DIRECTION_X) == 0) then
      d2 = tv%celx
      tfente%dirH     = DIRECTION_X
      tfente%dirfente = DIRECTION_Z
    end if
  else if (ijk(DIRECTION_Z) == 1) then
    d1 = tv%celz
    tfente%dirE = DIRECTION_Z
    if (ijk(DIRECTION_X) == 0) then
      d2 = tv%celx
      tfente%dirH     = DIRECTION_X
      tfente%dirfente = DIRECTION_Y
    else if (ijk(DIRECTION_Y) == 0) then
      d2 = tv%cely
      tfente%dirH     = DIRECTION_Y
      tfente%dirfente = DIRECTION_X
    end if
  
  end if
  if ((d1==0.0).or.(d2==0.0)) then
    print '("Erreur rang=",I5,", fente trop large, Arret !!!!!!  nom de la fente : ",A)',&
           rang,TRIM(tfente%nom)
    stop
  end if
  
  if (tfente%w>d1 .or. tfente%w<=0.0) then
    print '("Erreur, problème dans la définition d''une fente, Arret !!!!!! nom de la fente : ",A)', &
                          TRIM(tfente%nom)
    stop  
  end if 
  if (tfente%nom_tsf == 'tsf_holland') then
    r1= integral_elliptique(sqrt(1-(tfente%w/d1)**2))
    r2= integral_elliptique(tfente%w/d1)
    tfente%epsr= (r1/r2)*(d1/d2)
  else if (tfente%nom_tsf == 'tsf_tixier') then
    tfente%epsr= 2*d1/(PI*d2)*log(d2/tfente%w+sqrt((d2/tfente%w)**2+1)) !2d1/pid2 * asinh(d2/w)
  else if (tfente%nom_tsf == 'tsf_uniform') then
    tfente%epsr= d1/tfente%w
  else if (tfente%nom_tsf == 'tsf_riley_turner') then
    tfente%epsr= 0.5*(d1/d2)*(log(2*d2/tfente%w)+1)
  else
    PRINT '("Erreur : Fente mince de nom : ",A," Nom du formalisme (tfs) non reconnu ! -->",A )', &
           TRIM(tfente%nom), TRIM(tfente%nom_tsf)
    stop
  end if

end do

end subroutine structure_fente_init

! ==========================================================
! == Subroutine structure_init                  ==
!===========================================================	
subroutine structure_init(t_s,t_v,t_lms)

type(st_structure), intent(inout) :: t_s
type(st_volume), intent(in)       :: t_v
type(st_liste_milieu_simple), intent(in) :: t_lms

integer  :: i,j
real     :: borne_volume(6),ncell(6)
type(st_structure_sibc), pointer ::tsibc
type(st_structure_bibc), pointer ::tbibc

borne_volume(1) = t_v%ivold
borne_volume(3) = t_v%jvold
borne_volume(5) = t_v%kvold
borne_volume(2) = t_v%ivolf
borne_volume(4) = t_v%jvolf
borne_volume(6) = t_v%kvolf
ncell(1:2) = t_v%nx
ncell(3:4) = t_v%ny
ncell(5:6) = t_v%nz

! prolongement des structures milieu dans les PMLs si elles existent
do i=1,t_s%nbmilieu
  if (t_s%t_milieu(1,i) <= 1)  then
    t_s%t_milieu(1,i) = t_v%ivold-1  ! xinf
  end if

  if (t_s%t_milieu(3,i) <= 1)  then 
    t_s%t_milieu(3,i) = t_v%jvold-1  ! yinf
  end if

  if (t_s%t_milieu(5,i) <= 1)  then
    t_s%t_milieu(5,i) = t_v%kvold-1  ! zinf
  end if
  
  if (t_s%t_milieu(2,i) >= t_v%nx+1)  then
    t_s%t_milieu(2,i) = t_v%ivolf+2  ! xsup
  end if
  
  if (t_s%t_milieu(4,i) >= t_v%ny+1)  then
    t_s%t_milieu(4,i) = t_v%jvolf+2  ! ysup
  end if
  
  if (t_s%t_milieu(6,i) >= t_v%nz+1)  then
    t_s%t_milieu(6,i) = t_v%kvolf+2  ! zsup
  end if

end do			

! prolongement des structures métalliques dans les PMLs si elles existent
do i=1,t_s%nbmetal
!  print '(T10,"n°=",I6,"  coord metal=",6I5)',i,t_s%t_metal(1:6,i)
  if (t_s%t_metal(1,i) <= 1)  t_s%t_metal(1,i) = t_v%ivold-1  ! xinf
  if (t_s%t_metal(3,i) <= 1)  t_s%t_metal(3,i) = t_v%jvold-1  ! yinf
  if (t_s%t_metal(5,i) <= 1)  t_s%t_metal(5,i) = t_v%kvold-1  ! zinf

  if (t_s%t_metal(2,i) >= t_v%nx+1)  t_s%t_metal(2,i) = t_v%ivolf+1  ! xsup
  if (t_s%t_metal(4,i) >= t_v%ny+1)  t_s%t_metal(4,i) = t_v%jvolf+1  ! ysup
  if (t_s%t_metal(6,i) >= t_v%nz+1)  t_s%t_metal(6,i) = t_v%kvolf+1  ! zsup
!  print '(T10,"n°=",I6,"  coord metal=",6I5)',i,t_s%t_metal(1:6,i)
end do			

! prolongement des objets dans les PMLs si nécessaires
do j=1,t_s%nbobjet
  do i=1,t_s%t_objet(j)%nbstructure
    if (t_s%t_objet(j)%coord(1,i) <= 1)  t_s%t_objet(j)%coord(1,i) = t_v%ivold-1  ! xinf
    if (t_s%t_objet(j)%coord(3,i) <= 1)  t_s%t_objet(j)%coord(3,i) = t_v%jvold-1  ! yinf
    if (t_s%t_objet(j)%coord(5,i) <= 1)  t_s%t_objet(j)%coord(5,i) = t_v%kvold-1  ! zinf

    if (t_s%t_objet(j)%coord(2,i) >= t_v%nx+1)  t_s%t_objet(j)%coord(2,i) = t_v%ivolf+1  ! xsup
    if (t_s%t_objet(j)%coord(4,i) >= t_v%ny+1)  t_s%t_objet(j)%coord(4,i) = t_v%jvolf+1  ! ysup
    if (t_s%t_objet(j)%coord(6,i) >= t_v%nz+1)  t_s%t_objet(j)%coord(6,i) = t_v%kvolf+1  ! zsup
  end do
end do

do j=1,t_s%nbfente
  if (t_s%t_fente(j)%coord(1) <= 1)  t_s%t_fente(j)%coord(1) = t_v%ivold-1  ! xinf
  if (t_s%t_fente(j)%coord(3) <= 1)  t_s%t_fente(j)%coord(3) = t_v%jvold-1  ! yinf
  if (t_s%t_fente(j)%coord(5) <= 1)  t_s%t_fente(j)%coord(5) = t_v%kvold-1  ! zinf

  if (t_s%t_fente(j)%coord(2) >= t_v%nx+1)  t_s%t_fente(j)%coord(2) = t_v%ivolf+1  ! xsup
  if (t_s%t_fente(j)%coord(4) >= t_v%ny+1)  t_s%t_fente(j)%coord(4) = t_v%jvolf+1  ! ysup
  if (t_s%t_fente(j)%coord(6) >= t_v%nz+1)  t_s%t_fente(j)%coord(6) = t_v%kvolf+1  ! zsup
end do

do j=1,t_s%nbplaque
  if (t_s%t_plaque(j)%coord(1) <= 1)  t_s%t_plaque(j)%coord(1) = t_v%ivold-1  ! xinf
  if (t_s%t_plaque(j)%coord(3) <= 1)  t_s%t_plaque(j)%coord(3) = t_v%jvold-1  ! yinf
  if (t_s%t_plaque(j)%coord(5) <= 1)  t_s%t_plaque(j)%coord(5) = t_v%kvold-1  ! zinf

  if (t_s%t_plaque(j)%coord(2) >= t_v%nx+1)  t_s%t_plaque(j)%coord(2) = t_v%ivolf+2  ! xsup
  if (t_s%t_plaque(j)%coord(4) >= t_v%ny+1)  t_s%t_plaque(j)%coord(4) = t_v%jvolf+2  ! ysup
  if (t_s%t_plaque(j)%coord(6) >= t_v%nz+1)  t_s%t_plaque(j)%coord(6) = t_v%kvolf+2  ! zsup
end do


! Prolongement SIBC dans les PMLs si nécessaire
do i=1,t_s%nbsibc
  tsibc=> t_s%t_sibc(i)
  do j=1,5,2
    if (tsibc%coord(j)<=1) tsibc%coord(j) = borne_volume(j)
    if (tsibc%coord(j+1)>=ncell(j+1)+1) tsibc%coord(j+1) = borne_volume(j+1)+1
  end do
  !print '("SIBC coord=",6I5)',tsibc%coord(1:6)
end do

! Prolongement BIBC dans les PMLs si nécessaire
do i=1,t_s%nbbibc
  tbibc=> t_s%t_bibc(i)
  do j=1,5,2
    if (tbibc%coord(j)<=1)              tbibc%coord(j)   = borne_volume(j)
    if (tbibc%coord(j+1)>=ncell(j+1)+1) tbibc%coord(j+1) = borne_volume(j+1)+1
  end do
  !print '("BIBC coord=",6I5)',tbibc%coord(1:6)
end do

call structure_fente_init(t_s,t_v)
call structure_surface_init(t_s,t_v)

call structure_mem_nboper(t_s,t_v)

!---- Recherche du type de remplissage électrique ---
if (t_v%sigma_e/=0) then
  t_s%type_milieu_dielec=VOL_DIELEC_PERTES_HOMOGENE
end if

if (t_s%nbfente >0 .or. t_s%flag_non_metal.or.t_s%flag_grille_metal .or. &
      t_s%nbsibc>0 .or. t_s%nbbibc>0 .or.t_lms%flag_epsr_nondefaut.or. &
      t_s%nbfil>0  .or. t_s%nbmetal>1000) then
  t_s%type_milieu_dielec=VOL_DIELEC_INHOMOGENE
end if

if (t_lms%flag_sige_nondefaut) then
  t_s%type_milieu_dielec=VOL_DIELEC_PERTES_INHOMOGENE
end if

!---- Recherche du type de remplissage magnétique ---
if (t_v%sigma_h/=0) then
  t_s%type_milieu_magnet=VOL_MAGNET_PERTES_HOMOGENE
end if

if (t_s%nbfente >0 .or. t_lms%flag_mur_nondefaut .or.t_s%nbfil>0) then
  t_s%type_milieu_magnet=VOL_MAGNET_INHOMOGENE
end if

if (t_lms%flag_sigh_nondefaut) then
  t_s%type_milieu_magnet=VOL_MAGNET_PERTES_INHOMOGENE
end if

!--> Affichage information sur les types de milieu dans le 
!     volume de calcul
select case (t_s%type_milieu_dielec)
  case(VOL_DIELEC_HOMOGENE)
    if (rang==RANG_MAITRE_AFF) print '(T10,"Espace diélectrique homogène sans pertes")'
  case(VOL_DIELEC_PERTES_HOMOGENE)
    if (rang==RANG_MAITRE_AFF) print '(T10,"Espace diélectrique homogène, avec pertes électriques homogènes")'
  case(VOL_DIELEC_INHOMOGENE)
    if (rang==RANG_MAITRE_AFF) print '(T10,"Espace diélectrique inhomogène sans pertes")'
  case(VOL_DIELEC_PERTES_INHOMOGENE)
    if (rang==RANG_MAITRE_AFF) print '(T10,"Espace diélectrique inhomogène avec pertes")'
end select  

select case (t_s%type_milieu_magnet)
  case(VOL_MAGNET_HOMOGENE)
    if (rang==RANG_MAITRE_AFF) print '(T10,"Espace magnétique homogène sans pertes")'
  case(VOL_MAGNET_PERTES_HOMOGENE)
    if (rang==RANG_MAITRE_AFF) print '(T10,"Espace magnétique homogène, avec pertes magnétiques homogènes")'
  case(VOL_MAGNET_INHOMOGENE)
    if (rang==RANG_MAITRE_AFF) print '(T10,"Espace magnétique inhomogène sans pertes")'
  case(VOL_MAGNET_PERTES_INHOMOGENE)
    if (rang==RANG_MAITRE_AFF) print '(T10,"Espace magnétique inhomogène avec pertes")'
end select  


    


end subroutine structure_init

! ==========================================================
! == Subroutine mem_nboper                                ==
! ==   Evaluation de la taille mémoire en octets dans la
! ==    structure et du nbre d'opération flottante si impactant
! ==     sur le temps de calcul
!===========================================================	
subroutine structure_mem_nboper(t_s,v)

type(st_structure), intent(inout) :: t_s
type(st_volume), intent(in)       :: v

t_s%nb_oper    = 0

if (t_s%flag_face_cell) then
  t_s%taille_mem = (v%ivolf-v%ivold+3)*(v%jvolf-v%jvold+3)*(v%kvolf-v%kvold+3)*IP_16B
end if

end subroutine structure_mem_nboper


! ==========================================================
! == Subroutine structure_surface_init                    ==
! == Stockage d'information sur les matériaux surfaciques ==
!===========================================================	
subroutine structure_surface_init(t_s,v)

type(st_structure), intent(inout) :: t_s
type(st_volume), intent(in)       :: v
integer  :: n,coord(6)
logical  :: flag
type(st_structure_sibc), pointer ::tsibc
type(st_structure_bibc), pointer ::tbibc

if (t_s%nbbibc>0 .or. t_s%nbsibc>0 .or. t_s%nbmetal>0) then
  t_s%flag_face_cell = .true.
  allocate(t_s%t_face_cell(v%ivold-1:v%ivolf+1,v%jvold-1:v%jvolf+1,v%kvold-1:v%kvolf+1))
  t_s%t_face_cell = 0
  !---- bits 0:2 --> face xoy,yoz et xoz
  !---- bits 3 --> face de type metal
  !---- bits 4 --> face de type sibc
  !---- bits 5 --> face de type bibc
  !
  !--------------------- Traitement des SURFACES METALS ------------------------
  do n=1,t_s%nbmetal
    coord = t_s%t_metal(1:6,n)
    flag = .true.
    if (t_s%t_metal(7,n)==0) flag = .false.
    call structure_surface_setface(t_s,coord,DEC_BIN_METAL,flag)
  end do  
  !--------------------- Traitement des SIBC ------------------------
  do n=1,t_s%nbsibc
    tsibc=> t_s%t_sibc(n)
    call structure_surface_setface(t_s,tsibc%coord,DEC_BIN_SIBC,tsibc%flag_active)
  end do  
  !--------------------- Traitement des BIBC ------------------------
  do n=1,t_s%nbbibc
    tbibc=> t_s%t_bibc(n)
    call structure_surface_setface(t_s,tbibc%coord,DEC_BIN_BIBC,tbibc%flag_active)
  end do  

  do n=1,t_s%nbfente
    flag = .false.
    call structure_surface_setface(t_s,t_s%t_fente(n)%coord(1:6),DEC_BIN_METAL,flag) ! rectification face métal
  end do
end if

! TEST !!!
!call structure_surface_test(t_s,v)
end subroutine structure_surface_init

! ==========================================================
! == Subroutine structure_surface_test                    ==
! == test de détection des matériaux surfacique           ==
!===========================================================	
subroutine structure_surface_test(t_s,v)

type(st_structure), intent(inout) :: t_s
type(st_volume), intent(in)       :: v
integer  :: n,i,j,k
character(LEN=LEN_MOT_CLE) :: ch

!==== Test Plan XOY ====
if (.not.t_s%flag_face_cell) return

if (rang==RANG_MAITRE_AFF) PRINT '(T5,"PLAN XOY, DESCRIPTION DES FACETTES METAL, SIBC, BIBC")'
do k=v%kvold-1,v%kvolf+1
do j=v%jvold-1,v%jvolf+1
do i=v%ivold-1,v%ivolf+1
  ch=""
  if (structure_face_metal(t_s,(/i,j,k/),PLAN_XOY)) ch = "METAL"
  if (structure_face_sibc(t_s,(/i,j,k/),PLAN_XOY))  ch=trim(ch)//" SIBC"
  if (structure_face_bibc(t_s,(/i,j,k/),PLAN_XOY))  ch=trim(ch)//" BIBC"
  n=structure_matqcq_face(t_s,(/i,j,k/),PLAN_XOY)  
  if (n>0 .and. rang==RANG_MAITRE_AFF) print '(T5," Facette XOY de type=",I1," (I,J,K)=",3I4,"  ",A)',n,i,j,k,TRIM(ch)
end do;end do;end do
!==== Test Plan YOZ ====
if (rang==RANG_MAITRE_AFF) PRINT '(T5,"PLAN YOZ, DESCRIPTION DES FACETTES METAL, SIBC, BIBC")'
do k=v%kvold-1,v%kvolf+1
do j=v%jvold-1,v%jvolf+1
do i=v%ivold-1,v%ivolf+1
  ch=""
  if (structure_face_metal(t_s,(/i,j,k/),PLAN_YOZ)) ch = "METAL"
  if (structure_face_sibc(t_s,(/i,j,k/),PLAN_YOZ))  ch=trim(ch)//" SIBC"
  if (structure_face_bibc(t_s,(/i,j,k/),PLAN_YOZ))  ch=trim(ch)//" BIBC"
  n=structure_matqcq_face(t_s,(/i,j,k/),PLAN_YOZ)  
  if (n>0.and. rang==RANG_MAITRE_AFF) print '(T5," Facette YOZ de type=",I1," (I,J,K)=",3I4,"  ",A)',n,i,j,k,TRIM(ch)
end do;end do;end do
!==== Test Plan XOZ ====
if (rang==RANG_MAITRE_AFF) PRINT '(T5,"PLAN XOZ, DESCRIPTION DES FACETTES METAL, SIBC, BIBC")'
do k=v%kvold-1,v%kvolf+1
do j=v%jvold-1,v%jvolf+1
do i=v%ivold-1,v%ivolf+1
  ch=""
  if (structure_face_metal(t_s,(/i,j,k/),PLAN_XOZ)) ch = "METAL"
  if (structure_face_sibc(t_s,(/i,j,k/),PLAN_XOZ))  ch=trim(ch)//" SIBC"
  if (structure_face_bibc(t_s,(/i,j,k/),PLAN_XOZ))  ch=trim(ch)//" BIBC"
  n=structure_matqcq_face(t_s,(/i,j,k/),PLAN_XOZ)  
  if (n>0.and. rang==RANG_MAITRE_AFF) print '(T5," Facette XOZ de type=",I1," (I,J,K)=",3I4,"  ",A)',n,i,j,k,TRIM(ch)
end do;end do;end do

end subroutine structure_surface_test

! ==========================================================
! == Function structure_surface_setface                   ==
!---- bits 0:2 --> face de type metal,face xoy,yoz et xoz
!---- bits 3:5 --> face de type sibc,face xoy,yoz et xoz
!---- bits 6:8 --> face de type bibc,face xoy,yoz et xoz
! ==========================================================
subroutine structure_surface_setface(t_s,coord,type_mat,flag)
type(st_structure), intent(inout) :: t_s
integer,dimension(6), intent(in)  :: coord  ! coord cellule
integer, intent(in)               :: type_mat ! type de matériau, décalage binaire
logical, intent(in)               :: flag ! face active ou pas

integer :: i,j,k,k1,k2,i1,i2,j1,j2

if (coord(XMIN)<coord(XMAX) .and. coord(YMIN)<coord(YMAX)) then
  k1 =coord(ZMIN);k2=coord(ZMAX)
  if (flag) then
    do j=coord(YMIN),coord(YMAX)-1
    do i=coord(XMIN),coord(XMAX)-1
      t_s%t_face_cell(i,j,k1) = IBSET(t_s%t_face_cell(i,j,k1),0+type_mat) 
      t_s%t_face_cell(i,j,k2) = IBSET(t_s%t_face_cell(i,j,k2),0+type_mat) 
    end do;end do;
  else
    do j=coord(YMIN),coord(YMAX)-1
    do i=coord(XMIN),coord(XMAX)-1
      t_s%t_face_cell(i,j,k1) = IBCLR(t_s%t_face_cell(i,j,k1),0+type_mat) 
      t_s%t_face_cell(i,j,k2) = IBCLR(t_s%t_face_cell(i,j,k2),0+type_mat) 
    end do;end do;      
  end if
end if
! plans YOZ 
if (coord(ZMIN)<coord(ZMAX) .and. coord(YMIN)<coord(YMAX)) then
  i1 =coord(XMIN);i2 =coord(XMAX)
  if (flag) then
    do k=coord(ZMIN),coord(ZMAX)-1
    do j=coord(YMIN),coord(YMAX)-1
      t_s%t_face_cell(i1,j,k) = IBSET(t_s%t_face_cell(i1,j,k),1+type_mat) 
      t_s%t_face_cell(i2,j,k) = IBSET(t_s%t_face_cell(i2,j,k),1+type_mat) 
    end do;end do;
  else
    do k=coord(ZMIN),coord(ZMAX)-1
    do j=coord(YMIN),coord(YMAX)-1
      t_s%t_face_cell(i1,j,k) = IBCLR(t_s%t_face_cell(i1,j,k),1+type_mat) 
      t_s%t_face_cell(i2,j,k) = IBCLR(t_s%t_face_cell(i2,j,k),1+type_mat) 
    end do;end do;      
  end if
end if
! plans ZOX 
if (coord(ZMIN)<coord(ZMAX) .and. coord(XMIN)<coord(XMAX)) then
  j1 =coord(YMIN);j2 =coord(YMAX) 
  if (flag) then
    do k=coord(ZMIN),coord(ZMAX)-1
    do i=coord(XMIN),coord(XMAX)-1
      t_s%t_face_cell(i,j1,k) = IBSET(t_s%t_face_cell(i,j1,k),2+type_mat) 
      t_s%t_face_cell(i,j2,k) = IBSET(t_s%t_face_cell(i,j2,k),2+type_mat) 
    end do;end do;
  else
    do k=coord(ZMIN),coord(ZMAX)-1
    do i=coord(XMIN),coord(XMAX)-1
      t_s%t_face_cell(i,j1,k) = IBCLR(t_s%t_face_cell(i,j1,k),2+type_mat) 
      t_s%t_face_cell(i,j2,k) = IBCLR(t_s%t_face_cell(i,j2,k),2+type_mat) 
    end do;end do;      
  end if
end if
end subroutine structure_surface_setface

! ==========================================================
! == Subroutine structure_matqcq_face                    ==
! == Détecte si la face de la cellule est de type metal ou
! == sibc ou bibc
!===========================================================	
function structure_matqcq_face(t_s,c,type_plan) result (ty_face)

type(st_structure), intent(in) :: t_s
integer, dimension(3), intent(in) :: c
integer, intent(in) :: type_plan
logical :: flag2, flag(3)
integer :: ty_face
integer :: i,n

ty_face=0
if (.not.t_s%flag_face_cell) return

n=t_s%t_face_cell(c(1),c(2),c(3))
flag(1)=BTEST(n,type_plan-1+DEC_BIN_METAL)
flag(2)=BTEST(n,type_plan-1+DEC_BIN_SIBC)
flag(3)=BTEST(n,type_plan-1+DEC_BIN_BIBC)

ty_face = 0
do i=1,3
  if (flag(i)) ty_face = IBSET(ty_face,i-1)
end do

end function structure_matqcq_face

! ==========================================================
! == Subroutine structure_surfacique_cellule              ==
!===========================================================	
function structure_surfacique_cellule(t_s,c) result(flagsurf_face)
!=== Déclaration des arguments ===
type(st_structure), intent(in) :: t_s
integer,dimension(3), intent(in)  :: c

logical, dimension(6) :: flagsurf_face
!===  Déclaration des variables  ===
integer  :: l,n
integer :: ty_face
logical :: flag(3)
!===   Début des traitements   ===
flagsurf_face = .false.				
if (.not.t_s%flag_face_cell) return
ty_face=0
n=t_s%t_face_cell(c(1),c(2),c(3))
! boucle sur touts les plans
do l=1,3
  flag(1)=BTEST(n,l-1+DEC_BIN_METAL)
  flag(2)=BTEST(n,l-1+DEC_BIN_SIBC)
  flag(3)=BTEST(n,l-1+DEC_BIN_BIBC)
!  print '("l,flag(1:3)=",I2,3L1)',l,flag(1:3)
  flagsurf_face(PC21(l)) = flag(1) .or.flag(2).or.flag(3) ! 1 3 5
end do

! XSUP
n=t_s%t_face_cell(c(1)+1,c(2),c(3))
flag(1)=BTEST(n,PLAN_YOZ-1+DEC_BIN_METAL)
flag(2)=BTEST(n,PLAN_YOZ-1+DEC_BIN_SIBC)
flag(3)=BTEST(n,PLAN_YOZ-1+DEC_BIN_BIBC)
!print '("XSUP,flag(1:3)=",3L1)',flag(1:3)
flagsurf_face(YOZ_SUP) = flag(1) .or.flag(2).or.flag(3)

! YSUP
n=t_s%t_face_cell(c(1),c(2)+1,c(3))
flag(1)=BTEST(n,PLAN_XOZ-1+DEC_BIN_METAL)
flag(2)=BTEST(n,PLAN_XOZ-1+DEC_BIN_SIBC)
flag(3)=BTEST(n,PLAN_XOZ-1+DEC_BIN_BIBC)
!print '("YSUP,flag(1:3)=",3L1)',flag(1:3)
flagsurf_face(XOZ_SUP) = flag(1) .or.flag(2).or.flag(3)

! ZSUP
n=t_s%t_face_cell(c(1),c(2),c(3)+1)
flag(1)=BTEST(n,PLAN_XOY-1+DEC_BIN_METAL)
flag(2)=BTEST(n,PLAN_XOY-1+DEC_BIN_SIBC)
flag(3)=BTEST(n,PLAN_XOY-1+DEC_BIN_BIBC)
!print '("ZSUP,flag(1:3)=",3L1)',flag(1:3)
flagsurf_face(XOY_SUP) = flag(1) .or.flag(2).or.flag(3)

end function structure_surfacique_cellule
! ==========================================================
! == Subroutine structure_face_metal                    ==
! == Détecte si la face de la cellule est de type metal ou
! == pas
!===========================================================	
function structure_face_metal(t_s,c,type_plan) result (flag_face)

type(st_structure), intent(in) :: t_s
integer, dimension(3), intent(in) :: c
integer, intent(in) :: type_plan
logical :: flag_face

flag_face=.false.
if (.not.t_s%flag_face_cell) return
flag_face=BTEST(t_s%t_face_cell(c(1),c(2),c(3)),type_plan-1+DEC_BIN_METAL) !0,1 ou 2
end function structure_face_metal

! ==========================================================
! == Subroutine structure_face_sibc                       ==
! == Détecte si la face de la cellule est de type sibc ou
! == pas
!===========================================================	
function structure_face_sibc(t_s,c,type_plan) result (flag_face)

type(st_structure), intent(in) :: t_s
integer, dimension(3), intent(in) :: c
integer, intent(in) :: type_plan
logical :: flag_face

flag_face=.false.
if (.not.t_s%flag_face_cell) return
flag_face=BTEST(t_s%t_face_cell(c(1),c(2),c(3)),type_plan-1+DEC_BIN_SIBC) !3,4 ou 5
end function structure_face_sibc

! ==========================================================
! == Subroutine structure_face_bibc                       ==
! == Détecte si la face de la cellule est de type bibc ou
! == pas
!===========================================================	
function structure_face_bibc(t_s,c,type_plan) result (flag_face)

type(st_structure), intent(in) :: t_s
integer, dimension(3), intent(in) :: c
integer, intent(in) :: type_plan
logical :: flag_face

flag_face=.false.
if (.not.t_s%flag_face_cell) return
flag_face=BTEST(t_s%t_face_cell(c(1),c(2),c(3)),type_plan-1+DEC_BIN_BIBC) !6,7 ou 8
end function structure_face_bibc


! ==========================================================
! == Function structure_objet_pointe(ts ,chaine)          ==
! == Retourne un pointeur pointant sur la                 ==
! ==========================================================
function structure_objet_pointe(ts,chaine) result(tso)

type(st_structure), intent(inout) :: ts
character(LEN=LEN_MOT_CLE), intent(in) :: chaine

type(st_structure_objet), pointer :: tso
integer :: i

nullify(tso)
do i=1,ts%nbobjet
  if (chaine==ts%t_objet(i)%nom) then
    tso => ts%t_objet(i)
    exit
  endif
enddo

end function structure_objet_pointe

! ==========================================================
! == Function structure_objet_num                         ==
! == Retourne le n° de l'objet correspondant au nom       ==
! == mentionné
! ==========================================================
integer function structure_objet_num(ts,chaine) result(num)

type(st_structure), intent(inout) :: ts
character(LEN=LEN_MOT_CLE), intent(in) :: chaine

integer :: i

num=-1
do i=1,ts%nbobjet
  if (chaine==ts%t_objet(i)%nom) then
    num =i
    exit
  endif
enddo
if (num==-1) then
  print '("Avertissement, Objet de nom ",A," NON RECONNU !!!, initialisation à Objet par défaut")',&
             trim(chaine)
  num=1
end if

end function structure_objet_num

! ==========================================================
! == Subroutine structure_metal_cellule                  ==
!===========================================================	
function structure_metal_cellule(t_s,c) result(flagmetal_face)
!=== Déclaration des arguments ===
type(st_structure), intent(in) :: t_s
integer,dimension(3), intent(in)  :: c

logical, dimension(6) :: flagmetal_face
!===  Déclaration des variables  ===
integer  :: i,j

!===   Début des traitements   ===
flagmetal_face = .false.				
! prolongement des structures métalliques dans les PMLs si elles existent
do i=1,t_s%nbmetal
  do j=1,3
    ! Test sur les plan XOYINF, XOYSUP, YOZINF ...
    if (t_s%t_metal(PC21(j),i)<=c(j) .and. t_s%t_metal(PC21(j)+1,i)>c(j)) then
      if (t_s%t_metal(PC23(j),i)<=c(PC1(j)) .and. t_s%t_metal(PC23(j)+1,i)>c(PC1(j))) then
	if (t_s%t_metal(PC25(j),i)==c(PC2(j)).or.t_s%t_metal(PC25(j)+1,i)==c(PC2(j))) then
	  if (t_s%t_metal(7,i)==1) then
	    flagmetal_face(PC21(j)) = .true.
	  else
	    flagmetal_face(PC21(j)) = .false.
	  end if
	end if
	if (t_s%t_metal(PC25(j),i)==(c(PC2(j))+1).or.t_s%t_metal(PC25(j)+1,i)==(c(PC2(j))+1)) then
	  if (t_s%t_metal(7,i)==1) then
	    flagmetal_face(PC21(j)+1) = .true.
	  else
	    flagmetal_face(PC21(j)+1) = .false.
	  end if
	end if
      end if
    end if
  end do

end do

end function structure_metal_cellule
	
!===========================================================	
! Subroutine structure_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
Subroutine structure_precalcul()
implicit none
!=== Déclaration des arguments ===

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine structure_precalcul


!===========================================================	
! Subroutine structure_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine structure_affichage(t_s)
!=== Déclaration des arguments ===
type(st_structure), intent(inout) :: t_s

!===  Déclaration des variables  ===
integer  :: i,j
character (LEN=LEN_MOT_CLE), dimension(3) :: ch
type(st_structure_fente), pointer :: tfente

!===   Début des traitements   ===
if (rang==RANG_MAITRE_AFF) then
  print *
  print '("------- MODULE STRUCTURE ---------")'
  !do i=1,t_s%nbmilieu
  !  print '(T10," Structure milieu n°:",I4," nommilieu, coordonnées :",A,6I5)',i,TRIM(t_s%nommilieu(i)),t_s%t_milieu(1:6,i)
  !end do				
  !do i=1,t_s%nbmetal
  !  print '(T10," Structure metal n°:",I4," coordonnées :",6I5)',i,t_s%t_metal(1:6,i)
  !end do				
  do j=1,t_s%nbobjet
    print '(T5," OBJET : ",A,"  ", A)',TRIM(t_s%t_objet(j)%nom),TRIM(t_s%t_objet(j)%typemilieu)
  !  do i=1,t_s%t_objet(j)%nbstructure
  !    print '(T15," Structure objet n°:",I4," coordonnées :",6I5)',i,t_s%t_objet(j)%coord(1:6,i)
  !  end do
  end do
  ch(1) = "X"
  ch(2) = "Y"
  ch(3) = "Z"
  
  do j=1,t_s%nbfente
    tfente =>t_s%t_fente(j) 
    print '(T10," FENTE : ",A," ; largeur = ", ES12.3e2, " ; epsr =",ES12.3e2)',&
		    TRIM(tfente%nom),tfente%w,tfente%epsr
    print '(T15," Direction E : ",A," ; Direction H = ", A, " ; Direction fente =",A)',&
		    TRIM(ch(tfente%dirE)),TRIM(ch(tfente%dirH)),TRIM(ch(tfente%dirfente))
    print '(T15," Nom du formalisme TSF : ",A)',TRIM(tfente%nom_tsf)
  end do		  
end if
End Subroutine structure_affichage


!===========================================================	
! Subroutine structure_deallocate
!  Libération de toute la mémoire de la structure st_structure
!===========================================================
subroutine structure_deallocate(ts)

!=== Déclaration des arguments ===
type(st_structure), intent(inout) :: ts

!===  Déclaration des variables  ===
integer :: i

!===   Début des traitements   ===
if (ts%nbmilieu>0) then
  deallocate(ts%t_milieu,ts%decal,ts%nommilieu)
end if
if (ts%nbmetal>0) deallocate(ts%t_metal)
if (ts%nbgrille_metal>0) deallocate(ts%t_grille_metal)

if (ts%nbobjet>0) then
  do i=1,ts%nbobjet
    if (associated(ts%t_objet(i)%coord)) deallocate(ts%t_objet(i)%coord)
  end do
  deallocate(ts%t_objet)
end if

if (ts%nbplaque>0) deallocate(ts%t_plaque)
if (ts%nbfente>0)  deallocate (ts%t_fente)
if (ts%nbsurf_rugueuse>0) deallocate (ts%t_surf_rugueuse)
if (ts%nbsibc>0) deallocate(ts%t_sibc)
if (ts%nbbibc>0) deallocate(ts%t_bibc)
end subroutine structure_deallocate

End Module mfdtd_structure	



