!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_structure_dispersive.f90
!// Purpose:     Traitement pour la mise à jour des champs
!//          dans les milieux dispersifs
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Octobre 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////
!================================================================
! Traitement des milieux dispersifs
!================================================================
module mfdtd_structure_dispersive
use m_constantes
use m_constantes_scientifiques
use m_math_operation
use m_milieu_dispersif, only : st_milieu_dispersif, &
                                    st_un_milieu_dispersif, &
				    st_un_pole_dispersif, &
				    identifie_milieu_dispersif, &
				    milieu_dispersif_pointe
use mfdtd_volume, only : st_volume
use mfdtd_structure, only : st_structure
use mfdtd_champ, only : st_champ


 
!=== Déclaration des types ===
!--- Description des coefficients pour une structure dispersive ---
type st_pole_disp_structure
  ! paramètre d'un milieu général
  logical  :: flag_reel, flag_complexe
  integer  :: nb_pole_reel, nb_pole_complexe

  ! borne de calcul propre à la composante
  integer, dimension(6)   :: borne 

  integer(kind=2), pointer, dimension(:,:,:) :: nu
  
  ! pointeur sur une des composantes de e ou h et 
  ! son coefficient
  real, dimension(:,:,:), pointer  :: eh, ceh2

  ! coefficient multiplicatif de Jd, Jd : courant de la méthode JEC
  real, dimension(:), pointer    :: cJr1
  real, dimension(:,:), pointer  :: cJr2 
  real, pointer, dimension(:,:,:,:) :: Jr
  
  ! coefficient multiplicatif de Jd, Jd : courant de la méthode JEC
  ! coefficient complexe
  complex, dimension(:), pointer    :: cJc1
  complex, dimension(:,:), pointer  :: cJc2 
  complex, pointer, dimension(:,:,:,:) :: Jc
  
  
  ! coefficient multiplicatif de Jd, Jd : courant pour Lorentz, ordre 2, méthode ADE
  real                           :: dt_eps0
  real, dimension(:), pointer    :: cJl1,cJl0
  real, dimension(:,:), pointer  :: cJl2,cJle
  real, pointer, dimension(:,:,:,:) :: Jl,Ql
  
end type st_pole_disp_structure


!--- la structure : structure_dispersive ---
type st_une_structure_dispersive
  character(LEN=LEN_MOT_CLE) :: nom_milieu
  
  integer, dimension(6)   :: borne
  
  real, dimension(:,:,:), pointer  :: Jd
  
  type (st_un_milieu_dispersif), pointer  :: un_md
  type(st_pole_disp_structure) :: t_ex, t_ey, t_ez
  type(st_pole_disp_structure) :: t_hx, t_hy, t_hz
   
end type st_une_structure_dispersive

!--- Structure globale pour les milieux dispersifs ---
type st_structure_dispersive
  integer  :: nb_structure
  logical  :: flag_eps, flag_mu
  type(st_une_structure_dispersive), dimension(:), pointer :: t_une_sd    

end type st_structure_dispersive
 
! Pas de variables globales...

contains

!===========================================================	
! Subroutine structure_dispersive_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine structure_dispersive_initdefaut(tsd)
 implicit none
!=== Déclaration des arguments ===
type(st_structure_dispersive), intent(inout) :: tsd

!===  Déclaration des variables  ===

!===   Début des traitements   ===
tsd%nb_structure = 0
tsd%flag_eps = .false.		
tsd%flag_mu = .false.		
NULLIFY(tsd%t_une_sd)	
	
End Subroutine structure_dispersive_initdefaut

!===========================================================	
! Subroutine structure_dispersive_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
Subroutine pole_disp_structure_initdefaut(tpds)
 implicit none
!=== Déclaration des arguments ===
type(st_pole_disp_structure), intent(inout) :: tpds

!===  Déclaration des variables  ===

!===   Début des traitements   ===
tpds%flag_reel     = .false.
tpds%flag_complexe = .false.
tpds%nb_pole_reel  = 0		
tpds%nb_pole_complexe = 0		
NULLIFY(tpds%nu); 	
	
End Subroutine pole_disp_structure_initdefaut


!===========================================================	
! Subroutine structure_dispersive_init
! Initialisation des variables, lorsque qu'il n'y a pas de lecture du fichier
! d'entrée
!===========================================================	
Subroutine structure_dispersive_init()
implicit none
!=== Déclaration des arguments ===

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
End Subroutine structure_dispersive_init
	
!===========================================================	
! Subroutine structure_dispersive_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
Subroutine structure_dispersive_precalcul(tsd,tmd,ts,tv,tc)
implicit none
!=== Déclaration des arguments ===
type(st_structure_dispersive), intent(inout) :: tsd
type(st_milieu_dispersif), intent(in) :: tmd
type(st_structure), intent(in) :: ts
type(st_volume), intent(in) :: tv
type(st_champ), intent(in) :: tc

!===  Déclaration des variables  ===
integer :: i,n,p
integer :: borne_volume(6)
real  :: coef2
type(st_une_structure_dispersive), pointer :: une_sd
!===   Début des traitements   ===

!--- on compte le nombre de structures dispersives ---
call structure_dispersive_initdefaut(tsd)
n = 0
do i=1, ts%nbmilieu
 if (identifie_milieu_dispersif(tmd,ts%nommilieu(i))) then
   n = n +1
 end if
end do

tsd%nb_structure = n
if (n>0) then
  allocate(tsd%t_une_sd(n))
else
  print '("Avertissement : pas de structures dispersives")'
  return
end if

borne_volume = tv%borne_volume
borne_volume(XMAX:ZMAX:2) = borne_volume(XMAX:ZMAX:2)+1
print '(T10,"Borne du volume de calcul limitant pour les milieux dispersifs=",6I6)',borne_volume
!--- on récupère pour chaque structure dispersive 
!    son milieu dispersif et ses bornes ---
n = 0
do i=1, ts%nbmilieu
 if (identifie_milieu_dispersif(tmd,ts%nommilieu(i))) then
   n = n +1
   une_sd => tsd%t_une_sd(n)
   une_sd%un_md => milieu_dispersif_pointe(tmd,ts%nommilieu(i))
   une_sd%nom_milieu = ts%nommilieu(i)
   une_sd%borne(1:6) = ts%t_milieu(1:6,i)
   ! correction borne inférieur si structure dans les frontières
    if (une_sd%borne(XMAX)>=borne_volume(XMAX)) une_sd%borne(XMAX) = borne_volume(XMAX) 
    if (une_sd%borne(YMAX)>=borne_volume(YMAX)) une_sd%borne(YMAX) = borne_volume(YMAX) 
    if (une_sd%borne(ZMAX)>=borne_volume(ZMAX)) une_sd%borne(ZMAX) = borne_volume(ZMAX) 
    une_sd%borne(XMIN) = max(une_sd%borne(XMIN),borne_volume(XMIN))
    une_sd%borne(YMIN) = max(une_sd%borne(YMIN),borne_volume(YMIN))
    une_sd%borne(ZMIN) = max(une_sd%borne(ZMIN),borne_volume(ZMIN))
    if (une_sd%borne(XMIN)>=une_sd%borne(XMAX) .or. &
        une_sd%borne(YMIN)>=une_sd%borne(YMAX) .or.&
	une_sd%borne(ZMIN)>=une_sd%borne(ZMAX)) then
      print '("ERREUR, Borne max doit etre > à Borne min :", 6I6)',une_sd%borne
      print '(" Dans le milieu dispersif n°",I7," de nom ",A)',n,trim(une_sd%nom_milieu)
      stop
    end if
 end if
end do

!--- allocation, calcul des coefficients utiles pour le calcul ---
!    ceci pour chaque pole de chaque milieu dispersif
do n=1, tsd%nb_structure 
  une_sd => tsd%t_une_sd(n)
  call pole_disp_structure_initdefaut(une_sd%t_ex)
  call pole_disp_structure_initdefaut(une_sd%t_ey)
  call pole_disp_structure_initdefaut(une_sd%t_ez)
  call pole_disp_structure_initdefaut(une_sd%t_hx)
  call pole_disp_structure_initdefaut(une_sd%t_hy)
  call pole_disp_structure_initdefaut(une_sd%t_hz)
  
  ! Allocation d'un tableau de stockage temporaire du courant
  allocate(une_sd%Jd(une_sd%borne(1):une_sd%borne(2),une_sd%borne(3):une_sd%borne(4),une_sd%borne(5):une_sd%borne(6)))  
  une_sd%Jd = 0.0
  
  if (une_sd%un_md%flag_eps) then
    tsd%flag_eps = une_sd%un_md%flag_eps
    ! alias vers ex, ey, ez et leurs coefficients
    une_sd%t_ex%eh => tc%ex
    une_sd%t_ey%eh => tc%ey
    une_sd%t_ez%eh => tc%ez
    une_sd%t_ex%ceh2 => tc%cex2
    une_sd%t_ey%ceh2 => tc%cey2
    une_sd%t_ez%ceh2 => tc%cez2

    ! bornes de calcul pour chaque composante epsilon
    une_sd%t_ex%borne(1:6) = une_sd%borne(1:6)
    une_sd%t_ey%borne(1:6) = une_sd%borne(1:6)
    une_sd%t_ez%borne(1:6) = une_sd%borne(1:6)
    ! rectification borne uniquement dans la direction de la composante
    une_sd%t_ex%borne(XMAX) = une_sd%borne(XMAX)-1
    une_sd%t_ey%borne(YMAX) = une_sd%borne(YMAX)-1
    une_sd%t_ez%borne(ZMAX) = une_sd%borne(ZMAX)-1
    
    ! Cas des frontières limites, les bornes de calcul sont celles 
    ! des champs.
    do i=XMIN,ZMIN,2
      if (une_sd%borne(i) <= borne_volume(i)) then
        une_sd%t_ex%borne(i) = tc%borne_ex(i)
        une_sd%t_ey%borne(i) = tc%borne_ey(i)
        une_sd%t_ez%borne(i) = tc%borne_ez(i)
      end if
    end do
    do i=XMAX,ZMAX,2
      if (une_sd%borne(i)>= borne_volume(i)) then
        une_sd%t_ex%borne(i) = tc%borne_ex(i)
        une_sd%t_ey%borne(i) = tc%borne_ey(i)
        une_sd%t_ez%borne(i) = tc%borne_ez(i)
      end if
      !if (une_sd%t_ex%borne(i)<=une_sd%t_ex%borne(i-1)) then
    end do
    ! Precalcul pour la composante Ex, calcul coefficients dispersifs
    if (une_sd%un_md%nb_pole_eps(1) > 0) &
      call pole_dispersif_structure(une_sd%t_ex,une_sd%un_md%t_epsx, &
                                  une_sd%un_md%nb_pole_eps(1),une_sd%borne,tv%dt,'eps_x',borne_volume)

    ! Precalcul pour la composante Ey, calcul coefficients dispersifs
    if (une_sd%un_md%nb_pole_eps(2) > 0) &
      call pole_dispersif_structure(une_sd%t_ey,une_sd%un_md%t_epsy, &
                                  une_sd%un_md%nb_pole_eps(2),une_sd%borne,tv%dt,'eps_y',borne_volume)

    ! Precalcul pour la composante Ez, calcul coefficients dispersifs
    if (une_sd%un_md%nb_pole_eps(3) > 0) &
      call pole_dispersif_structure(une_sd%t_ez,une_sd%un_md%t_epsz, &
                                  une_sd%un_md%nb_pole_eps(3),une_sd%borne,tv%dt,'eps_z',borne_volume)

  end if
  
  if (une_sd%un_md%flag_mu) then
    tsd%flag_mu = une_sd%un_md%flag_mu
    ! alias vers hx, hy, hz
    une_sd%t_hx%eh => tc%hx
    une_sd%t_hy%eh => tc%hy
    une_sd%t_hz%eh => tc%hz
    une_sd%t_hx%ceh2 => tc%chx2
    une_sd%t_hy%ceh2 => tc%chy2
    une_sd%t_hz%ceh2 => tc%chz2

    ! bornes de calcul pour chaque composante de mu
    une_sd%t_hx%borne(1:6) = une_sd%borne(1:6)
    une_sd%t_hy%borne(1:6) = une_sd%borne(1:6)
    une_sd%t_hz%borne(1:6) = une_sd%borne(1:6)
    une_sd%t_hx%borne(YMAX) = une_sd%borne(YMAX)-1
    une_sd%t_hx%borne(ZMAX) = une_sd%borne(ZMAX)-1
    une_sd%t_hy%borne(ZMAX) = une_sd%borne(ZMAX)-1
    une_sd%t_hy%borne(XMAX) = une_sd%borne(XMAX)-1
    une_sd%t_hz%borne(XMAX) = une_sd%borne(XMAX)-1
    une_sd%t_hz%borne(YMAX) = une_sd%borne(YMAX)-1
    
    ! Cas des frontières limites, les bornes de calcul sont celles 
    ! des composantes du champ magnétique
    do i=XMIN,ZMIN,2 ! bornes inférieures
      if (une_sd%borne(i) <= borne_volume(i)) then
        une_sd%t_hx%borne(i) = tc%borne_hx(i)
        une_sd%t_hy%borne(i) = tc%borne_hy(i)
        une_sd%t_hz%borne(i) = tc%borne_hz(i)
      end if
    end do
    do i=XMAX,ZMAX,2
      if (une_sd%borne(i)>=borne_volume(i)) then
        une_sd%t_hx%borne(i) = tc%borne_hx(i)
        une_sd%t_hy%borne(i) = tc%borne_hy(i)
        une_sd%t_hz%borne(i) = tc%borne_hz(i)
      end if
    end do

    ! Precalcul pour la composante Hx, calcul coefficients dispersifs
    if (une_sd%un_md%nb_pole_mu(1) > 0) &
      call pole_dispersif_structure(une_sd%t_hx,une_sd%un_md%t_mux, &
                                  une_sd%un_md%nb_pole_mu(1),une_sd%borne,tv%dt,'mu_x',borne_volume)

    ! Precalcul pour la composante Hy, calcul coefficients dispersifs
    if (une_sd%un_md%nb_pole_mu(2) > 0) &
      call pole_dispersif_structure(une_sd%t_hy,une_sd%un_md%t_muy, &
                                  une_sd%un_md%nb_pole_mu(2),une_sd%borne,tv%dt,'mu_y',borne_volume)

    ! Precalcul pour la composante Hz, calcul coefficients dispersifs
    if (une_sd%un_md%nb_pole_mu(3) > 0) &
      call pole_dispersif_structure(une_sd%t_hz,une_sd%un_md%t_muz, &
                                  une_sd%un_md%nb_pole_mu(3),une_sd%borne,tv%dt,'mu_z',borne_volume)
  end if

end do				

End Subroutine structure_dispersive_precalcul

!===========================================================	
! Subroutine pole_dispersif_structure
! Calculs avant les itérations temporelles des coefficients
! pour les poles dispersifs.
!===========================================================	
Subroutine pole_dispersif_structure(pds,les_pd, nb_pole, borne,dt,type_param,borne_volume)
implicit none
!=== Déclaration des arguments ===
integer, intent(in)  :: nb_pole
type(st_pole_disp_structure), intent(inout) :: pds
type(st_un_pole_dispersif), dimension(1:nb_pole),intent(in) :: les_pd
integer, dimension(6), intent(inout)  :: borne,borne_volume ! dimension de la structure
real, intent(in) :: dt
character (LEN=*), intent(in)  :: type_param

!===  Déclaration des variables  ===
complex  :: gamma,gammadt, G
real     :: betadt,a,b
integer  :: i,j
logical  :: flag_oldcomplexe
!===   Début des traitements   ===

! on compte le nombre de poles réels et complexes
do i = 1,nb_pole
 if (les_pd(i)%flag_reel) then
   pds%nb_pole_reel = pds%nb_pole_reel + 1
   pds%flag_reel = .true.
 else
   pds%nb_pole_complexe = pds%nb_pole_complexe + 1 
   pds%flag_complexe = .true.
 end if
end do

!--- Allocation  et calcul des coefficients pour les poles réelles---
if (pds%flag_reel) then
  allocate(pds%cJr1(1:pds%nb_pole_reel))
  allocate(pds%cJr2(1:4,1:pds%nb_pole_reel))
  allocate(pds%Jr(pds%borne(1):pds%borne(2),pds%borne(3):pds%borne(4), &
                  pds%borne(5):pds%borne(6),1:pds%nb_pole_reel))
  pds%cJr1 = 0.0
  pds%cJr2 = 0.0
  pds%Jr   = 0.0
  
  j = 0
  do i = 1,nb_pole
    if (les_pd(i)%flag_reel) then
      j = j+1
      pds%cJr1(j)   = EXP(-les_pd(i)%alpha * dt)
      pds%cJr2(1,j) = les_pd(i)%alpha* les_pd(i)%A* dt**2 * EXP(-les_pd(i)%alpha*dt/2) 
      !print '(T10,"pds%cJr1(j)=",ES15.5)',pds%cJr1(j)
      !print '(T10,"pds%cJr2(1,j)=",ES15.5)',pds%cJr2(1,j)
      pds%cJr2(2,j) = pds%cJr2(1,j) /2.0 ! coefficient à l'interface
      pds%cJr2(3,j) = pds%cJr2(1,j) /4.0 ! coefficient sur les aretes interfaces
    end if
  end do
end if  

!--- Allocation  et calcul des coefficients pour les poles complexes ---
flag_oldcomplexe=.false. ! On ignore la suite
!if (flag_oldcomplexe) then ! Variables et coefficients complexes
if (pds%flag_complexe) then ! Variables et coefficients complexes
  allocate(pds%cJl1(1:pds%nb_pole_complexe))
  allocate(pds%cJl0(1:pds%nb_pole_complexe))
  allocate(pds%cJl2(1:4,1:pds%nb_pole_complexe))
  allocate(pds%cJle(1:4,1:pds%nb_pole_complexe))
  allocate(pds%Jl(pds%borne(1):pds%borne(2),pds%borne(3):pds%borne(4), &
                  pds%borne(5):pds%borne(6),1:pds%nb_pole_complexe))
  allocate(pds%Ql(pds%borne(1):pds%borne(2),pds%borne(3):pds%borne(4), &
                  pds%borne(5):pds%borne(6),1:pds%nb_pole_complexe))
  pds%cJl0 = 0.0;  pds%cJl1 = 0.0
  pds%cJl2 = 0.0;  pds%Jl   = 0.0
  pds%cJle = 0.0;
  pds%Ql   = 0.0;
  pds%dt_eps0 = dt/EPSILON_0
  j = 0
  do i = 1,nb_pole
    if (.not. les_pd(i)%flag_reel) then
      j = j+1
    pds%cJl0(j) = exp(-2.0*les_pd(i)%alpha*dt)
    pds%cJl1(j) = (1-pds%cJl0(j))/(2*les_pd(i)%alpha)
    pds%cJl2(1,j) = pds%cJl1(j)*EPSILON_0*(les_pd(i)%B*les_pd(i)%beta-les_pd(i)%A*les_pd(i)%alpha)
    
    pds%cJl1(j) = pds%cJl1(j)*dt*(les_pd(i)%alpha**2+les_pd(i)%beta**2)
    pds%cJle(1,j) = EPSILON_0*les_pd(i)%A
    pds%cJl2(2,j) = pds%cJl2(1,j) / 2.0 ! coefficient à l'interface 
    pds%cJl2(3,j) = pds%cJl2(1,j) / 4.0! coefficient sur les aretes interfaces
    pds%cJle(2,j) = pds%cJle(1,j) / 2.0
    pds%cJle(3,j) = pds%cJle(1,j) / 4.0
    betadt = sqrt(les_pd(i)%alpha**2+les_pd(i)%beta**2)*dt
    if (betadt>=1.0) then
        PRINT '(T5,"ERREUR, Milieu de Lorentz instable car dtW0>1 !!!! dtW0=",ES13.5," sur le pole n° ",I3)',betadt,i
        PRINT '(T5,"  Résolution du problème par simplification du modèle, hypothèse statique !!!")'
	stop
      end if
      print '(T5,"n°pole=",I2,"   cJl0,cJl1,cJl2",3ES15.5E2)',i,pds%cJl0(j),pds%cJl1(j),pds%cJl2(1,j)
    end if
  end do
end if

!--- Allocation  et calcul des coefficients pour les poles complexes ---
!if (pds%flag_complexe) then ! Variables et coefficients complexes
if (flag_oldcomplexe) then ! Variables et coefficients complexes
  allocate(pds%cJc1(1:pds%nb_pole_complexe))
  allocate(pds%cJc2(1:4,1:pds%nb_pole_complexe))
  allocate(pds%Jc(pds%borne(1):pds%borne(2),pds%borne(3):pds%borne(4), &
                  pds%borne(5):pds%borne(6),1:pds%nb_pole_complexe))
  pds%cJc1 = cmplx(0.0,0.0)
  pds%cJc2 = cmplx(0.0,0.0)
  pds%Jc   = cmplx(0.0,0.0)
   
  j = 0
  do i = 1,nb_pole
    if (.not. les_pd(i)%flag_reel) then
      j = j+1
      G = CMPLX(les_pd(i)%A,les_pd(i)%B)
      !betadt = modulo(les_pd(i)%beta*dt,2*PI)
      betadt = les_pd(i)%beta*dt
      !gammadt= CMPLX(les_pd(i)%alpha*dt,betadt)
      gamma = CMPLX(les_pd(i)%alpha,les_pd(i)%beta)
      
      !a = cos(betadt)
      !b = sin(betadt)
      !if (a<0) a=cos(betadt+PI)
      !if (b<0) b=sin(betadt+PI)
      !pds%cJc1(j)    = EXP(-les_pd(i)%alpha*dt)*cmplx(a,-b)
      !gammadt= CMPLX(les_pd(i)%alpha*dt/2,betadt/2)
      !a = cos(betadt*0.5)
      !b = sin(betadt*0.5)
      !if (a<0) a = cos(betadt*0.5+PI)
      !if (b<0) b = sin(betadt*0.5+PI)
      !pds%cJc2(1,j) = (gamma * G * dt**2)*EXP(-les_pd(i)%alpha*dt*0.5)*cmplx(a,-b)
      pds%cJc1(j)    = EXP(-gamma*dt)
      pds%cJc2(1,j) = gamma * G * dt**2 *EXP(-gamma*dt/2) 
      pds%cJc2(2,j) = pds%cJc2(1,j) / 2.0 ! coefficient à l'interface 
      pds%cJc2(3,j) = pds%cJc2(1,j) / 4.0! coefficient sur les aretes interfaces
      print '(T5,"n°pole=",I2,"   G=,gamma=",4ES15.5E2)',i,G,gamma
      print '(T5,"n°pole=",I2,"   cJc1",2ES15.5E2)',i,pds%cJc1(j)
      print '(T5,"n°pole=",I2,"   cJc2",2ES15.5E2)',i,pds%cJc2(1,j)
    end if
  end do
end if


!--- traitement des bords ---
! On utilise les bornes de la structure !!!!
! borne agrandi dans le cas ou celle-ci est dans les frontières
allocate(pds%nu(borne(1):borne(2),borne(3):borne(4),borne(5):borne(6)))

pds%nu = 1 ! coefficient du milieu

select case(type_param)
case('eps_x')				
  if (borne(YMIN)>borne_volume(YMIN)) pds%nu(:,borne(YMIN),:) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(YMAX)<borne_volume(YMAX)) pds%nu(:,borne(YMAX),:) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(ZMIN)>borne_volume(ZMIN)) pds%nu(:,:,borne(ZMIN)) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(ZMAX)<borne_volume(ZMAX)) pds%nu(:,:,borne(ZMAX)) = 2  ! coefficient du milieu moyenné à l'interface
  
  if ((borne(YMIN)>borne_volume(YMIN)).and.(borne(ZMIN)>borne_volume(ZMIN))) &
    pds%nu(:,borne(YMIN),borne(ZMIN)) = 3  ! coefficient du milieu moyenné sur les aretes
  
  if ((borne(YMAX)<borne_volume(YMAX)).and.(borne(ZMIN)>borne_volume(ZMIN))) &
    pds%nu(:,borne(YMAX),borne(ZMIN)) = 3  ! coefficient du milieu moyenné sur les aretes
  if ((borne(YMIN)>borne_volume(YMIN)).and.(borne(ZMAX)<borne_volume(ZMAX))) &
    pds%nu(:,borne(YMIN),borne(ZMAX)) = 3  ! coefficient du milieu moyenné sur les aretes
  if ((borne(YMAX)<borne_volume(YMAX)).and.(borne(ZMAX)<borne_volume(ZMAX))) &
    pds%nu(:,borne(YMAX),borne(ZMAX)) = 3  ! coefficient du milieu moyenné sur les aretes
  
  
case('eps_y')				
  if (borne(XMIN)>borne_volume(XMIN)) pds%nu(borne(XMIN),:,:) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(XMAX)<borne_volume(XMAX)) pds%nu(borne(XMAX),:,:) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(ZMIN)>borne_volume(ZMIN)) pds%nu(:,:,borne(ZMIN)) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(ZMAX)<borne_volume(ZMAX)) pds%nu(:,:,borne(ZMAX)) = 2  ! coefficient du milieu moyenné à l'interface
  
  if ((borne(XMIN)>borne_volume(XMIN)).and.(borne(ZMIN)>borne_volume(ZMIN))) &
    pds%nu(borne(XMIN),:,borne(ZMIN)) = 3  ! coefficient du milieu moyenné sur les aretes
  if ((borne(XMAX)<borne_volume(XMAX)).and.(borne(ZMIN)>borne_volume(ZMIN))) &
    pds%nu(borne(XMAX),:,borne(ZMIN)) = 3  ! coefficient du milieu moyenné sur les aretes
  if ((borne(XMIN)>borne_volume(XMIN)).and.(borne(ZMAX)<borne_volume(ZMAX))) &
    pds%nu(borne(XMIN),:,borne(ZMAX)) = 3  ! coefficient du milieu moyenné sur les aretes
  if ((borne(XMAX)<borne_volume(XMAX)).and.(borne(ZMAX)<borne_volume(ZMAX))) &
    pds%nu(borne(XMAX),:,borne(ZMAX)) = 3  ! coefficient du milieu moyenné sur les aretes
case('eps_z')				
  if (borne(XMIN)>borne_volume(XMIN)) pds%nu(borne(XMIN),:,:) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(XMAX)<borne_volume(XMAX)) pds%nu(borne(XMAX),:,:) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(YMIN)>borne_volume(YMIN)) pds%nu(:,borne(YMIN),:) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(YMAX)<borne_volume(YMAX)) pds%nu(:,borne(YMAX),:) = 2  ! coefficient du milieu moyenné à l'interface
  
  if ((borne(XMIN)>borne_volume(XMIN)).and.(borne(YMIN)>borne_volume(YMIN))) &
    pds%nu(borne(XMIN),borne(YMIN),:) = 3  ! coefficient du milieu moyenné sur les aretes
  if ((borne(XMAX)<borne_volume(XMAX)).and.(borne(YMIN)>borne_volume(YMIN))) &
    pds%nu(borne(XMAX),borne(YMIN),:) = 3  ! coefficient du milieu moyenné sur les aretes
  if ((borne(XMIN)>borne_volume(XMIN)).and.(borne(YMAX)<borne_volume(YMAX))) &
    pds%nu(borne(XMIN),borne(YMAX),:) = 3  ! coefficient du milieu moyenné sur les aretes
  if ((borne(XMAX)<borne_volume(XMAX)).and.(borne(YMAX)<borne_volume(YMAX))) &
    pds%nu(borne(XMAX),borne(YMAX),:) = 3  ! coefficient du milieu moyenné sur les aretes
case('mu_x')				
  if (borne(XMIN)>borne_volume(XMIN)) pds%nu(borne(XMIN),:,:) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(XMAX)<borne_volume(XMAX)) pds%nu(borne(XMAX),:,:) = 2  ! coefficient du milieu moyenné à l'interface
case('mu_y')				
  if (borne(YMIN)>borne_volume(YMIN)) pds%nu(:,borne(YMIN),:) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(YMAX)<borne_volume(YMAX)) pds%nu(:,borne(YMAX),:) = 2  ! coefficient du milieu moyenné à l'interface
case('mu_z')
  if (borne(ZMIN)>borne_volume(ZMIN)) pds%nu(:,:,borne(ZMIN)) = 2  ! coefficient du milieu moyenné à l'interface
  if (borne(ZMAX)<borne_volume(ZMAX)) pds%nu(:,:,borne(ZMAX)) = 2  ! coefficient du milieu moyenné à l'interface
end select
				
End Subroutine pole_dispersif_structure

!===========================================================	
! Subroutine structure_dispersive_E_calcul_E
!  Calcul pendant les itérations temporelles des structures
! dispersives électriques
!===========================================================	
Subroutine structure_dispersive_E_calcul_E(tsd)
implicit none
!=== Déclaration des arguments ===
type(st_structure_dispersive), intent(inout) :: tsd

!===  Déclaration des variables  ===
integer :: n
type(st_une_structure_dispersive), pointer :: une_sd

!===   Début des traitements   ===

!--- boucle sur toutes les structures dispersives ---
do n=1, tsd%nb_structure 
  une_sd => tsd%t_une_sd(n)
      call une_structure_disp_calcul_EH(une_sd,une_sd%t_ex)
      call une_structure_disp_calcul_EH(une_sd,une_sd%t_ey)
      call une_structure_disp_calcul_EH(une_sd,une_sd%t_ez)
end do

End Subroutine structure_dispersive_E_calcul_E

!===========================================================	
! Subroutine structure_dispersive_H_calcul_H
!  Calcul pendant les itérations temporelles des structures
! dispersives magnétiques
!===========================================================	
Subroutine structure_dispersive_H_calcul_H(tsd)
implicit none
!=== Déclaration des arguments ===
type(st_structure_dispersive), intent(inout) :: tsd

!===  Déclaration des variables  ===
integer :: n
type(st_une_structure_dispersive), pointer :: une_sd

!===   Début des traitements   ===

!--- boucle sur toutes les structures dispersives ---
do n=1, tsd%nb_structure 
  une_sd => tsd%t_une_sd(n)
      call une_structure_disp_calcul_EH(une_sd,une_sd%t_hx)
      call une_structure_disp_calcul_EH(une_sd,une_sd%t_hy)
      call une_structure_disp_calcul_EH(une_sd,une_sd%t_hz)
end do

End Subroutine structure_dispersive_H_calcul_H



!===========================================================	
! Subroutine structure_dispersive_E_calcul_J
!  Calcul pendant les itérations temporelles des structures
! dispersives électriques
!===========================================================	
Subroutine structure_dispersive_E_calcul_J(tsd)
implicit none
!=== Déclaration des arguments ===
type(st_structure_dispersive), intent(inout) :: tsd

!===  Déclaration des variables  ===
integer :: n
type(st_une_structure_dispersive), pointer :: une_sd

!===   Début des traitements   ===

!--- boucle sur toutes les structures dispersives ---
do n=1, tsd%nb_structure 
  une_sd => tsd%t_une_sd(n)
      call une_structure_disp_calcul_JM(une_sd%t_ex)
      call une_structure_disp_calcul_JM(une_sd%t_ey)
      call une_structure_disp_calcul_JM(une_sd%t_ez)
end do
End Subroutine structure_dispersive_E_calcul_J

!===========================================================	
! Subroutine structure_dispersive_H_calcul_M
!  Calcul pendant les itérations temporelles des structures
! dispersives magnétiques
!===========================================================	
Subroutine structure_dispersive_H_calcul_M(tsd)
implicit none
!=== Déclaration des arguments ===
type(st_structure_dispersive), intent(inout) :: tsd

!===  Déclaration des variables  ===
integer :: n
type(st_une_structure_dispersive), pointer :: une_sd

!===   Début des traitements   ===

!--- boucle sur toutes les structures dispersives ---
do n=1, tsd%nb_structure 
  une_sd => tsd%t_une_sd(n)
      call une_structure_disp_calcul_JM(une_sd%t_hx)
      call une_structure_disp_calcul_JM(une_sd%t_hy)
      call une_structure_disp_calcul_JM(une_sd%t_hz)			
end do


End Subroutine structure_dispersive_H_calcul_M

!===========================================================	
! Subroutine une_structure_disp_calcul_EH
!  Calcul pendant les itérations temporelles
!  Mise à jour de E ou H
!===========================================================	
Subroutine une_structure_disp_calcul_EH(une_sd,pds)
implicit none
!=== Déclaration des arguments ===
type(st_une_structure_dispersive), intent(inout) :: une_sd ! une structure dispersive
type(st_pole_disp_structure), intent(inout) :: pds ! les poles dispersifs

!===  Déclaration des variables  ===
integer :: i,j,k,p

!===   Début des traitements   ===

! Sommation des courants de chaque pole
une_sd%Jd(:,:,:) = 0.0
do p=1,pds%nb_pole_reel
  do k=pds%borne(ZMIN),pds%borne(ZMAX)
  do j=pds%borne(YMIN),pds%borne(YMAX)
  do i=pds%borne(XMIN),pds%borne(XMAX)
    une_sd%Jd(i,j,k) = une_sd%Jd(i,j,k) + pds%Jr(i,j,k,p)
  end do; end do; end do;
end do
do p=1,pds%nb_pole_complexe
  do k=pds%borne(ZMIN),pds%borne(ZMAX)
  do j=pds%borne(YMIN),pds%borne(YMAX)
  do i=pds%borne(XMIN),pds%borne(XMAX)
!    une_sd%Jd(i,j,k) = une_sd%Jd(i,j,k) + REAL(pds%Jc(i,j,k,p))
    une_sd%Jd(i,j,k) = une_sd%Jd(i,j,k) + pds%Jl(i,j,k,p)*pds%dt_eps0
  end do; end do; end do;
end do 

! Mise à jour du champ  à T = nDT en fonction du courant à T =(n-1/2)DT 
do k=pds%borne(ZMIN),pds%borne(ZMAX)
do j=pds%borne(YMIN),pds%borne(YMAX)
do i=pds%borne(XMIN),pds%borne(XMAX)
  pds%eh(i,j,k) = pds%eh(i,j,k) - pds%ceh2(i,j,k) * une_sd%Jd(i,j,k)
!  if (i==pds%borne(XMIN)) print '("ceh2 = ",ES15.6e2," Jd = ",2ES15.6e2)',  pds%ceh2(i,j,k),une_sd%Jd(i,j,k)
!  if (i==pds%borne(XMIN)+1) print '("ceh2 = ",ES15.6e2," Jd = ",2ES15.6e2)',  pds%ceh2(i,j,k),une_sd%Jd(i,j,k)
end do; end do; end do;

  
End Subroutine une_structure_disp_calcul_EH

!===========================================================	
! Subroutine une_structure_disp_calcul_JM
!  Calcul pendant les itérations temporelles
!  Mise à jour des courants (électriques ou magnétiques) 
!    de la méthode JEC
!===========================================================	
Subroutine une_structure_disp_calcul_JM(pds)
implicit none
!=== Déclaration des arguments ===
type(st_pole_disp_structure), intent(inout) :: pds ! les poles dispersifs

!===  Déclaration des variables  ===
integer :: i,j,k,p

!===   Début des traitements   ===

!--- boucle sur tous les poles  ---
do p=1,pds%nb_pole_reel
    ! Calcul du courant T = (n+1/2)DT réel  
    do k=pds%borne(ZMIN),pds%borne(ZMAX)
    do j=pds%borne(YMIN),pds%borne(YMAX)
    do i=pds%borne(XMIN),pds%borne(XMAX)
      pds%Jr(i,j,k,p) = pds%Jr(i,j,k,p) * pds%cJr1(p) - &
	                pds%eh(i,j,k) * pds%cJr2(pds%nu(i,j,k),p)
    end do; end do; end do;

end do
do p=1,pds%nb_pole_complexe
    ! Calcul du courant T = (n+1/2)DT complexe  
    do k=pds%borne(ZMIN),pds%borne(ZMAX)
    do j=pds%borne(YMIN),pds%borne(YMAX)
    do i=pds%borne(XMIN),pds%borne(XMAX)
      pds%Ql(i,j,k,p) = pds%Ql(i,j,k,p) + pds%Jl(i,j,k,p) -pds%eh(i,j,k)*pds%cJle(pds%nu(i,j,k),p)
    end do; end do; end do;
    do k=pds%borne(ZMIN),pds%borne(ZMAX)
    do j=pds%borne(YMIN),pds%borne(YMAX)
    do i=pds%borne(XMIN),pds%borne(XMAX)
      pds%Jl(i,j,k,p) = pds%Jl(i,j,k,p) * pds%cJl0(p) - &
			pds%Ql(i,j,k,p) * pds%cJl1(p) + &
	                pds%eh(i,j,k)   * pds%cJl2(pds%nu(i,j,k),p)
!      pds%Jc(i,j,k,p) = pds%Jc(i,j,k,p) * pds%cJc1(p) - &
!	                pds%eh(i,j,k) * pds%cJc2(pds%nu(i,j,k),p)
    end do; end do; end do;
end do
  
End Subroutine une_structure_disp_calcul_JM

!===========================================================	
! Subroutine structure_dispersive_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine structure_dispersive_affichage(tsd)
implicit none
!=== Déclaration des arguments ===
type(st_structure_dispersive), intent(inout) :: tsd

!===  Déclaration des variables  ===
integer  :: i
type(st_une_structure_dispersive), pointer :: une_sd

!===   Début des traitements   ===
print '("**************************************************")'
print '("Nombre de structures dispersives : ",I4)', tsd%nb_structure 

do i=1, tsd%nb_structure 
  une_sd => tsd%t_une_sd(i)
  print '(T5,"Milieu dispersif de nom : ",A)', TRIM(une_sd%nom_milieu)
  print '(T10,"Les bornes (imin,imax,jmin,jmax,kmin,kmax) = (",6I5,")")',une_sd%borne(1:6)
  if (une_sd%un_md%flag_eps) then
    print '(T5," Composante epsilonX ")'
    call pole_disp_structure_affichage(une_sd%t_ex)
    print '(T5," Composante epsilonY ")'
    call pole_disp_structure_affichage(une_sd%t_ey)
    print '(T5," Composante epsilonZ ")'
    call pole_disp_structure_affichage(une_sd%t_ez)
  else
    print '(T5," Composante muX ")'
    call pole_disp_structure_affichage(une_sd%t_hx)    
    print '(T5," Composante muY ")'
    call pole_disp_structure_affichage(une_sd%t_hy)
    print '(T5," Composante muZ ")'
    call pole_disp_structure_affichage(une_sd%t_hz)
  
  end if
end do
print '("**************************************************")'
				
End Subroutine structure_dispersive_affichage

!===========================================================	
! Subroutine structure_dispersive_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
Subroutine pole_disp_structure_affichage(pds)
implicit none
!=== Déclaration des arguments ===
type(st_pole_disp_structure), intent(inout) :: pds

!===  Déclaration des variables  ===

!===   Début des traitements   ===

print '(T10,"Nombre de poles (réelles,complexes) = (",I2,",",I2,")")',pds%nb_pole_reel,pds%nb_pole_complexe
!print '(T10,"Les bornes (imin,imax,jmin,jmax,kmin,kmax) = (",6I5,")")',pds%borne(1:6)
				
End Subroutine pole_disp_structure_affichage

End Module mfdtd_structure_dispersive	

