!////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_structure_fil.f90
!// Purpose:     Traitement du formalisme des fils minces de Holland  ==
!//  et de Giraudon pour les fils le long des axes
!// Author:      Christophe Guiffaut
!// Modified by:
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Février 2004
!// Copyright:   (c) Christophe Guiffaut, équipe CNRS IRCOM/DEM
!//              Utilisation des sources réservées à l'IRCOM
!// Licence  :
!////////////////////////////////////////////////////////////////////////

! == Type st_structure_fil                                ==
! == Subroutine structure_fil_initdefaut              ==
! == Subroutine structure_fil_lecture                 ==
! == Subroutine structure_fil_init          ==
! == Subroutine structure_fil_precalcul               ==
! == Subroutine structure_fil_calcul                  ==
! == Subroutine structure_fil_affichage               ==
! == Subroutine structure_fil_postcalcul              ==
! ==========================================================

module mfdtd_structure_fil

use m_constantes

use m_constantes_scientifiques, only : PI, EPSILON_0, MU_0, CVIDE

use m_entree , only : st_fichier, &
                                  lineread, &
				  ligne_precedente, &
				  multibackspace, &
				  numbackspace_set, &
				  nb_mot_chaine, &
				  ctrl_nb_mot_chaine, &
				  lecture_stop, &
				  len_int
use m_math_operation, only : dp

use m_generateur, only : st_generateur, &
                              generateur_calcul_t,&
			      generateur_existe_actif
			      
use m_commun_identifiant, only :BC_ZPADE, BC_QZPADE,&
			        BC_QZPC
				
use m_maillage

use mfdtd_volume, only : st_volume
use mfdtd_champ, only : st_champ
use mfdtd_cpml,   only : st_cpml

use mfdtd_milieu_volume, only : st_milieu_volume, &
				 milieu_volume_extrait_sige, &
                                 milieu_volume_extrait_epsilon

use mfdtd_structure_fil_tube, only : maillage_disque, &
                                      cellule_du_disque, &
				      permittivite_cell, &
				      distribution_disque

use mfdtd_filhol_commun

implicit none


! ==========================================================
! == Definitions des differents types                     ==
! == Type st_generateur_localise                          ==
! == Type st_fil                                          ==
! == Type st_structure_fil                                ==
! ==========================================================


!------------- STRUCTURE POUR LES JONCTIONS MULTI-filS ------------------
type st_fil_noeud_1seg
  real, pointer    :: courant, charge
  real  :: dt_signe,cq,cq_q
end type st_fil_noeud_1seg

type st_fil_noeud_jonction
  integer :: nbseg;
  type(st_fil_noeud_1seg), dimension(:),pointer :: seg
end type st_fil_noeud_jonction



!------------- STRUCTURE POUR LES JONCTIONS FIL-COAX  ----
type st_jonction_fil_coax
  character(LEN=LEN_MOT_CLE)  :: nom
  character(LEN=LEN_MOT_CLE), dimension(2)  :: nomfil,nomextr
  real               :: signe
  real               :: cq3,cq1(3),cq2(3)
  real, pointer :: Q1,Q2,Q3,I1,I2,I3 
  integer                  :: extrfil,extrcoax ! position jonction
  type(st_unfil),pointer   :: fil,coax
end type st_jonction_fil_coax



! ------------------- Liste d'objets de type fil ------------------------
type st_fil_pointeur
  type(st_unfil),pointer :: fil
end type st_fil_pointeur

type st_fil_objet
  integer  :: nbfil
  character(LEN=LEN_MOT_CLE)  ::nom
  character(LEN=LEN_MOT_CLE),dimension(:),pointer  :: nom_fil
  type(st_fil_pointeur), dimension(:),pointer     :: p
end type st_fil_objet

! ------------------- STRUCTURE PRINCIPALE ------------------------
type st_structure_fil
  logical  :: flag_disque, flag_data, flag_JNF_auto
  real  :: dt
  real  :: s_dd
  real, dimension(:,:), pointer  :: sdisque
  
  ! plan de masse
  integer :: plan_masse,pos_plan_masse
  logical :: flag_plan_masse
  
  integer :: n_fil,n_objet
  type(st_unfil), dimension(:), pointer :: l_fil
  type(st_fil_objet), dimension(:), pointer :: l_objet

  !----- tableau et structure pour les noeuds de jonctions multi-fils --------
  integer :: n_noeud
  integer, dimension(:,:), pointer :: l_coord_noeud
  type(st_fil_noeud_jonction), dimension(:), pointer :: l_noeud
  
  !----- Complément pour les Fils isolés ------
  integer :: n_filiso
  
  !--- Circuit de Pade ---
  integer :: n_zp
  
  !----- Jonction Fil-Coaxial ------
  integer :: n_jfc
  type(st_jonction_fil_coax), dimension(:), pointer :: l_jfc 
  
  !----- Jonction Quadripole -------
  integer :: n_jfq

  !----- Jonction Fil isolé ------
  integer :: n_jfi
end type st_structure_fil

contains



! ==========================================================
! == Subroutine structure_fil_lecture                     ==
! == Lecture dans le fichier d'entrée...                  ==
! ==========================================================
subroutine structure_fil_lecture(f,st,v,g,tm)

type(st_fichier), intent(inout) :: f
type(st_structure_fil), intent(inout) :: st
type(st_volume), intent(in) :: v
type(st_generateur), intent(inout) :: g
type(st_maillage), intent(in) :: tm

character(LEN=LEN_MOT_CLE) :: corbeille,nom, lieu,lieu1
character(LEN=LEN_MOT_CLE) ::  gene, nomfil, premiermot
logical :: trouve, flag_disque, flag_moyenne_rect, flag_epsr
logical :: flag_interface
real  ::  h_interface
real, dimension(2) :: epsr_interface
integer :: i, j, j1, j2, seg,nbfil
integer :: i_noeud,ifil,ifiliso,iobjet,i_jf,i_jfc,i_jfq,i_jfi,i_zp
integer :: p(2), p1, p2
integer :: dir
integer :: defautextremite, typeextremite
integer :: coord(1:6),coord3(1:3)
real    :: epsilonr, r_disque
real, dimension(3) :: delta
real :: valeur, e0, r0, phase, retard, rayon, R_lineique
integer  :: status,nbmot
integer, dimension(1:2) :: l

type(st_fil_objet), pointer      :: tobj
type(st_unfil), pointer          :: fil
type(st_jonction_fil_coax),pointer :: pjf
!--- DEBUT DES TRAITEMENTS ---
NULLIFY(st%l_noeud)
NULLIFY(st%l_coord_noeud)
call numbackspace_set(f,0)

!---------- on compte le nombre de fils et de jonctions ----------------------
st%n_fil = 0 ; st%n_objet = 0
st%n_noeud = 0;
st%flag_disque = .false.
st%flag_data = .true.
st%flag_JNF_auto = .false.
st%n_jfc = 0
st%n_jfq = 0

st%plan_masse = PLAN_XOY
st%pos_plan_masse = 1
st%flag_plan_masse =.false.

!--- Les fils isolés ---
st%n_filiso =0;
st%n_jfi =0;

st%n_zp = 0

i=1;

do
  premiermot = lineread(f,1)
  if (premiermot==f%nomfinbloc) then
    exit
  else if (premiermot(1:1)=='F'.or.premiermot(1:4)=='COAX') then
    st%n_fil = st%n_fil  + 1
  else if (premiermot(1:9)=='OBJET_FIL') then
    st%n_objet = st%n_objet  + 1
  ELSE IF (premiermot(1:3) == 'JNF' .or. premiermot(1:3) == 'JFF') then
    st%n_noeud = st%n_noeud + 1
  ELSE IF (premiermot(1:3) == 'JFC') then
    st%n_jfc = st%n_jfc + 1
  end if
enddo

!on remonte au début du bloc
i = multibackspace(f)
if (st%n_fil == 0) then
  print '(/,t10,"ATTENTION, Aucun fil trouvé dans le bloc correspondant ")'
  !call lecture_stop(f)
  st%flag_data = .false.
  return
end if

!--------- Allocation mémoire des structures et initialisation --------
allocate(st%l_fil(st%n_fil))
do i=1,st%n_fil
  call structure_unfil_initdefaut(st%l_fil(i))
end do


if (st%n_objet>0) allocate(st%l_objet(st%n_objet))

if (st%n_noeud>0) then
  allocate(st%l_coord_noeud(3,st%n_noeud))
  st%l_coord_noeud(:,:) = 0;
end if

if (st%n_jfc>0) then
  allocate(st%l_jfc(st%n_jfc))
end if


! Initialisation des jonctions fil-pml

ifil  = 0 ; iobjet = 0
ifiliso=0;
i_noeud = 0 ;i_jf = 0
i_zp = 0;
i_jfc =0; i_jfq =0
i_jfi=0;

epsilonr = 1.0 ! valeur par défaut de la permittivité
flag_disque = .false.
flag_epsr = .false.
r_disque = 0.0
! Par défaut, extrémite en circuit ouvert
defautextremite = CO
flag_moyenne_rect = .false. ! uniquement pour le formalisme amélioré
phase = 0.0; retard=0.0;
epsr_interface = 0.0
h_interface    = 0.0
flag_interface = .false.
R_lineique = 0.0
!-------------------------- LECTURE DES DONNÉES -------------------------
do
  premiermot = lineread(f)
  if (premiermot(1:1) == 'F' .or. premiermot(1:4) == 'COAX' ) then ! fil
      ifil = ifil + 1
      fil => st%l_fil(ifil)
      delta = 0.0
      !----------------------------------------------------------------------
      if (premiermot(1:1) == 'F') then
	nbmot = ctrl_nb_mot_chaine(f,f%chaine,8,11)
	if (nbmot == 8) then
	  read(f%chaine,*,ERR=999) nom, coord(1:6),rayon
	else if (nbmot == 11) then
	  read(f%chaine,*,ERR=999) nom, coord(1:6),rayon,delta(1:3) 
	else
	  call lecture_stop(f)      
	end if
      !----------------------------------------------------------------------
      else if (premiermot(1:4) == 'COAX') then
        allocate(fil%t_coax)
	nbmot = ctrl_nb_mot_chaine(f,f%chaine,11,14)
	if (nbmot == 11) then
	  read(f%chaine,*,ERR=999) nom, coord(1:6),fil%t_coax%r0,rayon,&
	                           fil%t_coax%epsr,fil%t_coax%sige
	else if (nbmot == 14) then
	  read(f%chaine,*,ERR=999) nom, coord(1:6),fil%t_coax%r0,rayon,&
	                           fil%t_coax%epsr,fil%t_coax%sige,delta(1:3)
	  fil%flag_coaxial = .true.
	  fil%flag_gv      = .false.
	else
	  call lecture_stop(f)      
	end if     
      end if
      !----------------------------------------------------------------------
      ! la direction est récupérée
      if (.not.verif_dir_fil(nom,coord,dir)) call lecture_stop(f)
      if (.not.verif_rayon_fil(nom,v%celx,v%cely,v%celz,rayon,dir)) call lecture_stop(f)
      if (.not.verif_decalage_fil(nom,v%celx,v%cely,v%celz,delta,dir)) call lecture_stop(f)
      
      coord= origine_cellule6(tm,coord)
      print '(t10,A," : Coords ds le repère de calcul = ",6I6)',trim(nom),coord

      ! on rempli la structure fil
      st%l_fil(ifil)%nom = nom
      st%l_fil(ifil)%coord(1:6) = coord(1:6)
      st%l_fil(ifil)%ldeb = coord(dir*2-1)
      st%l_fil(ifil)%lfin = coord(dir*2)
      st%l_fil(ifil)%nseg = coord(dir*2) - coord(dir*2-1)
      st%l_fil(ifil)%dir = dir
      st%l_fil(ifil)%epsilonr = epsilonr
      st%l_fil(ifil)%flag_epsr = flag_epsr
      st%l_fil(ifil)%jonction_inf = defautextremite
      st%l_fil(ifil)%jonction_sup = defautextremite
      st%l_fil(ifil)%flag_disque = flag_disque
      st%l_fil(ifil)%r_disque = r_disque
      st%l_fil(ifil)%moyenne_e_rectangle =  flag_moyenne_rect
      st%l_fil(ifil)%delta3 = delta
      st%l_fil(ifil)%decal_cell(1) = delta(PC1(dir))
      st%l_fil(ifil)%decal_cell(2) = delta(PC2(dir))
      st%l_fil(ifil)%rayon = rayon
      st%l_fil(ifil)%R_lineique = R_lineique
      
      ! Traitement particulier pour une interface
      st%l_fil(ifil)%h_interface = h_interface
      st%l_fil(ifil)%epsr_interface = epsr_interface
      st%l_fil(ifil)%flag_interface = flag_interface 

      ! allocation
      allocate(st%l_fil(ifil)%ro(0:st%l_fil(ifil)%nseg+1))
      allocate(st%l_fil(ifil)%lo(0:st%l_fil(ifil)%nseg+1))
      allocate(st%l_fil(ifil)%co(0:st%l_fil(ifil)%nseg+1))
!      allocate(st%l_fil(ifil)%zpo(0:st%l_fil(ifil)%nseg+1))
      st%l_fil(ifil)%ro(:) = 0.0
      st%l_fil(ifil)%lo(:) = 0.0
      st%l_fil(ifil)%co(:) = 0.0
!      st%l_fil(ifil)%zpo(:) = 0.0
      cycle
    !--------------------- CAS DES COMPOSANTS PASSIFS ---------------------
  else if (premiermot(1:1) == 'R' .or. premiermot(1:1) == 'C' .or. &
             premiermot(1:1) =='L') then
      status = ctrl_nb_mot_chaine(f,f%chaine,4)
      read(f%chaine,*,ERR=999) nom, nomfil, seg, valeur
      j = verif_nom_fil(st,nomfil)
      if (j==0) call lecture_stop(f)
      if (.not.verif_segment_fil(st%l_fil(j)%nom,st%l_fil(j)%nseg,seg)) call lecture_stop(f)
      if (premiermot(1:1) == 'R') then
        st%l_fil(j)%ro(seg) =  st%l_fil(j)%ro(seg) + valeur
      else if (premiermot(1:1) == 'C') then
        st%l_fil(j)%co(seg) = valeur
      else if (premiermot(1:1) == 'L') then
        st%l_fil(j)%lo(seg) = valeur
      end if
      cycle
  end if

    !------------------- LES GÉNÉRATEURS DE COURANT ET DE TENSION ---------------------
    select case(premiermot(1:2))
      case('GI')
        print '("ERREUR, Générateur de courant non traité acctuellement")'
        call lecture_stop(f)
        cycle
	
      case('GV')
        nbmot = ctrl_nb_mot_chaine(f,f%chaine,6)
        read(f%chaine,*,ERR=999) nom, nomfil, seg, gene, e0, r0
        j = verif_nom_fil(st,nomfil)
        if (st%l_fil(j)%flag_gv) then
	  print '("ERREUR, un fil ne peut pas avoir deux générateurs !!! ")'
	  call lecture_stop(f)
	end if
        if (j==0) call lecture_stop(f)
        if (.not.verif_segment_fil(st%l_fil(j)%nom,st%l_fil(j)%nseg,seg)) call lecture_stop(f)
        allocate(st%l_fil(j)%gv)
        st%l_fil(j)%flag_gv =.true.
        st%l_fil(j)%ro(seg) =  st%l_fil(j)%ro(seg) + r0
        st%l_fil(j)%gv%nom    = nom
        st%l_fil(j)%gv%seg    = seg
        st%l_fil(j)%gv%nomgene= gene
        st%l_fil(j)%gv%e0     = e0
        st%l_fil(j)%gv%r0     = r0
        st%l_fil(j)%gv%phase  = phase
        st%l_fil(j)%gv%retard = retard
        
	if (.not.generateur_existe_actif(g,gene)) then
	    call lecture_stop(f)
	end if
	
        cycle
    end select
    
    !-------------------------- LES JONCTIONS -----------------------------------
    select case(premiermot(1:3))
      case('JNF')
        i_noeud = i_noeud+1
        read(f%chaine,*,ERR=999) nom, coord3(1:3)
        st%l_coord_noeud(1:3,i_noeud)= origine_cellule3(tm,coord3)
	print '(t10,"JNF, coord =",3I6)',st%l_coord_noeud(1:3,i_noeud)
        cycle

      case('JFP','JFV') ! jonction fil-plaque et fil-vide
        if (premiermot(1:3)=='JFP') then
          typeextremite = CC
        else
          typeextremite = CO
        end if
        ! Extration des données de la chaine en fonction du nombre de mots
        nbmot = ctrl_nb_mot_chaine(f,f%chaine,3,4)
        if (nbmot == 3) then
          read(f%chaine,*,ERR=999) nom, lieu, nomfil
          lieu1 = lieu
        else if (nbmot == 4) then
          read(f%chaine,*,ERR=999) nom, lieu,lieu1, nomfil
        end if

        j = verif_nom_fil(st,nomfil) !n° du fil
        if (j==0) call lecture_stop(f)
        p(1) = fil_position_jonction(lieu)  !extremite debut ou fin
        p(2) = fil_position_jonction(lieu1) !extremite debut ou fin
        if (ANY(p<0)) call lecture_stop(f)
        if (ANY(p==EXTR_DEBUT)) st%l_fil(j)%jonction_inf = typeextremite
        if (ANY(p==EXTR_FIN))   st%l_fil(j)%jonction_sup = typeextremite
        cycle
      !------------------------------------------------------------------------
      case('JFF') ! jonction fil - fil deux à deux
	nbmot = ctrl_nb_mot_chaine(f,f%chaine,2)
	read(f%chaine,*,ERR=999) nom,nbfil
	do i=1,nbfil
	  premiermot = lineread(f) 
  	  nbmot = ctrl_nb_mot_chaine(f,f%chaine,2)
	  read(f%chaine,*,ERR=999)  lieu, nomfil
	  j = verif_nom_fil(st,nomfil); if (j==0) call lecture_stop(f);
          p(1) = fil_position_jonction(lieu)
	  if (p(1)<0) call lecture_stop(f)
          if (p(1)==EXTR_DEBUT) then
	    st%l_fil(j)%jonction_inf = JFF
          else
  	    st%l_fil(j)%jonction_sup = JFF
	  end if
	  
	  if (i==1) then ! ajout d'une jonction 
            i_noeud = i_noeud+1
	    if (p(1)==EXTR_DEBUT) then
	      st%l_coord_noeud(1:3,i_noeud) = st%l_fil(j)%coord(1:5:2)
	    else
	      st%l_coord_noeud(1:3,i_noeud) = st%l_fil(j)%coord(2:6:2)
	    end if
	  end if
	 ! print '("N° noeud=",I3,"/",I3)',i_noeud,st%n_noeud

	enddo
	cycle

      !------------------------------------------------------------------------
      case('JFC') ! jonction fil - coaxial
        i_jfc = i_jfc+1
	pjf => st%l_jfc(i_jfc)
	nbmot = ctrl_nb_mot_chaine(f,f%chaine,5)
	read(f%chaine,*,ERR=999) pjf%nom,pjf%nomextr(1),pjf%nomfil(1),pjf%nomextr(2),pjf%nomfil(2)
	if (.not.((pjf%nomfil(1)(1:1) == 'F' .and. pjf%nomfil(2)(1:3) == 'COAX') .or. &
	    (pjf%nomfil(1)(1:3) == 'COAX' .and. pjf%nomfil(2)(1:1) == 'F'))) then
	  print '("ERREUR, JONCTION mal défini")'
	  call lecture_stop(f);
	end if
	
        do i=1,2
	  j = verif_nom_fil(st,pjf%nomfil(i)); if (j==0) call lecture_stop(f);
          p(i) = fil_position_jonction(pjf%nomextr(i))
	  if (p(i)<0) call lecture_stop(f)
          if (p(i)==EXTR_DEBUT) then
	    st%l_fil(j)%jonction_inf = JFC
          else
  	    st%l_fil(j)%jonction_sup = JFC
	  end if
	  if (pjf%nomfil(i)(1:1) == 'F') then
	    pjf%fil => st%l_fil(j)
	    pjf%extrfil = p(i)
	  else
	    pjf%coax => st%l_fil(j)	  
	    pjf%extrcoax = p(i)
	  end if
	end do
	cycle
      !------------------------------------------------------------------------
	
    end select

    if (premiermot(1:4) == 'JPML') then
        ! Extration des données de la chaine en fonction du nombre de mots
        nbmot = ctrl_nb_mot_chaine(f,f%chaine,3,4)
        if (nbmot == 3) then
          read(f%chaine,*,ERR=999) nom, lieu, nomfil
          lieu1 = lieu
        else if (nbmot == 4) then
          read(f%chaine,*,ERR=999) nom, lieu,lieu1, nomfil
        end if

        j = verif_nom_fil(st,nomfil) !n° du fil
        if (j==0) call lecture_stop(f)
        p(1) = fil_position_jonction(lieu)  !extremite debut ou fin
        p(2) = fil_position_jonction(lieu1) !extremite debut ou fin
        if (ANY(p<0)) call lecture_stop(f)
        if (ANY(p==EXTR_DEBUT)) st%l_fil(j)%jonction_inf = JPML
        if (ANY(p==EXTR_FIN))   st%l_fil(j)%jonction_sup = JPML
        cycle
    !--------------------------------------------------------------------
    else if (premiermot(1:9)=='OBJET_FIL') then
      i = nb_mot_chaine(f%chaine)
      iobjet = iobjet + 1
      tobj => st%l_objet(iobjet)
      tobj%nom = premiermot
      tobj%nbfil = i-1
      allocate(tobj%nom_fil(1:i-1))
      allocate(tobj%p(1:i-1))
      read(f%chaine,*,ERR=999) nom,tobj%nom_fil(1:i-1)
      do i=1,tobj%nbfil
        j = verif_nom_fil(st,tobj%nom_fil(i)) !n° du fil
        if (j==0) call lecture_stop(f)
        tobj%p(i)%fil => st%l_fil(j)
      end do
      cycle
    end if
    !--------------------------------------------------------------------
    select case(premiermot)

      case('epsr')
        status = ctrl_nb_mot_chaine(f,f%chaine,2)
        read(f%chaine,*,ERR=999) nom, epsilonr
        flag_epsr = .true.
      case('jonction_fil')
        status = ctrl_nb_mot_chaine(f,f%chaine,2)
        read(f%chaine,*,ERR=999) corbeille, nom
        if (nom == 'cc') then
	  defautextremite = CC
        else if (nom == 'co') then
          defautextremite = CO
        else
          print *,'!!! Mot clé n°2 non reconnu !!!'
          call lecture_stop (f)
        end if
      case('resistance_lineique')
        status = ctrl_nb_mot_chaine(f,f%chaine,2)
        read(f%chaine,*,ERR=999) nom, R_lineique
      case('phase')
        status = ctrl_nb_mot_chaine(f,f%chaine,2)
        read(f%chaine,*,ERR=999) corbeille, phase
        cycle
      case('retard')
        status = ctrl_nb_mot_chaine(f,f%chaine,2)
        read(f%chaine,*,ERR=999) corbeille, retard
        cycle
      case ('plan_masse')
        status = ctrl_nb_mot_chaine(f,f%chaine,3)
        read(f%chaine,*,ERR=999) corbeille,nom,st%pos_plan_masse
	if (nom=='xoy') then
	  st%plan_masse = PLAN_XOY
	else if (nom=='yoz') then
	  st%plan_masse = PLAN_YOZ
	else if (nom=='xoz'.or.nom=='zox') then
	  st%plan_masse = PLAN_XOZ
	else
	  call lecture_stop(f)
	end if
	st%flag_plan_masse =.true.
        cycle
      
      case('disque')
        status = ctrl_nb_mot_chaine(f,f%chaine,2)
        read(f%chaine,*,ERR=999) corbeille, r_disque
        flag_disque = .true.
        st%flag_disque = .true.
        cycle
      case('moyenne_champ_sur_rectangle')
        flag_moyenne_rect = .true.
      case('interface')
        status = ctrl_nb_mot_chaine(f,f%chaine,4)
        read(f%chaine,*,ERR=999) corbeille, h_interface, epsr_interface(1:2)
        flag_interface = .true.
      case('jnf_non_auto')
        st%flag_JNF_auto = .false.
      case('jnf_auto')
        st%flag_JNF_auto = .true.
      case default
        if (f%chaine==f%nomfinbloc) exit
        print *,'!!! Mot clé non reconnu !!!'
        call lecture_stop (f)

    end select
enddo

if (st%flag_JNF_auto) call fil_detection_jonction_noeud(st)
return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(f)

end subroutine structure_fil_lecture

! ==========================================================
! == Subroutine structure_fil_ecriture                    ==
! == Ecriture dans le fichier d'entrée...                  ==
! ==========================================================
subroutine structure_fil_ecriture(f,st,tm)

type(st_fichier), intent(inout) :: f
type(st_structure_fil), intent(inout) :: st
type(st_maillage), intent(in) :: tm

character(len=LEN_CHAINE) :: chaine
integer :: i,j,coord3(3),coord(6)
real    :: epsilonr,r_disque,R_lineique
type(st_fil_objet), pointer :: lobjet
type(st_unfil), pointer :: fil

!---------------- ECRITURE DONNÉES DE CHAQUE FIL -----------------------
epsilonr = 0.0
r_disque  = 0.0
R_lineique = 0.0
do i=1,st%n_fil
  fil => st%l_fil(i)
  
  ! LES OPTIONS
  if (fil%flag_epsr) then
    if (fil%epsilonr /= epsilonr) then
      write(f%unite,'(T3,A,3x,G10.4)') 'epsr',fil%epsilonr
      epsilonr=fil%epsilonr
    end if
  end if
  
  if (fil%R_lineique /= R_lineique) then
      write(f%unite,'(T3,A,3x,G10.4)') 'resistance_lineique',fil%R_lineique
      R_lineique=fil%R_lineique
  end if
  
  if (fil%moyenne_e_rectangle) write(f%unite,'(T3,A)') 'moyenne_champ_sur_rectangle'
  
  if (fil%flag_disque) then
    if (fil%r_disque /= r_disque) then
      write(f%unite,'(T3,A,3x,G14.6)') 'disque',fil%r_disque
      r_disque=fil%r_disque
    end if
  end if
  
  ! DESCRIPTION FIL 
  coord= origine_cellule6(tm,fil%coord,.true.)
  if (all(fil%delta3 == 0.0)) then
    write(f%unite,'(T3,A,3x,6I4,G14.6)')  trim(fil%nom),coord(1:6),fil%rayon
  else
    write(f%unite,'(T3,A,3x,6I4,4G14.6)')  trim(fil%nom),coord(1:6),fil%rayon,fil%delta3
  end if
  
  !--- Les générateurs de tension ---  
  if (fil%flag_gv) then
    if (fil%gv%retard /= 0.0) write(f%unite,'(T5,A,3x,G14.6)')  'retard',fil%gv%retard
    if (fil%gv%phase  /= 0.0) write(f%unite,'(T5,A,3x,G14.6)')  'phase',fil%gv%phase
    write(f%unite,'(T5,A,3x,A,3x,I4,3x,A,2G14.6)')  trim(fil%gv%nom),trim(fil%nom), &
          fil%gv%seg,trim(fil%gv%nomgene),fil%gv%e0, fil%gv%r0
    fil%ro(fil%gv%seg) = fil%ro(fil%gv%seg) - fil%gv%r0	  
  end if

  
  ! Les circuits R
  do j=1,fil%nseg
    if (fil%ro(j)>1.0e-6) write(f%unite,'(T5,A,3x,A,I4,G14.6)')  'R',trim(fil%nom),j,fil%ro(j)
  end do
  if (fil%flag_gv)  fil%ro(fil%gv%seg) = fil%ro(fil%gv%seg) + fil%gv%r0	  
  
  ! Les circuits L
  do j=1,fil%nseg
    if (fil%lo(j)/=0.0) write(f%unite,'(T5,A,3x,A,I4,G14.6)')  'L',trim(fil%nom),j,fil%lo(j)
  end do
  ! Les circuits C
  do j=1,fil%nseg
    if (fil%co(j)/=0.0) write(f%unite,'(T5,A,3x,A,I4,G14.6)')  'C',trim(fil%nom),j,fil%co(j)
  end do
  
  write(f%unite,*)
end do


!---------------- ECRITURE DES JONCTIONS ------------------------------
do i=1,st%n_fil
  fil => st%l_fil(i)
  select case(fil%jonction_inf)
    case(JPML)
      write(f%unite,'(T3,A,3x,A,3x,A)')  'JPML','deb',trim(fil%nom)
    case(CO)
      write(f%unite,'(T3,A,3x,A,3x,A)')  'JFV','deb',trim(fil%nom)
    case(CC)
      write(f%unite,'(T3,A,3x,A,3x,A)')  'JFP','deb',trim(fil%nom)
  end select
  select case(fil%jonction_sup)
    case(JPML)
      write(f%unite,'(T3,A,3x,A,3x,A)')  'JPML','fin',trim(fil%nom)
    case(CO)
      write(f%unite,'(T3,A,3x,A,3x,A)')  'JFV','fin',trim(fil%nom)
    case(CC)
      write(f%unite,'(T3,A,3x,A,3x,A)')  'JFP','fin',trim(fil%nom)
  end select
END DO
!---------------- ECRITURE DES JONCTIONS NOEUDS -----------------------
if (.not.st%flag_JNF_auto) write(f%unite,'(T3,A)') 'jnf_non_auto'
DO i=1,st%n_noeud
  coord3= origine_cellule3(tm,st%l_coord_noeud(1:3,i),.true.)
  write(f%unite,'(T3,A,3x,3I5)') 'JNF',coord3(1:3)
end do

write(f%unite,*)


!---------------- ECRITURE DES OBJETS FILS -----------------------
DO i=1,st%n_objet
  lobjet => st%l_objet(i)
  chaine = ''
  do j=1,lobjet%nbfil
    chaine = trim(chaine)//' '//trim(lobjet%nom_fil(j))
  end do
  write(f%unite,'(T3,A,3x,A)') trim(lobjet%nom),trim(chaine)
end do
end subroutine structure_fil_ecriture

! ==========================================================
! == Function fil_detection_jonction_noeud                ==
! == st : liste de fil                                    ==
! == Détection automatique des jonctions entre fils
! ==========================================================
subroutine fil_detection_jonction_noeud(st)
type(st_structure_fil), intent(inout) :: st

integer :: i,j,n,nt,n1
integer :: dir2,dir3
integer, allocatable, dimension(:,:) :: c_noeud
integer,dimension(3,3) :: redir=RESHAPE((/0,3,2,  3,0,1,  2,1,0/), (/3,3/))
type(st_unfil), pointer :: fil1,fil2

!------------ On compte le nombre de noeud -----------------
n=0
do i=1,st%n_fil
  fil1 => st%l_fil(i)
  do j=i+1,st%n_fil
    fil2 => st%l_fil(j)
    if (fil2%dir == fil1%dir) then ! fil parallèle
      dir2 = PC1(fil1%dir)
      dir3 = PC2(fil1%dir)
      if (fil1%coord(dir2*2) /= fil2%coord(dir2*2)) cycle
      if (fil1%coord(dir3*2) /= fil2%coord(dir3*2)) cycle
      if ((fil1%coord(fil1%dir*2)   /= fil2%coord(fil1%dir*2-1)) .and. & 
          (fil1%coord(fil1%dir*2-1) /= fil2%coord(fil1%dir*2  ))) cycle
      n=n+1
      cycle
    end if
    dir3 = redir(fil1%dir,fil2%dir) !donne la direction perpendiculaire aux 2 fils
    if (fil1%coord(dir3*2)/=fil2%coord(dir3*2)) cycle ! plan différent
    
    if (fil1%coord(fil1%dir*2-1)>fil2%coord(fil1%dir*2).or. &
        fil1%coord(fil1%dir*2)  <fil2%coord(fil1%dir*2)) cycle
    if (fil2%coord(fil2%dir*2-1)>fil1%coord(fil2%dir*2).or. &
        fil2%coord(fil2%dir*2)  <fil1%coord(fil2%dir*2)) cycle
    
    ! sinon jonction
    n=n+1
  end do
end do
print '(T5," Nombre de jonctions noeud détecté= ",I7)',n

!------------ recherche des coordonnées de chaque noeud -----------------
allocate(c_noeud(1:4,1:n))
c_noeud = 0
nt=n
n=0
do i=1,st%n_fil
  fil1 => st%l_fil(i)
  do j=i+1,st%n_fil
    fil2 => st%l_fil(j)
    if (fil2%dir == fil1%dir) then ! fil parallèle
      dir2 = PC1(fil1%dir)
      dir3 = PC2(fil1%dir)
      if (fil1%coord(dir2*2) /= fil2%coord(dir2*2)) cycle
      if (fil1%coord(dir3*2) /= fil2%coord(dir3*2)) cycle
      ! la jonction est réalisée si les fils se touchent à leurs extrémités
      ! et si la jonction est par défaut JFP
      if ((fil1%coord(fil1%dir*2)   == fil2%coord(fil1%dir*2-1)) .or. & 
          (fil1%coord(fil1%dir*2-1) == fil2%coord(fil1%dir*2  ))) then
         ! jonction
        n=n+1
        c_noeud(dir2,n) = fil1%coord(dir2*2)
        c_noeud(dir3,n) = fil1%coord(dir3*2)
        if (fil1%coord(fil1%dir*2) == fil2%coord(fil1%dir*2-1)) then
          c_noeud(fil1%dir,n) = fil1%coord(fil1%dir*2)
	  if ((fil2%jonction_inf == CO .or.fil2%jonction_inf == JNF .or. fil2%jonction_inf==JFF).and.&
	      (fil1%jonction_sup == CO .or.fil1%jonction_sup == JNF .or. fil1%jonction_sup==JFF)) then
	     fil1%jonction_sup = JNF
	     fil2%jonction_inf = JNF
	  else
	      print '(T5,"Noeud entre deux fils colinéaires=",3I5, " --> pas de jonctions JNF, les fils =",2(A,1x))',&
	          c_noeud(1:3,n),trim(fil1%nom),trim(fil2%nom)
	      n=n-1
	  end if 
	else
          c_noeud(fil1%dir,n) = fil2%coord(fil1%dir*2)
	  if ((fil1%jonction_inf == CO .or.fil1%jonction_inf == JNF .or. fil1%jonction_inf==JFF).and.&
	      (fil2%jonction_sup == CO .or.fil2%jonction_sup == JNF .or. fil2%jonction_sup==JFF)) then
	     fil1%jonction_inf = JNF
	     fil2%jonction_sup = JNF
	  else
	      print '(T5,"Noeud entre deux fils colinéaires=",3I5, " --> pas de jonctions JNF, les fils =",2(A,1x))',&
	          c_noeud(1:3,n),trim(fil1%nom),trim(fil2%nom)
	      n=n-1
	  end if 
        end if	
      end if
      cycle
    end if
    dir3 = redir(fil1%dir,fil2%dir) !donne la direction perpendiculaire aux 2 fils
    if (fil1%coord(dir3*2)/=fil2%coord(dir3*2)) cycle
    
    if (fil1%coord(fil1%dir*2-1)>fil2%coord(fil1%dir*2).or. &
        fil1%coord(fil1%dir*2)  <fil2%coord(fil1%dir*2)) cycle
    if (fil2%coord(fil2%dir*2-1)>fil1%coord(fil2%dir*2).or. &
        fil2%coord(fil2%dir*2)  <fil1%coord(fil2%dir*2)) cycle
    
    ! sinon jonction
    n=n+1
    c_noeud(dir3,n) = fil1%coord(dir3*2)
    c_noeud(fil1%dir,n) = fil2%coord(fil1%dir*2)
    c_noeud(fil2%dir,n) = fil1%coord(fil2%dir*2)
    !-------- Fil 1 - Jonction INF ------
    if (fil1%coord(fil1%dir*2-1)==c_noeud(fil1%dir,n)) then
      if (fil1%jonction_inf == CO .or.fil1%jonction_inf == JNF .or. fil1%jonction_inf==JFF) then
	fil1%jonction_inf = JNF
      else
        print '(T5,"Noeud entre deux fils =",3I5, " --> pas de jonctions JNF, les fils =",2(A,1x))',&
	          c_noeud(1:3,n),trim(fil1%nom),trim(fil2%nom)
        n=n-1
	cycle
      end if
    end if
    !-------- Fil 1 - Jonction SUP ------
    if (fil1%coord(fil1%dir*2  )==c_noeud(fil1%dir,n)) then
      if (fil1%jonction_sup == CO .or.fil1%jonction_sup == JNF .or. fil1%jonction_sup==JFF) then
	fil1%jonction_sup = JNF
      else
        print '(T5,"Noeud entre deux fils =",3I5, " --> pas de jonctions JNF, les fils =",2(A,1x))',&
	          c_noeud(1:3,n),trim(fil1%nom),trim(fil2%nom)
        n=n-1
	cycle
      end if
    end if
    
    !-------- Fil 2 - Jonction INF ------
    if (fil2%coord(fil2%dir*2-1)==c_noeud(fil2%dir,n)) then
      if (fil2%jonction_inf == CO .or.fil2%jonction_inf == JNF .or. fil2%jonction_inf==JFF) then
	fil2%jonction_inf = JNF
      else
        print '(T5,"Noeud entre deux fils =",3I5, " --> pas de jonctions JNF, les fils =",2(A,1x))',&
	          c_noeud(1:3,n),trim(fil1%nom),trim(fil2%nom)
        n=n-1
	cycle
      end if
    end if
    !-------- Fil 2 - Jonction sup ------
    if (fil2%coord(fil2%dir*2  )==c_noeud(fil2%dir,n)) then
      if (fil2%jonction_sup == CO .or.fil2%jonction_sup == JNF .or. fil2%jonction_sup==JFF) then
	fil2%jonction_sup = JNF
      else
        print '(T5,"Noeud entre deux fils =",3I5, " --> pas de jonctions JNF, les fils =",2(A,1x))',&
	          c_noeud(1:3,n),trim(fil1%nom),trim(fil2%nom)
        n=n-1
	cycle
      end if
    end if
  end do
end do
!--------- on désactive les doublons -----------------
print '(T5," Nombre de jonctions noeud exclu = ",I7)',nt-n
nt = n
n1=0
do i=1,nt
  do j=i+1,nt
    if (all(c_noeud(1:3,i)==c_noeud(1:3,j))) c_noeud(4,j) = 1 ! 1 pour désactivation
  end do
end do

n1 = sum(c_noeud(4,:))
print '(T5," Nombre de jonctions noeud en doublon = ",I7)',n1
if (st%n_noeud>0) deallocate(st%l_coord_noeud)
st%n_noeud = nt - n1
print '(T5," Nombre de jonctions noeud réel = ",I7)',st%n_noeud
allocate(st%l_coord_noeud(1:3,st%n_noeud))
n=0
do i=1,nt
  if (c_noeud(4,i)==0) then
    n=n+1
    st%l_coord_noeud(1:3,n) = c_noeud(1:3,i)
  end if
end do

deallocate(c_noeud)
end subroutine fil_detection_jonction_noeud

! ==========================================================
! == Function structure_fil_objet_pointe                  ==
! == st : liste de fil                                    ==
! == Retourne un pointeur pointant sur la                 ==
! == st_unfil de nom "chaine"                             ==
! ==========================================================
function structure_fil_objet_pointe(st,chaine) result(tob)

implicit none

type(st_structure_fil), intent(inout) :: st
character(LEN=LEN_MOT_CLE), intent(in) :: chaine

type(st_fil_objet), pointer :: tob
integer :: i

nullify(tob)
do i=1,st%n_objet
  if (chaine==st%l_objet(i)%nom) then
    tob => st%l_objet(i)
    exit
  endif
enddo

end function structure_fil_objet_pointe


! ==========================================================
! == Initialisation du type st_fil_segment              ==
! ==========================================================
subroutine fil_segment_init(s)

type(st_fil_segment), intent(inout) :: s
s%v = CVIDE
s%flag_c = .false.
s%flag_noeud = .false. ! sera à true si qc1 allouée
s%sige_eps = 0.0
NULLIFY(s%vc)
NULLIFY(s%c_vc)
end subroutine fil_segment_init

! ==========================================================
! == Subroutine verification_fil                        ==
! == Verifie que les six entiers forment un fil         ==
! == Déduit la direction du fil     ==
! ==========================================================
logical function verif_dir_fil(nom,coord6,direction)

implicit none

character(LEN=LEN_MOT_CLE), intent(in) :: nom
integer, dimension(1:6), intent(in)    :: coord6
integer, intent(out)  :: direction

integer  :: i,j
logical  :: flagsup

verif_dir_fil = .false.
direction = -1

! vérifie que le fil en est un : coordonnées égales dans 2 directions
j=0
do i=1,3
 if (coord6(i*2-1) == coord6(i*2)) j = j+1
end do

if (j/=2) then
  print '(/,t10,"ATTENTION, ", A, " ne represente pas un fil !!!")', TRIM(nom)
  return
endif

! vérifie que les coordonnées sont correctes dans la direction du fil
do i=1,3
 if (coord6(i*2) > coord6(i*2-1)) then
   direction = i
   verif_dir_fil = .true.
   return
 end if
end do

print '(/,t10,"ATTENTION, fil de nom :", A)', TRIM(nom)
print '(/,t10,"Veillez a ce que indices de debut < indices de fin")'

end function verif_dir_fil

! ==========================================================
! == Subroutine verif_existence_fil                     ==
! == Verifie que le fil n'existe pas deja               ==
! ==========================================================
logical function verif_existence_fil(st,i,nom,coord)

type(st_structure_fil), intent(in) :: st
character(LEN=LEN_MOT_CLE), intent(in) :: nom
integer, intent(in)  :: i
integer, dimension(1:6), intent(in) :: coord

integer :: j

verif_existence_fil = .true.
do j=1,i
  if (nom==st%l_fil(j)%nom) then
    print '(/,t10,"ATTENTION, le nom ", A, " existe deja : stop")', TRIM(nom)
    verif_existence_fil = .false.
  endif
  if (ALL(coord(:) == st%l_fil(j)%coord(:))) then
    print '(/,t10,"ATTENTION, le fil ", A, &
      &" existe deja sous le nom ", A, " : stop")', &
      TRIM(nom), TRIM(st%l_fil(j)%nom)
    verif_existence_fil = .false.
  endif
enddo

end function verif_existence_fil

! ==========================================================
! == Subroutine verif_rayon_fil                           ==
! == Vérifie que le rayon du fil est inférieur à la taille
! ==  de la cellule                                       ==
! ==========================================================
logical function verif_rayon_fil(nomfil,celx,cely,celz,rayon,dir)

implicit none

character(LEN=LEN_MOT_CLE), intent(in) :: nomfil
real, intent(in)  :: celx, cely, celz, rayon
integer, intent(in)   :: dir

real :: cel1,cel2,cel(1:3),scel,sfil

cel(1) = celx
cel(2) = cely
cel(3) = celz
cel1 = cel(PC1(dir))
cel2 = cel(PC2(dir))
verif_rayon_fil = .true.
scel = cel1*cel2
sfil = PI*rayon**2

if (sfil >= (scel*0.95)) then
  verif_rayon_fil = .false.
  print '(/,t10," Diametre du fil ", A, " doit etre inférieur à taille cellule !!!")', TRIM(nomfil)
end if
end function verif_rayon_fil

! ==========================================================
! == Subroutine verif_decalage_fil                        ==
! == Vérifie que les offsets du fil ne le sortent pas de la
! == cellule                                              ==
! ==========================================================
logical function verif_decalage_fil(nomfil,celx,cely,celz,delta,dir)

character(LEN=LEN_MOT_CLE), intent(in) :: nomfil
real, intent(in)  :: celx, cely, celz
real, intent(in)   :: delta(1:3)
integer, intent(in) :: dir
logical :: verif
real :: celmin,cel(1:3)

cel(1) = celx
cel(2) = cely
cel(3) = celz

verif_decalage_fil = .true.
!verif = ALL(delta >= cel)
if (.not.(ALL(delta <= cel))) then
  verif_decalage_fil = .false.
  print '(/,t10,"Décalage du fil : ", A, " doit etre inférieur à taille cellule !!!")', TRIM(nomfil)
end if
end function verif_decalage_fil


! ==========================================================
! == Function verif_nom_gene_fil()                       ==
! == Verifie si le nom de générateur existe               ==
! == Dans le cas contraire : stop                         ==
! ==========================================================
integer function verif_nom_gene_fil(st,nom)

type(st_structure_fil), intent(in) :: st
character(LEN=LEN_MOT_CLE), intent(in) :: nom

logical :: trouve
integer :: j

trouve = .false.
do j=1,st%n_fil
  if (st%l_fil(j)%flag_gv) then
    if (nom==st%l_fil(j)%gv%nom) then
      trouve = .true.
      exit
    endif
  end if
enddo
if (.not.trouve) then
  print '(/,t10,"ATTENTION, Le générateur ", A, " n`existe pas : stop")',TRIM(nom)
  j = 0
endif

verif_nom_gene_fil = j

end function verif_nom_gene_fil



! ==========================================================
! == Function verif_nom_fil()                           ==
! == Verifie si le fil d'attache existe                 ==
! == Dans le cas contraire : stop                         ==
! ==========================================================
integer function verif_nom_fil(st,nom)

type(st_structure_fil), intent(in) :: st
character(LEN=LEN_MOT_CLE), intent(in) :: nom

logical :: trouve
integer :: j

trouve = .false.
do j=1,st%n_fil
  if (nom==st%l_fil(j)%nom) then
    trouve = .true.
    exit
  endif
enddo
if (.not.trouve) then
  print '(/,t10,"ATTENTION, Le fil ", A, " n`existe pas : stop")', TRIM(nom)
  j = 0
endif

verif_nom_fil = j

end function verif_nom_fil


! ==========================================================
! == Function position                                    ==
! == Donne l'extremite du fil ou du fil concernée  par  ==
! == la jonction
! ==========================================================
integer function fil_position_jonction(lieu)

character(LEN=LEN_MOT_CLE), intent(in) :: lieu

if (lieu=='deb') then
  fil_position_jonction = EXTR_DEBUT
elseif (lieu=='fin') then
  fil_position_jonction = EXTR_FIN
else
  print '(/,t10,"Erreur de description des jonctions !!!")'
  fil_position_jonction = -1
endif

end function fil_position_jonction

! ==========================================================
! == Function critere_stabilite                           ==
! == Calcul du critere de stabilité du fil                ==
! == si dtfil > dtfdtd alors avertissement                ==
! ==========================================================
real function critere_stabilite(dtfdtd,L,v,dl,d1,d2) result(dtfil)

real, intent(in) :: dtfdtd, L,v,dl,d1,d2

real   :: A,dt0,dtmaxfdtd,cdtmax

dtmaxfdtd = 1/(CVIDE*SQRT(1.0/d1**2 + 1.0/d2**2 + 1.0/dl**2))

dt0 = dl/v
A   = (MU_0*v**2.0/L) * (1.0/(d1*d2)+(2.0/PI)*(1.0/d1**2+1.0/d2**2))
dtfil = dt0/SQRT(1+A*dt0**2*0.25)

!print '(T10," --> Critère de stabilité : dt = ",EN16.7e2, " s < dtfil = ",EN16.7e2, " s")', &
!                   dtfdtd,dtfil
if (dtfil<dtfdtd) then
  print '(T10," AVERTISSEMENT GRAVE : NON RESPECT DU CRITÈRE DE STABILITÉ DU FIL")'
  print '(T10," CAR DTFIL < DTFDTD  ----------> RISQUE DE TROMPETTE FUNCTION !!!!!!")'
  print '(T10," PROPOSITION : ARRETER TRIDIMO PUIS METTRE DANS LE BLOC <volume_calcul> :")'
  
  cdtmax = dtfil/dtmaxfdtd * 0.95
  print '(T10,"   cdtmax  ",F5.2)',cdtmax
  stop
  
end if


end function critere_stabilite

! ==========================================================
! == Subroutine verif_segment_fil                         ==
! == Verifie que 1<seg < nombre de segments du fil        ==
! ==========================================================
logical function verif_segment_fil(nomfil,nsegfil,iseg)

implicit none
character(LEN=LEN_MOT_CLE), intent(in) :: nomfil
integer, intent(in) :: nsegfil,iseg

integer :: l

verif_segment_fil = .true.
if (iseg>nsegfil) then
  l = len_int(nsegfil)
  print '(/,t10,"ATTENTION, le fil ", A, " ne possede que ", &
    &i4, " segments : stop")', TRIM(nomfil), nsegfil
  verif_segment_fil = .false.
end if
if (iseg<1) then
  print '(/,t10,"ATTENTION, un n° de segment de fil >= 1 !!! ")'
  verif_segment_fil = .false.
endif

end function verif_segment_fil

! ==========================================================
! == Subroutine structure_fil_precalcul                   ==
! == Calculs avant les itérations temporelles             ==
! ==========================================================
subroutine structure_fil_precalcul(tf,v,tm,t_gene)
implicit none
!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf
type(st_volume), intent(in) :: v
type(st_milieu_volume), intent(in)  :: tm
type(st_generateur), intent(inout)     :: t_gene

!=== Declaration des variables locales ===
integer :: i,j,k,n, coordaxe,coordseg
real  :: epsilon_segment,sige_segment,vitesse_segment,vmax
real  :: s_discret
real  :: lb ,R12,h,retard
real  :: dtfil
integer, dimension(2) :: decal_front
real, dimension(2) :: position_fil
real  ::cm1,cm2,cm4
real, dimension(:,:), pointer  :: sdisque
real, dimension(3) :: delta3d ! pas spatiaux 3d transverses
type(st_unfil), pointer :: fil
type(st_fil_segment), pointer  :: tseg

!=== debut des traitements ===
tf%dt = v%dt

!--------------  Préparation pour une distribution sur un disque ------------------
!                       MAILLAGE NUMÉRIQUE DU DISQUE
if (tf%flag_disque) then
  allocate(sdisque(NB_MAILLE_DISQUE,NB_MAILLE_DISQUE))
  allocate(tf%sdisque(NB_MAILLE_DISQUE,NB_MAILLE_DISQUE))
  sdisque = 0.0  ; tf%sdisque = 0.0
  tf%s_dd = maillage_disque(NB_MAILLE_DISQUE,1.0, sdisque)
end if


!---------------------- TRAITEMENT SUR TOUS LES FILS ---------------------------
do n=1, tf%n_fil
  fil => tf%l_fil(n)
  print '(T5,"--> FIL de nom ",A," ; N°=",I5)',TRIM(fil%nom),n
  !-- calcul du générateur --
  if (fil%flag_gv) then
    allocate(fil%tabgene(v%ntinc))
!    fil%tabgene = 0.0
    i = generateur_calcul_t(t_gene, fil%gv%nomgene,1.0, &
          fil%gv%phase, fil%gv%retard, v%dt, v%ntinc, fil%tabgene)
  end if

  ! aux point coordonnées min du fil
  delta3d(1:3) = v%dxyz(1:3)
  fil%d3d_t(1) = delta3d(PC1(fil%dir))      ! pas spatial transverse
  fil%d3d_t(2) = delta3d(PC2(fil%dir)) ! pas spatial transverse
  fil%surface_rect  = fil%d3d_t(1) * fil%d3d_t(2)
  position_fil(1) = fil%coord(PC1(fil%dir)*2)
  position_fil(2) = fil%coord(PC2(fil%dir)*2)
  
  !---------- Calcul de l'inductance propre ------------------
  if (fil%flag_disque) then
      fil%lp = inductance_propre_disque(fil%r_disque,fil%rayon, MU_0)
  else
      fil%lp = inductance_propre_holland(fil%d3d_t,fil%rayon, MU_0)
      !fil%cp_emoy = fil%cp_emoy/fil%lp
      !fil%surface_rect = fil%surface_rect/fil%lp
      !fil%lp=1.0
   end if
  
  !------ Calcul des pondérations sur les champs moyens  --------
  fil%cp_emoy(1) = (1-fil%decal_cell(1)/fil%d3d_t(1))*(1-fil%decal_cell(2)/fil%d3d_t(2))
  fil%cp_emoy(2) = (fil%decal_cell(1)/fil%d3d_t(1))*(1-fil%decal_cell(2)  /fil%d3d_t(2))
  fil%cp_emoy(3) = (1-fil%decal_cell(1)/fil%d3d_t(1))*(fil%decal_cell(2)/fil%d3d_t(2))
  fil%cp_emoy(4) = (fil%decal_cell(1)  /fil%d3d_t(1))*(fil%decal_cell(2)  /fil%d3d_t(2))
  !fil%cp_emoy  = 0.25

  if (tf%flag_plan_masse .and. (fil%dir == tf%plan_masse.or.PC1(fil%dir)==tf%plan_masse)) then
    i = tf%pos_plan_masse - fil%coord(PC2(tf%plan_masse)*2)
    if (i==0 .or.i==1) then
    
      h = abs(i*v%dxyz(PC2(tf%plan_masse)) - fil%delta3(PC2(tf%plan_masse)))
      lb = inductance_mutuelle_berenger(fil%d3d_t,fil%rayon, MU_0,h)
      print '(T15," PLAN MASSE, h=",ES15.6,",  Lb = ",ES15.6)',h,lb
      fil%lp=fil%lp+lb
      !fil%flag_plan_masse = .true.
      !--- Calcul du retard du courant image  ---
      !epsilon_segment = milieu_volume_extrait_epsilon(tm,fil%coord(XMIN),fil%coord(YMIN),&
!						       fil%coord(ZMIN),fil%dir)
      !if (fil%flag_epsr) epsilon_segment = fil%epsilonr *EPSILON_0
      !vitesse_segment = CVIDE * SQRT(EPSILON_0/epsilon_segment)
      !retard = 2*h/vitesse_segment
      !fil%ir_pm = ceiling(retard/v%dt)
      !fil%p_pm = fil%ir_pm*1.0 -retard/v%dt
      !print '(T15," PLAN MASSE, retard=",ES15.6,",  p = ",ES15.6,", itr=",I3)',&
      !              retard,fil%p_pm,fil%ir_pm
      
      if (i==0) then
        fil%cp_emoy(1)=0.0
	if (fil%dir == tf%plan_masse) then
	  fil%cp_emoy(2)=0.0
	else
	  fil%cp_emoy(3)=0.0
	end if
      else
        fil%cp_emoy(4)=0.0
	if (fil%dir == tf%plan_masse) then
	  fil%cp_emoy(2)=0.0
	else
	  fil%cp_emoy(3)=0.0
	end if
      end if
    end if
  end if
  print '(T15,"cp_e=",4ES15.5)',fil%cp_emoy(1:4)


  !-------------------- Traitement spécifique au disque -----------------------
  if (fil%flag_disque) then
    ! position fil en nombre de cellules
    position_fil(1) = position_fil(1) + fil%decal_cell(1)/fil%d3d_t(1)
    position_fil(2) = position_fil(2) + fil%decal_cell(2)/fil%d3d_t(2)
    fil%position_fil = position_fil
    tf%sdisque(:,:) = sdisque(:,:)

    ! surface réelle
    tf%sdisque(:,:) = tf%sdisque(:,:) * (2*fil%r_disque / NB_MAILLE_DISQUE)**2
    fil%s_discret = 0.0
    fil%s_discret = SUM(tf%sdisque)
    
    ! Cellules concernées par le disque       
    call cellule_du_disque(fil%uc,fil%vc,position_fil(1:2),fil%r_disque,fil%d3d_t(1:2))
    
    ! Surface pour chaque composante
    allocate(fil%surface(fil%uc(1):fil%uc(2),fil%vc(1):fil%vc(2)))
    fil%surface = 0.0
    fil%surface = distribution_disque(fil%uc,fil%vc,position_fil(1:2), &
	     fil%r_disque,fil%d3d_t(1:2),NB_MAILLE_DISQUE,tf%sdisque) 

    ! Alias sur le champ électrique
    fil%nbcomposante = (fil%uc(2) - fil%uc(1)+1) * (fil%vc(2) - fil%vc(1)+1)
    
    ! Coefficients pondératifs pour le courant et le champ E
    allocate(fil%cp_j(fil%nbcomposante))
    allocate(fil%cp_e(fil%nbcomposante))
    fil%cp_j = 0.0; fil%cp_e = 0.0;
    call coeff_j_e_disque(fil)  

    ! Calcul de la permittivité moyenne sur le disque et de chaque composante E
    fil%eps_moy = permittivite_cell(fil%uc,fil%vc,fil%ldeb,fil%dir,fil%surface,tm)
  end if

  !----------------------------------------------------------------------------
  
  !------- Allongement du fil si frontiere PML -------
  fil%ldeb_frontiere = fil%ldeb
  fil%lfin_frontiere = fil%lfin
  
  if (fil%jonction_inf == JPML) fil%ldeb_frontiere = v%borne_volume(fil%dir*2-1)
  if (fil%jonction_sup == JPML) fil%lfin_frontiere = v%borne_volume(fil%dir*2)
  
  decal_front(1) = fil%ldeb_frontiere-fil%ldeb !valeur négative ou nulle
  decal_front(2) = fil%lfin_frontiere-fil%lfin
  fil%coord_rel(1) = decal_front(1)
  fil%coord_rel(2) = fil%nseg+1+decal_front(2)
  ! pas spatial suivant la direction du fil
  allocate(fil%dl   (fil%coord_rel(1):fil%coord_rel(2)))
  allocate(fil%l_seg(fil%coord_rel(1):fil%coord_rel(2)))
  !if (fil%flag_plan_masse) then
  !  allocate(fil%I_avant(1:fil%ir_pm,fil%coord_rel(1):fil%coord_rel(2)))  
  !  fil%I_avant = 0.0
  !end if
  fil%dl(fil%coord_rel(1):fil%coord_rel(2)) = v%dxyz(fil%dir)
!  if (fil%dir == X) then
!    fil%dl(fil%coord_rel(1):fil%coord_rel(2)) = &
!          v%dx(fil%ldeb_frontiere:fil%lfin_frontiere)
!  else if (fil%dir == Y) then
!    fil%dl(fil%coord_rel(1):fil%coord_rel(2)) = &
!          v%dy(fil%ldeb_frontiere:fil%lfin_frontiere)
!  else
!    fil%dl(fil%coord_rel(1):fil%coord_rel(2)) = &
!          v%dz(fil%ldeb_frontiere:fil%lfin_frontiere)
!  end if

  print '(T10," Coordonnées relatives =",2I5)',fil%coord_rel(1:2)


  ! On récupère la permittivité sur le fil
  do k=1,2
    if (k==1) then
      coordaxe = 1
    else
      coordaxe = fil%nseg
    end if
    if (fil%dir == X) then
      epsilon_segment = milieu_volume_extrait_epsilon(tm,coordaxe,fil%coord(YMIN),&
						    fil%coord(ZMIN),fil%dir)
      sige_segment    = milieu_volume_extrait_sige(tm,coordaxe,fil%coord(YMIN),&
						    fil%coord(ZMIN),fil%dir)
    else if (fil%dir == Y) then						    
      epsilon_segment = milieu_volume_extrait_epsilon(tm,fil%coord(XMIN),coordaxe,&
						    fil%coord(ZMIN),fil%dir)
      sige_segment    = milieu_volume_extrait_sige(tm,fil%coord(XMIN),coordaxe,&
						    fil%coord(ZMIN),fil%dir)
    else ! Z
      epsilon_segment = milieu_volume_extrait_epsilon(tm,fil%coord(XMIN),fil%coord(YMIN),&
						    coordaxe,fil%dir)
      sige_segment    = milieu_volume_extrait_sige(tm,fil%coord(XMIN),fil%coord(YMIN),&
						    coordaxe,fil%dir)
    end if
    fil%eps_extr(k) = epsilon_segment    
    fil%sige_extr(k) = sige_segment    
  end do

  !--> prise en compte d'un résistance linéique
  if (fil%R_lineique>0.0) then
    fil%ro = fil%ro+fil%R_lineique*fil%dl(0:fil%nseg+1)
  end if


  !----------------------------------------------------------------------------
  !-------------- TRAITEMENT DE CHAQUE SEGMENT DU FIL -------------------------
  vmax = 0
  do k=fil%coord_rel(1),fil%coord_rel(2)
    tseg => fil%l_seg(k)
    call fil_segment_init(tseg)
    ! on récupère epsilon et on déduit la vitesse de propagation
    coordaxe = fil%ldeb+k-1
    coordseg = k
    if (k<=0)  then 
      coordaxe= fil%ldeb 
      coordseg = 1
    else if (k>=fil%nseg+1) then
      coordaxe= fil%lfin-1
      coordseg = fil%nseg
    end if 
    
    ! On récupère la permittivité sur le fil
    if (fil%dir == X) then
      epsilon_segment = milieu_volume_extrait_epsilon(tm,coordaxe,fil%coord(YMIN),&
						    fil%coord(ZMIN),fil%dir)
      sige_segment    = milieu_volume_extrait_sige(tm,coordaxe,fil%coord(YMIN),&
						    fil%coord(ZMIN),fil%dir)
    else if (fil%dir == Y) then						    
      epsilon_segment = milieu_volume_extrait_epsilon(tm,fil%coord(XMIN),coordaxe,&
						    fil%coord(ZMIN),fil%dir)
      sige_segment    = milieu_volume_extrait_sige(tm,fil%coord(XMIN),coordaxe,&
						    fil%coord(ZMIN),fil%dir)
    else ! Z
      epsilon_segment = milieu_volume_extrait_epsilon(tm,fil%coord(XMIN),fil%coord(YMIN),&
						    coordaxe,fil%dir)
      sige_segment    = milieu_volume_extrait_sige(tm,fil%coord(XMIN),fil%coord(YMIN),&
						    coordaxe,fil%dir)
    
    end if						    
    ! Si utilisateur a défini une permittivité relative pour tout le fil
    if (fil%flag_epsr) epsilon_segment = fil%epsilonr *EPSILON_0

    vitesse_segment = CVIDE * SQRT(EPSILON_0/epsilon_segment)
    tseg%v = vitesse_segment
    vmax = max(vmax,vitesse_segment)
    ! allocation tableau pour I, Q, Emoy,  Vsrc
    allocate(tseg%cur)
    allocate(tseg%qc)
    allocate(tseg%vsrc)
    allocate(tseg%emoy)
    if (k>fil%coord_rel(1)) fil%l_seg(k-1)%qc1 =>tseg%qc

    tseg%qc = 0.0
    tseg%emoy = 0.0 ; tseg%vsrc = 0.0
    tseg%cur = 0.0

    !--- présence d'une capacité sur le segment ---
    if (fil%co(coordseg) > 0 .and. fil%nom(1:1)=='F') then
!      print '("n°segment=",I2)',k
      allocate(tseg%vc)
      allocate(tseg%c_vc)
      tseg%flag_c = .true.
      tseg%vc = 0.0; 
      tseg%c_vc = v%dt/fil%co(coordseg)
    end if

    ! coefficient relatif à la dérivée du courant
    if (fil%ro(coordseg)>0 .and. fil%nom(1:1)=='F') then ! solution exponentielle
      cm1 = exp(-fil%ro(coordseg)*v%dt / (fil%lp*fil%dl(k) + fil%lo(coordseg)))
      cm2 = 1.0
      cm4 = (1-cm1)*fil%dl(k)/fil%ro(coordseg)
    else
      cm1 = fil%lp/v%dt + fil%lo(coordseg) /(fil%dl(k)*v%dt) - fil%ro(coordseg) /(2*fil%dl(k))
      cm2 = fil%lp/v%dt + fil%lo(coordseg) /(fil%dl(k)*v%dt) + fil%ro(coordseg) /(2*fil%dl(k))
      cm4 = 1.0
    end if

    ! calcul du coefficient  multiplicatif du courant
    tseg%c_cur = cm1/cm2

    ! Calcul du coefficient  multiplicatif de la charge
    tseg%c_qc = fil%lp * vitesse_segment**2*cm4 /(fil%dl(k)*cm2)

    ! Modification du coefficient  multiplicatif de la charge si
    ! présence d'une interface :
    if (fil%flag_interface) then ! affichage pour un seul segment
      lb = inductance_mutuelle_berenger(fil%d3d_t,fil%rayon, MU_0,fil%h_interface)
      R12= (fil%epsr_interface(2)- fil%epsr_interface(1))/SUM(fil%epsr_interface)
      tseg%c_qc = tseg%c_qc * (1-R12*lb/fil%lp)
      if (k==1) then
        print '(T10,"FIL - Coefficient avec Interface : ")'
        print '(T10,"   R12 = ",F5.3)',R12
        PRINT '(T10,"   Inductance mutuelle :",ES15.6e2)',lb
        PRINT '(T10,"   Inductance propre   :",ES15.6e2)',fil%lp
      end if
    end if
    
    
    ! Calcul du coefficient multiplicatif du champ moyen
    tseg%c_emoy = cm4/cm2
    
    ! Calcul du coefficient  multiplicatif de la tension
    tseg%c_vsrc = cm4/(cm2*fil%dl(k))
   ! if (n==3) then
      !print '("nufil,nuseg = ",2I3,"c_cur,c_vsrc,c_emoy =",3ES15.7E2)',n,k,tseg%c_cur,tseg%c_vsrc,tseg%c_emoy
   ! end if

    !------------------------------------------------------------
    ! coefficient multiplicatif du courant, équation de la charge
    tseg%cq_q   = (1-sige_segment*v%dt/(2*epsilon_segment))/(1+sige_segment*v%dt/(2*epsilon_segment))
    tseg%cq_cur = (v%dt / fil%dl(k))/(1+sige_segment*v%dt/(2*epsilon_segment))
    
    tseg%sige_eps = sige_segment/epsilon_segment
    tseg%dt_epsilon = (v%dt/epsilon_segment)/(1+sige_segment*v%dt/(2*epsilon_segment))
    
    ! Calcul des pondérations pour les densités de courants
    tseg%cp_j(1) = (tseg%dt_epsilon)*fil%cp_emoy(1) / (fil%surface_rect)
    tseg%cp_j(2) = (tseg%dt_epsilon)*fil%cp_emoy(2) / (fil%surface_rect)
    tseg%cp_j(3) = (tseg%dt_epsilon)*fil%cp_emoy(3) / (fil%surface_rect)
    tseg%cp_j(4) = (tseg%dt_epsilon)*fil%cp_emoy(4) / (fil%surface_rect)

!    print '("i,cq_cur,c_cur,c_qc=",I3,3ES15.6e2)',k,tseg%cq_cur,tseg%c_cur,tseg%c_qc

    !print '(T12,"iseg=",I5)',k
    !print '(T15,"cp_e=",4ES15.5)',fil%cp_emoy(1:4)
    !print '(T15,"cp_j=",4ES15.5)',tseg%cp_j(1:4)
    !print '(T15,"CI1,CIQ,CIemoy=",3ES15.5)',tseg%c_cur,tseg%c_qc,tseg%c_emoy
    !print '(T15,"CQ1,2=",2ES15.5)',tseg%cq_q,tseg%cq_cur

  end do ! fin boucle sur les segments
  if (.not.fil%flag_disque) then
    dtfil = critere_stabilite(v%dt,fil%lp,vmax,v%dxyz(fil%dir),fil%d3d_t(1),fil%d3d_t(2))
  end if

  !------------------------------------------------------------------------------
  !   TRAITEMENT D'UN FIL DE TYPE COAXIAL
  IF (fil%flag_coaxial) then
    call fil_coaxial_precalcul(fil,v%dt)
  END IF
!------------------------------------------------------------------------------


!--------------------------------------------------------------------------------------------------
  ! Segment 0 et nseg+1, le vecteur courant est celui du segment 1 et nse
!    fil%l_seg(fil%coord_rel(1))%cur => fil%l_seg(fil%coord_rel(1)+1)%cur
!    fil%l_seg(fil%coord_rel(2))%cur => fil%l_seg(fil%coord_rel(1)-1)%cur
!  print '("c_cur = ",2ES15.6E2)',fil%l_seg(35:36)%c_cur
!  print '("c_qc = ",2ES15.6E2)',fil%l_seg(35:36)%c_qc
!  print '("c_emoy = ",2ES15.6E2)',fil%l_seg(35:36)%c_emoy
!  print '("c_vsrc = ",2ES15.6E2)',fil%l_seg(35:36)%c_vsrc
!  print '("cq_cur = ",2ES15.6E2)',fil%l_seg(35:36)%cq_cur
end do ! Fin boucle sur les fils


if (tf%flag_disque) then
  deallocate(sdisque)
  deallocate(tf%sdisque) ; nullify(tf%sdisque)
  
  do n=1, tf%n_fil
    fil => tf%l_fil(n)
    deallocate(fil%ro,fil%lo,fil%co)
  end do
end if


!================= Traitement de la jonction Fil-coaxial =============
if (tf%n_jfc>0) call jonction_FF_precalcul(tf)


!================ Jonction multi-fils =====
if (tf%n_noeud>0) call structure_fil_noeud_precalcul(tf)

end subroutine structure_fil_precalcul

! ==========================================================
! == Subroutine jonction_FF_precalcul                     ==
! == Jonction Fil-Coaxial et Fil avec Quadripole          ==
! ==========================================================
subroutine jonction_FF_precalcul(tf)
!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf

!=== Declaration des variables locales ===
type(st_unfil), pointer          :: fil
type(st_jonction_fil_coax),pointer :: pjf
type(st_fil_segment), pointer :: tseg
real  :: c1,c2,c3,dl1,dl2,s1,s2
integer  :: i,n 
!=== DEBUT DES TRAITEMENTS ===
do i=1,tf%n_jfc
  pjf => tf%l_jfc(i)
  
    dl1 = pjf%fil %dl(1)/2
    dl2 = pjf%coax%dl(1)/2
    c2 = pjf%coax%Cp
    s1 = 1.0
    s2 = 1.0
    !--------------------------------------------------------------
    !--- Traitement pointeur et coefficient pour les charges 
    !    d'extrémités du coaxial ---
    if (pjf%extrcoax==EXTR_DEBUT) then
      pjf%Q2 => pjf%coax%t_coax%Q(1)  ! charge de lame
      pjf%I2 => pjf%coax%t_coax%I(1)  ! courant de lame
      tseg =>pjf%coax%l_seg(1)
      pjf%Q3 => tseg%qc   ! charge du blindage externe
      pjf%I3 => tseg%cur  ! courant du blindage externe
      c3 = 1/(pjf%coax%lp*tseg%v**2)
      pjf%cq3 = tf%dt/dl2
    else
      n = pjf%coax%nseg
      pjf%Q2 => pjf%coax%t_coax%Q(n+1)  ! charge de lame
      pjf%I2 => pjf%coax%t_coax%I(n)    ! courant de lame
      tseg =>pjf%coax%l_seg(n)
      nullify(tseg%qc1); allocate(tseg%qc1)
      pjf%Q3 => tseg%qc1   ! charge du blindage externe
      pjf%I3 => tseg%cur   ! courant du blindage externe
      tseg%flag_noeud = .true.
      c3 = 1/(pjf%coax%lp*tseg%v**2)
      pjf%cq3 = -tf%dt/dl2
      s2 = -1.0      
    end if
    !--------------------------------------------------------------
    !--- Traitement du fil nu
    if (pjf%extrfil==EXTR_DEBUT) then
      tseg =>pjf%fil%l_seg(1)
      pjf%Q1 => tseg%qc   ! charge du fil
      pjf%I1 => tseg%cur  ! courant du fil
      c1 = 1/(pjf%fil%lp*tseg%v**2)      
    else
      n = pjf%fil%nseg
      tseg =>pjf%fil%l_seg(n)
      nullify(tseg%qc1); allocate(tseg%qc1)
      tseg%flag_noeud = .true.
      pjf%Q1 => tseg%qc1   ! charge du fil
      pjf%I1 => tseg%cur   ! courant du fil
      c1 = 1/(pjf%fil%lp*tseg%v**2)
      s1 = -1.0      
    end if
    pjf%Q1 = 0.0; 
    pjf%Q2 = 0.0;
    pjf%Q3 = 0.0;
    
    pjf%cq1(1) = -s1*2*tf%dt*c1/(c1*dl1+c2*dl2)
    pjf%cq1(2) = -s2*2*tf%dt*c1/(c1*dl1+c2*dl2)
    pjf%cq1(3) = -s2*2*tf%dt*c1/(c1*dl1+c2*dl2)*c2/c3
    
    pjf%cq2(1) = -s1*2*tf%dt*c2/(c1*dl1+c2*dl2)
    pjf%cq2(2) = -s2*2*tf%dt*c2/(c1*dl1+c2*dl2)
    pjf%cq2(3) = +s2*2*tf%dt*c2/(c1*dl1+c2*dl2)*c1/c3
    !--------------------------------------------------------------

end do


end subroutine jonction_FF_precalcul


! ==========================================================
! == Subroutine fil_coaxial_precalcul                     ==
! == Precacul pour les cables coaxials                    ==
subroutine fil_coaxial_precalcul(fil,dt)
!=== Declaration des arguments ===
type(st_unfil), intent(inout) :: fil
real, intent(in)   :: dt
!=== Declaration des variables locales ===
integer :: k,coordseg
real :: Lp,Cp,cm1,cm2,cm3,cm4
real :: eps
type(st_fil_coaxial), pointer :: coax

!=== debut des traitements ===
coax =>fil%t_coax
allocate(coax%I(fil%coord_rel(1):fil%coord_rel(2)))
allocate(coax%Q(fil%coord_rel(1):fil%coord_rel(2)))
allocate(coax%cI1(fil%coord_rel(1):fil%coord_rel(2)))
allocate(coax%cIq(fil%coord_rel(1):fil%coord_rel(2)))
allocate(coax%cIvsrc(fil%coord_rel(1):fil%coord_rel(2)))
allocate(coax%cQ1(fil%coord_rel(1):fil%coord_rel(2)))
allocate(coax%cQ2(fil%coord_rel(1):fil%coord_rel(2)))
coax%I     =0.0
coax%Q     =0.0
coax%cI1   =0.0
coax%cIq   =0.0
coax%cIvsrc=0.0
coax%cQ1   =0.0
coax%cQ2   =0.0

if (fil%flag_gv) then
  coax%flag_gv = .true.
  fil%flag_gv  = .false.
end if
eps = EPSILON_0*coax%epsr
! Inductance d'un coaxial
Lp = (MU_0/(2*PI))*log(fil%rayon/coax%r0)
Cp = 2*PI*eps     /log(fil%rayon/coax%r0)
coax%Lp = Lp
coax%Cp = Cp

do k=fil%coord_rel(1),fil%coord_rel(2)
  coordseg = k
  if (k<=0)  then 
    coordseg = 1
  else if (k>=fil%nseg+1) then
    coordseg = fil%nseg
  end if 
  
  ! coefficient relatif à la dérivée du courant
  if (fil%ro(coordseg)>0) then ! solution exponentielle
      cm1 = exp(-fil%ro(coordseg)*dt / (Lp*fil%dl(k) + fil%lo(coordseg)))
      cm2 = 1.0
      cm4 = (1-cm1)*fil%dl(k)/fil%ro(coordseg)
  else
      cm1 = (Lp + fil%lo(coordseg) /fil%dl(k))/dt - fil%ro(coordseg) /(2*fil%dl(k))
      cm2 = (Lp + fil%lo(coordseg) /fil%dl(k))/dt + fil%ro(coordseg) /(2*fil%dl(k))
      cm4 = 1.0
  end if

    ! calcul du coefficient  multiplicatif du courant
    coax%cI1 = cm1/cm2

    ! Calcul du coefficient  multiplicatif de la charge
    coax%cIq = fil%lp * cm4 /(Lp*Cp*fil%dl(k)*cm2)

    ! Calcul du coefficient  multiplicatif de la tension
    coax%cIvsrc = cm4/(cm2*fil%dl(k))

    !------------------------------------------------------------
    ! coefficient multiplicatif du courant, équation de la charge
    coax%cq1   = (1-coax%sige*dt/(2*eps))/(1+coax%sige*dt/(2*eps))
    coax%cq2   = (dt / fil%dl(k))        /(1+coax%sige*dt/(2*eps))
end do

end subroutine fil_coaxial_precalcul

! ==========================================================
! == Subroutine inductance_propre_holland                 ==
! == Calcul de l'inductance propre selon la formule       ==
! == de Holland
! ==========================================================
real function inductance_propre_holland(d3d_t,rayon,mu)
implicit none
!=== Declaration des arguments ===
real, dimension(2), intent(in)  :: d3d_t
real, intent(in)    :: rayon,mu
!=== Declaration des variables locales ===
real  :: d1,d2
!=== debut des traitements ===
d1 = d3d_t(1)
d2 = d3d_t(2)

inductance_propre_holland = (mu/(4*PI)) * (LOG((d1**2+d2**2)/(4*rayon**2)) + &
    d1/d2 * ATAN(d2/d1) + d2/d1 * ATAN(d1/d2) + PI*rayon**2 /(d1*d2) -3)

! Pour les gros fils, on utilise l'inductance modifié de Grando :
if ((rayon >0.3 *d1).or.(rayon >0.3*d2)) then
  inductance_propre_holland = inductance_propre_holland /(1-PI*rayon**2 /(d1*d2))
end if

end function inductance_propre_holland

! ==========================================================
! == Subroutine inductance_mutuelle_berenger              ==
! == Calcul de l'inductance mutuelle de Bérenger          ==
! == Intégrale double sur une surface d'une cellule FDTD  ==
! == réexprimée en fonction de theta et r                 ==
! ==========================================================
real function inductance_mutuelle_berenger(delta3d,rayon,mu,h)
implicit none
!=== Declaration des arguments ===
real, dimension(2), intent(in)  :: delta3d
real, intent(in)    :: rayon,mu
real, intent(in)    :: h
!=== Declaration des variables locales ===
integer  :: i,j
real  :: bt1,bt2
real  :: d,psi
real  :: dth, dr  ! pas spatiaux suivant theta et r
real  :: l12, theta,theta1,angleref, r
!=== debut des traitements ===


! Calcul de l'angle psi et de la distance d entre les 2 fils 
d = 2*h
if (h>0) then
    psi = PI/2
else
    psi = -PI/2
end if

! angle de référence :
angleref = ATAN2(delta3d(2),delta3d(1))

! bornes et pas spatial suivant theta
bt1 = 0.0
bt2 = 2*PI
dth = (bt2-bt1) / NB_ECH_THETA

!--- Integration suivant theta ---
l12 = 0.0
do j = 0, NB_ECH_THETA-1
  ! angle theta
  theta = j * dth + dth /2
  ! pas spatial suivant r
  theta1=theta
  if (theta>pi) theta1 = theta-pi
  if ((theta1 < angleref).or.(theta1>(pi-angleref))) then
    dr = (delta3d(1)/(2*ABS(COS(theta1))) - rayon) / NB_ECH_R
  else
    dr = (delta3d(2)/(2*ABS(SIN(theta1))) - rayon) / NB_ECH_R  
  end if
  !--- Integration suivant r ---
  do i = 0, NB_ECH_R-1
    r = rayon + i * dr + dr/2
    l12 = l12+ dr*r*LOG((r**2+d**2 -2*r     *d*COS(psi-theta)) / &
                    (rayon**2+d**2 -2*rayon*d*COS(psi-theta)))
  end do
end do

inductance_mutuelle_berenger = l12 * dth * mu / (4*PI*delta3d(1)*delta3d(2))  

end function inductance_mutuelle_berenger

! ==========================================================
! == Subroutine inductance_propre_disque                  ==
! == Calcul de l'inductance propre pour une densité       ==
! == de courant répartie sur un disque
! r_disque, rayon : rayon du disque et du fil, dans la
!     meme unité
! mu : perméabilité
! ==========================================================
real function inductance_propre_disque(r_disque,rayon,mu)
implicit none
!=== Declaration des arguments ===
real, intent(in)    :: r_disque, rayon, mu
!=== Declaration des variables locales ===

!=== debut des traitements ===
! version avec la surface de la section du fil
inductance_propre_disque = (mu/(2*PI)) * (LOG(r_disque/rayon) - 0.5 + &
                                    rayon**2 / (2 *r_disque**2) )
!inductance_propre_disque = (mu/(2*PI)) * (LOG(r_disque/rayon) )
! version ou on retire la surface de la section du fil
!inductance_propre_disque = (mu/(2*PI)) * r_disque**2 / (r_disque**2 -rayon**2) * &
!        (LOG(r_disque/rayon) - 0.5 + rayon**2 / (4*r_disque**2) )


end function inductance_propre_disque


! ==========================================================
! == function cp_j_disque                               ==
! == Evalue les cellules touchées par le disque  du fil   ==
! fil : structure décrivant un fil
! r_disque : rayon du disque
! surface_rect : surface de la cellule fdtd
! ==========================================================
subroutine coeff_j_e_disque(fil)
implicit none

!=== Declaration des arguments ===
type(st_unfil), intent(inout) :: fil

!=== Declaration des variables ===
integer  :: i, j, k
!=== Début des traitements ===
k = 1
! Calcul de la surface sur laquelle le courant est distribué
!fil%s_disque = PI* (r_disque**2 - tfil%rayon**2)
fil%s_disque = PI* fil%r_disque**2
do j = fil%vc(1), fil%vc(2)
do i = fil%uc(1), fil%uc(2)
  fil%cp_j(k) = fil%surface(i,j) / (fil%s_disque*fil%surface_rect)
  fil%cp_e(k) = fil%surface(i,j) / (fil%s_disque)
  k = k + 1
end do
end do

end subroutine coeff_j_e_disque

! =============================================================================
! == Subroutine structure_unfil_PML                                          ==
! =============================================================================
subroutine structure_unfil_PML(fil,cpml,dt)
implicit none
!=== Declaration des arguments ===
type(st_unfil), intent(inout) :: fil
type(st_cpml), intent(in) :: cpml
real,intent(in)    :: dt

!=== Declaration des variables locales ===
integer :: i
type(st_fil_segment), pointer :: tseg

!=== Début des traitements ===
if (fil%jonction_inf == JPML) then
  allocate(fil%pml_deb)
  if (fil%dir == X) then
    fil%pml_deb%debI = 1 - (fil%ldeb - cpml%xinf%idebh)
    fil%pml_deb%finI = 1 - (fil%ldeb - cpml%xinf%ifinh)
    fil%pml_deb%debQ = 1 - (fil%ldeb - cpml%xinf%idebe)
    fil%pml_deb%finQ = 1 - (fil%ldeb - cpml%xinf%ifine)
  else if (fil%dir == Y) then
    fil%pml_deb%debI = 1 - (fil%ldeb - cpml%yinf%jdebh)
    fil%pml_deb%finI = 1 - (fil%ldeb - cpml%yinf%jfinh)
    fil%pml_deb%debQ = 1 - (fil%ldeb - cpml%yinf%jdebe)
    fil%pml_deb%finQ = 1 - (fil%ldeb - cpml%yinf%jfine)
  else
    fil%pml_deb%debI = 1 - (fil%ldeb - cpml%zinf%kdebh)
    fil%pml_deb%finI = 1 - (fil%ldeb - cpml%zinf%kfinh)
    fil%pml_deb%debQ = 1 - (fil%ldeb - cpml%zinf%kdebe)
    fil%pml_deb%finQ = 1 - (fil%ldeb - cpml%zinf%kfine)  
  end if
  allocate(fil%pml_deb%yI  (fil%pml_deb%debI:fil%pml_deb%finI))
  allocate(fil%pml_deb%yQ  (fil%pml_deb%debQ:fil%pml_deb%finQ))
  fil%pml_deb%yQ = 0.0
  fil%pml_deb%yI = 0.0
  allocate(fil%pml_deb%cyI1(fil%pml_deb%debI:fil%pml_deb%finI))
  allocate(fil%pml_deb%cyI2(fil%pml_deb%debI:fil%pml_deb%finI))
  
  allocate(fil%pml_deb%cyQ1(fil%pml_deb%debQ:fil%pml_deb%finQ))
  allocate(fil%pml_deb%cyQ2(fil%pml_deb%debQ:fil%pml_deb%finQ))
  if (fil%dir == X) then
    fil%pml_deb%cyI1 = cpml%xinf%absh
    fil%pml_deb%cyI2 = cpml%xinf%absh2
    fil%pml_deb%cyQ1 = cpml%xinf%abse
    fil%pml_deb%cyQ2 = cpml%xinf%abse2
  else if (fil%dir == Y) then
    fil%pml_deb%cyI1 = cpml%yinf%absh
    fil%pml_deb%cyI2 = cpml%yinf%absh2
    fil%pml_deb%cyQ1 = cpml%yinf%abse
    fil%pml_deb%cyQ2 = cpml%yinf%abse2
  else
    fil%pml_deb%cyI1 = cpml%zinf%absh
    fil%pml_deb%cyI2 = cpml%zinf%absh2
    fil%pml_deb%cyQ1 = cpml%zinf%abse
    fil%pml_deb%cyQ2 = cpml%zinf%abse2
  end if

!  do i=fil%pml_deb%debI,fil%pml_deb%finI
!    print '("i,cyI1,cyQ1,cyI2,cyQ2=",I3,4ES15.6e2)',i,fil%pml_deb%cyI1(i), &
!      fil%pml_deb%cyQ1(i),fil%pml_deb%cyI2(i),fil%pml_deb%cyQ2(i)
!  end do
end if
if (fil%jonction_sup == JPML) then
  allocate(fil%pml_fin)
  if (fil%dir == X) then
    fil%pml_fin%debI = fil%nseg - (fil%lfin - cpml%xsup%idebh)
    fil%pml_fin%finI = fil%nseg - (fil%lfin - cpml%xsup%ifinh)
    fil%pml_fin%debQ = fil%nseg - (fil%lfin - cpml%xsup%idebe)
    fil%pml_fin%finQ = fil%nseg - (fil%lfin - cpml%xsup%ifine)
  else if (fil%dir == Y) then
    fil%pml_fin%debI = fil%nseg - (fil%lfin - cpml%ysup%jdebh)
    fil%pml_fin%finI = fil%nseg - (fil%lfin - cpml%ysup%jfinh)
    fil%pml_fin%debQ = fil%nseg - (fil%lfin - cpml%ysup%jdebe)
    fil%pml_fin%finQ = fil%nseg - (fil%lfin - cpml%ysup%jfine)
  else
    fil%pml_fin%debI = fil%nseg - (fil%lfin - cpml%zsup%kdebh)
    fil%pml_fin%finI = fil%nseg - (fil%lfin - cpml%zsup%kfinh)
    fil%pml_fin%debQ = fil%nseg - (fil%lfin - cpml%zsup%kdebe)
    fil%pml_fin%finQ = fil%nseg - (fil%lfin - cpml%zsup%kfine)  
  end if
  allocate(fil%pml_fin%yI(fil%pml_fin%debI:fil%pml_fin%finI))
  allocate(fil%pml_fin%yQ(fil%pml_fin%debQ:fil%pml_fin%finQ))
  fil%pml_fin%yI = 0.0
  fil%pml_fin%yQ = 0.0

  allocate(fil%pml_fin%cyI1(fil%pml_fin%debI:fil%pml_fin%finI))
  allocate(fil%pml_fin%cyI2(fil%pml_fin%debI:fil%pml_fin%finI))
  
  allocate(fil%pml_fin%cyQ1(fil%pml_fin%debQ:fil%pml_fin%finQ))
  allocate(fil%pml_fin%cyQ2(fil%pml_fin%debQ:fil%pml_fin%finQ))
  if (fil%dir == X) then
    fil%pml_fin%cyI1 = cpml%xsup%absh
    fil%pml_fin%cyI2 = cpml%xsup%absh2
    fil%pml_fin%cyQ1 = cpml%xsup%abse
    fil%pml_fin%cyQ2 = cpml%xsup%abse2
  else if (fil%dir == Y) then
    fil%pml_fin%cyI1 = cpml%ysup%absh
    fil%pml_fin%cyI2 = cpml%ysup%absh2
    fil%pml_fin%cyQ1 = cpml%ysup%abse
    fil%pml_fin%cyQ2 = cpml%ysup%abse2
  else
    fil%pml_fin%cyI1 = cpml%zsup%absh
    fil%pml_fin%cyI2 = cpml%zsup%absh2
    fil%pml_fin%cyQ1 = cpml%zsup%abse
    fil%pml_fin%cyQ2 = cpml%zsup%abse2
  end if

!  do i=fil%pml_fin%debI,fil%pml_fin%finI
!    print '("i,cyI1,cyQ1,cyI2,cyQ2=",I3,4ES15.6e2)',i,fil%pml_fin%cyI1(i), &
!      fil%pml_fin%cyQ1(i),fil%pml_fin%cyI2(i),fil%pml_fin%cyQ2(i)
!  end do
end if


end subroutine structure_unfil_PML

! =============================================================================
! == Subroutine structure_fil_noeud_precalcul                                ==
! == Jonction multi-fils avec le formalisme de Holland seulement !           ==
! == Hypothèse : 1 fil par fil !!!!!                                         ==
! =============================================================================
subroutine structure_fil_noeud_precalcul(tf)
!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf
!=== Declaration des variables locales ===
integer :: i,n,d1,d2
integer :: j,k
integer :: nbsegtotal
real    :: capa_noeud,G_noeud
integer, dimension(6) :: nuseg,nufil
integer, dimension(6) :: nbseg
real, dimension(6) :: dl,capaparmetre,conductance

type(st_unfil), pointer :: fil
type(st_fil_segment), pointer :: tseg
type(st_fil_noeud_1seg), pointer :: pns

!=== Début des traitements ===
if (tf%n_noeud == 0) return
allocate(tf%l_noeud(tf%n_noeud))

Print '(/,T5,"----  Description des jonction-noeuds des fils filaires ---")'

!----- recherche des fils concernés par chaque noeud -----
do n=1,tf%n_noeud
  j=0
  nbseg(:) = 0
  do i=1,tf%n_fil
    fil => tf%l_fil(i)
    !print '(T10,"  noeud (",3I4,"), ",A,"  coord (deb,fin) = ",2I6)',tf%l_coord_noeud(1:3,n),TRIM(fil%nom),&
!		    fil%ldeb,fil%lfin
    if ((fil%lfin>=tf%l_coord_noeud(fil%dir,n)) .and. &
        (fil%ldeb<=tf%l_coord_noeud(fil%dir,n))) then
      d1 = PC1(fil%dir)
      d2 = PC2(fil%dir)
      if (tf%l_coord_noeud(d1,n) == fil%coord(d1*2).and. &
          tf%l_coord_noeud(d2,n) == fil%coord(d2*2)) then
        ! le fil touche le noeud
        j=j+1
        nuseg(j) = tf%l_coord_noeud(fil%dir,n)-fil%ldeb+1
        nufil(j) = i
        nbseg(j) = 2
        if ((nuseg(j)==1).or.(nuseg(j)==fil%nseg+1)) nbseg(j) = 1
        print '(T10,"  noeud (",3I4,"), ",A," : nuseg=",I3," ; nbseg=",I2)',&
           tf%l_coord_noeud(1:3,n),TRIM(fil%nom),nuseg(j),nbseg(j)
      end if
    end if
  end do
  nbsegtotal = SUM(nbseg)
  if (nbsegtotal > 6) then
    print '(/,"!!!!!!! Erreur dans le bloc <structure_fil> :  ")'
    print '(/,"  Il y a",I2," segments arrivant sur le noeud n° : ",I4)',nbsegtotal,n
    stop
  end if
  if (j == 0) then
    print '(/,"!!!!!!! Erreur dans le bloc <structure_fil> :  ")'
    print '(/,"  Il y a 0 segment arrivant sur le noeud n° : ",I4)',n
    stop
  end if
  if (nbsegtotal>2) print '(T5,"  Noeud (",3I4,"), nbsegtotal=",I2)',&
                          tf%l_coord_noeud(1:3,n),nbsegtotal

  ! Récupère les pointeurs sur courant et charge pour chaque
  ! noeud, calcul coefficients
  tf%l_noeud(n)%nbseg=nbsegtotal
  allocate(tf%l_noeud(n)%seg(tf%l_noeud(n)%nbseg))
   ! traite tous les segments du noeud
  k = 0;dl(:) =0.0;
  do i=1,j
    k = k+1
    pns =>tf%l_noeud(n)%seg(k)
    fil => tf%l_fil(nufil(i))
    tseg => fil%l_seg(nuseg(i))
    ! Charge non modifiée par l'algo. classique :
    tseg%cq_cur = 0.0 ! coeff nulle uniquement sur le noeud
    tseg%cq_q   = 1.0

    pns%dt_signe = tf%dt ! signe positif par défaut
    capaparmetre(k) = 1/(fil%lp*tseg%v**2)
    dl(k) = fil%dl(nuseg(i))*0.5
    conductance(k) = tseg%sige_eps*capaparmetre(k)*dl(k)
    if (nuseg(i) == 1) then ! premier noeud
      pns%courant => tseg%cur
      pns%charge  => tseg%qc
      if (fil%jonction_inf == CO .or.fil%jonction_inf == JNF .or. fil%jonction_inf==JFF) then
        fil%jonction_inf = JNF
      else
        print '("ERREUR, tentative interdite de conversion vers une jonction JNF")'
        print '("       sur la jonction inf du fil=",A)',trim(fil%nom)
        print '(" Sans doute qu''il s''agit d''une jonction à plus de 2 fils avec une jonction illicite prédéfinie-:)")'
	stop 
      end if
      
    else if (nuseg(i)==fil%nseg+1) then ! dernier noeud
      if (fil%jonction_sup == CO .or.fil%jonction_sup == JNF .or. fil%jonction_sup==JFF) then
        fil%jonction_sup = JNF
      else
        print '("ERREUR, tentative interdite de conversion vers une jonction JNF")'
        print '("       sur la jonction sup du fil=",A)',trim(fil%nom)
        print '(" Sans doute qu''il s''agit d''une jonction à plus de 2 fils avec une jonction illicite prédéfinie-:)")'
	stop 
      end if
      
      tseg => fil%l_seg(nuseg(i)-1)
      pns%courant => tseg%cur
      nullify(tseg%qc1)
      allocate(tseg%qc1)
      tseg%flag_noeud = .true.
      pns%charge => tseg%qc1
      pns%charge = 0; ! c'est une nouvelle charge
      pns%dt_signe = -tf%dt !changement de signe
!    print '(/," i=",I2,"; !!pns%dt_signe =",ES15.6E2)',i,pns%dt_signe

      capaparmetre(k) = 1/(fil%lp*tseg%v**2)
      dl(k) = fil%dl(nuseg(i)-1)*0.5
      conductance(k) = tseg%sige_eps*capaparmetre(k)*dl(k)
    else ! 2 charges à calculer, gestion de 2 segments consécutifs
      ! 1er segment, celui déjà pointé, après le noeud
      ! reste les pointeurs courant et charge à définir :
      pns%courant => tseg%cur
      pns%charge  => tseg%qc

      ! 2ème segment, celui avant le noeud :
      k=k+1
      tseg => fil%l_seg(nuseg(i)-1)
      pns =>tf%l_noeud(n)%seg(k)
      pns%courant => tseg%cur
      nullify(tseg%qc1)
      allocate(tseg%qc1)
      tseg%flag_noeud = .true.
      pns%charge => tseg%qc1
      pns%charge = 0;
      pns%dt_signe = -tf%dt !changement de signe
      capaparmetre(k) = 1/(fil%lp*tseg%v**2)
      dl(k) = fil%dl(nuseg(i)-1)*0.5
      conductance(k) = tseg%sige_eps*capaparmetre(k)*dl(k)
    end if
  end do

  ! Somme de toutes les capacités :
  capa_noeud = 0.0
  G_noeud = 0.0
  do i=1,nbsegtotal
    capa_noeud = capa_noeud + dl(i)*capaparmetre(i)
    G_noeud = G_noeud+conductance(i)
  end do

  ! Calcul du coefficient pour la charge :
  do i=1,nbsegtotal
    pns =>tf%l_noeud(n)%seg(i)
    pns%cq   = (capaparmetre(i) / capa_noeud) /&
               (1.0+tf%dt*G_noeud/(2*capa_noeud))
	       
    pns%cq_q = (1.0-tf%dt*G_noeud/(2*capa_noeud))/&
	       (1.0+tf%dt*G_noeud/(2*capa_noeud))
    print '(" i=",I2,"; pns%cq,pns%cq_q =",2ES15.6E2)',i,pns%cq,pns%cq_q
    !print '(" i=",I2,"; pns%dt_signe =",ES15.6E2)',i,pns%dt_signe
  end do

end do

deallocate(tf%l_coord_noeud); NULLIFY(tf%l_coord_noeud);
end subroutine structure_fil_noeud_precalcul


! ==========================================================
! == Subroutine structure_fil_alias_champ                 ==
! == Alias vers les composantes du champ electrique       ==
! ==========================================================
subroutine structure_fil_alias_champ(tf,tc,cpml)
implicit none
!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf
type(st_champ), intent(in) :: tc
type(st_cpml), intent(in) :: cpml

!=== Declaration des variables locales ===
integer :: i,j,k,n
type(st_unfil), pointer :: fil
integer, dimension(6) :: coord

!=== debut des traitements ===
! boucle sur tous les fils
do n=1, tf%n_fil
  fil => tf%l_fil(n)
  call structure_unfil_PML(fil,cpml,tf%dt)

  if (fil%flag_disque) then
    call structure_fil_alias_fil_champ(fil, tc)

  end if
  coord(:) = fil%coord(:)
  ! Alias vers les composantes du champ sous l'influence du fil
  if (fil%dir == X) then
    fil%e1 => tc%ex(fil%ldeb_frontiere:fil%lfin_frontiere,coord(YMIN),  coord(ZMIN)  )
    fil%e2 => tc%ex(fil%ldeb_frontiere:fil%lfin_frontiere,coord(YMIN)+1,coord(ZMIN)  )
    fil%e3 => tc%ex(fil%ldeb_frontiere:fil%lfin_frontiere,coord(YMIN),  coord(ZMIN)+1)
    fil%e4 => tc%ex(fil%ldeb_frontiere:fil%lfin_frontiere,coord(YMIN)+1,coord(ZMIN)+1)
  else if (fil%dir == Y) then
    fil%e1 => tc%ey(coord(XMIN),  fil%ldeb_frontiere:fil%lfin_frontiere,coord(ZMIN)  )
    fil%e2 => tc%ey(coord(XMIN),  fil%ldeb_frontiere:fil%lfin_frontiere,coord(ZMIN)+1)
    fil%e3 => tc%ey(coord(XMIN)+1,fil%ldeb_frontiere:fil%lfin_frontiere,coord(ZMIN)  )
    fil%e4 => tc%ey(coord(XMIN)+1,fil%ldeb_frontiere:fil%lfin_frontiere,coord(ZMIN)+1)
  else
    fil%e1 => tc%ez(coord(XMIN),  coord(YMIN),  fil%ldeb_frontiere:fil%lfin_frontiere)
    fil%e2 => tc%ez(coord(XMIN)+1,coord(YMIN),  fil%ldeb_frontiere:fil%lfin_frontiere)
    fil%e3 => tc%ez(coord(XMIN),  coord(YMIN)+1,fil%ldeb_frontiere:fil%lfin_frontiere)
    fil%e4 => tc%ez(coord(XMIN)+1,coord(YMIN)+1,fil%ldeb_frontiere:fil%lfin_frontiere)
  end if
end do
end subroutine structure_fil_alias_champ

! ==========================================================
! == Subroutine structure_fil_alias_fil_champ             ==
! == Alias vers les composantes du champ electrique       ==
! pour un fil dont la densité de courant se rapporte à un
! disque.
! ==========================================================
subroutine structure_fil_alias_fil_champ(fil, tc)
implicit none

!=== Declaration des arguments ===
type(st_unfil), intent(inout) :: fil
type(st_champ), intent(in) :: tc

!=== Declaration des variables locales ===
integer :: i,j,k
integer, dimension(6) :: coord

!=== debut des traitements ===
  ! Alias vers les composantes du champ sous l'influence du disque du fil
coord(:) = fil%coord(:)
!write (100,*)
allocate(fil%p(fil%nbcomposante))

  k = 1
!  write (100,*) 'l=',l,'  deb=',fil%ldeb,'  fin=',fil%lfin
do j = fil%vc(1), fil%vc(2)
do i = fil%uc(1), fil%uc(2)
  if (fil%dir == X) then
    fil%p(k)%e => tc%ex(fil%ldeb_frontiere:fil%lfin_frontiere,i,j)

  else if (fil%dir == Y) then
    fil%p(k)%e => tc%ey(j,fil%ldeb_frontiere:fil%lfin_frontiere,i)
  else
    fil%p(k)%e => tc%ez(i,j,fil%ldeb_frontiere:fil%lfin_frontiere)
  end if
  k = k+1
end do
end do

end subroutine structure_fil_alias_fil_champ


! ==========================================================
! == Subroutine structure_fil_affichage                   ==
! ==                                                      ==
! == Affichage des caractéristiques des fils filaires   ==
! ==========================================================
subroutine structure_fil_affichage(tf)

implicit none

!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf
character(LEN=LEN_MOT_CLE), dimension(0:7)  :: nomextr
character, dimension(1:3)  :: ch_dir

!=== Declaration des variables locales ===
type(st_unfil), pointer :: fil
integer :: i,j,k

!=== debut des traitements ===
nomextr(CO) = 'circuit-ouvert'
nomextr(CC) = 'court-circuit'
nomextr(JPML) = 'Jonction fil-PML'
nomextr(JFF) = 'Jonction fil-fil'
nomextr(JNF) = 'Jonction noeud multi-fils'
nomextr(JFC) = 'Jonction Fil-Coaxial'
nomextr(JFQ) = 'Jonction Fil-Quadripole-Fil'
nomextr(JFI) = 'Jonction Fil-Fil isolé'

ch_dir(1) = 'X'
ch_dir(2) = 'Y'
ch_dir(3) = 'Z'
print '("================== STRUCTURE FIL ===================")'
do i=1, tf%n_fil
  fil => tf%l_fil(i)
  print '(t2, "Nom du fil :", A)',TRIM(fil%nom)

  print '(t5, "Extrémite Initiale : ", A)',TRIM(nomextr(fil%jonction_inf))
  print '(t5, "Extrémite Finale   : ", A)',TRIM(nomextr(fil%jonction_sup))
  print '(t5, "Coordonnées relatives segment fil = ", 2I5)',fil%coord_rel(1:2)
  print '(t5, "Inductance fil = ", ES13.5E2)',fil%lp
  print '(t5, "Direction fil : ", A)',ch_dir(fil%dir)

  if (fil%flag_disque) then
    print '(t5, "Surface réelle pour le calcul de J : ",ES11.4E2)', fil%s_disque
    print '(t5, "Surface distribuée pour le calcul de J : ",ES11.4E2)', fil%s_discret
    print '(t5, "Permittivité moyenne : ",ES11.4E2)', fil%eps_moy/EPSILON_0
    print '(t5, "Nombre de composantes de E concernées : ",I2)', fil%nbcomposante 
    print '(t5, "Position transverse des composantes de E : ",I4," à",I4," ;", &
                & I4," à",I4)', fil%uc(1),fil%uc(2),fil%vc(1),fil%vc(2)
    print '(t5, "Surface du disque contenu par chaque composante : ")'
    do k =fil%vc(1), fil%vc(2)
      write (*,'( t5,10ES11.4E2)') fil%surface(:,k)        
    end do
  end if
end do
print '("===================================================")'
print *

end subroutine structure_fil_affichage


! ==========================================================
! == Subroutine structure_fil_calcul_Qjonction            ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine structure_fil_calcul_Qjonction(tf)
implicit none

!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf

!=== Declaration des variables locales ===
integer :: i,k
real :: somi
type(st_fil_noeud_1seg), pointer :: pns
!=== debut des traitements ===

!--- Traitement des jonctions noeuds multi-fils
!!!$OMP PARALLEL PRIVATE(pns,somi,i,k) IF (tf%n_noeud>50)
!!!$OMP DO SCHEDULE(DYNAMIC,50)
!CDIR NOVECTOR
do k=1, tf%n_noeud
  somi=0
!CDIR NOVECTOR
  do i=1, tf%l_noeud(k)%nbseg
    pns => tf%l_noeud(k)%seg(i)
    somi = somi+ pns%courant * pns%dt_signe
  end do
!CDIR NOVECTOR
  do i=1, tf%l_noeud(k)%nbseg
    pns => tf%l_noeud(k)%seg(i)
    pns%charge = pns%charge*pns%cq_q -somi * pns%cq
  end do
end do
!!!$OMP END DO
!!!$OMP END PARALLEL 

end subroutine structure_fil_calcul_Qjonction

! ==========================================================
! == Subroutine structure_fil_calcul_Q_E                  ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine structure_fil_calcul_Q_E(tf)
implicit none

!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf

!=== Declaration des variables locales ===
integer :: i,j,k,l
real    :: coeff,cur
real(dp) :: I1,I2,I3,V1,V2,V3
type(st_fil_segment), pointer  :: tseg, tseg_m1
type(st_unfil), pointer :: fil
type(st_fil_coaxial), pointer :: coax
type(st_jonction_fil_coax),pointer :: pjf

!=== DEBUT DES TRAITEMENTS ===


!====== Traitement de la jonction Fil-coaxial ========
do i=1,tf%n_jfc
    pjf => tf%l_jfc(i)
    pjf%Q1 = pjf%Q1 + pjf%cq1(1)*pjf%I1 + pjf%cq1(2)*pjf%I2 + pjf%cq1(3)*pjf%I3
    pjf%Q2 = pjf%Q2 + pjf%cq2(1)*pjf%I1 + pjf%cq2(2)*pjf%I2 + pjf%cq2(3)*pjf%I3
    pjf%Q3 = pjf%Q3 + pjf%cq3   *pjf%I3
end do

!================== boucle sur tous les fils ========================
!!!$OMP PARALLEL PRIVATE(fil,tseg,tseg_m1,coax,i,j,k,l,coeff) IF (tf%n_fil>50)
!!!$OMP DO SCHEDULE(DYNAMIC,50)
do k=1, tf%n_fil
  fil => tf%l_fil(k)

  !  extremite en court circuit, pas de traitement specifique
  !  extremite en circuit ouvert :
  if (fil%jonction_inf == CO) then
    tseg => fil%l_seg(1)
    tseg%qc = tseg%cq_q*tseg%qc - 2*tseg%cq_cur * tseg%cur !ajout facteur
  end if
  if (fil%jonction_sup == CO) then
    tseg => fil%l_seg(fil%nseg+1)
    tseg_m1 => fil%l_seg(fil%nseg)
    tseg%qc = tseg%cq_q*tseg%qc + 2*tseg%cq_cur * tseg_m1%cur
  end if

  !------------------- CALCUL DE Q A L'INSTANT n ------------------------------
  do j=fil%coord_rel(1)+2, fil%coord_rel(2)-1
    tseg => fil%l_seg(j)
    tseg_m1 => fil%l_seg(j-1)
    tseg%qc = tseg%cq_q*tseg%qc - tseg%cq_cur * (tseg%cur - tseg_m1%cur)
  end do

  if (fil%flag_coaxial) then
    coax => fil%t_coax
    do j=fil%coord_rel(1)+2, fil%coord_rel(2)-1
      coax%Q(j) = coax%Q(j)*coax%cQ1(j) - coax%cQ2(j)*(coax%I(j)-coax%I(j-1)) 
    end do
    if (fil%jonction_inf == CO) then
      coax%Q(1) = coax%Q(1)*coax%cQ1(1) - 2*coax%cQ2(1)*coax%I(1)
    end if
    if (fil%jonction_sup == CO) then
      coax%Q(fil%nseg+1) = coax%Q(fil%nseg+1)*coax%cQ1(fil%nseg+1) + &
                            2*coax%cQ2(fil%nseg+1)*coax%I(fil%nseg)
    end if
  end if
end do


!--------------------------------------------------------------------

!!!$OMP END DO
!!!$OMP END PARALLEL 

!!!$OMP PARALLEL PRIVATE(fil,tseg,tseg_m1,coax,i,j,k,l,coeff) IF (tf%n_fil>50)
!!!$OMP DO SCHEDULE(DYNAMIC,50)
do k=1, tf%n_fil
  fil => tf%l_fil(k)

  !------------------------- JONCTION PML  ------------------------------------
  if (fil%jonction_inf == JPML) then
    do i=fil%pml_deb%debQ,fil%pml_deb%finQ
      fil%pml_deb%yQ(i) = fil%pml_deb%yQ(i)      * fil%pml_deb%cyQ1(i) + &
         (fil%l_seg(i)%cur - fil%l_seg(i-1)%cur) * fil%pml_deb%cyQ2(i)  
      
      fil%l_seg(i)%qc = fil%l_seg(i)%qc - fil%l_seg(i)%cq_cur * fil%pml_deb%yQ(i)
    end do
  end if
  
  if (fil%jonction_sup == JPML) then
    do i=fil%pml_fin%debQ,fil%pml_fin%finQ
      fil%pml_fin%yQ(i) = fil%pml_fin%yQ(i)      * fil%pml_fin%cyQ1(i) + &
         (fil%l_seg(i)%cur - fil%l_seg(i-1)%cur) * fil%pml_fin%cyQ2(i)  
      
      fil%l_seg(i)%qc = fil%l_seg(i)%qc - fil%l_seg(i)%cq_cur * fil%pml_fin%yQ(i)
    end do
  end if
  !-----------------------------------------------------------------------------
  
  !------ Traitement de la capacité  -----------
  do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
    tseg => fil%l_seg(j)
    if (tseg%flag_c) then
      tseg%vc = tseg%vc + tseg%cur * tseg%c_vc
    end if
  end do
end do
!!!$OMP END DO
!!!$OMP END PARALLEL 


!!$OMP PARALLEL PRIVATE(fil,tseg,tseg_m1,coax,i,j,l,coeff) IF (tf%n_fil>50)
!!$OMP DO SCHEDULE(DYNAMIC,50)
do k=1, tf%n_fil
  fil => tf%l_fil(k)
  ! Mise à jour des E à l'instant n avec I de n-1/2
  if (fil%flag_disque) then
    do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
      tseg => fil%l_seg(j)
      coeff = tseg%dt_epsilon*tseg%cur
      i = j-fil%coord_rel(1)
      do l=1, fil%nbcomposante
	fil%p(l)%e(i) = fil%p(l)%e(i) - coeff*fil%cp_j(l)
      end do
    end do

  else
    do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
      i = j-fil%coord_rel(1)
      tseg => fil%l_seg(j)
      fil%e1(i) = fil%e1(i) - tseg%cp_j(1) * tseg%cur
      fil%e2(i) = fil%e2(i) - tseg%cp_j(2) * tseg%cur
      fil%e3(i) = fil%e3(i) - tseg%cp_j(3) * tseg%cur
      fil%e4(i) = fil%e4(i) - tseg%cp_j(4) * tseg%cur

    end do
    !if (fil%flag_plan_masse) then
    !  do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
!	i = j-fil%coord_rel(1)
!	cur = fil%I_avant(fil%ir_pm,j) * (1-fil%p_pm)+fil%I_avant(fil%ir_pm-1,j) * fil%p_pm
	!cur = fil%l_seg(j)%cur
!	fil%e1(i) = fil%e1(i) + tseg%cp_j(1) * cur
!	fil%e2(i) = fil%e2(i) + tseg%cp_j(2) * cur
!	fil%e3(i) = fil%e3(i) + tseg%cp_j(3) * cur
!	fil%e4(i) = fil%e4(i) + tseg%cp_j(4) * cur
 !     end do
  !  end if

  end if
end do
!!$OMP END DO
!!$OMP END PARALLEL 

end subroutine structure_fil_calcul_Q_E


! ==========================================================
! == Subroutine structure_fil_calcul_Emoy_I               ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine structure_fil_calcul_Emoy_I(tf,iter)
implicit none

!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf
integer, intent(in)  :: iter

!=== Declaration des variables locales ===
integer :: i,j,k,l
type(st_fil_segment), pointer  :: tseg
type(st_unfil), pointer :: fil
type(st_fil_coaxial), pointer :: coax

!=== debut des traitements ===


!------------------------------------------------------------------------
! Le champ E doit etre completement mise à jour avant son utilisation du
! boucle sur tous les fils
!!$OMP PARALLEL PRIVATE(fil,tseg,i,j,l) IF (tf%n_fil>50)
!!$OMP DO SCHEDULE(DYNAMIC,50)
do k=1, tf%n_fil
  fil => tf%l_fil(k)
  if (fil%flag_disque .and. (.not.fil%moyenne_e_rectangle)) then
    do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
      tseg => fil%l_seg(j)
      tseg%emoy = 0.0
      i = j-fil%coord_rel(1)
      do l=1, fil%nbcomposante
          tseg%emoy = tseg%emoy + fil%p(l)%e(i) * fil%cp_e(l)
      end do
    end do

  else
    do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
      tseg => fil%l_seg(j)
      i = j-fil%coord_rel(1)
      ! calcul de emoyen de chaque fil
      tseg%emoy =    fil%e1(i) * fil%cp_emoy(1) + &
                     fil%e2(i) * fil%cp_emoy(2) + &
                     fil%e3(i) * fil%cp_emoy(3) + &
                     fil%e4(i) * fil%cp_emoy(4)
    end do

  end if
end do
!!$OMP END DO
!!$OMP END PARALLEL 

!------------------------------------------------------------------------
!!$OMP PARALLEL PRIVATE(fil,tseg,i,j,coax) IF (tf%n_fil>50)
!!$OMP DO SCHEDULE(DYNAMIC,50)
!! Calcul de I à l'instant n+1/2
do k=1, tf%n_fil
  fil => tf%l_fil(k)
  do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
    tseg => fil%l_seg(j)
    tseg%cur = tseg%c_cur*tseg%cur - tseg%c_qc*(tseg%qc1-tseg%qc) + tseg%c_emoy*tseg%emoy
  end do

  ! Calcul du generateur de tension et mise à jour du courant
  if (fil%flag_gv) then
    j=fil%gv%seg
    tseg => fil%l_seg(j)
    tseg%vsrc = fil%gv%e0 * fil%tabgene(iter)
    tseg%cur = tseg%cur + tseg%c_vsrc*tseg%vsrc
  end if

  ! Présence d'une capacité
  do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
    tseg => fil%l_seg(j)
    if (tseg%flag_c) then
      tseg%cur = tseg%cur - tseg%c_vsrc*tseg%vc
    end if
  end do
  
  !-------------------- Calcul propa sur lame du cable coaxial -----------------
  if (fil%flag_coaxial) then
    coax => fil%t_coax
    do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
      coax%I(j) = coax%I(j)*coax%cI1(j) - coax%cIq(j)*(coax%Q(j+1)-coax%Q(j)) 
    end do
    if (coax%flag_gv) then
      j=fil%gv%seg
      coax%I(j) = coax%I(j) + coax%cIvsrc(j) * fil%gv%e0 * fil%tabgene(iter)
    end if
  end if
end do ! fin boucle sur les fils
!!$OMP END DO
!!$OMP END PARALLEL 


  !----------------------------- JONCTION PML  --------------------------------
!!!$OMP PARALLEL PRIVATE(fil,i,j) IF (tf%n_fil>50)
!!!$OMP DO SCHEDULE(DYNAMIC,50)
do k=1, tf%n_fil
  fil => tf%l_fil(k)
  if (fil%jonction_inf == JPML) then
    do i=fil%pml_deb%debI,fil%pml_deb%finI      
      fil%pml_deb%yI(i) = fil%pml_deb%yI(i)      * fil%pml_deb%cyI1(i) + &
            (fil%l_seg(i)%qc1 - fil%l_seg(i)%qc) * fil%pml_deb%cyI2(i)  
      
      fil%l_seg(i)%cur = fil%l_seg(i)%cur - fil%l_seg(i)%c_qc * fil%pml_deb%yI(i)
    end do
  end if
  if (fil%jonction_sup == JPML) then
    do i=fil%pml_fin%debI,fil%pml_fin%finI
    
      fil%pml_fin%yI(i) = fil%pml_fin%yI(i)      * fil%pml_fin%cyI1(i) + &
            (fil%l_seg(i)%qc1 - fil%l_seg(i)%qc) * fil%pml_fin%cyI2(i)
      
      fil%l_seg(i)%cur = fil%l_seg(i)%cur - fil%l_seg(i)%c_qc * fil%pml_fin%yI(i)
    end do
  end if
  !----------------------------------------------------------------------------
  !if (fil%flag_plan_masse) then
  !  do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
  !    do i=2,fil%ir_pm
  !      fil%I_avant(i,j) = fil%I_avant(i-1,j)
  !    end do 
  !    fil%I_avant(1,j) = fil%l_seg(j)%cur
  !  end do
  !end if  
end do ! fin boucle sur les fils
!!!$OMP END DO
!!!$OMP END PARALLEL 



end subroutine structure_fil_calcul_Emoy_I

! ==========================================================
! == Subroutine structure_fil_calcul_Emoy_I               ==
! == Calcul pendant les itérations temporelles            ==
! ==========================================================
subroutine structure_fil_calcul_Emoy_I_v0(tf,iter)
implicit none

!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf
integer, intent(in)  :: iter

!=== Declaration des variables locales ===
integer :: i,j,k,l
type(st_unfil), pointer :: fil
!type(st_fil_segment), pointer  :: tseg

!=== debut des traitements ===


!------------------------------------------------------------------------
! Le champ E doit etre completement mise à jour avant son utilisation du
! boucle sur tous les fils
!$OMP PARALLEL PRIVATE(i,j,l) IF (tf%n_fil>50)
!$OMP DO SCHEDULE(DYNAMIC,50)
do k=1, tf%n_fil
  if (tf%l_fil(k)%flag_disque .and. (.not.tf%l_fil(k)%moyenne_e_rectangle)) then
    do j=tf%l_fil(k)%coord_rel(1)+1, tf%l_fil(k)%coord_rel(2)-1
      tf%l_fil(k)%l_seg(j)%emoy = 0.0
      i = j-tf%l_fil(k)%coord_rel(1)
      do l=1, tf%l_fil(k)%nbcomposante
          tf%l_fil(k)%l_seg(j)%emoy = tf%l_fil(k)%l_seg(j)%emoy + &
	                              tf%l_fil(k)%p(l)%e(i) * tf%l_fil(k)%cp_e(l)
      end do
    end do

  else
    do j=tf%l_fil(k)%coord_rel(1)+1, tf%l_fil(k)%coord_rel(2)-1
      i = j-tf%l_fil(k)%coord_rel(1)
      ! calcul de emoyen de chaque fil
      tf%l_fil(k)%l_seg(j)%emoy =   &
                     tf%l_fil(k)%e1(i) * tf%l_fil(k)%cp_emoy(1) + &
                     tf%l_fil(k)%e2(i) * tf%l_fil(k)%cp_emoy(2) + &
                     tf%l_fil(k)%e3(i) * tf%l_fil(k)%cp_emoy(3) + &
                     tf%l_fil(k)%e4(i) * tf%l_fil(k)%cp_emoy(4)
    end do

  end if
end do
!$OMP END DO
!$OMP END PARALLEL 

!------------------------------------------------------------------------
!$OMP PARALLEL PRIVATE(i,j) IF (tf%n_fil>50)
!$OMP DO SCHEDULE(DYNAMIC,50)
!! Calcul de I à l'instant n+1/2
do k=1, tf%n_fil
  do j=tf%l_fil(k)%coord_rel(1)+1, tf%l_fil(k)%coord_rel(2)-1
    tf%l_fil(k)%l_seg(j)%cur = tf%l_fil(k)%l_seg(j)%c_cur * tf%l_fil(k)%l_seg(j)%cur - &
                               tf%l_fil(k)%l_seg(j)%c_qc * (tf%l_fil(k)%l_seg(j)%qc1-tf%l_fil(k)%l_seg(j)%qc) +&
			       tf%l_fil(k)%l_seg(j)%c_emoy *tf%l_fil(k)%l_seg(j)%emoy
  end do

  ! Calcul du generateur de tension et mise à jour du courant
  if (tf%l_fil(k)%flag_gv) then
    j=tf%l_fil(k)%gv%seg
    tf%l_fil(k)%l_seg(j)%vsrc = tf%l_fil(k)%gv%e0 * tf%l_fil(k)%tabgene(iter)
    tf%l_fil(k)%l_seg(j)%cur  = tf%l_fil(k)%l_seg(j)%cur + &
                                tf%l_fil(k)%l_seg(j)%c_vsrc*tf%l_fil(k)%l_seg(j)%vsrc
  end if

  ! Présence d'une capacité
  do j=tf%l_fil(k)%coord_rel(1)+1, tf%l_fil(k)%coord_rel(2)-1
    if (tf%l_fil(k)%l_seg(j)%flag_c) then
      tf%l_fil(k)%l_seg(j)%cur = tf%l_fil(k)%l_seg(j)%cur - &
                                 tf%l_fil(k)%l_seg(j)%c_vsrc*tf%l_fil(k)%l_seg(j)%vc
    end if
  end do
  
  !-------------------- Calcul propa sur lame du cable coaxial -----------------
  if (tf%l_fil(k)%flag_coaxial) then
    do j=tf%l_fil(k)%coord_rel(1)+1, tf%l_fil(k)%coord_rel(2)-1
      tf%l_fil(k)%t_coax%I(j) = tf%l_fil(k)%t_coax%I(j)*tf%l_fil(k)%t_coax%cI1(j) - &
                                tf%l_fil(k)%t_coax%cIq(j)*&
			       (tf%l_fil(k)%t_coax%Q(j+1)-tf%l_fil(k)%t_coax%Q(j)) 
    end do
    if (tf%l_fil(k)%t_coax%flag_gv) then
      j=tf%l_fil(k)%gv%seg
      tf%l_fil(k)%t_coax%I(j) = tf%l_fil(k)%t_coax%I(j) + tf%l_fil(k)%t_coax%cIvsrc(j) * &
                              tf%l_fil(k)%gv%e0 * tf%l_fil(k)%tabgene(iter)
    end if
  end if
end do ! fin boucle sur les fils
!$OMP END DO
!$OMP END PARALLEL 

  !----------------------------- JONCTION PML  --------------------------------
!!!$OMP PARALLEL PRIVATE(fil,i,j) IF (tf%n_fil>50)
!!!$OMP DO SCHEDULE(DYNAMIC,50)
do k=1, tf%n_fil
  fil => tf%l_fil(k)
  if (fil%jonction_inf == JPML) then
    do i=fil%pml_deb%debI,fil%pml_deb%finI      
      fil%pml_deb%yI(i) = fil%pml_deb%yI(i)      * fil%pml_deb%cyI1(i) + &
            (fil%l_seg(i)%qc1 - fil%l_seg(i)%qc) * fil%pml_deb%cyI2(i)  
      
      fil%l_seg(i)%cur = fil%l_seg(i)%cur - fil%l_seg(i)%c_qc * fil%pml_deb%yI(i)
    end do
  end if
  if (fil%jonction_sup == JPML) then
    do i=fil%pml_fin%debI,fil%pml_fin%finI
    
      fil%pml_fin%yI(i) = fil%pml_fin%yI(i)      * fil%pml_fin%cyI1(i) + &
            (fil%l_seg(i)%qc1 - fil%l_seg(i)%qc) * fil%pml_fin%cyI2(i)
      
      fil%l_seg(i)%cur = fil%l_seg(i)%cur - fil%l_seg(i)%c_qc * fil%pml_fin%yI(i)
    end do
  end if
  !----------------------------------------------------------------------------
  !if (fil%flag_plan_masse) then
  !  do j=fil%coord_rel(1)+1, fil%coord_rel(2)-1
  !    do i=2,fil%ir_pm
  !      fil%I_avant(i,j) = fil%I_avant(i-1,j)
  !    end do 
  !    fil%I_avant(1,j) = fil%l_seg(j)%cur
  !  end do
  !end if

end do ! fin boucle sur les fils
!!!$OMP END DO
!!!$OMP END PARALLEL 



end subroutine structure_fil_calcul_Emoy_I_v0

! ==========================================================
! == Subroutine structure_fil_deallocate                  ==
! == Libération de la mémoire vive allouée                ==
! ==========================================================
subroutine structure_fil_deallocate(tf)
implicit none

!=== Declaration des arguments ===
type(st_structure_fil), intent(inout) :: tf

!=== Declaration des variables locales ===
integer :: i,k,n,ok

type(st_unfil),pointer :: fil
type(st_fil_segment), pointer :: tseg
type(st_fil_noeud_1seg),pointer :: fns
!== DEBUT DES TRAITEMENTS ==

! Certaines variables sont libérérées dans structure_fil_precalcul
!------- LIBERATION MEMOIRE SUR CHAQUE FIL ---------------
do n=1, tf%n_fil
  fil => tf%l_fil(n)
  
  !--- Mémoire sur chaque segment de fil ---
  do k=fil%coord_rel(1),fil%coord_rel(2)
    tseg => fil%l_seg(k)
    if (ASSOCIATED(tseg%vc)) then
       deallocate(tseg%vc,tseg%c_vc)
    end if
    deallocate(tseg%vsrc,tseg%emoy)
    deallocate(tseg%qc,tseg%cur)
    if (tseg%flag_noeud) deallocate(tseg%qc1)
  end do
  
  deallocate(fil%l_seg)

  !--- Mémoire pour la distribution sur un disque ---
  if (fil%flag_disque) then
    deallocate(fil%cp_j,fil%cp_e)
    deallocate(fil%surface)
    do i=1,fil%nbcomposante
      nullify(fil%p(i)%e)
    end do
    deallocate(fil%p)
  end if

  !----------- Concernant les PML --------
  if (fil%jonction_inf == JPML) then
    deallocate(fil%pml_deb%cyI1,fil%pml_deb%cyI2)
    deallocate(fil%pml_deb%cyQ1,fil%pml_deb%cyQ2)
    deallocate(fil%pml_deb%yI,fil%pml_deb%yQ)
    DEALLOCATE(fil%pml_deb)
  end if
  if (fil%jonction_sup == JPML) then
    deallocate(fil%pml_fin%cyI1,fil%pml_fin%cyI2)
    deallocate(fil%pml_fin%cyQ1,fil%pml_fin%cyQ2)
    deallocate(fil%pml_fin%yI,fil%pml_fin%yQ)
    DEALLOCATE(fil%pml_fin)
  end if
  !---------------------------------------------
  deallocate(fil%dl)
  NULLIFY(fil%e1); NULLIFY(fil%e3)
  NULLIFY(fil%e2); NULLIFY(fil%e4)
  
    
  if (fil%flag_gv) then
    deallocate(fil%tabgene)
    deallocate(fil%gv)
  end if
  
  if (fil%flag_coaxial) then
    deallocate(fil%t_coax%I,fil%t_coax%Q)
    deallocate(fil%t_coax%cI1,fil%t_coax%cIq,fil%t_coax%cIvsrc)
    deallocate(fil%t_coax%cQ1,fil%t_coax%cQ2)
    deallocate(fil%t_coax)
  end if
end do
deallocate(tf%l_fil)

!------- LIBERATION MEMOIRE DES OBJETS ---------------
do n=1, tf%n_objet
  deallocate(tf%l_objet(n)%nom_fil)
  do i=1,tf%l_objet(n)%nbfil
    NULLIFY(tf%l_objet(n)%p(i)%fil)
  end do
  deallocate(tf%l_objet(n)%p)
end do
deallocate(tf%l_objet,STAT=ok)

!------- LIBERATION MEMOIRE DES OBJETS ---------------
do n=1, tf%n_noeud
  do i=1,tf%l_noeud(n)%nbseg
    NULLIFY(tf%l_noeud(n)%seg(i)%courant)
    NULLIFY(tf%l_noeud(n)%seg(i)%charge)
  end do 
  deallocate(tf%l_noeud(n)%seg)
end do
deallocate(tf%l_noeud,STAT=ok)

do n=1,tf%n_jfc
!  nullify()
end do
deallocate(tf%l_jfc,STAT=ok)


end subroutine structure_fil_deallocate


end module mfdtd_structure_fil


