!/////////////////////////////////////////////////////////////////////////////
!// Name:        mfdtd_temps_cpu.f90
!// Purpose:     Evaluation du temps CPU des différentes étapes de calcul
!                   et affichage
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :  Octobre -  2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! ==========================================================
! == Module module_temps_cpu                              ==
! == Evaluation des temps de calcul                       ==
! == Type st_temps_cpu                                    ==
! ==========================================================
module mfdtd_temps_cpu
use m_constantes
implicit none

! ==========================================================
! == Type temps_cpu                            ==
! ==========================================================
type st_temps_cpu
  real  :: t_cpu
  real  :: t_debut,t_fin
  real  :: t_calcul_e, t_frontiere_e, t_modif_excite_e
  real  :: t_calcul_h, t_frontiere_h, t_modif_excite_h
  real  :: t_calcul_annexe
  real  :: t_pre_calcul,t_post_calcul
  real  :: t_commE,t_commH ! pour MPI seulement
end type st_temps_cpu
 
contains

! ==========================================================
! == Subroutine temps_cpu_initdefaut                      ==
! == Initialisation par defaut des variables              ==
! ==========================================================	
subroutine temps_cpu_initdefaut(tc)

type(st_temps_cpu), intent(inout) :: tc
character(LEN=15)      :: ladate, heure, zone
integer, dimension(8) :: dh

call date_and_time(ladate,heure,zone,dh)

if (rang==RANG_MAITRE_AFF) &
  print '(" DATE = ",A,"/",A,"/",A,"  -  HEURE = ",A,"H",A,"mn",A,"s")',ladate(7:8),ladate(5:6),ladate(1:4),&
                                                 heure(1:2),heure(3:4),heure(5:6)

tc%t_cpu = 0
call CPU_TIME(tc%t_debut)
tc%t_fin=0.0
tc%t_calcul_e = 0
tc%t_calcul_h = 0
tc%t_frontiere_e = 0
tc%t_frontiere_h = 0
tc%t_modif_excite_e = 0
tc%t_modif_excite_h = 0
tc%t_calcul_annexe = 0
tc%t_pre_calcul = 0
tc%t_post_calcul = 0
  
end subroutine temps_cpu_initdefaut

! ==========================================================
! == Subroutine temps_cpu_calcul                          ==
! == Calcul temps CPU en seconde                          ==
! ==========================================================	
subroutine temps_cpu_calcul(t)

real, intent(out) ::t

!===   Début des traitements   ===
call CPU_TIME(t)
  
end subroutine temps_cpu_calcul

! ==========================================================
! == Subroutine temps_cpu_calcul_un_param                 ==
! == Calcul temps CPU en seconde pour un paramètre        ==
! ==========================================================	
subroutine temps_cpu_calcul_un_param(t_cpu,t_param)

real, intent(inout) :: t_cpu
real, intent(inout) :: t_param

real   :: t

!===   Début des traitements   ===
call CPU_TIME(t)
t_param = t_param + t - t_cpu
t_cpu = t
end subroutine temps_cpu_calcul_un_param

! ==========================================================
! == Subroutine Convertit_s_en_heure_mn_s                 ==
! == Conversion en heure, minute, seconde                 ==
! ==========================================================	
function Convertit_s_en_heure_mn_s(tsec) result(t3)

real, intent(in) :: tsec
integer, dimension(3) :: t3

real   :: t

!===   Début des traitements   ===
t3(1) = INT(tsec /3600)
t3(2) = INT((tsec-3600*t3(1))/60)
t3(3) = INT(tsec-3600*t3(1)-60*t3(2))

end function Convertit_s_en_heure_mn_s

! ==========================================================
! == Subroutine temps_cpu_affichage                       ==
! == Calcul temps CPU en seconde pour un paramètre        ==
! ==========================================================	
subroutine temps_cpu_affichage(tc)

type(st_temps_cpu), intent(in) :: tc

integer, dimension(3)  :: t3
real                   :: t,titer
character(LEN=15)      :: ladate, heure, zone
integer, dimension(8) :: dh
!===   Début des traitements   ===
t = tc%t_fin - tc%t_debut
t3 = Convertit_s_en_heure_mn_s(t) 
titer = t-tc%t_pre_calcul-tc%t_post_calcul
!if (rang==RANG_MAITRE_AFF) then
  print '(T5," Temps de calcul total  = ",I2,"h",I2,"mn",I2,"s")',t3(1),t3(2),t3(3)
  print '(T5," Temps de calcul pendant les itérations temporelles  = ",F12.2,"s")',titer
  print '(T10," Temps de calcul champ E  = ",f12.4," secondes")',tc%t_calcul_e
  print '(T10," Temps de calcul champ H  = ",f12.4," secondes")',tc%t_calcul_h
  print '(T10," Temps de calcul frontiere E  = ",f12.4," secondes")',tc%t_frontiere_e
  print '(T10," Temps de calcul frontiere H  = ",f12.4," secondes")',tc%t_frontiere_h
  print '(T10," Temps de calcul modif et excitation E  = ",f12.4," secondes")',tc%t_modif_excite_e
  print '(T10," Temps de calcul modif et excitation H  = ",f12.4," secondes")',tc%t_modif_excite_h
  print '(T10," Temps de calcul annexe  = ",f12.4," secondes")',tc%t_calcul_annexe
  print '(T10," Temps de pre-calcul  = ",f12.4," secondes")',tc%t_pre_calcul
  print '(T10," Temps de post-calcul  = ",f12.4," secondes")',tc%t_post_calcul
!end if
print *
!print '(T10," Rapport temps cellule PML / temps cellule utile =",F8.3)',
call date_and_time(ladate,heure,zone,dh)
if (rang==RANG_MAITRE_AFF) &
  print '(" DATE = ",A,"/",A,"/",A,"  -  HEURE = ",A,"H",A,"mn",A,"s")',ladate(7:8),ladate(5:6),ladate(1:4),&
                                                 heure(1:2),heure(3:4),heure(5:6)
end subroutine temps_cpu_affichage


end module mfdtd_temps_cpu



