!/////////////////////////////////////////////////////////////////////////////
!// Name:        temsi_compress_structure.f90
!// Purpose:     Programme principal de compression des géométries
!//   maillées se trouvant dans le bloc structure 
!// Concerne le solveur FDTD
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     Avril 2008
!// Copyright:   (c) Christophe Guiffaut, équipe CNRS IRCOM/DEM
!//              Utilisation des sources réservées à l'IRCOM
!// Licence  :
!/////////////////////////////////////////////////////////////////////////////

!===========================================================
!== Programme principal : FDTD3D                          ==
!===========================================================
program  temsi_compress_structure

use m_constantes, only : LEN_CHAINE

use mfdtd_structure
use m_liste_milieu, only : st_liste_milieu_simple,liste_milieu_nu,liste_milieu_nom
use mfdtd_fdtd3d
use mfdtd_compress
implicit none
integer, parameter :: IP_1= selected_int_kind(2) ! jusqu'à +-10^2
!integer, parameter  :: IP_16B = selected_int_kind(4)


type(st_fdtd3d)  :: fd ! Structure principale
integer :: nbstructure
character(LEN=LEN_CHAINE) :: nom_fichier_entree,nom_fichier_sortie
character(len=LEN_CHAINE) :: nombloc

! nombre_arguments : Nb arguments sur la commande de lancement de TRIDIMO
integer  :: nombre_arguments
integer  :: statut  !variable de statut à l'ouverture du fichier
integer  :: pos,l
integer, external :: iargc
external :: getarg


!==================== OUVERTURE DU FICHIER D'ENTRÉE ================
print '("=====================================================")'
print '("==      PROGRAMME TEMSI : COMPRESSION STRUCTURES   ==")'
print '("==          (POUR LE SOLVEUR TEMSI-FD)             ==")'
print '("=====================================================")'

! -- Recupere le nombre de d'arguments sur la ligne de commande --
nombre_arguments = iargc()
if (nombre_arguments/=1) then
  print *, 'Etourdi ! faudrait  me donner un fichier de données !!!'
  print *, 'Erreur ligne de commande : usage : tridimo nom_fichier.'
  stop
endif

! -- Recupere le nom du fichier d'entree --
call getarg(nombre_arguments,nom_fichier_entree)


! -- Positionne quelques de fd --
fd%t_fichier%nom_fichier = nom_fichier_entree

print '(//,"===  Initialisation par défaut ...  ===")'
call fdtd3d_initdefaut(fd)

print '(//,"===  Lecture du fichier d''entree...  ===")'
call fdtd3d_lecture(fd)


!==============================================================================
print '(//,"===  Initialisation  ===")'

! -- Remplissage de la structure des milieux simples ---
!if (fd%flag_milieu_dispersif) then
!  print '(T5,"--- INIT MILIEU DISPERSIF ---")'
!  call milieu_dispersif_init(fd%t_milieu_dispersif)
!end if

! -- Réglage des flags de milieu, création de la structure
!    contenant tous les milieux simples ---
if (fd%flag_liste_milieu_simple) then
  print '(T5,"--- INIT LISTE MILIEU SIMPLE ---")'
  call liste_milieu_simple_init(fd%t_liste_milieu_simple, &
                                fd%t_milieu_dispersif, &
                                fd%t_volume%mu_r, fd%t_volume%epsilon_r,&
				fd%t_volume%sigma_e, fd%t_volume%sigma_h)
endif


! -- A besoin de t_frontiere & t_volume --
! -- Determine le volume reel de calcul --
print '(T5,"--- INIT FRONTIERES ---")'
call frontiere_init(fd%t_frontiere,fd%t_volume)



! -- A besoin de connaitre le volume reel de calcul --
if (fd%flag_structure) THEN
  print '(T5,"--- INIT STRUCTURE ---")'
  call structure_init(fd%t_structure,fd%t_volume,fd%t_liste_milieu_simple)
END IF

!==============================================================================



!==================== COMPRESSION DES VOLUMES MAILLÉS =========================
if (fd%t_structure%nbmilieu>0) then
  print '("--> COMPRESSION DES VOLUMES MAILLÉS:")'
  nbstructure = compress_structure_volume(fd%t_structure,fd%t_volume,fd%t_liste_milieu_simple)
end if
!=========== COMPRESSION DES SURFACES MAILLÉS DE TYPE METAL ===================
if (fd%t_structure%nbmetal>0) then
  print '("--> COMPRESSION DES SURFACES DE TYPE MÉTAL:")'
  print '("    ! ne traite pas les grilles ou les fils de type metal ")'
  nbstructure = compress_metal(fd%t_structure,fd%t_volume)
end if

!=========== COMPRESSION DES SURFACES MAILLÉS DE TYPE PLAQUE MINCE ===================
if (fd%t_structure%nbplaque>0) then
  print '("--> COMPRESSION DES SURFACES DE TYPE PLAQUE MINCE:")'
  nbstructure = compress_plaque_mince(fd%t_structure,fd%t_volume)
end if


!======================= FICHIER de sortie =================================
pos = scan(nom_fichier_entree,'.',.true.)
l = len_trim(nom_fichier_entree);
nom_fichier_sortie = nom_fichier_entree(1:pos-1)//'_m'//nom_fichier_entree(pos:l);
print '(//,"===  Ecriture du fichier d''entree...  ===")'
!------------------ ECRITURE --------------------
fd%t_fichier%nom_fichier = nom_fichier_sortie
call fdtd3d_ecriture(fd)

print '( "  --- FIN DES TRAITEMENTS : OK  ---")'


end program temsi_compress_structure	



