!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_dans_fichier.f90
!// Purpose:     Fonction source lue dans un fichier
!// Author:      Christophe Guiffaut
!// Modified by:
!// Created:     08 octobre 2004
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

module m_gene_dans_fichier
use m_entree , only : st_fichier, lineread, &
                                  lecture_stop, &
				  unitefichier, &
				  entier_chaine, &
				  COMMENTAIRE, &
				  ctrl_nb_mot_chaine
use m_constantes
use m_constantes_scientifiques, only : PI
use m_gene_identifiant

implicit none

!=== Déclaration des types ===

! Structure du sinus 
type st_gene_dans_fichier
  integer :: colonne
  integer :: nbech
  real :: dt
  real, dimension(:), pointer  :: f
  character (LEN=LEN_CHAINE)   :: nomfichier
end type st_gene_dans_fichier


contains

!===========================================================	
! Subroutine gene_dans_fichier_initdefaut
!===========================================================	
subroutine gene_dans_fichier_initdefaut(t_df)
!=== Déclaration des arguments ===
type(st_gene_dans_fichier), intent(inout)    :: t_df

!===   Début des traitements   ===
t_df%dt = 0.0
nullify(t_df%f)
t_df%colonne = 1
t_df%nomfichier = ''
t_df%nbech = 0

end subroutine gene_dans_fichier_initdefaut
!===========================================================	
! Subroutine gene_dans_fichier_copy
!===========================================================	
subroutine gene_dans_fichier_copy(tin,tout)
!=== Déclaration des arguments ===
type(st_gene_dans_fichier), intent(in)    :: tin
type(st_gene_dans_fichier), intent(out)    :: tout

!===   Début des traitements   ===
tout%dt = tin%dt
nullify(tout%f)
tout%colonne = tin%colonne
tout%nomfichier = tin%nomfichier
tout%nbech = tin%nbech
if (tout%nbech>0) then
  allocate(tout%f(tout%nbech))
  tout%f=tin%f
end if

end subroutine gene_dans_fichier_copy

!===========================================================	
! Subroutine gene_dans_fichier_lecture
! Lecture dans le fichier d'entrée des paramètres 
! sur le générateur sinus
!===========================================================	
subroutine gene_dans_fichier_lecture(t_fich,t_df)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_dans_fichier), intent(inout)    :: t_df

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine, status
logical, dimension(1)  :: flag
!===   Début des traitements   ===

!---------- INITIALISATION ------------
call gene_dans_fichier_initdefaut(t_df)
flag(:) = .false.
!-------------------------------------

do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)

select case(premiermot(1:lchaine))
    case('dt')
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)
      read(t_fich%chaine,*,err=999) corbeille,t_df%dt
      if (t_df%dt <= 0.0) call lecture_stop(t_fich)
    case('nom_fichier')
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)
      read(t_fich%chaine,*,err=999) corbeille,t_df%nomfichier
      ! lecture des valeurs de la fonction dans le fichier
      flag(1) = .true.
    case('colonne')
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)
      read(t_fich%chaine,*,err=999) corbeille,t_df%colonne
      if (t_df%colonne <1) call lecture_stop(t_fich)
    case default
      if (.not.ALL(flag)) then
	print '("Erreur, manque une donnée : dt ou nom de fichier ! ")'
        call lecture_stop(t_fich)
      end if
      call gene_dans_fichier_lect_fct(t_df)
      exit
  end select
end do   
return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(t_fich)


end subroutine gene_dans_fichier_lecture

!===========================================================	
! Subroutine gene_dans_fichier_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur la fonction source
!===========================================================	
subroutine gene_dans_fichier_ecriture(f,t_df)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_dans_fichier), intent(inout)    :: t_df

write(f%unite,'(T5,A,3x,A)')  'nom_fichier',trim(t_df%nomfichier)
if (t_df%colonne>1) write(f%unite,'(T5,A,3x,I3)')  'colonne',t_df%colonne
if (t_df%dt>0.0)    write(f%unite,'(T5,A,3x,ES13.6E2)')  'dt',t_df%dt

end subroutine gene_dans_fichier_ecriture


! ==========================================================
! == Subroutine gene_dans_fichier_lect_fct                ==
!===========================================================	
subroutine gene_dans_fichier_lect_fct(t_df)
!--- DECLARATION DES ARGUMENTS ---
type(st_gene_dans_fichier), intent(inout) :: t_df

!--- DECLARATION DES VARIABLES LOCALES ---
integer  :: unite, statut
integer  :: numligne,nbvaleur
integer  :: i,j
real, dimension(0:t_df%colonne-1) :: v
!character(len=20*t_df%colonne)  :: chaine
character(len=LEN_MOT_CLE) :: longueur,mot
character(LEN=30),dimension(1:t_df%colonne) :: chaine

!--- DEBUT DES TRAITEMENTS ---
! -- Recupere un descripteur libre le fichier d'entree --
unite =  unitefichier()
if (unite<=0) then
  print '(/,t10,"ATTENTION, pas de descripteur de fichier disponible => STOP !!!")'
  stop
endif
! -- Ouvre le fichier d'entree en lecture --
open(unite,file=t_df%nomfichier,status = 'old',action='read',iostat=statut)
if (statut>0) then
  print '(/,t10,"ATTENTION, ouverture de <",A,"> impossible => STOP !!!")', &
    TRIM(t_df%nomfichier)
  stop
endif

longueur = entier_chaine(20*t_df%colonne) ! 10000 caractères sur une ligne maxi
numligne = 0
!------------------------------------------------------------------------------
! ----------- On compte le nombre de valeurs de la fonction -------------------
if (rang==RANG_MAITRE_AFF) print '(T15,"colonne de la fct src associé au fichier =",I5)',(t_df%colonne)
do 
 ! read(unite,'(a'//TRIM(longueur)//')',end=800) chaine
  read(unite,'(A)',end=800) mot
  mot = ADJUSTL(mot) !on retire les blancs de début de la chaine
  ! On ignore les lignes de commentaires et les chaines vides
  if ((mot=='').or.(mot(1:1)==COMMENTAIRE)) cycle
  numligne = numligne + 1
end do
!------------------------------------------------------------------------------

800 allocate(t_df%f(1:numligne))
t_df%nbech=numligne
t_df%f = 0.0
i=0
!print '("numligne=",I4)', numligne
numligne = 0
rewind(unite)
!longueur = entier_chaine(t_df%colonne-1) ! 10000 caractères sur une ligne maxi
!formatch = '('//TRIM(longueur)//'A)';

do 
  !read(unite,'(a'//TRIM(longueur)//')',end=900) chaine
!  read(unite,'(A)',end=900) mot
  read(unite,*,end=900) chaine(1:t_df%colonne)
  mot = ADJUSTL(chaine(1)) !on retire les blancs de début de la chaine
  ! compteur ligne 
  numligne = numligne + 1
  !print '(T10,"mot=",A,"  chaine=",2A)',trim(mot),(chaine(1)),(chaine(2))
  ! On ignore les lignes de commentaires et les chaines vides
  if ((mot=='').or.(mot(1:1)==COMMENTAIRE)) then
    cycle
  else ! une chaine de données
    i = i+1
     read(chaine(t_df%colonne),*,ERR=901) t_df%f(i)
    !if (t_df%colonne == 1) then 
    !  read(chaine(),*,ERR=901) t_df%f(i)
    !else
    !  read(chaine,*,ERR=901) v(1:t_df%colonne-1),t_df%f(i)
!      print '(I4,ES15.6e2)', i,t_df%f(i)
   ! end if
  endif
enddo
!------------------------------------------------------------------------------

! fermeture du fichier
900 close(unite)
    return
   
901 print '("Erreur à la lecture de la ligne : ",I4,"dans le fichier de nom ",A)',&
	numligne,TRIM(t_df%nomfichier)
    STOP '----> Arret !!!!!!!!!'


end subroutine gene_dans_fichier_lect_fct
	
!===========================================================	
! Subroutine gene_dans_fichier_precalcul
! Calcul de la fonction sur les ntinc itérations temporelles
! avec un pas dt de la FDTD
! La phase est ignorée.
!===========================================================	
function gene_dans_fichier_calcul_t(t_df, A0, retard, &
                                dt, ntinc,tab1d) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_dans_fichier), intent(in)    :: t_df
real, intent(in)          :: A0
real, intent(in)          :: retard, dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      :: func_dervaleur
!===  Déclaration des variables  ===
integer   :: i, n1, n2
integer   :: ip,ir
real      :: t,p,c1_6
!===   Début des traitements   ===	
n1 = size(t_df%f); ! nbre de valeurs
tab1d(:) = 0.0
!print '(T15,"t_df%dt=",ES15.6)',t_df%dt
ir = FLOOR(retard /dt)
if (t_df%dt ==0.0) then
  n2 = min(n1,ntinc-ir)
  tab1d(ir:n2+ir-1) = t_df%f(1:n2)
  func_dervaleur = t_df%f(n2)
else
  ! limitation sur le pas temporel de la fonction donné par l'utilisateur
  if ((t_df%dt < dt/10.0).or.(t_df%dt > dt*10000.0)) then
    print '("  Attention, dt_fdtd/10<=dt_fonction <= dt_fdtd*100 !!!!!")'
    stop 'Arret !!!!!!!!!!!!!!!!!!'
  else 
    tab1d(0) = t_df%f(1) ! valeur pour t=0.0s
    c1_6 = 1.0 / 6.0 ! coefficient pour interpolation ordre 4
    do i=1,ntinc-1-ir
      t = i*dt ! temps désiré
      ip = FLOOR(t/t_df%dt)
      p  = t/t_df%dt - ip*1.0 !facteur d'interpolation compris entre 0 et 1
      if (ip+2 >n1) exit ! on sort
      if (ip<=0.or.ip==n1-3) then
        tab1d(i+ir) = (1.0-p)*t_df%f(ip+1) + p * t_df%f(ip+2)
      else
        tab1d(i+ir) = - p*(p-1)*(p-2)*c1_6*t_df%f(ip)  &
		     + (p**2-1)*(p-2)*0.5*t_df%f(ip+1)  &
		     - p*(p+1)*(p-2)*0.5 *t_df%f(ip+2) &
		     + p*(p**2-1)*c1_6   *t_df%f(ip+3)
      end if
      
!      print '(T15,"ip,p,i+ir,tab1d=",I6,F6.2,I6,ES15.6)',ip,p,i+ir,tab1d(i+ir)
!      write (50,*) ip,p,i+ir,t_df%f(ip),tab1d(i+ir)
    end do
  end if
end if
tab1d = tab1d*A0

end function gene_dans_fichier_calcul_t

! ==========================================================
! == Subroutine subroutine gene_dans_fichier_deallocate   ==
!===========================================================	
subroutine gene_dans_fichier_deallocate(t_df)
!--- DECLARATION DES ARGUMENTS ---
type(st_gene_dans_fichier), intent(inout) :: t_df

!--- DÉBUT DES TRAITEMENTS ---
if (associated(t_df%f)) then
  deallocate(t_df%f)
  nullify(t_df%f)
end if
end subroutine gene_dans_fichier_deallocate

end module m_gene_dans_fichier



