!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_derivee_gaussien.f90
!// Purpose:      Traite la fonction source de type dérivée de la gaussien  ne
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     janvier 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!===========================================================
! == Generateur de fonctions dérivée-gaussiennes          ==
! == Module module_gene_derive_gauss                  ==
! == Type st_gene_derive_gauss                        ==
! == Subroutine gene_derive_gauss_inidefaut           ==
! == Subroutine gene_derive_gauss_lecture             ==
! == Subroutine gene_derive_gauss_init      ==
! == Subroutine gene_derive_gauss_calcul_t     ==
! == Subroutine gene_derive_gauss_calcul_1t    ==
! == Suborutine gene_derive_gauss_calcul              ==
! == Suborutine gene_derive_gauss_affichage           ==
! == Subroutine gene_derive_gauss_postcalcul          ==
! == Pas de variables globales...                         ==
!===========================================================

module m_gene_derive_gaussien
use m_entree , only : st_fichier, lineread, &
                                  lecture_stop, &
				  ctrl_nb_mot_chaine
use m_constantes 
use m_constantes_scientifiques, only : PI
use m_gene_identifiant 

implicit none
real, parameter :: GDG_ATTFMAX = 10.0
real, parameter :: GDG_ATTPIED = 1000.0

!=== Déclaration des types ===

! Structure de la fonction dérivée-gaussienne 
type st_gene_derive_gauss
  integer :: type_def
  real    :: duree, tm, attpied, fmax, attfmax
  real    :: T  ! 2*T = largeur de la gaussienne à 1/3 du max d'amplitude 
  real    :: t0  ! t0 = duree /2  
end type st_gene_derive_gauss


contains
!===========================================================	
! Subroutine gene_derive_gauss_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_derive_gauss_initdefaut(t_gdg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_derive_gauss), intent(inout)  :: t_gdg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_gdg%type_def = 0
t_gdg%fmax = 0.0 
t_gdg%attfmax = GDG_ATTFMAX 
t_gdg%attpied = GDG_ATTPIED
t_gdg%tm     = 0.0
t_gdg% duree = 0.0 
t_gdg%t0    = 0.0
t_gdg%T     = 0.0
end subroutine gene_derive_gauss_initdefaut


!===========================================================	
! Subroutine gene_derive_gauss_lecture
! Lecture dans le fichier d'entrée des paramètres 
! sur le générateur sino-gaussien
!===========================================================	
subroutine gene_derive_gauss_lecture(t_fich,t_gdg)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_derive_gauss), intent(inout)    :: t_gdg

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine,status
logical  :: flagduree

!===   Début des traitements   ===

call gene_derive_gauss_initdefaut(t_gdg)

flagduree = .false.
do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)
 
  select case(premiermot(1:lchaine))
    case(BFS_DUREE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gdg%duree
      t_gdg%t0 = t_gdg%duree / 2
      t_gdg%T  = t_gdg%t0 / (SQRT(LOG(2.5*t_gdg%attpied * &
                              SQRT(LOG(2.5*t_gdg%attpied)) )))
      flagduree = .true.
      t_gdg%fmax = 0.0
      t_gdg%type_def = 2
    case(BFS_TM)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gdg%tm
      ! temps de montée de 10 % à 90 %
      ! on déduit le retard t0 = duree / 2
      t_gdg%t0 = SQRT(LOG(t_gdg%attpied))*t_gdg%tm /(SQRT(LOG(10.0))-SQRT(LOG(10.0/9)))
      t_gdg%T  = t_gdg%t0 / (SQRT(LOG(2.5*t_gdg%attpied * &
                            SQRT(LOG(2.5*t_gdg%attpied)) )))
      flagduree = .false.
      t_gdg%type_def = 1

    case(BFS_FMAX)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gdg%fmax
      t_gdg%T  = SQRT(LOG(5.5*t_gdg%attfmax))/(PI*t_gdg%fmax)
      t_gdg%t0 = t_gdg%T * (SQRT(LOG(2.5*t_gdg%attpied * &
                            SQRT(LOG(2.5*t_gdg%attpied)) )))   
      flagduree = .false.
      t_gdg%fmax = 0.0
      t_gdg%type_def = 0
    case(BFS_ATT_FMAX)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gdg%attfmax
      if (t_gdg%fmax <= 0) then
       	print *,'Avertissement :  Att_fmax définie alors que fmax pas encore définie' 
      else 
        t_gdg%T  = SQRT(LOG(5.5*t_gdg%attfmax))/(PI*t_gdg%fmax)
        t_gdg%t0 = t_gdg%T * (SQRT(LOG(2.5*t_gdg%attpied * &
                            SQRT(LOG(2.5*t_gdg%attpied)) )))   
        flagduree = .false.
      end if
    case(BFS_ATT_PIED)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gdg%attpied
      if (flagduree) then
        ! forme de la gaussienne modifiée par la valeur du pied
        t_gdg%T  = t_gdg%t0 / (SQRT(LOG(2.5*t_gdg%attpied * &
                              SQRT(LOG(2.5*t_gdg%attpied)) )))
      else
        ! forme de la gaussienne non modifiée par la valeur du pied
        t_gdg%t0 = t_gdg%T * (SQRT(LOG(2.5*t_gdg%attpied * &
                            SQRT(LOG(2.5*t_gdg%attpied)) )))   
      end if
    case default
      if ((t_gdg%t0 == 0.0) .or. (t_gdg%T == 0.0)) then
	print *,'fonction dérivée gaussienne mal définie !!!' 
	call lecture_stop(t_fich)
      end if
      exit 
  end select
end do    

end subroutine gene_derive_gauss_lecture


!===========================================================	
! Subroutine gene_derive_gauss_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur gaussien
!===========================================================	
subroutine gene_derive_gauss_ecriture(f,t_gdg)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_derive_gauss), intent(inout)    :: t_gdg

if (t_gdg%type_def == 0) then
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_FMAX,t_gdg%fmax
else if (t_gdg%type_def == 1) then
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_TM,t_gdg%tm
else
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_DUREE,t_gdg%duree
end if

if (t_gdg%attpied /= GDG_ATTPIED) write(f%unite,'(T5,A,3x,ES10.3E2)')  BFS_ATT_PIED,t_gdg%attpied
if (t_gdg%attfmax /= GDG_ATTFMAX) write(f%unite,'(T5,A,3x,ES10.3E2)')  BFS_ATT_FMAX,t_gdg%attfmax

end subroutine gene_derive_gauss_ecriture

!===========================================================	
! Subroutine gene_derive_gauss_init
! Initialisation des variables
!===========================================================	
subroutine gene_derive_gauss_init(t_gdg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_derive_gauss), intent(inout)    :: t_gdg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_derive_gauss_init
	
!===========================================================	
! Subroutine gene_derive_gauss_calcul_t
! Calcul de la fonction derivee-gaussienne sur le nombre d'itérations
! temporelles, paramètres transmis en argument
! Pas de phase avec la gaussienne
! Calculs avant les itérations temporelles
!===========================================================	
function gene_derive_gauss_calcul_t(t_gdg, A0, retard, &
                                dt, ntinc,tab1d) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_derive_gauss), intent(in)    :: t_gdg
real, intent(in)          :: A0
real, intent(in)          :: retard, dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      :: func_dervaleur
!===  Déclaration des variables  ===
integer   :: i
real      :: T_2, A0_T

!===   Début des traitements   ===	
T_2 = t_gdg%T**2
A0_T = -2*A0/t_gdg%T
do i =0, ntinc-1
  tab1d(i) = A0_T * (i*dt-t_gdg%t0-retard) & 
              * EXP(-(i*dt-retard-t_gdg%t0)**2 / T_2) 
end do
! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)


end function gene_derive_gauss_calcul_t

!===========================================================	
! Subroutine gene_derive_gauss_calcul_1t
! Calcul de la fonction derivee-gaussienne pour un t
! Pas de phase avec la gaussienne
! Calculs avant les itérations temporelles
!===========================================================	
function gene_derive_gauss_calcul_1t(t_gdg, A0, &
                 retard,t,dt) result (func_dervaleur)
implicit none

!=== Déclaration des arguments ===
type(st_gene_derive_gauss), intent(in)    :: t_gdg
real, intent(in)          :: A0
real, intent(in)          :: retard, t,dt
real                      :: func_dervaleur
!===  Déclaration des variables  ===
real      :: T_2, A0_dt_1

!===   Début des traitements   ===	
T_2 = t_gdg%T**2
A0_dt_1 = A0/dt

func_dervaleur = A0_dt_1 * (t-t_gdg%t0-retard) * &
                    EXP(- (t-retard-t_gdg%t0)**2 / T_2)

end function gene_derive_gauss_calcul_1t

!===========================================================	
! Subroutine gene_derive_gauss_calcul
!  Calcul pendant les itérations temporelles
!===========================================================	
subroutine gene_derive_gauss_calcul(t_gdg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_derive_gauss), intent(inout)    :: t_gdg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_derive_gauss_calcul

!===========================================================	
! Subroutine gene_derive_gauss_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_derive_gauss_affichage(t_gdg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_derive_gauss), intent(inout)    :: t_gdg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_derive_gauss_affichage

!===========================================================	
! Subroutine gene_derive_gauss_deallocate
!===========================================================	
subroutine gene_derive_gauss_deallocate(t_gdg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_derive_gauss), intent(inout)    :: t_gdg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end Subroutine gene_derive_gauss_deallocate


end module m_gene_derive_gaussien



