!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_double_expo.f90
!// Purpose:      Traite la fonction source de type double exponentielle
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     janvier 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! == Type st_gene_double_expo                    ==
! == Subroutine gene_double_expo_inidefaut       ==
! == Subroutine gene_double_expo_lecture         ==
! == Subroutine gene_double_expo_init  ==
! == Subroutine gene_double_expo_precalcul       ==
! == Suborutine gene_double_expo_calcul          ==
! == Suborutine gene_double_expo_affichage       ==
! == Subroutine gene_double_expo_postcalcul      ==
! == Pas de variables globales...                         ==
!===========================================================

module m_gene_double_expo
use m_entree , only : st_fichier, lineread,&
                                  lecture_stop, &
				  ctrl_nb_mot_chaine
use m_constantes 
use m_gene_identifiant 
implicit none
!=== Déclaration des types ===

! Structure de la fonction double-exponentielle 
type st_gene_double_expo
  real    :: tm,D
  
end type st_gene_double_expo


contains
!===========================================================	
! Subroutine gene_double_expo_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_double_expo_initdefaut(t_gde)

implicit none

!=== Déclaration des arguments ===
type(st_gene_double_expo), intent(inout)  :: t_gde

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_gde%tm = 0.0
t_gde%D  = 0.0

end subroutine gene_double_expo_initdefaut


!===========================================================	
! Subroutine gene_double_expo_lecture
! Lecture dans le fichier d'entrée des paramètres 
! sur le générateur sino-gaussien
!===========================================================	
subroutine gene_double_expo_lecture(t_fich,t_gde)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_double_expo), intent(inout)    :: t_gde

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine,status
!===   Début des traitements   ===
do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)
  if (t_fich%nomfinbloc/=TRIM(premiermot)) then
    ! Les paramètres de la gaussienne
    status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
  end if
  select case(premiermot(1:lchaine))
    case(BFS_LMH)
      read(t_fich%chaine,*) corbeille,t_gde%D
    case(BFS_TM)
      read(t_fich%chaine,*) corbeille,t_gde%tm
    case default
      exit
  end select
end do    
end subroutine gene_double_expo_lecture

!===========================================================	
! Subroutine gene_double_expo_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur gaussien
!===========================================================	
subroutine gene_double_expo_ecriture(f,t_gde)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_double_expo), intent(inout)    :: t_gde

write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_LMH,t_gde%D
write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_TM,t_gde%tm

end subroutine gene_double_expo_ecriture


!===========================================================	
! Subroutine gene_double_expo_init
! Initialisation des variables
!===========================================================	
subroutine gene_double_expo_init(t_gde)
!=== Déclaration des arguments ===
type(st_gene_double_expo), intent(inout)    :: t_gde

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_double_expo_init
	
!===========================================================	
! Subroutine gene_double_expo_calcul_t
! Calculs avant les itérations temporelles
!===========================================================	
function gene_double_expo_calcul_t(t_gde,A0, retard, &
                                dt, ntinc,tab1d) result(func_dervaleur)

!=== Déclaration des arguments ===
type(st_gene_double_expo), intent(inout)    :: t_gde
real, intent(in)          :: A0
real, intent(in)          :: retard, dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      :: func_dervaleur

!===  Déclaration des variables  ===
integer   :: i,j,iretard
real      :: a,b,anorm

!===   Début des traitements   ===
a = log(2.0)/t_gde%D
b = log(9.0)/t_gde%tm
anorm = exp(-a*log(a/b)/(a-b))-exp(-b*log(a/b)/(a-b))
iretard = nint(retard/dt)
tab1d = 0.0
do i =iretard, ntinc-1
  j = i-iretard
  tab1d(i) = A0*(exp(-a*(j*dt))-exp(-b*(j*dt)))/anorm
end do
! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)
				
end function gene_double_expo_calcul_t

!===========================================================	
! Subroutine gene_double_expo_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_double_expo_affichage(t_gde)

implicit none

!=== Déclaration des arguments ===
type(st_gene_double_expo), intent(inout)    :: t_gde

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_double_expo_affichage

!===========================================================	
! Subroutine gene_double_expo_deallocate
!===========================================================	
subroutine gene_double_expo_deallocate(t_gde)

implicit none

!=== Déclaration des arguments ===
type(st_gene_double_expo), intent(inout)    :: t_gde

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end Subroutine gene_double_expo_deallocate


end module m_gene_double_expo



