!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_échelon_exp.f90
!// Purpose:      Traite la fonction source de type échelon exponentielle
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     janvier 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!===========================================================
! == Generateur de fonctions echelon exponentielle        ==
! == Module module_gene_echelon_exp                       ==
! == Type st_gene_echelon_exp                             ==
! == Subroutine gene_echelon_exp_inidefaut                ==
! == Subroutine gene_echelon_exp_lecture                  ==
! == Subroutine gene_echelon_exp_init           ==
! == Subroutine gene_echelon_exp_calcul_t          ==
! == Subroutine gene_echelon_exp_calcul_1t         ==
! == Suborutine gene_echelon_exp_calcul                   ==
! == Suborutine gene_echelon_exp_affichage                ==
! == Subroutine gene_echelon_exp_postcalcul               ==
! == Pas de variables globales...                         ==
!===========================================================

module m_gene_echelon_exp
use m_entree, only : st_fichier, lineread, &
                                  lecture_stop, &
				  ctrl_nb_mot_chaine
use m_constantes 
use m_constantes_scientifiques, only : PI
use m_gene_identifiant 
!=== Déclaration des types ===

! Structure de la fonction echelon exponentielle 
type st_gene_echelon_exp
  integer :: type_def
  real    :: tm    ! temps de montée
  real    :: pente  
end type st_gene_echelon_exp


contains
!===========================================================	
! Subroutine gene_echelon_exp_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_echelon_exp_initdefaut(t_ee)

implicit none

!=== Déclaration des arguments ===
type(st_gene_echelon_exp), intent(inout)  :: t_ee

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_ee%type_def= 0
t_ee%tm      = 0.0
t_ee%pente   = 0.0

end subroutine gene_echelon_exp_initdefaut


!===========================================================	
! Subroutine gene_echelon_exp_lecture
! Lecture dans le fichier d'entrée des paramètres 
! Les variables de la structure t0 et T sont calculées ici
! car ils dépendent du type de paramètre défini par l'utilisateur
! sur le générateur sinus
!===========================================================	
subroutine gene_echelon_exp_lecture(t_fich,t_ee)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_echelon_exp), intent(inout)    :: t_ee

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine, status
logical  :: flagduree

!===   Début des traitements   ===

!initialisation par défaut
call gene_echelon_exp_initdefaut(t_ee)

flagduree = .false.
do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)
  select case(premiermot(1:lchaine))
      
    case(BFS_TM)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_ee%tm
      ! temps de montée de 10 % à 90 %
      t_ee%pente = t_ee%tm / LOG(9.0)
      t_ee%type_def= 0
      
    case(BFS_EE_PENTE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_ee%pente
      t_ee%tm = t_ee%pente * LOG(9.0)
      t_ee%type_def= 1
       
    case default
      if ((t_ee%tm <= 0.0) .or. (t_ee%pente <= 0.0)) then
	print *,'fonction échelon exponentielle mal définie !!!' 
	call lecture_stop(t_fich)
      end if
      exit 
  end select
end do    
end subroutine gene_echelon_exp_lecture

!===========================================================	
! Subroutine gene_echelon_exp_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur échelon exponentielle
!===========================================================	
subroutine gene_echelon_exp_ecriture(f,t_ee)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_echelon_exp), intent(inout)    :: t_ee

if (t_ee%type_def == 0) then
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_TM,t_ee%tm
else if (t_ee%type_def == 1) then
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_EE_PENTE,t_ee%pente
end if

end subroutine gene_echelon_exp_ecriture



!===========================================================	
! Subroutine gene_echelon_exp_init
! Initialisation des variables
!===========================================================	
subroutine gene_echelon_exp_init(t_ee)

implicit none

!=== Déclaration des arguments ===
type(st_gene_echelon_exp), intent(inout)    :: t_ee

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_echelon_exp_init
	
!===========================================================	
! Subroutine gene_echelon_exp_calcul_t
! Calcul de la fonction échelon exponentielle sur le nombre d'itérations
! temporelles, paramètres transmis en argument
! Pas de phase
! Calculs avant les itérations temporelles
!===========================================================	
function gene_echelon_exp_calcul_t(t_ee, A0, retard, &
                     dt, ntinc,tab1d) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_echelon_exp), intent(in)    :: t_ee
real, intent(in)          :: A0
real, intent(in)          :: retard, dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      ::func_dervaleur
!===  Déclaration des variables  ===
integer   :: i,n

!===   Début des traitements   ===	
n=NINT(retard/dt)
tab1d=0.0
do i =0, ntinc-1-n
  tab1d(i+n) = A0 * (1 - EXP(-(i*dt)/t_ee%pente)) 
end do
! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)


end function gene_echelon_exp_calcul_t

!===========================================================	
! Subroutine gene_echelon_exp_calcul_1t
! Calcul de la fonction échelon exponentielle pour un t donné
! Pas de phase 
!===========================================================	
function gene_echelon_exp_calcul_1t(t_ee, A0, retard,t) &
                                  result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_echelon_exp), intent(in)    :: t_ee
real, intent(in)          :: A0
real, intent(in)          :: retard, t
real                      :: func_dervaleur
!===  Déclaration des variables  ===

!===   Début des traitements   ===	
func_dervaleur = A0 * (1- EXP(-(t-retard)/t_ee%pente))

end function gene_echelon_exp_calcul_1t


!===========================================================	
! Subroutine gene_echelon_exp_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_echelon_exp_affichage(t_ee)

implicit none

!=== Déclaration des arguments ===
type(st_gene_echelon_exp), intent(inout)    :: t_ee

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_echelon_exp_affichage

!===========================================================	
! Subroutine gene_echelon_exp_deallocate
!===========================================================	
subroutine gene_echelon_exp_deallocate(t_ee)

implicit none

!=== Déclaration des arguments ===
type(st_gene_echelon_exp), intent(inout)    :: t_ee

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end Subroutine gene_echelon_exp_deallocate


end module m_gene_echelon_exp



