!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_gaussien.f90
!// Purpose:      Traite la fonction source de type gaussien  
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     janvier 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!===========================================================
! == Generateur de fonctions gaussiennes                  ==
! == Module module_gene_gaussien                          ==
! == Type st_gene_gaussien                                ==
! == Subroutine gene_gaussien_inidefaut                   ==
! == Subroutine gene_gaussien_lecture                     ==
! == Subroutine gene_gaussien_init              ==
! == Subroutine gene_gaussien_calcul_t             ==
! == Subroutine gene_gaussien_calcul_1t            ==
! == Suborutine gene_gaussien_calcul                      ==
! == Suborutine gene_gaussien_affichage                   ==
! == Subroutine gene_gaussien_postcalcul                  ==
! == Pas de variables globales...                         ==
!===========================================================

module m_gene_gaussien
use m_entree, only : st_fichier, lineread, &
                                  lecture_stop, &
				  ctrl_nb_mot_chaine
use m_constantes 
use m_constantes_scientifiques, only : PI
use m_gene_identifiant 

implicit none
real, parameter :: GG_ATTFMAX = 10.0
real, parameter :: GG_ATTPIED = 1000.0

!=== Déclaration des types ===

! Structure de la fonction gaussienne 
type st_gene_gaussien
  INTEGER :: type_def
  real    :: duree, tm, attpied, fmax, attfmax
  real    :: T  ! 2*T = largeur de la gaussienne à 1/3 du max d'amplitude 
  real    :: t0  ! t0 = duree /2
end type st_gene_gaussien


contains
!===========================================================	
! Subroutine gene_gaussien_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_gaussien_initdefaut(t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_gaussien), intent(inout)  :: t_gg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_gg%type_def = 0
t_gg%fmax    = 0.0 
t_gg%attfmax = GG_ATTFMAX  
t_gg%attpied = GG_ATTPIED
t_gg%tm      = 0.0
t_gg% duree  = 0.0 
t_gg%t0      = 0.0
t_gg%T       = 0.0

end subroutine gene_gaussien_initdefaut


!===========================================================	
! Subroutine gene_gaussien_lecture
! Lecture dans le fichier d'entrée des paramètres 
! Les variables de la structure t0 et T sont calculées ici
! car ils dépendent du type de paramètre défini par l'utilisateur
! sur le générateur sinus
!===========================================================	
subroutine gene_gaussien_lecture(t_fich,t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_gaussien), intent(inout)    :: t_gg

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine,status
logical  :: flagduree

!===   Début des traitements   ===

!initialisation par défaut
call gene_gaussien_initdefaut(t_gg)

flagduree = .false.
do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)
  
  if (t_fich%nomfinbloc/=TRIM(premiermot)) then
    ! Les paramètres de la gaussienne
  end if
  
  select case(premiermot(1:lchaine))
    case(BFS_DUREE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gg%duree
      t_gg%t0 = t_gg%duree / 2
      t_gg%T  = t_gg%t0 / (SQRT(LOG(t_gg%attpied)))
      t_gg%fmax = 0.0
      flagduree = .true.
      t_gg%type_def = 2
      
    case(BFS_TM)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gg%tm
      ! temps de montée de 10 % à 90 %
      ! on déduit le retard t0 = duree / 2
      t_gg%type_def = 1
      t_gg%t0 = SQRT(LOG(t_gg%attpied))*t_gg%tm /(SQRT(LOG(10.0))-SQRT(LOG(10.0/9)))
      t_gg%T  = t_gg%t0 / (SQRT(LOG(t_gg%attpied)))
      flagduree = .false.
      t_gg%fmax = 0.0
      
    case(BFS_FMAX)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gg%fmax
      t_gg%T  = SQRT(LOG(t_gg%attfmax))/(PI*t_gg%fmax)
      t_gg%t0 = t_gg%T * SQRT(LOG(t_gg%attpied))   
      t_gg%type_def = 0
      flagduree = .false.
       
    case(BFS_ATT_FMAX)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gg%attfmax
      if (t_gg%fmax <= 0) then
       	print *,'Avertissement :  Att_fmax définie alors que fmax pas encore définie' 
      else 
        t_gg%T  = SQRT(LOG(t_gg%attfmax))/(PI*t_gg%fmax)
        t_gg%t0 = t_gg%T * SQRT(LOG(t_gg%attpied))   
        flagduree = .false.
      end if
      
    case(BFS_ATT_PIED)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gg%attpied
      if (flagduree) then
        ! forme de la gaussienne modifiée par la valeur du pied
        t_gg%T  = t_gg%t0 / (SQRT(LOG(t_gg%attpied)))
      else
        ! forme de la gaussienne non modifiée par la valeur du pied
        t_gg%t0 = t_gg%T * SQRT(LOG(t_gg%attpied)) 
      end if
    case default
      if ((t_gg%t0 <= 0.0) .or. (t_gg%T <= 0.0)) then
	print *,'fonction gaussienne mal définie !!!' 
	call lecture_stop(t_fich)
      end if    
      exit
  end select
end do    
end subroutine gene_gaussien_lecture

!===========================================================	
! Subroutine gene_gaussien_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur gaussien
!===========================================================	
subroutine gene_gaussien_ecriture(f,t_gg)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_gaussien), intent(inout)    :: t_gg

if (t_gg%type_def == 0) then
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_FMAX,t_gg%fmax
else if (t_gg%type_def == 1) then
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_TM,t_gg%tm
else
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_DUREE,t_gg%duree
end if

if (t_gg%attpied /= GG_ATTPIED) write(f%unite,'(T5,A,3x,ES10.3E2)')  BFS_ATT_PIED,t_gg%attpied
if (t_gg%attfmax /= GG_ATTFMAX) write(f%unite,'(T5,A,3x,ES10.3E2)')  BFS_ATT_FMAX,t_gg%attfmax

end subroutine gene_gaussien_ecriture



!===========================================================	
! Subroutine gene_gaussien_init
! Initialisation des variables
!===========================================================	
subroutine gene_gaussien_init(t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_gaussien), intent(inout)    :: t_gg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_gaussien_init
	
!===========================================================	
! Subroutine gene_gaussien_calcul_t
! Calcul de la fonction gaussienne sur le nombre d'itérations
! temporelles, paramètres transmis en argument
! Pas de phase avec la gaussienne
! Calculs avant les itérations temporelles
!===========================================================	
function gene_gaussien_calcul_t(t_gg, A0, retard, &
                                dt, ntinc,tab1d) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_gaussien), intent(in)    :: t_gg
real, intent(in)          :: A0
real, intent(in)          :: retard, dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      ::func_dervaleur
!===  Déclaration des variables  ===
integer   :: i
real      :: T_2

!===   Début des traitements   ===	
T_2 = t_gg%T**2
do i =0, ntinc-1
  tab1d(i) = A0 * EXP(-(i*dt-retard-t_gg%t0)**2/T_2) 
end do
! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)


end function gene_gaussien_calcul_t

!===========================================================	
! Subroutine gene_gaussien_calcul_1t
! Calcul de la fonction gaussienne pour un t donné
! Pas de phase avec la gaussienne
! Calculs avant les itérations temporelles
!===========================================================	
function gene_gaussien_calcul_1t(t_gg, A0, retard,t) &
                                  result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_gaussien), intent(in)    :: t_gg
real, intent(in)          :: A0
real, intent(in)          :: retard, t
real                      :: func_dervaleur
!===  Déclaration des variables  ===
real      :: T_2

!===   Début des traitements   ===	
T_2 = t_gg%T**2   
func_dervaleur = A0 * EXP(-(t-retard-t_gg%t0)**2/T_2)

end function gene_gaussien_calcul_1t

!===========================================================	
! Subroutine gene_gaussien_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_gaussien_affichage(t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_gaussien), intent(inout)    :: t_gg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_gaussien_affichage

!===========================================================	
! Subroutine gene_gaussien_deallocate
!  
!===========================================================	
subroutine gene_gaussien_deallocate(t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_gaussien), intent(inout)    :: t_gg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end Subroutine gene_gaussien_deallocate

end module m_gene_gaussien



