!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_heidler.f90
!// Purpose:      Traite la fonction source de type gene_heidler
!              
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     2004
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////


module m_gene_heidler
use m_entree, only : st_fichier, lineread, &
                                  lecture_stop, &
				  ctrl_nb_mot_chaine
use m_constantes 
use m_constantes_scientifiques, only : PI
use m_gene_identifiant

implicit none
!=== Déclaration des types ===

! Structure de la fonction gaussienne 
type st_ungene_heidler
  integer :: n
  real    :: I0, eta
  real    :: alpha, beta
end type st_ungene_heidler

type st_gene_heidler
  integer :: nbh  ! le nombre de fonction de heidler
  type(st_ungene_heidler),pointer,dimension(:) :: th
end type st_gene_heidler


contains
!===========================================================	
! Subroutine gene_heidler_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_heidler_initdefaut(th)
!=== Déclaration des arguments ===
type(st_gene_heidler), intent(inout)  :: th

!===   Début des traitements   ===
th%nbh    = 0 
nullify(th%th)
end subroutine gene_heidler_initdefaut

!===========================================================	
! Subroutine ungene_heidler_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine ungene_heidler_initdefaut(th)
!=== Déclaration des arguments ===
type(st_ungene_heidler), intent(inout)  :: th

!===   Début des traitements   ===
th%n = 0
th%eta  = 0.0
th%I0   = 0.0
th%alpha = 0.0
th%beta  = 0.0
end subroutine ungene_heidler_initdefaut

!===========================================================	
! Subroutine gene_heidler_copy
!===========================================================	
subroutine gene_heidler_copy(thin,thout)
!=== Déclaration des arguments ===
type(st_gene_heidler), intent(in)  :: thin
type(st_gene_heidler), intent(out)  :: thout

integer :: i
!===   Début des traitements   ===
thout%nbh = thin%nbh
nullify(thout%th)
if (thout%nbh==0) return
allocate(thout%th(thout%nbh))
do i=1,thout%nbh
  thout%th(i) = thin%th(i) ! copie de la structure
end do

end subroutine gene_heidler_copy


!===========================================================	
! Subroutine gene_heidler_lecture
! Lecture dans le fichier d'entrée des paramètres 
! Les variables de la structure t0 et T sont calculées ici
! car ils dépendent du type de paramètre défini par l'utilisateur
! sur le générateur sinus
!===========================================================	
subroutine gene_heidler_lecture(t_fich,th)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_heidler), intent(inout)    :: th

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine,status,i

!===   Début des traitements   ===
!print '(T5,"gene_heidler_lecture")'
allocate(th%th(th%nbh))
do i=1,th%nbh
  call ungene_heidler_initdefaut(th%th(i))
end do

!initialisation par défaut
i = 0;
do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)
  
 
  if (premiermot(1:1) == 'N') then
    status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
    read(premiermot(2:2),*,ERR=999) i
    if (i > th%nbh .or. i<1) call lecture_stop(t_fich)
    read(t_fich%chaine,*,ERR=999) corbeille,th%th(i)%n
    !print'(I3," n=",I3)',i,th%th(i)%n
    
  else if (premiermot(1:1) == 'I') then
    status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
    read(premiermot(2:2),*,ERR=999) i
    if (i > th%nbh .or. i<1) call lecture_stop(t_fich)
    read(t_fich%chaine,*,ERR=999) corbeille,th%th(i)%I0
    !print'(I3," I0=",ES15.4)',i,th%th(i)%I0

  else if (premiermot(1:3) == 'ETA') then
    status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
    read(premiermot(4:4),*,ERR=999) i
    if (i > th%nbh .or. i<1) call lecture_stop(t_fich)
    read(t_fich%chaine,*,ERR=999) corbeille,th%th(i)%eta
    !print'(I3," eta=",ES15.4)',i,th%th(i)%eta

  else if (premiermot(1:5) == 'ALPHA') then
    status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
    read(premiermot(6:6),*,ERR=999) i
    if (i > th%nbh .or. i<1) call lecture_stop(t_fich)
    read(t_fich%chaine,*,ERR=999) corbeille,th%th(i)%alpha
    !print'(I3," alpha=",ES15.4)',i,th%th(i)%alpha

  else if (premiermot(1:4) == 'BETA') then
    status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
    read(premiermot(5:5),*,ERR=999) i
    if (i > th%nbh .or. i<1) call lecture_stop(t_fich)
    read(t_fich%chaine,*,ERR=999) corbeille,th%th(i)%beta
    !print'(I3," beta=",ES15.4)',i,th%th(i)%beta
  else
    !print '(A)', TRIM(premiermot)
    exit
  end if
end do

return
999 print '(/,t10,"Erreur d''extraction des données de la ligne !!!")'
    call lecture_stop(t_fich)

end subroutine gene_heidler_lecture

!===========================================================	
! Subroutine gene_heidler_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur de heidler
!===========================================================	
subroutine gene_heidler_ecriture(f,th)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_heidler), intent(inout)    :: th

integer :: i
character(LEN=2)  :: ch

do i=1,th%nbh
  write(ch,'(I2)') i
  ch = ADJUSTL(ch)
  write(f%unite,'(T5,A,A,3x,I2)')  'N',trim(ch),th%th(i)%n
  write(f%unite,'(T5,A,A,3x,ES13.6E2)')  'I',trim(ch),th%th(i)%I0
  write(f%unite,'(T5,A,A,3x,ES13.6E2)')  'ETA',trim(ch),th%th(i)%eta
  write(f%unite,'(T5,A,A,3x,ES13.6E2)')  'ALPHA',trim(ch),th%th(i)%alpha
  write(f%unite,'(T5,A,A,3x,ES13.6E2)')  'BETA',trim(ch),th%th(i)%beta
  write(f%unite,*)
end do

end subroutine gene_heidler_ecriture


!===========================================================	
! Subroutine gene_heidler_init
! Initialisation des variables
!===========================================================	
subroutine gene_heidler_init(th)
!=== Déclaration des arguments ===
type(st_gene_heidler), intent(inout)    :: th

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_heidler_init
	
!===========================================================	
! Subroutine gene_heidler_calcul_t
! Calcul de la fonction gaussienne sur le nombre d'itérations
! temporelles, paramètres transmis en argument
! Pas de phase avec la gaussienne
! Calculs avant les itérations temporelles
!===========================================================	
function gene_heidler_calcul_t(th, A0, retard, &
                                dt, ntinc,tab1d) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_heidler), intent(in)    :: th
real, intent(in)          :: A0
real, intent(in)          :: retard, dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      ::func_dervaleur
!===  Déclaration des variables  ===
integer   :: i,j
real(kind=8)      :: a
type(st_ungene_heidler), pointer :: h
!===   Début des traitements   ===	
tab1d = 0.0
do i =0, ntinc-1
   
  do j=1,th%nbh
    h => th%th(j)
    a = ((i*dt-retard)*h%alpha)**h%n
    tab1d(i) = tab1d(i) + A0*(h%I0/h%eta)*EXP(-h%beta*(i*dt-retard)) *a/(a+1)
  end do
!write(100,*) i*dt, tab1d(i)
end do
!close(100)
! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)


end function gene_heidler_calcul_t


!===========================================================	
! Subroutine gene_heidler_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_heidler_affichage(th)
!=== Déclaration des arguments ===
type(st_gene_heidler), intent(inout)    :: th

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_heidler_affichage

!===========================================================	
! Subroutine gene_heidler_deallocate
!===========================================================	
subroutine gene_heidler_deallocate(th)
!=== Déclaration des arguments ===
type(st_gene_heidler), intent(inout)    :: th

!===   Début des traitements   ===
if (th%nbh>0) then
  deallocate(th%th)
  nullify(th%th)
end if
end Subroutine gene_heidler_deallocate


end module m_gene_heidler



