!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_iemn_expo_q.f90
!// Purpose:      Traite la fonction source de type iemn expo quotient
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     2004
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

! == Type st_gene_iemn_expo_q                                ==
! == Subroutine gene_iemn_expo_q_inidefaut                   ==
! == Subroutine gene_iemn_expo_q_lecture                     ==
! == Subroutine gene_iemn_expo_q_init              ==
! == Subroutine gene_iemn_expo_q_calcul_t             ==
! == Subroutine gene_iemn_expo_q_calcul_1t            ==
! == Suborutine gene_iemn_expo_q_calcul                      ==
! == Suborutine gene_iemn_expo_q_affichage                   ==
! == Subroutine gene_iemn_expo_q_postcalcul                  ==
! == Pas de variables globales...                         ==
!===========================================================
module m_gene_iemn_expo_q
use m_entree , only : st_fichier, lineread, &
                                  lecture_stop, &
				  ctrl_nb_mot_chaine
use m_constantes 
use m_constantes_scientifiques, only : PI
use m_gene_identifiant

!=== Déclaration des types ===

! Structure de la fonction gaussienne
type st_gene_iemn_expo_q
  real    :: alpha,beta
  real    :: tpq
  real    :: t0  ! t0 = duree /2
end type st_gene_iemn_expo_q

contains
!===========================================================	
! Subroutine gene_iemn_expo_q_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_iemn_expo_q_initdefaut(t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_iemn_expo_q), intent(inout)  :: t_gg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_gg%alpha    = 0.0 
t_gg%beta     = 0.0  
t_gg%tpq      = 0.0
t_gg%t0       = 0.0
end subroutine gene_iemn_expo_q_initdefaut


!===========================================================	
! Subroutine gene_iemn_expo_q_lecture
! Lecture dans le fichier d'entrée des paramètres 
! Les variables de la structure t0 et T sont calculées ici
! car ils dépendent du type de paramètre défini par l'utilisateur
! sur le générateur sinus
!===========================================================	
subroutine gene_iemn_expo_q_lecture(t_fich,t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_iemn_expo_q), intent(inout)    :: t_gg

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine,status

!===   Début des traitements   ===

!initialisation par défaut
call gene_iemn_expo_q_initdefaut(t_gg)

do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)
  
!  if (t_fich%nomfinbloc/=TRIM(premiermot)) then
!  end if
  
  select case(premiermot(1:lchaine))
    case(BFS_IEQ_ALPHA)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gg%alpha
      
    case(BFS_IEQ_BETA)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gg%beta
      
    case(BFS_IEQ_TPQ)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gg%tpq
       
    case default
!      if (t_gg%t0 <= 0.0) then
!	print *,'fonction iemn expo. quotient  mal définie !!!' 
!	call lecture_stop(t_fich)
!      end if    
      exit
  end select
end do    
end subroutine gene_iemn_expo_q_lecture

!===========================================================	
! Subroutine gene_iemn_expo_q_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur iemn exponentiel quotient
!===========================================================	
subroutine gene_iemn_expo_q_ecriture(f,t_gg)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_iemn_expo_q), intent(inout)    :: t_gg

write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_IEQ_ALPHA,t_gg%alpha
write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_IEQ_BETA,t_gg%beta
write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_IEQ_TPQ,t_gg%tpq

end subroutine gene_iemn_expo_q_ecriture


!===========================================================	
! Subroutine gene_iemn_expo_q_init
! Initialisation des variables
!===========================================================	
subroutine gene_iemn_expo_q_init(t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_iemn_expo_q), intent(inout)    :: t_gg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_iemn_expo_q_init
	
!===========================================================	
! Subroutine gene_iemn_expo_q_calcul_t
! Calcul de la fonction gaussienne sur le nombre d'itérations
! temporelles, paramètres transmis en argument
! Pas de phase avec la gaussienne
! Calculs avant les itérations temporelles
!===========================================================	
function gene_iemn_expo_q_calcul_t(tie, A0, retard, &
                                dt, ntinc,tab1d) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_iemn_expo_q), intent(in)    :: tie
real, intent(in)          :: A0
real, intent(in)          :: retard, dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      ::func_dervaleur
!===  Déclaration des variables  ===
integer   :: i,n
real      :: c1,t0

!===   Début des traitements   ===	
c1 = (tie%alpha/tie%beta)**(tie%beta/(tie%alpha+tie%beta)) * &
          (tie%alpha+tie%beta)/tie%alpha
t0 = retard + tie%tpq - log(tie%alpha/tie%beta)/(tie%alpha+tie%beta) 
n=Nint(retard/dt)
tab1d =0.0
do i =n, ntinc-1
  if ((tie%alpha*(i*dt-t0))>20) then
    tab1d(i) = A0*c1*EXP(-tie%beta*(i*dt-t0))
  else
    tab1d(i) = A0*c1*EXP(tie%alpha*(i*dt-t0))/(1+exp((tie%alpha+tie%beta)*(i*dt-t0)))
  end if
!write(100,*) i*dt, tab1d(i)
end do
!close(100)
! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)


end function gene_iemn_expo_q_calcul_t

!===========================================================	
! Subroutine gene_iemn_expo_q_calcul_1t
! Calcul de la fonction gaussienne pour un t donné
! Pas de phase avec la gaussienne
! Calculs avant les itérations temporelles
!===========================================================	
function gene_iemn_expo_q_calcul_1t(tie, A0, retard,t) &
                                  result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_iemn_expo_q), intent(in)    :: tie
real, intent(in)          :: A0
real, intent(in)          :: retard, t
real                      :: func_dervaleur
!===  Déclaration des variables  ===
real      :: c1,t0

!===   Début des traitements   ===	
c1 = (tie%alpha/tie%beta)**(tie%beta/(tie%alpha+tie%beta)) * &
          (tie%alpha+tie%beta)/tie%alpha
t0 = retard + tie%tpq - log(tie%alpha/tie%beta)/(tie%alpha+tie%beta)  
func_dervaleur = A0*c1*EXP(tie%alpha*(t-t0))/(1+exp((tie%alpha+tie%beta)*(t-t0)))

end function gene_iemn_expo_q_calcul_1t


!===========================================================	
! Subroutine gene_iemn_expo_q_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_iemn_expo_q_affichage(t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_iemn_expo_q), intent(inout)    :: t_gg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_iemn_expo_q_affichage

!===========================================================	
! Subroutine gene_iemn_expo_q_deallocate
!===========================================================	
subroutine gene_iemn_expo_q_deallocate(t_gg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_iemn_expo_q), intent(inout)    :: t_gg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end Subroutine gene_iemn_expo_q_deallocate


end module m_gene_iemn_expo_q
