!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_ondelette_rayleigh.f90
!// Purpose:      Traite la fonction source de type ondelette de Rayleigh
!                      ou pulse de Riker
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     janvier 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!===========================================================
! == Generateur de fonctions ondelette de rayleigh        ==
! == Module module_gene_rayleigh                ==
! == Type st_gene_rayleigh                      ==
! == Subroutine gene_rayleigh_inidefaut         ==
! == Subroutine gene_rayleigh_lecture           ==
! == Subroutine gene_rayleigh_init    ==
! == Subroutine gene_rayleigh_calcul_t   ==
! == Subroutine gene_rayleigh_calcul_1t  ==
! == Suborutine gene_rayleigh_calcul            ==
! == Suborutine gene_rayleigh_affichage         ==
! == Subroutine gene_rayleigh_postcalcul        ==
! == Pas de variables globales...                         ==
!===========================================================

module m_gene_rayleigh
use m_entree , only : st_fichier, lineread, &
                                  lecture_stop, &
				  ctrl_nb_mot_chaine
use m_constantes 
use m_constantes_scientifiques, only : PI
use m_gene_identifiant 

implicit none
real, parameter :: GOR_ATTPIED = 100.0
integer, parameter :: GOR_ORDRE = 3


!=== Déclaration des types ===

type st_gene_rayleigh
  real    :: attpied, f0, t0
  integer :: ordre
end type st_gene_rayleigh


contains
!===========================================================	
! Subroutine gene_rayleigh_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_rayleigh_initdefaut(t_gor)

implicit none

!=== Déclaration des arguments ===
type(st_gene_rayleigh), intent(inout)  :: t_gor

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_gor%attpied = GOR_ATTPIED
t_gor%ordre   = GOR_ORDRE
t_gor%f0      = 0.0
t_gor%t0      = 0.0

end subroutine gene_rayleigh_initdefaut


!===========================================================	
! Subroutine gene_rayleigh_lecture
! Lecture dans le fichier d'entrée des paramètres 
! sur le générateur sino-gaussien
!===========================================================	
subroutine gene_rayleigh_lecture(t_fich,t_gor)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_rayleigh), intent(inout)    :: t_gor

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine,status
real     :: puissance
!===   Début des traitements   ===

call gene_rayleigh_initdefaut(t_gor)
do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)

  select case(premiermot(1:lchaine))
    case(BFS_OR_F_AMAX)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gor%f0
    case(BFS_OR_ORDRE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gor%ordre
    case(BFS_ATT_PIED)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gor%attpied
    case default
      if ((t_gor%f0 <= 0.0) .or. (t_gor%ordre <= 0)) then
        print *,'fonction ondelette de rayleigh mal définie !!!' 
	call lecture_stop(t_fich)
      end if

      ! Calcul du retard t0
      puissance = 1.0 / (t_gor%ordre * 1.0 + 1.0)
      t_gor%t0 = t_gor%ordre / (2*PI * t_gor%f0) * (t_gor%attpied**puissance)

      exit
  end select
end do    

end subroutine gene_rayleigh_lecture

!===========================================================	
! Subroutine gene_rayleigh_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur iemn exponentiel quotient
!===========================================================	
subroutine gene_rayleigh_ecriture(f,t_gor)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_rayleigh), intent(inout)    :: t_gor

write(f%unite,'(T5,A,3x,ES10.3E2)')  BFS_OR_F_AMAX,t_gor%f0
if (GOR_ORDRE /= t_gor%ordre)     write(f%unite,'(T5,A,3x,I3)')  BFS_OR_ORDRE,t_gor%ordre
if (GOR_ATTPIED /= t_gor%attpied)    write(f%unite,'(T5,A,3x,ES10.3E2)')  BFS_ATT_PIED,t_gor%attpied

end subroutine gene_rayleigh_ecriture


!===========================================================	
! Subroutine gene_rayleigh_init
! Initialisation des variables
!===========================================================	
subroutine gene_rayleigh_init(t_gor)

implicit none

!=== Déclaration des arguments ===
type(st_gene_rayleigh), intent(inout)    :: t_gor

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_rayleigh_init

!===========================================================	
! Subroutine gene_rayleigh_calcul_t
! Calcul de la fonction ondelette de rayleigh sur le nombre d'itérations
! temporelles, paramètres transmis en argument
! Calculs avant les itérations temporelles
!===========================================================	
function gene_rayleigh_calcul_t(t_gor, A0, phase, retard, &
                                dt, ntinc,tab1d) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_rayleigh), intent(in)    :: t_gor
real, intent(in)          :: A0
real, intent(in)          :: retard,phase,dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      :: func_dervaleur
!===  Déclaration des variables  ===
integer   :: i
real      :: pi2f0_n
integer   :: nplus1
real      :: phaserad, f0_2pi
complex   :: cim1


!===   Début des traitements   ===	
cim1 = CMPLX(0.0,1.0)
nplus1 = t_gor%ordre + 1
pi2f0_n = PI * 2 * t_gor%f0 / t_gor%ordre		
phaserad = phase * PI / 180.0
do i =0, ntinc-1
  tab1d(i) = A0 *  REAL( EXP(cim1 * (PI/2.0 + phaserad)) / &
                  (cim1 + pi2f0_n * (i*dt-retard-t_gor%t0))**nplus1 ) 
end do
! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)


end function gene_rayleigh_calcul_t

!===========================================================	
! Subroutine gene_rayleigh_calcul_1t
! Calcul de la fonction ondelette de rayleigh pour un temps donné
! Calculs avant les itérations temporelles
!===========================================================	
function gene_rayleigh_calcul_1t(t_gor, A0, &
                  phase, retard, t) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_rayleigh), intent(in)    :: t_gor
real, intent(in)          :: A0
real, intent(in)          :: retard,phase,t
real                      :: func_dervaleur
!===  Déclaration des variables  ===
real      :: pi2f0_n
integer   :: nplus1
real      :: phaserad, f0_2pi
complex   :: cim1


!===   Début des traitements   ===	
cim1 = CMPLX(0.0,1.0)
nplus1 = t_gor%ordre + 1
pi2f0_n = PI * 2 * t_gor%f0 / t_gor%ordre		
phaserad = phase * PI / 180.0

func_dervaleur = A0 *  REAL( EXP(cim1 * (PI/2.0 + phaserad)) / &
                  (cim1 + pi2f0_n * (t-retard-t_gor%t0)**nplus1 ) ) 


end function gene_rayleigh_calcul_1t


	
!===========================================================	
! Subroutine gene_rayleigh_precalcul
! Calculs avant les itérations temporelles
!===========================================================	
subroutine gene_rayleigh_precalcul(t_gor)

implicit none

!=== Déclaration des arguments ===
type(st_gene_rayleigh), intent(inout)    :: t_gor

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_rayleigh_precalcul


!===========================================================	
! Subroutine gene_rayleigh_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_rayleigh_affichage(t_gor)

implicit none

!=== Déclaration des arguments ===
type(st_gene_rayleigh), intent(inout)    :: t_gor

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_rayleigh_affichage

!===========================================================	
! Subroutine gene_rayleigh_deallocate
!===========================================================	
subroutine gene_rayleigh_deallocate(t_gor)

implicit none

!=== Déclaration des arguments ===
type(st_gene_rayleigh), intent(inout)    :: t_gor

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end Subroutine gene_rayleigh_deallocate


end module m_gene_rayleigh



