!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_pulse_sinus.f90
!// Purpose:      Traite la fonction pulse-sinus  
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Lieu     :  Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Created  :     2003
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!===========================================================
! == Module module_gene_pulse_sinus                             ==
! == Type st_gene_pulse_sinus                                   ==
! == Subroutine gene_pulse_sinus_inidefaut                      ==
! == Subroutine gene_pulse_sinus_lecture                        ==
! == Subroutine gene_pulse_sinus_init                 ==
! == Subroutine gene_pulse_sinus_calcul_t                ==
! == Subroutine gene_pulse_sinus_calcul_1t               ==
! == Suborutine gene_pulse_sinus_calcul                         ==
! == Suborutine gene_pulse_sinus_affichage                      ==
! == Subroutine gene_pulse_sinus_postcalcul                     ==
! == Pas de variables globales...                         ==
!===========================================================

module m_gene_pulse_sinus
use m_entree , only : st_fichier, lineread, &
                     lecture_stop, &
		 ctrl_nb_mot_chaine
use m_constantes
use m_constantes_scientifiques, only : PI
use m_gene_identifiant 

!=== Déclaration des types ===

! Structure du sinus 
type st_gene_pulse_sinus
  real    :: freq_centrale
  REAL    :: duree, tm
  
end type st_gene_pulse_sinus


contains
!===========================================================	
! Subroutine gene_pulse_sinus_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_pulse_sinus_initdefaut(t_ps)

implicit none

!=== Déclaration des arguments ===
type(st_gene_pulse_sinus), intent(inout)  :: t_ps

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_ps%freq_centrale = 0.0e9
t_ps%tm = 0.0
t_ps%duree = 0.0

end subroutine gene_pulse_sinus_initdefaut


!===========================================================	
! Subroutine gene_pulse_sinus_lecture
! Lecture dans le fichier d'entrée des paramètres 
! sur le générateur sinus
!===========================================================	
subroutine gene_pulse_sinus_lecture(t_fich,t_ps)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_pulse_sinus), intent(inout)    :: t_ps

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine, status
logical, dimension(3)  :: flag
!===   Début des traitements   ===

call gene_pulse_sinus_initdefaut(t_ps)
flag(:) = .false.
do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)

select case(premiermot(1:lchaine))
    case(BFS_F0)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)
      read(t_fich%chaine,*) corbeille,t_ps%freq_centrale
      flag(1) = .true.
    case(BFS_TM)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)
      read(t_fich%chaine,*) corbeille,t_ps%tm
      flag(2) = .true.
    case(BFS_DUREE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)
      read(t_fich%chaine,*) corbeille,t_ps%duree
      flag(3) = .true.
    case default
      if (.not.ALL(flag)) then
	print '("Erreur, manque une donnée : tm, durée ou fc ! ")'
        call lecture_stop(t_fich)
	
      end if
      exit
  end select
end do    
end subroutine gene_pulse_sinus_lecture

!===========================================================	
! Subroutine gene_pulse_sinus_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur sinus
!===========================================================	
subroutine gene_pulse_sinus_ecriture(f,t_ps)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_pulse_sinus), intent(inout)    :: t_ps

write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_F0,t_ps%freq_centrale
write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_TM,t_ps%tm
write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_DUREE,t_ps%duree

end subroutine gene_pulse_sinus_ecriture

!===========================================================	
! Subroutine gene_pulse_sinus_init
! Initialisation des variables
!===========================================================	
subroutine gene_pulse_sinus_init(t_ps)

implicit none

!=== Déclaration des arguments ===
type(st_gene_pulse_sinus), intent(inout)    :: t_ps

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_pulse_sinus_init
	
!===========================================================	
! Subroutine gene_pulse_sinus_precalcul
! Calcul de la fonction sinus sur le nombre d'itérations
! temporelles, paramètre transmis en argument
! Calculs avant les itérations temporelles
!===========================================================	
function gene_pulse_sinus_calcul_t(t_ps, A0, phase, retard, &
                                dt, ntinc,tab1d) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_pulse_sinus), intent(in)    :: t_ps
real, intent(in)          :: A0
real, intent(in)          :: phase, retard, dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      :: func_dervaleur
!===  Déclaration des variables  ===
integer   :: i, nretard, n1, n2, n3
real      :: phaserad, f0_2pi, pi_2tm

!===   Début des traitements   ===	
f0_2pi = 2*PI*t_ps%freq_centrale		
phaserad = phase * PI / 180.0

nretard = NINT(retard / dt)
n1 = nretard + NINT(t_ps%tm/dt)
n2 = nretard + NINT((t_ps%duree-t_ps%tm)/dt)
n3 = nretard + INT(t_ps%duree/dt)
pi_2tm = PI / (2*t_ps%tm)
tab1d(:) = 0.0
n1 = min(n1,ntinc-1)
n2 = min(n2,ntinc-1)
n3 = min(n3,ntinc-1)

do i =nretard, n1
  tab1d(i) = A0*SIN(f0_2pi*(dt*i) + phaserad) * (sin(pi_2tm*(dt*i)))**2
end do

do i =n1+1, n2
  tab1d(i) = A0*SIN(f0_2pi*(dt*i) + phaserad)
end do

do i =n2+1, n3
  tab1d(i) = A0*SIN(f0_2pi*(dt*i) + phaserad) * (sin(pi_2tm*(dt*i-t_ps%duree)))**2
end do

! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)


end function gene_pulse_sinus_calcul_t


!===========================================================	
! Subroutine gene_pulse_sinus_calcul
!  Calcul pendant les itérations temporelles
!===========================================================	
subroutine gene_pulse_sinus_calcul(t_ps)

implicit none

!=== Déclaration des arguments ===
type(st_gene_pulse_sinus), intent(inout)    :: t_ps

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_pulse_sinus_calcul

!===========================================================	
! Subroutine gene_pulse_sinus_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_pulse_sinus_affichage(t_ps)

implicit none

!=== Déclaration des arguments ===
type(st_gene_pulse_sinus), intent(inout)    :: t_ps

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_pulse_sinus_affichage

!===========================================================	
! Subroutine gene_pulse_sinus_deallocate
!  Libération de la mémoire
!===========================================================	
subroutine gene_pulse_sinus_deallocate(t_ps)

implicit none

!=== Déclaration des arguments ===
type(st_gene_pulse_sinus), intent(inout)    :: t_ps

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end Subroutine gene_pulse_sinus_deallocate

!===========================================================	
! Subroutine gene_pulse_sinus_impression
!  Ecriture des résultats dans les fichiers de sortie
!===========================================================	
subroutine gene_pulse_sinus_impression(t_ps)

implicit none

!=== Déclaration des arguments ===
type(st_gene_pulse_sinus), intent(inout)    :: t_ps

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_pulse_sinus_impression

end module m_gene_pulse_sinus



