!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_sino_gaussien.f90
!// Purpose:      Traite la fonction source de type sino-gaussien  
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     janvier 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!===========================================================
! == Generateur de fonctions sino-gaussiennes             ==
! == Module module_gene_sino_gauss                     ==
! == Type st_gene_sino_gauss                           ==
! == Subroutine gene_sino_gauss_inidefaut              ==
! == Subroutine gene_sino_gauss_lecture                ==
! == Subroutine gene_sino_gauss_init         ==
! == Subroutine gene_sino_gauss_calcul_t        ==
! == Subroutine gene_sino_gauss_calcul_1t       ==
! == Suborutine gene_sino_gauss_calcul                 ==
! == Suborutine gene_sino_gauss_affichage              ==
! == Subroutine gene_sino_gauss_postcalcul             ==
! == Pas de variables globales...                         ==
!===========================================================

module m_gene_sino_gauss
use m_entree, only : st_fichier, lineread, &
                                  lecture_stop, &
				  ctrl_nb_mot_chaine
use m_constantes 
use m_constantes_scientifiques, only : PI 
use m_gene_identifiant 

implicit none
real, parameter :: GSG_ATTFMAX = 10.0
real, parameter :: GSG_ATTPIED = 100.0

!=== Déclaration des types ===

! Structure de la fonction sino-gaussienne 
type st_gene_sino_gauss
  integer :: type_def
  real    :: duree, tm, attpied, f0, bandefreq, attfmax
  real    :: T  ! 2*T = largeur de la gaussienne à 1/3 du max d'amplitude 
  real    :: t0  ! t0 = duree /2
  
end type st_gene_sino_gauss


contains
!===========================================================	
! Subroutine gene_sino_gauss_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_sino_gauss_initdefaut(t_gsg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sino_gauss), intent(inout)  :: t_gsg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_gsg%type_def = 0
t_gsg%f0 = 0.0
t_gsg%bandefreq = 0.0 
t_gsg%attfmax = GSG_ATTFMAX  
t_gsg%attpied = GSG_ATTPIED
t_gsg%tm      = 0.0
t_gsg% duree  = 0.0
t_gsg%t0      = 0.0
t_gsg%T       = 0.0
 
end subroutine gene_sino_gauss_initdefaut


!===========================================================	
! Subroutine gene_sino_gauss_lecture
! Lecture dans le fichier d'entrée des paramètres 
! sur le générateur sino-gaussien
!===========================================================	
subroutine gene_sino_gauss_lecture(t_fich,t_gsg)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_sino_gauss), intent(inout)    :: t_gsg

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine,status
logical  :: flagduree
!===   Début des traitements   ===

call gene_sino_gauss_initdefaut(t_gsg)

flagduree = .false.
do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)
  select case(premiermot(1:lchaine))
    case(BFS_DUREE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gsg%duree
      t_gsg%t0 = t_gsg%duree / 2
      t_gsg%T  = t_gsg%t0 / (SQRT(LOG(t_gsg%attpied)))
      flagduree = .true.
      t_gsg%bandefreq = 0.0
      t_gsg%type_def = 2
    case(BFS_TM)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gsg%tm
      ! temps de montée de 10 % à 90 %
      ! on déduit le retard t0 = duree / 2
      t_gsg%t0 = SQRT(LOG(t_gsg%attpied))*t_gsg%tm /(SQRT(LOG(10.0))-SQRT(LOG(10.0/9)))
      t_gsg%T  = t_gsg%t0 / (SQRT(LOG(t_gsg%attpied)))
      t_gsg%bandefreq = 0.0
      flagduree = .false.
      t_gsg%type_def = 1
    case(BFS_F0)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gsg%f0
    case(BFS_SG_BANDE)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gsg%bandefreq
      t_gsg%T  = SQRT(LOG(t_gsg%attfmax))/(PI*t_gsg%bandefreq/2)
      t_gsg%t0 = t_gsg%T * SQRT(LOG(t_gsg%attpied))   
      flagduree = .false.
      t_gsg%type_def = 0
    case(BFS_ATT_FMAX)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gsg%attfmax
      if (t_gsg%bandefreq <= 0) then
       	print *,'Avertissement :  Att_fmax définie alors que bande_freq pas encore définie' 
      else 
        t_gsg%T  = SQRT(LOG(t_gsg%attfmax))/(PI*t_gsg%bandefreq/2)
        t_gsg%t0 = t_gsg%T * SQRT(LOG(t_gsg%attpied))   
        flagduree = .false.
      end if
    case(BFS_ATT_PIED)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2)
      read(t_fich%chaine,*) corbeille,t_gsg%attpied
      if (flagduree) then
        ! forme de la gaussienne modifiée par la valeur du pied
        t_gsg%T  = t_gsg%t0 / (SQRT(LOG(t_gsg%attpied)))
      else
        ! forme de la gaussienne non modifiée par la valeur du pied
        t_gsg%t0 = t_gsg%T * SQRT(LOG(t_gsg%attpied)) 
      end if
      
    case default
      if ((t_gsg%t0 <= 0.0) .or. (t_gsg%T <= 0.0)) then
	print *,'fonction sino-gaussienne mal définie !!!' 
	call lecture_stop(t_fich)
      end if
      exit 
  end select
end do 
   
end subroutine gene_sino_gauss_lecture

!===========================================================	
! Subroutine gene_sino_gauss_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur sino_gaussien
!===========================================================	
subroutine gene_sino_gauss_ecriture(f,t_gsg)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_sino_gauss), intent(inout)    :: t_gsg

if (t_gsg%type_def == 0) then
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_SG_BANDE,t_gsg%bandefreq
else if (t_gsg%type_def == 1) then
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_TM,t_gsg%tm
else
  write(f%unite,'(T5,A,3x,ES13.6E2)')  BFS_DUREE,t_gsg%duree
end if
write(f%unite,'(T5,A,3x,ES10.3E2)')  BFS_F0,t_gsg%f0
if (t_gsg%attpied /= GSG_ATTPIED) write(f%unite,'(T5,A,3x,ES10.3E2)')  BFS_ATT_PIED,t_gsg%attpied
if (t_gsg%attfmax /= GSG_ATTFMAX) write(f%unite,'(T5,A,3x,ES10.3E2)')  BFS_ATT_FMAX,t_gsg%attfmax

end subroutine gene_sino_gauss_ecriture


!===========================================================	
! Subroutine gene_sino_gauss_init
! Initialisation des variables
!===========================================================	
subroutine gene_sino_gauss_init(t_gsg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sino_gauss), intent(inout)    :: t_gsg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_sino_gauss_init
	
!===========================================================	
! Subroutine gene_sino_gauss_calcul_t
! Calcul de la fonction sino-gaussienne sur le nombre d'itérations
! temporelles, paramètres transmis en argument
! Calculs avant les itérations temporelles
!===========================================================	
function gene_sino_gauss_calcul_t(t_gsg, A0, phase, retard, &
                                dt, ntinc,tab1d) result (func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sino_gauss), intent(in)    :: t_gsg
real, intent(in)          :: A0
real, intent(in)          :: retard,phase,dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      :: func_dervaleur
!===  Déclaration des variables  ===
integer   :: i
real      :: T_2
real      :: phaserad, f0_2pi


!===   Début des traitements   ===	
T_2 = t_gsg%T**2
f0_2pi = 2*PI*t_gsg%f0		
phaserad = phase * PI / 180.0
do i =0, ntinc-1
  tab1d(i) = A0 * SIN(f0_2pi*(dt*i-retard-t_gsg%t0) + phaserad) * &
                  EXP(-(i*dt-retard-t_gsg%t0)**2/T_2) 
end do
! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)


end function gene_sino_gauss_calcul_t

!===========================================================	
! Subroutine gene_sino_gauss_calcul_1t
! Calcul de la fonction sino-gaussienne pour un temps donné
! Calculs avant les itérations temporelles
!===========================================================	
function gene_sino_gauss_calcul_1t(t_gsg, &
              A0, phase, retard,t) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sino_gauss), intent(in)    :: t_gsg
real, intent(in)          :: A0
real, intent(in)          :: retard,phase,t
real                      :: func_dervaleur
!===  Déclaration des variables  ===
integer   :: i
real      :: T_2
real      :: phaserad, f0_2pi

!===   Début des traitements   ===	
T_2 = t_gsg%T**2
f0_2pi = 2*PI*t_gsg%f0		
phaserad = phase * PI / 180.0

func_dervaleur = A0 * &
   SIN(f0_2pi*(t-retard-t_gsg%t0) + phaserad) * EXP(-(t-retard-t_gsg%t0)**2/T_2)

end function gene_sino_gauss_calcul_1t

!===========================================================	
! Subroutine gene_sino_gauss_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_sino_gauss_affichage(t_gsg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sino_gauss), intent(inout)    :: t_gsg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_sino_gauss_affichage

!===========================================================	
! Subroutine gene_sino_gauss_deallocate
! 
!===========================================================	
subroutine gene_sino_gauss_deallocate(t_gsg)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sino_gauss), intent(inout)    :: t_gsg

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end Subroutine gene_sino_gauss_deallocate

end module m_gene_sino_gauss



