!/////////////////////////////////////////////////////////////////////////////
!// Name:        m_gene_sinus.f90
!// Purpose:      Traite la fonction sinus  
!// Author:      Christophe Guiffaut
!// Modified by:  Christophe Guiffaut
!// Created  :     Janvier 2002
!// Lieu     : XLIM - Faculté des sciences - 123 Av. Albert Thomas - Limoges
!// Copyright:   (c) CNRS, Université de Limoges
!// Licence  : CECILL-C - Octobre 2006
!/////////////////////////////////////////////////////////////////////////////

!===========================================================
! == Generateur de fonction sinus                         ==
! == Module module_gene_sinus                             ==
! == Type st_gene_sinus                                   ==
! == Subroutine gene_sinus_inidefaut                      ==
! == Subroutine gene_sinus_lecture                        ==
! == Subroutine gene_sinus_init                 ==
! == Subroutine gene_sinus_calcul_t                ==
! == Subroutine gene_sinus_calcul_1t               ==
! == Suborutine gene_sinus_calcul                         ==
! == Suborutine gene_sinus_affichage                      ==
! == Subroutine gene_sinus_postcalcul                     ==
! == Pas de variables globales...                         ==
!===========================================================

module m_gene_sinus
use m_entree, only : st_fichier, lineread, &
                                  lecture_stop, &
				  ctrl_nb_mot_chaine
use m_constantes
use m_constantes_scientifiques, only : PI
use m_gene_identifiant 

implicit none
!=== Déclaration des types ===

! Structure du sinus 
type st_gene_sinus
  real    :: freq_centrale
end type st_gene_sinus


contains
!===========================================================	
! Subroutine gene_sinus_initdefaut
! Initialisation par défaut des variables contenant les paramètres
! définis par l'utilisateur dans le fichier d'entrée
!===========================================================	
subroutine gene_sinus_initdefaut(t_sinus)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sinus), intent(inout)  :: t_sinus

!===  Déclaration des variables  ===

!===   Début des traitements   ===
t_sinus%freq_centrale = 0.0e9

end subroutine gene_sinus_initdefaut


!===========================================================	
! Subroutine gene_sinus_lecture
! Lecture dans le fichier d'entrée des paramètres 
! sur le générateur sinus
!===========================================================	
subroutine gene_sinus_lecture(t_fich,t_sinus)

implicit none

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: t_fich
type(st_gene_sinus), intent(inout)    :: t_sinus

!===  Déclaration des variables  ===
character (len=LEN_MOT_CLE)  :: premiermot, corbeille
integer  :: lchaine, status
!===   Début des traitements   ===

call gene_sinus_initdefaut(t_sinus)

do
  premiermot = lineread(t_fich)
  lchaine = LEN_TRIM(premiermot)
  if (t_fich%nomfinbloc == trim(premiermot)) exit
  select case(premiermot(1:lchaine))
    case(BFS_F0)
      status = ctrl_nb_mot_chaine(t_fich,t_fich%chaine,2,2)
      read(t_fich%chaine,*) corbeille,t_sinus%freq_centrale
    case default
      exit
  end select
end do    
end subroutine gene_sinus_lecture

!===========================================================	
! Subroutine gene_sinus_ecriture
! Ecriture dans le fichier d'entrée des paramètres 
! sur le générateur sinus
!===========================================================	
subroutine gene_sinus_ecriture(f,t_sinus)

!=== Déclaration des arguments ===
type(st_fichier), intent(inout)       :: f
type(st_gene_sinus), intent(inout)    :: t_sinus

write(f%unite,'(T5,A,3x,ES12.5E2)')  BFS_F0,t_sinus%freq_centrale

end subroutine gene_sinus_ecriture


!===========================================================	
! Subroutine gene_sinus_init
! Initialisation des variables
!===========================================================	
subroutine gene_sinus_init(t_sinus)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sinus), intent(inout)    :: t_sinus

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_sinus_init
	
!===========================================================	
! Subroutine gene_sinus_precalcul
! Calcul de la fonction sinus sur le nombre d'itérations
! temporelles, paramètre transmis en argument
! Calculs avant les itérations temporelles
!===========================================================	
function gene_sinus_calcul_t(t_sinus, A0, phase, retard, &
                                dt, ntinc,tab1d) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sinus), intent(in)    :: t_sinus
real, intent(in)          :: A0
real, intent(in)          :: phase, retard, dt
integer, intent(in)       :: ntinc
real, dimension(0:), intent(inout)   :: tab1d
real                      :: func_dervaleur
!===  Déclaration des variables  ===
integer   :: i
real      :: phaserad, f0_2pi

!===   Début des traitements   ===	
f0_2pi = 2*PI*t_sinus%freq_centrale		
phaserad = phase * PI / 180.0
do i =0, ntinc-1
  tab1d(i) = A0*SIN(f0_2pi*(dt*i-retard) + phaserad)
end do
! retourne la dernière valeur 
func_dervaleur = tab1d(ntinc-1)


end function gene_sinus_calcul_t

!===========================================================	
! Subroutine gene_sinus_calcul_1t
! Calcul de la fonction sinus pour un t donné
! Calculs avant les itérations temporelles
!===========================================================	
function gene_sinus_calcul_1t(t_sinus, A0, phase, &
                           retard,t) result(func_dervaleur)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sinus), intent(in)    :: t_sinus
real, intent(in)          :: A0
real, intent(in)          :: phase, retard,t
real                      :: func_dervaleur
!===  Déclaration des variables  ===
real      :: phaserad, f0_2pi
!===   Début des traitements   ===
f0_2pi = 2*PI*t_sinus%freq_centrale		
phaserad = phase * PI / 180.0
func_dervaleur = A0*SIN(f0_2pi*(t-retard) + phaserad)

end function gene_sinus_calcul_1t

!===========================================================	
! Subroutine gene_sinus_calcul
!  Calcul pendant les itérations temporelles
!===========================================================	
subroutine gene_sinus_calcul(t_sinus)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sinus), intent(inout)    :: t_sinus

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_sinus_calcul

!===========================================================	
! Subroutine gene_sinus_affichage
!  Affichage de résultats dans le terminal
!===========================================================	
subroutine gene_sinus_affichage(t_sinus)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sinus), intent(inout)    :: t_sinus

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end subroutine gene_sinus_affichage

!===========================================================	
! Subroutine gene_sinus_deallocate
!
!===========================================================	
subroutine gene_sinus_deallocate(t_sinus)

implicit none

!=== Déclaration des arguments ===
type(st_gene_sinus), intent(inout)    :: t_sinus

!===  Déclaration des variables  ===

!===   Début des traitements   ===
				
end Subroutine gene_sinus_deallocate

end module m_gene_sinus



